/* -----------------------------------------------------------------------
   Tests des cas limites des valeurs numeriques avec NX
   (int, long, float, double).
   auteur: OH.

   Contenu: ce fichier contient des appels aux fonctions de manipulation
	    des types int, long, float, double.
   ----------------------------------------------------------------------- */

static char *SCCS_ID = "%Z% DCSA/MLS %M%   %I%     %G%";

#include <stdio.h>
#include <string.h>
#include <limits.h>
#include "nx.h"
#include "machine_types.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/errno.h>

#define		TRUE	1
#define		FALSE	0


static double		double_min, double_max;

/* -------------------------------------------------------------
   Fonction de mise a zero des valeurs des tableaux numeriques
   traites (tabi[] et tabf[]).
   ------------------------------------------------------------- */
static void
_RAZ_Data(
	)
{
double_min=0.0;
double_max=0.0;
}


/* -------------------------------------------------------------
   Fonction de d'initialisation des valeurs des tableaux
   numeriques traites (tabi[] et tabf[]).
   ------------------------------------------------------------- */
static void
_Init_Data(
	)
{
double_min=DBL_MIN;
double_max=DBL_MAX;
}


static int
_Print_Values(
	)
{
static char	*funcname="_Print_Values";

printf("-----------------------------------------------\n");
printf("\tdouble_min=\t\t%lf\n",double_min);
printf("\tdouble_max=\t\t%lf\n",double_max);
printf("-----------------------------------------------\n");

return(0);
}


/* -------------------------------------------------------------
   Fonction de creation des differents fichiers de donnees
   couvrant tous les formats disponibles sous NX.
   ------------------------------------------------------------- */
static int
_Write_Data(
	char	*filename,
	char	*format
	)
{
static char	*funcname="_Write_Data";
FILE		*fd=NULL;
float		float_max;

if((fd=fnxopen(filename,"w"))==NULL)
  {
  printf("%s: pb d'execution de fnxopen(\"%s\").\n",funcname,filename);
  return(-1);
  }

if(fnxchfrm(fd,format)<0)
  {
  printf("%s: pb d'execution de fnxchfrm(%s).\n",funcname,format);
  return(-1);
  }
float_max=FLT_MAX;
if(fnxwrite_float(fd,1,&float_max)<0)
  {
  printf("%s: pb d'execution de fnxwrite_float(%s) avec float_max.\n",
	 funcname,format);
  return(-1);
  }
/*
if(fnxwrite_double(fd,1,&double_min)<0)
  {
  printf("%s: pb d'execution de fnxwrite_double(%s) avec double_min.\n",
	 funcname,format);
  return(-1);
  }
*/

if(fnxclose(fd)<0)
  {
  printf("%s: pb d'execution de fnxclose(%s).\n",funcname,format);
  return(-1);
  }

return(0);
}


/* -------------------------------------------------------------
   Fonction de lecture des differents fichiers de donnees crees
   couvrant tous les formats disponibles sous NX.
   ------------------------------------------------------------- */
static int
_Read_Data(
	char	*filename,
	char	*format
	)
{
static char	*funcname="_Read_Data";
FILE		*fd=NULL;
float		float_max;

if((fd=fnxopen(filename,"r"))==NULL)
  {
  printf("%s: pb d'execution de fnxopen(\"%s\").\n",funcname,filename);
  return(-1);
  }

if(fnxchfrm(fd,format)<0)
  {
  printf("%s: pb d'execution de fnxchfrm(%s).\n",funcname,format);
  return(-1);
  }
if(fnxread_float(fd,1,&float_max)<0)
  {
  printf("%s: pb d'execution de fnxread_float(%s) avec float_max.\n",
	 funcname,format);
  return(-1);
  }
/*
if(fnxread_double(fd,1,&double_min)<0)
  {
  printf("%s: pb d'execution de fnxread_double(%s) avec double_min.\n",funcname,format);
  return(-1);
  }
*/

if(fnxclose(fd)<0)
  {
  printf("%s: pb d'execution de fnxclose(%s).\n",funcname,format);
  return(-1);
  }

return(0);
}


/* -------------------------------------------------------------
   ------------------------------------------------------------- */
int
main(
	int	argc,
	char	**argv
	)
{
static char	*syntax="-LIMIT test_directory";
char 		filename[512];	/* buffer pour nom de fichier de donnees */
char 		test_dir[256];	/* nom du directory des donnees du test */
struct 		stat buf;	/* struct d'etat du directory du test */
int		cr;

if(argc!=2)
  {
  printf("Syntaxe: %s %s\n",argv[0],syntax);
  return(-1);
  }

if(stat(argv[1],&buf)<0)
  {
  printf("%s: pb avec le repertoire %s. Test Abort.\n",argv[0],argv[1]);
  perror(argv[1]);
  return(-1);
  }

strcpy(test_dir,argv[1]);

_Init_Data();
printf("\n------------------------------------------------------\n");
printf("Les valeurs initiales du test sont:\n");
_Print_Values();
printf("------------------------------------------------------\n\n");

/* --------------------- ecriture des donnees --------------------- */
/* ------------- Ecriture au format IEEE_32_LE ------------- */
sprintf(filename,"%s/NX_Test_IE32_LE",test_dir);
if(_Write_Data(filename,"li32")!=0)
  {
  printf("%s: pb d'execution de _Write_Data(li32).\n",argv[0]);
  return(-1);
  }

/* --------------------- lecture des donnees --------------------- */
/* ------------- Lecture du format IEEE_32_LE ------------- */
_RAZ_Data();
sprintf(filename,"%s/NX_Test_IE32_LE",test_dir);
if(_Read_Data(filename,"li32")!=0)
  {
  printf("%s: pb d'execution de _Read_Data(li32).\n",argv[0]);
  return(-1);
  }

printf("%s: fin du test.\n",argv[0]);

return(0);
}
