/* identification du source */

#ifndef lint
    static const char* const sccs_id = "%W% %G% Pierre Brochard";
#endif

#if defined(X11) /* seulement si X11 est defini */

/* Includes */

#include "SpxP.h"
#include <X11/Xatom.h>
#include "SpxCallback.h"
#include "SpxOpenP.h"
#include "nx.h"
#include "SpxCallbackBase.h"
#include <X11/StringDefs.h>

typedef struct
{
	SPXFILE* spxfile;
	Widget wid;
	XtCallbackProc proc;
	XtPointer clientdata;
	const char* reason;
} SpxTimer;

typedef struct
{
	Widget wid;
	XtCallbackProc proc;
	XtPointer clientdata;
	const char* reason;
} SpxWindow;

typedef struct
{
	SpxWindow** enteraff;
	SpxWindow** leaveaff;
	SpxWindow** bpressaff;
	SpxWindow** breleaseaff;
	SpxWindow** mnotifyaff;
	unsigned nb_enteraff;
	unsigned nb_leaveaff;
	unsigned nb_bpressaff;
	unsigned nb_breleaseaff;
	unsigned nb_mnotifyaff;
} SpxCallbackReg;

static SpxCallbackReg* spx_callback_reg = NULL;

static void SpxTimerCB(XtPointer client_data,XtIntervalId* tt);
static void SpxEnterAffCB(Widget w,XtPointer client_data,
		XEvent* event,Boolean* ptb);
static void SpxLeaveAffCB(Widget w,XtPointer client_data,
		XEvent* event,Boolean* ptb);
static void SpxBpressAffCB(Widget w,XtPointer client_data,
		XEvent* event,Boolean* ptb);
static void SpxBreleaseAffCB(Widget w,XtPointer client_data,
		XEvent* event,Boolean* ptb);
static void SpxMnotifyAffCB(Widget w,XtPointer client_data,
		XEvent* event,Boolean* ptb);

static void
SpxTimerCB(XtPointer client_data,XtIntervalId* tt)
{
	SpxTimer* const spxtimer = (SpxTimer*) client_data;
	Window win = XtWindow(spxtimer->wid);

	if(!win)
	{
	    XtAppAddTimeOut(XtWidgetToApplicationContext(spxtimer->wid),20,
			    (XtTimerCallbackProc) SpxTimerCB,client_data);
	}
	else
	{
	    int temp[2];
	    FILE* file_fich = spxtimer->spxfile->file;
	    unsigned i = 0;
	    SpxCallbackReg* const scbr = spx_callback_reg;
	    SpxWindow** sw = NULL;
	    unsigned nsw = 0;
	    XtEventHandler xeh = NULL;
	    int type = 0;

	    if(!strcmp(spxtimer->reason,SpxNenterAffCallback))
	    {
		sw = scbr->enteraff;
		nsw = scbr->nb_enteraff;
		xeh = (XtEventHandler) SpxEnterAffCB;
		type = 4;
	    }
	    else if(!strcmp(spxtimer->reason,SpxNleaveAffCallback))
	    {
		sw = scbr->leaveaff;
		nsw = scbr->nb_leaveaff;
		xeh = (XtEventHandler) SpxLeaveAffCB;
		type = 5;
	    }
	    else if(!strcmp(spxtimer->reason,SpxNbpressAffCallback))
	    {
		sw = scbr->bpressaff;
		nsw = scbr->nb_bpressaff;
		xeh = (XtEventHandler) SpxBpressAffCB;
		type = 6;
	    }
	    else if(!strcmp(spxtimer->reason,SpxNbpressbaseAffCallback))
	    {
		sw = scbr->bpressaff;
		nsw = scbr->nb_bpressaff;
		xeh = (XtEventHandler) SpxBpressAffCB;
		type = 16;
	    }
	    else if(!strcmp(spxtimer->reason,SpxNbreleaseAffCallback))
	    {
		sw = scbr->breleaseaff;
		nsw = scbr->nb_breleaseaff;
		xeh = (XtEventHandler) SpxBreleaseAffCB;
		type = 7;
	    }
	    else if(!strcmp(spxtimer->reason,SpxNbreleasebaseAffCallback))
	    {
		sw = scbr->breleaseaff;
		nsw = scbr->nb_breleaseaff;
		xeh = (XtEventHandler) SpxBreleaseAffCB;
		type = 17;
	    }
	    else if(!strcmp(spxtimer->reason,SpxNmnotifyAffCallback))
	    {
		sw = scbr->mnotifyaff;
		nsw = scbr->nb_mnotifyaff;
		xeh = (XtEventHandler) SpxMnotifyAffCB;
		type = 8;
	    }
	    else if(!strcmp(spxtimer->reason,SpxNmnotifybaseAffCallback))
	    {
		sw = scbr->mnotifyaff;
		nsw = scbr->nb_mnotifyaff;
		xeh = (XtEventHandler) SpxMnotifyAffCB;
		type = 18;
	    }

	    for(i=0;i<nsw;i++)
	    {

		if(sw[i]->proc == spxtimer->proc && 
				sw[i]->clientdata == spxtimer->clientdata &&
				sw[i]->wid == spxtimer->wid &&
				!strcmp(sw[i]->reason,spxtimer->reason))
		{
		    break;
		}

	    }

	    if(i<nsw)
	    {
		temp[0] = type;
		XtAddEventHandler(spxtimer->wid,NoEventMask,True,xeh,
				(XtPointer) sw[i]);
		temp[1] = (int) win;
		fnxwriti(file_fich,2,temp);
		fnxwrite(file_fich,"@",1);
		fnxflush(file_fich);
	    }

	    SpxSync(spxtimer->spxfile);
	    SpxClose(spxtimer->spxfile);

	    free((char*) spxtimer);
	}

}

static void
SpxEnterAffCB(Widget w,XtPointer client_data,XEvent* event,Boolean* ptb)
{

	if(event->type == ClientMessage &&
			event->xclient.message_type == XA_STRING &&
			event->xclient.format == 8 &&
			!strcmp(event->xclient.data.b,"SPX_ENTER_AFF"))
	{
	    SpxWindow* const spxwindow = (SpxWindow*) client_data;

	    spxwindow->proc(w,spxwindow->clientdata,NULL);
	}

}

static void
SpxLeaveAffCB(Widget w,XtPointer client_data,XEvent* event,Boolean* ptb)
{

	if(event->type == ClientMessage &&
			event->xclient.message_type == XA_STRING &&
			event->xclient.format == 8 &&
			!strcmp(event->xclient.data.b,"SPX_LEAVE_AFF"))
	{
	    SpxWindow* const spxwindow = (SpxWindow*) client_data;

	    spxwindow->proc(w,spxwindow->clientdata,NULL);
	}

}

static void
SpxBpressAffCB(Widget w,XtPointer client_data,XEvent* event,Boolean* ptb)
{

	if(event->type == ClientMessage &&
			event->xclient.message_type == XA_INTEGER &&
			event->xclient.format == 32)
	{
	    register long int* bp = event->xclient.data.l;

	    if(bp[0] == ButtonPress)
	    {
		SpxWindow* const spxwindow = (SpxWindow*) client_data;

		if(!strcmp(spxwindow->reason,SpxNbpressAffCallback))
		{
		    SpxButtonCallbackStruct call_data;

		    call_data.x = bp[1];
		    call_data.y = bp[2];
		    call_data.state = bp[3];
		    call_data.button = bp[4];
		    spxwindow->proc(w,spxwindow->clientdata,&call_data);
		}

	    }
	    else if(bp[0] == ButtonPressBase)
	    {
		SpxWindow* const spxwindow = (SpxWindow*) client_data;

		if(!strcmp(spxwindow->reason,SpxNbpressbaseAffCallback))
		{
		    SpxButtonBaseCallbackStruct call_data;
		    float* x;
		    float* y;

		    x = (float*) &bp[1];
		    y = (float*) &bp[2];
		    call_data.x = *x;
		    call_data.y = *y;
		    call_data.state = bp[3];
		    call_data.button = bp[4];
		    spxwindow->proc(w,spxwindow->clientdata,&call_data);
		}

	    }

	}

}

static void
SpxBreleaseAffCB(Widget w,XtPointer client_data,XEvent* event,Boolean* ptb)
{

	if(event->type == ClientMessage &&
			event->xclient.message_type == XA_INTEGER &&
			event->xclient.format == 32)
	{
	    register long int* bp = event->xclient.data.l;

	    if(bp[0] == ButtonRelease)
	    {
		SpxWindow* const spxwindow = (SpxWindow*) client_data;

		if(!strcmp(spxwindow->reason,SpxNbreleaseAffCallback))
		{
		    SpxButtonCallbackStruct call_data;

		    call_data.x = bp[1];
		    call_data.y = bp[2];
		    call_data.state = bp[3];
		    call_data.button = bp[4];
		    spxwindow->proc(w,spxwindow->clientdata,&call_data);
		}

	    }
	    else if(bp[0] == ButtonReleaseBase)
	    {
		SpxWindow* const spxwindow = (SpxWindow*) client_data;

		if(!strcmp(spxwindow->reason,SpxNbreleasebaseAffCallback))
		{
		    SpxButtonBaseCallbackStruct call_data;
		    float* x;
		    float* y;

		    x = (float*) &bp[1];
		    y = (float*) &bp[2];
		    call_data.x = *x;
		    call_data.y = *y;
		    call_data.state = bp[3];
		    call_data.button = bp[4];
		    spxwindow->proc(w,spxwindow->clientdata,&call_data);
		}

	    }

	}

}

static void
SpxMnotifyAffCB(Widget w,XtPointer client_data,XEvent* event,Boolean* ptb)
{

	if(event->type == ClientMessage &&
			event->xclient.message_type == XA_INTEGER &&
			event->xclient.format == 32)
	{
	    register long int* bp = event->xclient.data.l;

	    if(bp[0] == MotionNotify)
	    {
		SpxWindow* const spxwindow = (SpxWindow*) client_data;

		if(!strcmp(spxwindow->reason,SpxNmnotifyAffCallback))
		{
		    SpxMotionCallbackStruct call_data;

		    call_data.x = bp[1];
		    call_data.y = bp[2];
		    call_data.state = bp[3];
		    spxwindow->proc(w,spxwindow->clientdata,&call_data);
		}

	    }
	    else if(bp[0] == MotionNotifyBase)
	    {
		SpxWindow* const spxwindow = (SpxWindow*) client_data;

		if(!strcmp(spxwindow->reason,SpxNmnotifybaseAffCallback))
		{
		    SpxMotionBaseCallbackStruct call_data;
		    float* x;
		    float* y;

		    x = (float*) &bp[1];
		    y = (float*) &bp[2];
		    call_data.x = *x;
		    call_data.y = *y;
		    call_data.state = bp[3];
		    spxwindow->proc(w,spxwindow->clientdata,&call_data);
		}

	    }

	}

}

void
SpxAddCallback(Widget w,const char* reason,XtCallbackProc proc,
		XtPointer clientdata,SPXFILE* spx_file)
{

	if(spx_file && spx_file->type == SPXSOCKET)
	{
	    SpxTimer* spxtimer = (SpxTimer*) malloc(sizeof(SpxTimer));
	    SpxWindow* spxwindow = (SpxWindow*) malloc(sizeof(SpxWindow));

	    spxtimer->spxfile = spx_file;
	    spxtimer->wid = w;
	    spxtimer->proc = proc;
	    spxtimer->clientdata = clientdata;
	    spxtimer->reason = reason;
	    spx_file->close_defer++;

	    if(!spx_callback_reg)
	    {
		SpxCallbackReg* scbr =
		    (SpxCallbackReg*) malloc(sizeof(SpxCallbackReg));

		if(!scbr)
		{
		    fprintf(stderr,"Pb alloc %s\n",__FILE__);
		    free((char*) spxtimer);
		    return;
		}

		scbr->enteraff = NULL;
		scbr->leaveaff = NULL;
		scbr->bpressaff = NULL;
		scbr->breleaseaff = NULL;
		scbr->mnotifyaff = NULL;
		scbr->nb_enteraff = 0;
		scbr->nb_leaveaff = 0;
		scbr->nb_bpressaff = 0;
		scbr->nb_breleaseaff = 0;
		scbr->nb_mnotifyaff = 0;
		spx_callback_reg = scbr;
	    }

	    spxwindow->proc = proc;
	    spxwindow->clientdata = clientdata;
	    spxwindow->wid = w;
	    spxwindow->reason = reason;

	    if(!strcmp(reason,SpxNenterAffCallback))
	    {
		SpxCallbackReg* const scbr = spx_callback_reg;
		SpxWindow** sw = scbr->enteraff;

		if(!sw)
		{
		    sw = (SpxWindow**) malloc(sizeof(SpxWindow*));
		    scbr->nb_enteraff = 1;
		}
		else
		{
		    sw = (SpxWindow**) realloc((char*) sw,
				    ++scbr->nb_enteraff*sizeof(SpxWindow*));
		}

		if(!sw)
		{
		    fprintf(stderr,"Pb alloc %s\n",__FILE__);
		    scbr->nb_enteraff = 0;
		    free((char*) spxwindow);
		    return;
		}

		scbr->enteraff = sw;
		sw[scbr->nb_enteraff-1] = spxwindow;
	    }
	    else if(!strcmp(spxtimer->reason,SpxNleaveAffCallback))
	    {
		SpxCallbackReg* const scbr = spx_callback_reg;
		SpxWindow** sw = scbr->leaveaff;

		if(!sw)
		{
		    sw = (SpxWindow**) malloc(sizeof(SpxWindow*));
		    scbr->nb_leaveaff = 1;
		}
		else
		{
		    sw = (SpxWindow**) realloc((char*) sw,
				    ++scbr->nb_leaveaff*sizeof(SpxWindow*));
		}

		if(!sw)
		{
		    fprintf(stderr,"Pb alloc %s\n",__FILE__);
		    scbr->nb_leaveaff = 0;
		    free((char*) spxwindow);
		    return;
		}

		scbr->leaveaff = sw;
		sw[scbr->nb_leaveaff-1] = spxwindow;
	    }
	    else if(!strcmp(spxtimer->reason,SpxNbpressAffCallback) ||
			    !strcmp(spxtimer->reason,SpxNbpressbaseAffCallback))
	    {
		SpxCallbackReg* const scbr = spx_callback_reg;
		SpxWindow** sw = scbr->bpressaff;

		if(!sw)
		{
		    sw = (SpxWindow**) malloc(sizeof(SpxWindow*));
		    scbr->nb_bpressaff = 1;
		}
		else
		{
		    sw = (SpxWindow**) realloc((char*) sw,
				    ++scbr->nb_bpressaff*sizeof(SpxWindow*));
		}

		if(!sw)
		{
		    fprintf(stderr,"Pb alloc %s\n",__FILE__);
		    scbr->nb_bpressaff = 0;
		    free((char*) spxwindow);
		    return;
		}

		scbr->bpressaff = sw;
		sw[scbr->nb_bpressaff-1] = spxwindow;
	    }
	    else if(!strcmp(spxtimer->reason,SpxNbreleaseAffCallback) ||
			    !strcmp(spxtimer->reason,
					    SpxNbreleasebaseAffCallback))
	    {
		SpxCallbackReg* const scbr = spx_callback_reg;
		SpxWindow** sw = scbr->breleaseaff;

		if(!sw)
		{
		    sw = (SpxWindow**) malloc(sizeof(SpxWindow*));
		    scbr->nb_breleaseaff = 1;
		}
		else
		{
		    sw = (SpxWindow**) realloc((char*) sw,
				    ++scbr->
				    nb_breleaseaff*sizeof(SpxWindow*));
		}

		if(!sw)
		{
		    fprintf(stderr,"Pb alloc %s\n",__FILE__);
		    scbr->nb_breleaseaff = 0;
		    free((char*) spxwindow);
		    return;
		}

		scbr->breleaseaff = sw;
		sw[scbr->nb_breleaseaff-1] = spxwindow;
	    }
	    else if(!strcmp(spxtimer->reason,SpxNmnotifyAffCallback) ||
			    !strcmp(spxtimer->reason,SpxNmnotifybaseAffCallback))
	    {
		SpxCallbackReg* const scbr = spx_callback_reg;
		SpxWindow** sw = scbr->mnotifyaff;

		if(!sw)
		{
		    sw = (SpxWindow**) malloc(sizeof(SpxWindow*));
		    scbr->nb_mnotifyaff = 1;
		}
		else
		{
		    sw = (SpxWindow**) realloc((char*) sw,
				    ++scbr->
				    nb_mnotifyaff*sizeof(SpxWindow*));
		}

		if(!sw)
		{
		    fprintf(stderr,"Pb alloc %s\n",__FILE__);
		    scbr->nb_mnotifyaff = 0;
		    free((char*) spxwindow);
		    return;
		}

		scbr->mnotifyaff = sw;
		sw[scbr->nb_mnotifyaff-1] = spxwindow;
	    }

	    XtAppAddTimeOut(XtWidgetToApplicationContext(w),20,
			    (XtTimerCallbackProc) SpxTimerCB,
			    (XtPointer) spxtimer);
	}

}

void
SpxRemoveCallback(Widget w,const char* reason,XtCallbackProc proc,
		XtPointer clientdata,SPXFILE* spx_file)
{

	if(spx_file && spx_file->type == SPXSOCKET)
	{
	    int temp[2];
	    FILE* file_fich = spx_file->file;
	    unsigned i = 0;
	    SpxCallbackReg* const scbr = spx_callback_reg;
	    SpxWindow*** psw = NULL;
	    SpxWindow** sw = NULL;
	    unsigned nsw = 0;
	    XtEventHandler xeh = NULL;
	    int type = 0;

	    if(!strcmp(reason,SpxNenterAffCallback))
	    {
		psw = &scbr->enteraff;
		nsw = scbr->nb_enteraff;
		xeh = (XtEventHandler) SpxEnterAffCB;
		type = 4;
	    }
	    else if(!strcmp(reason,SpxNleaveAffCallback))
	    {
		psw = &scbr->leaveaff;
		nsw = scbr->nb_leaveaff;
		xeh = (XtEventHandler) SpxLeaveAffCB;
		type = 5;
	    }
	    else if(!strcmp(reason,SpxNbpressAffCallback))
	    {
		psw = &scbr->bpressaff;
		nsw = scbr->nb_bpressaff;
		xeh = (XtEventHandler) SpxBpressAffCB;
		type = 6;
	    }
	    else if(!strcmp(reason,SpxNbpressbaseAffCallback))
	    {
		psw = &scbr->bpressaff;
		nsw = scbr->nb_bpressaff;
		xeh = (XtEventHandler) SpxBpressAffCB;
		type = 16;
	    }
	    else if(!strcmp(reason,SpxNbreleaseAffCallback))
	    {
		psw = &scbr->breleaseaff;
		nsw = scbr->nb_breleaseaff;
		xeh = (XtEventHandler) SpxBreleaseAffCB;
		type = 7;
	    }
	    else if(!strcmp(reason,SpxNbreleasebaseAffCallback))
	    {
		psw = &scbr->breleaseaff;
		nsw = scbr->nb_breleaseaff;
		xeh = (XtEventHandler) SpxBreleaseAffCB;
		type = 17;
	    }
	    else if(!strcmp(reason,SpxNmnotifyAffCallback))
	    {
		psw = &scbr->mnotifyaff;
		nsw = scbr->nb_mnotifyaff;
		xeh = (XtEventHandler) SpxMnotifyAffCB;
		type = 8;
	    }
	    else if(!strcmp(reason,SpxNmnotifybaseAffCallback))
	    {
		psw = &scbr->mnotifyaff;
		nsw = scbr->nb_mnotifyaff;
		xeh = (XtEventHandler) SpxMnotifyAffCB;
		type = 18;
	    }

	    sw = *psw;

	    for(i=0;i<nsw;i++)
	    {

		if(sw[i]->proc == proc && 
				sw[i]->clientdata == clientdata &&
				sw[i]->wid == w &&
				!strcmp(sw[i]->reason,reason))
		{
		    break;
		}

	    }

	    if(i<nsw)
	    {
		temp[0] = type;
		XtRemoveEventHandler(w,NoEventMask,True,xeh,(XtPointer) sw[i]);

		free((char*) sw[i]);

		sw[i] = sw[--nsw];

		if(nsw)
		{
		    *psw = (SpxWindow**) realloc((char*) sw,
				    nsw*sizeof(SpxWindow*));
		}
		else
		{
		    free((char*) sw);
		    *psw = NULL;
		}

		temp[1] = 0;
		fnxwriti(file_fich,2,temp);
		fnxwrite(file_fich,"@",1);
		fnxflush(file_fich);
	    }

	}

}

unsigned
SpxSetCurseur(unsigned curseur,SPXFILE* spx_file)
{

	if(spx_file && spx_file->type == SPXSOCKET)
	{
	    int temp[2];
	    FILE* file_fich = spx_file->file;
	    int cur = 0;

	    temp[0] = 9;
	    temp[1] = (int) curseur;
	    fnxwriti(file_fich,2,temp);
            fnxflush(file_fich);
	    fnxreadi(file_fich,1,&cur);
	    fnxwrite(file_fich,"@",1);
            fnxflush(file_fich);

	    return (unsigned) cur;
	}
	else
	{
	    return (unsigned) -1;
	}

}

/************************************************************
  asservissement de la position de l'afficheur a la position
  du client w decalee de decalx et decaly
  ***********************************************************/

typedef struct
{
	SPXFILE* spxfile;
	int decalx;
	int decaly;
	Widget w;
} _SpxPosition;

static _SpxPosition** decal = NULL;
static unsigned nb_decal = 0;

static void
_SpxPositionContrainte(Widget w,XtPointer client_data,XEvent* event,
		Boolean* ptb)
{

	if(event->type == ConfigureNotify)
	{
	    _SpxPosition* const decalt = (_SpxPosition*) client_data;

	    const int x = event->xconfigure.x + decalt->decalx;
	    const int y = event->xconfigure.y + decalt->decaly;

	    SpxSetPosabsAfficheur(x,y,decalt->spxfile);
	}

}

void
SpxSetPosAfficheur(Widget w,int decalx,int decaly,SPXFILE* spx_file)
{
	Position x = 0,y = 0;
	int xpos,ypos;

	XtVaGetValues(w,XtNx,&x,XtNy,&y,NULL);
	xpos = x + decalx;
	ypos = y + decaly;
	SpxSetPosabsAfficheur(xpos,ypos,spx_file);
}

SpxPositionId
SpxAddPosAfficheur(Widget w,int decalx,int decaly,SPXFILE* spx_file)
{
	SpxSetPosAfficheur(w,decalx,decaly,spx_file);

	if(!decal)
	{
	    decal = (_SpxPosition**) malloc(sizeof(_SpxPosition*));
	    nb_decal = 1;
	}
	else
	{
	    decal = (_SpxPosition**) realloc((char*) decal,
			    ++nb_decal*sizeof(_SpxPosition*));
	}

	if(!decal)
	{
	    fprintf(stderr,"Pb alloc %s\n",__FILE__);
	    nb_decal = 0;
	    return -1;
	}

	decal[nb_decal-1] = (_SpxPosition*) malloc(sizeof(_SpxPosition));

	if(!decal[nb_decal-1])
	{
	    fprintf(stderr,"Pb alloc %s\n",__FILE__);
	    return -1;
	}

	{
	    _SpxPosition* const decal_temp =  decal[nb_decal-1];
 
	    spx_file->close_defer++;

	    decal_temp->decalx = decalx;
	    decal_temp->decaly = decaly;
	    decal_temp->w = w;
	    decal_temp->spxfile = spx_file;

	    XtAddEventHandler(w,StructureNotifyMask,False,_SpxPositionContrainte,
			    (XtPointer) decal_temp);
	}

	return nb_decal;
}

void
SpxRemovePosAfficheur(SpxPositionId spi)
{
	unsigned i = spi - 1;

	if(i<nb_decal && decal[i])
	{
	    unsigned j = 0;

	    XtRemoveEventHandler(decal[i]->w,StructureNotifyMask,False,
			    _SpxPositionContrainte,(XtPointer) decal[i]);
	    SpxClose(decal[i]->spxfile);
	    free((char*) decal[i]);
	    decal[i] = NULL;

	    while(j<nb_decal && !decal[j++]);

	    if(j == nb_decal)
	    {
		free((char*) decal);
		decal = NULL;
		nb_decal = 0;
	    }

	}

}

/****************************************************************
  asservissement du mapping de l'afficheur a l'expose du client
  ***************************************************************/

typedef struct
{
	SPXFILE* spxfile;
	Widget w;
} _SpxMap;

static _SpxMap** map = NULL;
static unsigned nb_map = 0;

static void
_SpxMapContrainte(Widget w,XtPointer client_data,XEvent* event,
		Boolean* ptb)
{

	if(event->type == VisibilityNotify)
	{
	    _SpxMap* const mapt = (_SpxMap*) client_data;

	    switch(event->xvisibility.state)
	    {
	      case 0 :
		SpxMapAfficheur(mapt->spxfile);
		break;
	      case 1 :
		break;
	      case 2 :
		SpxUnmapAfficheur(mapt->spxfile);
		break;
	    }

	}

}

SpxMapId
SpxAddMapAfficheur(Widget w,SPXFILE* spx_file)
{

	if(!map)
	{
	    map = (_SpxMap**) malloc(sizeof(_SpxMap*));
	    nb_map = 1;
	}
	else
	{
	    map = (_SpxMap**) realloc((char*) map,++nb_map*sizeof(_SpxMap*));
	}

	if(!map)
	{
	    fprintf(stderr,"Pb alloc %s\n",__FILE__);
	    nb_map = 0;
	    return -1;
	}

	map[nb_map-1] = (_SpxMap*) malloc(sizeof(_SpxMap));

	if(!map[nb_map-1])
	{
	    fprintf(stderr,"Pb alloc %s\n",__FILE__);
	    return -1;
	}

	{
	    _SpxMap* const map_temp =  map[nb_map-1];
 
	    spx_file->close_defer++;

	    map_temp->w = w;
	    map_temp->spxfile = spx_file;

	    XtAddEventHandler(w,VisibilityChangeMask,False,_SpxMapContrainte,
			    (XtPointer) map_temp);
	}

	return nb_map;
}

void
SpxRemoveMapAfficheur(SpxMapId spi)
{

	if(spi > 0)
	{
	    const unsigned i = spi - 1;

	    if(i<nb_map && map[i])
	    {
		unsigned j = 0;

		XtRemoveEventHandler(map[i]->w,VisibilityChangeMask,False,
				_SpxMapContrainte,(XtPointer) map[i]);
		SpxClose(map[i]->spxfile);
		free((char*) map[i]);
		map[i] = NULL;

		while(j<nb_map && !map[j++]);

		if(j == nb_map)
		{
		    free((char*) map);
		    map = NULL;
		    nb_map = 0;
		}

	    }

	}

}

#else /* !X11 */

#include <stdio.h>
#include "SpxCallback.h"
#include "SpxOpenP.h"
#include "nx.h"

void
SpxAddCallback(Widget w,const char* reason,XtCallbackProc proc,
		XtPointer clientdata,SPXFILE* spx_file)
{
	fprintf(stderr,"SpxAddCallback de la librairie Narcisse inactivee\n");
}

void
SpxRemoveCallback(Widget w,const char* reason,XtCallbackProc proc,
		XtPointer clientdata,SPXFILE* spx_file)
{
	fprintf(stderr,"SpxRemoveCallback de la librairie Narcisse inactivee\n");
}

unsigned
SpxSetCurseur(unsigned curseur,SPXFILE* spx_file)
{
	fprintf(stderr,"SpxSetCurseur de la librairie Narcisse inactivee\n");

	return 0;
}

void
SpxSetPosrelAfficheur(Widget w,int decalx,int decaly,SPXFILE* spx_file)
{
	fprintf(stderr,"SpxSetPosrelAfficheur de la librairie Narcisse "
			"inactivee\n");
}

SpxPositionId
SpxAddPosAfficheur(Widget w,int decalx,int decaly,SPXFILE* spx_file)
{
	fprintf(stderr,"SpxAddPosAfficheur de la librairie Narcisse inactivee\n");

	return 0;
}

void
SpxRemovePosAfficheur(SpxPositionId spi)
{
	fprintf(stderr,"SpxRemovePosAfficheur de la librairie "
			"Narcisse inactivee\n");
}

SpxMapId
SpxAddMapAfficheur(Widget w,SPXFILE* spx_file)
{
	fprintf(stderr,"SpxAddMapAfficheur de la librairie Narcisse inactivee\n");

	return 0;
}

void
SpxRemoveMapAfficheur(SpxMapId spi)
{
	fprintf(stderr,"SpxRemoveMapAfficheur de la librairie "
			"Narcisse inactivee\n");
}

#endif /* fin !X11 */

void
SpxSortie(SPXFILE* spx_file)
{

	if(spx_file && spx_file->type == SPXSOCKET)
	{
	    int temp[2];
	    FILE* file_fich = spx_file->file;

	    temp[0] = 10;
	    temp[1] = 555;
	    fnxwriti(file_fich,2,temp);
	    fnxclose(file_fich);
	}

}

void
SpxSetPosabsAfficheur(int x,int y,SPXFILE* spx_file)
{

	if(spx_file && spx_file->type == SPXSOCKET)
	{
	    int temp[3];
	    FILE* file_fich = spx_file->file;

	    temp[0] = 1001;
	    temp[1] = x;
	    temp[2] = y;
	    fnxwriti(file_fich,3,temp);
	    fnxwrite(file_fich,"@",1);
            fnxflush(file_fich);
	}

}

void
SpxMapAfficheur(SPXFILE* spx_file)
{

	if(spx_file && spx_file->type == SPXSOCKET)
	{
	    int temp;
	    FILE* file_fich = spx_file->file;

	    temp = 1002;
	    fnxwriti(file_fich,1,&temp);
	    fnxwrite(file_fich,"@",1);
            fnxflush(file_fich);
	}

}

void
SpxUnmapAfficheur(SPXFILE* spx_file)
{

	if(spx_file && spx_file->type == SPXSOCKET)
	{
	    int temp;
	    FILE* file_fich = spx_file->file;

	    temp = 1003;
	    fnxwriti(file_fich,1,&temp);
	    fnxwrite(file_fich,"@",1);
            fnxflush(file_fich);
	}

}
