/***********************************************************/
/*   routines d'ouvertures et de fermetures de connexions  */
/*   ou fichiers de la librairie Narcisse                    */
/***********************************************************/

#ifndef lint
    static const char* const sccs_id = "%W% %G% Pierre Brochard";
#endif

/***********************************************************/
/*   routines d'ouvertures et de fermetures de connexions  */
/*   ou fichiers de la librairie Narcisse : declarations     */
/***********************************************************/

#include "SpxP.h"
#include "nx.h"
#include "nxsystem.h"
#include "SpxOpen.h"
#include "SpxOpenP.h"
#include "SpxVisuDri.h"
#include "U8const.h"

/***********************************************************/
/*   routines d'ouvertures et de fermetures de connexions  */
/*   ou fichiers de la librairie Narcisse : definitions      */
/***********************************************************/

static int
SpxInit(const char* nom_fichier,char* nomfich,unsigned* type,
		const char* sp_handler,int debug);

SPXFILE*
SpxOpen(char* nom_fichier)
{
	char nomfich[MAX_FILENAME_LENGTH];
	unsigned type = SPXSOCKET;
	FILE* file_fich = NULL;
	SPXFILE* spx_file = NULL;
	const int debug_sp3 = getenv("DEBUG_SP3") != NULL;
	const int sync_sp3 = getenv("SYNC_SP3") != NULL;

	SpxInit(nom_fichier,nomfich,&type,"spx",debug_sp3);

	if(type == SPXSOCKET)
	{
	    unsigned compt = 0;
	    const int temp = -5;

	    while(!file_fich)
	    {
		file_fich = fnxopen(nomfich,"w");

		if(debug_sp3)
		{
		    printf("SpxOpen> fnxopen appele, file_fich = %p\n",
				    file_fich);
		}

		if(!file_fich)
		{
		    fnxperror("Demande de connexion");
		    sleep(2);
		    compt++;

		    if(compt >= 10)
		    {
			fprintf(stderr,"Ouvrer Narcisse\nsinon "
					"attendez qu'il soit moins occupe\n");

			return NULL;
		    }
		    else
			SpxInit(nom_fichier,nomfich,&type,"spx",debug_sp3);

		}

	    }

	    fnxwriti(file_fich,1,&temp);
	    fnxwrite(file_fich,"@",1);
            fnxflush(file_fich);
	}

	spx_file = (SPXFILE*) malloc(sizeof(SPXFILE));

	if(!spx_file)
	{
	    fprintf(stderr,"Pb allocation de memoire : SpxOpen a echoue\n");

	    return NULL;
	}

	memset((char*) spx_file,0,sizeof(SPXFILE));

	spx_file->file = file_fich;
	spx_file->type = type;
	strcpy(spx_file->nom_fichier,nomfich);
	spx_file->debug = debug_sp3;
	spx_file->sync = sync_sp3;
	spx_file->spxarg = NULL;
	spx_file->spxarg_taille = 0;

	return spx_file;
}

int
SpxDebut(SPXFILE* spx_file)
{
	int type;

	if(!spx_file) return -1;

	type = spx_file->type;

	if(type == SPXVISU)
	{
	    char nomfich[MAX_FILENAME_LENGTH];
	    char path[MAX_FILENAME_LENGTH];
	    char* const suffixe = ".spx";

	    memset(path,0,sizeof(path));

	    SpxOpenFichier(path,suffixe);
	    strcpy(nomfich,path);
	    strcat(nomfich,"@ie32");

	    spx_file->file = fnxopen(nomfich,"w");

	    if(!spx_file->file)
	    {
		fprintf(stderr,"Pb ouverture fichier : SpxDebut a echoue\n");

		return -1;
	    }

	    strcpy(spx_file->path,path);
	}
	else if(type == SPXFICHIER || type == SPXFICHIERASCII)
	{
	    char file_temp[MAX_FILENAME_LENGTH];
	    char* const nomfich = spx_file->nom_fichier;
	    char* marqueur = type == SPXFICHIER ? strstr(nomfich,".spx") :
		strstr(nomfich,".sp3");
	    const int numero = spx_file->compteur_fichier +
		spx_file->compteur_fichier_rc;
	    int delta;

	    if(!marqueur)
	    {
		marqueur = strchr(nomfich,'@');
	    }

	    if(marqueur)
	    {
		delta = marqueur-nomfich;
	    }
	    else
	    {
		delta = strlen(nomfich);
	    }

	    memcpy(file_temp,nomfich,delta);
	    file_temp[delta] = '\0';
	    sprintf(file_temp+delta,"%5d",numero);
	    SpxRemplaceBlanc(file_temp+delta);

	    if(marqueur) strcat(file_temp,marqueur);

	    spx_file->file = fnxopen(file_temp,"w");

	    if(!spx_file->file)
	    {
		fprintf(stderr,"Pb ouverture fichier : SpxDebut a echoue\n"
			"file_temp = %s\n",file_temp);

		return -1;
	    }

	}

	spx_file->is_rc_last = 0;

	return 0;
}

void
SpxFin(SPXFILE* spx_file)
{
	int type;

	if(!spx_file) return;

	type = spx_file->type;

	if(type == SPXVISU)
	{

	    if(spx_file->file)
	    {
		fnxclose(spx_file->file);
		spx_file->file = NULL;
	    }

	    SpxFermeFichier(spx_file->path);
	}
	else if(type == SPXFICHIER || type == SPXFICHIERASCII)
	{

	    if(spx_file->file)
	    {
		fnxclose(spx_file->file);
		spx_file->file = NULL;
		spx_file->compteur_fichier++;
	    }

	}

}

void
SpxClose(SPXFILE* spx_file)
{

	if(!spx_file) return;

	if(spx_file->close_defer)
	{
	    spx_file->close_defer--;
	}
	else
	{
	    const int type = spx_file->type;
	    const int compteur_somme =  spx_file->compteur_fichier +
		spx_file->compteur_fichier_rc;

	    if(spx_file->sync)
	    {
		SpxSync(spx_file);
	    }

	    if(spx_file->file)
	    {
		fnxclose(spx_file->file);
		spx_file->file = NULL;
	    }
	    
	    if(type == SPXFICHIERCOL)
	    {
		const int tfcol = spx_file->compteur_fichier_f_col ==  1;
		const int tccol = spx_file->compteur_fichier_c_col ==  1;
		const int tpcol = spx_file->compteur_fichier_p_col ==  1;

		if(tfcol || tccol || tpcol)
		{
		    char file_temp_old[MAX_FILENAME_LENGTH];
		    char file_temp[MAX_FILENAME_LENGTH];
		    char* const nomfich = spx_file->nom_fichier;
		    char* marqueur = strstr(nomfich,".col");
		    int delta;

		    if(!marqueur)
		    {
			marqueur = strchr(nomfich,'@');
		    }

		    if(marqueur)
		    {
			delta = marqueur-nomfich;

			if(!memcmp(nomfich+delta-2,".c",2) ||
				!memcmp(nomfich+delta-2,".f",2) ||
				!memcmp(nomfich+delta-2,".p",2))
			    delta -= 2;

		    }
		    else
		    {
			delta = strlen(nomfich);
		    }

		    memcpy(file_temp,nomfich,delta);
		    file_temp[delta] = '\0';
		    memcpy(file_temp_old,file_temp,delta+1);

		    if(tfcol)
		    {
			memcpy(file_temp_old+delta,"00000.f",8);
			memcpy(file_temp+delta,".f",3);
			strcat(file_temp,".col");
			strcat(file_temp_old,".col");
			rename(file_temp_old,file_temp);
		    }

		    if(tccol)
		    {
			memcpy(file_temp_old+delta,"00000.c",8);
			memcpy(file_temp+delta,".c",3);
			strcat(file_temp,".col");
			strcat(file_temp_old,".col");
			rename(file_temp_old,file_temp);
		    }

		    if(tpcol)
		    {
			memcpy(file_temp_old+delta,"00000.p",8);
			memcpy(file_temp+delta,".p",3);
			strcat(file_temp,".col");
			strcat(file_temp_old,".col");
			rename(file_temp_old,file_temp);
		    }

		}

	    }
	    else if((type == SPXFICHIER || type == SPXFICHIERASCII) &&
			    (compteur_somme == 1 ||
					    (spx_file->compteur_fichier == 1 &&
					    spx_file->compteur_fichier_rc == 1)))
	    {
		char file_temp_old[MAX_FILENAME_LENGTH];
		char file_temp[MAX_FILENAME_LENGTH];
		char* nomfich = spx_file->nom_fichier;
		char* nomfich_temp;
		char* marqueur;
		int delta;
		char* ctemp;
		char* const chnum = spx_file->is_rc_last || compteur_somme == 1 ?
		    "00000" : "00001";
		char* const chnum_rc =
		    spx_file->is_rc_last && compteur_somme != 1 ?
		    "00001" : "00000";

		if((ctemp = strchr(nomfich,'@')))
		{
		    *ctemp = '\0';
		}

		if(nomfich && nomfich[0] == '|')
		{
		    char* const csupt = strchr(nomfich,'>');
		    char* const cblat = strrchr(nomfich,' ');

		    nomfich_temp = csupt ? csupt + 1 :
			(cblat ? cblat + 1 : nomfich);
		}
		else
		{
		    nomfich_temp = nomfich;
		}

		ctemp = strrchr(nomfich_temp,' ');
		nomfich = ctemp ? ctemp + 1 : nomfich_temp;
		marqueur = type == SPXFICHIER ? strstr(nomfich,".spx") :
		    strstr(nomfich,".sp3");

		if(!marqueur)
		{
		    marqueur = strchr(nomfich,'@');
		}

		if(marqueur)
		{
		    delta = marqueur-nomfich;
		}
		else
		{
		    delta = strlen(nomfich);
		}

		strcpy(file_temp,nomfich);
		memcpy(file_temp_old,file_temp,delta+1);
		memcpy(file_temp_old+delta,chnum,6);

		if(marqueur) strcat(file_temp_old,marqueur);

		if(spx_file->compteur_fichier == 1)
		{
		    rename(file_temp_old,file_temp);
		}

		if(spx_file->compteur_fichier_rc == 1)
		{

		    if((ctemp = strstr(file_temp,".Z")))
		    {
			*ctemp = '\0';
		    }
		    else if((ctemp = strstr(file_temp,".gz")))
		    {
			*ctemp = '\0';
		    }

		    strcat(file_temp,"rc");
		    memcpy(file_temp_old,file_temp,delta+1);
		    memcpy(file_temp_old+delta,chnum_rc,6);
		    strcat(file_temp_old,file_temp+delta);
		    rename(file_temp_old,file_temp);
		}

	    }
	    else if(type == SPXVISU)
	    {
		SpxFermeFichier(spx_file->path);
	    }
	    else if(type == SPXFICHIERUCD)
	    {
		const int tucd = spx_file->compteur_fichier_ucd ==  1;

		if(tucd)
		{
		    char file_temp_old[MAX_FILENAME_LENGTH];
		    char file_temp[MAX_FILENAME_LENGTH];
		    char* const nomfich = spx_file->nom_fichier;
		    char* marqueur = strstr(nomfich,".inp");
		    int delta;

		    if(!marqueur)
		    {
			marqueur = strchr(nomfich,'@');
		    }

		    if(marqueur)
		    {
			delta = marqueur-nomfich;
		    }
		    else
		    {
			delta = strlen(nomfich);
		    }

		    memcpy(file_temp,nomfich,delta);
		    file_temp[delta] = '\0';
		    memcpy(file_temp_old,file_temp,delta+1);
		    memcpy(file_temp_old+delta,"00000",6);
		    strcat(file_temp,".inp");
		    strcat(file_temp_old,".inp");
		    rename(file_temp_old,file_temp);
		}

	    }

	    SpxVPurgeArg(spx_file);
	    free(spx_file);
	}

}

void
SpxSync(SPXFILE* spx_file)
{

	if(!spx_file) return;

	if(spx_file->type == SPXSOCKET)
	{
	    int temp = -4;
	    FILE* file_fich = spx_file->file;

	    fnxwriti(file_fich,1,&temp);
            fnxflush(file_fich);
	    fnxreadi(file_fich,1,&temp);
	    fnxwrite(file_fich,"@",1);
            fnxflush(file_fich);

	    if(spx_file->debug)
	    {
		printf("SpxSync> synchro SPXSOCKET execute %d\n",temp);
	    }

	}
	else if(spx_file->type == SPXSOCKETCOL)
	{
	    char temp[128];
	    unsigned ltemp;
	    int syn;
	    FILE* file_fich = spx_file->file;

	    strcpy(temp,"###COL.sync###\n");
	    ltemp = strlen(temp);
	    fnxwrite(file_fich,temp,ltemp);
            fnxflush(file_fich);

	    if(spx_file->debug)
	    {
		printf("SpxSync> synchro SPXSOCKETCOL write fait\n");
	    }

	    fnxreadi(file_fich,1,&syn);

	    if(spx_file->debug)
	    {
		printf("SpxSync> synchro SPXSOCKETCOL execute %d\n",syn);
	    }

	}

}

SPXFILE*
SpcOpen(char* nom_fichier)
{
	char nomfich[MAX_FILENAME_LENGTH];
	unsigned type = SPXSOCKET;
	FILE* file_fich = NULL;
	SPXFILE* spx_file = NULL;
	const int debug_sp3 = getenv("DEBUG_SP3") != NULL;

	SpxInit(nom_fichier,nomfich,&type,"spc",debug_sp3);

	if(type == SPXSOCKET)
	{
	    unsigned compt = 0;
	    const int temp = -5;

	    while(!file_fich)
	    {
		file_fich = fnxopen(nomfich,"w");

		if(!file_fich)
		{
		    fnxperror("Demande de connexion");
		    sleep(2);
		    compt++;

		    if(compt >= 10)
		    {
			fprintf(stderr,"Ouvrer Narcisse \n");
			fprintf(stderr,
				"Sinon attendez qu'il soit moins occupe\n");
			return NULL;
		    }
		    else
			SpxInit(nom_fichier,nomfich,&type,"spc",debug_sp3);

		}

	    }

	    fnxwriti(file_fich,1,&temp);
	    fnxwrite(file_fich,"@",1);
            fnxflush(file_fich);
	}
	else
	{
	    type = SPXFICHIER;
	    nomfich[0] = '\0';
	}

	spx_file = (SPXFILE*) malloc(sizeof(SPXFILE));

	if(!spx_file)
	{
	    fprintf(stderr,"Pb allocation de memoire : SpcOpen a echoue\n");

	    return NULL;
	}

	memset((char*) spx_file,0,sizeof(SPXFILE));

	spx_file->file = file_fich;
	spx_file->type = type;
	strcpy(spx_file->path,"");
	strcpy(spx_file->nom_fichier,nomfich);
	spx_file->debug = debug_sp3;
	spx_file->spxarg = NULL;
	spx_file->spxarg_taille = 0;

	return spx_file;
}

/****************************************************************/
/*   Open ou le handler name vaut sp3                           */
/*   on n'envoit pas le code -5                                 */
/****************************************************************/

SPXFILE*
Sp3Open(char* nom_fichier)
{
	char nomfich[MAX_FILENAME_LENGTH];
	unsigned type = SPXSOCKET;
	FILE* file_fich = NULL;
	SPXFILE* spx_file = NULL;
	const int debug_sp3 = getenv("DEBUG_SP3") != NULL;

	SpxInit(nom_fichier,nomfich,&type,"sp3",debug_sp3);

	if(type == SPXSOCKET)
	{
	    unsigned compt = 0;

	    while(!file_fich)
	    {
		file_fich = fnxopen(nomfich,"w");

		if(!file_fich)
		{
		    fnxperror("Demande de connexion");
		    sleep(2);
		    compt++;

		    if(compt >= 10)
		    {
			fprintf(stderr,"Ouvrer Narcisse \nsinon ");
			fprintf(stderr,"attendez qu'il soit moins occupe \n");

			return NULL;
		    }
		    else
			SpxInit(nom_fichier,nomfich,&type,"sp3",debug_sp3);

		}

	    }

	}

	spx_file = (SPXFILE*) malloc(sizeof(SPXFILE));

	if(!spx_file)
	{
	    fprintf(stderr,"Pb allocation de memoire : Sp3Open a echoue\n");

	    return NULL;
	}

	memset((char*) spx_file,0,sizeof(SPXFILE));

	spx_file->file = file_fich;
	spx_file->type = type;
	strcpy(spx_file->nom_fichier,nomfich);
	spx_file->debug = debug_sp3;
	spx_file->spxarg = NULL;
	spx_file->spxarg_taille = 0;

	return spx_file;
}

/****************************************************************/
/*   Open ou le handler name n'est pas modifie                  */
/*   ou vaut spx s'il n'existe pas                              */
/*   Si on a affaire a un fichier ordinaire on ne rajoute pas   */
/*   de suffixe                                                 */
/*   on n'envoit pas le code -5                                 */
/*   utilise en particulier dans psycp (main de nxstream)       */
/****************************************************************/

SPXFILE*
SpetoileOpen(char* nom_fichier)
{
	char nomfich[MAX_FILENAME_LENGTH];
	unsigned type = SPXSOCKET;
	FILE* file_fich = NULL;
	SPXFILE* spx_file = NULL;
	const int debug_sp3 = getenv("DEBUG_SP3") != NULL;

	SpxInit(nom_fichier,nomfich,&type,NULL,debug_sp3);

	if(type == SPXSOCKET)
	{
	    unsigned compt = 0;

	    while(!file_fich)
	    {
		file_fich = fnxopen(nomfich,"w");

		if(!file_fich)
		{
		    fnxperror("Demande de connexion");
		    sleep(2);
		    compt++;

		    if(compt >= 10)
		    {
			fprintf(stderr,"Ouvrer Narcisse \nsinon ");
			fprintf(stderr,"attendez qu'il soit moins occupe \n");

			return NULL;
		    }
		    else
			SpxInit(nom_fichier,nomfich,&type,NULL,debug_sp3);

		}

	    }

	}

	spx_file = (SPXFILE*) malloc(sizeof(SPXFILE));

	if(!spx_file)
	{
	    fprintf(stderr,"Pb allocation de memoire : SpetoileOpen a echoue\n");

	    return NULL;
	}

	memset((char*) spx_file,0,sizeof(SPXFILE));

	spx_file->file = file_fich;
	spx_file->type = type;
	strcpy(spx_file->nom_fichier,nomfich);
	spx_file->debug = debug_sp3;
	spx_file->spxarg = NULL;
	spx_file->spxarg_taille = 0;

	return spx_file;
}

/****************************************************************/
/*   Open ou le handler name vaut col pour envoit de cartes     */
/*   de couleurs au format COL                                  */
/*   on n'envoit pas le code -5                                 */
/****************************************************************/

SPXFILE*
SpcolOpen(char* nom_fichier)
{
	char nomfich[MAX_FILENAME_LENGTH];
	unsigned type = SPXSOCKET;
	FILE* file_fich = NULL;
	SPXFILE* spx_file = NULL;
	const int debug_sp3 = getenv("DEBUG_SP3") != NULL;
	const int sync_sp3 = getenv("SYNC_SP3") != NULL;

	SpxInit(nom_fichier,nomfich,&type,"col",debug_sp3);
	printf("Filename = %s, %s\n",nom_fichier,nomfich);

	if(type == SPXSOCKET)
	{
	    unsigned compt = 0;

	    while(!file_fich)
	    {
		file_fich = fnxopen(nomfich,"w");

		if(!file_fich)
		{
		    fnxperror("Demande de connexion");
		    sleep(2);
		    compt++;

		    if(compt >= 10)
		    {
			fprintf(stderr,"Ouvrer Narcisse \nsinon "
					"attendez qu'il soit moins occupe\n");

			return NULL;
		    }
		    else
			SpxInit(nom_fichier,nomfich,&type,"col",debug_sp3);

		}

	    }

	    type = SPXSOCKETCOL;
	}

	spx_file = (SPXFILE*) malloc(sizeof(SPXFILE));

	if(!spx_file)
	{
	    fprintf(stderr,"Pb allocation de memoire : SpcolOpen a echoue\n");

	    return NULL;
	}

	memset((char*) spx_file,0,sizeof(SPXFILE));

	spx_file->file = file_fich;
	spx_file->type = type;
	strcpy(spx_file->nom_fichier,nomfich);
	spx_file->debug = debug_sp3;
	spx_file->sync = sync_sp3;
	spx_file->spxarg = NULL;
	spx_file->spxarg_taille = 0;

	return spx_file;
}

/**********************************************************************/
/*      fonction SpxInit (decodage du nom de fichier)                 */
/*      le code retour indique la presence du serveur Narcisse :        */
/*          1 => Narcisse existe a coup sur (il faut "NARCISSE.n"         */
/*               dans le nom du fichier ou comme champ port de        */
/*               DEST_SP3)                                            */
/*          -1 => Narcisse n'existe pas a coup sur                      */
/*          0 => Narcisse existe peut-etre (on ne verifie pas)          */
/*      nom_fichier est le nom fourni en entree.                      */
/*          Il est complete par la variable d'environnement DEST_SP3  */
/*      nomfich est le nom fourni en sortie                           */
/*      type est le type de fichier fourni en sortie                  */
/*      sp_handler est le handler que l'on veut (entree)              */
/**********************************************************************/

static int
SpxInit(const char* nom_fichier,char* nomfich,unsigned* type,
		const char* sp_handler,int debug)
{
	char* const suffixe = ".spx";
	char* const suffixe_ascii = ".sp3";
	char* const suffixe_col = ".col";
	int testsock = 0;
	int testdest_sp3 = 0;
	char nomfich_temp[MAX_FILENAME_LENGTH];
	char* exist_format = NULL;
	int ier = 0;
	int numero = 1;
	static char* nom_code =
		"\141\0\162\21\170\13\170\35\75\35"
		"\75\35\75\35\75\35\75\35\75\35"
		"\75\35\75\35\75\35\75\35\75\35"
		"\75\35\75\35\75\35\75\35\75\35"
		"\75\35\75\35\75\35\75\35\75\35"
		"\75\35\75\35\75\35\75\35\75\35"
		"\75\35\75\35\75\35\75\35\75\35"
		"\75\35\75\35\75\35\75\35\75\35";
	const int inom_logiciel =
	    nom_code[0] == '\141' && nom_code[1] == '\0' &&
	    nom_code[2] == '\162' && nom_code[3] == '\21' &&
	    nom_code[4] == '\170' && nom_code[5] == '\13' &&
	    nom_code[6] == '\170' && nom_code[7] == '\35' &&
	    nom_code[8] == '\75' && nom_code[9] == '\35' &&
	    nom_code[10] == '\75' && nom_code[11] == '\35';
	char nom_logiciel[10];
	unsigned lnom_logiciel;
	unsigned lnom_logiciel1;

	{
	    char* cc;
	    unsigned taille;
	    char mask[2];
	    unsigned i = 0;

	    if(!inom_logiciel)
	    {
		cc = "\156\75\144\47\157\52\4";
		mask[0] = 0x3e;
	    }
	    else
	    {
		cc = "\75\174\56\155\44\167\44\141\117";
		mask[0] = 0x73;
	    }

	    taille = strlen(cc);
	    nom_logiciel[taille] = '\0';
	    mask[1] = '\0';

	    for(;i<taille;i++)
	    {
		mask[0] = mask[0] ^ cc[i];
		nom_logiciel[i] = mask[0];
		mask[0] = cc[i];
	    }

	}

	if(debug)
	{
	    printf("SpxInit> nom_logiciel : %s\n",nom_logiciel);
	}

	lnom_logiciel = strlen(nom_logiciel);
	lnom_logiciel1 = lnom_logiciel + 1;
 	nomfich_temp[0] = '\0';

	if(nom_fichier)
	{
	    strcpy(nomfich_temp,nom_fichier);
	}

	if((exist_format = strchr(nomfich_temp,'@')))
	{
	    *exist_format = '\0';
	}

	testdest_sp3 = !nomfich_temp || (nomfich_temp &&
		(!strlen(nomfich_temp) ||
		(strlen(nomfich_temp) && nomfich_temp[0] == ' ') ||
		(strlen(nomfich_temp) >= lnom_logiciel1 &&
		!strncmp(nomfich_temp,nom_logiciel,lnom_logiciel))));
	testsock = !nomfich_temp || (nomfich_temp && ((*nomfich_temp != '|' &&
		strpbrk(nomfich_temp,"+: ")) || !strlen(nomfich_temp) ||
		(strlen(nomfich_temp) && nomfich_temp[0] == ' ') ||
		(strlen(nomfich_temp) >= lnom_logiciel1 &&
		!strncmp(nomfich_temp,nom_logiciel,lnom_logiciel))));
 	nomfich_temp[0] = '\0';

	if(testdest_sp3 && getenv("DEST_SP3"))
	{
	    strcpy(nomfich_temp,getenv("DEST_SP3"));

	    if((exist_format = strchr(nomfich_temp,'@')))
	    {
		*exist_format = '\0';
	    }

	    testsock = nomfich_temp && ((*nomfich_temp != '|' &&
		strpbrk(nomfich_temp,"+: ")) || !strlen(nomfich_temp) ||
		(strlen(nomfich_temp) && nomfich_temp[0] == ' '));
	    strcpy(nomfich_temp,getenv("DEST_SP3"));
	}
	else if(testdest_sp3)
	  strcpy(nomfich_temp,"@ie32");
	else
	  strcpy(nomfich_temp,nom_fichier);

	if(testsock)
	{
	    char machine[136];
	    char port[136];
	    char handler[136];
	    char name[L_U8GetUser];
	    char r_name[L_U8GetUser];
	    char format[136];
	    char* marqueur = NULL;
	    char* marqueur2 = NULL;
	    int calcul_port = 0;
	    int calcul_r_name = 0;
	    int calcul_handler = 0;

	    machine[0] = '\0';
	    port[0] = '\0';
	    handler[0] = '\0';
	    name[0] = '\0';
	    r_name[0] = '\0';
	    format[0] = '\0';

	/* calcul format (chaine @format) : il faut que le serveur Narcisse
	   ait le meme format ou le format par defaut. */

	    if((marqueur = strchr(nomfich_temp,'@')))
	    {
		strcpy(format,marqueur);
		*marqueur = '\0';
	    }

	/* calcul du handler */

	    if(sp_handler && strlen(sp_handler) != 0)
	    {
		strcpy(handler,sp_handler);
		calcul_handler = 1;
	    }

	/* calcul du nom de l'utilisateur */

	    {
		char username[L_U8GetUser];

		U8GetUser(username);
		strcpy(name,username);
	    }

	/* calcul hostname ... */

	    if(nom_fichier && strlen(nom_fichier) >= lnom_logiciel1 &&
			    !strncmp(nom_fichier,nom_logiciel,lnom_logiciel))
	    {
		calcul_port = 1;
	    }

	    marqueur = strpbrk(nomfich_temp,"+: ");

	    if(!marqueur || marqueur == nomfich_temp)
	    {
		U8GetHostname(machine,136);

		if(!marqueur)
		{
		    calcul_port = 1;
		    calcul_r_name = 1;
		}

	    }

	    if(marqueur)
	    {

		if(marqueur != nomfich_temp)
		{
		    *marqueur = '\0';
		    strcpy(machine,nomfich_temp);
		}

		*marqueur = '+';
		marqueur++;

		marqueur2 = strpbrk(marqueur,"+: ");

		if(marqueur2)
		{
		    *marqueur2 = '\0';
		}
		else
		{
		    calcul_r_name = 1;
		}

		if(!strlen(marqueur) ||
			(strlen(marqueur) == 1 && !atoi(marqueur)) ||
			(nom_fichier && strlen(nom_fichier) >= lnom_logiciel1 &&
			!strncmp(nom_fichier,nom_logiciel,lnom_logiciel)) ||
			(strlen(marqueur) >= lnom_logiciel1 &&
			!strncmp(marqueur,nom_logiciel,lnom_logiciel)))
		{
		    calcul_port = 1;
		}

		if(!calcul_port)
		{
		    strcpy(port,marqueur);
		}

		if(marqueur2)
		{
		    char* marqueur3 = NULL;

		    *marqueur2 = '+';
		    marqueur2++;
		    marqueur3 = strpbrk(marqueur2,"+: ");

		    if(marqueur3)
		    {
			char* marqueur4 = NULL;

			if(!calcul_handler)
			{
			    *marqueur3 = '\0';

			    if(strlen(marqueur2))
			    {
				strcpy(handler,marqueur2);
			    }
			    else
			    {
				strcpy(handler,"spx");
			    }

			    *marqueur3 = '+';
			}

			marqueur3++;

			if((marqueur4 = strpbrk(marqueur3,"@+: ")))
			{
			    *marqueur4 = '\0';
			}

			if(!strlen(marqueur3) || strlen(marqueur3) > L_U8GetUser)
			{
			    calcul_r_name = 1;
			}
			else
			{
			    strcpy(r_name,marqueur3);
			}

		    }
		    else
		    {
			calcul_r_name = 1;

			if(!calcul_handler)
			{

			    if(strlen(marqueur2) != 0)
			    {
				strcpy(handler,marqueur2);
			    }
			    else
			    {
				strcpy(handler,"spx");
			    }

			}

		    }

		    *marqueur2 = '\0';
		}

	    }

	/* calcul du remote name */

	    if(calcul_r_name)
	    {
		strcpy(r_name,name);
	    }

	/* calcul du port socket de Narcisse si inexistant ou 0 ou NARCISSE.n
	   sinon nx fera le travail tout seul et dans ce cas on n'a pas
	   le numero de Narcisse (a voir) */

	    if(calcul_port)
	    {
		int num_port = -1;
		int num_proc = -1;
		char port_name_reel[1024];
		int lport_name = 1024;
		char h_name[MAXHOSTNAMELEN];

		port_name_reel[0] = '\0';

		if(nom_fichier && strlen(nom_fichier) >= lnom_logiciel1 &&
			!strncmp(nom_fichier,nom_logiciel,lnom_logiciel))
		{
		    numero = atoi(nom_fichier+lnom_logiciel);
		}
		else if(marqueur && strlen(marqueur) >= lnom_logiciel1 &&
			!strncmp(marqueur,nom_logiciel,lnom_logiciel))
		{
		    numero = atoi(marqueur+lnom_logiciel);
		}

		if(debug)
		{
		    printf("SpxInit> appel de fnx_find_port\n");
		}

	    /* si meme machine => sockets UNIX sinon INET */

		gethostname(h_name,sizeof(h_name));

		if(!strcmp(h_name,machine)) /* UNIX */
		{
		    nom_logiciel[lnom_logiciel-1] = 'U';
		}
		else /* INET */
		{
		    nom_logiciel[lnom_logiciel-1] = '\0';
		}

	    /* recherche du port */

		if(fnx_find_port(machine,r_name,nom_logiciel,numero,
				&num_proc,&num_port,port_name_reel,
				&lport_name,&numero))
		{

		    if(nom_logiciel[lnom_logiciel-1] == 'U')
		    {
			nom_logiciel[lnom_logiciel-1] = '\0';

			if(fnx_find_port(machine,r_name,nom_logiciel,numero,
					&num_proc,&num_port,port_name_reel,
					&lport_name,&numero))
			{
			    ier = -1;
			    num_port = 2101;
			    lport_name = 1;
			}
			else
			{
			    ier = 1;
			}

		    }
		    else
		    {
			ier = -1;
			num_port = 2101;
			lport_name = 1;
		    }

		}
		else
		{
		    ier = 1;
		}

		nom_logiciel[lnom_logiciel-1] = '.';

		if(debug)
		{
		    printf("SpxInit> num_port %d\n",num_port);
		    printf("SpxInit> num_proc %d\n",num_proc);
		    printf("SpxInit> lport_name %d\n",lport_name);
		}

		if(lport_name > 1)
		{
		    sprintf(port,"\"%s\"",port_name_reel);
		}
		else
		{
		    sprintf(port,"%d",num_port);
		}

	    }

	    strcpy(nomfich,machine);
	    strcat(nomfich,"+");
	    strcat(nomfich,port);
	    strcat(nomfich,"+");
	    strcat(nomfich,handler);
	    strcat(nomfich,"+");
	    strcat(nomfich,name);
	    strcat(nomfich,format);

	    if(debug)
	    {
		printf("SpxInit> nomfich SPXSOCKET %s\n",nomfich);
	    }

	}
	else if(nomfich_temp && strlen(nomfich_temp) == 4 &&
		strcmp(nomfich_temp,"VISU") == 0)
	{
	    strcpy(nomfich,"VISU.spx");
	    *type = SPXVISU;

	    if(debug)
	    {
		printf("SpxInit> nomfich VISU\n");
	    }

	}
	else
	{
	    const int handler_col = sp_handler && !strcmp(sp_handler,"col");
	    char* test_col = strstr(nomfich_temp,suffixe_col);
	    char* test_spx = strstr(nomfich_temp,suffixe);
	    char* test_sp3 = strstr(nomfich_temp,suffixe_ascii);

	    if(!handler_col && !test_sp3 && !test_col)
	    {

		if((exist_format = strchr(nomfich_temp,'@')))
		{
		    *exist_format = '\0';
		}

		if(sp_handler && !test_spx)
		{
		    strcat(nomfich_temp,suffixe);
		}

		strcat(nomfich_temp,"@ie32");
		*type = SPXFICHIER;
	    }
	    else if(handler_col || test_col)
	    {

		if(test_spx)
		{
		    *test_spx = '\0';
		}

		if(test_sp3)
		{
		    *test_sp3 = '\0';
		}

		if(!test_col)
		{
		    strcat(nomfich_temp,suffixe_col);
		}

		*type = SPXFICHIERCOL;
	    }
	    else
	    {
		*type = SPXFICHIERASCII;
	    }

	    strcpy(nomfich,nomfich_temp);

	    if(debug)
	    {
		printf("SpxInit> nomfich %s\n",nomfich);
	    }

	}

	if(numero < 1) numero = 1;

	if(ier == 1) ier = numero;

	return ier;
}

/**********************************************************************/
/*      Fonction SpxWhich                                             */
/*      le code retour indique la presence du serveur Narcisse :        */
/*       >= 1 => Narcisse existe a coup sur (il faut "NARCISSE.n"         */
/*               ou rien ou 0 dans le nom du fichier ou comme champ   */
/*               port de DEST_SP3) et on renvoit le numero du Narcisse  */
/*               contacte                                             */
/*          0 => Narcisse existe mais on ne donne pas son numero        */
/*          -1 => Narcisse n'existe pas a coup sur                      */
/*      nom_fichier est le nom fourni en entree.                      */
/*          Il est complete par la variable d'environnement DEST_SP3  */
/**********************************************************************/

int
SpxWhich(const char* nom_fichier)
{
	char nomfich[MAX_FILENAME_LENGTH];
	unsigned type = SPXSOCKET;
	const int debug_sp3 = getenv("DEBUG_SP3") != NULL;
	int spx_init = 0;
#if defined(NARCISSE)
	const int noprint_old = fnx_noprint_error(1);
#endif

	spx_init = SpxInit(nom_fichier,nomfich,&type,"test",debug_sp3);

	if(debug_sp3)
	{
	    printf("spx_init = %d\n",spx_init);
	}

    /* Valeurs pour spx_init :
       1 => Narcisse existe a coup sur (il faut "NARCISSE.n" ou rien ou 0       
       dans le nom du fichier ou comme champ port de  DEST_SP3).
       0 => Narcisse existe peut-etre (SpxInit n a pas calcule le port car il est
       donne different de rien, 0 ou NARCISSE.n).
       -1 => Narcisse n'existe pas a coup sur. */

	if(spx_init >= 0 && type == SPXSOCKET)
	{
	    FILE* const file_test = fnxopen(nomfich,"w");

	    if(file_test)
	    {
		fnxclose(file_test);
	    }
	    else
	    {
		spx_init = -1;
	    }

	}
	else if(spx_init >= 0)
	{
	    spx_init = -1;
	}

#if defined(NARCISSE)
	fnx_noprint_error(noprint_old);
#endif

	return spx_init;
}

/**********************************************************************/
/*      Fonction SpxQuery                                             */
/*      le code retour indique la presence du serveur Narcisse :        */
/*          1 => Narcisse existe a coup sur                             */
/*          -1 => Narcisse n'existe pas a coup sur                      */
/*      nom_fichier est le nom fourni en entree.                      */
/*          Il est complete par la variable d'environnement DEST_SP3  */
/**********************************************************************/

int
SpxQuery(const char* nom_fichier)
{
	char nomfich[MAX_FILENAME_LENGTH];
	unsigned type = SPXSOCKET;
	const int debug_sp3 = getenv("DEBUG_SP3") != NULL;
	int spx_init = 0;
#if defined(NARCISSE) /* dans ce cas on est sur que fnx_noprint_error existe */
	const int noprint_old = fnx_noprint_error(1);
#endif

	spx_init = SpxInit(nom_fichier,nomfich,&type,"test",debug_sp3);

    /* Valeurs pour spx_init :
       1 => Narcisse existe a coup sur (il faut "NARCISSE.n" ou rien ou 0       
       dans le nom du fichier ou comme champ port de  DEST_SP3).
       0 => Narcisse existe peut-etre (SpxInit n a pas calcule le port car il est
       donne different de rien, 0 ou NARCISSE.n).
       -1 => Narcisse n'existe pas a coup sur. */

	if(spx_init >= 0 && type == SPXSOCKET)
	{
	    FILE* const file_test = fnxopen(nomfich,"w");

	    if(file_test)
	    {
		spx_init = 1;
		fnxclose(file_test);
	    }
	    else
	    {
		spx_init = -1;
	    }

	}
	else if(spx_init >= 0)
	{
	    spx_init = -1;
	}

#if defined(NARCISSE)
	fnx_noprint_error(noprint_old);
#endif

	return spx_init;
}

/**********************************************************************/
/*     Fonction SpxLance : lance un serveur Narcisse sur un fichier     */
/*  donne en local et renvoie >= 0 si bon ou -1 si impossible         */
/*     narcisseexe = chemin absolu de l'executable Narcisse V1            */
/*     nom_fichier = nom du fichier dont on veut recuperer le contenu */
/*     machine = machine ou lancer Narcisse ("" => machine locale)      */
/*     port donne en retour le port generalise (au sens nx) du Narcisse */
/*  lance (sa taille definie dans le programme appelant doit-etre     */
/*  suffisante pour contenir le resultat - non verifie)               */
/*     port_name est le sous-champ name du port sur lequel on ouvre   */
/*  Narcisse. Il vaut NARCISSE par defaut (NULL ou "\0") (voir nx.h)      */
/*     L'output de Narcisse est envoye sur /dev/null                    */
/**********************************************************************/

int
SpxLance(const char* narcisseexe,const char* nom_fichier,
		const char* machine,const char* port_name,char port[])
{
	int compteur = 0;
	int num_nouveau = 0;
	const int limite = 60;
	const char* scrip0 = !strlen(machine) ? "" : "ssh ";
	const char* scrip = !strlen(machine) ?
	    " sh -c 'SERVEUR_INET_SP3=SERVER+0." :
	    " \"sh -c 'SERVEUR_INET_SP3=SERVER+0.";
	static char scrip1[] =
	    "+@ie32;export SERVEUR_INET_SP3;"
	    "SERVEUR_UNIX_SP3=SERVER+/UNIX/.";
	static char scrip2[] =
	    "U+@ie32;export SERVEUR_UNIX_SP3;"
	    "FICHIER_SP3=";
	static char scrip3[] =
	    ";export FICHIER_SP3;";
	char* templa = NULL;
	int atempla = 0;

	port[0] = '\0';

	if(!port_name || !strlen(port_name))
	{
	    templa = "NARCISSE";
	}
	else if(strstr(port_name,"XXXXXX") == port_name + strlen(port_name) - 6)
	{
	    const unsigned lport_name = strlen(port_name) + 1;

	    templa = (char*) malloc(lport_name);

	    if(!templa) return -1;

	    memcpy(templa,port_name,lport_name);
	    mktemp(templa);
	    atempla = 1;
	}
	else
	{
	    templa = (char*) port_name;
	}

	{
	    const unsigned lnomfich = strlen(scrip) + strlen(scrip1) +
		strlen(scrip2) + strlen(scrip3) + strlen(narcisseexe) +
		strlen(nom_fichier) + strlen(machine) + strlen(scrip0) + 
		2*strlen(templa) + 13;
	    char* nomfich = (char*) malloc(lnomfich);

	    if(!nomfich)
	    {

		if(atempla) free(templa);

		return -1;
	    }

	    strcpy(nomfich,scrip0);
	    strcat(nomfich,machine);
	    strcat(nomfich,scrip);
	    strcat(nomfich,templa);
	    strcat(nomfich,scrip1);
	    strcat(nomfich,templa);
	    strcat(nomfich,scrip2);
	    strcat(nomfich,nom_fichier);
	    strcat(nomfich,scrip3);
	    strcat(nomfich,narcisseexe);
	    strcat(nomfich,!strlen(machine) ? "'&" : "'\"&");
	    printf("nomfich = %s\n",nomfich);
	    fflush(stdout);
	    system(nomfich);

	    free(nomfich);
	}

	strcpy(port,machine);
	strcat(port,"+.");
	strcat(port,templa);

	if(atempla) free(templa);

	if(!strlen(machine))
	{
	    strcat(port,"U.0");
	}
	else
	{
	    strcat(port,".0");
	}

	while((num_nouveau = SpxWhich(port)) < 0 && compteur < limite)
	{
	    compteur++;
	    sleep(1);
	}

	if(compteur < limite)
	{
	    return num_nouveau;
	}
	else
	{
	    return -1;
	}

}

/**********************************************************************/
/*     Fonction SpcLOpen : appelle SpxLance et ouvre une connexion    */
/*     SpcOpen vers le Narcisse lance par SpxLance                      */
/*     utile pour la lecture de fichiers au format Narcisse avec l'aide */
/*     de SpxGetDonneeAbstraite                                       */
/*     narcisseexe = chemin absolu de l'executable Narcisse V1            */
/*     nom_fichier = nom du fichier dont on veut recuperer le contenu */
/*     machine = machine ou lancer Narcisse ("" => machine locale)      */
/*     port_name est le sous-champ name du port sur lequel on ouvre   */
/*  Narcisse. Il vaut NARCISSE par defaut (NULL ou "\0") (voir nx.h)      */
/*     L'output de Narcisse est envoye sur /dev/null                    */
/**********************************************************************/

SPXFILE*
SpcLOpen(const char* narcisseexe,const char* nom_fichier,const char* machine,
		const char* port_name)
{
	char port[128];
	const int num = SpxLance(narcisseexe,nom_fichier,machine,port_name,port);
	SPXFILE* spx_file = NULL;

	if(num < 0)
	{
	    printf("SpxLOpen : erreur : pb de lancement de Narcisse\n");
	    return NULL;
	}

	strcat(port,"+@ie32");
	spx_file = SpcOpen(port);

	return spx_file;
}

/**********************************************************************/
/* utilitaire pour chaines                                            */
/**********************************************************************/

void
SpxRemplaceBlanc(char* c)
{
	char* p = c;

	while (*p)
	{
	    *p = (*p == ' ') ? '0' : *p;
	    p++;
	}

}

/**********************************************************************/
/*  ouverture d'une connexion pour fichiers de type UCD d'AVS         */
/*   on n'envoit pas le code -5 et le handler vaut ucd                */
/**********************************************************************/

SPXFILE*
SpuOpen(char* nom_fichier)
{
	char nomfich[MAX_FILENAME_LENGTH];
	unsigned type = SPXSOCKET;
	FILE* file_fich = NULL;
	SPXFILE* spx_file = NULL;
	const int debug_sp3 = getenv("DEBUG_SP3") != NULL;
	const int sync_sp3 = getenv("SYNC_SP3") != NULL;

	SpxInit(nom_fichier,nomfich,&type,"ucd",debug_sp3);

	if(type == SPXSOCKET)
	{
	    type = SPXSOCKETUCD;
	}
	else if (type == SPXFICHIER)
	{
	    char* const test_format = strchr(nomfich,'@');

	    if(test_format)
	    {
		*test_format = '\0';
	    }

	    type = SPXFICHIERUCD;
	}

	spx_file = (SPXFILE*) malloc(sizeof(SPXFILE));

	if(!spx_file)
	{
	    fprintf(stderr,"Pb allocation de memoire : SpuOpen a echoue\n");

	    return NULL;
	}

	memset((char*) spx_file,0,sizeof(SPXFILE));

	spx_file->file = file_fich;
	spx_file->type = type;
	strcpy(spx_file->nom_fichier,nomfich);
	spx_file->debug = debug_sp3;
	spx_file->sync = sync_sp3;
	spx_file->spxarg = NULL;
	spx_file->spxarg_taille = 0;

	return spx_file;
}
