// ***********************************************************
//     permet d'imprimer les temps user, system et elaps
//     entre 2 appels donnes ci-dessous
// ***********************************************************

struct SccsIdS6Timer
{
	static const char* const sccs_id;
};

const char* const SccsIdS6Timer::sccs_id = "%W% %G% Pierre Brochard";

// ***********************************************************
//   includes et declarations generales
// ***********************************************************

#include "dinosaure/S6timer.h"
#include <stddef.h>

#if defined(HP_UX_10) && defined(_INCLUDE_POSIX4_SOURCE)
#   include <sys/sigevent.h>
#endif

#include "nxstreamiocompat.h"

#include <time.h>
#include <sys/time.h>

#if defined(OSF1)
extern "C" {
#  include <sys/resource.h>
}
#else
#  include <sys/resource.h>
#endif

#if defined(SUNOS_5)
#  include <sys/times.h>
#  include <limits.h>
extern "C" {long _sysconf(int);}
#endif

#if defined(CRAY) || defined(HP_UX) || defined(LINUX)
#  include <sys/times.h>
#endif

#ifdef _IBMR2
extern "C" {int getrusage(int,rusage*);}
#endif

#if !defined(CRAY) && !defined(HPUX) && !defined(SUNOS_5) && !defined(LINUX)
#   define S6NOTTIMES
#elif defined(S6NOTTIMES)
#   undef S6NOTTIMES
#endif

#ifndef S6NOTTIMES
#   ifndef CLK_TCK
        static long CLK_TCK = sysconf(_SC_CLK_TCK);
#endif
#endif

// ************************************************************
//    variables statiques
// ************************************************************

static int timer_inhib = 0;

// ************************************************************
//    Mise en place de la pile des timers de taille <= 100 
// ************************************************************

static double UserDepartSelf[100] /* en microsecondes */ ;
static double UserArriveeSelf     /* en microsecondes */ ;
static double SysDepartSelf[100] /* en microsecondes */ ;
static double SysArriveeSelf     /* en microsecondes */ ;
static double UserDepartChildren[100] /* en microsecondes */ ;
static double UserArriveeChildren     /* en microsecondes */ ;
static double SysDepartChildren[100] /* en microsecondes */ ;
static double SysArriveeChildren     /* en microsecondes */ ;

static double ElapsDepart[100];
static double ElapsArrivee;

static int compteur = 0;

static ostream* ptost = NULL;

// ************************************************************
//    routines
// ************************************************************

void
TimerInhib(int inhib)
{
	timer_inhib = inhib;
}

void
TimerDetInit()
{

	if(timer_inhib) return;

#if defined(S6NOTTIMES)
	rusage usage;

	if(getrusage(RUSAGE_SELF,&usage) != 0) cerr << "Pb rusage" << endl;

	UserDepartSelf[compteur] = usage.ru_utime.tv_usec +
		usage.ru_utime.tv_sec * 1000000.;
	SysDepartSelf[compteur] = usage.ru_stime.tv_usec +
		usage.ru_stime.tv_sec * 1000000.;

	if(getrusage(RUSAGE_CHILDREN,&usage) != 0) cerr << "Pb rusage" << endl;

	UserDepartChildren[compteur] = usage.ru_utime.tv_usec +
		usage.ru_utime.tv_sec * 1000000.;
	SysDepartChildren[compteur] = usage.ru_stime.tv_usec +
		usage.ru_stime.tv_sec * 1000000.;

	time_t temps = time(&temps);

	ElapsDepart[compteur] = double(temps);
#else
	tms usage;

	if((ElapsDepart[compteur] = times(&usage)) == -1) cerr << "Pb " <<
	  __FILE__ << ' ' << __LINE__ << endl;

	UserDepartSelf[compteur] = usage.tms_utime;
	SysDepartSelf[compteur] = usage.tms_stime;
	UserDepartChildren[compteur] = usage.tms_cutime;
	SysDepartChildren[compteur] = usage.tms_cstime;
#endif
	compteur++;

	if(compteur >= 100)
	{
	    cerr << "TimerInit : compteur > 100 : remise a zero" << endl;
	    compteur = 1;
	    UserDepartSelf[0] = 0.;
	    SysDepartSelf[0] = 0.;
	    UserDepartChildren[0] = 0.;
	    SysDepartChildren[0] = 0.;
	    ElapsDepart[0] = 0.;
	}

}

void
TimerDetReport(ostream& os,int fois, int cote)
{

	if(timer_inhib) return;

	double DureeElaps;
	double DureeUserSelf, DureeSysSelf;
	double DureeUserChildren, DureeSysChildren;

	compteur--;

	if(compteur < 0)
	{
	    compteur = 0;
	    UserDepartSelf[0] = 0.;
	    SysDepartSelf[0] = 0.;
	    UserDepartChildren[0] = 0.;
	    SysDepartChildren[0] = 0.;
	    ElapsDepart[0] = 0.;
	}

#if defined(S6NOTTIMES)
	rusage usage;

	if(getrusage(RUSAGE_SELF,&usage) != 0) cerr << "Pb rusage" << endl;

	UserArriveeSelf = double(usage.ru_utime.tv_usec) +
	    double(usage.ru_utime.tv_sec) * 1000000;
	SysArriveeSelf = double(usage.ru_stime.tv_usec) +
	    double(usage.ru_stime.tv_sec) * 1000000;

	if(getrusage(RUSAGE_CHILDREN,&usage) != 0) cerr << "Pb rusage" << endl;

	UserArriveeChildren = double(usage.ru_utime.tv_usec) +
	    double(usage.ru_utime.tv_sec) * 1000000;
	SysArriveeChildren = double(usage.ru_stime.tv_usec) +
	    double(usage.ru_stime.tv_sec) * 1000000;

	time_t temps = time(&temps);

	ElapsArrivee = double(temps);
#else
	tms usage;

	if((ElapsArrivee = times(&usage)) == -1) cerr << "Pb " << __FILE__ <<
	  ' ' << __LINE__ << endl;

	UserArriveeSelf = usage.tms_utime;
	SysArriveeSelf = usage.tms_stime;
	UserArriveeChildren = usage.tms_cutime;
	SysArriveeChildren = usage.tms_cstime;
#endif

#if defined(S6NOTTIMES)
	DureeUserSelf = double(UserArriveeSelf - UserDepartSelf[compteur]) /
		1000000.0 / fois;
	DureeSysSelf = double(SysArriveeSelf - SysDepartSelf[compteur]) /
		1000000.0 / fois;
	DureeUserChildren = double(UserArriveeChildren -
		UserDepartChildren[compteur]) / 1000000.0 / fois;
	DureeSysChildren = double(SysArriveeChildren -
		SysDepartChildren[compteur]) / 1000000.0 / fois;
	DureeElaps = double(ElapsArrivee - ElapsDepart[compteur]) / fois;
#else
	DureeUserSelf = double(UserArriveeSelf - UserDepartSelf[compteur]) /
		CLK_TCK / fois;
	DureeSysSelf = double(SysArriveeSelf - SysDepartSelf[compteur]) /
		CLK_TCK / fois;
	DureeUserChildren = double(UserArriveeChildren -
		UserDepartChildren[compteur]) / CLK_TCK / fois;
	DureeSysChildren = double(SysArriveeChildren -
		SysDepartChildren[compteur]) / CLK_TCK / fois;
	DureeElaps = double(ElapsArrivee - ElapsDepart[compteur])/CLK_TCK/fois;
#endif

	os << "n = " << cote << " c = " << compteur <<
	        " U_self = " << DureeUserSelf <<
	        " S_self = " << DureeSysSelf <<
	        " U_children = " << DureeUserChildren <<
	        " S_children = " << DureeSysChildren <<
	        " E = " << DureeElaps << endl;

}

void
TimerDetReport(int fois, int cote)
{

  if(ptost)
    TimerDetReport(*ptost,fois,cote);
  else
    TimerDetReport(cout,fois,cote);

}

void
TimerDetReport(ostream& os,int fois, int cote,const char* texte)
{

	if(timer_inhib) return;

	os << texte << "\n";
	TimerDetReport(os,fois,cote);
}

void
TimerDetReport(int fois, int cote,const char* texte)
{

  if(ptost)
    TimerDetReport(*ptost,fois,cote,texte);
  else
    TimerDetReport(cout,fois,cote,texte);

}

void
TimerDetReport(const char* texte)
{

	if(timer_inhib) return;

	compteur--;

	if(compteur < 0)
	{
	    compteur = 0;
	    UserDepartSelf[0]=0.;
	    SysDepartSelf[0]=0.;
	    UserDepartChildren[0]=0.;
	    SysDepartChildren[0]=0.;
	    ElapsDepart[0]=0.;
	}

	if(texte)
	  if(ptost)
	    *ptost << texte << endl;
	  else
	    cout << texte << endl;

}

void
SetOStream(ostream* const tptost)
{
  ptost = tptost;
}

/* fin de fichier */
