#ifndef _MACHINE_TYPES_H
#define _MACHINE_TYPES_H

/* Logiciel Narcisse.
   Copyright, droits d'auteurs : CEA/DAM-Ile de France
   Auteur : Pierre Brochard (CEA/DAM-Ile de France)
   %W% %G%
*/

/*------------------------------------------------------------------------------

	Definition des types numeriques simples (entier et reel) 
		-- independamment de la machine --
	=========================================================

		int_2		entier sur  2 octets
		int_4		entier sur  4 octets
		int_8		entier sur  8 octets

		u_int_2		entier non signe sur  2 octets
		u_int_4		entier non signe sur  4 octets
		u_int_8		entier non signe sur  8 octets

		float_4		reel   sur  4 octets
		float_8		reel   sur  8 octets
		float_16	reel   sur 16 octets

	NB : Certains types peuvent ne pas etre definis pour une machine
	     particuliere

	Definition des types generiques int_type et float_type
        ======================================================
	En l'absence de toute definition (macro) explicite a la compilation, les
	defauts suivants sont pris :
		Station de travail UNIX	avec OS 32 bits
			entier sur 32 bits et reel sur 32 bits
		Station de travail UNIX	avec OS 64 bits et toute machine Cray
			entier sur 64 bits et reel sur 64 bits

	Il est possible de forcer ces choix avec les definitions (macro)
	explicites suivantes a la compilation :

		-DINTEGER_16	force   int_type a  16 bits
		-DINTEGER_32	force   int_type a  32 bits
		-DINTEGER_64	force   int_type a  64 bits

		-DREAL_32	force float_type a  32 bits
		-DREAL_64	force float_type a  64 bits
		-DREAL_128	force float_type a 128 bits

	Remarques : 
	==========
	1/ La definition (macro) correspondant au choix effectue est rendue
	   disponible meme si elle n'a pas ete faite explicitement a la 
	   compilation. Elle peut etre utilisee dans les fichiers incluant
	   directement ou non le present fichier

	2/ Certaines definitions peuvent ne pas etre satisfaites sur une machine
	   particuliere : La compilation ne sera alors tout simplement pas
	   possible 


	ATTENTION - ATTENTION - ATTENTION - ATTENTION - ATTENTION - ATTENTION - 

	   Ce fichier etant destine a etre inclus dans Basic.x, toute
	   modification entrainerait une recompilation generale de tous
	   les projets ...

		Il est donc IMPORTANT de faire cette inclusion ENTRE
		<> pour desactiver la gestion des dependances

	ATTENTION - ATTENTION - ATTENTION - ATTENTION - ATTENTION - ATTENTION - 

------------------------------------------------------------------------------*/




/*------------------------------------------------------------------------------
Typedefs independants machine en ANSI-C & FP IEEE par Benoit Dupont de Dinechin.
------------------------------------------------------------------------------*/
#include <limits.h>
#include <float.h>

#include <stdlib.h>

#define UT_INT32_MAX 0x7FFFFFFFl
#if (!defined(INT_2) && SHRT_MAX < UT_INT32_MAX)
#define INT_2 short
#endif  
#if (!defined(INT_2) && INT_MAX < UT_INT32_MAX)
#define INT_2 int
#endif     
#if (!defined(INT_4) && SHRT_MAX == UT_INT32_MAX)
#define INT_4 short
#endif     
#if (!defined(INT_4) && INT_MAX == UT_INT32_MAX)
#define INT_4 int
#endif
#if (!defined(INT_4) && LONG_MAX == UT_INT32_MAX)
#define INT_4 long
#endif     
#if (!defined(INT_8) && INT_MAX > UT_INT32_MAX)
#define INT_8 int
#endif
#if (!defined(INT_8) && LONG_MAX > UT_INT32_MAX) 
#define INT_8 long
#endif
#if (!defined(INT_8))
#define INT_8 long long
#endif

/* on n'utilise pas IEEE64_MAX_EXP 1024 a cause d'un bug cpp & float.h Solaris */
#define IEEE64_MIN_EXP -1021
#if (!defined(FLOAT_4) && FLT_MIN_EXP > IEEE64_MIN_EXP)
#define FLOAT_4 float
#endif
#if (!defined(FLOAT_8) && FLT_MIN_EXP == IEEE64_MIN_EXP)
#define FLOAT_8 float 
#endif
#if (!defined(FLOAT_8) && DBL_MIN_EXP == IEEE64_MIN_EXP)
#define FLOAT_8 double
#endif
#if (!defined(FLOAT_16) && DBL_MIN_EXP < IEEE64_MIN_EXP)
#define FLOAT_16 double
#endif  
#if (!defined(FLOAT_16) && LDBL_MIN_EXP < IEEE64_MIN_EXP)
#define FLOAT_16 long double
#endif
/* HACK ALERT! on MIPS/SGI, DBL_MIN_EXP == LDBL_MIN_EXP && sizeof(double) < sizeor(long double) */
#if (!defined(FLOAT_16) && defined(mips) && defined(sgi))
#define FLOAT_16 long double
#endif

#ifdef INT_2
typedef INT_2 int_2;
typedef unsigned INT_2 u_int_2;
#define U_INT_2
#endif
#ifdef INT_4
typedef INT_4 int_4;
typedef unsigned INT_4 u_int_4;
#define U_INT_4
#endif
#ifdef INT_8
typedef INT_8 int_8;
typedef unsigned INT_8 u_int_8;
#define U_INT_8
#endif
#ifdef FLOAT_4
typedef FLOAT_4 float_4;
#endif
#ifdef FLOAT_8
typedef FLOAT_8 float_8;
#endif
#ifdef FLOAT_16
typedef FLOAT_16 float_16;
#endif

/* en ANSI-C, long est le seul type garanti pouvant contenir un ptrdiff_t  */
#if (LONG_MAX == UT_INT32_MAX)
#define DEFAULT_INTEGER_32
#define DEFAULT_REAL_32
#else
#define DEFAULT_INTEGER_64
#define DEFAULT_REAL_64
#endif

/*-----------------------------------------------------------
	Definition des type generiques int_type	et u_int_type
-----------------------------------------------------------*/

#if defined   (INTEGER_16)
	typedef int_2		int_type;
	typedef u_int_2		u_int_type;
#elif defined (INTEGER_32)
	typedef int_4		int_type;
	typedef u_int_4		u_int_type;
#elif defined (INTEGER_64)
	typedef int_8		int_type;
	typedef u_int_8		u_int_type;
#else
#   if defined (DEFAULT_INTEGER_32)
#	define INTEGER_32
	typedef int_4		int_type;
	typedef u_int_4		u_int_type;
#   elif defined (DEFAULT_INTEGER_64)
#	define INTEGER_64
	typedef int_8		int_type;
	typedef u_int_8		u_int_type;
#   else
	"Erreur > Impossible de definir les types generiques int_type et u_int_type"
#   endif
#endif


/*-----------------------------------------------------------
	Definition du type generique float_type	
-----------------------------------------------------------*/

#if defined   (REAL_32)
	typedef float_4		float_type;
#elif defined (REAL_64)
	typedef float_8		float_type;
#elif defined (REAL_128)
	typedef float_16	float_type;
#else
#   if defined (DEFAULT_REAL_32)
#	define REAL_32
	typedef float_4		float_type;
#   elif defined (DEFAULT_REAL_64)
#	define REAL_64
	typedef float_8		float_type;
#   else
	"Erreur > Impossible de definir le type generique float_type"
#   endif
#endif

#define __GENERIC_SIMPLE_TYPES_INSTANTIATED

#endif

