#ifndef U8REGEX_H
#define U8REGEX_H

/* %W% %G% Pierre Brochard */

/*----------------------------------------------------*/
/* routines pour regular expressions                  */
/*----------------------------------------------------*/

#if defined(U8SYSV_REGEX)
#   undef U8SYSV_REGEX
#endif

#if defined(AIX) || defined(OSF1) || defined(CRAY) ||\
	(defined(SUNOS) && !defined(SUNOS_4)) ||\
	(defined(IRIX) && !defined(IRIX_4) && !defined(IRIX_5)) ||\
	(defined(HP_UX) && !defined(HP_UX_8) && !defined(HP_UX_9)) ||\
	(defined(LINUX) && !defined(LINUX_1))
#   define U8SYSV_REGEX
#endif

#if defined(U8SYSV_REGEX)
/*
   We define re_comp() and re_exec() in terms of the SYS V functions
   (XPG4, POSIX.2)
 */
#include <regex.h>

static regex_t compiled_regex;
static int compiled_one = 0;

static const char*
U8re_comp (const char* pattern)
{

	if (!pattern) return NULL;

	if (compiled_one)
	{
	    regfree (&compiled_regex);
	    compiled_one = 0;
	}

	if (regcomp (&compiled_regex, pattern,0)  != 0)
	    return ("Unable to compiled with regcomp()");

	compiled_one = 1;
	return NULL;
}

static int
U8re_exec (const char* line)
{

	if (!compiled_one) return -1;

	if (regexec (&compiled_regex, line, 0, (regmatch_t *) NULL, 0) == 0)
	    return 1;
	else
	    return 0;
	
}
#else
#   include <unistd.h>
#   include <re_comp.h>
#   define U8re_comp(p) re_comp((p))
#   define U8re_exec(l) re_exec((l))
#endif /* ! U8SYSV_REGEX */

/*
 * Match string against the extended regular expression in
 * pattern, treating errors as no match.
 *
 * return 1 for match, 0 for no match
 */

static int
match(const char* mystr,const char* pattern)
{

	if(U8re_comp(pattern))
	{
	    return 0;      /* report error */
	}

	if(U8re_exec(mystr) != 1)
	{
	    return 0;      /* report error */
	}
	else
	{
	    return 1;
	}

}

#endif /* U8REGEX_H */
