#ifndef P4GRAF2_H
#define P4GRAF2_H

// %W% %G% Pierre Brochard

#include "minotaure/P4graf1.h"
#include "dinosaure/P4tabgenbase.h"

enum TypeDonnee4d {DONNEE_NOEUD = 0,DONNEE_BRAS = 1,DONNEE_MAILLE = 2,
		   DONNEE_VOLUME = 3};
enum TypeTraceFil {TRACE_NONE = -1,TRACE_EN_X = 0,TRACE_EN_Y = 1,
		   TRACE_EN_XY = 2};
enum P4TypeCoupe {P4SAVE_GEOM = 0,P4COUPE_MONO = 1,P4COUPE_FIL_3D = 2,
		  P4COUPE_FIL_4D = 3};

class GeomPure;
class GeomPure4;
class Sommet4;
class Int4;

#if defined(P4VERSION_AVEC_TEMPLATE) &&  defined(P4TABSOMGRI_IMPLEMENT)
/* version avec template */
    class Sommet;
    class GrilleSimple;
    class GrilleAuto;

#   if P4VERSION_AVEC_TEMPLATE == 4
        typedef TabGEN<Sommet> TabSommet;
        typedef TabGEN<GrilleSimple> TabGrilleSimple;
        typedef TabGEN<GrilleAuto> TabGrilleAuto;
#   else
#       define TabSommet TabGEN<Sommet>
#       define TabGrilleSimple TabGEN<GrilleSimple>
#       define TabGrilleAuto TabGEN<GrilleAuto>
#   endif

#else /* version sans template */
    class TabSommet;
    class TabGrilleSimple;
    class TabGrilleAuto;
#endif /* fin version sans template */

class P4Graf2 : public P4Graf1
{
    protected :

	Sommet4&         _borne_min;
	Sommet4&	 _borne_max;
	Int4&		 _log;
	TabGrilleSimple& _tabgrillesimple;
	TabGrilleAuto&	 _tabgrilleauto;
	Opt34d		 _fil;
	Opt34d		 _fac;
	Opt34d		 _niv;
	Opt34d		 _lis;
	TypeTraceFil	 _type_trace_fil;
	TypeDonnee4d	 _td4d;
	P4Boolean	 _cherche_mailles;
	int              _marqueur; // pour les mailles a 1 sommet
	GeomPure*	 pt_geo_temp; // pour le break dans trigeom

	int P4Coordonnees(const GeomPure4& geo,REEL xpos,REEL ypos,
			TabSommet& ts,Tabunsigned& tm);

	int P4Lisse(const Sommet4& s1,const Sommet4& s2,const Sommet4& s3);
	int P4Niv(const Sommet4& s1,const Sommet4& s2,const Sommet4& s3);
	int P4Coupe(const Sommet4& s1,const Sommet4& s2,const Sommet4& s3,
			REEL a,REEL b,REEL c,REEL d,REEL e,
			TabREEL_Sbuf& seg_deb,TabREEL_Sbuf& seg_fin,
			const P4TypeCoupe type_coupe = P4COUPE_MONO);

    public :

	P4Graf2();
	~P4Graf2();

	Opt34d  P4Lis() const {return _lis;};
	Opt34d  P4Fil() const {return _fil;};

	void P4Init34d(Opt34d fil=OPT_3D,Opt34d fac=OPT_3D,
			Opt34d niv=OPT_3D,Opt34d lis=OPT_3D)
	{_fil=fil;_fac=fac;_niv=niv;_lis=lis;};

	void P4TypeTraceFil(TypeTraceFil type_trace_fil = TRACE_EN_XY)
	{_type_trace_fil = type_trace_fil;};

	void P4TypeDonnee4d(TypeDonnee4d td4d = DONNEE_NOEUD)
	{_td4d = td4d;};

	void P4Marqueur(int marqueur = 5)
	{_marqueur = marqueur;};

	void P4ChercheMaillesInit(const P4Boolean cherche_mailles)
	{_cherche_mailles=cherche_mailles;};

	void P4DefEchelles(const REEL xmin=0.,const REEL ymin=0.,
			const REEL zmin=0.,const REEL cmin=0.,
			const REEL xmax=0.,const REEL ymax=0.,
			const REEL zmax=0.,const REEL cmax=0.,
			const int logx=ECH3DLIN,const int logy=ECH3DLIN,
			const int logz=ECH3DLIN,const int logc=ECH3DLIN);
	void P4DefCminmax(const REEL cmin = 0.,const REEL cmax = 0.);
	void P4LitEchelles(REEL& xmin,REEL& ymin,REEL& zmin,
			REEL& cmin,REEL& xmax,REEL& ymax,REEL& zmax,
			REEL& cmax,int& logx,int& logy,int& logz,int& logc);

	int P4LigneBrisee(const REEL_S x[],const REEL_S y[],
			const REEL_S z[],const REEL_S c[],int npoints,
			P4Boolean trace_lent=FALSE,int nombre_cesures=1) ;
	int P4LigneBrisee(const REEL_S x[],const REEL_S y[],
			const REEL_S z[],int npoints,
			P4Boolean trace_lent=FALSE,int nombre_cesures=1) ;

	int P4LigneBrisee(const GeomPure4& geo,
			P4Boolean trace_lent=FALSE,int nombre_cesures=1) ;

	int P4Polygone(REEL_S x[],REEL_S y[],REEL_S z[],int npoints);

	int P4Filaire(const REEL_S x[],const REEL_S y[],const REEL_S z[],
			const REEL_S c[],int dimx,int dimy,int dimz,int dimc,
			int nx,int ny);
	int P4Filaire(const REEL_S x[],const REEL_S y[],const REEL_S z[],
			const REEL_S c[],int dimx,int dimy,int dimz,int dimc,
			int nx,int ny,unsigned mailles[],unsigned nmailles,
			int dimmailles,const Fil opt_fil);
	int P4Filaire(const REEL_S x[],const REEL_S y[],const REEL_S z[],
			const REEL_S c[],int dimx,int dimy,int dimz,int dimc,
			int nx,int ny,int mailles[],int nmailles,
			int dimmailles,const Fil opt_fil);
	int P4Filaire(const int nbrnoeuds,const int nbrmailles,
			const REEL_S x[],const REEL_S y[],const REEL_S z[],
			const REEL_S c[],const int mailles[][3],
			const Fil opt_fil);

	int P4Lissage(const REEL_S x[],const REEL_S y[],const REEL_S z[],
			const REEL_S c[],int dimx,int dimy,int dimz,int dimc,
			int nx,int ny,unsigned mailles[],unsigned nmailles,
			int dimmailles,const Lisse opt_lisse);
	int P4Lissage(const REEL_S x[],const REEL_S y[],const REEL_S z[],
			const REEL_S c[],int dimx,int dimy,int dimz,int dimc,
			int nx,int ny,int mailles[],int nmailles,
			int dimmailles,const Lisse opt_lisse);
	int P4Lissage(const int nbrnoeuds,const int nbrmailles,
			const REEL_S x[],const REEL_S y[],const REEL_S z[],
			const REEL_S c[],const int mailles[][3],
			const Lisse opt_lisse);

	int P4CoupePlan(const REEL_S x[],const REEL_S y[],const REEL_S z[],
			int dimx,int dimy,int dimz,int nx,int ny,
			unsigned mailles[],unsigned nmailles,int dimmailles,
			REEL a,REEL b,REEL c,REEL d);

	int P4Bornes(const GeomPure4& geo);
	int P4Filaire(const GeomPure4& geo);
	int P4Filaire(const GeomPure4& geo,const Fil opt_fil);
	int P4Lissage(const GeomPure4& geo,const Lisse opt_lisse);
	int P4CoupePlan(const GeomPure4& geo,
			REEL a,REEL b,REEL c,REEL d,REEL e);

	int P4TraceGeom(const GeomPure4& geo,const Fil opt_fil);
	int P4TraceGeom(const GeomPure4& geo,const Lisse opt_lisse);
	int P4TraceGeom(const GeomPure4& geo,
			REEL a,REEL b,REEL c,REEL d,REEL e,
			GeomPure4* const pt_geo_coupe = NULL);

	int P4Fil(const GeomPure4& geo,const Tabunsigned& ord,Fil opt_fil);
	int P4Facette(const GeomPure4& geo,const Tabunsigned& ord);
	int P4Lisse(const GeomPure4& geo,const Tabunsigned& ord,Lisse opt_lisse);
	int P4Coupe(const GeomPure4& geo,
			REEL a,REEL b,REEL c,REEL d,REEL e,
			GeomPure4* const pt_geo_coupe = NULL);

	int P4Imin(const TabREEL_S& z);
	int P4Imax(const TabREEL_S& z);
#ifndef REQRS
	int P4Imin(int np,const REEL_S z[]);
	int P4Imax(int np,const REEL_S z[]);
#endif
	int P4Imin(int np,const REEL z[]);
	int P4Imax(int np,const REEL z[]);

	void P4CalcOrdreSimple(const GeomPure4& geo,Tabunsigned& ordt);

    // gestion des grilles 3d

	void P4TraceGrilles();
	void P4AnnuleGrilles();

	void P4DefCaracGrille(int taille_caract_titres,
			int taille_caract_echelles) ;

	void P4AnnuleGrillesSimple();
	void P4NouvelleGrilleSimple();
	void P4DefTitreSimple(const char* titre_axe_bas,
			const char* titre_axe_haut,
			const char* titre_axe_droite,
			const char* titre_axe_gauche,
			const char* unite_axe_bas,const char* unite_axe_haut,
			const char* unite_axe_droite,
			const char* unite_axe_gauche);
	void P4DefGrilleSimple(Face face,REEL decalx,
			P4Boolean texte_horizontal,int grad_axex,
			int grad_axey,int* type_axe,P4Boolean* label_axe,
			int* grad_inter_axe, int* factorisation_axe) ;

	void P4AnnuleGrillesAuto();
	void P4NouvelleGrilleAuto();
	void P4DefTitreAuto(const char* titre_x,const char* titre_y,
			const char* titre_z,const char* unite_x,
			const char* unite_y,const char* unite_z);
	void P4DefGrilleAuto(REEL decalx,REEL decaly,REEL decalz,
			P4Boolean texte_horizontal,int grad_axex,
			int grad_axey,int grad_axez,int type_grilles,
			int* grad_inter) ;

    // fin gestion des grilles 3d

	friend ostream& operator << (ostream& ost,const P4Graf2& p23) ;
};

#endif
