#ifndef IHM_H
#define IHM_H

// **************************************************************
//   Librairie de gestion d'IHM independante du Toolkit
// **************************************************************

// %W% %G% G. Colin de Verdiere modifie P. Brochard

#include "nxstream/U8serveur.h"

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xlib.h>

#if XlibSpecificationRelease < 5
    typedef char* XPointer;
#endif

// *************************************************
//       gestion de l'IHM (X11-Xt-Motif)
// *************************************************

class ElementVisuel : virtual public ElementBase
{
    public:

	ElementVisuel();
	ElementVisuel(const char* const label);
	ElementVisuel(const Widget& wid);
	virtual ~ElementVisuel();

	virtual ElementVisuel* ElVisuel();

      // Recuperation d'infos du Widget

	Widget GetId() const {return _widgetid;};
	XtAppContext GetContext() const {return _appcontext;};
	Colormap ColorMap();
	char* GetLabel() const {return _label;};

      // Fonctions de comportement du Widget utile suivant les Widgets

	virtual void SelectCB(XtPointer call_data);
	virtual void UnSelectCB(XtPointer call_data);
      
      // Fonctions d'etat du Widget 

	virtual void Cache();
	virtual void Montre();
	virtual void Actif();
	virtual void Inactif();
	virtual void ModeTrace(int mode) { _DisplayTrace = mode; };

    // calcul de l'IHM

	virtual void Realize();

    protected :

	static void LeBoutonSelCB(Widget , XtPointer client_data,
			XtPointer call_data);

	static XtAppContext   _appcontext;
	static Display*       _dpy;
	static int	      _coreadwrite;
	static Visual*        _visual;
	static int            _DisplayTrace; // Indicateur D'impression
	static Widget         _ref_break; // ref d'un eventuel bouton break
	static Widget         _ref_break2; // ref d'un eventuel 2eme bouton break
	static Widget         _ref_break3; // ref d'un eventuel 3eme bouton break
	static Bool           _IRPVal; // valeur ressource Motif
				//XmNinitialResourcesPersistent par defaut
	static Bool           _KFPVal; // valeur ressource Motif
				// XmNkeyboardFocusPolicy par defaut
	static Bool           _quit; // si True on quitte le plus tot possible

	static Widget*        _widgettab; // table des widgets existants
	static unsigned       _widgetnb; // taille table des widgets

	static void Register(const Widget& wid);
	static int IsRegister(const Widget& wid);
	static void Destroy(const Widget& wid);

	static Widget*        _widgetcol; // pour modif colormap

	static void RegisterCol(const Widget& wid);
	static void UnRegisterCol(const Widget& wid);
	static Widget FindPrivateShell(const Widget parent);
	static Widget MakePrivateShell(const Widget parent,const char* name,
			Arg* args,const Cardinal num_args);

	Widget                _widgetid;
        Widget                _parent;
	char*                 _label;

    private :

	static void UnRegister(const Widget& wid,int indice=-1);
	static void UnRegisterChild(const Widget& wid,int indice=-1);
};

class ElementVisInp : public ElementVisuel,public ElementInput
{      
    public :

	ElementVisInp();
	~ElementVisInp();

    // bug des cfront ATT USL 2.0 2.1 3.0.1
    // on est oblige de surdefinir ElInput et ElVisuel dans chaque
    // classe derivee (tables virtuelles fausses)

	ElementInput* ElInput() {return this;};
	ElementVisuel* ElVisuel() {return this;};

    // Gestionnaire de fichiers

	U8InputId AddInputDriver(nxstream* file,int mode);
	void RemoveInputDriver(U8InputId input_id);
};

// ---------------------------------------------
//      fenetre principale Motif
// ---------------------------------------------

class RootWin : public ElementVisInp
{      
    public :

	RootWin(int& argc, char **argv,int choix = 1,const char* visual = NULL);
	~RootWin();

	ElementInput* ElInput() {return this;};
	ElementVisuel* ElVisuel() {return this;};

	static Bool ExpTest(Display*,XEvent*,XPointer);
	static void ExpIHM(int sig = 0);

    // cree par la commande bitmap toto.h 32x32 &

	void ChangeIcone(char *bits, int w, int h);

    // action

	void Realize();

    // gestion de fichiers (socket)

	void LectureInputCB(int fid,U8InputId);

    // Boucle generale de traitement

	void ScruteEvenement(int bloque_geom = 1);

    private :

	static void Quit(Widget,XtPointer,XEvent*,Boolean*);
};

// ---------------------------------------------
//   Classe des conteneurs d'elements visuels
// ---------------------------------------------

class ConteneurBase : public ElementVisuel
{ 
    public:

	ConteneurBase();
	ConteneurBase(const char* const label);
	~ConteneurBase();
      
   // Configuration du conteneur

	void Margin(int delta);

	void Lignes(int i = 1);
	void Colonnes(int i = 1);
};

class Conteneur : public ConteneurBase
{ 
    public:

	Conteneur(const Widget& parent, const char* classe="CONT");
	~Conteneur();
};

class ConteneurExclusif : public Conteneur
{
    public:

	ConteneurExclusif(const Widget& parent,const char* classe="CTEX");
	~ConteneurExclusif();
};

class ConteneurJoli : public ElementVisuel
{
    public:

	ConteneurJoli(const Widget& parent,const char* classe="CONTJ");
	~ConteneurJoli();
};

// ---------------------------------------------
//               Ligne separateur
// ---------------------------------------------

class Separateur : public ElementVisuel
{      
    public:

	Separateur(const Widget& parent,const char* classe="SEP");
	~Separateur();
};

// ---------------------------------------------
//         Classe des boutons
// ---------------------------------------------

class Bouton : public ElementVisuel
{      
    public:

	Bouton(const Widget& parent,const char* label,const char* classe="BTN");
	~Bouton();

	void SelectCB(XtPointer call_data);

    private:

	int    _selected;
	void*  _labelInterne;
};

// ---------------------------------------------
//     Classe des boutons a choix multiples
// ---------------------------------------------

class BoutonDeChoix : public ElementVisuel
{      
    public:

	BoutonDeChoix(const Widget& parent, const char* label,
		const char* classe="BTCX");
	~BoutonDeChoix();

	void UnSelectCB(XtPointer call_data);
	void SelectCB(XtPointer call_data);
	int IsSelected() {return _selected;};
	void Change(int choix) ;

    private:

	static void LeBoutonChangeCB(Widget , XtPointer client_data,
			XtPointer call_data);

	int    _selected;
        void * _labelInterne;
};

// ---------------------------------------------
//     Classe des menus deroulants
// ---------------------------------------------

class MenuBar : public ElementVisuel
{      
    public:

	MenuBar(const Widget& parent, const char* label,
		const char* classe = "MBAR");
	~MenuBar();

	ElementVisuel* ElVisuel() {return this;};
};

class PullDownTop : public ConteneurBase
{      
    public:

	PullDownTop(const Widget& parent, const char* label,
		const char* classe="MENTP",int nbcol = 1);
	~PullDownTop();

    //void Colonne(int nbcol = 1);
	void Margin(int delta);

	Widget GetCascade() const {return _cascade;};

    private:

	Widget _menubar;
	Widget _shell;
	Widget _cascade;
	void*  _labelInterne;
};

class PullDownTopPanel : public ConteneurBase
{      
    public:

	PullDownTopPanel(const Widget& parent,
		const char* classe="MENTP",int nbcol = 1);
	~PullDownTopPanel();

    private:

	Widget _shell;
};

class PullDownTopCascade : public ElementVisuel
{      
    public:

	PullDownTopCascade(const Widget& parent, const Widget& menu,
		const char* label,const char* classe="MENTP");
	~PullDownTopCascade();

        void Montre();

    private:

	Widget _menubar;
	Widget _menu;
	void*  _labelInterne;
};

class PullDownSubMenu : public ConteneurBase
{   
    public:
  
        PullDownSubMenu(const Widget& parent, const char* label,
                  const char* classe="MENSB");
        ~PullDownSubMenu();
    
        void Cache();
        void Montre();
  
    private:

        Widget _shell;
        Widget _cascade;
        void*  _labelInterne;
};

class PullDownSubMenuExclusif : public PullDownSubMenu
{   
    public:
  
        PullDownSubMenuExclusif(const Widget& parent, const char* label,
                  const char* classe="MENSB");
        ~PullDownSubMenuExclusif();
};

class PullDownEntry : public ElementVisuel
{   
    public:

	PullDownEntry();
	PullDownEntry(const Widget& parent, const char* label,
		const char* classe="MENEN");
	~PullDownEntry();

	void SelectCB(XtPointer call_data);
	void Init(const Widget & parent, const char *label,
		const char *classe);

	void Couleur(unsigned red,unsigned green,unsigned blue);
	void Couleur(Pixel pix);

   private:

      void* _labelInterne;
};

// ---------------------------------------------
//      Classe des popups
// ---------------------------------------------

class PopUp : public ElementVisuel
{   
    public:

        PopUp(const Widget& parent, const char* titre,int nx=10,int ny=10,
			const char* classe="POPUP");
        ~PopUp();

        void Cache();
        void Montre();
	void BloqueGeom();
	void LibereGeom();
};

// ---------------------------------------------
//      Classe des popups sans bandeau
// ---------------------------------------------

class PopUpSansBandeau : public ElementVisuel
{   
    public:

        PopUpSansBandeau(const Widget& parent, const char* titre,
			int nx=10,int ny=10,const char* classe="POPUP");
        ~PopUpSansBandeau();

        void Cache();
        void Montre();

    private:

	static void MyInputEnterWindow(Widget widgetid,XtPointer client_data,
			XEvent* event,Boolean*);
	void SelectEnterWindow(XtPointer calldata);
};

// ---------------------------------------------
//      Classe des popups iconifiable
// ---------------------------------------------

class PopUpIconic : public ElementVisuel
{   
    public:

        PopUpIconic(const Widget& parent, const char* titre,
			int nx=10,int ny=10,const char* classe="POPUP");
        ~PopUpIconic();

        void Cache();
        void Montre();
};

// ---------------------------------------------
//      Classe des labels statics
// ---------------------------------------------

class Label : public ElementVisuel
{   
    public :

	Label(const Widget& parent,const char* label,const char* classe="LBL");
	~Label();

	static Bool ExpTest(Display*,XEvent*,XPointer win);

	virtual void Change(const char *neuf);

    private :

      void* _labelInterne;
};

// -----------------------------------------------------------------------
//      Classe des labels statics simples (athena Widget)
//      on travaille sur des char* (String) au lieu de XmString (16 bits)
//      on devrait etre plus rapide !!
// -----------------------------------------------------------------------

class LabelSimple : public ElementVisuel
{   
    public :

	LabelSimple(const Widget& parent,const char* label,
		    const char* classe = "LBLS");
	~LabelSimple();

	virtual void Change(const char *neuf);
};

// ---------------------------------------------
//     Classe des labels editables pour saisie
// ---------------------------------------------

class Saisie : public ElementVisuel
{   
    public:

        Saisie(const Widget& parent, const char* label,short int taille = 20,
		const char* classe = "TXT");
        ~Saisie();
 
        virtual void AEteModifie(XtPointer call_data);
        void SelectCB(XtPointer call_data);
      
	virtual void Change(const char* neuf);
	virtual char* GetText();
	int AEteModifie() const {return _aetemodifie;};

    private:

	static void SaisieChangeCB(Widget , XtPointer client_data,
			XtPointer call_data);

	int _aetemodifie;
};

// ---------------------------------------------
//     Classe des sliders de saisie
// ---------------------------------------------

class Slider : public ElementVisuel
{   	
    public:
   
        Slider(const Widget& parent, const char* label, const int& mini,
		const int& maxi, const int& ValeurInitiale,
		const char* classe = "SLIDER",int width = 200);
        ~Slider();
 
	void SelectCB(XtPointer call_data);
      
	virtual void Change(int newmin, int newmax, int newpos);
	int GetValue() { return _lavaleur; };

    private:

	int    _lavaleur;
	int    _lavaleur_min;
	int    _lavaleur_max;
	void*  _labelInterne;
};

// ---------------------------------------------
//     Classe des fenetres avec scrollbar
// ---------------------------------------------

class ScrollWin : public ElementVisuel
{
    public:

        ScrollWin(const Widget& parent,const char* classe = "SCROLW",
			int largeur = 100,int hauteur = 100);
        ~ScrollWin();
};

// ---------------------------------------------
//     Classe des listes a choix simple
// ---------------------------------------------

class ListeSimple : public ElementVisuel
{
    public:

	ListeSimple(const Widget& parent, const char* classe = "LIST",
			int largeur = 170,int hauteur = 200);
        ~ListeSimple();

	void Single();
	void Extended();

	void SelectCB(XtPointer call_data);
	virtual void SelectDoubleCB(XtPointer call_data);
	virtual void SelectExtendedCB(XtPointer call_data);

	void AjouteElement(const char* txt);
	void Montre();
	void PurgeListe();

	int GetLastSelected() const {return _lastselected;};
	int GetNombreSelected() const {return _nbselected;};
	const int* GetSelected() const {return _selected;};

    private:

	static void LaListeSelectCB(Widget , XtPointer client_data, 
			XtPointer call_data);
	static void LaListeDoubleSelectCB(Widget , XtPointer client_data, 
			XtPointer call_data);
	static void LaListeExtendedSelectCB(Widget , XtPointer client_data, 
			XtPointer call_data);

	int _nombre;
   	int _lastselected;
	int _nbselected;
	int* _selected;
	int _nbselectedall;
	int* _selectedall;
   	void* tablo;
   	ScrollWin* _scr;

	static Bool Test(Display*,XEvent* event,XPointer win);
};

#endif
