#ifndef WIDAFFX_H
#define WIDAFFX_H

// Widget d'affichage de L3 sous Motif/X11
// Interface utilisateur

// %W% %G% G. Colin de Verdiere
// modifs Pierre Brochard

#include "poulpe/IHM.h"
#include "poulpe/Help.h"
#include "P4types.h"

// *****************************************************
//      boutons du panneau d'impression
// *****************************************************

class U8Print;

class Saisie_B : public Saisie
{
    private:

	int _qui; // mode printer (0), fichier (1) ou fichier debut (2)
	char* _printer;
	char* _filename;
	char* _filedeb;
	U8Print* _ptr_print;

    public:

	Saisie_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			short int taille = 20,
			const char* classe = "TXT");
	void SelectCB(XtPointer call_data);
	void SetMode(int mode);
	void Qui(int qui);
	void SetFile(const char* const filename);
	void SetFileDeb(const char* const filedeb);
	void SetPrinter(const char* const printer);
};

class Titre_B : public Saisie
{
    private:

	U8Print* _ptr_print;

    public:

	Titre_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			short int taille = 20,
			const char* classe = "TXT");
	void SelectCB(XtPointer call_data);
};

class U8AfficheurX;

class Ok_B : public Bouton
{
    private:

	PopUp* _p;
	Saisie_B* _sb;
	Titre_B* _titre;

    public:

	Ok_B(PopUp* pup, const Widget& parent,const char* label,
		Saisie_B* sb = NULL,Titre_B* titre = NULL,
		const char* classe = "BTN");

	void SelectCB(XtPointer call_data);
};

class Cancel_B : public Bouton
{
    private:

	PopUp* _p;

    public:

	Cancel_B(PopUp *pup, const Widget& parent,const char* label,
		const char* classe = "BTN") : 
		Bouton(parent, label, classe) { _p = pup; };

	void SelectCB(XtPointer call_data);
};

class Aide_B : public Bouton
{
    public :

	Aide_B(const Widget& parent, const char *label, 
		const char* classe = "BTN") :
		Bouton(parent,label,classe) {_aide = NULL;};
	~Aide_B() {};
      
	void SelectCB(XtPointer call_data);

    private :

	Help* _aide;
};

class Print_B : public BoutonDeChoix
{
    private:

	Saisie_B* _lit;
	U8Print* _ptr_print;

    public:

	Print_B(Saisie_B* const s, const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe = "BTCX");

	void SelectCB(XtPointer call_data);
};

class File_B : public BoutonDeChoix
{
    private:

	Saisie_B* _lit;
	U8Print* _ptr_print;

    public:

	File_B(Saisie_B* const s, const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe = "BTCX");

	void SelectCB(XtPointer call_data);
};

class FileDeb_B : public BoutonDeChoix
{
    private:

	Saisie_B* _lit;
	U8Print* _ptr_print;

    public:

	FileDeb_B(Saisie_B* const s, const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe = "BTCX");

	void SelectCB(XtPointer call_data);
};

class Color_B : public BoutonDeChoix
{
    private:

	U8Print* _ptr_print;

    public:

	Color_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe="BTCX") : 
			BoutonDeChoix(parent, label, classe),
			_ptr_print(ptp) {};

	void SelectCB(XtPointer call_data);
	void UnSelectCB(XtPointer call_data);
};

class ToutNoir_B : public BoutonDeChoix
{
    private:

	U8Print* _ptr_print;

    public:

	ToutNoir_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe = "BTCX") : 
			BoutonDeChoix(parent, label, classe),
    			_ptr_print(ptp) {};

	void SelectCB(XtPointer call_data);
	void UnSelectCB(XtPointer call_data);
};

class NoirEtBlanc_B : public BoutonDeChoix
{
    private:

	U8Print* _ptr_print;

    public:

	NoirEtBlanc_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe = "BTCX") : 
			BoutonDeChoix(parent, label, classe),
    			_ptr_print(ptp) {};

	void SelectCB(XtPointer call_data);
	void UnSelectCB(XtPointer call_data);
};

class Portrait_B : public BoutonDeChoix
{
    private:

	U8Print* _ptr_print;

    public:

	Portrait_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe = "BTCX") : 
			BoutonDeChoix(parent, label, classe),
    			_ptr_print(ptp) {};

	void SelectCB(XtPointer call_data);
	void UnSelectCB(XtPointer call_data);
};

class Landscape_B : public BoutonDeChoix
{
    private:

	U8Print* _ptr_print;

    public:

	Landscape_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe = "BTCX") : 
			BoutonDeChoix(parent, label, classe),
    			_ptr_print(ptp) {};

	void SelectCB(XtPointer call_data);
	void UnSelectCB(XtPointer call_data);
};

class ReverseVideo_B : public BoutonDeChoix
{
    private:

	U8Print* _ptr_print;

    public:

	ReverseVideo_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe = "BTCX") : 
			BoutonDeChoix(parent, label, classe),
    			_ptr_print(ptp) {};

	void SelectCB(XtPointer call_data);
	void UnSelectCB(XtPointer call_data);
};

class absolute_B : public BoutonDeChoix
{
    private:

	U8Print* _ptr_print;

    public:

	absolute_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe = "BTCX") : 
			BoutonDeChoix(parent, label, classe),
    			_ptr_print(ptp) {};

	void SelectCB(XtPointer call_data);
	void UnSelectCB(XtPointer call_data);
};

class EPS_B : public BoutonDeChoix
{
    private:

	U8Print* _ptr_print;

    public:

	EPS_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe = "BTCX") : 
			BoutonDeChoix(parent, label, classe),
			_ptr_print(ptp) {};

	void SelectCB(XtPointer call_data);
	void UnSelectCB(XtPointer call_data);
};

class textes_noir_B : public BoutonDeChoix
{
    private:

	U8Print* _ptr_print;

    public:

	textes_noir_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe="BTCX") : 
			BoutonDeChoix(parent, label, classe),
    			_ptr_print(ptp) {};

	void SelectCB(XtPointer call_data);
	void UnSelectCB(XtPointer call_data);
};

class Ident_B : public BoutonDeChoix
{
    private:

	U8Print* _ptr_print;

    public:
	Ident_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe = "BTCX") : 
			BoutonDeChoix(parent, label, classe),
    			_ptr_print(ptp) {};

	void SelectCB(XtPointer call_data);
	void UnSelectCB(XtPointer call_data);
};

class Logo_B : public BoutonDeChoix
{
    private:

	U8Print* _ptr_print;

    public:

	Logo_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe = "BTCX") : 
			BoutonDeChoix(parent, label, classe),
    			_ptr_print(ptp) {};

	void SelectCB(XtPointer call_data);
	void UnSelectCB(XtPointer call_data);
};

class Temps_B : public BoutonDeChoix
{
    private:

	U8Print* _ptr_print;

    public:

	Temps_B(const Widget& parent, const char* label,
			U8Print* const ptp = NULL,
			const char* classe = "BTCX") : 
			BoutonDeChoix(parent, label, classe),
			_ptr_print(ptp) {};

	void SelectCB(XtPointer call_data);
	void UnSelectCB(XtPointer call_data);
};

// **********************************************
// break bouton .................................
// **********************************************

struct BreakCommuniq
{
	BreakCommuniq();
	virtual ~BreakCommuniq();

	virtual void Action() = 0;
};

class BreakBouton : public Bouton
{
    private:

	BreakCommuniq* _communiq;

    public:

	BreakBouton(const Widget& parent,const char* label,
			const char* classe = "BTN_break",
			BreakCommuniq* const communiq = NULL);
	~BreakBouton() {};

	void SelectCB(XtPointer call_data);
};

// *****************************************************
//       boutons sur l'afficheur
// *****************************************************

// *****************************************************
//        Boutons et panneau Impression
// *****************************************************

struct U8Print
{
	U8Print();
	virtual ~U8Print();

	virtual void DestFich(const char* const fichier) = 0;
	virtual void DestFichDeb(char* const fichier) = 0;
	virtual void DestImp(const char* const imprimante) = 0;
	virtual void DestType(int type) = 0;
	virtual void Couleur(int coul) = 0;
	virtual void Ident(int ident) = 0;
	virtual void Chrono(int chrono) = 0;
	virtual void VideoInv(int video_inv) = 0;
	virtual void Logo(int logo) = 0;
	virtual void Eps(int eps) = 0;
	virtual void TexteNoir(int texte_noir) = 0;
	virtual void DimExacte(int dim_exacte) = 0;
	virtual void Landscape(int landscape) = 0;
	virtual void Titre(const char* const titre) = 0;
};

// Bouton impression

class PrintEnvoi : public Bouton
{
    private:

	U8AfficheurX* _affx;

    public:

	PrintEnvoi(const Widget& parent,const char* label,
			U8AfficheurX* const affx = NULL,
			const char* classe = "BTN");

	void SelectCB(XtPointer call_data);
};

// Boutons appel du panneau de reglage des parametres d'impression

class PrintPanel;

class PrintBouton : public Bouton
{
    private:

	PrintPanel* _print;

    public:

	PrintBouton(const Widget& parent,const char* label,
			PrintPanel* const print = NULL,
			const char* classe = "BTN");
	~PrintBouton();

	void SelectCB(XtPointer call_data);
};

class PrintEntry : public PullDownEntry
{
    private:

	PrintPanel* _print;

    public:

	PrintEntry(const Widget& parent,const char* label,
			PrintPanel* const print = NULL,
			const char* classe = "MENEN");
	~PrintEntry();

	void SelectCB(XtPointer call_data);
};

// Panneau de reglage des parametres d'impression

class PrintPanel
{
    private:

	PopUp* _reglage;
   
	Conteneur* _c0;
	Label* _lit_mes;
	Saisie_B* _lit;
   
   	ConteneurExclusif* _c2;
	Print_B* _print;
	File_B* _file;
	FileDeb_B* _filedeb;

	ConteneurExclusif* _c3;
	Color_B* _couleur;
	ToutNoir_B* _toutNoir;
	NoirEtBlanc_B* _noirEtBlanc;

	ConteneurExclusif* _c4;
	Portrait_B* _portrait;
	Landscape_B* _landscape;

	Label* _titre_mes;
	Titre_B* _titre;

	Conteneur* _opt2;
	Ident_B* _identification;
	Logo_B* _logo;
	Temps_B* _temps;
	EPS_B* _eps;
	ReverseVideo_B* _reversevideo;
	absolute_B* _absolute;
   	textes_noir_B* _textes_noir;

	Conteneur* _c1;
	Ok_B* _ok;
	Cancel_B* _cancel;
	Aide_B* _aide;

	int _done;
	Widget _parent;

	U8Print* _ptr_print;

	char** _labels;
	unsigned _labels_taille;

    // Etat du paneau Impression

	char* _imp_dest_fich;
	char* _imp_dest_fichd;
	char* _imp_dest_imp;
	char* _imp_titre;
	int _imp_dest_type;
	int _imp_couleur;
	int _imp_ident;
	int _imp_chrono;
	int _imp_video_inv;
	int _imp_logo;
	int _imp_eps;
	int _imp_texte_noir;
	int _imp_dim_exacte;
	int _imp_landscape;

    public:

	PrintPanel(const Widget& root,
			const char* const labels[] = NULL,
			unsigned labels_taille = 0,
			U8Print* const ptp = NULL);
	~PrintPanel();

	void Change(int imp_dest_type,const char* const imp_dest_fich,
			const char* const imp_dest_fichd,
			const char* const imp_dest_imp,
			int imp_couleur,int imp_ident,int imp_chrono,
			int imp_texte_noir,int imp_logo,int imp_eps,
			int imp_video_inv,int imp_dim_exacte,int imp_landscape,
			const char* const imp_titre);

	void Montre();
	void Cache();
};

// **************************************************
//        redraw bouton
// **************************************************

class RedrawBouton: public Bouton
{
    private :

        U8AfficheurX* _affx;

    public:

	RedrawBouton(const Widget& parent,const char* label,
		U8AfficheurX* const affx = NULL,
		const char* classe="BTN");

        void ChangeAffX(U8AfficheurX* const affx = NULL);
	void SelectCB(XtPointer call_data);
};

// **************************************************
//        save bouton
// **************************************************

struct SaveCommuniq
{
	SaveCommuniq();
	virtual ~SaveCommuniq();

	virtual void AddNomFichier(char*) = 0;
};

class SaveBouton: public Bouton
{
    private :

	SaveCommuniq* _ptr;
        U8AfficheurX* _affx;

    public:

	SaveBouton(const Widget& parent,const char* label,
		SaveCommuniq* const ptr = NULL,
		U8AfficheurX* const affx = NULL,
		const char* classe="BTN");

        void ChangeAffX(U8AfficheurX* const affx = NULL);
	void SelectCB(XtPointer call_data);
};

// *********************************************************
//    enfants de taille bouton
// *********************************************************

struct TailleCommuniq
{
	TailleCommuniq();
	virtual ~TailleCommuniq();

	virtual void Action(unsigned largeur,unsigned hauteur,
		REEL clip_largeur = 1.,REEL clip_hauteur = 1.) = 0;
};

class TailleBouton;

class OkTaille : public Bouton
{
    private:

	PopUp* _p;
	TailleBouton* _ptr;

    public:

	OkTaille(PopUp* pup, const Widget& parent,const char* label,
		TailleBouton* const ptr=NULL,const char* classe="BTN");

	void SelectCB(XtPointer call_data);
};

class AnnuleTaille : public Bouton
{
    private:

	PopUp* _p;
	TailleBouton* _ptr;

    public:

	AnnuleTaille(PopUp* pup, const Widget& parent,const char* label,
		TailleBouton* const ptr=NULL,const char* classe="BTN");

	void SelectCB(XtPointer call_data);
};

class AideTaille : public Bouton
{
    public :

	AideTaille(const Widget& parent, const char *label, 
		const char* classe = "BTN") :
		Bouton(parent,label,classe) {_aide = NULL;};
	~AideTaille() {};
      
	void SelectCB(XtPointer call_data);

    private :

	Help* _aide;
};

// **************************************************
//   classe TailleBouton
// **************************************************

class Dessin;

class TailleBouton: public Bouton
{
    private:

	PopUp*		_pere;

	Conteneur*	_tout;

	Conteneur*	_param_largeur;
	Label*		_label_largeur;
	Saisie*		_saisie_largeur;

	Conteneur*	_param_hauteur;
	Label*		_label_hauteur;
	Saisie*		_saisie_hauteur;
	
	Conteneur*	_param_clip_largeur;
	Label*		_label_clip_largeur;
	Saisie*		_saisie_clip_largeur;

	Conteneur*	_param_clip_hauteur;
	Label*		_label_clip_hauteur;
	Saisie*		_saisie_clip_hauteur;
	
	TailleCommuniq*	_ptr;
	Dessin*		_ptr_dessin;

	Conteneur*	_param;
	OkTaille*	_ok;
	AnnuleTaille*	_annule;
	AideTaille*	_aide;

	int		_done;
	Widget		_parent;

	char**          _labels;
	unsigned        _labels_taille;

    public:

	TailleBouton(const Widget& parent,
			const Widget& root,const char* label,
			const char* classe="BTN",
			TailleCommuniq* const ptr = NULL,
			Dessin*	const ptr_dessin = NULL,
			const char* const labels[] = NULL,
			unsigned labels_taille = 0);
	~TailleBouton();

	void SelectCB(XtPointer call_data);

	char* GetLargeur();
	char* GetHauteur();
	char* GetClipLargeur();
	char* GetClipHauteur();

	void SetLargeur(unsigned largeur);
	void SetHauteur(unsigned hauteur);
	void SetClipLargeur(REEL clip_largeur);
	void SetClipHauteur(REEL clip_hauteur);

	void Resize(unsigned largeur,unsigned hauteur,REEL clip_largeur,
			REEL clip_hauteur);
	void Action(unsigned largeur,unsigned hauteur,REEL clip_largeur,
			REEL clip_hauteur);
};

class DessinAnnuler : public Bouton
{
    private :

	ElementVisuel* _pto;

    public :

	DessinAnnuler(const Widget& parent, const char* label,
		const char* classe,ElementVisuel* pto = NULL) :
		Bouton(parent,label,classe),_pto(pto) {};
	~DessinAnnuler() {};

	void Attach(ElementVisuel* pto) {_pto=pto;};
	void SelectCB(XtPointer call_data);
};

class DessinAide : public Bouton
{
    private :

	Help* _help;
	const char* _help_label;

    public :
	   
	DessinAide(const Widget& parent, const char* label, 
		const char* help_label,const char* classe) :
		Bouton(parent,label,classe),_help(NULL),_help_label(help_label) {};
	~DessinAide() {};
      
	void SelectCB(XtPointer call_data);
};

// ***************************************************************
//         dessin
// ***************************************************************

class U8Coord;
class U8Pixmap;
class U8Agrand;
class U8Reduc;
class U8Unzoom;
class U8CalcImage;
class U8CalcImageL3;

class Dessin : public ElementVisuel
{
    public:

	Dessin(const Widget& parent, const char* classe,
			SaveCommuniq* const ptr = NULL,
			TailleCommuniq* const ptr_taille = NULL,
			BreakCommuniq* const ptr_break = NULL,
		        const char* const labels[] = NULL,
		        const unsigned labels_taille = 0,int master = 0,
			U8Coord* const ptc = NULL,
			U8Pixmap* const ptm = NULL,
			U8Agrand* const pta = NULL,
			U8Reduc* const pte = NULL,
			U8Unzoom* const ptu = NULL,
			U8CalcImage* const pti = NULL,
			U8CalcImageL3* const ptl = NULL,
			U8Print* const ptp = NULL,
			int iconic = 0,int sans_bandeau = 0,
			PrintPanel* const print = NULL);
	~Dessin();

	void Resize(Widget widgetid,XtPointer call_data);
	void ResizeClip(Widget widgetid);
	void ResizeBoutons(Widget widgetid);

	char* Largeur() const;
	char* Hauteur() const;
	char* ClipLargeur() const;
	char* ClipHauteur() const;

	void Resize(unsigned largeur,unsigned hauteur,
			REEL clip_largeur = 1.,REEL clip_hauteur = 1.);
	void Active();
	void Create();
	void Cache();
	void Clear(int);
	void Clear();

    // pour clients de _affx

        void EnterWindowClient(int win);
        void LeaveWindowClient(int win);
        void ButtonPressClient(int win);
        void ButtonReleaseClient(int win);
        void MotionNotifyClient(int win);

        void ButtonPressBaseClient(int win);
        void ButtonReleaseBaseClient(int win);
        void MotionNotifyBaseClient(int win);

	void SetCurseur(int cur);
	int GetCurseur() const;

	void SetPosition(int x,int y);

    // impose _affx comme afficheur courant pour sanglier

	void Driver();

    // pour transmettre une demande de mode pixmap

        void ModePixmap(P4Boolean mp);

    // pour transmettre une demande de mode zoom carre ou non

        void ModeZoomCarre(P4Boolean zc);

    // Pour le panneau Impression

	void ChangeImp(int imp_dest_type,const char* const imp_dest_fich,
			const char* const imp_dest_fichd,
			const char* const imp_dest_imp,
			int imp_couleur,int imp_ident,int imp_chrono,
			int imp_texte_noir,int imp_logo,int imp_eps,
			int imp_video_inv,int imp_dim_exacte,int imp_landscape,
			const char* const imp_titre);

    private:

	ElementVisuel*	_ecran;
	Widget		_parent;
	Widget		_form;
	Widget		_frm;
	Widget		_frm2;
	Widget		_form2;
	Widget		_scrwin;
	Widget    	_canvas; // surface effective de trace

	U8AfficheurX*   _affx; // driver X11 (version C++)

	PullDownTop*     _actions;

	PrintBouton*	_print_appel;
	PrintEnvoi*	_print_envoi;
	PrintPanel*     _print;
	P4Boolean       _print_type_alloc;
	RedrawBouton*	_redraw;
	SaveBouton*	_save;
	TailleBouton*	_taille;
	BreakBouton*	_break;

	DessinAnnuler*  _annuler;
	DessinAide*     _aide;

	int 		_done;
	int		_master;
	int		_iconic; // si 1 => afficheur iconifiable separement
	int		_sans_bandeau; // si 1 => afficheur sans WM
	SaveCommuniq*	_ptr;
	TailleCommuniq*	_ptr_taille;
	BreakCommuniq*	_ptr_break;
	U8Coord*	_ptr_coord;
	U8Pixmap*	_ptr_pixmap;
	U8Agrand*	_ptr_agrand;
	U8Reduc*	_ptr_reduc;
	U8Unzoom*	_ptr_unzoom;
	U8CalcImage*    _ptr_calcimage;
	U8CalcImageL3*  _ptr_calcimage_l3;
	U8Print*	_ptr_print;

	unsigned	_largeur;
	unsigned	_hauteur;
	REEL		_clip_largeur;
	REEL		_clip_hauteur;
	Dimension       _widget_height;
	Bool            _noresize;
	unsigned        _delwid1;
	unsigned        _delwid2;
	unsigned        _delscrw;
	unsigned        _delscrh;
	char**          _labels;
	unsigned        _labels_taille;

	static void MyResize(Widget widgetid,XtPointer client_data,
			     XtPointer call_data);
	static void MyExpose(Widget widgetid,XtPointer client_data,
			     XtPointer call_data);
	static void MyResizeClip(Widget widgetid,XtPointer client_data,
			     XEvent* event,Boolean*);
	static void MyResizeBoutons(Widget widgetid,XtPointer client_data,
			     XEvent* event,Boolean*);
};

// **********************************************************
//               fonctions membres inline
// **********************************************************

inline
BreakCommuniq::BreakCommuniq()
{
}

inline
BreakCommuniq::~BreakCommuniq()
{
}

inline
SaveCommuniq::SaveCommuniq()
{
}

inline
SaveCommuniq::~SaveCommuniq()
{
}

inline
TailleCommuniq::TailleCommuniq()
{
}

inline
TailleCommuniq::~TailleCommuniq()
{
}

inline
OkTaille::OkTaille(PopUp* pup, const Widget& parent,const char* label,
		TailleBouton* const ptr,const char* classe) : 
		Bouton(parent, label, classe)
{
	_p = pup;
	_ptr = ptr;
}

inline 
AnnuleTaille::AnnuleTaille(PopUp* pup, const Widget& parent,
		const char* label,
		TailleBouton* const ptr,const char* classe) : 
		Bouton(parent, label, classe)
{
	_p = pup;
	_ptr = ptr;
}

inline void
TailleBouton::Resize(unsigned largeur,unsigned hauteur,
		     REEL clip_largeur,REEL clip_hauteur)
{
	_ptr_dessin->Resize(largeur,hauteur,clip_largeur,clip_hauteur);
}

inline void
TailleBouton::Action(unsigned largeur,unsigned hauteur,REEL clip_largeur,
		REEL clip_hauteur)
{
	_ptr->Action(largeur,hauteur,clip_largeur,clip_hauteur);
}

inline char*
TailleBouton::GetLargeur()
{
	return _saisie_largeur->GetText();
}

inline char*
TailleBouton::GetHauteur()
{
	return _saisie_hauteur->GetText();
}

inline char*
TailleBouton::GetClipLargeur()
{
	return _saisie_clip_largeur->GetText();
}

inline char*
TailleBouton::GetClipHauteur()
{
	return _saisie_clip_hauteur->GetText();
}

inline void
SaveBouton::ChangeAffX(U8AfficheurX* const affx)
{
	_affx = affx;
}

inline void
RedrawBouton::ChangeAffX(U8AfficheurX* const affx)
{
	_affx = affx;
}

inline
Ok_B::Ok_B(PopUp* pup, const Widget& parent,const char* label,
		Saisie_B* sb,Titre_B* titre,const char* classe) : 
		Bouton(parent, label, classe),
		_p(pup),_sb(sb),_titre(titre)
{
}

inline void
Saisie_B::Qui(int qui)
{
	_qui = qui;
}

inline void
Dessin::ChangeImp(int imp_dest_type,const char* const imp_dest_fich,
		const char* const imp_dest_fichd,
		const char* const imp_dest_imp,
		int imp_couleur,int imp_ident,int imp_chrono,
		int imp_texte_noir,int imp_logo,int imp_eps,
		int imp_video_inv,int imp_dim_exacte,int imp_landscape,
		const char* const imp_titre)
{

	if(_print)
	{
	    _print->Change(imp_dest_type,imp_dest_fich,imp_dest_fichd,
			    imp_dest_imp,imp_couleur,imp_ident,imp_chrono,
			    imp_texte_noir,imp_logo,imp_eps,
			    imp_video_inv,imp_dim_exacte,imp_landscape,
			    imp_titre);
	}

}

inline
PrintBouton::PrintBouton(const Widget& parent,const char* label,
		PrintPanel* const print,
		const char* classe) :
		Bouton(parent,label,classe),
		_print(print)
{
}

inline
PrintBouton::~PrintBouton()
{
}

inline
PrintEntry::PrintEntry(const Widget& parent,const char* label,
		PrintPanel* const print,
		const char* classe) :
		PullDownEntry(parent,label,classe),
		_print(print)
{
}

inline
PrintEntry::~PrintEntry()
{
}

inline
PrintEnvoi::PrintEnvoi(const Widget& parent,const char* label,
		U8AfficheurX* const affx,const char* classe) : 
		Bouton(parent, label, classe),
		_affx(affx)
{
}

#endif
