// identification

struct SccsIdP4Geom
{
	static const char* const sccs_id;
};

const char* const SccsIdP4Geom::sccs_id = "%W% %G% Pierre Brochard";

// includes

#include "poulpe/IHM.h"
#define P4TABSOMGRI_IMPLEMENT
#include "P4geom.h"
#include "minotaure/P4sanglier.h"
#include "dinosaure/P4quicksort.h"
#include "dinosaure/S6timer.h"
#include "P4tabsommet.h"

//#if defined(SCX_X) && SCX_X >= 0x500 // essai d instanciation explicite
//template ostream& operator << (ostream&,TabREELbuf);
//#endif

//#if defined(SCX_X) && SCX_X >= 0x500 
// SC5 patche aout 1999 ne semble pas trouver P4tabgen.C pour
// ostream& operator << (ostream&,TabGEN)
//#   include "dinosaure/P4tabgen.C"
//#endif

// implementation

GeomPure::GeomPure()
{
	_ecriture = TRUE;
	ptma = new MailleNS;
}

GeomPure::GeomPure(const GeomPure& g)
{
	_ecriture = g._ecriture;
	_x.Lier(g._x);
	_y.Lier(g._y);
	_z.Lier(g._z);
	ptma = g.ptma->NewCopy();
	*ptma = *(g.ptma);
}

GeomPure::GeomPure(const TabREEL_S& x,const TabREEL_S& y,
		const TabREEL_S& z,const MailleSt4& mail)
{
	_ecriture = TRUE;
	_x.Lier(x);
	_y.Lier(y);
	_z.Lier(z);
	ptma = new MailleSt4;
	*ptma = mail;
}

GeomPure::GeomPure(const TabREEL_S& x,const TabREEL_S& y,const TabREEL_S& z,
		const MailleNS& mail)
{
	_ecriture = TRUE;
	_x.Lier(x);
	_y.Lier(y);
	_z.Lier(z);
	ptma = new MailleNS;
	*ptma = mail;
}

GeomPure::~GeomPure()
{
	delete ptma;
}

void
GeomPure::Enleve()
{
	_x.Enleve();
	_y.Enleve();
	_z.Enleve();
}

void
GeomPure::AjoutePoint(REEL point_x,REEL point_y,REEL point_z)
{
	_x.Ajoute(REEL_S(point_x));
	_y.Ajoute(REEL_S(point_y));
	_z.Ajoute(REEL_S(point_z));
}

void
GeomPure::AjoutePoints(const TabREEL_S& x,const TabREEL_S& y,const TabREEL_S& z)
{
	_x.Ajoute(x);
	_y.Ajoute(y);
	_z.Ajoute(z);
}

int
GeomPure::Projete(const int nx,const int ny,const int dimx,
		const int dimy,const int dimz,const REEL_S x[],const REEL_S y[],
		const REEL_S z[],const Bool3& lxyz,const Sommet& orig,
		const Proj& proj)
{

    // -------------------------------------------------------------
    // calcul et stockage dans this du projete du maillage x,y,z
    // dimx = 0 et dimy = -1 => maillage rectangulaire
    // -------------------------------------------------------------

	if(nx == 0 || ny == 0) return -1;

	register const int nxy = nx*ny;

	Sommet xyz;
	Sommet uvda;
	TabREEL_Sbuf temp_x(nxy);temp_x.Enleve();
	TabREEL_Sbuf temp_y(nxy);temp_y.Enleve();
	TabREEL_Sbuf temp_z(nxy);temp_z.Enleve();

        register int alp = 1;
	register int bet = dimy;

	if(dimy == -1)
	{
	    alp = 0;
	    bet = 1;
	}

	register int testz = dimz == 0;

	for(register int j = 0;j < ny;j++)
	{
	    register const int dimxj = dimx*j;
	    register const int betj = bet*j;
	    register const int dimzj = dimz*j;

	    for(register int i = 0;i < nx;i++)
	    {
		xyz(REEL(x[i+dimxj]),REEL(y[alp*i+betj]),
				testz ? 0. : REEL(z[i+dimzj]));
		uvda = proj.Projete(lxyz,orig,xyz);
		temp_x.Ajoute((REEL_S) uvda.X());
		temp_y.Ajoute((REEL_S) uvda.Y());
		temp_z.Ajoute((REEL_S) uvda.Z());
	    }

	}

	_x.Ajoute(temp_x);
	_y.Ajoute(temp_y);
	_z.Ajoute(temp_z);

	return 0;
}

int
GeomPure::AjoutePoints(const int nx,const int ny,const int dimx,
		const int dimy,const int dimz,
		const REEL_S x[],const REEL_S y[],const REEL_S z[])
{
	_x.Ajoute(x,nx,ny,dimx);
	_y.Ajoute(y,nx,ny,dimy);
	_z.Ajoute(z,nx,ny,dimz);

	return 0;
}

int
GeomPure::AjoutePoints(const int nx,const int ny,const int dimx,
		const int dimy,const int dimz,
		const REEL_S x[],const REEL_S y[],const REEL_S z[],
		const Bool3& lxyz)
{

	if(lxyz.AllFalse())
	{
	    return AjoutePoints(nx,ny,dimx,dimy,dimz,x,y,z);
	}

	REEL_S* RESTRICT xt = new REEL_S[nx*ny];
	REEL_S* RESTRICT yt = new REEL_S[nx*ny];
	REEL_S* RESTRICT zt = new REEL_S[nx*ny];

	if(!xt || !yt || !zt)
	{
	    delete[] xt;delete[] yt;delete[] zt;
	    cerr << "pb allocation memoire dans GeomPure4::AjoutePoints" <<
		endl;
	    return -1;
	}

        register int alp = 1;
	register int bet = dimy;
	
	if(dimy == -1)
	{
	    alp = 0;
	    bet = 1;
	}

	register int l = 0 ;
	register const int testz = dimz == 0;
	register const P4Boolean lbx = lxyz.Bx();
	register const P4Boolean lby = lxyz.By();
	register const P4Boolean lbz = lxyz.Bz();

	for(register int j = 0;j < ny;j++)
	{
	    register const int dimxj = dimx*j;
	    register const int betj = bet*j;
	    register const int dimzj = dimz*j;

	    for(register int i = 0;i < nx;i++,l++)
	    {
		register const REEL_S xx = x[i+dimxj];
		register const REEL_S yy = y[alp*i+betj];
		register const REEL_S zz = testz ? 0. : z[i+dimzj];

		xt[l] = P4Cvmgt(P4Log(xx),xx,lbx);
		yt[l] = P4Cvmgt(P4Log(yy),yy,lby);
		zt[l] = P4Cvmgt(P4Log(zz),zz,lbz);
	    }

	}

	_x.Ajoute(xt,nx*ny);
	_y.Ajoute(yt,nx*ny);
	_z.Ajoute(zt,nx*ny);

	delete[] xt;delete[] yt;delete[] zt;

	return 0;
}

int
GeomPure::InvProjete(const GeomPure& geo_projete,const Bool3& lxyz,
		const Sommet& orig,const Proj& proj)
{
    // on donne une geometrie projete (geo_projete) et on restaure la 
    // geometrie initiale pour les points compris entre le nombre max de points
    // de this et celui de geo_projete
    // on suppose que les points deja existant de this representent
    // la geometrie initiale

	Sommet xyz;
	Sommet uvda;
	const unsigned npo_deja_fait = NombrePoints();
	const unsigned npo = geo_projete.NombrePoints();
	const unsigned dnpo = npo-npo_deja_fait;
	TabREEL_Sbuf temp_x(dnpo);temp_x.Enleve();
	TabREEL_Sbuf temp_y(dnpo);temp_y.Enleve();
	TabREEL_Sbuf temp_z(dnpo);temp_z.Enleve();
	const REEL_S* const x = geo_projete.X();
	const REEL_S* const y = geo_projete.Y();
	const REEL_S* const z = geo_projete.Z();

	for(unsigned i = npo_deja_fait;i < npo;i++)
	{
	    xyz.S(x[i],y[i],z[i]);
	    uvda=proj.InvProjete(lxyz,orig,xyz);
	    temp_x.Ajoute((REEL_S) uvda.X());
	    temp_y.Ajoute((REEL_S) uvda.Y());
	    temp_z.Ajoute((REEL_S) uvda.Z());
	}

	_x.Ajoute(temp_x);
	_y.Ajoute(temp_y);
	_z.Ajoute(temp_z);

	return 0;
}

int
GeomPure::Projete2d(const Sommet& orig,const Proj& proj)
{
	register REEL_S* RESTRICT const x = _x;
	register REEL_S* RESTRICT const y = _y;
	register const REEL_S* RESTRICT const z = _z;
	register const unsigned taille = unsigned(_x);

	if(proj.Conic())
	{
	    Sommet xyz;
	    Sommet uvda;

	    for(register unsigned i=0;i<taille;i++)
	    {
		xyz(x[i],y[i],z[i]);
		uvda = proj.ProjeteConic2d(orig,xyz);
		x[i] = uvda.X();
		y[i] = uvda.Y();
	    }

	}
	else
	{
	    register const REEL o1 = orig.X();
	    register const REEL o2 = orig.Y();
	    register const REEL o3 = orig.Z();

	    for(register unsigned i=0;i<taille;i++)
	    {
		proj.ProjeteNonConicRapid2d(o1,o2,o3,x[i],y[i],z[i]);
	    }

	}

	return 0;
}

ostream&
operator << (ostream& ost,GeomPure g)
{
	ost << "Impression de la geometrie\n";

	const unsigned tai = g._x;

	ost << " Points : " << tai << endl;

	Sommet s1(0.,0.,0.);
	unsigned i;

	for (i=0;i<tai;i++)
	{
	   s1(g._x(i),g._y(i),g._z(i));
           ost << " numero " << i << " " << s1 << endl;
	}

	Tabunsignedbuf nu;
	const unsigned nma = g.ptma->Nmailles();

	for (i=0;i<nma;i++)
	{
	   ost << " maille numero " << i << " numero sommets : ";
	   g.ptma->DonneSommets(nu,i);

	   const unsigned taille = nu;

	   for (unsigned j=0;j<taille;j++) 
	   {
		ost << nu(j) << " ";
	   }

	   nu.Enleve();
	   ost << endl;
	}

	return ost;
}

void
GeomPure::ChercheMailles(const REEL xm,const REEL ym,
		TabSommet& ts,Tabunsigned& tma,const P4Boolean timer) const
{

    // *------------------------------------------------------------*
    // *   recherche de toutes les mailles                          *
    // *   intersectant une droite xm=constante, ym=constante       *
    // *------------------------------------------------------------*

	if(timer) TimerDetInit();

	const unsigned nma = ptma->Nmailles();
	Tabunsignedbuf nu;
	TabREELbuf xnew,ynew,znew;
	TabREELbuf anew,bnew,cnew;
	TabREELbuf Anew,Bnew,Cnew;
	TabREELbuf csnew;
	TabREELbuf fnew,Fnew,alnew;
	const REEL_S* RESTRICT const x = _x;
	const REEL_S* RESTRICT const y = _y;
	const REEL_S* RESTRICT const z = _z;

	cout << "\nRecherche de maille " << xm << " " << ym << endl;

	for(unsigned num_maille=0;num_maille<nma;num_maille++)
	{
	    ptma->DonneSommets(nu,num_maille);

	    const unsigned nnu = unsigned(nu);
	    const unsigned* const tnu = nu;

	    xnew.Enleve();
	    ynew.Enleve();
	    znew.Enleve();
	    anew.Enleve();
	    bnew.Enleve();
	    cnew.Enleve();
	    Anew.Enleve();
	    Bnew.Enleve();
	    Cnew.Enleve();
	    csnew.Enleve();
	    fnew.Enleve();
	    Fnew.Enleve();
	    alnew.Enleve();

	    unsigned i;

	    for (i=0;i<nnu;i++)
	    {
		xnew.Ajoute((REEL) x[tnu[i]]);
		ynew.Ajoute((REEL) y[tnu[i]]);
		znew.Ajoute((REEL) z[tnu[i]]);
	    }

	    for (i=0;i<nnu;i++)
	    {
		const unsigned j = (i+1)%nnu;

		anew.Ajoute(ynew(j)-ynew(i));
		bnew.Ajoute(xnew(i)-xnew(j));
		cnew.Ajoute(ynew(i)*xnew(j)-ynew(j)*xnew(i));
	    }

	    P4Boolean degen = FALSE;

	    for (i=0;i<nnu;i++)
	    {
		const unsigned j = (i+1)%nnu;

		if(anew(i)*bnew(j)-anew(j)*bnew(i) == 0.)
		{
		    degen = TRUE;
		    break;
		}

	    }

	    int testm = 1;
	    REEL zm = 0.;

	    if(degen == FALSE)
	    {
		const int nnu2 = nnu-2;
		const int nnu3 = nnu-3;

		for (i=0;int(i)<nnu3;i++)
		{
		    const unsigned j=(i+1)%nnu;
		    const unsigned k=(j+1)%nnu;
		    const unsigned l=(k+1)%nnu;
		    const REEL xxci=anew(i)*cnew(k)-cnew(i)*anew(k);
		    const REEL yyci=bnew(j)*anew(l)-anew(j)*bnew(l);
		    const REEL zzci=cnew(l)*anew(j)-anew(l)*cnew(j);
		    const REEL ttci=bnew(k)*anew(i)-anew(k)*bnew(i);
		    const REEL uuci=cnew(k)*bnew(i)-cnew(i)*bnew(k);
		    const REEL vvci=bnew(j)*cnew(l)-cnew(j)*bnew(l);

		    Anew.Ajoute(xxci*yyci+zzci*ttci);
		    Bnew.Ajoute(yyci*uuci+vvci*ttci);
		    Cnew.Ajoute(vvci*xxci-zzci*uuci);
		}

		for (i=0;i<nnu;i++)
		{
		    csnew.Ajoute(1.);

		    int j;

		    for (j=0;j<nnu3;j++)
		    {
			csnew.ModVal(csnew(i)*
				(Anew(j)*xnew(i)+Bnew(j)*ynew(i)+Cnew(j)),i);
		    }

		    for (j=0;j<nnu2;j++)
		    {
			const unsigned k=(j+i+1)%nnu;

			csnew.ModVal(csnew(i)/
				(anew(k)*xnew(i)+bnew(k)*ynew(i)+cnew(k)),i);
		    }

		}

		for (i=0;i<nnu;i++) fnew.Ajoute(anew(i)*xm+bnew(i)*ym+cnew(i));

		for (i=0;int(i)<nnu3;i++)
				Fnew.Ajoute(Anew(i)*xm+Bnew(i)*ym+Cnew(i));

		REEL som = 0.;

		for (i=0;i<nnu;i++)
		{
		    REEL all = csnew(i);
		    int j;

		    for(j=0;j<nnu3;j++) all/=Fnew(j);

		    for(j=0;j<nnu2;j++) all*=fnew((j+i+1)%nnu);

		    alnew.Ajoute(all);
		    zm+=all*znew(i);
		    som+=all;
		}

		for (i=0;i<nnu;i++)
		{
		    const REEL all=alnew(i)/som;
		    testm = testm && all*(all-1.)<=0.;
		}

		zm/=som;
	    }
	    else
	    {
		REEL xc = 0.;
		REEL yc = 0.;
		REEL zc = 0.;

		for (i=0;i<nnu;i++)
		{
		    xc += xnew(i);
		    yc += ynew(i);
		    zc += znew(i);
		}

		xc /= nnu;
		yc /= nnu;
		zc /= nnu;

		for(i=0;i<nnu;i++)
		{
		    const unsigned j=(i+1)%nnu;
		    const REEL a0=ynew(j)-ynew(i);
		    const REEL b0=xnew(i)-xnew(j);
		    const REEL c0=ynew(i)*xnew(j)-ynew(j)*xnew(i);
		    const REEL a1=yc-ynew(j);
		    const REEL b1=xnew(j)-xc;
		    const REEL c1=ynew(j)*xc-yc*xnew(j);
		    const REEL a2=ynew(i)-yc;
		    const REEL b2=xc-xnew(i);
		    const REEL c2=yc*xnew(i)-ynew(i)*xc;
		    const REEL ds0=a1*xnew(i)+b1*ynew(i)+c1;
		    const REEL ds1=a2*xnew(j)+b2*ynew(j)+c2;
		    const REEL ds2=a0*xc+b0*yc+c0;
		    REEL ids0=0.;
		    REEL ids1=0.;
		    REEL ids2=0.;

		// cas mailles triangulaire degeneree

		    if(ds0 == 0. || ds1 == 0. || ds2 == 0.) 
		    {
			continue;
		    }
		    else
		    {
			ids0 = 1./ds0;
			ids1 = 1./ds1;
			ids2 = 1./ds2;
		    }


		    REEL al0=ids0*(a1*xm+b1*ym+c1);
		    REEL al1=ids1*(a2*xm+b2*ym+c2);
		    REEL al2=ids2*(a0*xm+b0*ym+c0);
		    REEL som=al0+al1+al2;

		    al0/=som;testm=testm && al0*(al0-1.)<=0.;
		    al1/=som;testm=testm && al1*(al1-1.)<=0.;
		    al2/=som;testm=testm && al2*(al2-1.)<=0.;

		    if(testm)
		    {
			zm=al0*znew(i)+al1*znew(j)+al2*zc;
			break;
		    }
		    else
		    {
			testm=1;
		    }

		}

		if(i == nnu) testm = 0;

	    }

	    if(testm)
	    {
		Sommet s(xm,ym,zm);

		ts.AjouteClean(s);
		tma.Ajoute(num_maille);

		cout << "\nnumero de maille " << num_maille;
		cout << "\nxm " << xm << " ym " << ym << " zm " << zm << '\n';
		cout << "x " << xnew;
		cout << "y " << ynew;
		cout << "z " << znew;
	    }

	}

	nu.Vide();
	xnew.Vide();
	ynew.Vide();
	znew.Vide();
	anew.Vide();
	bnew.Vide();
	cnew.Vide();
	Anew.Vide();
	Bnew.Vide();
	Cnew.Vide();
	csnew.Vide();
	fnew.Vide();
	Fnew.Vide();
	alnew.Vide();

	if(timer)
	{
	    TimerDetReport(1,nma,"Temps calcul ChercheMailles");
	}

}

int
Geom::Break()
{
	RootWin::ExpIHM();

	if(_ecriture == FALSE)
	{
	    return -1;
	}

	return 0;
}

Test
Geom::Compare(unsigned numaill1,unsigned numaill2,P4Boolean init)
{

   //******************************************************************
   //* On Compare 2 mailles de la geometrie entre elles               *
   //* le resultat est a trois valeurs : 1 < 2                  (INF) *
   //*                                   1 > 2                  (SUP) *
   //*                                   1 et 2 non comparables (IND) *
   //* cette comparaison est interessante pour trier un maillage      *
   //* projete avec Geom::trigeom                                     *
   //* si init=TRUE,il faut avoir d'abord appele Geom::compminx et    * 
   //* Geom::compminy                                                 *
   //* pour initialiser _mininvx, _tx, _mininvy, _ty                  *
   //******************************************************************

    // pour vectorisation CRAY : sur CRAY les pointeurs temp restrict doivent
    // etre declares en tete, sinon l'info restrict n'est pas prise en compte.

#if defined(CRAY) && !defined(_CRAYMPP)
	const REEL_S* RESTRICT const rx = _x;
	const REEL_S* RESTRICT const ry = _y;
	const REEL_S* RESTRICT const rz = _z;
#endif

	compteur1++;

	if(init)
	{
	    int l = _mininvx(numaill1);
	    int m = _mininvx(numaill2);
            if(m-l >= (int)_tx(l)) return IND;
            if(l-m >= (int)_tx(m)) return IND;
            l=_mininvy(numaill1);
            m=_mininvy(numaill2);
            if(m-l >= (int)_ty(l)) return IND;
            if(l-m >= (int)_ty(m)) return IND;
	}

	compteur2++;

	static Tabunsignedbuf nu,nv;

	ptma->DonneSommets(nu,numaill1);
	ptma->DonneSommets(nv,numaill2);

	const unsigned nnu = nu;
	const unsigned nnv = nv;

	static REEL* RESTRICT x1 = NULL;
	static REEL* RESTRICT y1t = NULL;
	static REEL* RESTRICT z1 = NULL;
	static unsigned ancien_nnu_x = 0;

	if(nnu > ancien_nnu_x)
	{
	    delete[] x1;delete[] y1t;delete[] z1;
	    ancien_nnu_x=nnu;
	    x1 = new REEL[ancien_nnu_x];
	    y1t = new REEL[ancien_nnu_x];
	    z1 = new REEL[ancien_nnu_x];

	    if(!x1 || !y1t || !z1)
	    {
		cerr << " Probleme memoire dans Compare " << 
			__FILE__ << ' ' << __LINE__ << endl;
		delete[] x1;delete[] y1t;delete[] z1;
		x1 = NULL;y1t = NULL;z1 = NULL;
		ancien_nnu_x = 0;
	    
		return IND;
	    }

	}

	static REEL* RESTRICT x2 = NULL;
	static REEL* RESTRICT y2 = NULL;
	static REEL* RESTRICT z2 = NULL;
	static unsigned ancien_nnv_x = 0;

	if(nnv > ancien_nnv_x)
	{
	    delete[] x2;delete[] y2;delete[] z2;
	    ancien_nnv_x = nnv;
	    x2 = new REEL[ancien_nnv_x];
	    y2 = new REEL[ancien_nnv_x];
	    z2 = new REEL[ancien_nnv_x];

	    if(!x2 || !y2 || !z2)
	    {
		cerr << " Probleme memoire dans Compare " << 
			__FILE__ << ' ' << __LINE__ << endl;
		delete[] x2;delete[] y2;delete[] z2;
		x2 = NULL;y2 = NULL;z2 = NULL;
		ancien_nnv_x = 0;
	    
		return IND;
	    }

	}

#if !defined(CRAY) || defined(_CRAYMPP)
	const REEL_S* RESTRICT const rx = _x;
	const REEL_S* RESTRICT const ry = _y;
	const REEL_S* RESTRICT const rz = _z;
#endif

	register unsigned i=0;

	for (;i<nnu;i++)
	{
	    register const unsigned inu = nu(i);

	    x1[i] = rx[inu];
	    y1t[i] = ry[inu];
	    z1[i] = rz[inu];
	}

	for (i=0;i<nnv;i++)
	{
	    register const unsigned inv = nv(i);

	    x2[i] = rx[inv];
	    y2[i] = ry[inv];
	    z2[i] = rz[inv];
	}

	if(!init)
	{

	//si une droite verticale passe entre les deux mailles elles ne sont
	//pas comparables

	    register REEL minx = *x1;
	    register const REEL* px = x1+1;

	    for (;px<x1+nnu;px++) minx = min(minx,*px);

	    register REEL maxx=*x2;

	    for (px=x2+1;px<x2+nnv;px++) maxx = max(maxx,*px);

	    if(maxx<=minx) {return IND;}

	    minx=*x2;

	    for (px=x2+1;px<x2+nnv;px++) minx = min(minx,*px);

	    maxx=*x1;

	    for (px=x1+1;px<x1+nnu;px++) maxx = max(maxx,*px);

	    if(maxx<=minx) {return IND;}

	//si une droite horizontale passe entre les deux mailles elles ne sont
	//pas comparables

	    minx=*y1t;

	    for (px=y1t+1;px<y1t+nnu;px++) minx = min(minx,*px);

	    maxx=*y2;

	    for (px=y2+1;px<y2+nnv;px++) maxx = max(maxx,*px);

	    if(maxx<=minx) {return IND;}

	    minx=*y2;

	    for (px=y2+1;px<y2+nnv;px++) minx = min(minx,*px);

	    maxx=*y1t;

	    for (px=y1t+1;px<y1t+nnu;px++) maxx = max(maxx,*px);

	    if(maxx<=minx) {return IND;}

	    compteur3++;
	}

   // Si une droite quelconque passe entre les deux mailles elles ne sont
   // pas comparables.

	REEL tmin = -Bmax,tmax = Bmax;
	const REEL* px;const REEL* py;

	for(px=x1,py=y1t;px<x1+nnu;px++,py++)
	{
	    register const REEL xx1 = *px,yy1 = *py;

	    for(REEL* pxx=x2,*pyy=y2;pxx<x2+nnv;pxx++,pyy++)
	    {
		register const REEL xx2 = *pxx,yy2 = *pyy;

		if(xx1 > xx2)
		{
		    register const REEL rap = (yy2-yy1)/(xx1-xx2);

		    tmin = max(rap,tmin);
		}
		else if(xx1 < xx2)
		{
		    register const REEL rap = (yy2-yy1)/(xx1-xx2);

		    tmax = min(rap,tmax);
		}
		else if(yy1 < yy2)
		{
		    tmin = Bmax;
		    tmax = -Bmax;

		    break;
		}

	    }

	    if(tmax < tmin) break;

	}

	if(tmax >= tmin) {return IND;}

#ifdef DBG
	REEL tmin1 = tmin,tmax1 = tmax;
#endif

	tmin = -Bmax,tmax = Bmax;

	for(px=x1,py=y1t;px<x1+nnu;px++,py++)
	{
	    register const REEL xx1 = *px,yy1 = *py;

	    for(REEL* pxx=x2,*pyy=y2;pxx<x2+nnv;pxx++,pyy++)
	    {
		register const REEL xx2 = *pxx,yy2 = *pyy;

		if(xx1 > xx2)
		{
		    register const REEL rap = (yy1-yy2)/(xx1-xx2);

		    tmin=max(rap,tmin);
		}
		else if(xx1 < xx2)
		{
		    register const REEL rap = (yy1-yy2)/(xx1-xx2);

		    tmax=min(rap,tmax);
		}
		else if(yy1 > yy2)
		{
		    tmin=Bmax;
		    tmax=-Bmax;

		    break;
		}

	    }

	    if(tmax < tmin) break;

	}

	if(tmax >= tmin) {return IND;}

#ifdef DBG
	REEL tmin2 = tmin,tmax2 = tmax;
#endif

	compteur4++;

   // si un plan z=cste passe entre les deux mailles et si elles ne
   // sont pas IND elles sont SUP ou INF

	register REEL minz = *z1;
	register const REEL* pz = z1+1;

	for (;pz<z1+nnu;pz++) minz = min(minz,*pz);

	register REEL maxz = *z2;

	for (pz=z2+1;pz<z2+nnv;pz++) maxz = max(maxz,*pz);

	if(maxz<=minz) {return SUP;}

	minz=*z2;

	for (pz=z2+1;pz<z2+nnv;pz++) minz = min(minz,*pz);

	maxz=*z1;

	for (pz=z1+1;pz<z1+nnu;pz++) maxz = max(maxz,*pz);

	if(maxz<=minz) {return INF;}

	compteur5++;

   // si un plan quelconque passe entre les deux mailles et si elles ne
   // sont pas IND elles sont SUP ou INF (a faire un jour peut etre ...)

   // pour les autres cas, on Compare

   // test si maille convexe et non croisees
   // sinon on decompose en sous-mailles convexes (a faire en
   // preprocesseur de la geometrie avec P4Geom4::Convexe => plus de mailles)

   // *****************************************************************
   // *calcul des parametres d'interpolations pour 2 mailles convexes *
   // *a nombre de cotes quelconques                                  *
   // *methode elements finis generalises par                         *
   // *fractions rationnelles :                                       *
   // *ref: Wachspress E. L. "A rational basis for function           *
   // * approximation" Lecture Notes in Mathematics (Proceedings of   *
   // * the Conference in Application of Numerical Analysis)          *
   // * vol 228 Springer-Verlag, Berlin, 1971 pp 223,252.             *
   // * + eclatement en mailles triangulaires pour les mailles        *
   // * degenerees                                                    *
   // *****************************************************************

	static REEL* RESTRICT a1 = NULL;
	static REEL* RESTRICT b1 = NULL;
	static REEL* RESTRICT c1 = NULL;
	static REEL* RESTRICT A1 = NULL;
	static REEL* RESTRICT B1 = NULL;
	static REEL* RESTRICT C1 = NULL;
	static REEL* RESTRICT cs1 = NULL;
	static REEL* RESTRICT f1 = NULL;
	static REEL* RESTRICT F1 = NULL;
	static unsigned ancien_nnu = 0;
	int ancien = 0;

	if(nnu > ancien_nnu)
	{
	    delete[] a1;delete[] b1;delete[] c1;
	    delete[] A1;delete[] B1;delete[] C1;
	    delete[] cs1;delete[] f1;delete[] F1;
	    ancien_nnu = nnu;
	    ancien = 1;

	    a1=new REEL[ancien_nnu];
	    b1=new REEL[ancien_nnu];
	    c1=new REEL[ancien_nnu];
	    cs1=new REEL[ancien_nnu];
	    f1=new REEL[ancien_nnu];

	    const unsigned ancien_nnu3 = max(ancien_nnu-3,1u);

	    A1=new REEL[ancien_nnu3];
	    B1=new REEL[ancien_nnu3];
	    C1=new REEL[ancien_nnu3];
	    F1=new REEL[ancien_nnu3];

	    if(!a1 || !b1 || !c1 || !A1 || !B1 || !C1
		|| !cs1 || !f1 || !F1)
	    {
		cerr << " Probleme memoire dans Compare " << 
			__FILE__ << ' ' << __LINE__ << endl;
		delete[] a1;delete[] b1;delete[] c1;
		delete[] A1;delete[] B1;delete[] C1;
		delete[] cs1;delete[] f1;delete[] F1;
		a1 = NULL;b1 = NULL;c1 = NULL;
		A1 = NULL;B1 = NULL;C1 = NULL;
		cs1 = NULL;f1 = NULL;F1 = NULL;
		ancien_nnu = 0;
	    
		return IND;
	    }

	}

	static REEL* RESTRICT a2 = NULL;
	static REEL* RESTRICT b2 = NULL;
	static REEL* RESTRICT c2 = NULL;
	static REEL* RESTRICT A2 = NULL;
	static REEL* RESTRICT B2 = NULL;
	static REEL* RESTRICT C2 = NULL;
	static REEL* RESTRICT cs2 = NULL;
	static REEL* RESTRICT f2 = NULL;
	static REEL* RESTRICT F2 = NULL;
	static unsigned ancien_nnv = 0;

	if(nnv > ancien_nnv)
	{
	    delete[] a2;delete[] b2;delete[] c2;
	    delete[] A2;delete[] B2;delete[] C2;
	    delete[] cs2;delete[] f2;delete[] F2;
	    ancien_nnv = nnv;
	    ancien = 1;

	    a2=new REEL[ancien_nnv];
	    b2=new REEL[ancien_nnv];
	    c2=new REEL[ancien_nnv];
	    cs2=new REEL[ancien_nnv];
	    f2=new REEL[ancien_nnv];

	    const unsigned ancien_nnv3 = max(ancien_nnv-3,1u);

	    A2=new REEL[ancien_nnv3];
	    B2=new REEL[ancien_nnv3];
	    C2=new REEL[ancien_nnv3];
	    F2=new REEL[ancien_nnv3];

	    if(!a2 || !b2 || !c2 || !A2 || !B2 || !C2
		|| !cs2 || !f2 || !F2)
	    {
		cerr << " Probleme memoire dans Compare " << 
			__FILE__ << ' ' << __LINE__ << endl;
		delete[] a2;delete[] b2;delete[] c2;
		delete[] A2;delete[] B2;delete[] C2;
		delete[] cs2;delete[] f2;delete[] F2;
		a2 = NULL;b2 = NULL;c2 = NULL;
		A2 = NULL;B2 = NULL;C2 = NULL;
		cs2 = NULL;f2 = NULL;F2 = NULL;
		ancien_nnv = 0;
	    
		return IND;
	    }

	}

	static REEL* RESTRICT al = NULL;

	if(ancien)
	{
	    delete[] al;

	    al = new REEL[max(ancien_nnu,ancien_nnv)];

	    if(!al)
	    {
		cerr << " Probleme memoire dans Compare " << 
			__FILE__ << ' ' << __LINE__ << endl;
		delete[] a1;delete[] b1;delete[] c1;
		delete[] A1;delete[] B1;delete[] C1;
		delete[] cs1;delete[] f1;delete[] F1;
		delete[] a2;delete[] b2;delete[] c2;
		delete[] A2;delete[] B2;delete[] C2;
		delete[] cs2;delete[] f2;delete[] F2;delete[] al;
		a1 = NULL;b1 = NULL;c1 = NULL;
		A1 = NULL;B1 = NULL;C1 = NULL;
		cs1 = NULL;f1 = NULL;F1 = NULL;
		a2 = NULL;b2 = NULL;c2 = NULL;
		A2 = NULL;B2 = NULL;C2 = NULL;
		cs2 = NULL;f2 = NULL;F2 = NULL;al = NULL;
		ancien_nnu = 0;
		ancien_nnv = 0;

		return IND;
	    }

	}

    // parametres d'interpolation... pour la deuxieme maille

	for (i=0;i<nnv;i++)
	{
	    register const unsigned j = (i+1)%nnv;

	    a2[i] = y2[j]-y2[i];
	    b2[i] = x2[i]-x2[j];
	    c2[i] = y2[i]*x2[j]-y2[j]*x2[i];
	}

    // test maille degeneree

	P4Boolean degen2=FALSE;

	for (i=0;i<nnv;i++)
	{
	    register const unsigned j = (i+1)%nnv;

	    if(a2[i]*b2[j]-a2[j]*b2[i] == 0.) degen2 = TRUE;

	}

	if(degen2 == FALSE)
	{

	    for (i=0;i<nnv-3;i++)
	    {
		register const unsigned j = (i+1)%nnv;
		register const unsigned k = (j+1)%nnv;
		register const unsigned l = (k+1)%nnv;
		const REEL xxci = a2[i]*c2[k]-c2[i]*a2[k];
		const REEL yyci = b2[j]*a2[l]-a2[j]*b2[l];
		const REEL zzci = c2[l]*a2[j]-a2[l]*c2[j];
		const REEL ttci = b2[k]*a2[i]-a2[k]*b2[i];
		const REEL uuci = c2[k]*b2[i]-c2[i]*b2[k];
		const REEL vvci = b2[j]*c2[l]-c2[j]*b2[l];

		A2[i] = xxci*yyci+zzci*ttci;
		B2[i] = yyci*uuci+vvci*ttci;
		C2[i] = vvci*xxci-zzci*uuci;
	    }

	    for (i=0;i<nnv;i++)
	    {
		register REEL cs2temp = 1.;
		register unsigned j;

		for (j=0;j<nnv-3;j++)
		{
		    cs2temp *= (A2[j]*x2[i]+B2[j]*y2[i]+C2[j]);
		}

		for (j=0;j<nnv-2;j++)
		{
		    register const unsigned k = (j+i+1)%nnv;

		    cs2temp /= (a2[k]*x2[i]+b2[k]*y2[i]+c2[k]);
		}

		cs2[i] = cs2temp;
	    }

	}

    // calcul zm1 et zm2 pour barycentre 1ere maille

	REEL xm = 0.;
	REEL ym = 0.;
	REEL zm1 = 0.;

	for (i=0;i<nnu;i++)
	{
	   xm += x1[i];
	   ym += y1t[i];
	   zm1 += z1[i];
	}

	xm/=nnu;
	ym/=nnu;
	zm1/=nnu;

    // zm2   ..........................

	REEL zm2 = 0.;
	int tm2 = 1;

	if(degen2 == FALSE)
	{

	// cas mailles non degenerees

	    for (i=0;i<nnv;i++) f2[i] = a2[i]*xm+b2[i]*ym+c2[i];

	    for (i=0;i<nnv-3;i++) F2[i] = A2[i]*xm+B2[i]*ym+C2[i];

	    register REEL som = 0.;

	    for (i=0;i<nnv;i++)
	    {
		register REEL all = cs2[i];
		register unsigned j;

		for (j=0;j<nnv-3;j++) all /= F2[j];

		for (j=0;j<nnv-2;j++) all *= f2[(j+i+1)%nnv];

		al[i] = all;
		zm2 += all*z2[i];
		som += all;
	    }

	    for (i=0;i<nnv;i++) // vectorise
	    {
		register const REEL all = al[i]/som;

		tm2 *= all*(all-1.) <= 0.;
	    }

	    zm2 /= som;
	}
	else
	{
	    REEL xc = 0.;
	    REEL yc = 0.;
	    REEL zc = 0.;

	    for (i=0;i<nnv;i++)
	    {
		xc += x2[i];
		yc += y2[i];
		zc += z2[i];
	    }

	    xc /= nnv;
	    yc /= nnv;
	    zc /= nnv;

	    for(i=0;i<nnv;i++)
	    {
		register const unsigned j = (i+1)%nnv;
		const REEL a0 = y2[j]-y2[i];
		const REEL b0 = x2[i]-x2[j];
		const REEL c0 = y2[i]*x2[j]-y2[j]*x2[i];
		const REEL a1t = yc-y2[j];
		const REEL b1t = x2[j]-xc;
		const REEL c1t = y2[j]*xc-yc*x2[j];
		const REEL a2t = y2[i]-yc;
		const REEL b2t = xc-x2[i];
		const REEL c2t = yc*x2[i]-y2[i]*xc;
		const REEL ds0 = a1t*x2[i]+b1t*y2[i]+c1t;
		const REEL ds1 = a2t*x2[j]+b2t*y2[j]+c2t;
		const REEL ds2 = a0*xc+b0*yc+c0;
		REEL ids0 = 0.;
		REEL ids1 = 0.;
		REEL ids2 = 0.;

	    // cas mailles triangulaire degeneree

		if(ds0 == 0. || ds1 == 0. || ds2 == 0.) 
		{
		    continue;
		}
		else
		{
		    ids0 = 1./ds0;
		    ids1 = 1./ds1;
		    ids2 = 1./ds2;
		}

		REEL al0 = ids0*(a1t*xm+b1t*ym+c1t);
		REEL al1 = ids1*(a2t*xm+b2t*ym+c2t);
		REEL al2 = ids2*(a0*xm+b0*ym+c0);
		const REEL som = al0+al1+al2;

		al0 /= som;tm2 = tm2 && al0*(al0-1.)<=0.;
		al1 /= som;tm2 = tm2 && al1*(al1-1.)<=0.;
		al2 /= som;tm2 = tm2 && al2*(al2-1.)<=0.;

		if(tm2)
		{
		    zm2 = al0*z2[i]+al1*z2[j]+al2*zc;
		    break;
		}
		else
		{
		    tm2 = 1;
		}

	    }

	    if(i == nnv) tm2 = 0;

	}

    // comparaison zm1 et zm2 et sortie si le barycentre en x et y de la
    // maille 1 est a l'interieur de la maille 2

	if(tm2) 
	{

	    if(zm1 < zm2)  {return INF;}
	    if(zm1 == zm2) {return IND;}
	    else           {return SUP;}

	}

    // parametres d'interpolation... pour la premiere maille

	for (i=0;i<nnu;i++)
	{
	    register const unsigned j=(i+1)%nnu;

	    a1[i] = y1t[j]-y1t[i];
	    b1[i] = x1[i]-x1[j];
	    c1[i] = y1t[i]*x1[j]-y1t[j]*x1[i];
	}

	// test maille degeneree

	P4Boolean degen1 = FALSE;

	for (i=0;i<nnu;i++)
	{
	    register const unsigned j=(i+1)%nnu;

	    if(a1[i]*b1[j]-a1[j]*b1[i] == 0.) degen1=TRUE;

	}

	if(degen1 == FALSE)
	{
	    const int nnu2 = nnu-2;
	    const int nnu3 = nnu-3;

	    for (i=0;int(i)<nnu3;i++)
	    {
		register const unsigned j=(i+1)%nnu;
		register const unsigned k=(j+1)%nnu;
		register const unsigned l=(k+1)%nnu;
		const REEL xxci=a1[i]*c1[k]-c1[i]*a1[k];
		const REEL yyci=b1[j]*a1[l]-a1[j]*b1[l];
		const REEL zzci=c1[l]*a1[j]-a1[l]*c1[j];
		const REEL ttci=b1[k]*a1[i]-a1[k]*b1[i];
		const REEL uuci=c1[k]*b1[i]-c1[i]*b1[k];
		const REEL vvci=b1[j]*c1[l]-c1[j]*b1[l];

		A1[i]=xxci*yyci+zzci*ttci;
		B1[i]=yyci*uuci+vvci*ttci;
		C1[i]=vvci*xxci-zzci*uuci;
	    }

	    for (i=0;i<nnu;i++)
	    {
		register REEL cs1temp = 1.;
		int j;

		for (j=0;j<nnu3;j++)
		{
		    cs1temp *= (A1[j]*x1[i]+B1[j]*y1t[i]+C1[j]);
		}

		for (j=0;j<nnu2;j++)
		{
		    register const unsigned k=(j+i+1)%nnu;

		    cs1temp /= (a1[k]*x1[i]+b1[k]*y1t[i]+c1[k]);
		}

		cs1[i] = cs1temp;
	    }

	}

    // calcul zm1 et zm2 pour barycentre 2eme maille

	xm = 0.;
	ym = 0.;
	zm2 = 0.;

	for (i=0;i<nnv;i++)
	{
	   xm += x2[i];
	   ym += y2[i];
	   zm2 += z2[i];
	}

	xm /= nnv;
	ym /= nnv;
	zm2 /= nnv;

    // zm1  .........................................

	zm1=0.;
	int tm1=1;

	if(degen1 == FALSE)
	{
	    const int nnu2 = nnu-2;
	    const int nnu3 = nnu-3;

	    for (i=0;i<nnu;i++) f1[i]=a1[i]*xm+b1[i]*ym+c1[i];

	    for (i=0;int(i)<nnu3;i++) F1[i]=A1[i]*xm+B1[i]*ym+C1[i];

	    REEL som=0.;

	    for (i=0;i<nnu;i++)
	    {
		REEL all = cs1[i];
		int j;

		for (j=0;j<nnu3;j++) all /= F1[j];

		for (j=0;j<nnu2;j++) all *= f1[(j+i+1)%nnu];

		al[i] = all;
		zm1 += all*z1[i];
		som += all;
	    }

	    for (i=0;i<nnu;i++) // vectorise
	    {
		register const REEL all = al[i]/som;

		tm1 *= all*(all-1.) <= 0.;
	    }

	    zm1/=som;
	}
	else
	{
	    REEL xc = 0.;
	    REEL yc = 0.;
	    REEL zc = 0.;

	    for (i=0;i<nnu;i++)
	    {
		xc += x1[i];
		yc += y1t[i];
		zc += z1[i];
	    }

	    xc/=nnu;
	    yc/=nnu;
	    zc/=nnu;

	    for(i=0;i<nnu;i++)
	    {
		register const unsigned j = (i+1)%nnu;
		const REEL a0 = y1t[j]-y1t[i];
		const REEL b0 = x1[i]-x1[j];
		const REEL c0 = y1t[i]*x1[j]-y1t[j]*x1[i];
		const REEL a1t = yc-y1t[j];
		const REEL b1t = x1[j]-xc;
		const REEL c1t = y1t[j]*xc-yc*x1[j];
		const REEL a2t = y1t[i]-yc;
		const REEL b2t = xc-x1[i];
		const REEL c2t = yc*x1[i]-y1t[i]*xc;
		const REEL ds0 = a1t*x1[i]+b1t*y1t[i]+c1t;
		const REEL ds1 = a2t*x1[j]+b2t*y1t[j]+c2t;
		const REEL ds2 = a0*xc+b0*yc+c0;
		REEL ids0=0.;
		REEL ids1=0.;
		REEL ids2=0.;

	    // cas mailles triangulaire degeneree

		if(ds0 == 0. || ds1 == 0. || ds2 == 0.) 
		{
		    continue;
		}
		else
		{
		    ids0=1./ds0;
		    ids1=1./ds1;
		    ids2=1./ds2;
		}


		REEL al0=ids0*(a1t*xm+b1t*ym+c1t);
		REEL al1=ids1*(a2t*xm+b2t*ym+c2t);
		REEL al2=ids2*(a0*xm+b0*ym+c0);
		const REEL som=al0+al1+al2;

		al0/=som;tm1=tm1 && al0*(al0-1.)<=0.;
		al1/=som;tm1=tm1 && al1*(al1-1.)<=0.;
		al2/=som;tm1=tm1 && al2*(al2-1.)<=0.;

		if(tm1)
		{
		    zm1=al0*z1[i]+al1*z1[j]+al2*zc;
		    break;
		}
		else
		{
		    tm1=1;
		}

	    }

	    if(i == nnu) tm1 = 0;

	}

    // comparaison zm1 et zm2 et sortie si le barycentre en x et y de la
    // maille 2 est a l'interieur de la maille 1

	if(tm1) 
	{

	    if(zm1 < zm2)  {return INF;}
	    if(zm1 == zm2) {return IND;}
	    else           {return SUP;}

	}

    // sinon on cherche toutes les intersections de deux segments
    // des deux mailles et on compare les z ayant l'ecart maximum

	static TabREELbuf z_inter_1,z_inter_2;
	const unsigned* const tnu = nu;
	const unsigned* const tnv = nv;

	for (register unsigned i1=0;i1<nnu;i1++)
	{
	    register const unsigned j1t = (i1+1)%nnu;

	    for (register unsigned i2=0;i2<nnv;i2++)
	    {
		register const unsigned j2 = (i2+1)%nnv;
		const REEL den = a2[i2]*b1[i1]-a1[i1]*b2[i2];

		if(den!=0. && tnu[i1]!=tnv[i2] && tnu[j1t]!=tnv[j2] &&
				tnu[i1]!=tnv[j2] && tnu[j1t]!=tnv[i2] &&
				(x1[i1]!=x2[i2] || y1t[i1]!=y2[i2] ||
						z1[i1]!=z2[i2]) &&
				(x1[i1]!=x2[j2] || y1t[i1]!=y2[j2] ||
						z1[i1]!=z2[j2]) &&
				(x1[j1t]!=x2[j2] || y1t[j1t]!=y2[j2] ||
						z1[j1t]!=z2[j2]) &&
				(x1[j1t]!=x2[i2] || y1t[j1t]!=y2[i2] ||
						z1[j1t]!=z2[i2]))
		{
		    const REEL xmt = (c1[i1]*b2[i2]-c2[i2]*b1[i1])/den;
		    const REEL ymt = (a1[i1]*c2[i2]-a2[i2]*c1[i1])/den;

		    if((x1[i1]-xmt)*(x1[j1t]-xmt)+
				    (y1t[i1]-ymt)*(y1t[j1t]-ymt)<=0. &&
				    (x2[i2]-xmt)*(x2[j2]-xmt)+
				    (y2[i2]-ymt)*(y2[j2]-ymt)<=0.)
		    {
			const REEL al2 = (abs(x1[i1]-xmt)+abs(y1t[i1]-ymt))/
				(abs(x1[i1]-x1[j1t])+abs(y1t[i1]-y1t[j1t]));
			const REEL al1 = 1.-al2;

			z_inter_1.Ajoute(al1*z1[i1]+al2*z1[j1t]);

			const REEL al4 = (abs(x2[i2]-xmt)+abs(y2[i2]-ymt))/
				(abs(x2[i2]-x2[j2])+abs(y2[i2]-y2[j2]));
			const REEL al3 = 1.-al4;

			z_inter_2.Ajoute(al3*z2[i2]+al4*z2[j2]);
		    }

		}

	    }

	}

	int max_intervalle = -1;
	REEL intervalle = 0.;
	unsigned lz_inter_1 = z_inter_1;

	for(i=0;i<lz_inter_1;i++)
	{
	    register REEL new_intervalle=0.;

	    if((new_intervalle=abs(z_inter_2(i)-z_inter_1(i))) >= intervalle)
	    {
		max_intervalle=i;
		intervalle=new_intervalle;
	    }

	}

	if(max_intervalle>=0)
	{
	    zm1=z_inter_1(max_intervalle);
	    zm2=z_inter_2(max_intervalle);

	    z_inter_1.Enleve();
	    z_inter_2.Enleve();

	    if(zm1 == zm2) {return IND;}
	    if(zm1 < zm2)  {return INF;}
	    else	   {return SUP;}

	}

	z_inter_1.Enleve();
	z_inter_2.Enleve();

	compteur6++;

#ifdef DBG
	if(compteur6 == 1)
	{
	    cout << " maille " << numaill1 << endl;

	    for(i=0;i<nnu;i++)
	    {
		cout << x1[i] << ' ' << y1t[i] << endl;
	    }

	    cout << " maille " << numaill2 << endl;

	    for(i=0;i<nnv;i++)
	    {
		cout << x2[i] << ' ' << y2[i] << endl;
	    }

	    cout << " tmin1 " << tmin1 << " tmax1 " << tmax1 << endl;
	    cout << " tmin2 " << tmin2 << " tmax2 " << tmax2 << endl;
	}
#endif

	return IND;

   // on peut aussi traiter les mailles degenerees avec Wachpress
   // plutot qu'en decoupant les mailles degenerees en triangle
   // comme implemente
   // (les prototypes sont a initialises avec init de la geom)
   // ...

}

void
Geom::Compminx()
{

    //**********************************************************
    //* on trie (quicksort) les min des x pour chaques mailles *
    //* on remplit _mininvx avec  un tableau qui contient la   *
    //* place dans le tri de chaque maille                     *
    //* et _tx avec la valeur Test apres passage dans Compare  *
    //* de deux elements successifs tries                      *
    //* ceci permet d'etre rapide dans le cas ou les mailles   *
    //* se cachent peu                                         *
    //**********************************************************

	Tabunsignedbuf nu;
	const unsigned nma = ptma->Nmailles();

	if(!nma)
	{
	    _tx_max = Dep;
	    _tx.Enleve();
	    _minx.Enleve();
	    _mininvx.Enleve();

	    return;
	}

	REEL_S* pf = new REEL_S[nma];
	REEL_S* qf = new REEL_S[nma];
	unsigned j;

	for(j=0;j<nma;j++)
	{
	    ptma->DonneSommets(nu,j);

	    register const unsigned nnu = nu;
	    register const unsigned* const tnu = nu;

	    register REEL_S minx = _x(tnu[0]);
	    register REEL_S maxx = minx;

	    for (register unsigned i=1;i<nnu;i++)
	    {
		register const REEL_S xnui = _x(tnu[i]);

		minx = min(minx,xnui);
		maxx = max(maxx,xnui);
	    }

	    pf[j] = minx;
	    qf[j] = maxx;
	}

	nu.Enleve();

	unsigned* RESTRICT indi = new unsigned[nma];
	quicksort(pf,indi,nma);
	unsigned* RESTRICT tind = new unsigned[nma];

	for(j=0;j<nma;j++)
	{
	    tind[j] = Dep;
	}

	const unsigned nma1 = nma-1;

	for(j=0;j<nma1;j++)
	{
	    register int imin = j;
	    register int imax = nma;
	    register int imil = 0;

	    while((imax-imin) > 1)
	    {
		imil = (imin+imax) >> 1;

		if(qf[indi[j]]<=pf[indi[imil]]) imax = imil;
		else imin = imil;

	    }

	    tind[j] = imax-j;
	}

	delete[] pf;
	delete[] qf;

	_tx_max = tind[0];

	for (j=1;j<nma1;j++)
	{
	    _tx_max = max(_tx_max,tind[j]);
	}

	_tx.Enleve();
	_tx.Ajoute(tind,nma);

	delete[] tind;

	unsigned* RESTRICT indj = new unsigned[nma];

	for (j=0;j<nma;j++)
	{
	    indj[indi[j]] = j;
	}

	_minx.Enleve();
	_minx.Ajoute(indi,nma);

	delete[] indi;

	_mininvx.Enleve();
	_mininvx.Ajoute(indj,nma);

	delete[] indj;
}

void
Geom::Compminy()
{

   //**********************************************************
   //* on trie (quicksort) les min des y pour chaque maille   *
   //* on remplit _mininvy avec  un tableau qui contient la   *
   //* place dans le tri de chaque maille                     *
   //* et _ty avec la valeur Test apres passage dans Compare  *
   //* de deux elements successifs tries                      *
   //* ceci permet d'etre rapide dans le cas ou les mailles   *
   //* se cachent peu                                         *
   //**********************************************************

	Tabunsignedbuf nu;
	const unsigned nma = ptma->Nmailles();

	if(!nma)
	{
	    _ty.Enleve();
	    _mininvy.Enleve();

	    return;
	}

	REEL_S* pf = new REEL_S[nma];
	REEL_S* qf = new REEL_S[nma];
	unsigned j;

	for(j=0;j<nma;j++)
	{
	   ptma->DonneSommets(nu,j);

	   register const unsigned nnu = nu;
	   register const unsigned* const tnu = nu;
	   register REEL_S minx = _y(tnu[0]);
	   register REEL_S maxx = minx;

	   for (register unsigned i=1;i<nnu;i++)
	   {
	       register const REEL_S ynu = _y(tnu[i]);

	       minx = min(minx,ynu);
	       maxx = max(maxx,ynu);
	   }

	   pf[j] = minx;
	   qf[j] = maxx;
	}

	nu.Enleve();

	unsigned* RESTRICT indi = new unsigned[nma];

	quicksort(pf,indi,nma);

	unsigned* RESTRICT tind = new unsigned[nma];

	for (j=0;j<nma;j++)
	{
	    tind[j] = Dep;
	}

	const unsigned nma1 = nma-1;

	for (j=0;j<nma1;j++)
	{
	    register int imin = j;
	    register int imax = nma;
	    register int imil = 0;

	    while((imax-imin) > 1)
	    {
		imil = (imin+imax) >> 1;

		if(qf[indi[j]]<=pf[indi[imil]]) imax = imil;
		else imin = imil;

	    }

	    tind[j] = imax-j;
	}

	delete[] pf;
	delete[] qf;

	_ty.Enleve();
	_ty.Ajoute(tind,nma);

	delete[] tind;

	unsigned* RESTRICT indj = new unsigned[nma];

	for (j=0;j<nma;j++)
	{
	   indj[indi[j]] = j;
	}

	delete[] indi;

	_mininvy.Enleve();
	_mininvy.Ajoute(indj,nma);

	delete[] indj;
}

void
Geom::Delminx()
{
	_minx.Enleve();
	_mininvx.Enleve();
	_tx.Enleve();
}

void
Geom::Delminy()
{
	_mininvy.Enleve();
	_ty.Enleve();
}
