#include "minotaure/P4graf0.h"
#include "P4proj.h"
#include "P4sommet.h"
#include "P4boite.h"
#include "P4bool3.h"
#include "P4tronc.h"
#include "P4declog.h"

/************************************************************************
 *  Rappel de bonne programmation en c++ :                              *
 *  Pour rajouter des fonctions membres de cette classe                 *
 *  utiliser l'heritage et ne pas modifier P4graf0.c !!                 *
 ************************************************************************/

struct SccsIdP4Graf0
{
	static const char* const sccs_id;
};

const char* const SccsIdP4Graf0::sccs_id = "%W% %G% Pierre Brochard";

P4Graf0::P4Graf0() :
		_ecriture(TRUE),
		_time(FALSE),_time_geom(FALSE),
		_projh(*new Proj(-0.5,0.5,0.,
			-0.292893,-0.292892,0.414215,
			-0.5,-0.5,-0.707105,
			-0.5,-0.5,-0.707105,
			0.5,0.5,1.,
			FALSE)),
		_proja(*new Proj(-0.5,0.5,0.,
			-0.292893,-0.292892,0.414215,
			-0.5,-0.5,-0.707105,
			-0.5,-0.5,-0.707105,
			0.5,0.5,1.,
			FALSE)),
		_orig(*new Sommet(.5,.5,.5)),
		_omin(*new Sommet(0.,0.,0.)),
		_omax(*new Sommet(1.,1.,1.)),
		_boite(*new Boite()),
		_boite_proj(*new Boite()),
		_lxyz(*new Bool4(FALSE,FALSE,FALSE,FALSE)),
		_lixyz(*new Bool3(TRUE,TRUE,TRUE)),
		_modetrace(0),
		_tronc(*new Tronc(TRONC3D_NOTRONC,TRONC3D_NOTRONC,
			TRONC3D_NOTRONC,TRUE,FALSE)),
		_dl(*new Declog()),
		_masq(MASQUE_SIMPLE_MAX),
		_karv_titre(2),_karv_echelle(1),
		_icob(1),_icoh(0),_icolor(0),
		_isupv(1),
		_c1(.707),_s1(.707),
		_c2(-.866),_s2(.5),
		_c3(1.),_s3(0.),
		_boit(TRUE),_nosoc(FALSE),
		_nobas(TRUE),_noba(TRUE),
		_mail(FALSE),_inseg(FALSE),
		_ax(45.),_ay(150.),_az(0.),
		_ccx(1.),_ccy(1.),_ccz(1.),
		_xl(.510),_yl(.505),_zl(.35),
		_zcotes(0.),_re(1.),_rey(1.),
		_flr(0.),_fl(0.),_fnr(1.),_fu(1.),_fup(1.),_fur(1.),
		_init(0),_ncotes(-1),
		_xcen(.5),_ycen(.5),_scalx(.9),_scaly(.9)
{
}

P4Graf0::P4Graf0(const P4Graf0& g) :
		_ecriture(g._ecriture),
		_time(g._time),_time_geom(g._time_geom),
		_projh(*new Proj(g._projh)),
		_proja(*new Proj(g._proja)),
		_orig(*new Sommet(g._orig)),
		_omin(*new Sommet(g._omin)),
		_omax(*new Sommet(g._omax)),
		_boite(*new Boite(g._boite)),
		_boite_proj(*new Boite(g._boite_proj)),
		_lxyz(*new Bool4(g._lxyz)),
		_lixyz(*new Bool3(g._lixyz)),
		_modetrace(g._modetrace),
		_tronc(*new Tronc(g._tronc)),
		_dl(*new Declog(g._dl)),
		_masq(g._masq),
		_karv_titre(g._karv_titre),_karv_echelle(g._karv_echelle),
		_icob(g._icob),_icoh(g._icoh),_icolor(g._icolor),
		_isupv(g._isupv),
		_c1(g._c1),_s1(g._s1),
		_c2(g._c2),_s2(g._s2),
		_c3(g._c3),_s3(g._s3),
		_boit(g._boit),_nosoc(g._nosoc),
		_nobas(g._nobas),_noba(g._noba),
		_mail(g._mail),_inseg(g._inseg),
		_ax(g._ax),_ay(g._ay),_az(g._az),
		_ccx(g._ccx),_ccy(g._ccy),_ccz(g._ccz),
		_xl(g._xl),_yl(g._yl),_zl(g._zl),
		_flr(g._flr),_fl(g._fl),_fnr(g._fnr),
		_fu(g._fu),_fup(g._fup),_fur(g._fur),
		_init(g._init),_ncotes(g._ncotes),
		_xcen(g._xcen),_ycen(g._ycen),_scalx(g._scalx),_scaly(g._scaly)	
{
}

P4Graf0::~P4Graf0()
{
	Proj* ph = &_projh;
	Proj* pa = &_proja;
	delete ph;delete pa;

	Sommet* sorig = &_orig;
	Sommet* somin = &_omin;
	Sommet* somax = &_omax;
	delete sorig;delete somin;delete somax;

	Boite* bo = &_boite;
	Boite* bop = &_boite_proj;
	delete bo;delete bop;

	Bool4* lx = &_lxyz;
	Bool3* lix = &_lixyz;
	delete lx;delete lix;

	Tronc* tr = &_tronc;
	delete tr;

	Declog* dl = &_dl;
	delete dl;
}

void P4Graf0::operator = (const P4Graf0& g)
{
	_ecriture=g._ecriture;

	_time=g._time,_time_geom=g._time_geom;

	_modetrace=g._modetrace;

	_masq=g._masq;

	_isupv=g._isupv;

	_c1=g._c1,_c2=g._c2,_c3=g._c3;

	_s1=g._s1,_s2=g._s2,_s3=g._s3;

	_karv_titre=g._karv_titre,_karv_echelle=g._karv_echelle;

	_boit=g._boit,_nosoc=g._nosoc,
	_nobas=g._nobas,_noba=g._noba,
	_mail=g._mail,_inseg=g._inseg;

	_ax=g._ax,_ay=g._ay,_az=g._az;

	_ccx=g._ccx,_ccy=g._ccy,_ccz=g._ccz;

	_xl=g._xl,_yl=g._yl,_zl=g._zl;

	_icob=g._icob,_icoh=g._icoh,_icolor=g._icolor;

	_init=g._init,_ncotes=g._ncotes;

	_zcotes=g._zcotes,_re=g._re,_rey=g._rey;

	_fup=g._fup,_fnr=g._fnr,_flr=g._flr,_fur=g._fur,_fl=g._fl,_fu=g._fu;

	_xcen = g._xcen,_ycen = g._ycen,_scalx = g._scalx,_scaly = g._scaly;	
}

ostream& operator << (ostream& ost,const P4Graf0& p23)
{
	ost << " Impression des membres statiques de P4Graf0\n";
	ost << "  time :\n" << p23._time << "\n";
	ost << "  time_geom :\n" << p23._time_geom << "\n";
	ost << "  ecriture :\n" << p23._ecriture << "\n";
	ost << "  projh :\n" << p23._projh;
	ost << "  proja :\n" << p23._proja;
	ost << "  orig :\n" << p23._orig;
	ost << "  omin :\n" << p23._omin;
	ost << "  omax :\n" << p23._omax;
	ost << "  boite :\n" << p23._boite;
	ost << "  boite projetee :\n" << p23._boite_proj;
	ost << "  lxyz :\n" << p23._lxyz;
	ost << "  lixyz :\n" << p23._lixyz;
	ost << "  masque :\n" << p23._masq << endl;
	ost << "  isupv :\n" << p23._isupv << endl;
	ost << "  c1,s1,c2,s2,c3,s3 :\n" << p23._c1 << " " << p23._s1 << " "
		<< p23._c2 << " " << p23._s2 << " " << p23._c3 << " " 
		<< p23._s3 << endl;
	ost << "  karv_titre,karv_echelle :\n" << p23._karv_titre << " " 
		<< p23._karv_echelle << endl;
	ost << "  boit,nosoc,nobas,noba,mail,inseg :\n"
		<< p23._boit << " " << p23._nosoc << " "
		<< p23._nobas << " " << p23._noba << " "
		<< p23._mail << " " << p23._inseg << endl;
	ost << "  ax,ay,az :\n" << p23._ax << " " << p23._ay << " " << p23._az
		<< endl;
	ost << "  ccx,ccy,ccz :\n" << p23._ccx << " " << p23._ccy << " " 
		<< p23._ccz << endl;
	ost << "  xl,yl,zl :\n" << p23._xl << " " << p23._yl << " " << p23._zl
		<< endl;
	ost << "  zcotes,re,rey :\n" << p23._zcotes << " " 
		<< p23._re << " " << p23._rey << endl;
	ost << "  fup,fnr,flr,fur,fl,fu :\n" << p23._fup << " " 
		<< p23._fnr << " " << p23._flr << " " << p23._fur << " " 
		<< p23._fl << " " << p23._fu << endl;
	ost << "  icob,icoh,icolor :\n" << p23._icob << " " << p23._icoh << " "
		<< p23._icolor << endl; 
	ost << "  init,ncotes :\n" << p23._init << " " << p23._ncotes << endl;
	ost << "  tronc :\n" << p23._tronc;
	ost << "  dl :\n" << p23._dl;
	ost << "  modetrace :\n" << p23._modetrace << endl;
	ost << endl;

	return ost;
}

int P4Graf0::P4ReOuvreL3()
{
	L3Tmgraf();
	L3ChoixDriver("negs");
	L3SautDePage();

	_ecriture=FALSE;

	return 0;
}

int P4Graf0::P4Ptvue(REEL site,REEL azimu,REEL dista,REEL xcen,
			REEL ycen,REEL scalxy,REEL redrv,REEL redrh)
{
	return P4Graf0::P4PtvueScxyGen( site, azimu, 0., dista, xcen,
			 ycen, scalxy, scalxy,redrv, redrh);
}

int P4Graf0::P4PtvueScxy(REEL site,REEL azimu,REEL dista,REEL xcen,
			REEL ycen,REEL scalx,REEL scaly,
			REEL redrv,REEL redrh)
{
	return P4Graf0::P4PtvueScxyGen( site, azimu, 0., dista, xcen,
			 ycen, scalx, scaly,redrv, redrh);
}

int P4Graf0::P4PtvueScxyGen(REEL site,REEL azimu,REEL roulis,REEL dista,
			REEL xcen,REEL ycen,REEL scalx,REEL scaly,
			REEL redrv,REEL redrh)
{
	_xcen = xcen;
	_ycen = ycen;
	_scalx = scalx;
	_scaly = scaly;

	const REEL pi   = M_PI;
	const REEL pim2 = 2.0*M_PI;
	const REEL pis2 = M_PI_2;
	const REEL pis  = M_PI/180.0;
	REEL xv[2],yv[2],zv[2],uu[8],vv[8] ;

	xv[0]=-.5;xv[1]=.5;
	yv[0]=-.5;yv[1]=.5;
	zv[0]=-.5;zv[1]=.5;

	int ier = -1;

	REEL a1 = mod(azimu,REEL(360.))*pis;if(a1<0.) a1 += pim2;
	REEL a2 = mod(redrh,REEL(360.))*pis;if(a2<0.) a2 += pim2;
	REEL b1 = mod( site,REEL(360.))*pis;if(b1<0.) b1 += pim2;
	REEL b2 = mod(redrv,REEL(360.))*pis;if(b2<0.) b2 += pim2;
	REEL c1 = mod(roulis,REEL(360.))*pis;if(c1<0.) c1 += pim2;
	REEL d = dista;

	if(d<0.)
	{
	    d=-d;a1=-a1;a2=-a2;b1=-b1;b2=-b2;
	}

	REEL a=a1-a2,b=b1-b2;
	REEL ca=cos(a),sa=sin(a);
	REEL cb=cos(b),sb=sin(b);
	REEL ca1=cos(a1),sa1=sin(a1);
	REEL cb1=cos(b1),sb1=sin(b1);
	REEL ca2=cos(a2);
	REEL cb2=cos(b2);
	REEL cc1=cos(c1),sc1=sin(c1);

	_isupv=1;if(sb1<0.) _isupv=-1;

	if(d==0.)
	{
	    _projh.Conic(FALSE);
	}
	else
	{
	    _projh.Conic(TRUE);

	    if(ca2==0.)
	    {
		cerr << " Dans P4Ptvue redressement des horizontales impossible"
		     << " avec cet angle : " << redrv << " on prend redrv = 0. "
		     << endl;
		a2=0.;
		a=a1-a2;b=b1-b2;
		ca=cos(a);sa=sin(a);
		cb=cos(b);sb=sin(b);
		ca1=cos(a1);sa1=sin(a1);
		cb1=cos(b1);sb1=sin(b1);
		ca2=cos(a2);
		cb2=cos(b2);
	    }

	    if(cb2==0.)
	    {
		cerr << " Dans P4Ptvue redressement des verticales impossible "
		     << "avec cet angle : " << redrv << " on prend redrv = 0. "
		     << endl;
		b2=0.;
		a=a1-a2;b=b1-b2;
		ca=cos(a);sa=sin(a);
		cb=cos(b);sb=sin(b);
		ca1=cos(a1);sa1=sin(a1);
		cb1=cos(b1);sb1=sin(b1);
		ca2=cos(a2);
		cb2=cos(b2);
	    }

	    if(d<.71)
	    {
		cerr << " Dans P4Ptvue abs(dista) < 0.71 : possibilite "
		     << "d'erreur de projection " << endl;
	    }

	}

	REEL dc1=-ca*cb,dc2=-sa*cb,dc3=-sb,dc0=1.;

	if(d!=0.)
	{
	    dc1=dc1/d;dc2=dc2/d;dc3=dc3/d;
	}

	REEL c11=-sa1/ca2;
	REEL c12= ca1/ca2;
	REEL c13= 0. /ca2;

	REEL c21=-ca1*sb1/cb2;
	REEL c22=-sa1*sb1/cb2;
	REEL c23=     cb1/cb2;

	REEL ccc=cc1*c11-sc1*c21;c21=sc1*c11+cc1*c21;c11=ccc;

	ccc=cc1*c12-sc1*c22;c22=sc1*c12+cc1*c22;c12=ccc;
	ccc=cc1*c13-sc1*c23;c23=sc1*c13+cc1*c23;c13=ccc;

	REEL c31=-ca1*cb1;
	REEL c32=-sa1*cb1;
	REEL c33=-sb1;

	REEL vh=-Bmax;
	REEL ud=-Bmax;
	REEL vb= Bmax;
	REEL ug= Bmax;
	REEL wb= Bmax;
	REEL wh=-Bmax;

	unsigned l=0;REEL u=0.;REEL v=0.;REEL w=0.;

	for(unsigned k=0;k<2;k++)
	{
	    for(unsigned j=0;j<2;j++)
	    {
		for(unsigned i=0;i<2;i++)
		{
		    d=dc0+dc1*xv[i]+dc2*yv[j]+dc3*zv[k];
		    REEL dc=1.;

		    if(_projh.Conic()) dc = P4Cvmgz(REEL(1.),d,d);

		    u=c11*xv[i]+c12*yv[j]+c13*zv[k];
		    v=c21*xv[i]+c22*yv[j]+c23*zv[k];
		    u=u/dc;v=v/dc;
		    w=c31*xv[i]+c32*yv[j]+c33*zv[k];

		    if(u<ug) ug=u;
		    if(u>ud) ud=u;
		    if(v<vb) vb=v;
		    if(v>vh) vh=v;
		    if(w<wb) wb=w;
		    if(w>wh) wh=w;

		    uu[l]=u;vv[l]=v;l++;
		}
	    }
	}

	if(ug>=ud || vb>=vh || wb>=wh)
	{
	    cerr << " Probleme dans P4Ptvue " << endl;return ier;
	}

	REEL cv=vh-vb;REEL cu=ud-ug;REEL cw=wh-wb;

	REEL scax=abs(scalx);if(scax==0.) scax=.95;

	if(scax<1.e-5)
	{
	    cerr << " dans P4Ptvue scax est < 1.e-5 : on prend 1. " << endl;
	    scax=1.;
	}

	REEL scay=abs(scaly);if(scay==0.) scay=.95;

	if(scay<1.e-5)
	{
	    cerr << " dans P4Ptvue scay est < 1.e-5 : on prend 1. " << endl;
	    scay=1.;
	}

	REEL cuu=scax/cu;REEL cvv=scay/cv;REEL cww=1./cw;

	ier=0;
	_boit=FALSE;

	_proja.P00(c11*cuu);_proja.P01(c12*cuu);_proja.P02(c13*cuu);
	_proja.P10(c21*cvv);_proja.P11(c22*cvv);_proja.P12(c23*cvv);
	_proja.P20(dc1);_proja.P21(dc2);_proja.P22(dc3);
	_proja.P30(c31*cww);_proja.P31(c32*cww);_proja.P32(c33*cww);
	_proja.Uor(xcen-.5*cuu*(ug+ud));_proja.Vor(ycen-.5*cvv*(vb+vh));
	_proja.Dor(dc0);
	_proja.Conic(_projh.Conic());

	REEL acx=pis2,acy=pis2,acz=pis2;
	REEL du =uu[1]-uu[0];if(du!=0.)  acx=atan((vv[1]-vv[0])/du) ;
	REEL dv =uu[2]-uu[0];if(dv!=0.)  acy=atan((vv[2]-vv[0])/dv) ;
	REEL dw =vv[4]-vv[0];if(dw!=0.)  acz=atan((uu[4]-uu[0])/dw) ;

	if(du<0.)  acx=acx+pi;
	if(dv<0.)  acy=acy+pi;
	if(dw<0.)  acz=acz+pi;

	_c1=cos(acx);_s1=sin(acx);
	_c2=cos(acy);_s2=sin(acy);
	_c3=cos(acz);_s3=sin(acz);

	P4Datas();

	return ier;
}

void P4Graf0::P4Datas()
{
	_flr=0.;_fl=0.;_fnr=1.;_fur=_fnr;

	if(_c3<0.) _c3=-_c3,_s3=-_s3;    // Pourquoi ???? (dans vvv)

	if(_s3<0.)      _fur=_fnr*_c3/(_c3-_s3);
	else if(_s3>0.) _flr=_fnr*_s3/(_c3+_s3);;

	_flr=0.;

	P4Dsp3(_flr,_fl,_fnr,_fu);
	_projh.P00(_proja.P00()*_fnr*_ccx);_projh.P01(_proja.P01()*_fnr*_ccy);
	_projh.P02(_proja.P02()*_fnr*_ccz);
	_projh.P10(_proja.P10()*_fup*_ccx);_projh.P11(_proja.P11()*_fup*_ccy);
	_projh.P12(_proja.P12()*_fup*_ccz);
	_projh.P20(_proja.P20()*_ccx);_projh.P21(_proja.P21()*_ccy);
	_projh.P22(_proja.P22()*_ccz);
	_projh.P30(_proja.P30()*_ccx);_projh.P31(_proja.P31()*_ccy);
	_projh.P32(_proja.P32()*_ccz);
	_projh.Uor(_proja.Uor()*_fnr+_flr);_projh.Vor(_proja.Vor()*_fup+_fl);
	_projh.Dor(_proja.Dor());

	if(_boit)
	{
	    _orig.X(_omin.X());
	    _orig.Y(_omin.Y());
	    _orig.Z(_omin.Z());
	}
	else
	{
	    _orig.X((_omin.X()+_omax.X())*.5);
	    _orig.Y((_omin.Y()+_omax.Y())*.5);
	    _orig.Z((_omin.Z()+_omax.Z())*.5);
	}

}

void P4Graf0::P4Dcoin(REEL decalx,REEL decaly,REEL decalz)
{
	REEL xxa=_omin.X(),yya=_omin.Y(),zza=_omin.Z();
	REEL xxr=_omax.X(),yyr=_omax.Y(),zzr=_omax.Z();

	REEL sss=xxa-(xxr-xxa)*decalx;xxr=xxr+(xxr-xxa)*decalx;xxa=sss;
	sss=yya-(yyr-yya)*decaly;yyr=yyr+(yyr-yya)*decaly;yya=sss;
	sss=zza-(zzr-zza)*decalz;zzr=zzr+(zzr-zza)*decalz;zza=sss;

	if(_lxyz.Bx())
	{
	    xxa=pow(10.,xxa);
	    xxr=pow(10.,xxr);
	}

	if(_lxyz.By())
	{
	    yya=pow(10.,yya);
	    yyr=pow(10.,yyr);
	}

	if(_lxyz.Bz())
	{
	    zza=pow(10.,zza);
	    zzr=pow(10.,zzr);
	}

	REEL coins[24];

	coins[0]=xxa;coins[1]=yya;coins[2]=zza;
	coins[3]=xxr;coins[4]=yya;coins[5]=zza;
	coins[6]=xxr;coins[7]=yyr;coins[8]=zza;
	coins[9]=xxa;coins[10]=yyr;coins[11]=zza;
	coins[12]=xxa;coins[13]=yyr;coins[14]=zzr;
	coins[15]=xxa;coins[16]=yya;coins[17]=zzr;
	coins[18]=xxr;coins[19]=yya;coins[20]=zzr;
	coins[21]=xxr;coins[22]=yyr;coins[23]=zzr;

	_boite.Boit(coins);

	Sommet xyz;
	Sommet uvda[8];

	for(unsigned l=0;l<8;l++)
	{
	    xyz(coins[3*l],coins[3*l+1],coins[3*l+2]);
	    uvda[l] = _projh.Projete(_lxyz,_orig,xyz);
	}

	_boite_proj.Boit(uvda);
	
	_init=-2;
}

void P4Graf0::P4Dcoin2(REEL x1,REEL y1t,REEL z1,REEL x2,REEL y2,REEL z2)
{
	REEL xxa=x1,yya=y1t,zza=z1;
	REEL xxr=x2,yyr=y2,zzr=z2;

	REEL coins[24];

	coins[0]=xxa;coins[1]=yya;coins[2]=zza;
	coins[3]=xxr;coins[4]=yya;coins[5]=zza;
	coins[6]=xxr;coins[7]=yyr;coins[8]=zza;
	coins[9]=xxa;coins[10]=yyr;coins[11]=zza;
	coins[12]=xxa;coins[13]=yyr;coins[14]=zzr;
	coins[15]=xxa;coins[16]=yya;coins[17]=zzr;
	coins[18]=xxr;coins[19]=yya;coins[20]=zzr;
	coins[21]=xxr;coins[22]=yyr;coins[23]=zzr;

	_boite.Boit(coins);

	Sommet xyz;
	Sommet uvda[8];

	for(unsigned l=0;l<8;l++)
	{
	    xyz(coins[3*l],coins[3*l+1],coins[3*l+2]);
	    uvda[l] = _projh.Projete(_lxyz,_orig,xyz);
	}

	_boite_proj.Boit(uvda);

	_init=-2;
}

void P4Graf0::P4Defpa(int ibas,int jcolor)
{

// ***********************************************************************
// *     - IBAS :                                                        *
// *    BAS_INCHANGE = -1 : inchange	                 	         *
// *    BAS_TRACE    =  1 : LA NAPPE "INFERIEURE" (VUE DU DESSOUS)       *
// *                   SERA TRACEE DANS LA COULEUR "JCOLOR"		 *
// *    BAS_NOTRACE  =  0 : NE SERA PAS					 *
// *                    A L'INITIALISATION, "TRUE"			 *
// *     - JCOLOR : COULEUR POUR LA SURFACE INFERIEURE EVENTUELLE	 *
// *                SI JCOLOR =0 OU >0                                   *
// *           SI JCOLOR<0 LA COULEUR EST LA MEME POUR LA FACE SUP. ET	 *
// *           LA FACE INF.						 *
// *     LA SURFACE SUPERIEURE EST TRACEE DANS LA COULEUR COURANTE AU 	 *
// *     MOMENT DE L'APPEL                                               *
// *      (donnee par P4dcolo), ET SI IL N'Y A PAS EU D'APPEL A "P4defpa"*
// *     DANS LA COULEUR "0"						 *
// *        LES TITRES SERONT TRACES DANS LA COULEUR COURANTE AU MOMENT	 *
// *        DE L'APPEL A "V3TITRES" , AINSI QUE LE SOCLE, SAUF POUR LES	 *
// *        TRAIT VUS EN TRANSPARENCE A TRAVERS LES FACES PROCHES DU	 *
// *        SOCLE, QUI SERONT TRACES DANS LA COULEUR DE LA FACE INF.	 *
// ***********************************************************************

	// L3RestaureGrille(0);  // s3ima1(0)

	if(ibas == BAS_TRACE) _nobas=FALSE;
	else if(ibas == BAS_NOTRACE) _nobas=TRUE;;

	_noba=_nobas;

	if(_init!=-2) _init=-1;

	_icoh=_icolor; 
	_icob=_icolor;

	if(jcolor>=0) _icob=jcolor;

}

void P4Graf0::P4Def2(int masque,int icotes,REEL zarg1)
{

	if(icotes<0)
	{
	    _ncotes=1;if(icotes==-2) _ncotes=0;
	    _zcotes=_omin.Z();if(zarg1!=Dep) _zcotes=zarg1;

	}
	else if(icotes==0)
	{
	    _ncotes=-1;
	    _zcotes=_omin.Z();if(zarg1!=Dep) _zcotes=zarg1;
	}

	switch(masque)
	{
	  case MASK_SIMPLE_MIN:
	    _masq=MASQUE_SIMPLE_MIN;
	    break;
	  case MASK_SIMPLE_MAX:
	    _masq=MASQUE_SIMPLE_MAX;
	    break;
	  case MASK_NOMASK:
	    _masq=MASQUE_ABSENT;
	    break;
	  case MASK_TRI:
	    _masq=MASQUE_TRI;
	    break;
	  case MASK_INCHANGE:
	    _masq = MASQUE_NONE;
	    break;
	}

}

void P4Graf0::P4Dtron(int jx,int jy,int jz)
{

    // initialisation troncature au bords de la boite

	P4Boolean notron=TRUE;
	P4Boolean fulltron=FALSE;

	if(jy==Dep && jz==Dep)
	{

	    if(jx==0) 
	    {
		notron=TRUE;
		fulltron=FALSE;
		_tronc.Tron(TRONC3D_NOTRONC,TRONC3D_NOTRONC,TRONC3D_NOTRONC,
			notron,fulltron);
	    }
	    else
	    {
		_tronc.Tron(TRONC3D_GAUDROI,TRONC3D_GAUDROI,TRONC3D_GAUDROI,
			notron,fulltron);
	    }

	}
	else
	{

	    if(jx<0) _tronc.Jx(TRONC3D_GAUCHE);
	    else     _tronc.Jx(jx&3);

	    if(jy<0) _tronc.Jy(TRONC3D_GAUCHE);
	    else     _tronc.Jy(jy&3);

	    if(jz<0) _tronc.Jz(TRONC3D_GAUCHE);
	    else     _tronc.Jz(jz&3);

	}

	_tronc.Notron(FALSE);
	_tronc.Fulltron(FALSE);

	if(_tronc.Jx()==TRONC3D_NOTRONC && _tronc.Jy()==TRONC3D_NOTRONC &&
		_tronc.Jz()==TRONC3D_NOTRONC) _tronc.Notron(TRUE);

	if(_tronc.Jx()==TRONC3D_GAUDROI && _tronc.Jy()==TRONC3D_GAUDROI &&
		_tronc.Jz()==TRONC3D_GAUDROI) _tronc.Fulltron(TRUE);

}

void P4Graf0::P4DefSpace(REEL x1,REEL y1t,REEL z1,REEL x2,REEL y2,REEL z2,
		int logx,int logy,int logz,P4Boolean sans_sauvegarde)
{

	P4Boolean logxx=_lxyz.Bx(),logyy=_lxyz.By(),logzz=_lxyz.Bz();

	if(logx != ECH3DINC) logxx=((P4Boolean) (logx==ECH3DLOG));

	if(logy != ECH3DINC) logyy=((P4Boolean) (logy==ECH3DLOG));

	if(logz != ECH3DINC) logzz=((P4Boolean) (logz==ECH3DLOG));

	if(abs(x1-x2) <= Bmin)
	{

	    if(logxx) x1=1.2*x1,x2=.8*x2;
	    else           x1=x1-1.,x2=x2+1.;

	}

	if(abs(y1t-y2) <= Bmin)
	{

	    if(logyy) y1t=1.2*y1t,y2=.8*y2;
	    else           y1t=y1t-1.,y2=y2+1.;

	}

	if(abs(z1-z2) <= Bmin)
	{

	    if(logzz) z1=1.2*z1,z2=.8*z2;
	    else           z1=z1-1. ,z2=z2+1.;

	}

	P4Vrbor(x1,x2,_dl.X2s1(),logxx,0);
	P4Vrbor(y1t,y2,_dl.Y2s1(),logyy,0);
	P4Vrbor(z1,z2,_dl.Z2s1(),logzz,0);

	if(!sans_sauvegarde)
	{

	   if(logx != ECH3DINC) _lxyz.Bx(logxx);

	   if(logy != ECH3DINC) _lxyz.By(logyy);

	   if(logz != ECH3DINC) _lxyz.Bz(logzz);

	    _ccx=1./(x2-x1);
	    _ccy=1./(y2-y1t);
	    _ccz=1./(z2-z1);

	    _omin.X(x1);_omin.Y(y1t);_omin.Z(z1);
	    _omax.X(x2);_omax.Y(y2);_omax.Z(z2);

	    P4Datas();
	}

}

void P4Graf0::P4Dnbdgr(REEL xnb,REEL ynb,REEL znb,REEL cnb,REEL hnb)
{

	if(xnb > 0.) _dl.X2s1(xnb);

	if(ynb > 0.) _dl.Y2s1(ynb);

	if(znb > 0.) _dl.Z2s1(znb);

	if(cnb > 0.) _dl.C2s1(cnb);

	if(hnb > 0.) _dl.H2s1(hnb);

}














