#ifndef P4GRILLE_H
#define P4GRILLE_H

// %W% %G% Pierre Brochard

#include "S6compat.h"
#include "P4constant.h"
#include "P4sommet.h"
#include "P4boite.h"
#include "P4bool3.h"
#include "minotaure/P4sanglier.h"

class GrilleElem
{
    protected :

        REEL      _decalx;
        P4Boolean _texte_horizontal;
        int       _grad_axex;
        int       _grad_axey;
        int       _taille_caract_titres;
        int       _taille_caract_echelles;
        int       _couleur;
        int       _couleur_titres;

    public :

        GrilleElem() ;
        GrilleElem(REEL decalx,P4Boolean texte_horizontal,int grad_axex,
            int grad_axey,int couleur,int couleur_titres,
            int taille_caract_titres,int taille_caract_echelles) ;
        GrilleElem(const GrilleElem& ge) ;
        virtual ~GrilleElem() {};

	void* operator new(size_t);
	void* operator new(size_t,void*);
#ifndef P4ANSI_DELETE
	void operator delete(void*,size_t);
#else
	void operator delete(void*);
	void operator delete(void*,void*);
#endif

        int operator == (const GrilleElem& ge) const;
        int operator != (const GrilleElem& ge) const;
        void operator = (const GrilleElem& ge) ;
        virtual void operator ++ () {};

        friend ostream& operator << (ostream& ost,GrilleElem ge) ;
};

class GrilleSimple : public GrilleElem
{
    private :

        Face               _face;
        char*              _titre_axe[4];
        char*              _unite_axe[4];
        int                _type_axe[4];
        P4Boolean          _label_axe[4];
        int                _grad_inter_axe[4];
        int                _factorisation_axe[4];

    public :

        GrilleSimple() ;
        GrilleSimple(Face face,REEL decalx,P4Boolean texte_horizontal,
            int grad_axex,int grad_axey,int couleur,int couleur_titres,
            int taille_caract_titres,int taille_caract_echelles,
            int* type_axe,P4Boolean* label_axe,
            int* grad_inter_axe, int* factorisation_axe,
            char** titre_axe,char** unite_axe) ;      
        GrilleSimple(const GrilleSimple& gs) ;
        ~GrilleSimple() ;

	void* operator new(size_t);
	void* operator new(size_t,void*);
#ifndef P4ANSI_DELETE
	void operator delete(void*,size_t);
#else
	void operator delete(void*);
	void operator delete(void*,void*);
#endif

        Face Fac() const {return _face;};
        REEL Decal() const {return _decalx;};
        void Trace(Sommet s1,Sommet s2,Sommet s3,Sommet s4,
            REEL xmin,REEL xmax,REEL ymin,REEL ymax,
            P4Boolean logx,P4Boolean logy) ;
        void DefTitre(int couleur_titres,
            const char* titre_axe_bas,const char* titre_axe_haut,
            const char* titre_axe_droite,const char* titre_axe_gauche,
            const char* unite_axe_bas,const char* unite_axe_haut,
            const char* unite_axe_droite,const char* unite_axe_gauche) ;
        void DefGrille(Face face,REEL decalx,P4Boolean texte_horizontal,
            int grad_axex,int grad_axey,int couleur,
            int taille_caract_titres,int taille_caract_echelles,
            int* type_axe,P4Boolean* label_axe,
            int* grad_inter_axe, int* factorisation_axe) ;

        int operator == (const GrilleSimple& gs) const;
        int operator != (const GrilleSimple& gs) const;
        void operator = (const GrilleSimple& gs) ;

        friend ostream& operator << (ostream& ost,GrilleSimple gs) ;
};

class GrilleAuto : public GrilleElem
{
    private :

        REEL      _decaly;
        REEL      _decalz;
        char*     _titres[3];
        char*     _unites[3];
        int       _type_grilles;
        int       _grad_inter[6];
        int       _grad_axez;

    public :

        GrilleAuto() ;
        GrilleAuto(REEL decalx,REEL decaly,REEL decalz,
            P4Boolean texte_horizontal,
            int grad_axex,int grad_axey,int grad_axez,
            int couleur,int couleur_titres,
            int taille_caract_titres,int taille_caract_echelles,
            int type_grilles,int* grad_inter,
            char** titres,char** unites) ;    
        GrilleAuto(const GrilleAuto& go) ;
        ~GrilleAuto() ;

	void* operator new(size_t);
	void* operator new(size_t,void*);
#ifndef P4ANSI_DELETE
	void operator delete(void*,size_t);
#else
	void operator delete(void*);
	void operator delete(void*,void*);
#endif

        Sommet Decal() const {Sommet s(_decalx,_decaly,_decalz);return s;};
        void Trace(Boite boite_proj,Boite boite,Bool4 lxyz) ;

        void DefTitre(int couleur_titres,
            const char* titre_x,const char* titre_y,const char* titre_z,
            const char* unite_x,const char* unite_y,const char* unite_z) ;

        void DefGrille(REEL decalx,REEL decaly,REEL decalz,
            P4Boolean texte_horizontal,int grad_axex,int grad_axey,
            int grad_axez,int couleur,int taille_caract_titres,
            int taille_caract_echelles,int type_grilles,
            int* grad_inter) ;

        int operator == (const GrilleAuto& go) const;
        int operator != (const GrilleAuto& go) const;
        void operator = (const GrilleAuto& go) ;

        friend ostream& operator << (ostream& ost,GrilleAuto go) ;
};

#endif



