#include "P4maillest8.h"

struct SccsIdP4MailleSt8
{
	static const char* const sccs_id;
};

const char* const SccsIdP4MailleSt8::sccs_id = "%W% %G% P. Brochard";

int
MailleSt8::DonneSommets(Tabunsignedbuf& t,unsigned numaille) const
{

	if(numaille<_nmailles)
	{
	   register const unsigned nx = _nx;
	   register const unsigned ny = _ny;
	   register const unsigned nz = (_nmailles+2*nx*ny-nx-ny)/
		(1+3*nx*ny-2*(nx+ny));
	   register const unsigned nm1 = (nx-1)*(ny-1)*nz;
	   register const unsigned nm2 = nm1 + (ny-1)*(nz-1)*nx;

	   if(numaille<nm1)
	   {
		register unsigned m1 = numaille%((nx-1)*(ny-1));
		register unsigned m2 = numaille/((nx-1)*(ny-1));
		register unsigned j = m1%(nx-1);
		register unsigned k = m1/(nx-1);
		unsigned nxprov[4];

		nxprov[0] = j + k*nx + m2*nx*ny;
		nxprov[1] = nxprov[0] + 1;
		nxprov[2] = nxprov[1] + nx;
		nxprov[3] = nxprov[2] - 1;

		t.ModTab(nxprov,4);

		return 0;
	    }
	    else if(numaille<nm2)
	    {
		register unsigned m1 = (numaille-nm1)%(nx*(ny-1));
		register unsigned m2 = (numaille-nm1)/(nx*(ny-1));
		register unsigned j = m1%nx;
		register unsigned k = m1/nx;
		unsigned nxprov[4];

		nxprov[0] = j + k*nx + m2*nx*ny;
		nxprov[1] = nxprov[0] + nx;
		nxprov[2] = nxprov[1] + nx*ny;
		nxprov[3] = nxprov[2] - nx;

		t.ModTab(nxprov,4);

		return 0;
	    }
	    else
	    {
		register unsigned m1 = (numaille-nm2)%((nx-1)*ny);
		register unsigned m2 = (numaille-nm2)/((nx-1)*ny);
		register unsigned j = m1%(nx-1);
		register unsigned k = m1/(nx-1);
		unsigned nxprov[4];

		nxprov[0] = j + k*nx + m2*nx*ny;
		nxprov[1] = nxprov[0] + 1;
		nxprov[2] = nxprov[1] + nx*ny;
		nxprov[3] = nxprov[2] - 1;

		t.ModTab(nxprov,4);

		return 0;
	    }

	}
	else
	{
	    cerr << "Erreur dans " << __FILE__ << " ligne " << __LINE__ << endl;
	    return -1;
	}

}

int
MailleSt8::LitTraceFil(Tabcharbuf& t,unsigned numaille) const
{

	if(numaille<_nmailles)
	{
	    unsigned char nxprov[4];

	    nxprov[0] = TRACE_FIL_OUI;
	    nxprov[1] = TRACE_FIL_OUI;
	    nxprov[2] = TRACE_FIL_OUI;
	    nxprov[3] = TRACE_FIL_OUI;
	    t.ModTab(nxprov,4);

	    return 0;
	}
	else
	{
	    cerr << "Erreur dans " << __FILE__ << " ligne " <<
		__LINE__ << endl;
	    return -1;
	}

}

int
MailleSt8::DonneMailles(Tabunsignedbuf& t,unsigned nuvolume) const
{
    // ---------------------------------------------------------------------
    // retourne 1 si pas de notion de volume pour le maillage
    // retourne 0 si le maillage contient la notion de volume
    // DonneMailles retourne alors dans t (1er parametre) les numeros de
    // mailles pour un volume dont on donne le numero (2eme parametre)
    // si erreur alors retourne -1
    // ---------------------------------------------------------------------

	register const unsigned nx = _nx - 1;
	register const unsigned ny = _ny - 1;
	register const unsigned nz = (_nmailles+2*nx*ny+nx+ny)/
		(3*nx*ny+nx+ny) - 1;
	register const unsigned nv = nx*ny*nz;

	if(nuvolume < nv)
	{
	    unsigned nxprov[6];

	    nxprov[0] = nuvolume;
	    nxprov[1] = nuvolume + nx*ny;
	    nxprov[2] = nxprov[1] + nx*ny*nz + nuvolume/nx;
	    nxprov[3] = nxprov[2] + 1;
	    nxprov[4] = nxprov[1] + (2*nx+1)*ny*nz + nx*(nuvolume/(nx*ny));
	    nxprov[5] = nxprov[4] + nx;

	    t.ModTab(nxprov,6);

	    return 0;
	}
	else
	{
	    cerr << "Erreur dans " << __FILE__ << " ligne " << __LINE__ << endl;
	    return -1;
	}

}

unsigned
MailleSt8::Nvolumes() const
{
    // ---------------------------------------------------------------------
    // retourne le nbre de volumes du maillage
    // ---------------------------------------------------------------------

	register const unsigned nx = _nx - 1;
	register const unsigned ny = _ny - 1;
	register const unsigned nz = (_nmailles+2*nx*ny+nx+ny)/
		(3*nx*ny+nx+ny) - 1;

	return nx*ny*nz;
}

int
MailleSt8::DonneMaillesSommet(Tabunsignedbuf& t,unsigned nusommet) 
{
	register const unsigned nx = _nx;
	register const unsigned ny = _ny;
	register const unsigned nmailles = _nmailles;
	register const unsigned nz = (nmailles+2*nx*ny-nx-ny)/
	    (1+3*nx*ny-2*(nx+ny));
	register const unsigned nsommets = nx*ny*nz;

	if(nusommet < nsommets)
	{
	    register const unsigned nx1 = nx-1;
	    register const unsigned ny1 = ny-1;
	    register const unsigned nz1 = nz-1;
	    register const unsigned nxy1 = nx1*ny1;
	    register const unsigned nm1 = nx1*ny1*nz;
	    register const unsigned nm2 = nm1 + ny1*nz1*nx;

	    register const unsigned m1 = nusommet%(nx*ny);
	    register const unsigned k = nusommet/(nx*ny);
	    register const unsigned i = m1%nx;
	    register const unsigned j = m1/ny;
	    register unsigned nb = 0;
	    unsigned nxprov[12];

	    if(!i)
	    {

		if(!j)
		{

		    if(!k)
		    {
			nxprov[0] = i + j*nx1 + k*nxy1;
			nxprov[1] = i + j*nx1 + k*nxy1 + nm1;
			nxprov[2] = i + j*nx1 + k*nxy1 + nm2;
			nb = 3;
		    }
		    else if(k == nz1)
		    {
			nxprov[0] = i + j*nx1 + k*nxy1;
			nxprov[1] = i + j*nx1 + (k-1)*nxy1 + nm1;
			nxprov[2] = i + j*nx1 + (k-1)*nxy1 + nm2;
			nb = 3;
		    }
		    else
		    {
			nxprov[0] = i + j*nx1 + k*nxy1;
			nxprov[1] = i + j*nx1 + (k-1)*nxy1 + nm1;
			nxprov[2] = nxprov[1] + nxy1;
			nxprov[3] = i + j*nx1 + (k-1)*nxy1 + nm2;
			nxprov[4] = nxprov[3] + nxy1;
			nb = 5;
		    }

		}
		else if(j == ny1)
		{

		    if(!k)
		    {
			nxprov[0] = i + (j-1)*nx1 + k*nxy1;
			nxprov[1] = i + (j-1)*nx1 + k*nxy1 + nm1;
			nxprov[2] = i + j*nx1 + k*nxy1 + nm2;
			nb = 3;
		    }
		    else if(k == nz1)
		    {
			nxprov[0] = i + (j-1)*nx1 + k*nxy1;
			nxprov[1] = i + (j-1)*nx1 + (k-1)*nxy1 + nm1;
			nxprov[2] = i + j*nx1 + (k-1)*nxy1 + nm2;
			nb = 3;
		    }
		    else
		    {
			nxprov[0] = i + (j-1)*nx1 + k*nxy1;
			nxprov[1] = i + (j-1)*nx1 + (k-1)*nxy1 + nm1;
			nxprov[2] = nxprov[1] + nxy1;
			nxprov[3] = i + j*nx1 + (k-1)*nxy1 + nm2;
			nxprov[4] = nxprov[3] + nxy1;
			nb = 3;
		    }

		}
		else if(!k)
		{
		    nxprov[0] = i + (j-1)*nx1 + k*nxy1;
		    nxprov[1] = nxprov[0] + nx1;
		    nxprov[2] = i + (j-1)*nx1 + k*nxy1 + nm1;
		    nxprov[3] = nxprov[2] + nx1;
		    nxprov[4] = i + j*nx1 + k*nxy1 + nm2;
		    nb = 5;
		}
		else if(k == nz1)
		{
		    nxprov[0] = i + (j-1)*nx1 + k*nxy1;
		    nxprov[1] = nxprov[0] + nx1;
		    nxprov[2] = i + (j-1)*nx1 + (k-1)*nxy1 + nm1;
		    nxprov[3] = nxprov[2] + nx1;
		    nxprov[4] = i + j*nx1 + (k-1)*nxy1 + nm2;
		    nb = 5;
		}
		else
		{
		    nxprov[0] = i + (j-1)*nx1 + k*nxy1;
		    nxprov[1] = nxprov[0] + nx1;
		    nxprov[2] = i + (j-1)*nx1 + (k-1)*nxy1 + nm1;
		    nxprov[3] = nxprov[2] + nx1;
		    nxprov[4] = nxprov[3] + nxy1;
		    nxprov[5] = nxprov[4] - nx1;
		    nxprov[6] = i + j*nx1 + (k-1)*nxy1 + nm2;
		    nxprov[7] = nxprov[6] + nxy1;
		    nb = 8;
		}

	    }
	    else if(i == nx1)
	    {

		if(!j)
		{

		    if(!k)
		    {
			nxprov[0] = i-1 + j*nx1 + k*nxy1;
			nxprov[1] = i + j*nx1 + k*nxy1 + nm1;
			nxprov[2] = i-1 + j*nx1 + k*nxy1 + nm2;
			nb = 3;
		    }
		    else if(k == nz1)
		    {
			nxprov[0] = i-1 + j*nx1 + k*nxy1;
			nxprov[1] = i + j*nx1 + (k-1)*nxy1 + nm1;
			nxprov[2] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
			nb = 3;
		    }
		    else
		    {
			nxprov[0] = i-1 + j*nx1 + k*nxy1;
			nxprov[1] = i + j*nx1 + (k-1)*nxy1 + nm1;
			nxprov[2] = nxprov[1] + nxy1;
			nxprov[3] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
			nxprov[4] = nxprov[3] + nxy1;
			nb = 5;
		    }

		}
		else if(j == ny1)
		{

		    if(!k)
		    {
			nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
			nxprov[1] = i + (j-1)*nx1 + k*nxy1 + nm1;
			nxprov[2] = i-1 + j*nx1 + k*nxy1 + nm2;
			nb = 3;
		    }
		    else if(k == nz1)
		    {
			nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
			nxprov[1] = i + (j-1)*nx1 + (k-1)*nxy1 + nm1;
			nxprov[2] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
			nb = 3;
		    }
		    else
		    {
			nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
			nxprov[1] = i + (j-1)*nx1 + (k-1)*nxy1 + nm1;
			nxprov[2] = nxprov[1] + nxy1;
			nxprov[3] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
			nxprov[4] = nxprov[3] + nxy1;
			nb = 5;
		    }

		}
		else if(!k)
		{
		    nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
		    nxprov[1] = nxprov[0] + nx1;
		    nxprov[2] = i + (j-1)*nx1 + k*nxy1 + nm1;
		    nxprov[3] = nxprov[2] + nx1;
		    nxprov[4] = i-1 + j*nx1 + k*nxy1 + nm2;
		    nb = 5;
		}
		else if(k == nz1)
		{
		    nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
		    nxprov[1] = nxprov[0] + nx1;
		    nxprov[2] = i + (j-1)*nx1 + (k-1)*nxy1 + nm1;
		    nxprov[3] = nxprov[2] + nx1;
		    nxprov[4] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
		    nb = 5;
		}
		else
		{
		    nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
		    nxprov[1] = nxprov[0] + nx1;
		    nxprov[2] = i + (j-1)*nx1 + (k-1)*nxy1 + nm1;
		    nxprov[3] = nxprov[2] + nx1;
		    nxprov[4] = nxprov[3] + nxy1;
		    nxprov[5] = nxprov[4] - nx1;
		    nxprov[6] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
		    nxprov[7] = nxprov[6] + nxy1;
		    nb = 8;
		}

	    }
	    else if(!j)
	    {

		if(!k)
		{
		    nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
		    nxprov[1] = nxprov[0] + 1;
		    nxprov[2] = i + j*nx1 + k*nxy1 + nm1;
		    nxprov[3] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
		    nxprov[4] = nxprov[3] + 1;
		    nb = 5;
		}
		else if(k == nz1)
		{
		    nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
		    nxprov[1] = nxprov[0] + 1;
		    nxprov[2] = i + j*nx1 + (k-1)*nxy1 + nm1;
		    nxprov[3] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
		    nxprov[4] = nxprov[3] + 1;
		    nb = 5;
		}
		else
		{
		    nxprov[0] = i-1 + j*nx1 + k*nxy1;
		    nxprov[1] = nxprov[0] + 1;
		    nxprov[2] = i + j*nx1 + (k-1)*nxy1 + nm1;
		    nxprov[3] = nxprov[2] + nxy1;
		    nxprov[4] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
		    nxprov[5] = nxprov[4] + 1;
		    nxprov[6] = nxprov[5] + nxy1;
		    nxprov[7] = nxprov[6] - 1;
		    nb = 8;
		}

	    }
	    else if(j == ny1)
	    {

		if(!k)
		{
		    nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
		    nxprov[1] = nxprov[0] + 1;
		    nxprov[2] = i + (j-1)*nx1 + k*nxy1 + nm1;
		    nxprov[3] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
		    nxprov[4] = nxprov[3] + 1;
		    nb = 5;
		}
		else if(k == nz1)
		{
		    nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
		    nxprov[1] = nxprov[0] + 1;
		    nxprov[2] = i + (j-1)*nx1 + (k-1)*nxy1 + nm1;
		    nxprov[3] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
		    nxprov[4] = nxprov[3] + 1;
		    nb = 5;
		}
		else
		{
		    nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
		    nxprov[1] = nxprov[0] + 1;
		    nxprov[2] = i + (j-1)*nx1 + (k-1)*nxy1 + nm1;
		    nxprov[3] = nxprov[5] + nxy1;
		    nxprov[4] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
		    nxprov[5] = nxprov[4] + 1;
		    nxprov[6] = nxprov[5] + nxy1;
		    nxprov[7] = nxprov[6] - 1;
		    nb = 8;
		}

	    }
	    else if(!k)
	    {
		nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
		nxprov[1] = nxprov[0] + 1;
		nxprov[2] = nxprov[1] + nx1;
		nxprov[3] = nxprov[2] - 1;
		nxprov[4] = i + (j-1)*nx1 + k*nxy1 + nm1;
		nxprov[5] = nxprov[4] + nx1;
		nxprov[6] = i-1 + j*nx1 + k*nxy1 + nm2;
		nxprov[7] = nxprov[6] + 1;
		nb = 8;
	    }
	    else if(k == nz1)
	    {
		nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
		nxprov[1] = nxprov[0] + 1;
		nxprov[2] = nxprov[1] + nx1;
		nxprov[3] = nxprov[2] - 1;
		nxprov[4] = i + (j-1)*nx1 + (k-1)*nxy1 + nm1;
		nxprov[5] = nxprov[4] + nx1;
		nxprov[6] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
		nxprov[7] = nxprov[6] + 1;
		nb = 8;
	    }
	    else
	    {
		nxprov[0] = i-1 + (j-1)*nx1 + k*nxy1;
		nxprov[1] = nxprov[0] + 1;
		nxprov[2] = nxprov[1] + nx1;
		nxprov[3] = nxprov[2] - 1;
		nxprov[4] = i + (j-1)*nx1 + (k-1)*nxy1 + nm1;
		nxprov[5] = nxprov[4] + nx1;
		nxprov[6] = nxprov[5] + nxy1;
		nxprov[7] = nxprov[6] - nx1;
		nxprov[8] = i-1 + j*nx1 + (k-1)*nxy1 + nm2;
		nxprov[9] = nxprov[8] + 1;
		nxprov[10] = nxprov[9] + nxy1;
		nxprov[11] = nxprov[10] - 1;
		nb = 12;
	    }

	    t.ModTab(nxprov,nb);

	    return 0;
	}
	else
	{
	    cerr << "Erreur dans " << __FILE__ << " ligne " << __LINE__ << endl;
	    return -1;
	}

}
