#ifndef S6BOUTGEN_H
#define S6BOUTGEN_H

// %W% %G% Pierre Brochard
// definitions des classes de bouttons generaux independants du toolkit

#include "poulpe/IHM.h"

class ConteneurColonne : public Conteneur
{
    public :

	ConteneurColonne(const Widget& parent, const char* classe,int col=1) :
		Conteneur(parent,classe) {ConteneurBase::Colonnes(col);};
	~ConteneurColonne() {};
};

class ConteneurLigne : public Conteneur
{
    public :

	ConteneurLigne(const Widget& parent, const char* classe,int ligne=1) :
		Conteneur(parent,classe) {ConteneurBase::Lignes(ligne);};
	~ConteneurLigne() {};
};

class ConteneurExclusifColonne : public ConteneurExclusif
{
    public :

	ConteneurExclusifColonne(const Widget& parent, const char* classe,
		int col=1) : ConteneurExclusif(parent,classe)
		{ConteneurBase::Colonnes(col);};
	~ConteneurExclusifColonne() {};
};

class ConteneurExclusifLigne : public ConteneurExclusif
{
    public :

	ConteneurExclusifLigne(const Widget& parent, const char* classe,
		int col=1) : ConteneurExclusif(parent,classe)
		{ConteneurBase::Lignes(col);};
	~ConteneurExclusifLigne() {};
};

// ********************************************************
// choix verticaux ........................................
// ********************************************************

class ChoixVertical;

class ChoixVerticalPDE : public PullDownEntry
{
    private:

	ChoixVertical* _pto;
	int _choix;

    public:

	ChoixVerticalPDE() : PullDownEntry() {_pto=NULL;_choix=0;};
	ChoixVerticalPDE(const Widget& parent, const char *label,
		int choix=0,const char *classe = "CVPDE",
		ChoixVertical* pto = NULL) :
		PullDownEntry(parent,label,classe) {_pto=pto;_choix=choix;};
	~ChoixVerticalPDE() {};

	void Init(const Widget& parent, const char* label,
			int choix=0,const char *classe = "CVPDE",
			ChoixVertical* pto = NULL) 
		{
			PullDownEntry::Init(parent,label,classe);
			_pto=pto;_choix=choix;
		}
	void SelectCB(XtPointer call_data);
	void Attach(ChoixVertical* pto) {_pto=pto;};
	char* Lab() const {return _label;};
};

class ChoixVertical : public ConteneurColonne
{
    protected:

	PullDownTop		_val;
	Label			_lab;
	ChoixVerticalPDE*	_bou;
	unsigned		_taille;
	int			_choix;

    public:

	ChoixVertical(const Widget& parent, const char* name,
		const char* label,const char** const val,
		int taille,int nbcol = 1,int col = 1);
	~ChoixVertical();

	void operator()(const char** val,int taille,int nbcol,int col);
	void operator()(char** val,int taille,int nbcol,int col)
		{
			(*this)((const char**) val,taille,nbcol,col);
		}

	void Choix(int choix) {_choix = choix;};
	int Choix() const {return _choix;};

	void ChangeLabel(const char* label) {_lab.Change(label);};
	void Couleur(Pixel pix[],unsigned nb_pix);
	void Couleur();
	void Couleur(int taille);

	virtual void ChargeInv() {};
	virtual void Change(int choix);
};

// *************************************************************
//     choix verticaux ou le meme panneau peut etre commun
//     a plusieurs cascade boutons
// *************************************************************

// boutons pour choix verticaux panel

class ChoixVerticalCascade;

class ChoixVerticalPanelPDE : public PullDownEntry
{
    private:

	ChoixVerticalCascade* _pto;
	int _choix;

    public:

	ChoixVerticalPanelPDE() : PullDownEntry() {_pto=NULL;_choix=0;};
	ChoixVerticalPanelPDE(const Widget& parent, const char *label,
		int choix=0,const char *classe = "CVPDE",
		ChoixVerticalCascade* pto = NULL) :
		PullDownEntry(parent,label,classe) {_pto=pto;_choix=choix;};
	~ChoixVerticalPanelPDE() {};

	void Init(const Widget& parent, const char* label,
		int choix=0,const char *classe = "CVPDE",
		ChoixVerticalCascade* pto = NULL) 
		{PullDownEntry::Init(parent,label,classe);
		_pto=pto;_choix=choix;};
	void SelectCB(XtPointer call_data);
	void Attach(ChoixVerticalCascade* pto) {_pto=pto;};
	char* Lab() const {return _label;};
};

// panel pour ChoixVerticalCascade

class ChoixVerticalPanel : public PullDownTopPanel
{
    protected:

	ChoixVerticalPanelPDE*	_bou;
	unsigned		_taille;
  int _iscolor;

    public:

	ChoixVerticalPanel(const Widget& parent, 
		const char** const val,
		int taille,int nbcol = 1);
	~ChoixVerticalPanel();

	void Attach(ChoixVerticalCascade* pto);

	void operator()(const char** val,int taille,int nbcol);
	void operator()(char** val,int taille,int nbcol)
		{
			(*this)((const char**) val,taille,nbcol);
		}

	char* Label(int choix);

	void Couleur(Pixel pix[],unsigned nb_pix);
	void Couleur();
	void Couleur(int taille);
};

// choix verticaux avec panel venant de l'exterieur....................

class ChoixVerticalPullDownTopCascade : public PullDownTopCascade
{
    public:

	ChoixVerticalPullDownTopCascade(const Widget& parent,const Widget& menu,
		const char* label,const char* classe="MENTP") :
		PullDownTopCascade(parent,menu,label,classe) {_pto = NULL;};
	~ChoixVerticalPullDownTopCascade() {};

	void Attach(ChoixVerticalCascade* pto) {_pto = pto;};

	void SelectCB(XtPointer call_data);

    private :

	ChoixVerticalCascade* _pto;
};

class ChoixVerticalCascade : public ConteneurColonne
{
    protected:

	ChoixVerticalPullDownTopCascade	_pdtc;
	Label				_lab;
	ChoixVerticalPanel&		_panel;
	PopUp* const 		_popupbloque;
	int				_choix;

    public:

	ChoixVerticalCascade(const Widget& parent,
		ChoixVerticalPanel& panel,
		const char* const name,const char* const label,
		const int col = 1,PopUp* const popupbloque = NULL);
	~ChoixVerticalCascade();

	void Attach();

	void Choix(int choix) {_choix = choix;};
	int Choix() const {return _choix;};

	void ChangeLabel(const char* const label);

	virtual void ChargeInv() {};
	virtual void Change();
	virtual void Change(int choix);
};

#endif
