// ------------------------------------------------------------------
// ******************************************************************
//		module gestion des serveurs sockets
// ******************************************************************
// ------------------------------------------------------------------

struct SccsIdS6GereFich
{
	static const char* const sccs_id;
};

const char* const SccsIdS6GereFich::sccs_id = "%W% %G% Pierre Brochard";

// includes

#include "poulpe/IHM.h"
#include "S6constant.h"
#include "nxstream.h"
#include "S6gerefich.h"
#include "S6points.h"
#include "S6pointsmtv.h"
#include "S6initfich.h"
#include "S6gereenv.h"
#include "nx.h"
#include "S6namecomp.h"
#include "S6flagssup.h"
#include "S6trace.h"
#include "S6gereconfig.h"
#include "S6gerefenet.h"
#include "S6initcourbe.h"
#include "S6boutfich.h"
#include "poulpe/L3Manip.h"
#include "S6outils.h"
#include "COL.h"
#include "minotaure/P4graf3.h"
#include "dinosaure/S6timer.h"
#include "dinosaure/S6string.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

// description des serveurs sockets 

nxstream* S6GereFichier::serveur_binaire = NULL;
nxstream* S6GereFichier::serveur_ascii = NULL;
nxstream* S6GereFichier::serveur_ucd = NULL;
nxstream* S6GereFichier::serveur_unix = NULL;
S6Points* S6GereFichier::pt_points = NULL;
char* S6GereFichier::fichier_points = NULL;
int S6GereFichier::nombre_serveurs = 0;
int S6GereFichier::num_psyche = 0;
SocketBinaire S6GereFichier::socket_binaire;
SocketAscii S6GereFichier::socket_ascii;
SocketUcd S6GereFichier::socket_ucd;
SocketCommunic S6GereFichier::socket_communic;
SocketL3 S6GereFichier::socket_l3;
SocketCol S6GereFichier::socket_col;

// *******************************************************************
//      ajout de drivers (voir IHM : XtAddInputDriver)
// *******************************************************************

void
SocketBinaire::LectureInputCB(int fid,U8InputId input_id)
{
	TimerDetInit();

    // gestion des fichiers sockets

	nxstream* const fdti = GetNxstream(fid);

	if(fdti)
	{
	    S6GereFichier::LitFichierSocketBinaire(fdti,input_id);
	}

	TimerDetReport(1,1,"Temps calcul total SocketBinaire");
	cout << endl;
}

void
SocketAscii::LectureInputCB(int fid,U8InputId input_id)
{

    // gestion des fichiers sockets

	nxstream* const fdti = GetNxstream(fid);

	if(fdti)
	{
	    S6GereFichier::LitFichierSocketAscii(fdti,input_id);
	}

}

void
SocketUcd::LectureInputCB(int fid,U8InputId input_id)
{

    // gestion des fichiers sockets

	nxstream* const fdti = GetNxstream(fid);

	if(fdti)
	{
	    S6GereFichier::LitFichierSocketUcd(fdti,input_id);
	}

}

void
SocketCommunic::LectureInputCB(int fid,U8InputId input_id)
{

    // gestion des fichiers sockets

	nxstream* fdti = GetNxstream(fid);

	if(fdti)
	{
	    S6GereFichier::LitFichierSocketCommunic(fdti,input_id);
	}

}

void
SocketL3::LectureInputCB(int fid,U8InputId input_id)
{

    // gestion des fichiers sockets

	nxstream* fdti = GetNxstream(fid);

	if(fdti)
	{
	    S6GereFichier::LitFichierSocketL3(fdti,input_id);
	}

}

void
SocketCol::LectureInputCB(int fid,U8InputId input_id)
{

    // gestion des fichiers sockets

	nxstream* fdti = GetNxstream(fid);

	if(fdti)
	{
	    S6GereFichier::LitFichierSocketCol(fdti,input_id);
	}

}

// *******************************************
//     Sauvegarde de S6Points
// *******************************************

void
S6GereFichier::SauvePoints()
{

	if(pt_points) pt_points->Sauve();

}

// *******************************************
//    lecture de fichiers sockets 
// *******************************************

int
S6GereFichier::LitFichierSocket(int fid)
{
	TimerDetInit();

	int ier = 0;

	cout << "Lecture socket " << nombre_serveurs << ' ' << fid << '\n';

	S6FlagsSup::lit_config = FALSE;

	if(nombre_serveurs >= 3)
	{

	    if(fid == int(fileno(stdin)))
	    {
		ier = LitInput(fid);
	    }
	    else if(serveur_binaire && fid == serveur_binaire->numfile())
	    {
		ier = LitFichierSocketBinaire();
	    }
	    else if(serveur_ascii && fid == serveur_ascii->numfile())
	    {
		ier = LitFichierSocketAscii();
	    }
	    else if(serveur_ucd && fid == serveur_ucd->numfile())
	    {
		ier = LitFichierSocketUcd();
	    }
	    else if(serveur_unix && fid == serveur_unix->numfile())
	    {
		cerr << "S6GereFichier> mode non implemete" << endl;
	    }

	}
	else
	{
	    nxstream* fichier_socket = NULL;
	    char handler_name[10];

	    memset(handler_name,0,sizeof(handler_name));

	    if(fid == int(fileno(stdin)))
	    {
		ier = LitInput(fid);
	    }
	    else
	    {
		nxstream* const fdti = ElementInput::GetNxstream(fid);
	
		if(fdti)
		{
		    fichier_socket = new nxstream(*fdti,handler_name);

		    if(fdti == serveur_binaire)
		    {
			cout << "S6GereFichier> handler_name " << handler_name <<
			    " sockets INET" << endl;
		    }
		    else if(fdti == serveur_unix)
		    {
			cout << "S6GereFichier> handler_name " << handler_name <<
			    " sockets UNIX" << endl;
		    }
		    else
		    {
			cout << "S6GereFichier> handler_name " << handler_name <<
			    endl;
		    }

		}

	    }

	    if(!strcmp(handler_name,"spx"))
	    {
		ier = LitFichierSocketBinaire(fichier_socket);
	    }
	    else if(!strcmp(handler_name,"sp3"))
	    {
		ier = LitFichierSocketAscii(fichier_socket);
	    }
	    else if(!strcmp(handler_name,"ucd"))
	    {
		ier = LitFichierSocketUcd(fichier_socket);
	    }
	    else if(!strcmp(handler_name,"spc"))
	    {
		ier = LitFichierSocketCommunic(fichier_socket);
	    }
	    else if(!strcmp(handler_name,"l3"))
	    {
		ier = LitFichierSocketL3(fichier_socket);
	    }
	    else if(!strcmp(handler_name,"col"))
	    {
		ier = LitFichierSocketCol(fichier_socket);
	    }
	    else if(!strcmp(handler_name,"test"))
	    {
		delete fichier_socket;
	    }
	    else
	    {
		delete fichier_socket;

		cerr << "S6GereFichier> Mauvaise adresse socket " <<
		    handler_name << endl;
	    }

	}

	TimerDetReport(1,1,"Temps calcul total LitFichierSocket");

	return ier;
}

// *******************************************
//    lecture de fichiers non sockets 
// *******************************************

int
S6GereFichier::LitFichier()
{
	int ier = 0;

	S6FlagsSup::lit_config = FALSE;

	if(!strcmp(S6NameComp::fichier_points,"input"))
	{
	    delete[] fichier_points;

	    const char* fp1 = S6NameComp::fichier_provi_1;

	    if(fp1)
	    {
		const size_t taille = strlen(fp1)+1;

		fichier_points = new char[taille];

		memcpy(fichier_points,fp1,taille);
	    }

	    ier = LitFichierAscii();
	}
	else if(!strcmp(S6NameComp::fichier_points,"interne"))
	{
	    ier = S6InitFich::InitImageDur();
	    pt_points->Typefichier(S6PHYSIQUE_4D);
	    pt_points->Init(FALSE,2);
	}
	else
	{
	    const size_t l_f_points = strlen(S6NameComp::fichier_points);
	    const char* const suffixe_fin =
		S6NameComp::fichier_points + l_f_points;

	// 'rc' :
	    const char* const suffixe_rc = suffixe_fin - 2;
	// 'sp3' ou 'spx' ou 'inp' ou 'mtv' :
	// inp = format AVS
        // mtv = format simple de RAYSHADE
	    const char* const suffixe_court = suffixe_fin - 3;
	// 'sp3.Z' ... :
	    const char* const suffixe_long = suffixe_fin - 5;
	// '.Z' :
	    const char* const suffixe_compress = suffixe_fin - 2;
	// 'sp3.gz' ... :
	    const char* const suffixe_long_gzip = suffixe_fin - 6;
	// '.gz' :
	    const char* const suffixe_gzip = suffixe_fin - 3;
	// 'l3' :
	    const char* const suffixe_l3 = suffixe_fin - 2;
	// 'l3.Z' ou 'l3.*.Z' :
	    const char* const suffixe_long_l3 = suffixe_fin - 4;
	// 'l3.gz' ou'l3.*.gz' :
	    const char* const suffixe_long_gzip_l3 = suffixe_fin - 5;
	// '.f.col', '.c.col' et '.p.col' :
	    const char* const suffixe_col = suffixe_fin - 6;
	// 'l3.*' :
	    char* prefixe_l3 = strrchr(S6NameComp::fichier_points,'/');

	    prefixe_l3 = prefixe_l3 ? prefixe_l3 + 1 :
		S6NameComp::fichier_points;

	    delete[] fichier_points;

	    const unsigned lfich = l_f_points + 1;

	    fichier_points = new char[lfich];

	    memcpy(fichier_points,S6NameComp::fichier_points,lfich);

	    if(!strcmp(suffixe_compress,".Z") &&
			(!strncmp(prefixe_l3,"l3.",3) ||
			!strcmp(suffixe_long_l3,"l3.Z")))
	    {
		delete[] fichier_points;

	        {  
		    ostrstream os;

		    static int numero = -1;numero++;

		    os << tmpdir << '/' << naruser << "/l3.Z" << numero <<
				".XXXXXX" << ends;
		    fichier_points = mktemp(os.str());
		}

		const char* fp1 = S6NameComp::fichier_points;
		const char* fp2 = fichier_points;

		if(fp1 && fp2)
		{			
		    const size_t taille1 = strlen(fp1);
		    const size_t taille2 = strlen(fp2);

		    char* ptos = new char[taille1+taille2+9];

		    memcpy(ptos,"zcat ",5);
		    memcpy(ptos+5,fp1,taille1);
		    memcpy(ptos+5+taille1," > ",3);
		    memcpy(ptos+8+taille1,fp2,taille2+1);

		    ier = system(ptos);

		    delete[] ptos;

		    cout << "S6GereFichier> lecture fichier L3 graphique" <<
			endl;

		    LitDisplayList(fichier_points);
		    unlink(fichier_points);
		}

	    }
	    else if(!strcmp(suffixe_gzip,".gz") &&
			(!strncmp(prefixe_l3,"l3.",3) ||
			!strcmp(suffixe_long_gzip_l3,"l3.gz")))
	    {
		delete[] fichier_points;

	        {  
		    ostrstream os;

		    static int numero = -1;numero++;

		    os << tmpdir << '/' << naruser << "/l3.gz" << numero <<
				".XXXXXX" << ends;
		    fichier_points = mktemp(os.str());
		}
		
		S6String ss("gzip -cd ",S6NameComp::fichier_points,
				    " > ",fichier_points);
		char* ptos = ss;

		ier = system(ptos);
		delete[] ptos;

		cout << " lecture fichier L3 graphique " << endl;

		LitDisplayList(fichier_points);
		unlink(fichier_points);
	    }
	    else if(!strncmp(prefixe_l3,"l3.",3) ||
			!strcmp(suffixe_l3,"l3"))
	    {
		cout << "S6GereFichier> lecture fichier L3 graphique " << endl;

		LitDisplayList(S6NameComp::fichier_points);
	    }
	    else if(!strcmp(suffixe_compress,".Z") &&
			(!strcmp(suffixe_long,"sp3.Z") ||
			!strcmp(suffixe_long,"spx.Z") ||
			!strcmp(suffixe_long,"inp.Z")))
	    {
		delete[] fichier_points;

	        {  
		    ostrstream os;

		    if(strcmp(suffixe_long,"sp3.Z") == 0)
		    {
	                static int numero = -1;numero++;

		        os << tmpdir << '/' << naruser << "/sp3.Z" << numero <<
				".XXXXXX" << ends;
	            }
		    else if(strcmp(suffixe_long,"spx.Z") == 0)
	            {
	                static int numero = -1;numero++;

		        os << tmpdir << '/' << naruser << "/spx.Z" << numero <<
				".XXXXXX" << ends;
		    }
		    else if(strcmp(suffixe_long,"inp.Z") == 0)
		    {
	                static int numero = -1;numero++;

		        os << tmpdir << '/' << naruser << "/inp.Z" << numero <<
				".XXXXXX" << ends;
		    }

		    fichier_points = mktemp(os.str());
		}

		{
		    S6String ss("zcat ",S6NameComp::fichier_points," > ",
				    fichier_points);
		    char* ptos = ss;

		    ier = system(ptos);
		    delete[] ptos;
		}

	        {
		    delete[] S6NameComp::fichier_config_attach;

		    const size_t lfca = strlen(S6NameComp::fichier_points) + 1;

		    S6NameComp::fichier_config_attach = new char[lfca];
		    memcpy(S6NameComp::fichier_config_attach,
				S6NameComp::fichier_points,lfca-3);
		    memcpy(S6NameComp::fichier_config_attach+lfca-3,"rc",3);

		    int my_fd = open(S6NameComp::fichier_config_attach,
				O_RDONLY);

		    if(my_fd < 0)
		    {
			errno = 0;

			delete[] S6NameComp::fichier_config;

			const size_t lll =
			    strlen(S6NameComp::fichier_config_home) + 1;

			S6NameComp::fichier_config = new char[lll];
			memcpy(S6NameComp::fichier_config,
				S6NameComp::fichier_config_home,lll);
		    }
		    else
		    {
			close(my_fd);

			delete[] S6NameComp::fichier_config;

			const size_t lll =
			    strlen(S6NameComp::fichier_config_attach) + 1;

			S6NameComp::fichier_config = new char[lll];
			memcpy(S6NameComp::fichier_config,
				S6NameComp::fichier_config_attach,lll);
			S6FlagsSup::lit_config = TRUE;
		    }

		}
		
		if(!strcmp(suffixe_long,"sp3.Z"))
		{
		    ier = LitFichierAscii();
	        }
		else if(!strcmp(suffixe_long,"spx.Z"))
	        {
		    ier = LitFichierBinaire();
		}
		else if(!strcmp(suffixe_long,"inp.Z"))
		{
		    ier = LitFichierUcd();
		}

		unlink(fichier_points);	
	    }
	    else if(!strcmp(suffixe_gzip,".gz") &&
			(!strcmp(suffixe_long_gzip,"sp3.gz") ||
			!strcmp(suffixe_long_gzip,"spx.gz") ||
			!strcmp(suffixe_long_gzip,"inp.gz")))
	    {
		delete[] fichier_points;

	        {  
		    ostrstream os;

		    if(strcmp(suffixe_long_gzip,"sp3.gz") == 0)
		    {
	                static int numero = -1;numero++;

		        os << tmpdir << '/' << naruser << "/sp3.gz" <<
			  numero << ".XXXXXX" << ends;
	            }
		    else if(strcmp(suffixe_long_gzip,"spx.gz") == 0)
	            {
	                static int numero = -1;numero++;

		        os << tmpdir << '/' << naruser << "/spx.gz" <<
			  numero << ".XXXXXX" << ends;
		    }
		    else if(strcmp(suffixe_long_gzip,"inp.gz") == 0)
		    {
	                static int numero = -1;numero++;

		        os << tmpdir << '/' << naruser << "/inp.gz" <<
			  numero << ".XXXXXX" << ends;
		    }

		    fichier_points = mktemp(os.str());
		}

		{
		    S6String ss("gzip -cd ",S6NameComp::fichier_points,
				    " > ",fichier_points);
		    char* ptos = ss;

		    ier = system(ptos);
		    delete[] ptos;
		}

	        {
		    delete[] S6NameComp::fichier_config_attach;

		    const size_t lfca = strlen(S6NameComp::fichier_points);

		    S6NameComp::fichier_config_attach = new char[lfca];
		    memcpy(S6NameComp::fichier_config_attach,
				S6NameComp::fichier_points,lfca-3);
		    memcpy(S6NameComp::fichier_config_attach+lfca-3,"rc",3);

		    int my_fd = open(S6NameComp::fichier_config_attach,
				O_RDONLY);

		    if(my_fd < 0)
		    {
			delete[] S6NameComp::fichier_config;

			const size_t lll =
			    strlen(S6NameComp::fichier_config_home) + 1;

			S6NameComp::fichier_config = new char[lll];
			memcpy(S6NameComp::fichier_config,
				S6NameComp::fichier_config_home,lll);
		    }
		    else
		    {
			close(my_fd);

			delete[] S6NameComp::fichier_config;

			const size_t lll =
			    strlen(S6NameComp::fichier_config_attach) + 1;

			S6NameComp::fichier_config = new char[lll];
			memcpy(S6NameComp::fichier_config,
				S6NameComp::fichier_config_attach,lll);
			S6FlagsSup::lit_config = TRUE;
		    }

		}
		
		if(!strcmp(suffixe_long_gzip,"sp3.gz"))
		{
		    ier = LitFichierAscii();
	        }
		else if(!strcmp(suffixe_long_gzip,"spx.gz"))
	        {
		    ier = LitFichierBinaire();
		}
		else if(!strcmp(suffixe_long_gzip,"inp.gz"))
		{
		    ier = LitFichierUcd();
		}

		unlink(fichier_points);
	    }
	    else if(!strcmp(suffixe_court,"sp3") ||
			!strcmp(suffixe_court,"spx") ||
			!strcmp(suffixe_court,"inp") ||
			!strcmp(suffixe_court,"mtv"))
	    {

	        {
		    delete[] S6NameComp::fichier_config_attach;

		    const size_t lfca = strlen(S6NameComp::fichier_points) + 3;

		    S6NameComp::fichier_config_attach = new char[lfca];
		    memcpy(S6NameComp::fichier_config_attach,
				S6NameComp::fichier_points,lfca-3);
		    memcpy(S6NameComp::fichier_config_attach+lfca-3,"rc",3);

		    int my_fd = open(S6NameComp::fichier_config_attach,
				O_RDONLY);

		    if(my_fd < 0)
		    {
			delete[] S6NameComp::fichier_config;

			const size_t lll =
			    strlen(S6NameComp::fichier_config_home) + 1;

			S6NameComp::fichier_config = new char[lll];
			memcpy(S6NameComp::fichier_config,
				S6NameComp::fichier_config_home,lll);
		    }
		    else
		    {
			close(my_fd);

			delete[] S6NameComp::fichier_config;

			const size_t lll =
			    strlen(S6NameComp::fichier_config_attach) + 1;

			S6NameComp::fichier_config = new char[lll];
			memcpy(S6NameComp::fichier_config,
				S6NameComp::fichier_config_attach,lll);
			S6FlagsSup::lit_config = TRUE;
		    }

		}
		
		if(!strcmp(suffixe_court,"sp3"))
		{
		    ier = LitFichierAscii();
	        }
		else if(!strcmp(suffixe_court,"spx"))
	        {
		    ier = LitFichierBinaire();
		}
		else if(!strcmp(suffixe_court,"inp"))
		{
		    ier = LitFichierUcd();
		}
		else if(!strcmp(suffixe_court,"mtv"))
		{
		    ier = LitFichierMtv();
		}
		
	    }
	    else if(!strcmp(suffixe_rc,"rc"))
	    {
		delete[] S6NameComp::fichier_config;

		const size_t lfic = strlen(S6NameComp::fichier_points)+1;

		S6NameComp::fichier_config = new char[lfic];

		memcpy(S6NameComp::fichier_config,S6NameComp::fichier_points,
				lfic);
		ier = S6GereConfig::Charge(S6NameComp::fichier_config);

		if(!pt_points)
		{
		    pt_points = new S6Points;
		}

		S6InitCourbe::Init();

		if(calcul_ihm) S6GereFenet::ReConfig();

		pt_points->Typefichier(S6CONFIG_SEULE);
	    }
	    else if(!strcmp(suffixe_col,".f.col"))
	    {

	    // lecture fichier colormap pour le fond et l'antifond

		ier = LitFichierFond();
	    }
	    else if(!strcmp(suffixe_col,".c.col"))
	    {

	    // lecture fichier colormap pour les (10) couleurs de base

		ier = LitFichierCol();
	    }
	    else if(!strcmp(suffixe_col,".p.col"))
	    {

	    // lecture fichier colormap pour les couleurs supplementaires
	    // pour echelle de couleurs

		ier = LitFichierPal();
	    }
	    else
	    {
		cerr << "S6GereFichier> fichier non reconnu : "
		    "pas de lecture " << endl;

		if(!pt_points)
		{
		    pt_points = new S6Points;
		}

	    }

	}

	return ier;
}

// **********************************************
//      test input
// **********************************************

int
S6GereFichier::LitInput(int fid)
{
	char c[2];

	c[0] = '\0';

	while (c[0] != '\n')
	{
	    read(fid, c, 1);
	    cout << c[0] << " ";
	}

	cout << endl;

	return 0;
}

// **********************************************
//      lecture fichier socket binaire
// **********************************************

int
S6GereFichier::LitFichierSocketBinaire(nxstream* fichier_socket_binaire,
		U8InputId input_id)
{
	int ier = 0;

	if(!fichier_socket_binaire)
	{
	    fichier_socket_binaire = new nxstream(*serveur_binaire);

	    cout << "S6GereFichier> accept" << endl;
	}

	cout << "S6GereFichier> input_id " << input_id << endl;

	if(input_id != -1)
	{
	    socket_binaire.RemoveInputDriver(input_id);

	    cout << "S6GereFichier> nouvelle lecture sur meme socket binaire" <<
		endl;
	}

	if(!fichier_socket_binaire || !fichier_socket_binaire->rdbuf())
	{
	    cerr << "S6GereFichier> Pb ouverture canal de lecture binaire :"
			" On ne lit pas " << endl;
	    return -1;
	}

	cout << "S6GereFichier> fichier_socket_binaire " <<
			(void*) fichier_socket_binaire << endl;

	if(!pt_points)
	{
	    pt_points = new S6Points;
	}
	else
	{
	    S6Points* pt_p = new S6Points(*pt_points);

	    delete pt_points;

	    pt_points = pt_p;
	}

    // Le format (ie32 ou autres) des fichiers sockets binaires spx est
    // choisi dans le script de lancement de Psyche par
    // la variable $SERVEUR_INET_SP3
    // le client doit choisir le meme format avec la librarie cliente psyche
    // par $DEST_SP3 ou le nom passe a SpxOpen
    // Usuellement on prend le format ie32 (IEEE 32 bits big endian)

	S6FlagsSup::socket = TRUE;
	*fichier_socket_binaire >> *pt_points;

    // si close

	char temp = '$';

	if(!fichier_socket_binaire->get(temp).good())
	{
	    delete fichier_socket_binaire;

	    cout << "S6GereFichier> close fichier_socket_binaire " <<
		temp << endl;
	}
	else
	{
	    socket_binaire.AddInputDriver(fichier_socket_binaire,0);

	    cout << "S6GereFichier> caractere fin de fichier " << temp << endl;
	}

	P4Boolean last = pt_points->TraceLastBase();
	P4Boolean trace = P4Boolean((pt_points->Typefichier() != S6CONFIG_SEULE
	    || calcul_socket != -1) && calcul_socket);

	S6Trace::Trace(S6FlagsSup::socket && trace,last);

	return ier;
}

// **********************************************
//      lecture fichier non socket binaire
// **********************************************

int
S6GereFichier::LitFichierBinaire()
{

	if(!fichier_points || *fichier_points == '|' ||
		strpbrk(fichier_points,"+: @"))
	{
	    cerr << "\nAttention dans " << __FILE__ << " ligne " <<
		__LINE__ << "\nvotre fichier a un nom commencant par | ou " <<
		"contenant les caracteres @ + : ou blanc\non ne lit pas\n" <<
		endl;

	    return -1;
	}

	int ier = 0;
	nxstream fisobi(fichier_points,"r");

	S6FlagsSup::socket = FALSE;

	if(!fisobi.stdiofile())
	{
	    cerr << "S6GereFichier> Pb ouverture fichier binaire : "
		"On ne lit pas" << endl;

	    return -1;
	}

    // le format des fichiers binaires spx est obligatoirement de type
    // "ie32" au sens nx = mis en dur dans la librarie cliente psyche
    // pour plus de surete

	fisobi.format("ie32");

	if(!pt_points)
	{
	    pt_points = new S6Points;
	}
	else
	{
	    S6Points* pt_p = new S6Points(*pt_points);

	    delete pt_points;

	    pt_points = pt_p;
	}

	while(fisobi.good())
	{
	    fisobi >> *pt_points;
	}

	return ier;
}

// **********************************************
//      lecture fichier socket ascii
// **********************************************

int
S6GereFichier::LitFichierSocketAscii(nxstream* fichier_socket_ascii,
		U8InputId input_id)
{
	int ier = 0;

	if(!fichier_socket_ascii)
	{
	    fichier_socket_ascii = new nxstream(*serveur_ascii);

	    cout << "S6GereFichier> accept\n";
	}

	cout << "S6GereFichier> input_id " << input_id << endl;

	if(input_id != -1)
	{
	    socket_ascii.RemoveInputDriver(input_id);

	    cout << "S6GereFichier> nouvelle lecture sur meme socket" << endl;
	}

	if(!fichier_socket_ascii || !fichier_socket_ascii->rdbuf())
	{
	    cerr << "Pb ouverture canal de lecture ASCII :"
		" On ne lit pas " << endl;
	    return -1;
	}

	cout << "S6GereFichier> fichier_socket_ascii " <<
			(void*) fichier_socket_ascii << endl;

	if(!pt_points)
	{
	    pt_points = new S6PointsAscii;
	}
	else
	{
	    S6Points* pt_p = new S6PointsAscii(*pt_points);

	    delete pt_points;

	    pt_points = pt_p;
	}

	S6FlagsSup::socket = TRUE;
	*fichier_socket_ascii >> *pt_points;

    // si close

	char temp = '$';

	if(!fichier_socket_ascii->get(temp).good())
	{
	    delete fichier_socket_ascii;

	    cout << "S6GereFichier> close fichier_socket_ascii " << temp << endl;
	}
	else
	{
	    socket_ascii.AddInputDriver(fichier_socket_ascii,0);

	    cout << "S6GereFichier> caractere fin de fichier " << temp << endl;
	}

	P4Boolean trace = P4Boolean((pt_points->Typefichier() != S6CONFIG_SEULE
	    || calcul_socket != -1) && calcul_socket);

	S6Trace::Trace(S6FlagsSup::socket && trace);

	return ier;
}

// **********************************************
//      lecture fichier non socket ascii
// **********************************************

int
S6GereFichier::LitFichierAscii()
{

	if(!fichier_points || *fichier_points == '|' ||
		strpbrk(fichier_points,"+: @"))
	{
	    cerr << "\nAttention dans " << __FILE__ << " ligne " <<
		__LINE__ << "\nvotre fichier a un nom commencant par | ou " <<
		"contenant les caracteres @ + : ou blanc\non ne lit pas\n" <<
		endl;

	    return -1;
	}

	int ier = 0;
	//nxstream fisobi(fichier_points,"r");
	ifstream fisobi(fichier_points,ios::in);

	/*if(fisobi.stdiofile() == NULL)
	{
	    cerr << "Pb ouverture fichier ASCII : On ne lit pas" << endl;
	    return -1;
	    }*/

	if(!pt_points)
	{
	    pt_points = new S6PointsAscii;
	}
	else
	{
	    S6Points* pt_p = new S6PointsAscii(*pt_points);

	    delete pt_points;

	    pt_points = pt_p;
	}

	S6FlagsSup::socket = FALSE;
	*pt_points >> fisobi;
	//fisobi >> *pt_points;

	return ier;
}

// **********************************************
//      lecture fichier socket ucd
// **********************************************

int
S6GereFichier::LitFichierSocketUcd(nxstream* fichier_socket_ucd,U8InputId)
{
	int ier = 0;

	if(!fichier_socket_ucd)
	{
	    fichier_socket_ucd = new nxstream(*serveur_ucd);
	}

	if(!fichier_socket_ucd || !fichier_socket_ucd->rdbuf())
	{
	    cerr << "Pb ouverture canal de lecture UCD :"
		" On ne lit pas " << endl;
	    return -1;
	}

	cout << "S6GereFichier> fichier_socket_ucd " <<
		(void*) fichier_socket_ucd << endl;

	if(!pt_points)
	{
	    pt_points = new S6PointsUcd;
	}
	else
	{
	    S6Points* pt_p = new S6PointsUcd(*pt_points);

	    delete pt_points;

	    pt_points = pt_p;
	}

    // Le format (ie32 ou autres) des fichiers sockets binaires spx est
    // choisi dans le script de lancement de Psyche par
    // la variable $SERVEUR_INET_SP3
    // le client doit choisir le meme format avec la librarie cliente psyche
    // par $DEST_SP3 ou le nom passe a SpxOpen
    // Usuellement on prend le format ie32 (IEEE 32 bits big endian)

	S6FlagsSup::socket = TRUE;
	*fichier_socket_ucd >> *pt_points;

	delete fichier_socket_ucd;

	S6Trace::Trace(calcul_socket);

	return ier;
}

// **********************************************
//      lecture fichier non socket ucd
// **********************************************

int
S6GereFichier::LitFichierUcd()
{

	if(!fichier_points || *fichier_points == '|' ||
		strpbrk(fichier_points,"+: @"))
	{
	    cerr << "\nAttention dans " << __FILE__ << " ligne " <<
		__LINE__ << "\nvotre fichier a un nom commencant par | ou " <<
		"contenant les caracteres @ + : ou blanc\non ne lit pas\n" <<
		endl;

	    return -1;
	}

	int ier = 0;
	nxstream fisobi(fichier_points,"r");

	if(fisobi.stdiofile() == NULL)
	{
	    cerr << "Pb ouverture fichier UCD : On ne lit pas" << endl;
	    return -1;
	}

	if(!pt_points)
	{
	    pt_points = new S6PointsUcd;
	}
	else
	{
	    S6Points* pt_p = new S6PointsUcd(*pt_points);

	    delete pt_points;

	    pt_points = pt_p;
	}

    // le format des fichiers binaires ucd est obligatoirement de type
    // natif au sens nx = mis en dur dans la librarie cliente psyche
    // pour plus de surete
    // ce qui rend le binaire ucd d'AVS non portable entre 2 machines

	S6FlagsSup::socket = FALSE;
	fisobi >> *pt_points;

	return ier;
}

// **********************************************
//      lecture fichier socket communication
// **********************************************

int
S6GereFichier::LitFichierSocketCommunic(
		nxstream* fichier_socket_communic,U8InputId input_id)
{
	int ier = 0;

	if(!fichier_socket_communic)
	{
	    fichier_socket_communic = new nxstream(*serveur_binaire);

	    cout << "S6GereFichier> accept\n";
	}

	cout << "S6GereFichier> input_id " << input_id <<
	    " fichier_socket_communic " << fichier_socket_communic << endl;

	if(input_id != -1)
	{
	    socket_communic.RemoveInputDriver(input_id);

	    cout << "S6GereFichier> nouvelle lecture sur meme socket" << endl;
	}

	if(!fichier_socket_communic || !fichier_socket_communic->rdbuf())
	{
	    cerr << "Pb ouverture canal de lecture communication :"
		" On ne lit pas" << endl;
	    return -1;
	}

	int type = 0, sor = 0;

    // Le format (ie32 ou autres) des fichiers sockets binaires spx est
    // choisi dans le script de lancement de Psyche par
    // la variable $SERVEUR_INET_SP3
    // le client doit choisir le meme format avec la librarie cliente psyche
    // par $DEST_SP3 ou le nom passe a SpxOpen
    // Usuellement on prend le format ie32 (IEEE 32 bits big endian)

	fichier_socket_communic->read(&type,1);

	cout << "S6GereFichier> type  = " << type << endl;

	if(fichier_socket_communic->good())
	{

	    switch(type)
	    {
	      case -5 :
		cout << "S6GereFichier> SpcOpen" << endl;

		break;

	      case -4 :
		{
		    int ret = 0;

		    fichier_socket_communic->write(&ret,1);
		    fichier_socket_communic->wflush();
		}

		break;

	      case 0 : // cas demande de coordonnees
		{
		    REEL xpos = 0,ypos = 0;
		    REEL_S xpos_s = 0,ypos_s = 0;

		    L3GetPosition(2,xpos,ypos);
		    xpos_s = xpos;
		    ypos_s = ypos;
		    fichier_socket_communic->write(&xpos_s,1);
		    fichier_socket_communic->write(&ypos_s,1);
		    fichier_socket_communic->wflush();
		}

		break;

	      case 1 : // cas demande de coordonnees 2d
		{
		    REEL xpos = 0,ypos = 0;
		    REEL_S xpos_s = 0,ypos_s = 0;

		    cout << "S6GereFichier> axe_x_min_reel " << axe_x_min_reel;
		    cout << "\nS6GereFichier> axe_y_min_reel " << axe_y_min_reel;
		    cout << "\nS6GereFichier> axe_x_max_reel " << axe_x_max_reel;
		    cout << "\nS6GereFichier> axe_y_max_reel " <<
			axe_y_max_reel << endl;

		    if(!S6Trace::l3_init) L3Ingraf();

		    L3DefSpaceLog(1,axe_x_min_reel,axe_y_min_reel,
				    axe_x_max_reel,axe_y_max_reel,
				    axe_log_x,axe_log_y);
		    L3GetPosition(1,xpos,ypos);
		    xpos_s = xpos;
		    ypos_s = ypos;
		    fichier_socket_communic->write(&xpos_s,1);
		    fichier_socket_communic->write(&ypos_s,1);
		    fichier_socket_communic->wflush();
		}

		break;

	      case 2 : // cas demande de coordonnees 3d
	      case 102 :
		{
		    REEL xpos = 0,ypos = 0,zpos = 0;
		    unsigned nmapos = 0,nblocpos = 0;

		    if(S6GereEnv::dest_inte)
		    {

			if(!S6Trace::l3_init) L3Ingraf();

			S6Trace::graf->P4Coordonnees(xpos,ypos,zpos,
					nmapos,nblocpos);
		    }

		    REEL_S xpos_s = xpos;
		    REEL_S ypos_s = ypos;
		    REEL_S zpos_s = zpos;
		    int nmapos_i = nmapos;
		    int nblocpos_i = nblocpos;

		    fichier_socket_communic->write(&xpos_s,1);
		    fichier_socket_communic->write(&ypos_s,1);
		    fichier_socket_communic->write(&zpos_s,1);

		    if(type == 102)
		    {
			fichier_socket_communic->write(&nmapos_i,1);
			fichier_socket_communic->write(&nblocpos_i,1);
		    }

		    fichier_socket_communic->wflush();
		}

		break;

	      case 3 : // cas demande de lecture des donnees abstraites
		{
		    TabS6DonneeAbstraite tsda = pt_points->DonneeAbstraite();
		    int nbbloc = tsda;

		    fichier_socket_communic->write(&nbbloc,1);

		    for(int i=0;i<nbbloc;i++)
		    {
			S6DonneeAbstraite* sda = tsda[i];
			int wri_temp = sda->Typefichier();

			fichier_socket_communic->write(&wri_temp,1);
			wri_temp = sda->Typec();
			fichier_socket_communic->write(&wri_temp,1);
			wri_temp = sda->Typemailles();
			fichier_socket_communic->write(&wri_temp,1);
			wri_temp = sda->Nx();
			fichier_socket_communic->write(&wri_temp,1);
			wri_temp = sda->Ny();
			fichier_socket_communic->write(&wri_temp,1);
			wri_temp = sda->Nz();
			fichier_socket_communic->write(&wri_temp,1);

			REEL_S* pt_temp = sda->X();
			int pt_temp_taille = sda->X();

			fichier_socket_communic->write(&pt_temp_taille,1);
			fichier_socket_communic->write(pt_temp,pt_temp_taille);
			pt_temp = sda->Y();
			pt_temp_taille = sda->Y();
			fichier_socket_communic->write(&pt_temp_taille,1);
			fichier_socket_communic->write(pt_temp,pt_temp_taille);
			pt_temp = sda->Z();
			pt_temp_taille = sda->Z();
			fichier_socket_communic->write(&pt_temp_taille,1);
			fichier_socket_communic->write(pt_temp,pt_temp_taille);
			pt_temp = sda->C();
			pt_temp_taille = sda->C();
			fichier_socket_communic->write(&pt_temp_taille,1);
			fichier_socket_communic->write(pt_temp,pt_temp_taille);

			wri_temp = sda->Nmailles();
			fichier_socket_communic->write(&wri_temp,1);
			wri_temp = sda->Mailles();
			fichier_socket_communic->write(&wri_temp,1);

			const unsigned* st_tempu = sda->Mailles();
			const int* st_temp = (const int*) st_tempu;

			fichier_socket_communic->write(st_temp,wri_temp);

			wri_temp = sda->Nvolume();
			fichier_socket_communic->write(&wri_temp,1);
			wri_temp = sda->Volume();
			fichier_socket_communic->write(&wri_temp,1);
			st_tempu = sda->Volume();
			st_temp = (const int*) st_tempu;
			fichier_socket_communic->write(st_temp,wri_temp);
		    }

		    fichier_socket_communic->wflush();
		}

		break;

	      case 4 :
		{
		// cas communication d'une Window qui sera cliente
		// de l'afficheur X11

		    int win = 0;

		    fichier_socket_communic->read(&win,1);

		    cout << "Enter Window cliente " << win << endl;

		    S6GereFenet::EnterWindowClient(win);
		}

		break;

	      case 5 :
		{
		// cas communication d'une Window qui sera cliente
		// de l'afficheur X11

		    int win = 0;

		    fichier_socket_communic->read(&win,1);

		    cout << "Leave Window cliente " << win << endl;

		    S6GereFenet::LeaveWindowClient(win);
		}

		break;

	      case 6 :
		{
		// cas communication d'une Window qui sera cliente
		// de l'afficheur X11

		    int win = 0;

		    fichier_socket_communic->read(&win,1);

		    cout << "Button Press cliente " << win << endl;

		    S6GereFenet::ButtonPressClient(win);
		}

		break;

	      case 16 :
		{
		// cas communication d'une Window qui sera cliente
		// de l'afficheur X11

		    int win = 0;

		    fichier_socket_communic->read(&win,1);

		    cout << "Button Press Base cliente " <<
			win << endl;

		    S6GereFenet::ButtonPressBaseClient(win);
		}

		break;

	      case 7 :
		{
		// cas communication d'une Window qui sera cliente
		// de l'afficheur X11

		    int win = 0;

		    fichier_socket_communic->read(&win,1);

		    cout << "Button Release cliente " << win << endl;

		    S6GereFenet::ButtonReleaseClient(win);
		}

		break;

	      case 17 :
		{
		// cas communication d'une Window qui sera cliente
		// de l'afficheur X11

		    int win = 0;

		    fichier_socket_communic->read(&win,1);

		    cout << "Button Release Base cliente " <<
			win << endl;

		    S6GereFenet::ButtonReleaseBaseClient(win);
		}

		break;

	      case 8 :
		{
		// cas communication d'une Window qui sera cliente
		// de l'afficheur X11

		    int win = 0;

		    fichier_socket_communic->read(&win,1);

		    cout << "Motion Notify cliente " << win << endl;

		    S6GereFenet::MotionNotifyClient(win);
		}

		break;

	      case 18 :
		{
		// cas communication d'une Window qui sera cliente
		// de l'afficheur X11

		    int win = 0;

		    fichier_socket_communic->read(&win,1);

		    cout << "Motion Notify Base cliente " <<
			win << endl;

		    S6GereFenet::MotionNotifyBaseClient(win);
		}

		break;

	      case 9 :
		{
		// cas communication d'une Window qui sera cliente
		// de l'afficheur X11

		    int cur = 0;
		    int old_cur = S6GereFenet::GetCurseurAff();

		    fichier_socket_communic->read(&cur,1);
		    fichier_socket_communic->write(&old_cur,1);
		    fichier_socket_communic->wflush();

		    cout << "Changement de curseur : " << old_cur <<
			" -> " << cur << endl;

		    S6GereFenet::SetCurseurAff(cur);
		}

		break;

	      case 10 :
		{
		// sortie de Psyche

		    fichier_socket_communic->read(&sor,1);
		}

		break;

	      case 1001 :
		{
		// asservissement de la position de l'afficheur X11
		// a un client X11

		    int x = 0,y = 0;

		    fichier_socket_communic->read(&x,1);
		    fichier_socket_communic->read(&y,1);

		    cout << "Changement de position de l'afficheur : x = " <<
			x << ", y = " << y << endl;

		    S6GereFenet::SetPositionAff(x,y);
		}

		break;

	      case 1002 :
		{
		// On montre l'afficheur X11

		    cout << "Montre affx" << endl;

		    S6GereFenet::MontreAff();
		}

		break;

	      case 1003 :
		{
		// On cache l'afficheur X11

		    cout << "Cache affx" << endl;

		    S6GereFenet::CacheAff();
		}

		break;

	      default :

		cerr << "S6GereFichier> code non reconnu" << endl;
		fichier_socket_communic->clear(ios::failbit |
				fichier_socket_communic->rdstate());

		break;
	    }

	}

    // si close

	char temp = '\0';

	if(!fichier_socket_communic->get(temp).good() || sor == 555)
	{
	    delete fichier_socket_communic;

	    cout << "S6GereFichier> close fichier_socket_communic" << endl;
	}
	else
	{
	    socket_communic.AddInputDriver(fichier_socket_communic,0);

	    cout << "S6GereFichier> caractere fin de fichier " << temp << endl;
	}

	if(sor == 555)
	{
	    S6GereFenet::Quit();
	}

	return ier;
}

// ********************************************************************
//      lecture fichier socket couleurs
//      lit des fichiers au format col sur une
//      socket
//      analyse ligne par ligne
//
//      si on rencontre ###COL.col### :
//      on stocke dans un fichier temporaire
//      jusqu'au prochain ###COL.col### et on appelle
//      S6GereFichier::LitFichierCol
//
//      si on rencontre ###COL.pal### :
//      on stocke dans un fichier temporaire
//      jusqu'au prochain ###COL.pal### et on appelle
//      S6GereFichier::LitFichierPal
//
//      si on rencontre ###COL.fond### :
//      on stocke dans un fichier temporaire
//      jusqu'au prochain ###COL.fond### et on appelle
//      S6GereFichier::LitFichierFond
// ********************************************************************

int
S6GereFichier::LitFichierSocketCol(nxstream* fichier_socket_col,
		U8InputId input_id)
{
	int ier = 0;

	if(!fichier_socket_col)
	{
	    fichier_socket_col = new nxstream(*serveur_binaire);

	    cout << "S6GereFichier> accept col" << endl;
	}

	cout << "S6GereFichier> input_id col " << input_id << endl;

	if(input_id != -1)
	{
	    socket_col.RemoveInputDriver(input_id);

	    cout << "S6GereFichier> nouvelle lecture sur meme socket col" <<
		endl;
	}

	if(!fichier_socket_col || !fichier_socket_col->rdbuf())
	{
	    cerr << "S6GereFichier : erreur : ouverture canal de lecture COL :"
		" On ne lit pas" << endl;
	    return -1;
	}

	const int lbuf = 1024;
	char buff[lbuf];

	fichier_socket_col->getline(buff,lbuf);

	const int lus = fichier_socket_col->gcount();

	if(lus > 0)
	{
	    const char* tcol = NULL;
	    const char* tpal = NULL;
	    const char* tfond = NULL;

	    if(strstr(buff,"###COL.sync###"))
	    {
		int syn = 0;

		fichier_socket_col->write(&syn,1);
		fichier_socket_col->wflush();
	    }
	    else if((tcol = strstr(buff,"###COL.col###")) ||
			    (tpal = strstr(buff,"###COL.pal###")) ||
			    (tfond = strstr(buff,"###COL.fond###")))
	    {
		static int numero = 0;

		delete[] fichier_points;

		{
		    ostrstream os;

		    os << tmpdir << '/' << naruser << "/soccol" << numero++ <<
			".XXXXXX" << ends;

		    fichier_points = mktemp(os.str());
		}

		ofstream ofs(fichier_points);

		while(fichier_socket_col->good())
		{
		    fichier_socket_col->getline(buff,lbuf);

		    const int lus2 = fichier_socket_col->gcount();

		    if(lus2 > 0)
		    {

			if(strstr(buff,"###COL.col###") && tcol ||
					strstr(buff,"###COL.pal###") && tpal ||
					strstr(buff,"###COL.fond###") && tfond)
			{
			    break;
			}
			else
			{
			    ofs.write(buff,strlen(buff));

			    if(lus2 != lbuf-1)
			    {
				ofs << '\n';
			    }

			}

		    }

		}

		ofs.close();

		if(tcol) LitFichierCol();
		else if(tpal) LitFichierPal();
		else if(tfond) LitFichierFond();

		unlink(fichier_points);
		S6Trace::Trace(calcul_socket != -1 && calcul_socket);
	    }

	}

    // si close

	if(fichier_socket_col->eof())
	{
	    delete fichier_socket_col;

	    cout << "close fichier_socket_col" << endl;
	}
	else
	{
	    socket_col.AddInputDriver(fichier_socket_col,0);
	}

	return ier;
}

// **********************************************
//      lecture fichier couleurs de base
// **********************************************

int
S6GereFichier::LitFichierCol()
{

    // lecture fichier colormap pour les (8) couleurs de base

        int nbrtab = 0;
        int rc = COL_InfoColorFile(fichier_points,&nbrtab);

        if(nbrtab<1 || rc != OK)
	{
	    ERR_Print_Messag(ERROR);
	    ERR_Clear_All();

	    return ERROR;
	}

        char name[80];
        TypeColorDef type;
        int nval = 0;
        int ncmp = 0;

        rc = COL_InfoColorTable(fichier_points,1,name,&type,&nval,&ncmp);

        if(rc != OK)
	{
	    ERR_Print_Messag(ERROR);
	    ERR_Clear_All();

	    return ERROR;
	}

        float* val = new float[nval*ncmp];

        rc = COL_GetColorValues(fichier_points,1,val);

        if(rc != OK)
	{
	    ERR_Print_Messag(ERROR);
	    ERR_Clear_All();

	    delete[] val;
	    return ERROR;
	}

        const int taille_col = parametre_map_col_t/3;
        int* const col = parametre_map_col;
        const float den = 1./(taille_col-1.);
        const float denf = 1./(nval-1.);
        int i = 0;

        switch(type)
        {
            case COL_CRGB :

                for(i=0;i<taille_col;i++)
                {
                    float indice = i*den;
                    register unsigned t_min = 0;
                    register unsigned t_max = nval-1;

                    while(t_max-t_min>1)
                    {
                        register unsigned t_mil = (t_min+t_max)/2;

                        if(indice > val[t_mil*ncmp])
                        {
                            t_min = t_mil;
                        }
                        else
                        {
                            t_max = t_mil;
                        }

                    }

                    register unsigned t_min1 = t_min + 1;
                    float delta = (indice - val[t_min*ncmp])/
                        (val[t_min1*ncmp] - val[t_min*ncmp]);
		    float delta1 = 1. - delta;

                    col[i] = int(255. *(delta*val[t_min1*ncmp+1] +
                        delta1*val[t_min*ncmp+1]));
                    col[i+taille_col] = int(255. *
			(delta*val[t_min1*ncmp+2] + delta1*val[t_min*ncmp+2]));
                    col[i+2*taille_col] = int(255. *
                        (delta*val[t_min1*ncmp+3] + delta1*val[t_min*ncmp+3]));
                }

                break;

            case COL_RGB :

                for(i=0;i<taille_col;i++)
                {
                    float indice = i*den;
                    register unsigned t_min = 0;
                    register unsigned t_max = nval-1;

                    while(t_max-t_min>1)
                    {
                        register unsigned t_mil = (t_min+t_max)/2;

                        if(indice > t_mil*denf)
                        {
                            t_min = t_mil;
                        }
                        else
                        {
                            t_max = t_mil;
                        }

                    }

                    register unsigned t_min1 = t_min + 1;
                    float delta = (indice - t_min*denf)/denf;
		    float delta1 = 1. - delta;

                    col[i] = int(255. * (delta*val[t_min1*ncmp] +
			delta1*val[t_min*ncmp]));
                    col[i+taille_col] = int(255. *
			(delta*val[t_min1*ncmp+1] + delta1*val[t_min*ncmp+1]));
                    col[i+2*taille_col] = int(255. *
                        (delta*val[t_min1*ncmp+2] + delta1*val[t_min*ncmp+2]));
                }

                break;

            case COL_CHLS :

                for(i=0;i<taille_col;i++)
                {
                    float indice = i*den;
                    register unsigned t_min = 0;
                    register unsigned t_max = nval-1;

                    while(t_max-t_min>1)
                    {
                        register unsigned t_mil = (t_min+t_max)/2;

                        if(indice > val[t_mil*ncmp])
                        {
                            t_min = t_mil;
                        }
                        else
                        {
                            t_max = t_mil;
                        }

                    }

                    register unsigned t_min1 = t_min + 1;
                    float delta = (indice - val[t_min*ncmp])/
                        (val[t_min1*ncmp] - val[t_min*ncmp]);
		    float delta1 = 1. - delta;
		    HLS hls;

                    hls.h = 360. * (delta*val[t_min1*ncmp+1] +
			delta1*val[t_min*ncmp+1]);
                    hls.l = 
			delta*val[t_min1*ncmp+2] + delta1*val[t_min*ncmp+2];
                    hls.s =
                        delta*val[t_min1*ncmp+3] + delta1*val[t_min*ncmp+3];

		    RGB rgb = L3HlsToRgb(hls);

		    col[i] = int(rgb.r);
		    col[i + taille_col] = int(rgb.g);
		    col[i + 2*taille_col] = int(rgb.b);
                }

                break;

            case COL_HLS :

                for(i=0;i<taille_col;i++)
                {
                    float indice = i*den;
                    register unsigned t_min = 0;
                    register unsigned t_max = nval-1;

                    while(t_max-t_min>1)
                    {
                        register unsigned t_mil = (t_min+t_max)/2;

                        if(indice > t_mil*denf)
                        {
                            t_min = t_mil;
                        }
                        else
                        {
                            t_max = t_mil;
                        }

                    }

                    register unsigned t_min1 = t_min + 1;
                    float delta = (indice - t_min*denf)/denf;
		    float delta1 = 1. - delta;
		    HLS hls;

                    hls.h = 360. * (delta*val[t_min1*ncmp] +
			delta1*val[t_min*ncmp]);
                    hls.l = 
			delta*val[t_min1*ncmp+1] + delta1*val[t_min*ncmp+1];
                    hls.s =
                        delta*val[t_min1*ncmp+2] + delta1*val[t_min*ncmp+2];

		    RGB rgb = L3HlsToRgb(hls);

		    col[i] = int(rgb.r);
		    col[i + taille_col] = int(rgb.g);
		    col[i + 2*taille_col] = int(rgb.b);
                }

                break;

            case COL_INVALID :

		cerr << "Fichier de couleur non lu" << endl;

		break;
        }

	delete[] val;

	return OK;
}

// **********************************************
//      lecture fichier couleurs palette
// **********************************************

int
S6GereFichier::LitFichierPal()
{

    // lecture fichier colormap pour les couleurs de la palette

        int nbrtab = 0;
        int rc = COL_InfoColorFile(fichier_points,&nbrtab);

        if(nbrtab<1 || rc !=OK)
	{
	    ERR_Print_Messag(ERROR);
	    ERR_Clear_All();

	    return ERROR;
	}

        char name[80];
        TypeColorDef type;
        int nval = 0;
        int ncmp = 0;

        rc = COL_InfoColorTable(fichier_points,1,name,&type,&nval,&ncmp);

        if(rc != OK)
	{
	    ERR_Print_Messag(ERROR);
	    ERR_Clear_All();

	    return ERROR;
	}

        float* val = new float[nval*ncmp];

        rc = COL_GetColorValues(fichier_points,1,val);

        if(rc != OK)
	{
	    ERR_Print_Messag(ERROR);
	    ERR_Clear_All();

	    delete[] val;
	    return ERROR;
	}

        const int taille_pal = parametre_map_pal_t/3;
        int* const pal = parametre_map_pal;
        const float den = 1./(taille_pal-1.);
        const float denf = 1./(nval-1.);
        int i = 0;

        switch(type)
        {
            case COL_CRGB :

                for(i=0;i<taille_pal;i++)
                {
                    float indice = i*den;
                    register unsigned t_min = 0;
                    register unsigned t_max = nval-1;

                    while(t_max-t_min>1)
                    {
                        register unsigned t_mil = (t_min+t_max)/2;

                        if(indice > val[t_mil*ncmp])
                        {
                            t_min = t_mil;
                        }
                        else
                        {
                            t_max = t_mil;
                        }

                    }

                    register unsigned t_min1 = t_min + 1;
                    float delta = (indice - val[t_min*ncmp])/
                        (val[t_min1*ncmp] - val[t_min*ncmp]);
		    float delta1 = 1. - delta;

                    pal[i] = int(255. * (delta*val[t_min1*ncmp+1] +
			delta1*val[t_min*ncmp+1]));
                    pal[i+taille_pal] = int(255. *
			(delta*val[t_min1*ncmp+2] + delta1*val[t_min*ncmp+2]));
                    pal[i+2*taille_pal] = int(255. *
                        (delta*val[t_min1*ncmp+3] + delta1*val[t_min*ncmp+3]));
                }

                break;

            case COL_RGB :

                for(i=0;i<taille_pal;i++)
                {
                    float indice = i*den;
                    register unsigned t_min = 0;
                    register unsigned t_max = nval-1;

                    while(t_max-t_min>1)
                    {
                        register unsigned t_mil = (t_min+t_max)/2;

                        if(indice > t_mil*denf)
                        {
                            t_min = t_mil;
                        }
                        else
                        {
                            t_max = t_mil;
                        }

                    }

                    register unsigned t_min1 = t_min + 1;
                    float delta = (indice - t_min*denf)/denf;
		    float delta1 = 1. - delta;

                    pal[i] = int(255. * (delta*val[t_min1*ncmp] +
			delta1*val[t_min*ncmp]));
                    pal[i+taille_pal] = int(255. *
			(delta*val[t_min1*ncmp+1] + delta1*val[t_min*ncmp+1]));
                    pal[i+2*taille_pal] = int(255. *
                        (delta*val[t_min1*ncmp+2] + delta1*val[t_min*ncmp+2]));
                }

                break;

            case COL_CHLS :

                for(i=0;i<taille_pal;i++)
                {
                    float indice = i*den;
                    register unsigned t_min = 0;
                    register unsigned t_max = nval-1;

                    while(t_max-t_min>1)
                    {
                        register unsigned t_mil = (t_min+t_max)/2;

                        if(indice > val[t_mil*ncmp])
                        {
                            t_min = t_mil;
                        }
                        else
                        {
                            t_max = t_mil;
                        }

                    }

                    register unsigned t_min1 = t_min + 1;
                    float delta = (indice - val[t_min*ncmp])/
                        (val[t_min1*ncmp] - val[t_min*ncmp]);
		    float delta1 = 1. - delta;
		    HLS hls;

                    hls.h = 360. * (delta*val[t_min1*ncmp+1] +
			delta1*val[t_min*ncmp+1]);
                    hls.l = 
			delta*val[t_min1*ncmp+2] + delta1*val[t_min*ncmp+2];
                    hls.s =
                        delta*val[t_min1*ncmp+3] + delta1*val[t_min*ncmp+3];

		    RGB rgb = L3HlsToRgb(hls);

		    pal[i] = int(rgb.r);
		    pal[i + taille_pal] = int(rgb.g);
		    pal[i + 2*taille_pal] = int(rgb.b);
                }

                break;

            case COL_HLS :

                for(i=0;i<taille_pal;i++)
                {
                    float indice = i*den;
                    register unsigned t_min = 0;
                    register unsigned t_max = nval-1;

                    while(t_max-t_min>1)
                    {
                        register unsigned t_mil = (t_min+t_max)/2;

                        if(indice > t_mil*denf)
                        {
                            t_min = t_mil;
                        }
                        else
                        {
                            t_max = t_mil;
                        }

                    }

                    register unsigned t_min1 = t_min + 1;
                    float delta = (indice - t_min*denf)/denf;
		    float delta1 = 1. - delta;
		    HLS hls;

                    hls.h = 360. * (delta*val[t_min1*ncmp] +
			delta1*val[t_min*ncmp]);
                    hls.l = 
			delta*val[t_min1*ncmp+1] + delta1*val[t_min*ncmp+1];
                    hls.s =
                        delta*val[t_min1*ncmp+2] + delta1*val[t_min*ncmp+2];

		    RGB rgb = L3HlsToRgb(hls);

		    pal[i] = int(rgb.r);
		    pal[i + taille_pal] = int(rgb.g);
		    pal[i + 2*taille_pal] = int(rgb.b);
                }

                break;

            case COL_INVALID :

		cerr << "Fichier de couleur non lu" << endl;

		break;
        }

	delete[] val;

	return OK;
}

// **********************************************
//      lecture fichier couleurs fond/antifond
// **********************************************

int
S6GereFichier::LitFichierFond()
{

    // lecture fichier colormap pour les couleurs du fond et de l'antifond

        int nbrtab = 0;
        int rc = COL_InfoColorFile(fichier_points,&nbrtab);

        if(nbrtab < 1 || rc != OK)
	{
	    ERR_Print_Messag(ERROR);
	    ERR_Clear_All();

	    return ERROR;
	}

        char name[80];
        TypeColorDef type;
        int nval = 0;
        int ncmp = 0;

        rc = COL_InfoColorTable(fichier_points,1,name,&type,&nval,&ncmp);

        if(rc != OK)
	{
	    ERR_Print_Messag(ERROR);
	    ERR_Clear_All();

	    return ERROR;
	}

        float* val = new float[nval*ncmp];

        rc = COL_GetColorValues(fichier_points,1,val);

        if(rc != OK)
	{
	    ERR_Print_Messag(ERROR);
	    ERR_Clear_All();

	    delete[] val;
	    return ERROR;
	}

        switch(type)
        {
            case COL_CRGB :

		{
		    int* const fond = parametre_map_fond;
		    int* const antifond = parametre_map_afond;
		    const int indice = (nval - 1)*ncmp;

		    fond[0] = int(255. * val[1]);
		    fond[1] = int(255. * val[2]);
		    fond[2] = int(255. * val[3]);

		    antifond[0] = int(255. * val[indice + 1]);
		    antifond[1] = int(255. * val[indice + 2]);
		    antifond[2] = int(255. * val[indice + 3]);
		}

                break;

            case COL_RGB :

		{
		    int* const fond = parametre_map_fond;
		    int* const antifond = parametre_map_afond;
		    const int indice = (nval - 1)*ncmp;

		    fond[0] = int(255. * val[0]);
		    fond[1] = int(255. * val[1]);
		    fond[2] = int(255. * val[2]);

		    antifond[0] = int(255. * val[indice]);
		    antifond[1] = int(255. * val[indice + 1]);
		    antifond[2] = int(255. * val[indice + 2]);
		}

                break;

            case COL_CHLS :

		{
		    int* const fond = parametre_map_fond;
		    int* const antifond = parametre_map_afond;
		    const int indice = (nval - 1)*ncmp;
		    HLS hls;

                    hls.h = 360. * val[1];
                    hls.l = val[2];
                    hls.s = val[3];

		    RGB rgb = L3HlsToRgb(hls);

		    fond[0] = int(rgb.r);
		    fond[1] = int(rgb.g);
		    fond[2] = int(rgb.b);

                    hls.h = 360. * val[indice + 1];
                    hls.l = val[indice + 2];
                    hls.s = val[indice + 3];

		    rgb = L3HlsToRgb(hls);

		    antifond[0] = int(rgb.r);
		    antifond[1] = int(rgb.g);
		    antifond[2] = int(rgb.b);
		}

                break;

            case COL_HLS :

		{
		    int* const fond = parametre_map_fond;
		    int* const antifond = parametre_map_afond;
		    const int indice = (nval - 1)*ncmp;
		    HLS hls;

                    hls.h = 360. * val[0];
                    hls.l = val[1];
                    hls.s = val[2];

		    RGB rgb = L3HlsToRgb(hls);

		    fond[0] = int(rgb.r);
		    fond[1] = int(rgb.g);
		    fond[2] = int(rgb.b);

                    hls.h = 360. * val[indice];
                    hls.l = val[indice + 1];
                    hls.s = val[indice + 2];

		    rgb = L3HlsToRgb(hls);

		    antifond[0] = int(rgb.r);
		    antifond[1] = int(rgb.g);
		    antifond[2] = int(rgb.b);
		}

                break;

            case COL_INVALID :

		cerr << "Fichier de couleur non lu" << endl;

		break;
        }

	delete[] val;

	return OK;
}

// **********************************************
//      lecture fichier socket l3
// **********************************************

int
S6GereFichier::LitFichierSocketL3(nxstream* fichier_socket_l3,
		U8InputId input_id)
{
	int ier = 0;

	if(!fichier_socket_l3)
	{
	    fichier_socket_l3 = new nxstream(*serveur_binaire);
	}

	cout << "S6GereFichier : input_id " << input_id << endl;

	if(input_id != -1)
	{
	    socket_l3.RemoveInputDriver(input_id);

	    cout << "S6GereFichier : nouvelle lecture sur meme socket" << endl;
	}

	if(!fichier_socket_l3 || !fichier_socket_l3->rdbuf())
	{
	    cerr << "S6GereFichier : Pb ouverture canal de lecture L3 :" <<
			" On ne lit pas" << endl;
	    return -1;
	}

	cout << "S6GereFichier : fichier_socket_l3 " <<
	    fichier_socket_l3 << endl;

	int fin = LitDisplayListFILE(fichier_socket_l3->stdiofile());

	if(fin == -1)
	{
	    delete fichier_socket_l3;

	    cout << "S6GereFichier> close fichier_socket_l3" << endl;
	}
	else if(fin == 0)
	{
	    socket_l3.AddInputDriver(fichier_socket_l3,0);
	}
	else // fin = 1 = synchronisation
	{
	    int flag = 0;

	    fichier_socket_l3->write(&flag,1);
	    fichier_socket_l3->wflush();
	    socket_l3.AddInputDriver(fichier_socket_l3,0);
	}

	if(fin != 1)
	{

	// pour sauvegarde automatique
	// pour l'instant prevoir fichier_l3_sauvegarde

	    static int num_fich_temp = 0;
	    P4Boolean lopen = TRUE;

	    if(strcmp(fichier_sauvegarde,"binaire") &&
			    strcmp(fichier_sauvegarde,"ascii") &&
			    strcmp(fichier_sauvegarde,"ucd"))
		lopen = FALSE;

	    if(lopen)
	    {
		cout << "S6GereFichier> num fichier = " << num_fich_temp << endl;

		const size_t tronc = 4;
		const size_t lnhs = strlen(negshost_sp3);
		char negshost_sp3_temp[tronc+1];
		const size_t lnst = min(lnhs,tronc);
		char* fichier_temp = NULL;

		memcpy(negshost_sp3_temp,negshost_sp3,lnst+1);

		if(lnst < tronc)
		{
		    memcpy(negshost_sp3_temp+lnst,"____",tronc-lnst);
		}

		negshost_sp3_temp[tronc] = '\0';

		S6String ss(S6NameComp::nom_dir_temp,"l3.",negshost_sp3_temp,
				num_fich_temp,4);

		fichier_temp = ss;

		cout << "S6GereFichier> fichier_temp = " << fichier_temp << endl;

		const int ihm_existe = S6GereEnv::ihm && S6GereFenet::root;
		const int ier = SauveDisplayList(fichier_temp);

		if(!ier)
		{
		    num_fich_temp++;

		    if(ihm_existe)
		    {
			S6NameComp::AjoutePileFichiersL3(fichier_temp);
			S6NameComp::num_fichier_l3 =
			    S6NameComp::taille_pile_l3-1;
			S6GereFenet::nom_fichier->Change(fichier_temp);
			S6GereFenet::gestion_fichier->ChangeL3(fichier_temp);
			S6GereFenet::gestion_fichier->
			    ChangeNumL3(S6NameComp::num_fichier_l3 + 1);
		    }

		}

		delete[] fichier_temp;
	    }

	}

	S6Trace::TraceFichierL3(fin != 1 && calcul_socket);

	return ier;
}

// **********************************************
//      lecture fichier non socket binaire
// **********************************************

int
S6GereFichier::LitFichierMtv()
{

	if(!fichier_points || *fichier_points == '|' ||
		strpbrk(fichier_points,"+: "))
	{
	    cerr << "\nAttention dans " << __FILE__ << " ligne " <<
		__LINE__ << "\nvotre fichier a un nom commencant par | ou " <<
		"contenant les caracteres @ + : ou blanc\non ne lit pas\n" <<
		endl;

	    return -1;
	}

	int ier = 0;
	nxstream fisobi(fichier_points,"r");

	S6FlagsSup::socket = FALSE;

	if(fisobi.stdiofile() == NULL)
	{
	    cerr << " Pb ouverture fichier mtv :" <<
			" On ne lit pas " << endl;
	    return -1;
	}

	if(!pt_points)
	{
	    pt_points = new S6PointsMtv;
	}
	else
	{
	    S6Points* pt_p = new S6PointsMtv(*pt_points);

	    delete pt_points;

	    pt_points = pt_p;
	}

	fisobi >> *pt_points;

	return ier;
}

void
S6GereFichier::ChangeCData()
{

	if(!pt_points)
	    return;

	const unsigned nbblocs = pt_points->DonneeAbstraite();

	if(!nbblocs)
	    return;

	for(unsigned i=0;i<nbblocs;i++)
	{
	    S6DonneeAbstraite* const sda = pt_points->DonneeAbstraite()(i);

	    if(sda)
	    {
		const unsigned ncda = sda->CDatas();

		if(ncda)
		{
		    const unsigned taille_i = max(0u,min(unsigned(donnee_noeud_numero),
								  ncda-1));

		    sda->C().Lier(sda->Datas()(taille_i));
		    sda->Typec(TypeC(sda->DatasTypeC()(taille_i)));
		}

	    }

	}

}
