#ifndef S6GEREFICHIER_H
#define S6GEREFICHIER_H

// %W% %G% Pierre Brochard

#include "poulpe/IHM.h"
#include "P4constant.h"

// --------------------------------------------------
// **************************************************
//    description des serveurs sockets et fichiers
// **************************************************
// --------------------------------------------------

// ****************************************************
//   ajout de drivers (voir IHM : XtAddInputDriver)
// ****************************************************

class S6Points;

class SocketBinaire : public ElementVisInp
{
    public :

	SocketBinaire() : ElementVisInp() {};
	~SocketBinaire() {};

	ElementInput* ElInput() {return this;};
	ElementVisuel* ElVisuel() {return this;};

	void LectureInputCB(int fid,U8InputId input_id);
};

class SocketAscii : public ElementVisInp
{
    public :

        SocketAscii() : ElementVisInp() {};
        ~SocketAscii() {};

	ElementInput* ElInput() {return this;};
	ElementVisuel* ElVisuel() {return this;};

        void LectureInputCB(int fid,U8InputId input_id);
};

class SocketUcd : public ElementVisInp
{
    public :

        SocketUcd() : ElementVisInp() {};
        ~SocketUcd() {};

	ElementInput* ElInput() {return this;};
	ElementVisuel* ElVisuel() {return this;};

        void LectureInputCB(int fid,U8InputId input_id);
};

class SocketCommunic : public ElementVisInp
{
    public :

        SocketCommunic() : ElementVisInp() {};
        ~SocketCommunic() {};

	ElementInput* ElInput() {return this;};
	ElementVisuel* ElVisuel() {return this;};

	void LectureInputCB(int fid,U8InputId input_id);
};

class SocketL3 : public ElementVisInp
{
    public :

        SocketL3() : ElementVisInp() {};
        ~SocketL3() {};

	ElementInput* ElInput() {return this;};
	ElementVisuel* ElVisuel() {return this;};

	void LectureInputCB(int fid,U8InputId input_id);
};

class SocketCol : public ElementVisInp
{
    public :

        SocketCol() : ElementVisInp() {};
        ~SocketCol() {};

	ElementInput* ElInput() {return this;};
	ElementVisuel* ElVisuel() {return this;};

	void LectureInputCB(int fid,U8InputId input_id);
};

// ******************************************
//     module de gestion de fichiers
// ******************************************

class S6GereFichier
{
	friend int main(int,char**);
	friend void Quit(int);
	friend class S6InitFich;
	friend class S6GereFenet;
	friend class S6Trace;
	friend class S6InitCourbe;
	friend class RootPsyche;
	friend class RootPsycheBatch;
	friend class NomFichier;
	friend class PsycheCommuniq;
	friend class CoupeLogique;
	friend class NombreCourbesSaisie;
	friend class GestionDonnees;
	friend class SocketBinaire;
	friend class SocketAscii;
	friend class SocketUcd;
	friend class SocketCommunic;
	friend class SocketL3;
	friend class SocketCol;
	friend class AvancePsyBouton;
	friend class ReculePsyBouton;
	friend class AllPsyBouton;
	friend class NomPsyFichier;
	friend class AvanceL3Bouton;
	friend class ReculeL3Bouton;
	friend class AllL3Bouton;
	friend class NomL3Fichier;
	friend class PsyNum;
	friend class L3Num;
	friend class GestionDonneesAppel;
	friend class Options3dChoixFin;
	friend class CourbesAppel;
	friend class EtatFichierSauve;
	friend class NumeroDonneeNoeudSaisie;

    private :

        virtual void dummy() = 0;

        static char* fichier_points;

        static nxstream* serveur_binaire;
        static nxstream* serveur_ascii;
        static nxstream* serveur_ucd;
        static nxstream* serveur_unix;

	static void SauvePoints();
        static S6Points* pt_points;

        static int nombre_serveurs;
        static int num_psyche; // si plusieurs Psyche ouvert

// ajout de drivers pour chaque connection socket (plusieurs fichiers sur
// la meme connection socket)

        static SocketBinaire socket_binaire;
        static SocketAscii socket_ascii;
        static SocketUcd socket_ucd;
        static SocketCommunic socket_communic;
        static SocketL3 socket_l3;
        static SocketCol socket_col;

// lecture de fichiers sockets

        static int LitFichierSocket(int fid);
        static int LitInput(int fid);
        static int LitFichierSocketCommunic(nxstream* f = NULL,
            U8InputId input_id = -1);
        static int LitFichierSocketBinaire(nxstream* f = NULL,
            U8InputId input_id = -1);
        static int LitFichierSocketAscii(nxstream* f = NULL,
            U8InputId input_id = -1);
        static int LitFichierSocketUcd(nxstream* f = NULL,
            U8InputId input_id = -1);
        static int LitFichierSocketL3(nxstream* f = NULL,
            U8InputId input_id = -1);
        static int LitFichierSocketCol(nxstream* f = NULL,
            U8InputId input_id = -1);

// lecture de fichiers non sockets

        static int LitFichier();
        static int LitFichierBinaire();
        static int LitFichierAscii();
        static int LitFichierUcd();
        static int LitFichierMtv();

// lecture de fichiers colormaps

        static int LitFichierCol();
        static int LitFichierPal();
        static int LitFichierFond();

    // Changement du tableau C fonction de donnee_noeud_numero (GestionDonnees).
	static void ChangeCData();
};

#endif
