// *********************************************************************
//  operateurs de lecture de fichiers .spx (binaires Psyche)
// *********************************************************************

struct SccsIdS6Points2
{
	static const char* const sccs_id;
};

const char* const SccsIdS6Points2::sccs_id = "%W% %G% Pierre Brochard";

// includes

#include "U8machcomp.h"

#ifdef CRAY
#   include <locale.h>
#endif

#ifdef _IBMR2
#   define _BSD
#endif

#include <ctype.h>
#include "P4constant.h"
#include "nx.h"
#include "S6points.h"
#include "S6namecomp.h"
#include "S6gereenv.h"
#include "S6flagssup.h"
#include "dinosaure/S6timer.h"
#include "dinosaure/P4quicksort.h"

//*********************************************************
//   implementation de la classe S6Points :
//   membres protected et private
//*********************************************************

// ********************************************************
//    Routines utiles
// ********************************************************

int
S6Points::LectureTexte(FILE* tf,char*& texte,int taille)
{

	if(taille < 0) return 0;

	char* tv = new char[taille+1];
	const int ier = fnxread(tf,tv,taille);

	tv[taille] = '\0';

	if(tv[0] != '\033' || !texte)
	{
	    delete[] texte;
	    texte = tv;
	}
	else
	{
	    delete[] tv;
	}

	return ier;
}

// *********************************************************
//       Conversion multibloc -> Non structure monobloc
// *********************************************************

void
S6Points::ConvMultiMono()
{

	if(_type == S6LOGIQUE_3D || _type == S6PHYSIQUE_3D ||
		_type == S6PHYSIQUE_4D || _type == S6PHYSIQUE_4D_NS ||
		_type == S6RECT_3D || _type == S6RECT_4D ||
		_type == S6PARRECT_4D)
	{

	    if(unsigned(_donnee_abstraite) <= 1)
	    {
		cout << "S6Points> conversion non necessaire" << endl;
		return;
	    }

	    TimerDetInit();

	    S6DonneeAbstraite* sda = new S6DonneeAbstraite;

	    sda->Typefichier(S6PHYSIQUE_4D_NS);

	    int npoints = 0;
	    unsigned taille = _donnee_abstraite;
	    unsigned i = 0;

	    for(;i<taille;i++)
	    {
		S6DonneeAbstraite* sda_old = _donnee_abstraite[i];

		sda->Typec(S6DONNEE_NOEUD);

		if(sda_old->Typec() == S6DONNEE_NOEUD)
		{
		    register const unsigned nx = sda_old->Nx();
		    register const unsigned ny = sda_old->Ny();
		    register const unsigned nxyz = nx*ny*sda_old->Nz();

		    npoints += nxyz;

		    if(unsigned(sda_old->X()) < nxyz)
		    {
			sda->X().Ajoute(sda_old->X(),nx,ny,0);
			sda->Y().Ajoute(sda_old->Y(),nx,ny,-1);
		    }
		    else
		    {
			sda->X().Ajoute(sda_old->X());
			sda->Y().Ajoute(sda_old->Y());
		    }

		    sda->Z().Ajoute(sda_old->Z());
		    sda->C().Ajoute(sda_old->C());
		}

	    }

	    sda->Nx(npoints);
	    sda->Ny(1);
	    sda->Nz(1);

	    Tabunsignedbuf pmailles(npoints);pmailles.Enleve();
	    Tabunsignedbuf lmailles(4*npoints);lmailles.Enleve();

	    unsigned num = 0;

	    npoints = 0;

	    for(i=0;i<taille;i++)
	    {
		S6DonneeAbstraite* sda_old = _donnee_abstraite[i];

		if(sda_old->Typec() == S6DONNEE_NOEUD)
		{
		    unsigned nmailles = sda_old->Nmailles();
		    const unsigned nx = unsigned(sda_old->Nx());
		    const unsigned ny = unsigned(sda_old->Ny());
		    const unsigned nz = unsigned(sda_old->Nz());

		    if(nmailles != 0)
		    {
			unsigned j;

			for(j=0;j<nmailles;j++)
			{
			    pmailles.Ajoute(sda_old->Mailles()[j] + num);
			}

			num = pmailles[unsigned(pmailles)-1];

			const unsigned n =
			    unsigned(sda_old->Mailles()) - nmailles;

			for(j=0;j<n;j++)
			{
			    lmailles.Ajoute(sda_old->Mailles()[nmailles+j] +
				npoints);
			}

		    }
		    else if(sda_old->Typefichier() == S6PARRECT_4D)
		    {
			register const unsigned nm1 = (nx-1)*(ny-1)*nz;
			register const unsigned nm2 = (ny-1)*(nz-1)*nx;
			register const unsigned nm3 = (nz-1)*(nx-1)*ny;
			unsigned j;

			nmailles = nm1 + nm2 + nm3;

			for(j=0;j<nm1;j++)
			{
			    pmailles.Ajoute(4u + num);
			    num += 4;

			    register const unsigned m1 = j%((nx-1)*(ny-1));
			    register const unsigned m2 = j/((nx-1)*(ny-1));
			    register const unsigned ji = m1%(nx-1);
			    register const unsigned ki = m1/(nx-1);
			    unsigned nxprov[4];

			    nxprov[0] = ji + ki*nx + m2*nx*ny + npoints;
			    nxprov[1] = nxprov[0] + 1;
			    nxprov[2] = nxprov[1] + nx;
			    nxprov[3] = nxprov[2] - 1;

			    lmailles.Ajoute(nxprov,4);
			}

			for(j=0;j<nm2;j++)
			{
			    pmailles.Ajoute(4u + num);
			    num += 4;

			    register const unsigned m1 = j%(nx*(ny-1));
			    register const unsigned m2 = j/(nx*(ny-1));
			    register const unsigned ji = m1%nx;
			    register const unsigned ki = m1/nx;
			    unsigned nxprov[4];

			    nxprov[0] = ji + ki*nx + m2*nx*ny + npoints;
			    nxprov[1] = nxprov[0] + nx;
			    nxprov[2] = nxprov[1] + nx*ny;
			    nxprov[3] = nxprov[2] - nx;

			    lmailles.Ajoute(nxprov,4);
			}

			for(j=0;j<nm3;j++)
			{
			    pmailles.Ajoute(4u + num);
			    num += 4;

			    register const unsigned m1 = j%((nx-1)*ny);
			    register const unsigned m2 = j/((nx-1)*ny);
			    register const unsigned ji = m1%(nx-1);
			    register const unsigned ki = m1/(nx-1);
			    unsigned nxprov[4];

			    nxprov[0] = ji + ki*nx + m2*nx*ny + npoints;
			    nxprov[1] = nxprov[0] + 1;
			    nxprov[2] = nxprov[1] + nx*ny;
			    nxprov[3] = nxprov[2] - 1;

			    lmailles.Ajoute(nxprov,4);
			}

		    }
		    else
		    {

			if(nx == 1)
			{
			    nmailles = ny-1;

                            for(unsigned j=0;j<nmailles;j++)
                            {
                                pmailles.Ajoute(2u + num);
                                num += 2;
    
                                unsigned nxprov[2];
    
                                nxprov[0] = j + npoints;
                                nxprov[1] = nxprov[0]+1;
    
                                lmailles.Ajoute(nxprov,2);
                            }

			}
			else
			{
			    nmailles = (nx-1) * (ny-1);

			    for(unsigned j=0;j<nmailles;j++)
			    {
				pmailles.Ajoute(4u + num);
				num += 4;

				const unsigned k = j%(nx-1);
				const unsigned l = j/(nx-1);
				unsigned nxprov[4];

				nxprov[0] = k+l*nx + npoints;
				nxprov[1] = nxprov[0]+1;
				nxprov[2] = nxprov[1]+nx;
				nxprov[3] = nxprov[2]-1;

				lmailles.Ajoute(nxprov,4);
			    }

			}

		    }

		    npoints += nx*ny*nz;
		}

	    }

	    sda->Nmailles(pmailles);
	    sda->Mailles().Ajoute(pmailles);
	    sda->Mailles().Ajoute(lmailles);

	    Enleve();

	    _donnee_abstraite.Ajoute(sda);

	    TimerDetReport(1,1,"Temps conversion Multi<->Mono");
	}

}

// ************************************************************************
//    genere un nom de fichier de sauvegarde dans $IMAGES_SP3/[.temp.pr]
// ************************************************************************

char*
S6Points::FichierTemp()
{
	cout << "S6Points> num fichier = " << num_fich_temp << endl;

	const size_t tronc = 4;
	char negshost_sp3_temp[tronc+1];
	const size_t lnegs = strlen(negshost_sp3);
	const size_t lnst = min(lnegs,tronc);

	memcpy(negshost_sp3_temp,negshost_sp3,lnst+1);

	if(lnst < tronc)
	{
	    memcpy(negshost_sp3_temp+lnst,"____",tronc-lnst);
	}

	negshost_sp3_temp[tronc] = '\0';

	ostrstream os;

	os << S6NameComp::nom_dir_temp << negshost_sp3_temp <<
	    setfill('0') << setw(4) << num_fich_temp << ends;

	return os.str();
}

void
S6Points::LectureBase(FILE* tf)
{
	int nbelem = 0;

	fnxreadi(tf,1,&nbelem);

	cout << "S6Points> nb elems " << nbelem << endl;

	unsigned taille = unsigned(_donnee_abstraite_base)+1;
	S6ContBase2D* const tsb = new S6ContBase2D;

	_donnee_abstraite_base.TraceLast(taille == 1 ? FALSE : TRUE);

	for(register int i=0;i<nbelem;i++)
	{
	    int souscode = 0;

	    fnxreadi(tf,1,&souscode);

	    S6SousTypeFichier stf = S6SousTypeFichier(souscode);

	    switch(stf)
	    {
	      case S6DELETEALL :

		EnleveBase();

		break;

	      case S6DELETE :
		{
		    int item = 0;

		    fnxreadi(tf,1,&item);

		    cout << "S6Points> item " << item << endl;

		    if(item >= 1 && item < int(taille))
		    {
			item--;

			delete _donnee_abstraite_base[item];

			_donnee_abstraite_base[item] = NULL;
			_recherche_base += item;
		    }

		    _donnee_abstraite_base.TraceLast(FALSE);
		}
		break;

	      case S6POINT :
		{
		    REEL_S xy[2];
		    int col;

		    fnxreadf(tf,2,xy);
		    fnxreadi(tf,1,&col);

		    S6Base2D* sb = new S6Base2DPoint(xy[0],xy[1],col);

		    tsb->Ajoute(sb);
		}
		break;

	      case S6SEGMENT :
		{
		    REEL_S xy[4];
		    int col;

		    fnxreadf(tf,4,xy);
		    fnxreadi(tf,1,&col);

		    S6Base2D* sb = new S6Base2DSegment(xy[0],xy[1],xy[2],
				    xy[3],col);

		    tsb->Ajoute(sb);
		}
		break;

	      case S6ARC :
		{
		    REEL_S xy[6];
		    int col;

		    fnxreadf(tf,6,xy);
		    fnxreadi(tf,1,&col);

		    S6Base2D* sb = new S6Base2DArc(xy[0],xy[1],xy[2],
				    xy[3],xy[4],xy[5],col);

		    tsb->Ajoute(sb);
		}
		break;

	      case S6RECTANGLE :
		{
		    REEL_S xy[4];
		    int col;

		    fnxreadf(tf,4,xy);
		    fnxreadi(tf,1,&col);

		    S6Base2D* sb = new S6Base2DRectangle(xy[0],xy[1],xy[2],
				    xy[3],col);

		    tsb->Ajoute(sb);
		}
		break;

	      case S6POLYLIGNE :
		{
		    int tai;

		    fnxreadi(tf,1,&tai);

		    REEL_S* x = new REEL_S[tai];
		    REEL_S* y = new REEL_S[tai];

		    if(!x || !y)
		    {
			delete[] x; delete[] y;

			cerr << "Erreur : pb alloc : " << __FILE__ <<
			    " ligne " << __LINE__ << endl;

			break;
		    }

		    fnxreadf(tf,tai,x);
		    fnxreadf(tf,tai,y);

		    int col;

		    fnxreadi(tf,1,&col);

		    S6Base2D* sb = new S6Base2DPolyline(x,y,tai,col);

		    delete[] x; delete[] y;

		    tsb->Ajoute(sb);
		}
		break;

	      case S6POLYGONE :
		{
		    int tai;

		    fnxreadi(tf,1,&tai);

		    REEL_S* x = new REEL_S[tai];
		    REEL_S* y = new REEL_S[tai];

		    if(!x || !y)
		    {
			delete[] x; delete[] y;

			cerr << "Erreur : pb alloc : " << __FILE__ <<
			    " ligne " << __LINE__ << endl;

			break;
		    }

		    fnxreadf(tf,tai,x);
		    fnxreadf(tf,tai,y);

		    int col;

		    fnxreadi(tf,1,&col);

		    S6Base2D* sb = new S6Base2DPolygone(x,y,tai,col);

		    delete[] x; delete[] y;

		    tsb->Ajoute(sb);
		}
		break;

	      case S6POLYGONECONT :
		{
		    int tai;

		    fnxreadi(tf,1,&tai);

		    REEL_S* x = new REEL_S[tai];
		    REEL_S* y = new REEL_S[tai];

		    if(!x || !y)
		    {
			delete[] x; delete[] y;

			cerr << "Erreur : pb alloc : " << __FILE__ <<
			    " ligne " << __LINE__ << endl;

			break;
		    }

		    fnxreadf(tf,tai,x);
		    fnxreadf(tf,tai,y);

		    int col[2];

		    fnxreadi(tf,2,col);

		    S6Base2D* sb = new S6Base2DPolygonecont(x,y,tai,
				    col[0],col[1]);

		    delete[] x; delete[] y;

		    tsb->Ajoute(sb);
		}
		break;

	      case S6TEXTE :

		break;

	      case S6TABPOINT :
		{
		    int tai;

		    fnxreadi(tf,1,&tai);

		    REEL_S* x = new REEL_S[tai];
		    REEL_S* y = new REEL_S[tai];

		    if(!x || !y)
		    {
			delete[] x; delete[] y;

			cerr << "Erreur : pb alloc : " << __FILE__ <<
			    " ligne " << __LINE__ << endl;

			break;
		    }

		    fnxreadf(tf,tai,x);
		    fnxreadf(tf,tai,y);

		    int col;

		    fnxreadi(tf,1,&col);

		    S6Base2D* sb = new S6Base2DTabPoint(x,y,tai,col);

		    delete[] x; delete[] y;

		    tsb->Ajoute(sb);
		}
		break;

	      case S6TABSEGMENT :
		{
		    int tai;

		    fnxreadi(tf,1,&tai);

		    REEL_S* x1 = new REEL_S[tai];
		    REEL_S* y1t = new REEL_S[tai];
		    REEL_S* x2 = new REEL_S[tai];
		    REEL_S* y2 = new REEL_S[tai];

		    if(!x1 || !y1t || !x2 || !y2)
		    {
			delete[] x1; delete[] y1t; delete[] x2; delete[] y2;

			cerr << "Erreur : pb alloc : " << __FILE__ <<
			    " ligne " << __LINE__ << endl;

			break;
		    }

		    fnxreadf(tf,tai,x1);
		    fnxreadf(tf,tai,y1t);
		    fnxreadf(tf,tai,x2);
		    fnxreadf(tf,tai,y2);

		    int col;

		    fnxreadi(tf,1,&col);

		    S6Base2D* sb = new S6Base2DTabSegment(x1,y1t,x2,y2,
				    tai,col);

		    delete[] x1; delete[] y1t; delete[] x2; delete[] y2;

		    tsb->Ajoute(sb);
		}
		break;

	      case S6TABARC :
		{
		    int tai;

		    fnxreadi(tf,1,&tai);

		    REEL_S* x1 = new REEL_S[tai];
		    REEL_S* y1t = new REEL_S[tai];
		    REEL_S* x2 = new REEL_S[tai];
		    REEL_S* y2 = new REEL_S[tai];
		    REEL_S* x3 = new REEL_S[tai];
		    REEL_S* y3 = new REEL_S[tai];

		    if(!x1 || !y1t || !x2 || !y2 || !x3 || !y3)
		    {
			delete[] x1; delete[] y1t;
			delete[] x2; delete[] y2;
			delete[] x3; delete[] y3;

			cerr << "Erreur : pb alloc : " << __FILE__ <<
			    " ligne " << __LINE__ << endl;

			break;
		    }

		    fnxreadf(tf,tai,x1);
		    fnxreadf(tf,tai,y1t);
		    fnxreadf(tf,tai,x2);
		    fnxreadf(tf,tai,y2);
		    fnxreadf(tf,tai,x3);
		    fnxreadf(tf,tai,y3);

		    int col;

		    fnxreadi(tf,1,&col);

		    S6Base2D* sb = new S6Base2DTabArc(x1,y1t,x2,y2,x3,y3,
				    tai,col);

		    delete[] x1; delete[] y1t;
		    delete[] x2; delete[] y2;
		    delete[] x3; delete[] y3;

		    tsb->Ajoute(sb);
		}
		break;

	      case S6TABRECTANGLE :
		{
		    int tai;

		    fnxreadi(tf,1,&tai);

		    REEL_S* x1 = new REEL_S[tai];
		    REEL_S* y1t = new REEL_S[tai];
		    REEL_S* x2 = new REEL_S[tai];
		    REEL_S* y2 = new REEL_S[tai];

		    if(!x1 || !y1t || !x2 || !y2)
		    {
			delete[] x1; delete[] y1t; delete[] x2; delete[] y2;

			cerr << "Erreur : pb alloc : " << __FILE__ <<
			    " ligne " << __LINE__ << endl;

			break;
		    }

		    fnxreadf(tf,tai,x1);
		    fnxreadf(tf,tai,y1t);
		    fnxreadf(tf,tai,x2);
		    fnxreadf(tf,tai,y2);

		    int col = 0;

		    fnxreadi(tf,1,&col);

		    S6Base2D* sb = new S6Base2DTabRectangle(x1,y1t,x2,y2,
				    tai,col);

		    delete[] x1; delete[] y1t; delete[] x2; delete[] y2;

		    tsb->Ajoute(sb);
		}
		break;

	      case S6TABPOLYLIGNE :
	      case S6TABPOLYGONE :
	      case S6TABPOLYGONECONT :
	      case S6TABTEXTE :

		break;
	    }

	}

	if(tsb->operator int())
	{
	    register unsigned taillerech = _recherche_base.Taille();
	    register unsigned vide = _recherche_base;

	    if(vide == taillerech)
	    {
		_donnee_abstraite_base.Ajoute(tsb);
	    }
	    else
	    {
		taille = vide+1;
		_donnee_abstraite_base[vide] = tsb;
		_recherche_base -= vide;
		_recherche_base.OptimiseTaille();
	    }

	    _donnee_abstraite_base.Last(taille-1);

	    if(S6FlagsSup::socket)
	    {
		int tai = taille;

		fnxwriti(tf,1,&tai);
		fnxflush(tf);
	    }

	}
	else
	{
	    delete tsb;
	}

}

void
S6Points::DonneBrasSommetNS(const TabREEL_S& cbras,const TabREEL_S& cnoeuds,
		int bras[])
{
	unsigned nsommets = cnoeuds;
	unsigned nbras = cbras;
	const unsigned dimbras = 2*nbras;
	const unsigned* ubras = (const unsigned*) bras;

    // inversion de bras

	unsigned* rang = new unsigned[dimbras];

	quicksort(ubras,rang,dimbras);

	const unsigned dimsommets = nsommets + dimbras;
	Tabunsigned tsommets(dimsommets);
	unsigned* som = new unsigned[dimbras];
	register unsigned i = 0;

	for(;i<nbras;i++)
	{
	    register const unsigned j = 2*i;

	    som[j+1] = som[j] = i;
	}

	unsigned* sommets = tsommets;
	unsigned* sompnsom = sommets + nsommets;

	for(i=0;i<dimbras;i++)
	{
	    register const unsigned ri = rang[i];

	    sommets[ubras[ri]]++;
	    sompnsom[i] = som[ri];
	}

	delete[] rang;
	delete[] som;

	for(i=1;i<nsommets;i++)
	{
	    sommets[i] += sommets[i-1];
	}

	Tabunsignedbuf ns;

	for(register unsigned j=0;j<nsommets;j++)
	{
	    register const unsigned jj = sommets[j];
	    register const unsigned ii = j ? sommets[j-1] : 0;

	    ns.ModTab(sompnsom+ii,jj-ii);

	    register const unsigned nns = ns;
	    register unsigned k = 0;
	    register REEL ccc = 0.;

	    switch(nns%4)
	    {
	      case 0 : while(k < nns)
	      {
		  ccc += cbras[ns[k++]];
		case 3:
		      ccc += cbras[ns[k++]];
		case 2:
		      ccc += cbras[ns[k++]];
		case 1:
		      ccc += cbras[ns[k++]];
	      }

	    }

	    if(nns) ccc /= nns;

	    cnoeuds[j] = ccc;
	}

}

void
S6Points::CorrectMailles(const Tabunsigned& mailles,unsigned nmailles,
		unsigned npoints)
{
	unsigned dimmailles = mailles;
	unsigned* const tmailles = mailles;

	if(!dimmailles || nmailles > dimmailles) return;

	tmailles[0] = max(tmailles[0],0u);

	unsigned i;

	for(i=1;i<nmailles;i++)
	{
	    tmailles[i] = max(tmailles[i],tmailles[i-1]);
	}

	if(npoints > 0)
	{
	    const unsigned npo1 = npoints - 1;

	    for(i=nmailles;i<dimmailles;i++)
	    {
		tmailles[i] = min(max(tmailles[i],0u),npo1);
	    }

	}

}

/* Fin du fichier */
