// ***************************************************************
// module de trace graphiques
// auteur P. Brochard
// ***************************************************************

struct SccsIdS6Trace
{
	static const char* const sccs_id;
};

const char* const SccsIdS6Trace::sccs_id = "%W% %G% P. Brochard";

// includes

#include "S6bouttons5.h"
#include "P4constant.h"
#include "sanglier.h"
#include "sanglierP.h"
#include "S6trace.h"
#include "minotaure/P4graf3.h"
#include "S6gereconfig.h"
#include "S6gerefich.h"
#include "S6points.h"
#include "S6flagssup.h"
#include "S6gereenv.h"
#include <ctype.h>
#include "poulpe/U8phigs.h"

#ifdef TRUE
#  undef TRUE
#endif

#ifdef FALSE
#  undef FALSE
#endif

#define finl '\n'

// init variables statiques de classe

P4Graf3* S6Trace::graf = new P4Graf3;
const REEL S6Trace::AspectTexte = 1.5;
const REEL S6Trace::EchelleTexte = 1.;
int S6Trace::num_courbe = 0;
VOLATILE P4Boolean S6Trace::breakpsyche = FALSE;
VOLATILE P4Boolean S6Trace::breakgestion_fichier = FALSE;
P4Boolean S6Trace::l3_init = FALSE;

extern const int REINITL3;

// fonctions membres

void
S6Trace::BreakPsyche(int sig)
{
	cout << "S6Trace> signal recu : " << sig << endl;

	graf->P4Ecriture(FALSE);
	breakpsyche = TRUE;
	breakgestion_fichier = TRUE;
}

REEL
S6Trace::Xphytolog(const REEL zmin,const REEL zmax,const int izech,
		const REEL z)
{
	REEL xphytolog = 0.;

	if(izech==0)     // cas lineaire
	{
	    xphytolog = ( z -zmin ) / ( zmax - zmin );
	}
	else             // cas logarithmique
	{
	    REEL zzmin = P4Log ( zmin );
            REEL zzmax = P4Log ( zmax );
            REEL zz    = P4Log ( z    );

            xphytolog  = ( zz -zzmin ) / ( zzmax - zzmin );
	}

	return xphytolog;
}

void
S6Trace::TraceEchellePolyg(int sous_image,const int n,const REEL y[],
		const int icol[],const REEL xg,const REEL xd)
{
	REEL xp[4],yp[4];

	xp[0] = xg;
	xp[1] = xg;
	xp[2] = xd;
	xp[3] = xd;

	for(int i=0;i<n;i++)
	{       
	    yp[0] = y[i];
	    yp[1] = y[i+1];
	    yp[2] = yp[1];
	    yp[3] = yp[0];

	    L3Couleur(icol[i]);
	    L3PointCourant(sous_image,xp[0],yp[0]);
	    L3Polygone(sous_image,xp,yp,4);
	}

}

void
S6Trace::TraceEchelleSegment(const int n,const REEL y[],const int icol[],
		const REEL xg,const REEL xd)
{
	REEL xde1 = xg+(xd-xg)*.1,xde2=xd-(xd-xg)*.1;

	for(int i=0;i<n;i++)
	{       
	    L3Couleur(icol[i]);
	    REEL yde = (y[i]+y[i+1])*.5;
	    L3Segment(2,&xde1,&yde,&xde2,&yde,1);
	}

}

void
S6Trace::TraceEchelleMarqueur(const int n,const REEL y[],const int icol[],
		const REEL xg,const REEL xd,const int marqueur[])
{
	REEL xde = (xg+xd)*.5;

	for(int i=0;i<n;i++)
	{       
	    L3Couleur(icol[i]);
	    REEL yde = (y[i]+y[i+1])*.5;
	    L3Marqueur(2,&xde,&yde,1,marqueur[i]);
	}

}

void
S6Trace::Termine()
{
	L3Tmgraf();
	l3_init = FALSE;

#ifdef GPHIGS
	if(S6GereEnv::dest_phigs || S6GereEnv::dest_phigsx11)
	{
	    endphigs();
	}
	else if(S6GereEnv::dest_phigscgm)
	{
	    closephigs();
	}
#endif

	delete graf;graf = NULL;
}

void
S6Trace::Init()
{

    // -initialisations, mise en page----------------------------------

	num_courbe++;

	P4Boolean init = TRUE;

	if(parametre_scene)
	{
	    S6FlagsSup::num_scene++;

	    if(S6FlagsSup::num_scene > 1) init = FALSE;

	}
	else if(S6FlagsSup::num_scene > 0) 
	{
	    S6FlagsSup::num_scene = 0;
	    init = FALSE;
	}

	cout << "\nS6Trace> TRACE NUMERO " << num_courbe << finl;

	if(!graf) graf = new P4Graf3;

	graf->P4Ecriture(TRUE);

	if(init)
	{
	    L3Ingraf();
	    l3_init = TRUE;

            RGB fond;
            RGB antifond;
            const int* pmf = parametre_map_fond;
            const int* apmf = parametre_map_afond;
            const int taille_col = parametre_map_col_t/3;
            const int taille_pal = parametre_map_pal_t/3;
            const int NombreCouleurs = taille_col + taille_pal + 2;

	    if(parametre_map == -1)
            {
                carte_de_couleurs ccc;
                const int taille_col_s = taille_col*sizeof(int);
                const int* const col = parametre_map_col;
                const int taille_pal_s = taille_pal*sizeof(int);
                const int* const pal = parametre_map_pal;

                ccc.n = NombreCouleurs;
                ccc.r = new int[NombreCouleurs];
                ccc.g = new int[NombreCouleurs];
                ccc.b = new int[NombreCouleurs];

                ccc.r[0] = *pmf++;
                ccc.g[0] = *pmf++;
                ccc.b[0] = *pmf++;
                ccc.r[1] = *apmf++;
                ccc.g[1] = *apmf++;
                ccc.b[1] = *apmf++;
                memcpy(ccc.r + 2,col,taille_col_s);
                memcpy(ccc.g + 2,col + taille_col,taille_col_s);
                memcpy(ccc.b + 2,col + 2*taille_col,taille_col_s);
                memcpy(ccc.r + 2 + taille_col,pal,taille_pal_s);
                memcpy(ccc.g + 2 + taille_col,pal + taille_pal,taille_pal_s);
                memcpy(ccc.b + 2 + taille_col,pal + 2*taille_pal,taille_pal_s);

                L3EcritCartesDeCouleurs(ccc);

		delete[] ccc.r;
		delete[] ccc.g;
		delete[] ccc.b;
            }
            else
            {
                fond.r = *pmf++;
                fond.g = *pmf++;
                fond.b = *pmf++;

                antifond.r = *apmf++;
                antifond.g = *apmf++;
                antifond.b = *apmf++;

                L3ChoixCartesDeCouleurs(nom_de_carte(parametre_map),
                    NombreCouleurs, &fond, &antifond);
            }

	    if(S6GereEnv::dest_gif || S6GereEnv::dest_png)
		L3Cadre(21.,29.7);
	    else
		L3Cadre(double(boite_tai_abs_x),double(boite_tai_abs_y));

	}

	L3DefPoliceCaractere(texte_generateur);

	if(zoom_gra_x_min == 0. && zoom_gra_x_max == 0. &&
		zoom_gra_y_min == 0. && zoom_gra_y_max == 0.)
	{
	    L3MiseEnPage(0,0.,0.,1.,1.,0.,0.,1.,1.); // 3d
	    L3MiseEnPage(2,0.,0.,1.,1.,0.,0.,1.,1.); // textes
	}
	else
	{
	    L3MiseEnPage(0,zoom_gra_x_min,zoom_gra_y_min,zoom_gra_x_max,
		zoom_gra_y_max,0.,0.,1.,1.);
	    L3MiseEnPage(2,zoom_gra_x_min,zoom_gra_y_min,zoom_gra_x_max,
		zoom_gra_y_max,0.,0.,1.,1.);
	}

	L3DefSpace(0,0.,0.,1.,1.);
	L3DefSpace(2,0.,0.,1.,1.);

	if(S6FlagsSup::choix_trace == S6FlagsSup::TRACE_2D)
	{
	    REEL xming = 14./99.,yming = 14./99.;
	    REEL xmaxg = 89./99.,ymaxg = 89./99.;

	    if(boite_cur_x_min != 0. || boite_cur_y_min != 0. ||
		boite_cur_x_max != 0. || boite_cur_y_max != 0.)
	    {
		xming = boite_cur_x_min;
		xmaxg = boite_cur_x_max;
		yming = boite_cur_y_min;
		ymaxg = boite_cur_y_max;
	    }

	    if(zoom_gra_x_min != 0. || zoom_gra_x_max != 0. ||
		zoom_gra_y_min != 0. || zoom_gra_y_max != 0.)
	    {
		xming = xming*(zoom_gra_x_max-zoom_gra_x_min) + zoom_gra_x_min;
		xmaxg = xmaxg*(zoom_gra_x_max-zoom_gra_x_min) + zoom_gra_x_min;
		yming = yming*(zoom_gra_y_max-zoom_gra_y_min) + zoom_gra_y_min;
		ymaxg = ymaxg*(zoom_gra_y_max-zoom_gra_y_min) + zoom_gra_y_min;
	    }

	    xming = xming*99.+1.;
	    xmaxg = xmaxg*99.+1.;
	    yming = yming*99.+1.;
	    ymaxg = ymaxg*99.+1.;

	    L3MiseEnPage(1,xming,yming,xmaxg,ymaxg,1.,1.,100.,100.);
	}

	L3Brillance(0.1);

    // -trace d'un carre pour forcer le fond a la couleur de fond----------

	if(init)
	{
	    S6Trace::Fond();
	}

}

void
S6Trace::Fond()
{

    // -trace d'un carre pour forcer le fond a la couleur de fond----------

	if(parametre_fond == 1)
	{
	    REEL xpag[4],ypag[4];

	    xpag[0] = 0.;ypag[0] = 1.;
	    xpag[1] = 1.;ypag[1] = 1.;
	    xpag[2] = 1.;ypag[2] = 0.;
	    xpag[3] = 0.;ypag[3] = 0.;
	    L3Couleur(0);
	    L3Polygone(2,xpag,ypag,4);
	}

}

void
S6Trace::Echelles34D()
{

    // -trace des echelles de couleur pour le 3d-4d------------------------

	REEL coef_taille = zoom_gra_coef_taille != 0. ?
		1./zoom_gra_coef_taille : 1.;

	if(hauteur_z_taille <= 0 && hauteur_c_taille <= 0) return;

	if((hauteur_z_type >= 1 || hauteur_c_type >= 1) &&
		S6FlagsSup::choix_trace != S6FlagsSup::TRACE_2D)
	{
	    REEL xtemp = min(hauteur_z_x_min,hauteur_z_x_max);
	    hauteur_z_x_max = max(hauteur_z_x_min,hauteur_z_x_max);
	    hauteur_z_x_min = xtemp;
	    REEL ytemp = min(hauteur_z_y_min,hauteur_z_y_max);
	    hauteur_z_y_max = max(hauteur_z_y_min,hauteur_z_y_max);
	    hauteur_z_y_min = ytemp;

	    xtemp = min(hauteur_c_x_min,hauteur_c_x_max);
	    hauteur_c_x_max = max(hauteur_c_x_min,hauteur_c_x_max);
	    hauteur_c_x_min = xtemp;
	    ytemp = min(hauteur_c_y_min,hauteur_c_y_max);
	    hauteur_c_y_max = max(hauteur_c_y_min,hauteur_c_y_max);
	    hauteur_c_y_min = ytemp;

	    int n_hauteurs = max(hauteur_z_taille,hauteur_c_taille);

	    REEL* yh = new REEL[n_hauteurs+2];
	    int*  ch = new int[n_hauteurs+1];

	    const int taille_col = parametre_map_col_t/3 + 2;
	    const int taille_pal = parametre_map_pal_t/3;

	    if(!yh || !ch)
	    {
		delete[] yh;delete[] ch;
		yh = NULL,ch = NULL;
		cerr << " pb allocation memoire dans P4Trace " << endl;
		return;
	    }

	    if(hauteur_z_type >= 1 && hauteur_z_taille > 0)
	    {
		TabREEL_S mhz(hauteur_z_taille+2);

		mhz.ModVal(hauteur_z[0],0);
		mhz.ModVal(hauteur_z[hauteur_z_taille-1],hauteur_z_taille+1);

		memcpy(&mhz[1],hauteur_z,hauteur_z_taille*sizeof(REEL_S));

		graf->P4CalcHauteurs(mhz,hauteur_z_taille+2,
			hauteur_z_h_min,hauteur_z_h_max,OPT_3D,
			P4Boolean(hauteur_z_log),TRUE);

		int i;

		for(i=0;i<hauteur_z_taille+2;i++)
		{
		    yh[i] = Xphytolog(mhz[0],mhz[hauteur_z_taille+1],
			hauteur_z_log,mhz[i]);
		}

		const REEL xc = REEL(taille_pal-1) /
			min(hauteur_z_taille,taille_pal-1);

		for(i=0;i<hauteur_z_taille+1;i++)
		{
		    ch[i] = int(xc*i+.1)%taille_pal+taille_col;
		}

		int modes = 0;

		if(hauteur_z_log)  modes += YCOLOG;

		if(zoom_gra_x_min == 0. && zoom_gra_x_max == 0. &&
			zoom_gra_y_min == 0. && zoom_gra_y_max == 0.)
		{
		    L3MiseEnPage(3,hauteur_z_x_min,hauteur_z_y_min,
			hauteur_z_x_max,hauteur_z_y_max,0.,0.,1.,1.);
		}
		else
		{
		    REEL hzxmin = hauteur_z_x_min * (zoom_gra_x_max -
			zoom_gra_x_min) + zoom_gra_x_min;
		    REEL hzxmax = hauteur_z_x_max * (zoom_gra_x_max -
			zoom_gra_x_min) + zoom_gra_x_min;
		    REEL hzymin = hauteur_z_y_min * (zoom_gra_y_max -
			zoom_gra_y_min) + zoom_gra_y_min;
		    REEL hzymax = hauteur_z_y_max * (zoom_gra_y_max -
			zoom_gra_y_min) + zoom_gra_y_min;

		    L3MiseEnPage(3,hzxmin,hzymin,hzxmax,hzymax,0.,0.,1.,1.);
		}

		L3DefSpaceLog(3,0.,0.,1.,1.,0,0);
		TraceEchellePolyg(3,hauteur_z_taille+1,yh,ch,0,1);
		L3DefGrille(3,GAUCHE,AXE_CREUX,TRUE,AUTOMATIQUE,AUCUNE);
		L3DefGrille(3,DROITE,AXE_ABSENT,FALSE,AUTOMATIQUE,AUCUNE);
		L3DefGrille(3,BAS,AXE_ABSENT,FALSE,AUTOMATIQUE,AUCUNE);
		L3DefGrille(3,HAUT,AXE_ABSENT,FALSE,AUTOMATIQUE,AUCUNE);
		L3DefTexte(3,int(hauteur_taille_z*coef_taille),0.);
		L3Couleur(grille_couleur);
		L3Grille(3,0.,mhz[0],1.,mhz[hauteur_z_taille+1],5,
				grille_grad_yech_nb,modes);
	    }

	    if(hauteur_c_type >= 1 && hauteur_c_taille > 0)
	    {
		TabREEL_S hc(hauteur_c_taille+2);

		hc.ModVal(hauteur_c[0],0);
		hc.ModVal(hauteur_c[hauteur_c_taille-1],hauteur_c_taille+1);

		memcpy(&hc[1],hauteur_c,hauteur_c_taille*sizeof(REEL_S));

		graf->P4CalcHauteurs(hc,hauteur_c_taille+2,
			hauteur_c_h_min,hauteur_c_h_max,OPT_4D,
			P4Boolean(hauteur_c_log),TRUE);

		int i;

		for(i=0;i<hauteur_c_taille+2;i++)
		{
		    yh[i] = Xphytolog(hc[0],hc[hauteur_c_taille+1],
			hauteur_c_log,hc[i]);
		}

		const REEL xc = REEL(taille_pal-1) /
			min(hauteur_c_taille,taille_pal-1);

		for(i=0;i<hauteur_c_taille+1;i++)
		{
		    ch[i] = int(xc*i+.1)%taille_pal+taille_col;
		}

		int modes = 0;

		if(hauteur_c_log)  modes += YCOLOG;

		if(zoom_gra_x_min == 0. && zoom_gra_x_max == 0. &&
			zoom_gra_y_min == 0. && zoom_gra_y_max == 0.)
		{
		    L3MiseEnPage(3,hauteur_c_x_min,hauteur_c_y_min,
			hauteur_c_x_max,hauteur_c_y_max,0.,0.,1.,1.);
		}
		else
		{
		    REEL hcxmin = hauteur_c_x_min * (zoom_gra_x_max -
			zoom_gra_x_min) + zoom_gra_x_min;
		    REEL hcxmax = hauteur_c_x_max * (zoom_gra_x_max -
			zoom_gra_x_min) + zoom_gra_x_min;
		    REEL hcymin = hauteur_c_y_min * (zoom_gra_y_max -
			zoom_gra_y_min) + zoom_gra_y_min;
		    REEL hcymax = hauteur_c_y_max * (zoom_gra_y_max -
			zoom_gra_y_min) + zoom_gra_y_min;

		    L3MiseEnPage(3,hcxmin,hcymin,hcxmax,hcymax,0.,0.,1.,1.);
		}

		L3DefSpaceLog(3,0.,0.,1.,1.,0,0);
		TraceEchellePolyg(3,hauteur_c_taille+1,yh,ch,0,1);
		L3DefGrille(3,GAUCHE,AXE_CREUX,TRUE,AUTOMATIQUE,AUCUNE);
		L3DefGrille(3,DROITE,AXE_ABSENT,FALSE,AUTOMATIQUE,AUCUNE);
		L3DefGrille(3,BAS,AXE_ABSENT,FALSE,AUTOMATIQUE,AUCUNE);
		L3DefGrille(3,HAUT,AXE_ABSENT,FALSE,AUTOMATIQUE,AUCUNE);
		L3DefTexte(3,int(hauteur_taille_c*coef_taille),0.);
		L3Couleur(grille_couleur);
		L3Grille(3,0.,hc[0],1.,hc[hauteur_c_taille+1],5,
				grille_grad_yech_nb,modes);
	    }

	    delete[] yh;delete[] ch;
	}

}

void
S6Trace::LegendesNiv()
{

    // -legendes des courbes de niveaux 4d------------------------------

	REEL coef_taille = zoom_gra_coef_taille != 0. ?
		1./zoom_gra_coef_taille : 1.;

	if(hauteur_label_type <= 0) return;

	if(hauteur_label_taille > 0 && S6FlagsSup::choix_trace !=
		S6FlagsSup::TRACE_2D)
	{
	    REEL xtemp = min(hauteur_label_x_min,hauteur_label_x_max);
	    hauteur_label_x_max = max(hauteur_label_x_min,hauteur_label_x_max);
	    hauteur_label_x_min = xtemp;
	    REEL ytemp = min(hauteur_label_y_min,hauteur_label_y_max);
	    hauteur_label_y_max = max(hauteur_label_y_min,hauteur_label_y_max);
	    hauteur_label_y_min = ytemp;

	    REEL* yh = new REEL[hauteur_label_taille+1];
	    int*  ch = new int[hauteur_label_taille+1];

	    if(!yh || !ch)
	    {
		delete[] yh;delete[] ch;
		yh = NULL,ch = NULL;
		cerr << " pb allocation memoire dans P4Trace " << endl;
		return;
	    }

	    const int taille_col = parametre_map_col_t/3 + 2;
	    const int taille_pal = parametre_map_pal_t/3;
	    const REEL xc = REEL(taille_pal-1) /
		min(hauteur_label_taille,taille_pal-1);
	    int i;

	    for(i=0;i<hauteur_label_taille;i++)
	    {
		yh[i] = Xphytolog(1.,REEL(hauteur_label_taille+1),
				0,REEL(i+1));
		yh[i] = (hauteur_label_y_max-hauteur_label_y_min)*yh[i]+
		    hauteur_label_y_min;
		ch[i] = int(xc*i+.1)%taille_pal+taille_col;
	    }

	    yh[hauteur_label_taille] = hauteur_label_y_max;
	    TraceEchelleSegment(hauteur_label_taille,yh,ch,
			hauteur_label_x_min,hauteur_label_x_max);

	    REEL xlstn = 0.;
	    int taille = max(10,int(hauteur_taille_label*coef_taille));
	    REEL xpo9 = hauteur_label_x_max;
	    char bufchar[256];
	    char decode[256];

	    for(i=0;i<hauteur_label_taille;i++)
	    {
		REEL ypo9 = (yh[i+1]+yh[i])*.5-.005;
		int lln = strlen(hauteur_label[i])+1;

		L3Couleur(ch[i]);
		L3DefTexte(2,taille,0.);
		L3PointCourant(2,xpo9,ypo9);

		xlstn = max(xlstn,REEL(lln));
		bufchar[0] = ' ',bufchar[1] = '\0';
		decode[0] = '\0';
		L3TranscodeChaine(strcat(bufchar,hauteur_label[i]),decode);
		L3Texte(2,decode);
	    }

	    xlstn *= coef_taille/taille;
	    L3Couleur(1);

	    REEL xx1=hauteur_label_x_min-.01,
		yy1=hauteur_label_y_min-.01,
		xx2=hauteur_label_x_max+.01 + xlstn,
		yy2=hauteur_label_y_min-.01;
	    L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

		xx1=hauteur_label_x_min-.01,
		yy1=hauteur_label_y_max+.01,
		xx2=hauteur_label_x_max+.01 + xlstn,
		yy2=hauteur_label_y_max+.01;
	    L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

		xx1=hauteur_label_x_min-.01,
		yy1=hauteur_label_y_min-.01,
		xx2=hauteur_label_x_min-.01,
		yy2=hauteur_label_y_max+.01;
	    L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

		xx1=hauteur_label_x_max+.01 + xlstn,
		yy1=hauteur_label_y_min-.01,
		xx2=hauteur_label_x_max+.01 + xlstn,
		yy2=hauteur_label_y_max+.01;
	    L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

	    delete[] yh;delete[] ch;
	}

}

void
S6Trace::Legendes2D()
{

    // -legende des courbes 2d-------------------------------------------

	REEL coef_taille = zoom_gra_coef_taille != 0. ?
		1./zoom_gra_coef_taille : 1.;

	if(courbe_label_type >= 1 && S6FlagsSup::choix_trace ==
		S6FlagsSup::TRACE_2D)
	{
	    REEL xtemp = min(courbe_label_x_min,courbe_label_x_max);
	    courbe_label_x_max = max(courbe_label_x_min,courbe_label_x_max);
	    courbe_label_x_min = xtemp;
	    REEL ytemp=min(courbe_label_y_min,courbe_label_y_max);
	    courbe_label_y_max = max(courbe_label_y_min,courbe_label_y_max);
	    courbe_label_y_min = ytemp;

	    int courbes_a_tracer = 0;
	    int ny = S6GereFichier::pt_points->DonneeAbstraite();
	    int flag = 3;
	    int i;

	    for(i = 0 ; i < ny ; i++)
	    {
		const int cotype = courbe_type[i];

		if(flag == 0) flag = 3;

		if(flag == 3 && cotype != -1) courbes_a_tracer++;
		else if(cotype != -1) flag--;

		if(flag == 3 && cotype == 7) flag--;

	    }

	    REEL* yh = new REEL[ny+1];
	    int*  jc = new int[ny+1];

	    if(!yh || !jc)
	    {
		delete[] yh;delete[] jc;
		yh=NULL,jc=NULL;
		cerr << "Pb allocation memoire dans P4Trace" << endl;
		return;
	    }

	    int* marqueur = new int[ny];
	    int j = -1;

	    flag = 3;

	    for(i=0;i<ny;i++)
	    {
		const int cotype = courbe_type[i];

		if(flag == 0) flag = 3;

		if(flag == 3 && cotype != -1)
		{
		    j++;
		    yh[j] = Xphytolog(1.,REEL(courbes_a_tracer+1),0,REEL(j+1));
		    yh[j] = (courbe_label_y_max-courbe_label_y_min)*yh[j]+
			courbe_label_y_min;
		    jc[j] = courbe_couleur[i];

		    if(cotype > 1 && cotype < 13 && cotype != 7)
		    {
			marqueur[j] = cotype > 7 ? cotype - 7 : cotype - 1;
		    }
		    else
		    {
			marqueur[j] = '-';
		    }

		}
		else if(cotype != -1) flag--;

		if(flag == 3 && cotype == 7) flag--;

	    }

	    yh[courbes_a_tracer] = courbe_label_y_max;

	    switch(courbe_legende_type)
	    {
	      case 0:
		TraceEchelleSegment(courbes_a_tracer,yh,jc,
				courbe_label_x_min,courbe_label_x_max);
		break;
	      case 1:
	    // cas ou la legende comporte des marqueurs
		TraceEchelleMarqueur(courbes_a_tracer,yh,jc,
				courbe_label_x_min,courbe_label_x_max,marqueur);
		break;
	      case 2:
	    // legende polygonale
		TraceEchellePolyg(2,courbes_a_tracer,yh,jc,
				courbe_label_x_min,courbe_label_x_max);
		break;
	      default:
		break;
	    }

	    delete[] marqueur;

	    int taille_carac = max(10,
			    int(courbe_taille_legende*coef_taille));
	    REEL xpo9 = courbe_label_x_max;
	    REEL taille_label = 0;
	    char bufchar[256];
	    char decode[256];

	    flag = 3;
	    j = -1;

	    for(i=0;i<ny;i++)
	    {

		if(flag == 0) flag = 3;

		if(flag == 3 && courbe_type[i] != -1)
		{
		    REEL ypo9 = (yh[j+2]+yh[j+1])*.5-.005;

		    j++;
		    L3Couleur(courbe_couleur[i]);
		    L3DefTexte(2,taille_carac,0.);
		    L3PointCourant(2,xpo9,ypo9);
		    bufchar[0] = ' ',bufchar[1] = '\0';
		    decode[0] = '\0';
		    L3TranscodeChaine(strcat(bufchar,courbe_label[i]),decode);
		    L3Texte(2,decode);

		    int xmin = 0,ymin = 0;
		    int xmax = 0,ymax = 0;
		    REEL xutilmin = 0,xutilmax = 0,yutil = 0;

		    if(texte_generateur)
		    {
			L3CadreTexteEvalEnglob(&xmin,&xmax,&ymin,&ymax);
			L3SelectionSousImage(2);
			L3ConvCoordL3VersUtil(xmin,ymin,&xutilmin,&yutil);
			L3ConvCoordL3VersUtil(xmax,ymax,&xutilmax,&yutil);
			taille_label = max(taille_label,xutilmax-xutilmin);
		    }
		    else
		    {
			taille_label = max(taille_label,
				REEL(strlen(courbe_label[i])+1)/taille_carac*
					coef_taille);
		    }

		}
		else if(courbe_type[i] != -1) flag--;

		if(flag == 3 && courbe_type[i] == 7) flag--;

	    }

	    if(courbe_legende_cadre)
	    {
		L3Couleur(1);

		REEL xx1 = courbe_label_x_min - .01,
		    yy1 = courbe_label_y_min - .01,
		    xx2 = courbe_label_x_max + .01 + taille_label,
		    yy2 = courbe_label_y_min - .01;
		L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

		xx1 = courbe_label_x_min - .01,
		    yy1 = courbe_label_y_max + .01,
		    xx2 = courbe_label_x_max + .01 + taille_label,
		    yy2 = courbe_label_y_max+.01;
		L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

		xx1 = courbe_label_x_min - .01,
		    yy1 = courbe_label_y_min - .01,
		    xx2 = courbe_label_x_min - .01,
		    yy2 = courbe_label_y_max + .01;
		L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);

		xx1 = courbe_label_x_max + .01 + taille_label,
		    yy1 = courbe_label_y_min - .01,
		    xx2 = courbe_label_x_max + .01 + taille_label,
		    yy2 = courbe_label_y_max + .01;
		L3Segment(2,&xx1,&yy1,&xx2,&yy2,1);
	    }

	    delete[] yh;delete[] jc;
	}

}

void
S6Trace::TitresAxes2d()
{
    // -titres des textes echelles x et y---------------------------

	REEL coef_taille = zoom_gra_coef_taille != 0. ?
	    1./zoom_gra_coef_taille : 1.;
	P4Boolean auto_boite = P4Boolean(boite_cur_x_min == 0. &&
			boite_cur_y_min == 0. && boite_cur_x_max == 0. &&
			boite_cur_y_max == 0.);

	REEL xpost1 = 0.,ypost1 = 0.;
	REEL xpost2 = 0.,ypost2 = 0.;
	REEL xpost3 = 0.,ypost3 = 0.;

	if(auto_boite)
	{
	    REEL xkkk = REEL(strlen(axe_valeur_x))/REEL(axe_taille_titre);
	    REEL ykkk = 5./REEL(axe_taille_echelle);

	    xpost1 = max(REEL(.01),REEL(89./99.-xkkk));
	    ypost1 = max(REEL(.01),REEL(14./99.-ykkk));

	    xkkk = 3./REEL(axe_taille_echelle);
	    ykkk /= 3.;

	    xpost2 = max(REEL(.01),REEL(14./99.-xkkk));
	    ypost2 = min(REEL(89./99.+ykkk),REEL(.99));

	    xkkk = REEL(strlen(axe_valeur_yd))/REEL(axe_taille_titre);

	    xpost3 = max(REEL(.01),REEL(89./99.-xkkk));
	    ypost3 = min(REEL(89./99.+ykkk),REEL(.99));
	}
	else
	{
	    REEL xkkk = REEL(strlen(axe_valeur_x))/REEL(axe_taille_titre);
	    REEL ykkk = 5./REEL(axe_taille_echelle);

	    xpost1 = max(REEL(.01),boite_cur_x_max-xkkk);
	    ypost1 = max(REEL(.01),boite_cur_y_min-ykkk);

	    xkkk = 3./REEL(axe_taille_echelle);
	    ykkk /= 3.;

	    xpost2 = max(REEL(.01),boite_cur_x_min-xkkk);
	    ypost2 = min(boite_cur_y_max+ykkk,REEL(.99));

	    xkkk = REEL(strlen(axe_valeur_yd))/REEL(axe_taille_titre);

	    xpost3 = max(REEL(.01),boite_cur_x_max-xkkk);
	    ypost3 = min(boite_cur_y_max+ykkk,REEL(.99));
	}

	if(!S6FlagsSup::seul)
	{
	    char decode[256];

	    L3Couleur(axe_couleur_titre);

	    if(axe_taille_titre == 300) axe_taille_titre = 100;

	    L3DefTexte(2,int(axe_taille_titre*coef_taille),0.);
	    L3PointCourant(2,xpost1,ypost1);

	    if(strlen(axe_valeur_x) > 0)
	    {
		L3TranscodeChaine(axe_valeur_x,decode);
		L3Texte(2,decode);
	    }

	    L3PointCourant(2,xpost2,ypost2);

	    if(strlen(axe_valeur_y) > 0)
	    {
		L3TranscodeChaine(axe_valeur_y,decode);
		L3Texte(2,decode);
	    }

	    L3PointCourant(2,xpost3,ypost3);

	    if(strlen(axe_valeur_yd) > 0)
	    {
		L3TranscodeChaine(axe_valeur_yd,decode);
		L3Texte(2,decode);
	    }

	}

}

void
S6Trace::Titres()
{

    // -TITRES------------------------------------------------------- 

	REEL coef_taille = zoom_gra_coef_taille != 0. ?
		1./zoom_gra_coef_taille : 1.;

	if(!S6FlagsSup::seul)
	{
	    char decode[256];
	    float ecart = -1.;

	    if(titre_affiche_haut >= 1)
	    {
		L3Couleur(titre_couleur_haut);
		L3DefTexte(0,int(titre_taille_haut*coef_taille),0.);
		decode[0] = '\0';
		L3TranscodeChaine(titre_valeur_haut,decode);
		L3AfficheTitres(0,decode,ecart, HAUT);

		if(titre_affiche_haut == 2) L3EncadreTexte();

	    }

	    if(titre_affiche_bas >= 1)
	    {
		L3Couleur(titre_couleur_bas);
		L3DefTexte(0,int(titre_taille_bas*coef_taille),0.);
		decode[0] = '\0';
		L3TranscodeChaine(titre_valeur_bas,decode);
		L3AfficheTitres(0,decode,ecart, BAS);

		if(titre_affiche_bas == 2) L3EncadreTexte();

	    }

	    if(titre_affiche_gauche >= 1)
	    {
		L3Couleur(titre_couleur_gauche);
		L3DefTexte(0,int(titre_taille_gauche*coef_taille),0.);
		decode[0] = '\0';
		L3TranscodeChaine(titre_valeur_gauche,decode);
		L3AfficheTitres(0,decode,ecart, GAUCHE);

		if(titre_affiche_gauche == 2) L3EncadreTexte();

	    }

	    if(titre_affiche_droit >= 1)
	    {
		L3Couleur(titre_couleur_droit);
		L3DefTexte(0,int(titre_taille_droit*coef_taille),0.);
		decode[0] = '\0';
		L3TranscodeChaine(titre_valeur_droit,decode);
		L3AfficheTitres(0,decode,ecart, DROITE);

		if(titre_affiche_droit == 2) L3EncadreTexte();

	    }

	    L3Couleur(grille_couleur);
	}

}

void
S6Trace::Textes()
{

    // -textes------------------------------------------------------------

	REEL coef_taille = zoom_gra_coef_taille != 0. ?
		1./zoom_gra_coef_taille : 1.;
	int taille = texte_affiche_taille;

	taille = min(taille,texte_couleur_taille);
	taille = min(taille,texte_taille_taille);
	taille = min(taille,texte_pos_x_taille);
	taille = min(taille,texte_pos_y_taille);
	taille = min(taille,texte_angle_taille);
	taille = min(taille,texte_valeur_taille);

	if(!S6FlagsSup::seul)
	{
	    char decode[256];

	    for(int i = 0 ; i < taille ; i++)
	    {

		if(texte_affiche[i] >= 1)
		{
		    int llm = strlen(texte_valeur[i]);

		    if(llm > 0)
		    {
			L3Couleur(texte_couleur[i]);
			double angle_texte = texte_angle[i];

			if(angle_texte > 180.) angle_texte -= 360.;

			L3DefTexte(2,int(texte_taille[i]*coef_taille),
					angle_texte);
			decode[0]='\0';
			L3TranscodeChaine(texte_valeur[i],decode);

			if(texte_generateur)
			{
			    L3GenTextePos(2,decode,REEL(texte_pos_x[i]),
					    REEL(texte_pos_y[i]),
					    EchelleTexte,AspectTexte,angle_texte,
					    VRAI,POSGB);

			    if(texte_affiche[i] == 2) L3EncadreTexte();

			}
			else
			{
			    L3PointCourant(2,REEL(texte_pos_x[i]),
					    REEL(texte_pos_y[i]));
			    L3Texte(2,decode);
			}

		   }

		}

	    }

	}

}

// *****************************************************************
//          traces 3d 
// *****************************************************************

int
S6Trace::FilOpt(int opt3d)
{
	Opt34d opt = OPT_3D;

	if(opt3d%8 == 0) opt = OPT_MONO;
	else if((opt3d-3)%8 == 0) opt = OPT_4D;

	return opt;
}

int
S6Trace::FacOpt(int opt3d)
{
	Opt34d opt = OPT_3D;

	switch(TypeTrace(opt3d))
	{
	  case FAC4D:
	  case FIL_FAC4D:
	  case FIL3D_FAC4D:
	  case FIL4D_FAC4D:
	  case FAC4D_NIV3D:
	  case FIL_FAC4D_NIV3D:
	  case FIL3D_FAC4D_NIV3D:
	  case FIL4D_FAC4D_NIV3D:
	  case FAC4D_NIV4D:
	  case FIL_FAC4D_NIV4D:
	  case FIL3D_FAC4D_NIV4D:
	  case FIL4D_FAC4D_NIV4D:
	    opt = OPT_4D;
	    break;
	  default:
	    opt = OPT_3D;
	    break;
	}

	return opt;
}

int
S6Trace::NivOpt(int opt3d)
{
	Opt34d opt = OPT_3D;

	switch(TypeTrace(opt3d))
	{
	  case NIV4D:
	  case FIL_NIV4D:
	  case FIL3D_NIV4D:
	  case FIL4D_NIV4D:
	  case FAC3D_NIV4D:
	  case FIL_FAC3D_NIV4D:
	  case FIL3D_FAC3D_NIV4D:
	  case FIL4D_FAC3D_NIV4D:
	  case FAC4D_NIV4D:
	  case FIL_FAC4D_NIV4D:
	  case FIL3D_FAC4D_NIV4D:
	  case FIL4D_FAC4D_NIV4D:
	    opt = OPT_4D;
	    break;
	  default:
	    opt = OPT_3D;
	    break;
	}

	return opt;
}

int
S6Trace::LisOpt(int opt3d)
{
	Opt34d opt = OPT_3D;

	switch(TypeTrace(opt3d))
	{
	  case LIS4D:
	  case FIL_LIS4D:
	  case FIL3D_LIS4D:
	  case FIL4D_LIS4D:
	    opt = OPT_4D;
	    break;
	  default:
	    opt = OPT_3D;
	    break;
	}

	return opt;
}

int
S6Trace::FilLisseOpt(int opt3d)
{
	FilLisse opt = P4FIL;

	switch(TypeTrace(opt3d))
	{
	  case FILMONO:
	  case FIL3D:
	  case FIL4D:
	    opt = P4FIL;
	    break;
	  case FAC3D:
	  case FAC4D:
	    opt = P4FAC;
	    break;
	  case NIV3D:
	  case NIV4D:
	    opt = P4NIV;
	    break;
	  case LIS3D:
	  case LIS4D:
	    opt = P4LIS;
	    break;
	  case FIL_FAC3D:
	  case FIL_FAC4D:
	  case FIL3D_FAC3D:
	  case FIL3D_FAC4D:
	  case FIL4D_FAC3D:
	  case FIL4D_FAC4D:
	    opt = P4FAC_FIL;
	    break;
	  case FIL_NIV3D:
	  case FIL_NIV4D:
	  case FIL3D_NIV3D:
	  case FIL3D_NIV4D:
	  case FIL4D_NIV3D:
	  case FIL4D_NIV4D:
	    opt = P4FIL_NIV;
	    break;
	  case FAC3D_NIV3D:
	  case FAC3D_NIV4D:
	  case FAC4D_NIV3D:
	  case FAC4D_NIV4D:
	    opt = P4FAC_NIV;
	    break;
	  case FIL_LIS3D:
	  case FIL_LIS4D:
	  case FIL3D_LIS3D:
	  case FIL3D_LIS4D:
	  case FIL4D_LIS3D:
	  case FIL4D_LIS4D:
	    opt = P4LIS_FIL;
	    break;
	  case FIL_FAC3D_NIV3D:
	  case FIL_FAC3D_NIV4D:
	  case FIL_FAC4D_NIV3D:
	  case FIL_FAC4D_NIV4D:
	  case FIL3D_FAC3D_NIV3D:
	  case FIL3D_FAC3D_NIV4D:
	  case FIL3D_FAC4D_NIV3D:
	  case FIL3D_FAC4D_NIV4D:
	  case FIL4D_FAC3D_NIV3D:
	  case FIL4D_FAC3D_NIV4D:
	  case FIL4D_FAC4D_NIV3D:
	  case FIL4D_FAC4D_NIV4D:
	    opt = P4FAC_FIL_NIV;
	    break;
	  default:
	    opt = P4FIL;
	    break;
	}

	return opt;
}

void
S6Trace::Option3dBloc(TypeTrace opt3d,int& fil_lisse,Opt34d& fil,Opt34d& fac,
		Opt34d& niv,Opt34d& lis)
{

	if(opt3d == TRACE_BLOC && option_3d_bloc_t >= 0)
	{
	    const unsigned taille = option_3d_bloc_t;
	    Opt34d* opt3d_bloc = new Opt34d[taille];
	    unsigned i = 0;

	    for(i = 0;i<taille;i++)
		opt3d_bloc[i] = Opt34d(FilOpt(option_3d_bloc[i]));

	    graf->P4FilOpt(opt3d_bloc,taille);

	    for(i = 0;i<taille;i++)
		opt3d_bloc[i] = Opt34d(FacOpt(option_3d_bloc[i]));

	    graf->P4FacOpt(opt3d_bloc,taille);

	    for(i = 0;i<taille;i++)
		opt3d_bloc[i] = Opt34d(NivOpt(option_3d_bloc[i]));

	    graf->P4NivOpt(opt3d_bloc,taille);

	    for(i = 0;i<taille;i++)
		opt3d_bloc[i] = Opt34d(LisOpt(option_3d_bloc[i]));

	    graf->P4LisOpt(opt3d_bloc,taille);

	    delete opt3d_bloc;
	    FilLisse* fillis = new FilLisse[taille];

	    for(i = 0;i<taille;i++)
		fillis[i] = FilLisse(FilLisseOpt(option_3d_bloc[i]));

	    graf->P4FilLisse(fillis,taille);

	    delete fillis;

	    fil = fac = niv = lis = OPT_NONE;
	    fil_lisse = P4USETAB;
	}

}

void
S6Trace::Option3dTBlocFil(int tfil)
{

	if(TypeTraceFil(tfil) == TRACE_NONE && option_3d_tbloc_fil_t >= 0)
	{
	    const unsigned taille = option_3d_tbloc_fil_t;

	    graf->P4TraceFilOpt(option_3d_tbloc_fil,taille);
	}

}

void
S6Trace::Option3dMaskBloc(int masq)
{

	if(Masque(masq) == MASQUE_NONE && option_3d_mask_bloc_t >= 0)
	{
	    const unsigned taille = option_3d_mask_bloc_t;

	    graf->P4MasqOpt(option_3d_mask_bloc,taille);
	}

}

void
S6Trace::Option3dMarqBloc(int marq)
{

	if(marq == -1 && option_3d_marq_bloc_t >= 0)
	{
	    const unsigned taille = option_3d_marq_bloc_t;

	    graf->P4MarqOpt(option_3d_marq_bloc,taille);
	}

}

void
S6Trace::CoupeHyp(int couphyp)
{

	if(Coupe(couphyp) == P4COUPE_TAB)
	{

	    if(coupe_hyperplan_bloc_t >= 0)
	    {
		const unsigned taille = coupe_hyperplan_bloc_t;

		graf->P4Coupe(coupe_hyperplan_bloc,taille);
	    }

	    if(coupe_plan_a_bloc_t >= 0) 
	    {
		const unsigned taille = coupe_plan_a_bloc_t;

		graf->P4CoupeA(coupe_plan_a_bloc,taille);
	    }

	    if(coupe_plan_b_bloc_t >= 0)
	    {
		const unsigned taille = coupe_plan_b_bloc_t;

		graf->P4CoupeB(coupe_plan_b_bloc,taille);
	    }

	    if(coupe_plan_c_bloc_t >= 0)
	    {
		const unsigned taille = coupe_plan_c_bloc_t;

		graf->P4CoupeC(coupe_plan_c_bloc,taille);
	    }

	    if(coupe_plan_d_bloc_t >= 0)
	    {
		const unsigned taille = coupe_plan_d_bloc_t;

		graf->P4CoupeD(coupe_plan_d_bloc,taille);
	    }

	    if(coupe_plan_e_bloc_t >= 0)
	    {
		const unsigned taille = coupe_plan_e_bloc_t;

		graf->P4CoupeE(coupe_plan_e_bloc,taille);
	    }

	}

}

void
S6Trace::TraceGeom(ostream& ost)
{
	graf->CouleurHaut(option_3d_cbloc_up,option_3d_cbloc_up_t);
	graf->CouleurBas(option_3d_cbloc_down,option_3d_cbloc_down_t);

	Opt34d fil = Opt34d(FilOpt(option_3d));
	Opt34d fac = Opt34d(FacOpt(option_3d));
	Opt34d niv = Opt34d(NivOpt(option_3d));
	Opt34d lis = Opt34d(LisOpt(option_3d));
	int fil_lisse = FilLisseOpt(option_3d);
	TypeTrace opt3d = TypeTrace(option_3d);

	Option3dBloc(opt3d,fil_lisse,fil,fac,niv,lis);
	graf->P4Init34d(fil,fac,niv,lis);

	TypeTraceFil tfil = TypeTraceFil(option_3d_trace_fil);

	graf->P4TypeTraceFil(tfil);
	Option3dTBlocFil(tfil);

	Coupe couphyp = P4COUPE_TAB;

	if(coupe_hyperplan == 0) couphyp = P4COUPE_NON;
	else if(coupe_hyperplan == 1) couphyp = P4COUPE_OUI;

	CoupeHyp(couphyp);

	Masque type_masque = Masque(option_3d_masque);

	Option3dMaskBloc(type_masque);

	int marq = option_3d_marqueur;

	Option3dMarqBloc(marq);

	switch (opt3d)
	{
	  case PAS_DE_TRACE :
	    break;
	  case FILMONO :
	    ost << "\nS6Trace> cas filaire monocouleur " << endl;
	    break;
	  case FIL3D :
	    ost << "\nS6Trace> cas filaire 3d " << endl;
	    break;
	  case FIL4D :
	    ost << "\nS6Trace> cas filaire 4d " << endl;
	    break;
	  case FAC3D :
	    ost << "\nS6Trace> cas facettes 3d " << endl;
	    break;
	  case FAC4D :
	    ost << "\nS6Trace> cas facettes 4d " << endl;
	    break;
	  case NIV3D :
	    ost << "\nS6Trace> cas niveaux 3d " << endl;
	    break;
	  case NIV4D :
	    ost << "\nS6Trace> cas niveaux 4d " << endl;
	    break;
	  case LIS3D :
	    ost << "\nS6Trace> cas lissage 3d " << endl;
	    break;
	  case LIS4D :
	    ost << "\nS6Trace> cas lissage 4d " << endl;
	    break;
	  case FIL_FAC3D :
	    ost << "\nS6Trace> cas filaire monocouleur + facettes 3d " << endl;
	    break;
	  case FIL_FAC4D :
	    ost << "\nS6Trace> cas filaire monocouleur + facettes 4d " << endl;
	    break;
	  case FIL3D_FAC3D :
	    ost << "\nS6Trace> cas filaire 3d + facettes 3d " << endl;
	    break;
	  case FIL3D_FAC4D :
	    ost << "\nS6Trace> cas filaire 3d + facettes 4d " << endl;
	    break;
	  case FIL4D_FAC3D :
	    ost << "\nS6Trace> cas filaire 4d + facettes 3d " << endl;
	    break;
	  case FIL4D_FAC4D :
	    ost << "\nS6Trace> cas filaire 4d + facettes 4d " << endl;
	    break;
	  case FIL_NIV3D :
	    ost << "\nS6Trace> cas filaire monocouleur + niveaux 3d " << endl;
	    break;
	  case FIL_NIV4D :
	    ost << "\n S6Trace>cas filaire monocouleur + niveaux 4d " << endl;
	    break;
	  case FIL3D_NIV3D :
	    ost << "\nS6Trace> cas filaire 3d + niveaux 3d " << endl;
	    break;
	  case FIL3D_NIV4D :
	    ost << "\nS6Trace> cas filaire 3d + niveaux 4d " << endl;
	    break;
	  case FIL4D_NIV3D :
	    ost << "\nS6Trace> cas filaire 4d + niveaux 3d " << endl;
	    break;
	  case FIL4D_NIV4D :
	    ost << "\nS6Trace> cas filaire 4d + niveaux 4d " << endl;
	    break;
	  case FAC3D_NIV3D :
	    ost << "\nS6Trace> cas facettes 3d + niveaux 3d " << endl;
	    break;
	  case FAC3D_NIV4D :
	    ost << "\nS6Trace> cas facettes 3d + niveaux 4d " << endl;
	    break;
	  case FAC4D_NIV3D :
	    ost << "\nS6Trace> cas facettes 4d + niveaux 3d " << endl;
	    break;
	  case FAC4D_NIV4D :
	    ost << "\nS6Trace> cas facettes 4d + niveaux 4d " << endl;
	    break;
	  case FIL_LIS3D :
	    ost << "\nS6Trace> cas filaire monocouleur + lissage 3d " << endl;
	    break;
	  case FIL_LIS4D :
	    ost << "\nS6Trace> cas filaire monocouleur + lissage 4d " << endl;
	    break;
	  case FIL3D_LIS3D :
	    ost << "\nS6Trace> cas filaire 3d + lissage 3d " << endl;
	    break;
	  case FIL3D_LIS4D :
	    ost << "\nS6Trace> cas filaire 3d + lissage 4d " << endl;
	    break;
	  case FIL4D_LIS3D :
	    ost << "\nS6Trace> cas filaire 4d + lissage 3d " << endl;
	    break;
	  case FIL4D_LIS4D :
	    ost << "\nS6Trace> cas filaire 4d + lissage 4d " << endl;
	    break;
	  case FIL_FAC3D_NIV3D :
	    ost << "\nS6Trace> cas filaire monocouleur + facettes 3d + "
	      << "niveaux 3d" << endl;
	    break;
	  case FIL_FAC3D_NIV4D :
	    ost << "\nS6Trace> cas filaire monocouleur + facettes 3d + "
	      << "niveaux 4d" << endl;
	    break;
	  case FIL_FAC4D_NIV3D :
	    ost << "\nS6Trace> cas filaire monocouleur + facettes 4d + "
	      << "niveaux 3d" << endl;
	    break;
	  case FIL_FAC4D_NIV4D :
	    ost << "\nS6Trace> cas filaire monocouleur + facettes 4d + "
	      << "niveaux 4d" << endl;
	    break;
	  case FIL3D_FAC3D_NIV3D :
	    ost << "\nS6Trace> cas filaire 3d + facettes 3d + "
	      << "niveaux 3d" << endl;
	    break;
	  case FIL3D_FAC3D_NIV4D :
	    ost << "\nS6Trace> cas filaire 3d + facettes 3d + "
	      << "niveaux 4d" << endl;
	    break;
	  case FIL3D_FAC4D_NIV3D :
	    ost << "\nS6Trace> cas filaire 3d + facettes 4d + "
	      << "niveaux 3d" << endl;
	    break;
	  case FIL3D_FAC4D_NIV4D :
	    ost << "\nS6Trace> cas filaire 3d + facettes 4d + "
	      << "niveaux 4d" << endl;
	    break;
	  case FIL4D_FAC3D_NIV3D :
	    ost << "\nS6Trace> cas filaire 4d + facettes 3d + "
	      << "niveaux 3d" << endl;
	    break;
	  case FIL4D_FAC3D_NIV4D :
	    ost << "\nS6Trace> cas filaire 4d + facettes 3d + "
	      << "niveaux 4d" << endl;
	    break;
	  case FIL4D_FAC4D_NIV3D :
	    ost << "\nS6Trace> cas filaire 4d + facettes 4d + "
	      << "niveaux 3d" << endl;
	    break;
	  case FIL4D_FAC4D_NIV4D :
	    ost << "\nS6Trace> cas filaire 4d + facettes 4d + "
	      << "niveaux 4d" << endl;
	    break;
	  case PIX3D :
	    ost << "\nS6Trace> cas pixel 3d" << endl;
	    break;
	  case PIX4D :
	    ost << "\nS6Trace> cas pixel 4d" << endl;
	    break;
	  case TRACE_BLOC:
	    ost << "\nS6Trace> cas trace de type bloc" << endl;
	    break;
	  default :
	    cerr << "S6Trace> Option " << opt3d << " non implementee : "
	      "on prend filaire 3D" << endl;
	    break;
	}

	if(opt3d == PIX3D)
	{
	    ost << "\nS6Trace> cas pixel 3d" << endl;
	    graf->P4InitPix34d(OPT_3D);
	    graf->P4TraceGeom();
	}
	else if(opt3d == PIX4D)
	{
	    ost << "\nS6Trace> cas pixel 4d" << endl;
	    graf->P4InitPix34d(OPT_4D);
	    graf->P4TraceGeom();
	}
	else if(opt3d != PAS_DE_TRACE)
	{
	    graf->P4TraceGeom(FilLisse(fil_lisse),couphyp,coupe_plan_a,
			    coupe_plan_b,coupe_plan_c,coupe_plan_d,coupe_plan_e);
	}

}

// *****************************************************************
//          trace des courbes 
// *****************************************************************

void
S6Trace::Trace2d(int cote,int nx,int ny,const Tabint& nxsto)
{

    // -trace des courbes 2d------------------------------------

	const int nx2 = max(2*nx-1,1);
	REEL* xd = new REEL[nx2];
	REEL* yd = new REEL[nx2];

	if(!xd || !yd)
	{
	    delete[] xd;delete[] yd;
	    xd = NULL;yd = NULL;
	    cerr << "S6Trace> pb allocation memoire dans p4trace " << endl;
	    return;
	}

	TabS6DonneeAbstraite tsda = S6GereFichier::pt_points->DonneeAbstraite();

	for(int j=0;j<ny;j++)
	{
	    const int cotype = courbe_type[j];

	    if(cotype >= 0 && courbe_cote[j] == cote)
	    {
		char decode[256];
		int i = 0;
		int jbarre = j;
		REEL_S* x = tsda[j]->X();
		REEL_S* y = tsda[j]->Y();

		L3Couleur(courbe_couleur[j]);

		switch (cotype)
		{
		  case 1 :

		    for(i=0;i<nxsto[j];i++)
		    {
			xd[2*i] = x[i];
			yd[2*i] = y[i];

			if(i != nxsto[j]-1)
			{
			    xd[2*i+1] = x[i+1];
			    yd[2*i+1] = y[i];
			}

		    }

		    L3PointCourant(1,*xd,*yd);
		    L3LigneBrisee(1,xd,yd,2*nxsto[j]-1);
		    break;

		  case 7 :
		  {

		      for(i=0;i<nxsto[j];i++)
		      {
			  xd[i] = x[i];
			  yd[i] = y[i];
		      }

		      int nxbe = nxsto[j];
		      REEL* miny = new REEL[nx];
		      REEL* maxy = new REEL[nx];

		      if(!miny || !maxy)
		      {
			  delete[] miny;delete[] maxy;
			  miny = NULL;maxy = NULL;

			  cerr << "S6Trace> pb allocation memoire dans " <<
			    __FILE__ << " ligne " << __LINE__ << endl;

			  return;
		      }

		      while(++j<ny && (cotype < 0 || courbe_cote[j] != cote));

		      if(j<ny)
		      {
			  REEL_S* my = tsda[j]->Y();

			  for(i=0;i<nxsto[j];i++)
			  {
			      miny[i] = my[i];
			  }

			  nxbe = min(nxbe,nxsto[j]);
		      }
		      else
		      {
			  nxbe = 0;
		      }

		      while(nxbe && ++j<ny && (cotype < 0 ||
					       courbe_cote[j] != cote));

		      if(nxbe && j<ny)
		      {
			  REEL_S* my = tsda[j]->Y();

			  for(i=0;i<nxsto[j];i++)
			  {
			      maxy[i] = my[i];
			  }

			  nxbe = min(nxbe,nxsto[j]);
			  L3BarreErreur(1,xd,yd,miny,maxy,nxbe);
		      }

		      delete[] miny;delete[] maxy;
		  }
		    break;

		  default :

		    if(cotype == 0 || (cotype > 7 && cotype < 13))
		    {

			for(i=0;i<nxsto[j];i++)
			{
			    xd[i] = x[i];
			    yd[i] = y[i];
			}

			L3PointCourant(1,*xd,*yd);
			L3LigneBrisee(1,xd,yd,nxsto[j]);
		    }
			
		    if(cotype != 0 && cotype < 13)
		    {
			register int cotype_temp = cotype;

			if(cotype_temp > 7) cotype_temp -= 6;

			for(i=0;i<nxsto[j];i++)
			{
			    xd[i] = x[i];
			    yd[i] = y[i];
			}

			L3PointCourant(1,*xd,*yd);
			L3Marqueur(1,xd,yd,nxsto[j],cotype_temp-1);
		    }

		}

	    // label de courbes

		REEL coef_taille = zoom_gra_coef_taille != 0. ?
			1./zoom_gra_coef_taille : 1.;

		L3DefTexte(1,int(courbe_taille_legende*coef_taille),0.);

		int lln = strlen(courbe_label[jbarre]);

		REEL xmax = axe_x_max_reel;
		REEL xmin = axe_x_min_reel;
		REEL ymax = courbe_cote[jbarre] == 0 ? axe_y_max_reel :
			axe_yd_max_reel;
		REEL ymin = courbe_cote[jbarre] == 0 ? axe_y_min_reel :
			axe_yd_min_reel;

		if(lln > 0 && courbe_label_type == 0)
		{
		    int isto = nxsto[jbarre]-1;

		    for(int ii=nxsto[jbarre]-1;ii>=0;ii--)
		    {

			if(x[ii] <= xmax && x[ii] >= xmin &&
			   y[ii] <= ymax && y[ii] >= ymin)
			{
			    isto = ii;
			    break;
			}

		    }

		    int isto1 = min(isto+1,nxsto[jbarre]-1);

		    isto = max(0,isto);

		    REEL yla = y[isto];

		    if(y[isto1] > ymax) yla = ymax;

		    if(y[isto1] < ymin) yla = ymin;

		    L3PointCourant(1,axe_x_max_reel,yla);
		    L3Troncature(1,CLIPPINGINACTIF);
		    L3TranscodeChaine(courbe_label[jbarre],decode);
		    L3Texte(1,decode);
		    L3Troncature(1,CLIPPINGACTIF);
		}

	    }

	}

	delete[] xd;delete[] yd;
}
