// *******************************************************
//      main de Psyche V1 2.x 
// *******************************************************

struct SccsIdS6Main
{
	static const char* const sccs_id;
};

const char* const SccsIdS6Main::sccs_id = "%W% %G% Pierre Brochard";

// ********************************************************
//  main de Narcisse/Psyche V1 x.y.z (x >= 2) declarations
// ********************************************************

#include "nxstream/U8serveur.h"
#include "P4constant.h"
#include "dinosaure/P4tabgenbasec.h"
#include "S6gereenv.h"
#include "S6ouvdriver.h"
#include "S6namecomp.h"
#include "S6initfich.h"
#include "S6gereconfig.h"
#include "S6gerefenet.h"
#include "S6trace.h"
#include "S6gerefich.h"
#include "S6traduc.h"
#include "S6nom.h"
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include "U8signal.h"
#include "nx.h"

// pour CRAY on met le handler de new car sur CRAY un new ou malloc
// non effectue ne le perturbe pas beaucoup !

#ifdef CRAY
#   include <new.h>

static void
S6NewHandler()
{
	cerr << "Erreur avec new : alloc non faite" << endl;
}
#endif

// *******************************************************
//      main de Psyche V1 definitions
// *******************************************************

int
main(int argc,char** argv)
{

    // Pb cout sur CRAY unicos_10

	cout << "main> test cout OK" << endl;

#if defined(IRIX) && defined(DBG)
	cout << "Option debug malloc" << endl;
	mallopt(M_DEBUG,1);
#endif

#if defined(SUNOS) && defined(DBG)
	cout << "Option debug malloc" << endl;
	//malloc_debug(2);
#endif

#ifdef CRAY
	set_new_handler(S6NewHandler);
#endif

    // lancement du serveur rpc de recherche de port s'il n'est pas deja lance

	const char* const ninr = getenv("NXINFO_NORPC");

	if(!ninr || !*ninr)
	    fnx_lance_rpc_port();

	cout << "main> fnx_lance_rpc_port fait" << endl;

    // Impression constantes utiles

	P4ImpConstantes();

	cout << "main> P4ImpConstantes fait" << endl;

    // generation du nom de Narcisse/Psyche V1.3 (Psyche ou Narcisse)

	S6Nom::Decode();

	cout << "main> debut fait" << endl;

    // flag d'erreur

	int ier = 0;

    // init variables d'environnement

	ier = S6GereEnv::ChargEnv();

	cout << "main> env initialise" << endl;

    // creation de variables externes utiles

	ier = S6NameComp::NameComp();

	cout << "main> name initialise" << endl;

    // relecture variables d'environnement (si dans fichier de config)

	ier = S6GereEnv::ChargEnv(S6NameComp::fichier_config);

	cout << "main> env config initialise" << endl;

    // creation de variables externes utiles (si modifs ds fichier de config)

	ier = S6NameComp::NameComp();

	cout << "main> name config initialise" << endl;

    // Tests utiles (caracteristiques de l'utilisateur)

	if(ier == -1)
	{
	    cerr << "main : erreur dans S6GereEnv" << endl;
	    exit(1);
	}

	if(!strcmp(naruser,"@@@@"))
	{
	    cerr << "main : la variable d'environnement USER n'est pas "
		"definie\net ne peux etre determinee automatiquement.\n"
		"Relancez apres l'avoir definie" << endl;
	    exit(1);
	}

	if(!strcmp(home,"@@@@"))
	{
	    cerr << "main : la variable d'environnement HOME n'est pas "
		"definie\net ne peux etre determinee automatiquement.\n"
		"Relancez apres l'avoir definie" << endl;
	    exit(1);
	}

    // ouverture NeGS et autres drivers si necessaire (sauf X11)

	ier = S6OuvDriver::OuvDriver();

	cout << "main> OuvDriver fait" << endl;

    // init musique + fichier initial (stdinput, image en dur ou fichier)

	ier = S6InitFich::InitFich();

	cout << "main> InitFich fait" << endl;

    // init config

	ier = S6GereConfig::Charge();
	ier = S6GereConfig::Charge(S6NameComp::fichier_config);

    // init traduc (langage de l'IHM)

	S6Traduc::Init(langage);

    // test ecriture config (reecriture)

	ier = S6GereConfig::Ecrit(S6NameComp::fichier_config);

    //  chargement fichier initial (stdinput, image en dur ou fichier)

	ier = S6GereFichier::LitFichier();

    //  gestion du break (reponse au kill -USR1)

	sigset(SIGUSR1,SIG_PF(S6Trace::BreakPsyche));

    //  gestion du kill soft (reponse au kill -TERM)

	sigset(SIGTERM,SIG_PF(S6GereFenet::Quit));

    // tampon (identification de l'executable)

	static char tampon[] =
		"\114\51\133\50\101\56\100\140\5\175"
		"\15\141\16\147\23\63\121\43\126\57"
		"\112\70\135\56\16\56\16\56\16\56"
		"\16\56\16\56\16\56\16\56\16\56"
		"\16\56\16\56\16\56\16\56\16\56"
		"\16\56\16\56\16\56\16\56\16\56"
		"\16\56\16\56\16\56\16\56\16\56"
		"\16\56\16\56\16\56\16\56\16\56";
	const unsigned taille = 80;
	char mask[2];

	mask[0] = 0x3a;
	mask[1] = '\0';

	char* buff = new char[taille + 1];

	buff[taille] = '\0';

	for(unsigned i=0;i<taille;i++)
	{
	    mask[0] = mask[0] ^ tampon[i];
	    buff[i] = mask[0];
	    mask[0] = tampon[i];
	}

	cout << "main : tampon : " << buff << endl;

	delete[] buff;

    // fenetrage ( + S6traduc.h ...)

	ier = S6GereFenet::InitFenet(argc,argv);

	return ier;
}
