/*                           N X - A M                                 */
/*                           ---------                                 */
/*                          C interface                                */

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.  
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/
/* source ident */
/***************************************************************************/

#ifndef lint
   static const char* const sccs_id = "%W% %G% Pierre Brochard";
#endif

/***************************************************************************/
/* includes */
/***************************************************************************/

#include "nxP.h"
#include "nxerror.h"
#include "nxoptim.h"
#include "nx_free.h"

#define NXP_H_IMPLEMENT
#   include "nx.h"
#undef NXP_H_IMPLEMENT

#if defined(SUNOS_4) || (defined(EPIX) && defined(_SYSTYPE_BSD43))
    extern char* sys_errlist[];
#   define strerror(a) sys_errlist[(a)]
#endif

/* ------------------------------------------------------------ */
/* init global variables                                        */
/* ------------------------------------------------------------ */

THREAD_LOCAL int NXERRNO = 0; /* error number : index of nxerror_list[]  */
static THREAD_LOCAL int NXNOPRINT = 0; /* pas d'impression de message d'erreurs nx si diff 0 */

static char* nxerror_list[] =
{
    "no error",
    "fnxopen : cant open file",
    "fnxopen : too many files",
    "net_addr : incorrect file name",
    "Nxwrite/read : erroneous unit",
    "Nxwrite : file not open",
    "nxwrite : callrpc error",
    "nxwriti : callrpc error",
    "nxwritf : callrpc error",
    "nxwritt : callrpc error",
    "nxdialo : callrpc error",
    "nxread : cant open socket in read mode",
    "nxreadi :  cant open socket in read mode",
    "nxreadf :  cant open socket in read mode",
    "nxreadt : cant open socket in read mode",
    "nxclose: erroneous fd index",
    "clientop : unknown host",
    "initform : unknown format",
    "read_char_xdr error",
    "read_int_xdr error",
    "read_float_xdr error",
    "write_char_xdr error",
    "write_int_xdr error",
    "write_float_xdr error",
    "nxreadt ou nxwritt : bad value icod ou ocod",
    "nxreadt - code type 1 error",
    "nxwritt - code type 2 error",
    "nxwritt - code type 1 error",
    "Double Cray to float ieee conversion not allowed",
    "Float ieee to double cray conversion not allowed",
    "nxchfrm - unknown format",
    "nxseek - unit is not a disk file",
    "nxseek - error",
    "nxtell - unit is not a disk file", 
    "nxwrite : cant open socket in write mode",
    "nxwriti :  cant open socket in write mode",
    "nxwritf :  cant open socket in write mode",
    "nxwritt : cant open socket in write mode",
    "conversion 64-32 sp on ieee host : underflow",
    "conversion 64-32 sp on ieee host : overflow",
    "conversion 64-32 dp on ieee host : underflow",
    "conversion 64-32 dp on ieee host : overflow",
    "conversion 64-32 int on ieee host : overflow",
    "conversion 64-32 sp on cray host : underflow",
    "conversion 64-32 sp on cray host : overflow",
    "conversion 64-32 dp on cray host : underflow",
    "conversion 64-32 dp on cray host : overflow",
    "conversion 64-32 int on cray host : overflow",
    "fnxopen : format error",
    "clientop : hostname not found",
    "clientop : not INET address",
    "clientop : cant alloc memory",
    "clientop : cant find temp name for bind",
    "fnxaccept2 : hostname not found",
    "fnxaccept2 : not INET or UNIX address",
    "net_addr : cant find temp name for bind",
    "net_addr : service is not existant",
    "net_addr : hostname not found",
    "net_addr : cant alloc memory",
    "net_addr : username not found",
    "fnxaccept2 : too many files",
    "fnxsetsyserror : cant alloc memory",
    "serverop : cant alloc memory",
    "serverop : getsockname failed",
    "fnxaccept2 : username not found",
    "serverreg : username not equal to result of U8GetUser",
    "serverecri : serverinfo already allocated",
    "nx_lance_rpc_port : daemon not thrown",
    "nx_lance_rpc_port : hostname not found",
    "fnx_find_port : hostname not found",
    "fnx_find_port : rpc server not found",
    "clientop : cant find server's port",
    "net_addr : /UNIX/ is reserved for server's side only",
    "fnx_find_port : UNIX socket asked on another machine",
    "clientop : serverinfo deja alloue",
    "nx_init_access : conversion not implemented",
    "conversion vms32-ie32 on ieee host : underflow",
    "conversion vms32-ie32 on cray host : underflow",
    "conversion ie32-vms32 on ieee host : overflow",
    "conversion ie32-vms32 on cray host : overflow",
    "type not implemented",
    "fnx_find_port : server not found"
};

/* ------------------------------------------------------------ */
/*                     FNX_NOPRINT_ERROR                        */
/* ------------------------------------------------------------ */

int
fnx_noprint_error(const int noprint)
{

    /* pas d'impression de message d'erreur nx
       si noprint different de 0. NXERRNO reste positionne.
       on retourne l'ancienne valeur de NXNOPRINT. */

	const int noprint_old = NXNOPRINT;

	NXNOPRINT = noprint;

	return noprint_old;
}

/* ------------------------------------------------------------ */
/*                     FNXSETSYSERROR                           */
/* ------------------------------------------------------------ */

void
fnxsetsyserror(const char* fcname)
{

	if(nxsys_err) free(nxsys_err);

	if(fcname && strlen(fcname))
	{
	    const size_t lfcname = strlen(fcname) + 1;

	    nxsys_err = (char*) malloc(lfcname);

	    if(!nxsys_err)
	    {
		NXERRNO = 61;
		return;
	    }

	    memcpy(nxsys_err,fcname,lfcname);
	}
	else
	    nxsys_err = NULL;

}

/* ------------------------------------------------------------ */
/*                     FNXPERROR                                */
/* ------------------------------------------------------------ */

static void
fnximperror(const char* fcname,const int err)
{
	static THREAD_LOCAL int nxerror_count = 0;

	if ((err >= 38) && (err <= 47))
	{
	    nxerror_count++;

	    if (nxerror_count > MAX_ERROR_CNT) return;

	}

	if(fcname && strlen(fcname))
	    fprintf(stderr,"%s : %s\n",fcname,nxerror_list[err]);
	else
	    fprintf(stderr,"%s\n",nxerror_list[err]);

}

void
fnxperror(const char* fcname)
{
	int err = NXERRNO;

	if(NXNOPRINT) return;

	while(err)
	{

	    if(err & 0x3ff)
	        fnximperror(fcname,err & 0x3ff);

	    err >>= 10;
	}

}

/* ------------------------------------------------------------ */
/*                     FNXSYSERROR                              */
/* ------------------------------------------------------------ */

void
fnxsyserror(const char* fcname)
{
	const int myerrno = errno;

   	if (myerrno > 0 && !NXNOPRINT)
	{

	    if(fcname && strlen(fcname))
	    {

		if(nxsys_err && strlen(nxsys_err))
		    fprintf(stderr,"%s : %s : %s\n",fcname,nxsys_err,
				    strerror(myerrno));
		else
		    fprintf(stderr,"%s : %s\n",fcname,strerror(myerrno));

	    }
	    else
	    {

		if(nxsys_err && strlen(nxsys_err))
		    fprintf(stderr,"%s : %s\n",nxsys_err,strerror(myerrno));
		else
		    fprintf(stderr,"%s\n",strerror(myerrno));

	    }

	}

	errno = 0;
	fnxsetsyserror(NULL);
}

/* ------------------------------------------------------------ */
/*                     NXPIOERROR                               */
/* ------------------------------------------------------------ */

int
nxpioerror(const char* fcname)
{
	const int myerrno = errno;

   	if (myerrno > 0 && !NXNOPRINT)
	{

	    if(fcname && strlen(fcname))
		fprintf(stderr,"%s : %s\n",fcname,strerror(myerrno));
	    else
		fprintf(stderr,"%s\n",strerror(myerrno));

	}

	errno = 0;

	return 0;
}
