/*                           N X - A M                                 */
/*                           ---------                                 */
/*                          C interface                                */

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.  
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/
/* source ident */
/***************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Copyr 1992 MA/AIM";
#endif

/* Modifs Pierre Brochard 1997/1998 */

/***************************************************************************/
/* includes */
/***************************************************************************/

#include "nxP.h"
#include "nxerror.h"
#include "nxfonc.h"

#define NXP_H_IMPLEMENT
#   include "nx.h"
#undef NXP_H_IMPLEMENT

#include "nxfrmt.h"

/* sources pour machines ieee 32/64 bits big/little endian :
   conversions vers et de machines ieee */

#include "nxconvP.h"

/* ============================================================ */
/*  protos pour fonctions statiques                             */
/* ============================================================ */

/* sur machines IEEE 32 bits big endian :
   int est ie32
   type est ie32 ou ie64
   float est ie32 = NXFLT
   double est ie64 = NXDBLE
   long double est ie128
   */

#if defined(NX_MACH_INT_IE64)
static void nx_int_ie64_from_ie32(FILE* fd,int nb,char* buff,int* rc);
static void nx_int_nblk_ie64_from_ie32(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_int_ie64_to_ie32(FILE* fd,int nb,const char* buff,int* rc);
static void nx_int_nblk_ie64_to_ie32(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
#endif

#if defined(NX_MACH_FLT_IE64)
static void nx_float_ie64_from_ie32(FILE* fd,int nb,char* buff,int* rc);
static void nx_float_nblk_ie64_from_ie32(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_float_ie64_to_ie32(FILE* fd,int nb,const char* buff,int* rc);
static void nx_float_nblk_ie64_to_ie32(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
#endif

#if defined(CRAY) && !defined(_CRAYMPP) && defined(_CRAYIEEE)
static void nx_type_from_ie32_ie64(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc);
static void nx_type_nblk_from_ie32_ie64(FILE* fd,char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);

static void nx_type_to_ie32_ie64(FILE* fd,int nb,const char* buff,
		const char* icod,const char* ocod,int* rc);
static void nx_type_nblk_to_ie32_ie64(FILE* fd,const char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);
#endif

#if defined(CRAY) && defined(_CRAYMPP)
static void nx_type_from_ie32_ie64(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc);
static void nx_type_nblk_from_ie32_ie64(FILE* fd,char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);

static void nx_type_to_ie32_ie64(FILE* fd,int nb,const char* buff,
		const char* icod,const char* ocod,int* rc);
static void nx_type_nblk_to_ie32_ie64(FILE* fd,const char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);

static void nx_type_from_ie64_ie128(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc);
static void nx_type_nblk_from_ie64_ie128(FILE* fd,char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);

static void nx_type_to_ie64_ie128(FILE* fd,int nb,const char* buff,
		const char* icod,const char* ocod,int* rc);
static void nx_type_nblk_to_ie64_ie128(FILE* fd,const char* buff,int nb,
		const char* icod,const char* ocod,int* nbecrits,int* rc);
#endif

#if defined(NX_MACH_INT_IE32)
static void nx_int_ie32_from_ie64(FILE* fd,int nb,char* buff,int* rc);
static void nx_int_nblk_ie32_from_ie64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_int_ie32_to_ie64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_int_nblk_ie32_to_ie64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
#endif

#if defined(NX_MACH_FLT_IE32)
static void nx_float_ie32_from_ie64(FILE* fd,int nb,char* buff,int* rc);
static void nx_float_nblk_ie32_from_ie64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_float_ie32_to_ie64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_float_nblk_ie32_to_ie64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
#endif

#if defined(IEEE) && !defined(NX_LITTLEENDIAN)
static void nx_type_from_ie64_ie128(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc);
static void nx_type_nblk_from_ie64_ie128(FILE* fd,char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);

static void nx_type_to_ie64_ie128(FILE* fd,int nb,const char* buff,
		const char* icod,const char* ocod,int* rc);
static void nx_type_nblk_to_ie64_ie128(FILE* fd,const char* buff,int nb,
		const char* icod,const char* ocod,int* nbecrits,int* rc);
#endif

#if defined(IEEE) && defined(NX_LITTLEENDIAN)
static void nx_int_li64_from_li32(FILE* fd,int nb,char* buff,int* rc);
static void nx_int_nblk_li64_from_li32(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_int_li64_to_li32(FILE* fd,int nb,const char* buff,int* rc);
static void nx_int_nblk_li64_to_li32(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);

static void nx_int_li32_from_li64(FILE* fd,int nb,char* buff,int* rc);
static void nx_int_nblk_li32_from_li64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_int_li32_to_li64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_int_nblk_li32_to_li64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);

static void nx_float_li64_from_li32(FILE* fd,int nb,char* buff,int* rc);
static void nx_float_nblk_li64_from_li32(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_float_li64_to_li32(FILE* fd,int nb,const char* buff,int* rc);
static void nx_float_nblk_li64_to_li32(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);

static void nx_float_li32_from_li64(FILE* fd,int nb,char* buff,int* rc);
static void nx_float_nblk_li32_from_li64(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_float_li32_to_li64(FILE* fd,int nb,const char* buff,int* rc);
static void nx_float_nblk_li32_to_li64(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);

static void nx_type_from_li64_li128(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc);
static void nx_type_nblk_from_li64_li128(FILE* fd,char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);

static void nx_type_to_li64_li128(FILE* fd,int nb,const char* buff,
		const char* icod,const char* ocod,int* rc);
static void nx_type_nblk_to_li64_li128(FILE* fd,const char* buff,int nb,
		const char* icod,const char* ocod,int* nbecrits,int* rc);
#endif

static void r_type_samehost(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc);
static void r_type_nblk_samehost(FILE* fd,char* buff,int nb,const char* icod,
		const char* ocod,int* nblus,int* rc);
static void w_type_samehost(FILE* fd,int nb,const char* buff,const char* icod,
		const char* ocod,int* rc);
static void w_type_nblk_samehost(FILE* fd,const char* buff,int nb,
		const char* icod,const char* ocod,int* nbecrits,int* rc);

/* ------------------------------------------------------------- */
/*  init_access for big endian ieee 32/64 bits machines */
/*  on lit au format big endian ieee 32/64 bits */
/*  et on converti au format ieee 32/64 bits big endian */
/* ------------------------------------------------------------- */

#if defined(CRAY) && !defined(_CRAYMPP) && defined(_CRAYIEEE)
const ACCESS*
nx_init_access_cray_ie32(void)
{

	static ACCESS my_access =
	{
	    NULL,
	    nx_int_ie64_from_ie32,
	    nx_int_ie64_from_ie32,
	    nx_int_ie64_from_ie32,
	    nx_float_ie64_from_ie32,
	    nx_float_ie64_from_ie32,
	    nx_type_from_ie32_ie64,
	    nx_int_nblk_ie64_from_ie32,
	    nx_int_nblk_ie64_from_ie32,
	    nx_int_nblk_ie64_from_ie32,
	    nx_float_nblk_ie64_from_ie32,
	    nx_float_nblk_ie64_from_ie32,
	    nx_type_nblk_from_ie32_ie64,
	    NULL,
	    nx_int_ie64_to_ie32,
	    nx_int_ie64_to_ie32,
	    nx_int_ie64_to_ie32,
	    nx_float_ie64_to_ie32,
	    nx_float_ie64_to_ie32,
	    nx_type_to_ie32_ie64, 
	    nx_int_nblk_ie64_to_ie32,
	    nx_int_nblk_ie64_to_ie32,
	    nx_int_nblk_ie64_to_ie32,
	    nx_float_nblk_ie64_to_ie32,
	    nx_float_nblk_ie64_to_ie32,
	    nx_type_nblk_to_ie32_ie64
	};

	return &my_access;
}
#endif

#if defined(CRAY) && defined(_CRAYMPP)
const ACCESS*
nx_init_access_mpp_ie32(void)
{

	static  ACCESS my_access =
	{
	    NULL,
	    nx_int_ie64_from_ie32,
	    nx_int_ie64_from_ie32,
	    nx_int_ie64_from_ie32,
	    NULL,
	    nx_float_ie64_from_ie32,
	    nx_type_from_ie32_ie64,
	    nx_int_nblk_ie64_from_ie32,
	    nx_int_nblk_ie64_from_ie32,
	    nx_int_nblk_ie64_from_ie32,
	    NULL,
	    nx_float_nblk_ie64_from_ie32,
	    nx_type_nblk_from_ie32_ie64,
	    NULL,
	    nx_int_ie64_to_ie32,
	    nx_int_ie64_to_ie32,
	    nx_int_ie64_to_ie32,
	    NULL,
	    nx_float_ie64_to_ie32,
	    nx_type_to_ie32_ie64, 
	    nx_int_nblk_ie64_to_ie32,
	    nx_int_nblk_ie64_to_ie32,
	    nx_int_nblk_ie64_to_ie32,
	    NULL,
	    nx_float_nblk_ie64_to_ie32,
	    nx_type_nblk_to_ie32_ie64
	};

	return &my_access;
}

const ACCESS*
nx_init_access_mpp_ie64(void)
{

	static  ACCESS my_access =
	{
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    nx_float_ie32_from_ie64,
	    NULL,
	    nx_type_from_ie64_ie128,
	    NULL,
	    NULL,
	    NULL,
	    nx_float_nblk_ie32_from_ie64,
	    NULL,
	    nx_type_nblk_from_ie64_ie128,
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    nx_float_ie32_to_ie64,
	    NULL,
	    nx_type_to_ie64_ie128,
	    NULL,
	    NULL,
	    NULL,
	    nx_float_nblk_ie32_to_ie64,
	    NULL,
	    nx_type_nblk_to_ie64_ie128
	};

	return &my_access;
}
#endif

#if defined(IEEE) && !defined(NX_LITTLEENDIAN)
const ACCESS*
nx_init_access_ieee_ie64(void)
{

	static  ACCESS my_access =
	{ 
	    NULL,
	    nx_int_ie32_from_ie64,
#ifdef NX_LONG_IS_64
	    NULL,
#else
	    nx_int_ie32_from_ie64,
#endif
	    NULL,
	    nx_float_ie32_from_ie64,
	    NULL,
	    nx_type_from_ie64_ie128,
	    nx_int_nblk_ie32_from_ie64,
#ifdef NX_LONG_IS_64
	    NULL,
#else
	    nx_int_nblk_ie32_from_ie64,
#endif
	    NULL,
	    nx_float_nblk_ie32_from_ie64,
	    NULL,
	    nx_type_nblk_from_ie64_ie128,
	    NULL,
	    nx_int_ie32_to_ie64,
#ifdef NX_LONG_IS_64
	    NULL,
#else
	    nx_int_ie32_to_ie64,
#endif
	    NULL,
	    nx_float_ie32_to_ie64,
	    NULL,
	    nx_type_to_ie64_ie128,
	    nx_int_nblk_ie32_to_ie64,
#ifdef NX_LONG_IS_64
	    NULL,
#else
	    nx_int_nblk_ie32_to_ie64,
#endif
	    NULL,
	    nx_float_nblk_ie32_to_ie64,
	    NULL,
	    nx_type_nblk_to_ie64_ie128
	} ;

	return &my_access;
}

const ACCESS*
nx_init_access_ieee_ie32(void)
{

	static  ACCESS my_access =
	{ 
	    NULL,
	    NULL,
#ifdef NX_LONG_IS_64
	    nx_int_ie64_from_ie32,
#else
	    NULL,
#endif
	    nx_int_ie64_from_ie32,
	    NULL,
	    nx_float_ie64_from_ie32,
	    r_type_samehost,
	    NULL,
#ifdef NX_LONG_IS_64
	    nx_int_nblk_ie64_from_ie32,
#else
	    NULL,
#endif
	    nx_int_nblk_ie64_from_ie32,
	    NULL,
	    nx_float_nblk_ie64_from_ie32,
	    r_type_nblk_samehost,
	    NULL,
	    NULL,
#ifdef NX_LONG_IS_64
	    nx_int_ie64_to_ie32,
#else
	    NULL,
#endif
	    nx_int_ie64_to_ie32,
	    NULL,
	    nx_float_ie64_to_ie32,
	    w_type_samehost,
	    NULL,
#ifdef NX_LONG_IS_64
	    nx_int_nblk_ie64_to_ie32,
#else
	    NULL,
#endif
	    nx_int_nblk_ie64_to_ie32,
	    NULL,
	    nx_float_nblk_ie64_to_ie32,
	    w_type_nblk_samehost,
	};

	return &my_access;
}
#endif

#if defined(IEEE) && defined(NX_LITTLEENDIAN) 
const ACCESS*
nx_init_access_little_li32(void)
{

	static  ACCESS my_access =
	{ 
	    NULL,
	    NULL,
#ifdef NX_LONG_IS_64
	    nx_int_li64_from_li32,
#else
	    NULL,
#endif
	    nx_int_li64_from_li32,
	    NULL,
	    nx_float_li64_from_li32,
	    r_type_samehost,
	    NULL,
#ifdef NX_LONG_IS_64
	    nx_int_nblk_li64_from_li32,
#else
	    NULL,
#endif
	    nx_int_nblk_li64_from_li32,
	    NULL,
	    nx_float_nblk_li64_from_li32,
	    r_type_nblk_samehost,
	    NULL,
	    NULL,
#ifdef NX_LONG_IS_64
	    nx_int_li64_to_li32,
#else
	    NULL,
#endif
	    nx_int_li64_to_li32,
	    NULL,
	    nx_float_li64_to_li32,
	    w_type_samehost,
	    NULL,
#ifdef NX_LONG_IS_64
	    nx_int_nblk_li64_to_li32,
#else
	    NULL,
#endif
	    nx_int_nblk_li64_to_li32,
	    NULL,
	    nx_float_nblk_li64_to_li32,
	    w_type_nblk_samehost,
	};

	return &my_access;
}

const ACCESS*
nx_init_access_little_li64(void)
{

	static  ACCESS my_access =
	{ 
	    NULL,
	    nx_int_li32_from_li64,
#ifdef NX_LONG_IS_64
	    NULL,
#else
	    nx_int_li32_from_li64,
#endif
	    NULL,
	    nx_float_li32_from_li64,
	    NULL,
	    nx_type_from_li64_li128,
	    nx_int_nblk_li32_from_li64,
#ifdef NX_LONG_IS_64
	    NULL,
#else
	    nx_int_nblk_li32_from_li64,
#endif
	    NULL,
	    nx_float_nblk_li32_from_li64,
	    NULL,
	    nx_type_nblk_from_li64_li128,
	    NULL,
	    nx_int_li32_to_li64,
#ifdef NX_LONG_IS_64
	    NULL,
#else
	    nx_int_li32_to_li64,
#endif
	    NULL,
	    nx_float_li32_to_li64,
	    NULL,
	    nx_type_to_li64_li128,
	    nx_int_nblk_li32_to_li64,
#ifdef NX_LONG_IS_64
	    NULL,
#else
	    nx_int_nblk_li32_to_li64,
#endif
	    NULL,
	    nx_float_nblk_li32_to_li64,
	    NULL,
	    nx_type_nblk_to_li64_li128,
	};

	return &my_access;
}
#endif

const ACCESS*
nx_init_access_same(void)
{

	static ACCESS my_access =
	{ 
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    r_type_samehost,
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    r_type_nblk_samehost,
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    w_type_samehost,
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    NULL,
	    w_type_nblk_samehost
	};

	return &my_access;
}

/*               Specific access functions                    */
/*               -------------------------                    */

/* ----------------------------------------------------------------------- */
/* 		          CRAY - IEEE                                      */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/* 		====>      NX_INT_IE64_FROM_IE32    <====                  */
/* ----------------------------------------------------------------------- */

#if defined(NX_MACH_INT_IE64)
static void
nx_int_ie64_from_ie32(FILE* fd,int nb,char* buff,int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_int_ie32_to_cr64(buff2,buff,nbt),
			"nx_int_ie64_from_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>      NX_INT_NBLK_IE64_FROM_IE32    <====             */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_ie64_from_ie32(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_int_ie32_to_cr64(buff2,buff,n),
			nblus,"nx_int_nblk_ie64_from_ie32",rc);
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_IE64_FROM_IE32    <====                  */
/* ----------------------------------------------------------------------- */

#if defined(NX_MACH_FLT_IE64)
static void
nx_float_ie64_from_ie32(FILE* fd,int nb,char* buff,int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_real_ie32_to_ie64(buff2,buff,nbt),
			"nx_float_ie64_from_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_IE64_FROM_IE32    <====             */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_ie64_from_ie32(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_real_ie32_to_ie64(buff2,buff,n),
			nblus,"nx_float_nblk_ie64_from_ie32",rc);
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_FROM_IE32_IE64    <====                   */
/* ----------------------------------------------------------------------- */

#if defined(CRAY) && !defined(_CRAYMPP) && defined(_CRAYIEEE)
static void
nx_type_from_ie32_ie64(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float cray  */
    /* conversion float ieee - double cray  */
    /* conversion double ieee - float cray */
    /* conversion double ieee - double cray */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_from_ie32_ie64"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,((nb+1)/2)*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_32,
					nxconv_real_ie32_to_ie64(buff2,buff,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,((nb+1)/2)*NX_SIZE_FLT);
		        NX_READBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_32,nxconv_real_ie32_to_ie128(
							buff2,buff,nbt),
					NXERRMES,rc),
			NX_READBUF(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					nb,;,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_ie64_to_ie128(
							buff2,buff,nbt),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_FROM_IE32_IE64    <====              */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_from_ie32_ie64(FILE* fd,char* buff,int nb,const char* icod,
		const char* ocod,int* nblus,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float cray  */
    /* conversion float ieee - double cray  */
    /* conversion double ieee - float cray */
    /* conversion double ieee - double cray */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_from_ie32_ie64"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,((nb+1)/2)*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_32,
					nxconv_real_ie32_to_ie64(buff2,buff,n),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,((nb+1)/2)*NX_SIZE_FLT);
		        NX_READBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_32,nxconv_real_ie32_to_ie128(
							buff2,buff,n),
					nblus,NXERRMES,rc),
			NX_READBUF_NBLK(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					nb,;,nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_ie64_to_ie128(
							buff2,buff,n),
					nblus,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_IE64_TO_IE32                               */
/* ----------------------------------------------------------------------- */

#if defined(NX_MACH_INT_IE64)
static void
nx_int_ie64_to_ie32(FILE* fd,int nb,const char* buff,int* rc)
{
	int cr = 0;
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_int_cr64_to_ie32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_int_ie64_to_ie32"),
			"nx_int_ie64_to_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_IE64_TO_IE32 <====                    */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_ie64_to_ie32(FILE* fd,const char* buff,int nb,int* nbecrits,int* rc)
{
	int cr = 0;
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_int_cr64_to_ie32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_int_nblk_ie64_to_ie32"),
			nbecrits,"nx_int_nblk_ie64_to_ie32",rc);
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_IE64_TO_IE32                             */
/* ----------------------------------------------------------------------- */

#if defined(NX_MACH_FLT_IE64)
static void
nx_float_ie64_to_ie32(FILE* fd,int nb,const char* buff,int* rc)
{
	int cr = 0;
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_real_ie64_to_ie32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_float_ie64_to_ie32"),
			"nx_float_ie64_to_ie32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_IE64_TO_IE32   <====                */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_ie64_to_ie32(FILE* fd,const char* buff,int nb,int* nbecrits,
		int* rc)
{
	int cr = 0;
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_real_ie64_to_ie32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_float_nblk_ie64_to_ie32"),
			nbecrits,"nx_float_nblk_ie64_to_ie32",rc);
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_TO_IE32_IE64   <====                      */
/* ----------------------------------------------------------------------- */

#if defined(CRAY) && !defined(_CRAYMPP) && defined(_CRAYIEEE)
static void
nx_type_to_ie32_ie64(FILE* fd,int nb,const char* buff,const char* icod,
		const char* ocod,int* rc)
{
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float cray - float ieee */
    /* conversion double cray - float ieee */
    /* conversion float cray - double ieee */
    /* conversion double cray - double ieee */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_to_ie32_ie64"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,((nb+1)/2)*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_32,
					nxconv_real_ie64_to_ie32(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,((nb+1)/2)*NX_SIZE_FLT);
		        NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_32,nxconv_real_ie128_to_ie32(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_WRITEBUF(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					nb,;,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_ie128_to_ie64(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_TO_IE32_IE64   <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_to_ie32_ie64(FILE* fd,const char* buff,int nb,const char* icod,
		const char* ocod,int* nbecrits,int* rc)
{
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float cray - float ieee */
    /* conversion double cray - float ieee */
    /* conversion float cray - double ieee */
    /* conversion double cray - double ieee */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_to_ie32_ie64"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,((nb+1)/2)*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_32,
					nxconv_real_ie64_to_ie32(buff,buff2,nbt,
							&cr);
					if(cr) fnxperror(NXERRMES),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,((nb+1)/2)*NX_SIZE_FLT);
		        NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_32,
					nxconv_real_ie128_to_ie32(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,
					memcpy(buff2,buff,nbt*NX_SIZE_64),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_64,
					nxconv_real_ie128_to_ie64(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					nbecrits,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 	        	    MPP - IEEE                                     */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_FROM_IE32_IE64    <====                   */
/* ----------------------------------------------------------------------- */

#if defined(CRAY) && defined(_CRAYMPP)
static void
nx_type_from_ie32_ie64(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre ieee - mpp :
       On lit en IEEE et on convertit en MPP
       <=> icod concerne buff donc MPP et ocod concerne buff2 donc IEEE
       pour icod SP = DP, pour ocod CP = SP
       buff est le buffer de donnees au format local machines mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_from_ie32_ie64"

	NXCONV_TYPE2(icod,ocod,nct);
	NXCONV_CONVERT2(nct,
			NX_READBUF(fd,buff,NX_SIZE_32,nb,buff,NX_SIZE_32,
					nb,;,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,((nb+1)/2)*NX_SIZE_FLT);
		        NX_READBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,
					NX_BUF_SIZE_32,nxconv_real_ie32_to_ie64(
							buff2,buff,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_64,nxconv_real_ie64_to_ie32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_READBUF(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_64,
					nb,;,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_FROM_IE32_IE64    <====              */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_from_ie32_ie64(FILE* fd,char* buff,int nb,const char* icod,
		const char* ocod,int* nblus,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre ieee - mpp :
       On lit en IEEE et on convertit en MPP
       <=> icod concerne buff donc MPP et ocod concerne buff2 donc IEEE
       pour icod SP = DP, pour ocod CP = SP
       buff est le buffer de donnees au format local machines mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_from_ie32_ie64"

	NXCONV_TYPE2(icod,ocod,nct);
	NXCONV_CONVERT2(nct,
			NX_READBUF_NBLK(fd,buff,NX_SIZE_32,nb,buff,NX_SIZE_32,
					nb,;,nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,((nb+1)/2)*NX_SIZE_FLT);
		        NX_READBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,
					NX_BUF_SIZE_32,nxconv_real_ie32_to_ie64(
							buff2,buff,n),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_64,nxconv_real_ie64_to_ie32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_READBUF_NBLK(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_64,
					nb,;,nblus,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_TO_IE32_IE64   <====                      */
/* ----------------------------------------------------------------------- */

static void
nx_type_to_ie32_ie64(FILE* fd,int nb,const char* buff,const char* icod,
		const char* ocod,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre ieee - mpp :
       On convertit du MPP et on ecrit en IEEE
       <=> icod concerne buff donc MPP et ocod concerne buff2 donc IEEE
       pour icod SP = DP, pour ocod CP = SP
       buff est le buffer de donnees au format local machines mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_to_ie32_ie64"

	NXCONV_TYPE2(icod,ocod,nct);
	NXCONV_CONVERT2(nct,
			NX_WRITEBUF(fd,buff,NX_SIZE_32,nb,buff,NX_SIZE_32,
					nb,;,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,((nb+1)/2)*NX_SIZE_FLT);
		        NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,
					NX_BUF_SIZE_32,nxconv_real_ie64_to_ie32(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_64,nxconv_real_ie32_to_ie64(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_WRITEBUF(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_64,
					nb,;,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_TO_IE32_IE64   <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_to_ie32_ie64(FILE* fd,const char* buff,int nb,const char* icod,
		const char* ocod,int* nbecrits,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre ieee - mpp :
       On convertit du MPP et on ecrit en IEEE
       <=> icod concerne buff donc MPP et ocod concerne buff2 donc IEEE
       pour icod SP = DP, pour ocod CP = SP
       buff est le buffer de donnees au format local machines mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_to_ie32_ie64"

	NXCONV_TYPE2(icod,ocod,nct);
	NXCONV_CONVERT2(nct,
			NX_ALLOCBUF(buff2,buff3,((nb+1)/2)*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff,NX_SIZE_32,nb,buff,NX_SIZE_32,
					nb,;,nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,((nb+1)/2)*NX_SIZE_FLT);
		        NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,
					NX_BUF_SIZE_32,nxconv_real_ie64_to_ie32(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_64,nxconv_real_ie32_to_ie64(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_64,
					nb,;,nbecrits,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 	        	    MPP - MPP                                      */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/* 		====>    nx_type_from_ie64_ie128     <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_type_from_ie64_ie128(FILE* fd, int nb, char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre ieee - mpp :
       On lit en IEEE 64 bits ou 128 et on convertit en MPP
       <=> icod concerne buff donc MPP et ocod concerne buff2 donc IEEE
       pour icod SP = DP, pour ocod CP = SP
       buff est le buffer de donnees au format local machines mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_from_ie64_ie128"

	NXCONV_TYPE2(icod,ocod,nct);
	NXCONV_CONVERT2(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_64,nxconv_real_ie64_to_ie32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
		        NX_READBUF(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_64,
					nb,;,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_128,
					nxconv_real_ie128_to_ie32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_64,
					NX_BUF_SIZE_128,
					nxconv_real_ie128_to_ie64(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    nx_type_nblk_from_ie64_ie128     <====            */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_from_ie64_ie128(FILE* fd,char* buff,int nb,const char* icod,
		const char* ocod,int* nblus,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre ieee - mpp :
       On lit en IEEE 64 ou 128 bits et on convertit en MPP
       <=> icod concerne buff donc MPP et ocod concerne buff2 donc IEEE
       pour icod SP = DP, pour ocod CP = SP
       buff est le buffer de donnees au format local machines mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_from_ie64_ie128"

	NXCONV_TYPE2(icod,ocod,nct);
	NXCONV_CONVERT2(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_64,nxconv_real_ie64_to_ie32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
		        NX_READBUF_NBLK(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_64,
					nb,;,nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_128,
					nxconv_real_ie128_to_ie32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_64,
					NX_BUF_SIZE_128,
					nxconv_real_ie128_to_ie64(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_TYPE_TO_IE64_IE128    <====                   */
/* ----------------------------------------------------------------------- */

static void
nx_type_to_ie64_ie128(FILE* fd, int nb,const char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre ieee - mpp :
       On convertit du MPP et on ecrit en IEEE 64 ou 128 bits
       <=> icod concerne buff donc MPP et ocod concerne buff2 donc IEEE
       pour icod SP = DP, pour ocod CP = SP
       buff est le buffer de donnees au format local machines mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_to_ie64_ie128"

	NXCONV_TYPE2(icod,ocod,nct);
	NXCONV_CONVERT2(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_64,nxconv_real_ie32_to_ie64(
							buff,buff2,nbt),
					NXERRMES,rc),
		        NX_WRITEBUF(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_64,
					nb,;,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_128,
					nxconv_real_ie32_to_ie128(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_64,
					NX_BUF_SIZE_128,
					nxconv_real_ie64_to_ie128(
							buff,buff2,nbt),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_TYPE_NBLK_TO_IE64_IE128    <====              */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_to_ie64_ie128(FILE* fd,const char* buff,int nb,const char* icod,
		const char* ocod,int* nbecrits,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre ieee - mpp :
       On convertit du MPP et on ecrit en IEEE 64 ou 128 bits
       <=> icod concerne buff donc MPP et ocod concerne buff2 donc IEEE
       pour icod SP = DP, pour ocod CP = SP
       buff est le buffer de donnees au format local machines mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_to_ie64_ie128"

	NXCONV_TYPE2(icod,ocod,nct);
	NXCONV_CONVERT2(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_64,nxconv_real_ie32_to_ie64(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
		        NX_WRITEBUF_NBLK(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_64,
					nb,;,nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_32,
					NX_BUF_SIZE_128,
					nxconv_real_ie32_to_ie128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_64,
					NX_BUF_SIZE_128,
					nxconv_real_ie64_to_ie128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		 ====>    IEEE      MPP    <====                           */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_INT_IE32_FROM_IE64  <====                     */
/* ----------------------------------------------------------------------- */

#if defined(NX_MACH_INT_IE32)
static void
nx_int_ie32_from_ie64(FILE* fd, int nb, char* buff, int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_cr64_to_ie32(buff2,buff,nbt,&cr);
			if (cr) fnxperror("nx_int_ie32_from_ie64"),
			"nx_int_ie32_from_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_IE32_FROM_IE64  <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_ie32_from_ie64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_cr64_to_ie32(buff2,buff,n,&cr);
			if (cr) fnxperror("nx_int_nblk_ie32_from_ie64"),
			nblus,"nx_int_nblk_ie32_from_ie64",rc);
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_FLOAT_IE32_FROM_IE64  <====                   */
/* ----------------------------------------------------------------------- */

#if defined(NX_MACH_FLT_IE32)
static void
nx_float_ie32_from_ie64(FILE* fd, int nb, char* buff, int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_ie64_to_ie32(buff2,buff,nbt,&cr);
			if(cr) fnxperror("nx_float_ie32_from_ie64"),
			"nx_float_ie32_from_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_IE32_FROM_IE64  <====               */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_ie32_from_ie64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_ie64_to_ie32(buff2,buff,n,&cr);
			if(cr) fnxperror("nx_float_nblk_ie32_from_ie64"),
			nblus,"nx_float_nblk_ie32_from_ie64",rc);
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_FROM_IE64_IE128   <====                   */
/* ----------------------------------------------------------------------- */

#if defined(IEEE) && !defined(NX_LITTLEENDIAN)
static void
nx_type_from_ie64_ie128(FILE* fd, int nb, char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float mpp - float ieee */
    /* conversion float mpp - double ieee */
    /* conversion double mpp - float ieee */
    /* conversion double mpp - double ieee */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_from_ie64_ie128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_ie64_to_ie32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
		        NX_READBUF(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					nb,;,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_ie128_to_ie32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_128,
					nxconv_real_ie128_to_ie64(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_FROM_IE64_IE128   <====              */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_from_ie64_ie128(FILE* fd, char* buff,int nb, const char* icod,
		const char* ocod,int* nblus,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float mpp - float ieee */
    /* conversion float mpp - double ieee */
    /* conversion double mpp - float ieee */
    /* conversion double mpp - double ieee */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_from_ie64_ie128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_ie64_to_ie32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
		        NX_READBUF_NBLK(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					nb,;,nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_ie128_to_ie32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_128,
					nxconv_real_ie128_to_ie64(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_INT_IE32_TO_IE64    <====                   */
/* ----------------------------------------------------------------------- */

#if defined(NX_MACH_INT_IE32)
static void 
nx_int_ie32_to_ie64(FILE* fd, int nb,const char* buff, int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_ie32_to_cr64(buff,buff2,nbt),
			"nx_int_ie32_to_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_IE32_TO_IE64  <=====                  */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_ie32_to_ie64(FILE* fd,const char* buff,int nb,int* nbecrits,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_ie32_to_cr64(buff,buff2,nbt),
			nbecrits,"nx_int_nblk_ie32_to_ie64",rc);
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_FLOAT_IE32_TO_IE64    <====                 */
/* ----------------------------------------------------------------------- */

#if defined(NX_MACH_FLT_IE32)
static void 
nx_float_ie32_to_ie64(FILE* fd,int nb,const char* buff,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_ie32_to_ie64(buff,buff2,nbt),
			"nx_float_ie32_to_ie64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_IE32_TO_IE64  <=====                */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_ie32_to_ie64(FILE* fd,const char* buff,int nb,int* nbecrits,
		int* rc)
{
	const size_t u = 2*nb*NX_SIZE_FLT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_ie32_to_ie64(buff,buff2,nbt),
			nbecrits,"nx_float_nblk_ie32_to_ie64",rc);
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_TO_IE64_IE128   <====                     */
/* ----------------------------------------------------------------------- */

#if defined(IEEE) && !defined(NX_LITTLEENDIAN)
static void
nx_type_to_ie64_ie128(FILE* fd, int nb,const char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float mpp */
    /* conversion double ieee - float mpp */
    /* conversion float ieee - double mpp */
    /* conversion double ieee - double mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_to_ie64_ie128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_ie32_to_ie64(
							buff,buff2,nbt),
					NXERRMES,rc),
		        NX_WRITEBUF(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					nb,;,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_ie32_to_ie128(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_128,
					nxconv_real_ie64_to_ie128(
							buff,buff2,nbt),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_TO_IE64_IE128   <====                */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_to_ie64_ie128(FILE* fd,const char* buff,int nb, const char* icod,
		const char* ocod,int* nbecrits,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float mpp */
    /* conversion double ieee - float mpp */
    /* conversion float ieee - double mpp */
    /* conversion double ieee - double mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_to_ie64_ie128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_ie32_to_ie64(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
		        NX_WRITEBUF_NBLK(fd,buff,NX_SIZE_64,nb,buff,
					NX_SIZE_DBLE,nb,;,nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_FLT,NX_BUF_SIZE_128,
					nxconv_real_ie32_to_ie128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_128,
					nxconv_real_ie64_to_ie128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}
#endif

/* ----------------------------------------------------------------------- */
/* fonctions valables seulement si sizeof(double precision) =              */
/* 2 * sizeof(simple)                                                      */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/* 		====>    R_TYPE_SAMEHOST   <====                           */
/* ----------------------------------------------------------------------- */

static void
r_type_samehost(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float - float */
    /* conversion float - double */
    /* conversion double - float */
    /* conversion double - double */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "r_type_samehost"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_READBUF(fd,buff,NX_SIZE_FLT,nb,buff,NX_SIZE_FLT,
					nb,;,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_READBUF(fd,buff2,NX_SIZE_FLT,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_FLT,
					nxconv_flt_dble(buff2,buff,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_DBLE,nb,buff,
					NX_SIZE_FLT,NX_BUF_SIZE_DBLE,
					nxconv_dble_flt(buff2,buff,nbt),
					NXERRMES,rc),
			NX_READBUF(fd,buff,NX_SIZE_DBLE,nb,buff,NX_SIZE_DBLE,
					nb,;,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    R_TYPE_NBLK_SAMEHOST   <====                      */
/* ----------------------------------------------------------------------- */

static void
r_type_nblk_samehost(FILE* fd,char* buff,int nb,const char* icod,
		const char* ocod,int* nblus,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float - float */
    /* conversion float - double */
    /* conversion double - float */
    /* conversion double - double */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "r_type_nblk_samehost"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_READBUF_NBLK(fd,buff,NX_SIZE_FLT,nb,buff,NX_SIZE_FLT,
					nb,;,nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_READBUF_NBLK(fd,buff2,NX_SIZE_FLT,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_FLT,
					nxconv_flt_dble(buff2,buff,n),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_DBLE,nb,buff,
					NX_SIZE_FLT,NX_BUF_SIZE_DBLE,
					nxconv_dble_flt(buff2,buff,n),
					nblus,NXERRMES,rc),
			NX_READBUF_NBLK(fd,buff,NX_SIZE_DBLE,nb,buff,
					NX_SIZE_DBLE,nb,;,nblus,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    W_TYPE_SAMEHOST   <====                           */
/* ----------------------------------------------------------------------- */

static void
w_type_samehost(FILE* fd,int nb,const char* buff,const char* icod,
		const char* ocod,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float cray  */
    /* conversion double ieee - float cray */
    /* conversion float ieee - double cray */
    /* conversion double ieee - double cray */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "w_type_samehost"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_WRITEBUF(fd,buff,NX_SIZE_FLT,nb,buff,NX_SIZE_FLT,
					nb,;,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_WRITEBUF(fd,buff2,NX_SIZE_FLT,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_FLT,nxconv_dble_flt(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_DBLE,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_DBLE,nxconv_flt_dble(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_WRITEBUF(fd,buff,NX_SIZE_DBLE,nb,buff,NX_SIZE_DBLE,
					nb,;,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    W_TYPE_NBLK_SAMEHOST   <====                      */
/* ----------------------------------------------------------------------- */

static void
w_type_nblk_samehost(FILE* fd,const char* buff,int nb,const char* icod,
		const char* ocod,int* nbecrits,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float cray  */
    /* conversion double ieee - float cray */
    /* conversion float ieee - double cray */
    /* conversion double ieee - double cray */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "w_type_nblk_samehost"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_WRITEBUF_NBLK(fd,buff,NX_SIZE_FLT,nb,buff,NX_SIZE_FLT,
					nb,;,nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_FLT,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_FLT,
					nxconv_dble_flt(buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_DBLE,nb,buff,
					NX_SIZE_FLT,NX_BUF_SIZE_DBLE,
					nxconv_flt_dble(buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_WRITEBUF_NBLK(fd,buff,NX_SIZE_DBLE,nb,buff,
					NX_SIZE_DBLE,nb,;,nbecrits,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		         LITTLEENDIAN      LI32/64                         */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_INT_LI64_FROM_LI32  <====                     */
/* ----------------------------------------------------------------------- */

#if defined(IEEE) && defined(NX_LITTLEENDIAN)
static void
nx_int_li64_from_li32(FILE* fd,int nb,char* buff,int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_int_li32_to_cr64(buff2,buff,nbt);
			nxconv_little_big64(buff,buff,nbt),
			"nx_int_li64_from_li32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_INT_LI32_FROM_LI64  <====                     */
/* ----------------------------------------------------------------------- */

static void
nx_int_li32_from_li64(FILE* fd, int nb, char* buff, int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_li64_to_ie32(buff2,buff,nbt,&cr);
			if (cr) fnxperror("nx_int_li32_from_li64");
			nxconv_little_big32(buff,buff,nbt),
			"nx_int_li32_from_li64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>      NX_INT_NBLK_LI64_FROM_LI32    <====             */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li64_from_li32(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_int_li32_to_cr64(buff2,buff,n);
			nxconv_little_big64(buff,buff,n),
			nblus,"nx_int_nblk_li64_from_li32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI32_FROM_LI64  <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li32_from_li64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_li64_to_ie32(buff2,buff,n,&cr);
			if (cr) fnxperror("nx_int_nblk_li32_from_li64");
			nxconv_little_big32(buff,buff,n),
			nblus,"nx_int_nblk_li32_from_li64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_FLOAT_LI64_FROM_LI32  <====                   */
/* ----------------------------------------------------------------------- */

static void
nx_float_li64_from_li32(FILE* fd, int nb, char* buff, int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_real_li32_to_li64(buff2,buff,nbt),
			"nx_float_li64_from_li32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		 ====>    NX_FLOAT_LI32_FROM_LI64  <====                   */
/* ----------------------------------------------------------------------- */

static void
nx_float_li32_from_li64(FILE* fd, int nb, char* buff, int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_li64_to_li32(buff2,buff,nbt,&cr);
			if(cr) fnxperror("nx_float_li32_from_li64"),
			"nx_float_li32_from_li64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_LI64_FROM_LI32    <====             */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li64_from_li32(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_real_li32_to_li64(buff2,buff,n),
			nblus,"nx_float_nblk_li64_from_li32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_LI32_FROM_LI64  <====               */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li32_from_li64(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_li64_to_li32(buff2,buff,n,&cr);
			if(cr) fnxperror("nx_float_nblk_li32_from_li64"),
			nblus,"nx_float_nblk_li32_from_li64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_INT_LI64_TO_LI32    <====                   */
/* ----------------------------------------------------------------------- */

static void 
nx_int_li64_to_li32(FILE* fd, int nb,const char* buff, int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_int_li64_to_ie32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_int_li64_to_li32");
			nxconv_little_big32(buff2,buff2,nbt),
			"nx_int_li64_to_li32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_INT_LI32_TO_LI64    <====                   */
/* ----------------------------------------------------------------------- */

static void 
nx_int_li32_to_li64(FILE* fd, int nb,const char* buff, int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_li32_to_cr64(buff,buff2,nbt);
			nxconv_little_big64(buff2,buff2,nbt),
			"nx_int_li32_to_li64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI64_TO_LI32 <====                    */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li64_to_li32(FILE* fd,const char* buff,int nb,int* nbecrits,int* rc)
{
	int cr = 0;
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_int_li64_to_ie32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_int_nblk_li64_to_li32");
			nxconv_little_big32(buff2,buff2,nbt),
			nbecrits,"nx_int_nblk_li64_to_li32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_LI32_TO_LI64  <=====                  */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_li32_to_li64(FILE* fd,const char* buff,int nb,int* nbecrits,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_INT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_64,
			nxconv_int_li32_to_cr64(buff,buff2,nbt);
			nxconv_little_big64(buff2,buff2,nbt),
			nbecrits,"nx_int_nblk_li32_to_li64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_FLOAT_LI64_TO_LI32    <====                 */
/* ----------------------------------------------------------------------- */

static void 
nx_float_li64_to_li32(FILE* fd, int nb,const char* buff, int* rc)
{
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_real_li64_to_li32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_float_li64_to_li32"),
			"nx_float_li64_to_li32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		   ====>    NX_FLOAT_LI32_TO_LI64    <====                 */
/* ----------------------------------------------------------------------- */

static void 
nx_float_li32_to_li64(FILE* fd,int nb,const char* buff,int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_li32_to_li64(buff,buff2,nbt),
			"nx_float_li32_to_li64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_LI64_TO_LI32   <====                */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li64_to_li32(FILE* fd,const char* buff,int nb,int* nbecrits,
		int* rc)
{
	int cr = 0;
	const size_t u = ((nb+1)/2)*NX_SIZE_64;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_64,NX_BUF_SIZE_32,
			nxconv_real_li64_to_li32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_float_nblk_li64_to_li32"),
			nbecrits,"nx_float_nblk_li64_to_li32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_LI32_TO_LI64  <=====                */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_li32_to_li64(FILE* fd,const char* buff,int nb,int* nbecrits,
		int* rc)
{
	const size_t u = 2*nb*NX_SIZE_32;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_64,
			nxconv_real_li32_to_li64(buff,buff2,nbt),
			nbecrits,"nx_float_nblk_li32_to_li64",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_FROM_LI64_LI128   <====                   */
/* ----------------------------------------------------------------------- */

static void
nx_type_from_li64_li128(FILE* fd, int nb, char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float mpp - float ieee */
    /* conversion float mpp - double ieee */
    /* conversion double mpp - float ieee */
    /* conversion double mpp - double ieee */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_from_li64_li128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_li64_to_li32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
		        NX_READBUF(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					nb,;,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_li128_to_li32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_128,
					nxconv_real_li128_to_li64(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_FROM_LI64_LI128   <====              */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_from_li64_li128(FILE* fd, char* buff,int nb, const char* icod,
		const char* ocod,int* nblus,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float mpp - float ieee */
    /* conversion float mpp - double ieee */
    /* conversion double mpp - float ieee */
    /* conversion double mpp - double ieee */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_from_li64_li128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_li64_to_li32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
		        NX_READBUF_NBLK(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					nb,;,nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_li128_to_li32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_128,
					nxconv_real_li128_to_li64(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_TO_LI64_LI128   <====                     */
/* ----------------------------------------------------------------------- */

static void
nx_type_to_li64_li128(FILE* fd, int nb,const char* buff,const char* icod,
		const char* ocod, int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float mpp */
    /* conversion double ieee - float mpp */
    /* conversion float ieee - double mpp */
    /* conversion double ieee - double mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_to_li64_li128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_li32_to_li64(
							buff,buff2,nbt),
					NXERRMES,rc),
		        NX_WRITEBUF(fd,buff,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					nb,;,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_128,
					nxconv_real_li32_to_li128(
							buff,buff2,nbt),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_128,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_128,
					nxconv_real_li64_to_li128(
							buff,buff2,nbt),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_TO_LI64_LI128   <====                */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_to_li64_li128(FILE* fd,const char* buff,int nb, const char* icod,
		const char* ocod,int* nbecrits,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float mpp */
    /* conversion double ieee - float mpp */
    /* conversion float ieee - double mpp */
    /* conversion double ieee - double mpp */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_to_li64_li128"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_li32_to_li64(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
		        NX_WRITEBUF_NBLK(fd,buff,NX_SIZE_64,nb,buff,
					NX_SIZE_DBLE,nb,;,nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_FLT,NX_BUF_SIZE_128,
					nxconv_real_li32_to_li128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,4*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_128,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_128,
					nxconv_real_li64_to_li128(
							buff,buff2,nbt),
					nbecrits,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}
#endif
