// *******************************************************************
//	Implementation de la librairie IHM (en motif)
// *******************************************************************

struct SccsIdU8Serveur
{
	static const char sccs_id[];
};

const char SccsIdU8Serveur::sccs_id[] = "%W% %G% Pierre Brochard";

// *******************************************************************
//	Implementation de la librairie IHM (en motif) declarations
// *******************************************************************

#include "U8machcomp.h"
#include "nxstream/U8serveur.h"
#include "nxstream.h"
#include "U8constant.h"

#if defined(__GNUC__)
#  include <sys/socket.h>
#endif

#include "U8signal.h"

// memmove

#if defined(SUNOS) && defined(SUNOS_4)
#   define memmove(a,b,c) bcopy((b),(a),(c))
#endif

// TRUE

#ifndef TRUE
#  define TRUE 1
#endif

#ifndef FALSE
#  define FALSE 0
#endif

// *************************************************
//      Initialisation des variables de classe
// *************************************************

int           ElementBase::poulpe_flag_sig_usr2 = 0;
int	      ElementBase::_mask = 0;

int           ElementInput::_nb_fd = 0;
nxstream**    ElementInput::_fd_tab = NULL;
U8Pointer*    ElementInput::_class_tab = NULL;
fd_set        ElementInput::_readfds;
fd_set        ElementInput::_writefds;

// *************************************************
//       base generale
// *************************************************

ElementBase::ElementBase() 
{
}

ElementBase::~ElementBase()
{
}

ElementInput*
ElementBase::ElInput()
{
	cerr << "Pb " << __FILE__ << " ligne " << __LINE__ << endl;

	return NULL;
}

ElementVisuel*
ElementBase::ElVisuel()
{
	cerr << "Pb " << __FILE__ << " ligne " << __LINE__ << endl;

	return NULL;
}

// *************************************************
//       gestion des fichiers (socket)
// *************************************************

ElementInput::ElementInput() : ElementBase()
{
}

ElementInput::~ElementInput()
{
}

ElementInput*
ElementInput::ElInput()
{
	return this;
}

void
ElementInput::LectureInputCB(int,U8InputId) 
{
	cout << "LectureInputCB de ElementInput " << endl;
}

void
ElementInput::LeDriverInputCB(U8Pointer client_data, int* fid,
		U8InputId* pt_input_id)
{
	U8Input& my_input = *(U8Input*) client_data;
	ElementInput& evs = *my_input.evs;

#ifdef IHM_SIGNAL_BSD
	U8_SIGNAL_RELSE;
#endif

	evs.LectureInputCB(*fid,*pt_input_id);

#ifdef IHM_SIGNAL_BSD
	U8_SIGNAL_HOLD;
#endif

}

void
ElementInput::LeDriverTimerCB(U8Pointer client_data,U8IntervalId* pt_interval_id)
{
	U8Input& my_input = *(U8Input*) client_data;
	int i = 0;

	for (;i<_nb_fd;i++)
	{

	    if(&my_input == (U8Input*) _class_tab[i] &&
			    my_input.xtd == *pt_interval_id &&
			    _fd_tab[i]->DataInBuffer())
	    {
		break;
	    }

	}

	if(i<_nb_fd)
	{
	    ElementInput& evs = *my_input.evs;
	    const int fd_index = _fd_tab[i]->numfile();

#ifdef IHM_SIGNAL_BSD
	    U8_SIGNAL_RELSE;
#endif

	    evs.LectureInputCB(fd_index,my_input.xid);

#ifdef IHM_SIGNAL_BSD
	    U8_SIGNAL_HOLD;
#endif

	}

}

U8InputId
ElementInput::AddInputDriver(nxstream* file,int mode)
{

	if(!file)
	{
	    cerr << "ElementInput : erreur : " << __FILE__ << " ligne " <<
		__LINE__ << "\nfile est NULL" << endl;

	    return -1;
	}

        if(_nb_fd == 0)
        {
            _fd_tab = (nxstream**) malloc(++_nb_fd * sizeof(nxstream*));
            _class_tab = (U8Pointer*) malloc(_nb_fd * sizeof(U8Pointer));

            FD_ZERO(&_readfds);
            FD_ZERO(&_writefds);
        }
        else
        {
            _fd_tab = (nxstream**) realloc((char*) _fd_tab,++_nb_fd *
			    sizeof(nxstream*));
            _class_tab = (U8Pointer*) realloc((char*) _class_tab,_nb_fd *
			    sizeof(U8Pointer));
        }

        U8Input* my_input = new U8Input;

        my_input->xid = _nb_fd - 1;
        my_input->xtd = 0;
        my_input->evs = this;

        _fd_tab[_nb_fd-1] = file;
        _class_tab[_nb_fd-1] = my_input;

	const int fd_index = file->numfile();
	nxstream* const parent = file->get_parent();
	FILE* const filer = parent ? parent->stdiofiler() : file->stdiofile();
	int fd_indexr = filer ? fileno(filer) : fd_index;

        switch(mode)
        {
	  case 0 :
	    FD_SET(fd_indexr,&_readfds);
	    break;
	  case 1 :
	    FD_SET(fd_index,&_writefds);
	    break;
	  case 2 :
	    FD_SET(fd_indexr,&_readfds);
	    FD_SET(fd_index,&_writefds);
	    break;
        }

        return _nb_fd-1;
}

void
ElementInput::RemoveInputDriver(U8InputId input_id)
{
        const long int xid = input_id;

        if(_nb_fd < 1)
        {
            cerr << "erreur : ElementInput : pas de driver a enlever" << endl;

            return;
        }

	nxstream* const file = _fd_tab[xid];
	int fd_index = file->numfile();
	nxstream* const parent = file->get_parent();
	FILE* const filer = parent ? parent->stdiofiler() : file->stdiofile();
	int fd_indexr = filer ? fileno(filer) : fd_index;

        FD_CLR(fd_indexr,&_readfds);
        FD_CLR(fd_index,&_writefds);

        if(_nb_fd == 1)
        {
            delete (U8Input*) (_class_tab[0]);

            free((char*) _fd_tab);_fd_tab = NULL;
            free((char*) _class_tab);_class_tab = NULL;

            _nb_fd = 0;
        }
        else
        {
            _nb_fd--;

            delete (U8Input*) (_class_tab[xid]);

            memmove(&_fd_tab[xid],&_fd_tab[xid+1],
			    (_nb_fd-xid)*sizeof(nxstream*));
            memmove(&_class_tab[xid],&_class_tab[xid+1],
			    (_nb_fd-xid)*sizeof(U8Pointer));

            _fd_tab = (nxstream**) realloc((char*) _fd_tab,_nb_fd *
			    sizeof(nxstream*));
            _class_tab = (U8Pointer*) realloc((char*) _class_tab,_nb_fd *
			    sizeof(U8Pointer));
        }

}

void
ElementInput::ScruteEvenement(int)
{
    // main loop

	while(TRUE)
	{
	    U8InputId num = 0;
	    int fdi = 0;
	    int ier = 0;

	    while(ier == 0) 
		num = Select(ier,fdi);

	    if(ier < 0)
	    {
		cerr << "ElementInput::Select : erreur " << __FILE__ <<
		    " ligne " << __LINE__ << endl;
		exit(1);
	    }

#ifdef DBG
	    cout << "ElementInput> Transmission activee :";
	    cout << "\n nbre connexions possibles " << _nb_fd;
	    cout << "\n nbre connexions actives " << ier;
	    cout << "\n numero connexion active " << num << endl;
#endif

	    U8Pointer class_num = _class_tab[num];

	    LeDriverInputCB(class_num,&fdi,&num);
	}

}

int
ElementInput::Select(int& rc,int& fdi)
{
	rc = 0;

	fd_set readfds = _readfds;
	fd_set writefds = _writefds;
	int isto = 0;

	for (int i = 0; i < _nb_fd; i++)
	{
	    nxstream* const file = _fd_tab[i];
	    nxstream* const parent = file->get_parent();
	    nxstream* const filer = parent ? parent : file;

	    if(filer->DataInBuffer())
	    {

		if(!rc)
		{
		    isto = i;
		    fdi = file->numfile();
		}

		rc += 1;
	    }

	}

	if(rc > 0)
	    return isto;

#if defined(HP_UX) && defined(__GNUC__) && !defined(_INCLUDE_POSIX4_SOURCE)
	int* pt_readfds = (int*) &readfds;
	int* pt_writefds = (int*) &writefds;
	const unsigned width = 32;
#else
	fd_set* pt_readfds = &readfds;
	fd_set* pt_writefds = &writefds;
	const int width = FD_SETSIZE;
#endif
	struct timeval timeout;

	timeout.tv_sec = 0;    
	timeout.tv_usec = 100000;   
	errno = 0;
	rc = select(width,pt_readfds,pt_writefds,NULL,&timeout);

	if (rc <= 0)
	    return 0;

	if (rc > 0)
	{
	    register int ii = 0;

	    for (; ii < _nb_fd; ii++)
	    {
		nxstream* const file = _fd_tab[ii];
	    // temporaire fd car bug ibmrs6000
		register int fd_index = file->numfile();
		nxstream* const parent = file->get_parent();
		FILE* const filer = parent ? parent->stdiofiler() :
		    file->stdiofile();
		register int fd_indexr = filer ? fileno(filer) : fd_index;

		if (FD_ISSET(fd_indexr,&readfds) || FD_ISSET(fd_index,&writefds))
		{
		    fdi = fd_index;
		    break;
		}

	    }

	    if(ii < _nb_fd)
		return ii;

	}

	cerr << "Attention non coherence readfds et _fd_tab " << __FILE__ <<
	    " ligne " << __LINE__ << endl;
	rc = -1;

	return 0;
}

nxstream*
ElementInput::GetNxstream2(int fid,U8InputId& num)
{
	register int i = 0;

	for(;i<_nb_fd;i++)
	{
	    register const int fd = _fd_tab[i]->numfile();

	    if(fd == fid)
	    {
		num = i;
		break;
	    }

	}

	nxstream* const file = i < _nb_fd ? _fd_tab[i] : NULL;

	return file;
}

nxstream*
ElementInput::GetNxstream(int fid)
{
	U8InputId num = 0;

	return GetNxstream2(fid,num);
}

/* Fin de Fichier */
