#include "U8machcomp.h"
#include "nxstream.h"

struct SccsIdnxstream
{
	static const char* const sccs_id;
};

const char* const SccsIdnxstream::sccs_id = "%W% %G% Pierre Brochard";

FILE*
nxstream::init(const char* name,const char* mode)
{
	FILE* f = fnxopen(name,mode);

	if(!f)
	{
	    f = fnxopen("/dev/null","r");
	}

	return f;
}

FILE*
nxstream::init(nxstream& nx_serveur)
{
	FILE* f = fnxaccept(nx_serveur.stdiofile());

	if(!f)
	{
	    f = fnxopen("/dev/null","r");
	}

	return f;
}

FILE*
nxstream::init(nxstream& nx_serveur,char* handler_name)
{
	FILE* f = fnxaccept2(nx_serveur.stdiofile(),handler_name);

	if(!f)
	{
	    f = fnxopen("/dev/null","r");
	}

	return f;
}

// ****************************************************************
// constructeurs
// ****************************************************************

nxstream::nxstream(FILE* f) :
#if defined(NAR_GNU_ISO)
		iostream(&buf),
#else
		iostream(),
#endif
		_parent(NULL),
		buf(f ? f : fnxopen("/dev/null","r")),
#ifdef __GNUC__
		_my_count(0),
#endif
		_shutdown(2)
{
#if !defined(NAR_GNU_ISO)
	ios::init(&buf);
#endif
}

nxstream::nxstream(const char* name,const char* mode) :
#if defined(NAR_GNU_ISO)
		iostream(&buf),
#else
		iostream(),
#endif
		_parent(NULL),
		buf(init(name,mode)),
#ifdef __GNUC__
		_my_count(0),
#endif
		_shutdown(2)
{
#if !defined(NAR_GNU_ISO)
	ios::init(&buf);
#endif
}

// La connexion read est bufferise par le iostream de nxstream
// dans le cas accept

// Dans le cas ouverture pour flux write non traite par nx mais par un ostream
// cree un nouveau nxstream(parent,1) qui delete son parent tout seul.

nxstream::nxstream(nxstream* nx_serveur,int mode_ostream) :
#if defined(NAR_GNU_ISO)
		iostream(&buf),
#else
		iostream(),
#endif
		_parent(nx_serveur),
#if defined(NAR_GNU_ISO)
		buf(nx_serveur->stdiofile()),
#else
		buf(mode_ostream ? nx_serveur->stdiofilew() :
				nx_serveur->stdiofile()),
#endif
#ifdef __GNUC__
		_my_count(0),
#endif
		_shutdown(2)
{
#if !defined(NAR_GNU_ISO)
	ios::init(&buf);
#endif
}

nxstream::nxstream(nxstream& nx_serveur) :
#if defined(NAR_GNU_ISO)
		iostream(&buf),
#else
		iostream(),
#endif
		_parent(NULL),
		buf(init(nx_serveur)),
#ifdef __GNUC__
		_my_count(0),
#endif
		_shutdown(2)
{
#if !defined(NAR_GNU_ISO)
	ios::init(&buf);
#endif

	if(!stdiofile())
	    clear(ios::failbit | rdstate());

}

nxstream::nxstream(nxstream& nx_serveur,char* handler_name) :
#if defined(NAR_GNU_ISO)
		iostream(&buf),
#else
		iostream(),
#endif
		_parent(NULL),
		buf(init(nx_serveur,handler_name)),
#ifdef __GNUC__
		_my_count(0),
#endif
		_shutdown(2)
{
#if !defined(NAR_GNU_ISO)
	ios::init(&buf);
#endif
}

nxstream::~nxstream()
{

	if(_parent)
	    delete _parent;
	else
	{
	    FILE* const f = stdiofile();

	    if(f)
	    {
		fnxclose_shutdown(f,_shutdown);
	    }

	}

}

/**
  Fonctions de lecture.
**/

void
nxstream::myreaderror()
{

	if(good())
	{
	    FILE* const reelfile = stdiofiler();

	    if(feof(reelfile))
		clear(ios::eofbit | rdstate());
	    else if(ferror(reelfile))
		clear(ios::failbit | rdstate());

	}

}

istream&
nxstream::read(char buffer[],int& count)
{
	myreaderror();

	if(good())
	{
	    FILE* const file = stdiofile();
	    const int lus = fnxread(file,buffer,count);

	    if(lus != count)
	    {

		if(feof(stdiofiler()))
		{
		    clear(ios::eofbit | rdstate());
		    count = lus;
		}
		else
		{
		    clear(ios::failbit | rdstate());
		    count = 0;
		}

	    }

	}
	else
	    count = 0;

	return *this;
}

istream& /* read non bloquant */
nxstream::read(int* buffer,int count)
{
	myreaderror();

	if(good())
	{
	    FILE* const file = stdiofile();
	    const int lus = fnxnbkri(file,buffer,count);

	    if(lus != count)
	    {

		if(feof(stdiofiler()))
		    clear(ios::eofbit | rdstate());
		else
		    clear(ios::failbit | rdstate());

	    }

	}

	return *this;
}

istream& /* read non bloquant */
nxstream::read(NXFLT buffer[],int count)
{
	myreaderror();

	if(good())
	{
	    FILE* const file = stdiofile();
	    const int lus = fnxnbkrf(file,buffer,count);

	    if(lus != count)
	    {

		if(feof(stdiofiler()))
		    clear(ios::eofbit | rdstate());
		else
		    clear(ios::failbit | rdstate());

	    }

	}

	return *this;
}

istream& /* read non bloquant */
nxstream::read(NXDBLE buffer[],int count)
{
	myreaderror();

	if(good())
	{
	    FILE* const file = stdiofile();
	    const int lus = fnxnbkrt(file,buffer,count,"dp","dp");

	    if(lus != count)
	    {

		if(feof(stdiofiler()))
		    clear(ios::eofbit | rdstate());
		else
		    clear(ios::failbit | rdstate());

	    }

	}

	return *this;
}

#if defined(CRAY) && !defined(_CRAYMPP)
istream& /* read non bloquant */
nxstream::read(double buffer[],int count)
{
	myreaderror();

	if(good())
	{
	    FILE* const file = stdiofile();
	    const int lus = fnxnbkrt(file,(NXFLT*) buffer,count,"sp","dp");

	    if(lus != count)
	    {

		if(feof(stdiofiler()))
		    clear(ios::eofbit | rdstate());
		else
		    clear(ios::failbit | rdstate());

	    }

	}

	return *this;
}
#endif

/**
  Fonctions d'ecriture.
**/

void
nxstream::mywriteerror()
{

	if(good())
	{
	    FILE* const reelfile = stdiofilew();

	    if(ferror(reelfile))
		clear(ios::failbit | rdstate());

	}

}

ostream&
nxstream::wflush()
{
	mywriteerror();

	if(good())
	{
	    const int ier = fnxflush(stdiofile());

	    if(ier < 0)
	    {
		clear(ios::failbit | rdstate());
	    }

	}

	return *this;
}

ostream&
nxstream::write(const char* buffer,int count)
{
	mywriteerror();

	if(good())
	{
	    const int ier = fnxwrite(stdiofile(),buffer,count);

	    if(ier < 0)
	    {
		clear(ios::failbit | rdstate());
	    }

	}

	return *this;
}

ostream&
nxstream::write(const int* buffer,int count)
{
	mywriteerror();

	if(good())
	{
	    const int ier = fnxwriti(stdiofile(),count,buffer);

	    if(ier < 0)
	    {
		clear(ios::failbit | rdstate());
	    }

	}

	return *this;
}

ostream&
nxstream::write(const NXFLT buffer[],int count)
{
	mywriteerror();

	if(good())
	{
	    const int ier = fnxwritf(stdiofile(),count,buffer);

	    if(ier < 0)
	    {
		clear(ios::failbit | rdstate());
	    }

	}

	return *this;
}

ostream&
nxstream::write(const NXDBLE buffer[],int count)
{
	mywriteerror();

	if(good())
	{
	    int ier = fnxwritt(stdiofile(),count,buffer,"dp","dp");

	    if(ier < 0)
	    {
		clear(ios::failbit | rdstate());
	    }

	}

	return *this;
}

#if defined(CRAY) && !defined(_CRAYMPP)
ostream&
nxstream::write(const double buffer[],int count)
{
	mywriteerror();

	if(good())
	{
	    const int ier = fnxwritt(stdiofile(),count,buffer,"sp","dp");

	    if(ier < 0)
	    {
		clear(ios::failbit | rdstate());
	    }

	}

	return *this;
}
#endif

int
nxstream::DataInBuffer()
{
	myreaderror();

    // utile pour serveur nx/poulpe test si le buffer en lecture
    // contient des donnees a lire
	if(good())
	{
	    FILE* const file = stdiofile();

	    return fnx_data_in_buffer(file);
	}
	else
	    return 0;

}

#ifdef __GNUC__
// bug on ne peut employer le get de istream GNU
istream&
nxstream::get(char* buffer,int limit,char delim)
{

	if(!good())
	{
	    _my_count = 0;
	    buffer[0] = '\0';

	    return *this;
	}

	FILE* const file = stdiofiler();

	for(int i=0;i<limit-1;i++)
	{
	    const int lu = getc(file);
	    
	    if(lu != EOF && char(lu) != delim)
	    {
	        buffer[i] = char(lu);
	    }
	    else if(lu != EOF)
	    {
	        ungetc(lu,file);
	        _my_count = i;
	        buffer[i] = '\0';

	        return *this;
	    }
	    else
	    {

		if(feof(file))
		    clear(ios::eofbit | rdstate());
		else
		    clear(ios::failbit | rdstate());

	        _my_count = i;
		buffer[i] = '\0';

	        return *this;
	    }
	    
	}
	
	_my_count = limit - 1;
	buffer[_my_count] = '\0';

	return *this;
}

istream&
nxstream::getline(char* buffer,int limit,char delim)
{

	if(!good())
	{
	    _my_count = 0;
	    buffer[0] = '\0';

	    return *this;
	}

	FILE* const file = stdiofiler();

	for(int i=0;i<limit-1;i++)
	{
	    const int lu = getc(file);
	    
	    if(lu != EOF && char(lu) != delim)
	    {
	        buffer[i] = char(lu);
	    }
	    else if(lu != EOF)
	    {
	        _my_count = i+1;
	        buffer[i] = '\0';

	        return *this;
	    }
	    else
	    {

		if(feof(file))
		    clear(ios::eofbit | rdstate());
		else
		    clear(ios::failbit | rdstate());

	        _my_count = i;
		buffer[i] = '\0';

	        return *this;
	    }
	    
	}
	
	_my_count = limit - 1;
	buffer[_my_count] = '\0';

	return *this;
}

int
nxstream::gcount()
{

	if(_my_count)
	{
	    const int count = _my_count;
	    
	    _my_count = 0;
	    
	    return count;
	}
	else
	{
	    return istream::gcount();
	}
	
}
#endif
