/********************************************/
/*     manipulations de fichiers L3         */
/********************************************/

#ifndef lint
    static const char* const sccs_id = "%W% %G% Pierre Brochard";
#endif

#include "U8constant.h"
#include "sanglier.h"
#include "sanglierP.h"
#include "L3ListeDesCodes.h"
#include "poulpe/L3Manip.h"

#define S6Max(a,b) (((a)>(b))? (a):(b))

/*  memmove  */

#if defined(SUNOS) && defined(SUNOS_4)
    void bcopy(void* b1,void* b2,int length);
#   define memmove(a,b,c) bcopy((b),(a),(c))
#endif

/**********************************************/
/* code l3 supplementaire pour synchro Psyche */
/**********************************************/

#define CODEL3SYNCHRO 10000

/********************************************/
/* variables globales et locales au fichier */
/********************************************/

struct ChaineL3_st
{
	struct ChaineL3_st* prec;
	struct ChaineL3_st* suiv;
	int* code;
};

typedef struct ChaineL3_st ChaineL3;

static char** pt_buffer_alloue = NULL;
static int taille_alloue = -1;

/********************************************/
/* routines                                 */
/********************************************/

static void DetruitDisplayList(void);

/* on alloue des int ou des ChaineL3 dans un ensemble de tableaux */

static void*
AlloueBuffer(unsigned taille)
{
	const unsigned taille_buffer = 65536;
	static unsigned pointeur_max = 0;
	static int alloue_buffer_reentrant = 0;
	const unsigned align_param = sizeof(void*);

	taille--;taille /= align_param;taille++;taille *= align_param;

	if(!pt_buffer_alloue)
	{
	    taille_alloue = 0;
	    pointeur_max = 0;

	    pt_buffer_alloue = (char**) malloc(sizeof(char*));

	    if(!pt_buffer_alloue || !(pt_buffer_alloue[taille_alloue] = (char*)
			malloc(taille_buffer)))
	    {
		fprintf(stderr,"L3Manip : pb alloc %s %d\n",__FILE__,__LINE__);
		DetruitDisplayList();

		if(alloue_buffer_reentrant)
		{
		    alloue_buffer_reentrant = 0;
		    return NULL;
		}

		alloue_buffer_reentrant = 1;

		return AlloueBuffer(taille);
	    }

#ifdef DBG
	    printf(" AlloueBuffer first\n");
#endif

	}

	if(pointeur_max + taille < taille_buffer)
	{
	    pointeur_max += taille;
	}
	else
	{
	    char** ptba_temp;

	    taille_alloue++;
	    ptba_temp = (char**) realloc((char*) pt_buffer_alloue,
			(taille_alloue + 1)*sizeof(char*));

	    if(!ptba_temp || !(ptba_temp[taille_alloue] = (char*)
			malloc(S6Max(taille_buffer,taille))))
	    {
		fprintf(stderr,"L3Manip : pb alloc %s %d\n",__FILE__,__LINE__);

		if(ptba_temp) pt_buffer_alloue = ptba_temp;

		DetruitDisplayList();

		if(alloue_buffer_reentrant)
		{
		    alloue_buffer_reentrant = 0;
		    return NULL;
		}

		alloue_buffer_reentrant = 1;

		return AlloueBuffer(taille);
	    }
	    else
	    {
		pt_buffer_alloue = ptba_temp;
	    }

#ifdef DBG
	    printf(" AlloueBuffer next\n");
#endif

	    pointeur_max = taille;
	}

	alloue_buffer_reentrant = 0;

	return (void*) (pt_buffer_alloue[taille_alloue]+pointeur_max-taille);
}

static int
VideBuffer(void)
{
	int i = 0,ier = 0;

#ifdef DBG
	printf(" VideBuffer\n");
#endif

	if(pt_buffer_alloue)
	{

	    for(i=0;i<=taille_alloue;i++)
	    {

		if(pt_buffer_alloue[i]) free(pt_buffer_alloue[i]);

	    }

	    free(pt_buffer_alloue);
	}

	pt_buffer_alloue = NULL;
	taille_alloue = -1;

	return ier;
}

static ChaineL3* debut = NULL;
static ChaineL3* fin = NULL;

static void
DetruitDisplayList(void)
{
	VideBuffer();

	debut = NULL;
	fin = NULL;
}

static void
AjouteItemDisplayList(int codel3[])
{
	ChaineL3* ptr = (ChaineL3*) AlloueBuffer(sizeof(ChaineL3));

	if(!debut) debut = ptr;

	{
	    unsigned codel3_taille = codel3[0] * sizeof(int);
	    int* code = (int*) AlloueBuffer(codel3_taille);

	    if(!debut) ptr = (ChaineL3*) AlloueBuffer(sizeof(ChaineL3));

	    ptr->code = code;
	    memcpy((char*) code,(char*) codel3,codel3_taille);

	    if (!fin)
	    {
		ptr->prec = NULL;
		ptr->suiv = NULL;
		fin = debut = ptr;
	    }
	    else
	    {
		fin->suiv = ptr;
		ptr->prec = fin;
		ptr->suiv = NULL;
		fin = ptr;
	    }

	}

}

int
SauveDisplayList(const char* file_sdl)
{
    /******************************************************/
    /* code de retour : -1 => pas de fichier sauve        */
    /*                  0  => fichier sauve, pas d'erreur */
    /******************************************************/

	unsigned lg_code_l3 = 0;
	unsigned alloc_code_l3 = 128;
	char* code_l3 = (char*) malloc(alloc_code_l3);
	FILE* sdl = fopen(file_sdl,"w");
	register ChaineL3* ptr = debut;

	if(!sdl)
	{
	    fprintf(stderr,"L3Manip : SauveDisplayList : "
			    "pb ouverture fichier L3\n");

	    if(code_l3) free(code_l3);

	    fclose(sdl);
	    unlink(file_sdl);

	    return -1;
	}

	if (!debut)
	{
	    fprintf(stderr,"L3Manip : SauveDisplayList : "
			    "pas de liste a sauver L3Manip\n");

	    if(code_l3) free(code_l3);

	    fclose(sdl);
	    unlink(file_sdl);

	    return -1;
	}

	while (ptr)
	{
	    register const unsigned longueur = (unsigned) ptr->code[0];

	    lg_code_l3 = 2 * longueur;

	    if(lg_code_l3 > alloc_code_l3)
	    {
		free(code_l3);
		code_l3 = (char*) malloc(lg_code_l3);
		alloc_code_l3 = lg_code_l3;
	    }

	    {
		register const int* const source = ptr->code;
		register unsigned i;
		register int j;

	    /* marche a la fois en little et big endian */

		for (i = 0,j = 0; i < longueur; i++)
		{
		    register const int cdl3 = source[i];

		    code_l3[j++] = (char) ((cdl3 >> 8) & 0xFF);
		    code_l3[j++] = (char) (cdl3 & 0xFF);
		}

	    }

	    fwrite(code_l3,1,lg_code_l3,sdl);
	    ptr = ptr->suiv;
	}

	fclose(sdl);

	if(code_l3) free(code_l3);

	return 0;
}

void
LitDisplayList(const char* file_sdl)
{
	FILE* sdl = fopen(file_sdl,"r");

	LitDisplayListFILE(sdl);

	fclose(sdl);
}

int
LitDisplayListFILE(FILE* sdl)
{
    /*****************************************************/
    /* code retour :                                     */
    /* -1 => EOF                                         */
    /* 0 CODEL3CLOSE                                     */
    /* 1 CODEL3SYNCHRO                                   */
    /*****************************************************/

	int alloc_code_l3 = 256;
	int* code_l3 = (int*) malloc(alloc_code_l3);

	if(!sdl)
	{
	    fprintf(stderr,"L3Manip : LitDisplayListFILE : "
			    "pb ouverture fichier L3\n");

	    if(code_l3) free(code_l3);

	    return -1;
	}

	DetruitDisplayList();

	while(1)
	{
	    int taille_code_l3 = 0;
	    char* ptr = NULL;
	    unsigned char code[2];
	    int lg_code_l3 = 0;
	    int lus = fread(code,1,2,sdl);

	    if(lus != 2) break;

	    {
		register unsigned char code1 = code[0];
		register unsigned char code2 = code[1];

		lg_code_l3 = (code1 << 8) | code2;
		taille_code_l3 = (lg_code_l3 + 1) * sizeof(int);
	    }

	    if(taille_code_l3 > alloc_code_l3)
	    {
		free(code_l3);
		code_l3 = (int*) malloc(taille_code_l3);
		alloc_code_l3 = taille_code_l3;
	    }

	    ptr = (char*) (code_l3 + (lg_code_l3+3)/2);
	    lus = fread(ptr,2,lg_code_l3-1,sdl);

	    if(lus != lg_code_l3-1) break;

	    code_l3[0] = lg_code_l3;

	    {

	    /* marche a la fois en little et big endian */

		register int* source = code_l3 + 1;
		register const int* const source_max = code_l3 + lg_code_l3;
		
		while(source < source_max)
		{
		    register const unsigned char code1 = *ptr++;
		    register const unsigned char code2 = *ptr++;
		    register int temp = code1 > 127 ? -1 : 0;

		    temp = (temp << 16) | (code1 << 8) | code2;
		    *source++ = temp;
		}

	    }

	    if(code_l3[1] == CODEL3SYNCHRO)
	    {

		if(code_l3) free(code_l3);

		return 1;
	    }

	    AjouteItemDisplayList(code_l3);

	    if(code_l3[1] == CODEL3CLOSE)
	    {

		if(code_l3) free(code_l3);

		return 0;
	    }

	}

	if(code_l3) free(code_l3);

	return -1;
}

void
TraceDisplayList(enum L3OpCl l3oc)
{
	static int code_flush[2] = {2,20};

	register ChaineL3* ptr = debut;
	register int code_l3 = 0;

	int flag_colormap = 0;

	if (!ptr) return;

	switch (l3oc)
	{
	  case AVOP_AVCL :

	    while (ptr)
	    {
		code_l3 = ptr->code[1];

		if(code_l3 == CODEL3COLORMAP)
		{

		    if((ptr->code[0])%3 != 0 ||
			ptr->code[2] != (ptr->code[0]-3)/3)
		    {
			fprintf(stdout,"L3Manip> Mauvaise carte de couleur\n");
			ptr = ptr->suiv;
			continue;
		    }
		    else
		    {
			flag_colormap = 0;
		    }

		}

		if(flag_colormap && code_l3 != CODEL3COLORMAP &&
			code_l3 != CODEL3CADRE && code_l3 != CODEL3EPAISSEUR &&
			code_l3 != CODEL3COULEUR && code_l3 != CODEL3BRILLANCE)
		{
		    static RGB fond = {255., 255., 255.};
		    static RGB antifond = {0., 0., 0.};
		    const int NombreCouleurs = 64;

		    fprintf(stdout,"L3Manip> On impose une carte de couleur\n");

		    flag_colormap = 0;
		    L3ChoixCartesDeCouleurs(RAINBOWHLS,NombreCouleurs,
			&fond, &antifond);
		}
		else if(code_l3 == CODEL3OPEN)
		{
		    flag_colormap = 1;
		}

		L3AppelDrivers(ptr->code);

		ptr = ptr->suiv;
	    }

	    break;

	  case SOP_AVCL :

	    while (ptr)
	    {
		code_l3 = ptr->code[1];

		if(code_l3 != CODEL3OPEN && code_l3 != CODEL3COLORMAP)
			L3AppelDrivers(ptr->code);

		ptr = ptr->suiv;
	    }

	    break;

	  case SOP_SCL :

	    while (ptr)
	    {
		code_l3 = ptr->code[1];

		if(code_l3 != 1 && code_l3 != CODEL3COLORMAP &&
			code_l3 != CODEL3CLOSE)
		{
		    L3AppelDrivers(ptr->code);
		}
		else if(code_l3 == CODEL3CLOSE)
		{
		    L3AppelDrivers(code_flush);
		}

		ptr = ptr->suiv;
	    }

	    break;

	  case AVOP_SCL :

	    while (ptr)
	    {
		code_l3 = ptr->code[1];

		if(code_l3 == CODEL3COLORMAP)
		{

		    if((ptr->code[0])%3 != 0 ||
			ptr->code[2] != (ptr->code[0]-3)/3)
		    {
			fprintf(stdout,"L3Manip> Mauvaise carte de couleur\n");
			ptr = ptr->suiv;
			continue;
		    }
		    else
		    {
			flag_colormap = 0;
		    }

		}

		if(flag_colormap && code_l3 != CODEL3COLORMAP &&
			code_l3 != CODEL3CADRE && code_l3 != CODEL3EPAISSEUR &&
			code_l3 != CODEL3COULEUR && code_l3 != CODEL3BRILLANCE)
		{
		    static RGB fond = {255., 255., 255.};
		    static RGB antifond = {0., 0., 0.};
		    const int NombreCouleurs = 64;

		    fprintf(stdout,"L3Manip> On impose une carte de couleur\n");

		    flag_colormap = 0;
		    L3ChoixCartesDeCouleurs(RAINBOWHLS,NombreCouleurs,
			&fond, &antifond);
		}
		else if(code_l3 == CODEL3OPEN)
		{
		    flag_colormap = 1;
		}

		if(code_l3 == CODEL3CLOSE)
		{
		    L3AppelDrivers(code_flush);
		}
		else
		{
		    L3AppelDrivers(ptr->code);
		}

		ptr = ptr->suiv;
	    }

	    break;
	}

}

void
TraceDisplayListMod(enum L3OpCl l3oc,REEL xposmin,REEL xposmax,
                REEL yposmin,REEL yposmax)
{
	static int code_flush[2] = {2,CODEL3FLUSH};

	register ChaineL3* ptr = NULL;
	register int code_l3 = 0;

	int flag_colormap = 0;

	if (!debut) return;

	ptr = debut;

	while (ptr)
	{
	    code_l3 = ptr->code[1];

	    if(code_l3 == CODEL3COLORMAP)
	    {

		if((ptr->code[0])%3 != 0 || ptr->code[2] != (ptr->code[0]-3)/3)
		{
		    fprintf(stdout,"L3Manip> Mauvaise carte de couleur\n");
		    ptr = ptr->suiv;
		    continue;
		}
		else
		{
		    flag_colormap = 0;
		}

	    }

	    if(flag_colormap && code_l3 != CODEL3COLORMAP &&
			code_l3 != CODEL3CADRE && code_l3 != CODEL3EPAISSEUR &&
			code_l3 != CODEL3COULEUR && code_l3 != CODEL3BRILLANCE)
	    {
		static RGB fond = {255., 255., 255.};
		static RGB antifond = {0., 0., 0.};
		const int NombreCouleurs = 64;

		fprintf(stdout,"L3Manip> On impose une carte de couleur\n");

		flag_colormap = 0;
		L3ChoixCartesDeCouleurs(RAINBOWHLS,NombreCouleurs,
			&fond, &antifond);
	    }

	    switch (code_l3)
	    {
		case CODEL3OPEN :

		    if(l3oc == AVOP_AVCL || l3oc == AVOP_SCL)
		    {
			L3AppelDrivers(ptr->code);
			flag_colormap = 1;
		    }

		    break;

		case CODEL3CLOSE :

		    if(l3oc == AVOP_AVCL || l3oc == SOP_AVCL)
		    {
			L3AppelDrivers(ptr->code);
		    }
		    else
		    {
			L3AppelDrivers(code_flush);
		    }

		    break;

		case CODEL3COLORMAP :

		    if(l3oc == AVOP_AVCL || l3oc == AVOP_SCL)
		    {
			L3AppelDrivers(ptr->code);
		    }

		    break;

		case CODEL3SEGMENT :

		    {
			const int* const code = ptr->code;
			int xl3min,yl3min,xl3max,yl3max;
			int segment_a_tracer = 0;
			int code_modifie[] = {6,8,0,0,0,0};

			REEL xmin,xmax;
			REEL ymin,ymax;
			const REEL deltax = xposmax - xposmin;
			const REEL deltay = yposmax - yposmin;

			memcpy(code_modifie,code,6*sizeof(int));
			L3MiseEnPage(0,0.,0.,1.,1.,0.,0.,1.,1.);
			L3DefSpace(0,0.,0.,1.,1.);
			L3SelectionSousImage(0);
			L3ConvCoordL3VersUtil(code[2],code[3],&xmin,&ymin);
			L3ConvCoordL3VersUtil(code[4],code[5],&xmax,&ymax);
			xmin = xmin*deltax + xposmin;
			ymin = ymin*deltay + yposmin;
			xmax = xmax*deltax + xposmin;
			ymax = ymax*deltay + yposmin;
			L3ConversionCoordonnees(xmin,ymin,&xl3min,&yl3min);
			L3ConversionCoordonnees(xmax,ymax,&xl3max,&yl3max);
			segment_a_tracer = L3ClippingSegment(xl3min,yl3min,
				xl3max,yl3max,&code_modifie[2]);

			if(segment_a_tracer) L3AppelDrivers(code_modifie);

		    }

		    break;

		case CODEL3MARQUEUR :

		    {
			const int* const code = ptr->code;
			int xl3pos,yl3pos;
			int point_a_tracer = 0;
			int code_modifie[] = {5,7,0,0,0};

			REEL xpos;
			REEL ypos;
			const REEL deltax = xposmax - xposmin;
			const REEL deltay = yposmax - yposmin;

			memcpy(code_modifie,code,5*sizeof(int));
			L3MiseEnPage(0,0.,0.,1.,1.,0.,0.,1.,1.);
			L3DefSpace(0,0.,0.,1.,1.);
			L3SelectionSousImage(0);
			L3ConvCoordL3VersUtil(code[2],code[3],&xpos,&ypos);
			xpos = xpos*deltax + xposmin;
			ypos = ypos*deltay + yposmin;
			L3ConversionCoordonnees(xpos,ypos,&xl3pos,&yl3pos);
			point_a_tracer = L3ClippingPoint(xl3pos,yl3pos);

			if(point_a_tracer)
			{
			    code_modifie[2] = xl3pos;
			    code_modifie[3] = yl3pos;
			    L3AppelDrivers(code_modifie);
			}

		    }

		    break;

		case CODEL3TEXTE :

		    {
			const int* const code = ptr->code;
			const unsigned lcode = ptr->code[0];
			const unsigned taille = lcode*sizeof(int);
			int xl3pos,yl3pos;
			int point_a_tracer = 0;
			int* code_modifie = (int*) malloc(taille);

			REEL xpos;
			REEL ypos;
			const REEL deltax = xposmax - xposmin;
			const REEL deltay = yposmax - yposmin;

			memcpy(code_modifie,code,taille);
			L3MiseEnPage(0,0.,0.,1.,1.,0.,0.,1.,1.);
			L3DefSpace(0,0.,0.,1.,1.);
			L3SelectionSousImage(0);
			L3ConvCoordL3VersUtil(code[2],code[3],&xpos,&ypos);
			xpos = xpos*deltax + xposmin;
			ypos = ypos*deltay + yposmin;
			L3ConversionCoordonnees(xpos,ypos,&xl3pos,&yl3pos);
			point_a_tracer = L3ClippingPoint(xl3pos,yl3pos);

			if(point_a_tracer)
			{
			    code_modifie[2] = xl3pos;
			    code_modifie[3] = yl3pos;
			    L3AppelDrivers(code_modifie);
			}

			free((char*) code_modifie);
		    }

		    break;

		case CODEL3POLYGONE :

		    {
			const int* const code = ptr->code;
			const int taille = (*code-2)/2;
			int i = 0;

			REEL* x = (REEL*) malloc(taille*sizeof(REEL));
			REEL* y = (REEL*) malloc(taille*sizeof(REEL));
			const REEL deltax = xposmax - xposmin;
			const REEL deltay = yposmax - yposmin;

			L3MiseEnPage(0,0.,0.,1.,1.,0.,0.,1.,1.);
			L3DefSpace(0,0.,0.,1.,1.);
			L3SelectionSousImage(0);

			for(i=0;i<taille;i++)
			{
			    L3ConvCoordL3VersUtil(code[2*i+2],code[2*i+3],
					    &x[i],&y[i]);
			    x[i] = x[i]*deltax + xposmin;
			    y[i] = y[i]*deltay + yposmin;
			}

			L3Polygone(0,x,y,taille);

			free(x);
			free(y);
		    }

		    break;

		default :

		    L3AppelDrivers(ptr->code);
		    break;
	    }

	    ptr = ptr->suiv;
	}

}

/* Fin de Fichier */
