//
// Gestion du repertoire 
//

struct SccsIdRepertoire
{
	static const char* const sccs_id;
};

const char* const SccsIdRepertoire::sccs_id = "%W% %G% G. Colin de Verdiere";

// modifie Pierre brochard

#include "U8constant.h"

#include <sys/stat.h>
#include <dirent.h>

#include "poulpe/IHM.h"
#include "poulpe/Repertoire.h"
#include "poulpe/Help.h"
#include "dinosaure/P4compfiltre.h"

// *************************************************************
//   classe de base ScrollList
// *************************************************************

ScrollList::ScrollList(const Widget& parent) :
		ListeSimple(parent),
		nbentries(0),
		filename(NULL)
{
}

ScrollList::~ScrollList()
{

	if (filename)
	{

	    for (int i = nbentries - 1;i >= 0;i--)
	    {
		delete[] filename[i];
	    }

	    delete[] filename;filename = NULL;

	    PurgeListe();
	}

}

void
ScrollList::ChangeListe(const char* const,const char* const)
{
}

int
TriDir(const void* name1, const void* name2)
{
	return(strcmp(*((char**) name1), *((char**) name2)));
}

void
ScrollList::ChangeListeGen(const char* const dir,const int type,
		const char* const filtre)
{
    // type = 0 => fichiers du repertoire courant mis dans la liste
    // type = 1 => repertoires du repertoire courant mis dans la liste
    // filtre pour fichiers ignore si repertoire

	if (filename)
	{

	    for (int i = nbentries - 1; i >= 0; i--)
	    {
		delete[] filename[i];
	    }

	    delete[] filename;filename = NULL;

	    PurgeListe();
	}

    // recherche des fichiers ou repertoires contenus dans le repertoire courant

	nbentries = 0;

	DIR* dirp = opendir((char*) dir);

	if (!dirp)
	{
	    cerr << "erreur : ScrollList : pb ouverture repertoire" << endl;
	}
	else
	{
	    struct stat stbuf;
	    dirent* dp = NULL;

	// On compte les fichiers ou reprtoires

	    for (dp = readdir(dirp); dp; dp = readdir(dirp))
	    {

		if(stat(dp->d_name, &stbuf) != -1)
		{
		    const int type_fich = type ? S_ISDIR(stbuf.st_mode) :
			S_ISREG(stbuf.st_mode);

		    if(type_fich) nbentries++;

		}

	    }

	    filename = new char*[nbentries];

	    if(!filename)
	    {
		cerr << "erreur : ScrollList : alloc " << __FILE__ <<  ' ' <<
		    __LINE__ << endl;
		return;
	    }

	    rewinddir(dirp);

	    int i = 0;

	    for (dp = readdir(dirp); dp && i<nbentries; dp = readdir(dirp))
	    {
		const char* const name = dp->d_name;
		int comp_filtre = type ? 1 : P4CompareFiltre(name,filtre);

		if(stat(dp->d_name, &stbuf) != -1)
		{
		    const int type_fich = type ? S_ISDIR(stbuf.st_mode) :
			S_ISREG(stbuf.st_mode);

		    if (type_fich && comp_filtre)
		    {
			const unsigned taille = strlen(name) + 1;

			filename[i] = new char[taille];
			memcpy(filename[i], name, taille);
			i++;
		    }

		}

	    }

	    nbentries = i;
	    closedir(dirp);

	    int deb = 0;

	    if(type && nbentries > 2 && *filename[0] == '.' &&
			    !strcmp(filename[1],".."))
	    {
		deb = 2;
	    }

	    if (nbentries > deb)
	    {
		qsort((char*) (filename + deb),(nbentries - deb),
				sizeof(char*), TriDir);
	    }
      
	// Mise a jour de la liste affichee

	    for (i = 0; i < nbentries; i++)
	    {
		AjouteElement(filename[i]);
	    }

	}

}

// *************************************************************
//   classe ScrollDirList
// *************************************************************

ScrollDirList::ScrollDirList(const Widget& parent,const char* const dir,
		Label* const fichier,Saisie* const chemin,
		ScrollFileList* const files,
		Saisie* const filtre) :
		ScrollList(parent),
		_files(files),_chemin(chemin),_filtre(filtre),_fichier(fichier)
{
	ChangeListe(dir,filtre->GetLabel());
}

ScrollDirList::~ScrollDirList()
{
}

void
ScrollDirList::ChangeListe(const char* const dir,const char* const filtre)
{

	if(dir && *dir && *dir != '.')
	{

	    if(strchr(dir,'/'))
		chdir((char*) dir);

	}

	char* const directory = getcwd(NULL, 1024);

	if(directory)
	{
	    _chemin->Change(directory);
	    ScrollList::ChangeListeGen(directory,1);
	    Montre();
	    _files->ChangeListe(directory,filtre);
	    free(directory);
	}

}

void
ScrollDirList::SelectCB(XtPointer call_data)
{	
	ListeSimple::SelectCB(call_data);
	
	const int i = GetLastSelected();

	if(i >= 0 && i < nbentries)
	{
	    const unsigned lfile = strlen(filename[i]) + 1;
	    char* neuf = new char[lfile];

	    if(!neuf) return;

	    memcpy(neuf,filename[i],lfile);

	    if(_chemin)
	    {
		_chemin->Change(neuf);
	    }

	    chdir(neuf);
	    delete[] neuf;

	    const char* filtre;

	    if(_filtre)
	    {
		filtre = _filtre->GetText();
	    }
	    else
	    {
		filtre = NULL;
	    }

	    ChangeListe(".",filtre);

	    if(_fichier)
	    {
		_fichier->Change(" ");
	    }

	}

}

// *************************************************************
//   classe ScrollFileList
// *************************************************************

ScrollFileList::ScrollFileList(const Widget& parent,const char* dir,
		Label* const fichier,Directory* const pere) :
		ScrollList(parent),
		_pere(pere),_fichier(fichier)
{
	ChangeListe(dir);
	Extended();
}

ScrollFileList::~ScrollFileList()
{
}

void
ScrollFileList::SelectCB(XtPointer call_data)
{
	ListeSimple::SelectCB(call_data);

	const int i = GetLastSelected();

	if(i >= 0 && i < nbentries)
	{
	    _fichier->Change(filename[i]);
	}

}

void
ScrollFileList::SelectDoubleCB(XtPointer call_data)
{
	ListeSimple::SelectDoubleCB(call_data);
	_pere->OKAction();
}

void
ScrollFileList::SelectExtendedCB(XtPointer call_data)
{
	ListeSimple::SelectExtendedCB(call_data);

	const int i = GetLastSelected();

	if(i >= 0 && i < nbentries)
	{
	    _fichier->Change(filename[i]);

	    const int* const selected = GetSelected();
	    const int nbselected = GetNombreSelected();
	    char** files_selected = new char*[nbselected];
	    int j = 0;

	    for(;j<nbselected;j++)
	    {
		files_selected[j] = filename[selected[j]];
	    }

	    _pere->OKAction(files_selected,nbselected);

	    delete[] files_selected;
	}

}

void
ScrollFileList::ChangeListe(const char* const dir,const char* const filtre)
{
	ScrollList::ChangeListeGen(dir,0,filtre);
	Montre();
}

// *********************************************************
//         bouton OK
// *********************************************************

OKButton::OKButton(const Widget& parent,const char* label,
		Directory* const pere,Communiq* const ptr) :
		Bouton(parent, label),
		_pere(pere),
		_fichierChoisi(NULL),
		_cheminChoisi(NULL),
		_ptr(ptr)
{
}

OKButton::~OKButton()
{
	delete[] _fichierChoisi;
	delete[] _cheminChoisi;
}

void
OKButton::SelectCB(XtPointer) 
{   
	delete[] _fichierChoisi;
	delete[] _cheminChoisi;

	const char* const nom_fich = _pere->GetNomFichier();

	_fichierChoisi = new char[strlen(nom_fich) + 1];
	strcpy(_fichierChoisi, nom_fich);
   
	if(!strcmp(_fichierChoisi, " "))
	{
	    strcpy(_fichierChoisi, "");
	}

   	const char* const nom_rep = _pere->GetNomRepertoire();

	_cheminChoisi = new char[strlen(nom_rep) + 1];
	strcpy(_cheminChoisi, nom_rep);
   
	if(_ptr)
	{
	    _ptr->CheminChoisi(_cheminChoisi,2);
	    _ptr->FichierChoisi(&_fichierChoisi,1,2);
	}

}

// *********************************************************
//         bouton Cancel
// *********************************************************

CancelButton::CancelButton(const Widget& parent, 
		const char* label,
		PopUp* const pere) : Bouton(parent, label)
{
	_pere = pere;
}

void
CancelButton::SelectCB(XtPointer) 
{
	_pere->Cache();
}

// *********************************************************
//         classe Directory
// *********************************************************

Directory::Directory(const Widget& root,const Widget& parent,
		const char* label,Communiq* const ptr,
		const char*const* labels,unsigned labels_taille) :
		PullDownEntry(parent, label),
		_done(FALSE),_LeTout(NULL),
		_LesBtnsJ(NULL),_LesBtns(NULL),
		_LesLabsJ(NULL),_LesLabs(NULL),
		_LesListesJ(NULL),_LesListes(NULL),
		_LesLabs2J(NULL),_LesLabs2(NULL),
		_direcs(NULL),_fichiers(NULL),
		_cancel(NULL),_ok(NULL),_help(NULL),
		_chemin(NULL),_filtre(NULL),
		_chemin_label(NULL),_filtre_label(NULL),
		_fichier(NULL),_fichier_label(NULL),
		_ptr(ptr),_labels(NULL)
{
	const unsigned taille = 7;

	_labels_taille = taille;

	_labels = new char*[taille];

	for(register unsigned i=0;i<taille;i++)
	{

	    if(labels && i<labels_taille && labels[i])
	    {
		_labels[i] = new char[strlen(labels[i])+1];

		strcpy(_labels[i],labels[i]);
	    }
	    else
	    {
		_labels[i] = new char[1];

		_labels[i][0] = '\0';
	    }

	}

	_pere = new PopUp(root, _labels[0],400,10);
}
  
Directory::~Directory(void)
{
	delete _pere;
	delete _LeTout;
	delete _LesBtnsJ;
	delete _LesLabsJ;
	delete _LesLabs2J;
	delete _LesListesJ;
	delete _LesBtns;
	delete _LesLabs;
	delete _LesLabs2;
	delete _LesListes;
	delete _direcs;
	delete _fichiers;
	delete _cancel;
	delete _ok;
	delete _help;
	delete _chemin;
	delete _fichier;
	delete _filtre;
	delete _chemin_label;
	delete _fichier_label;
	delete _filtre_label;

	for(register unsigned i=0;i<_labels_taille;i++)
	{
	    delete[] _labels[i];
	}

	delete[] _labels;
}

void
Directory::OKAction()
{
	_ok->SelectCB(NULL);
}

void
Directory::OKAction(char** selected,int nbselected)
{

	if(_ptr)
	{
	    const char* const nom_rep = GetNomRepertoire();

	    _ptr->CheminChoisi(nom_rep,0);
	    _ptr->FichierChoisi(selected,nbselected,0);
	}

}

void
Directory::SelectCB(XtPointer)
{

	if (!_done)
	{
	    _LeTout = new Conteneur(_pere->GetId(), "CNTRL0");

	    _LesListesJ = new ConteneurJoli(_LeTout->GetId(), "CNTRL3");
	    _LesListes = new Conteneur(_LesListesJ->GetId(), "CNTRL3");
	    _LesListes->Colonnes(1);

	    _LesBtnsJ = new ConteneurJoli(_LeTout->GetId(), "CNTRL3");
	    _LesBtns = new Conteneur(_LesBtnsJ->GetId(), "CNTRL3");
	    _LesBtns->Lignes(3);

	    _LesLabs2J = new ConteneurJoli(_LeTout->GetId(), "CNTRL3");
	    _LesLabs2 = new Conteneur(_LesLabs2J->GetId(), "CNTRL3");
	    _LesLabs2->Lignes(1);

	    _LesLabsJ = new ConteneurJoli(_LeTout->GetId(), "CNTRL3");
	    _LesLabs = new Conteneur(_LesLabsJ->GetId(), "CNTRL3");
	    _LesLabs->Lignes(2);

	    _fichier_label = new Label(_LesLabs2->GetId(),_labels[4]);
	    _fichier = new Label(_LesLabs2->GetId(), "");

	    _filtre_label = new Label(_LesLabs->GetId(),_labels[5]);
	    _filtre = new SaisieFiltre(_LesLabs->GetId(), "*",this);
	    _chemin_label = new Label(_LesLabs->GetId(),_labels[6]);
	    _chemin = new SaisieDir(_LesLabs->GetId(), "", this);

	    cout << "Directory> repertoire initial " <<
		_ptr->DirectoryInitial() << endl;

	    if(!_ptr || _ptr->DirectoryInitial() == NULL)
	    {
		_fichiers = new ScrollFileList(_LesListes->GetId(), ".",
				_fichier,this);
		_direcs = new ScrollDirList(_LesListes->GetId(), ".",
				_fichier,_chemin, _fichiers,_filtre);
	    }
	    else
	    {
		_fichiers = new ScrollFileList(_LesListes->GetId(),
				_ptr->DirectoryInitial(),_fichier,this);
		_direcs = new ScrollDirList(_LesListes->GetId(),
				_ptr->DirectoryInitial(),_fichier,_chemin,
				_fichiers,_filtre);
	    }

	    _ok = new OKButton(_LesBtns->GetId(), _labels[1], this, _ptr);
	    _cancel = new CancelButton(_LesBtns->GetId(), _labels[2], _pere);
	    _help = new AideButton(_LesBtns->GetId(), _labels[3],"BTN_OAA");

	    _pere->BloqueGeom();
	    _done = TRUE;
	}

	_pere->Montre();
}

char*
Directory::GetNomFichier()
{
	return _fichiers->GetNomFichier();
}

char*
Directory::GetNomRepertoire()
{
	return _direcs->GetNomRepertoire();
}

// *********************************************************
//         classe AideButton
// *********************************************************

void
AideButton::SelectCB(XtPointer call_data)
{
	Bouton::SelectCB(call_data);

	if(!_help)
	{
	    cout << "AideButton> creation pop-up-help Repertoire " << endl;

	    _help = new Help(GetId(),"Aide repertoire","REPERTOIRE_HELP");
	}	

	_help->Montre();
}

// *********************************************************
//         classe SaisieDir
// *********************************************************

SaisieDir::SaisieDir(const Widget& parent,const char* label,
		Directory* const pere) :
		Saisie(parent, label),
		_pere(pere)
{
}

void
SaisieDir::SelectCB(XtPointer)
{
	chdir(GetText());

	ScrollDirList* const dir = _pere->GetDir();
	SaisieFiltre* const filtre = _pere->GetFiltre();

	dir->ChangeListe(".",filtre->GetText());
}

// *********************************************************
//         classe SaisieFiltre
// *********************************************************

SaisieFiltre::SaisieFiltre(const Widget& parent,const char* label,
		Directory* const pere) :
		Saisie(parent, label),
		_pere(pere)
{
}

void
SaisieFiltre::SelectCB(XtPointer)
{
	ScrollDirList* const dir = _pere->GetDir();
	const char* const filtre = GetText();

	dir->ChangeListe(".",filtre);
}
