// ***********************************************************
// *    classe Driver SVG                                   *
// ***********************************************************

struct SccsIdU8svg
{
	static const char* const sccs_id;
};

const char* const SccsIdU8svg::sccs_id = "%W% %G% Pierre Brochard";

#include "U8constant.h"
#include "poulpe/U8svg.h"
#include "sanglier.h"
#include "L3ListeDesCodes.h"
#include "nx.h"
#include "dinosaure/S6string.h"

#ifndef TRUE
#   define TRUE 1
#endif

#ifndef FALSE
#   define FALSE 0
#endif

#define MAX_STRING   256

#define S6Max(a,b) (((a)>(b))? (a):(b))
#define S6Min(a,b) (((a)<(b))? (a):(b))

// *************************************************************************
// * implementation classe U8SVG (driver pour le standard W3C SVG)
// *************************************************************************

int U8SVG::_numfile = 0;
U8SVG* U8SVG::_svg = NULL;
const int U8SVG::_maxcolors = 256;
const int U8SVG::_defncolors = 16;
const char* U8SVG::_defaultmap[U8SVG::_defncolors] = 
{
   "#ffffff","#0000ff","#00ff00","#ffff00",
   "#ff8000","#ff0000","#ff00ff","#c8c8c8",
   "#8c8c8c","#00ffff","#74bf40","#8c8000",
   "#ffaa91","#ffc8c8","#bf00bf","#000000"
};

U8SVG::U8SVG() :
		_coul(new char[8*_maxcolors]),
		_filedebut(NULL),
		_filename(NULL),
		_filenx(NULL),
		_file(NULL),
		_factor(1./30.),
		_xmin(-32768),
		_xmax(32767),
		_ymin(-32768),
		_ymax(32767),
		_a(_factor),
		_b(-_factor*_xmin),
		_c(-_factor),
		_d(_factor*_ymax),
		_scaley(65535/512*_factor),
		_ncolors(0),
		_color(-2),
		_opengroup(FALSE)
{
}

U8SVG::~U8SVG()
{
	delete[] _coul;
	delete[] _filedebut;
	delete[] _filename;
}

void
U8SVG::NomDebutFichier(const char* filedebut)
{
	delete _filedebut;

	S6String ss(filedebut);

	_filedebut = ss;
}

// ***********************************************************
//         fonctions L3* appelees par L3Driver
// ***********************************************************

void
U8SVG::L3Open(const int [])
{
	_color = -2;
	_opengroup = FALSE;

	{
	    ostrstream os;

	    os << _filedebut << setfill('0') << setw(4) << _numfile <<
		".svg" << ends;
	    _filename = os.str();
	}

	_filenx = fnxopen(_filename,"w");
	_file = fnx_write_desc(_filenx);
	fprintf(_file,"<?xml version=\"1.0\" standalone=\"yes\"?>\n");
	fprintf(_file,"<!DOCTYPE svg SYSTEM \"svg.dtd\">\n");

	_a = _factor;
	_c = -_factor;

	const int xmin = int(_a * double(_xmin) + _b);
	const int ymin = int(_c * double(_ymin) + _d);
	const int xmax = int(_a * double(_xmax) + _b);
	const int ymax = int(_c * double(_ymax) + _d);
	const int xminv = S6Min(xmin,xmax);
	const int xmaxv = S6Max(xmin,xmax);
	const int yminv = S6Min(ymin,ymax);
	const int ymaxv = S6Max(ymin,ymax);

    // Pour taille des textes et epaisseur des traits :
	_scaley = (ymaxv - yminv)/512.; 

    // En SVG, (xmin,ymin) est mappe sur le coin en haut et a gauche
    // de la zone reservee dans le browser.
    // fill="1" stroke="0" pour les polygones (defaut du "graphics state")

	fprintf(_file,"<svg xmlns=\"http://www.w3.org/2000/svg\" width=\"512\" "
		"height=\"512\" viewBox=\"%d %d %d %d\">\n",
		xminv,yminv,xmaxv,ymaxv);
	fprintf(_file,"<g style=\"fill:none;stroke:none;stroke-width:%g\">\n",
		_scaley);

    // init table de couleurs

	_ncolors = _defncolors;
	_ncolors = _ncolors > _maxcolors ? _maxcolors : _ncolors;

	int i = 0,j = 0;

	for (; j < _defncolors;i+=8,j++)
	{
	    sprintf(_coul+i,"%s\0",_defaultmap[j]);
	}

}

void
U8SVG::L3Close(const int [])
{

	if(imagevide)
	{
	    fnxclose(_filenx);
	    unlink(_filename);
	}
	else
	{

	    if(_opengroup)
	    {
		fprintf(_file,"</g>\n");
	    }

	    fprintf(_file,"</g>\n</svg>\n");
	    fnxclose(_filenx);
	    _numfile++;
	}

	_filenx = NULL;
	_file = NULL;
	delete _filename;_filename = NULL;
}

// items de controle

void
U8SVG::L3Cadre(const int bloc_l3[])
{
	register const int cadrex = bloc_l3[2];
	register const int cadrey = bloc_l3[3];

    // Est ce utile : je ne crois pas

	_a = _factor * double(cadrex / 185.);
	_c = -_factor * double(cadrey / 185.);
}

void
U8SVG::L3Colormap(const int bloc_l3[])
{

	if (bloc_l3[0] < 8) return;

	_ncolors = bloc_l3[2];
	_ncolors = _ncolors > _maxcolors ? _maxcolors : _ncolors;

	const int imax = 8*_ncolors;
	int i = 0,j = 3;

	for(;i<imax;i+=8,j+=3)
	{
	    sprintf(_coul+i,"#%2.2x%2.2x%2.2x\0",bloc_l3[j],bloc_l3[j+1],
			    bloc_l3[j+2]);
	}

}

void
U8SVG::L3Couleur(const int bloc_l3[])
{
	const int color = bloc_l3[2];

	if (color != _color)
	{

	    if(_opengroup)
	    {
		fprintf(_file,"</g>\n");
	    }
	    
	    _color = (color < _ncolors) ? color : _ncolors - 1;
	    _opengroup = TRUE;

	    const char* const coul = _coul+8*_color;

	    fprintf(_file,"<g style=\"fill:%s;stroke:%s\">",coul,coul);
	}

}

void
U8SVG::L3Flush(const int[])
{
	fflush(_file);
}

void
U8SVG::L3Default(const int[])
{
}

// Items de trace

void
U8SVG::L3Texte(const int bloc_l3[])
{
	char tstring[MAX_STRING];
	register const int* const p = bloc_l3 + 1;
	const int len = *(p + 5);

	if(len > MAX_STRING - 1)
	{
	    fprintf(stderr, "Chaine trop longue non traitee\n");
	    return;
	}

	const char* chaine = (char*) (p + 6);

	for(register int ii = 0;ii<len;ii++)
	{
	    tstring[ii] = chaine[ii+2*(ii/2)+2];
	}

	tstring[len] = '\0';

    // 18.5 (cm) / 2.5 (=> pouces) * 72 (pt/pouce) = 504 puis * 2 <=> soft
    // car la taille de la fonte est une hauteur. On considere que la largeur
    // est la moitie.
	const double cx = _a/_factor*_scaley;
	const double cy = -_c/_factor*_scaley;
	const double cr = sqrt((cx*cx+cy*cy)*.5);
	const int ch_he = int(1008.*cr/p[3]);
	const int xl3 = int(_a * double(p[1]) + _b);
	const int yl3 = int(_c * double(p[2]) + _d);

	fprintf(_file,"<text x=\"%d\" y=\"%d\" style=\"font-size:%d;"
			"font-family:Courier\">%s</text>",xl3,yl3,ch_he,tstring);

    /* Traitement de l'angle a voir :
	const double ang = double(p[4]) / 1024.0;
	const double atang = atan(cx != 0. ? tan(ang)*cy/cx : 0.);
	const double sang = cos(ang) >= 0. ? atang : atang + M_PI;*/

}

void
U8SVG::L3Pixel(const int bloc_l3[])
{
}

void
U8SVG::L3Marqueur(const int bloc_l3[])
{
	const int* const p = bloc_l3 + 1;
	const int Xm = int(_a * p[1] + _b);
	const int Ym = int(_c * p[2] + _d);
	const int clip = int(_taillel3*_factor);

    /* test du clipping */

	if ((Xm < 0) || (Xm > clip)) return;

	if ((Ym < 0) || (Ym > clip)) return;

	const int d1 = 16;
	const int d2 = 32;

    /*
      Sauf pour les lettres, on dessine directement le marqueur
      en segments
    */

	switch (p[3])
	{
	  case 1:
	/* '+' */
	    fprintf(_file,"<g>");
	    fprintf(_file,"<polyline points=\"%d,%d,%d,%d\"/>\n",
			    Xm,Ym-d2,Xm,Ym+d2);
	    fprintf(_file,"<polyline points=\"%d,%d,%d,%d\"/>\n",
			    Xm-d2,Ym,Xm+d2,Ym);
	    fprintf(_file,"</g>");
	    break;
	  case 2:
	/* '*' */
	    fprintf(_file,"<g>");
	    fprintf(_file,"<polyline points=\"%d,%d,%d,%d\"/>\n",
			    Xm,Ym-d2,Xm,Ym+d2);
	    fprintf(_file,"<polyline points=\"%d,%d,%d,%d\"/>\n",
			    Xm-d2,Ym,Xm+d2,Ym);
	    fprintf(_file,"<polyline points=\"%d,%d,%d,%d\"/>\n",
			    Xm-d2,Ym-d2,Xm+d2,Ym+d2);
	    fprintf(_file,"<polyline points=\"%d,%d,%d,%d\"/>\n",
			    Xm-d2,Ym+d2,Xm+d2,Ym-d2);
	    fprintf(_file,"</g>");
	    break;
	  case 3:
	/* 'o' */
	    fprintf(_file,"<g>");
	    fprintf(_file,"<polyline points=\"%d,%d,%d,%d,%d,%d,"
			    "%d,%d,%d,%d\"/>\n",
			    Xm-d2,Ym-d2,Xm-d2,Ym+d2,Xm+d2,Ym+d2,
			    Xm+d2,Ym-d2,Xm-d2,Ym-d2);
	    fprintf(_file,"</g>");
	    break;
	  case 4:
	/* 'x' */
	    fprintf(_file,"<g>");
	    fprintf(_file,"<polyline points=\"%d,%d,%d,%d\"/>\n",
			    Xm-d2,Ym-d2,Xm+d2,Ym+d2);
	    fprintf(_file,"<polyline points=\"%d,%d,%d,%d\"/>\n",
			    Xm-d2,Ym+d2,Xm+d2,Ym-d2);
	    fprintf(_file,"</g>");
	    break;
	  default:
	/* '.' ou lettre (centree) */

	    if (*p > 256)
	    {
		const char* const temp = ((char*) p) + sizeof(int) - 2;
		char tstring[2];

		tstring[0] = temp[0];
		tstring[1] = '\0';

		fprintf(_file,"<text x=\"%d\" y=\"%d\" style=\"font-size:%d;"
			"font-family:Courier\">%s</text>",Xm,Ym,12,tstring);
	    }
	    else // gros point
	    {
		fprintf(_file,"<g>");
		fprintf(_file,"<polyline points=\"%d,%d,%d,%d,%d,%d,"
				"%d,%d,%d,%d\"/>\n",
				Xm-d1,Ym-d1,Xm-d1,Ym+d1,Xm+d1,Ym+d1,
				Xm+d1,Ym-d1,Xm-d1,Ym-d1);
		fprintf(_file,"</g>");
	    } /* fin else if *p > 256 */

	}

}

void
U8SVG::L3Segment(const int bloc_l3[])
{
	register const int nb2 = bloc_l3[0]/2 - 2;
	register const int ix1 = int(_a * double(bloc_l3[2]) + _b);
	register const int iy1 = int(_c * double(bloc_l3[3]) + _d);
	register int i = 0;

	fprintf(_file,"<polyline points=\"%d,%d",ix1,iy1);

	for(;i < nb2;i++)
	{
	    register const int ix2 = int(_a * double(bloc_l3[4+2*i]) + _b);
	    register const int iy2 = int(_c * double(bloc_l3[5+2*i]) + _d);

	    fprintf(_file,",%d,%d",ix2,iy2);
	}

	fprintf(_file,"\"/>\n");
}

void
U8SVG::L3Polygone(const int bloc_l3[])
{
	register const int nb2 = bloc_l3[0]/2 - 2;
	register const int ix1 = int(_a * double(bloc_l3[2]) + _b);
	register const int iy1 = int(_c * double(bloc_l3[3]) + _d);
	register int i = 0;

	fprintf(_file,"<polygon points=\"%d,%d",ix1,iy1);

	for(;i < nb2;i++)
	{
	    register const int ix2 = int(_a * double(bloc_l3[4+2*i]) + _b);
	    register const int iy2 = int(_c * double(bloc_l3[5+2*i]) + _d);

	    fprintf(_file,",%d,%d",ix2,iy2);
	}

	fprintf(_file,",%d,%d\"/>\n",ix1,iy1);
}
// Fin de Fichier
