/****************/
/* L3Croix.c */
/****************/

#include <stdio.h>

#include "L3Environnement.h"
#include "L3Croix.h"

#ifndef lint
    static const char sccs_id[] = "%W% %G% G. Colin de Verdiere";
#endif

/*-
 * Procedure utilisateur : L3Croix
 *--------------------------------------------------------------------------*
 * Parametres :
 *   sousimage    : numero de la sous-image concernee
 *   x            : tableau des abcisses utilisateur des croix
 *   y            : tableau des ordonnees utilisateur des croix
 *   npoints      : nombre de points a marquer
 *   taille	  : taille de la croix (1. = pleine page)
 *
 * Valeur de retour : aucune
 *
 *
 * Erreurs :
 *   nombre de points a marquer negatif ou nul
 *   taille incorrectype de marqueur
 ****************************************************************************/

void
L3Croix(int sousimage, REEL x[], REEL y[], int npoints, REEL taille)
{
   int             i;
   int             codel3[10];
   int             xl3, yl3;
   REEL            longu = (LARGEURPAGEL3 * taille) / 2;
   int             atracer[4];

   if (npoints > 0) {
      if ((taille > 0.) && (taille <= 1.)) {
	 SETSOUSIMAGE(sousimage);

	 for (i = 0; i < npoints; i++) {
	    L3ConversionCoordonnees(x[i], y[i], &xl3, &yl3);
	    if (L3ClippingSegment(xl3 - (int) longu, yl3,
				  xl3 + (int) longu, yl3,
				  atracer) == VRAI) {
	       codel3[0] = 6;
	       codel3[1] = CODEL3SEGMENT;
	       codel3[2] = atracer[0];
	       codel3[3] = atracer[1];
	       codel3[4] = atracer[2];
	       codel3[5] = atracer[3];
	       L3AppelDrivers(codel3);
	    }
	    if (L3ClippingSegment(xl3, yl3 - (int) longu,
				  xl3, yl3 + (int) longu,
				  atracer) == VRAI) {
	       codel3[0] = 6;
	       codel3[1] = CODEL3SEGMENT;
	       codel3[2] = atracer[0];
	       codel3[3] = atracer[1];
	       codel3[4] = atracer[2];
	       codel3[5] = atracer[3];
	       L3AppelDrivers(codel3);
	    }
	 }
      } else {
	 fprintf(stderr,
			"L3Croix : taille incorrecte :%f\n", taille);
      }
   } else {
      fprintf(stderr,
		     "L3Croix : nombre de croix negatif ou nul :%d\n", npoints);
   }
}
