/****************/
/* L3DefTexte.c */
/****************/

#include <stdio.h>
#include <math.h>

#include "L3Environnement.h"
#include "L3DefTexte.h"

#ifndef lint
    static const char sccs_id[] = "%W% %G% DMA/AIM/LM";
#endif

/*******************************************************************************
 * Procedure utilisateur : L3DefTexte                                          *
 *-----------------------------------------------------------------------------*
 * Parametres :                                                                *
 *   sousimage   : numero de la sous-image concernee                           *
 *   ncaracligne : nombre de caracteres par ligne                              *
 *                   valeur par defaut a l'initialisation : 128                *
 *   angle       : angle d'orientation du texte en degres                      *
 *                   valeur par defaut a l'initialisation : 0.0                *
 *                                                                             *
 * Valeur de retour : aucune.                                                  *
 *                                                                             *
 * Description : voir document CEL.V DMA/SIE/IG NT 58.9 de juillet 1987        *
 *          "GRAF CIGALE"                                                      *
 *                                                                             *
 * Erreurs :                                                                   *
 *   nombre de caracteres par ligne incorrect                                  *
 *   angle d'orientation du texte incorrect                                    *
 *******************************************************************************/

void
L3DefTexte(int sousimage,int ncaracligne,double angle)
{

	if (ncaracligne > 0)
	{

		if (angle >= -180.0 && angle <= 180.0)
		{
			SETSOUSIMAGE(sousimage);

			switch (ncaracligne)
			{
			    case 1:
				NBCAR = 128;
				break;
			    case 2:
				NBCAR = 73;
				break;
			    case 3:
				NBCAR = 48;
				break;
			    case 4:
				NBCAR = 36;
				break;
			    case 5:
				NBCAR = 29;
				break;
			    case 6:
				NBCAR = 24;
				break;
			    case 7:
				NBCAR = 20;
				break;
			    case 8:
				NBCAR = 18;
				break;
			    default:
				NBCAR = ncaracligne;
			}

			ANGLETEXTE = angle * L3Pi / 180.0;
		}
		else
		{
			/*----------------------------------------*/
			/* Erreur : valeur de l'angle incorrecte. */
			/*----------------------------------------*/

			fprintf(stderr,"L3DefTexte : valeur de l'angle ");
			fprintf(stderr,"incorrecte :%f\n", angle);
		}

	}
	else
	{
		/*----------------------------------------------------*/
		/* Erreur : nombre de caracteres par ligne incorrect. */
		/*----------------------------------------------------*/

		fprintf(stderr,"L3DefTexte : nombre de caracteres ");
		fprintf(stderr,"par ligne incorrect :%d\n", ncaracligne);
	}

}
