/*
 * Module interface qui permet d'appeler le C depuis FORTRAN
 * pour les routines 3D
 */

/* Auteur G. Colin de Verdiere DMA/SIE/IG %H% */

#ifndef lint
   static const char* const sccs_id = "%W% %G% G. Colin de Verdiere";
#endif

#include <stdio.h>
#include <malloc.h>
#include <math.h>
#include <ctype.h>
#include <string.h>

#if defined(SUNOS) && defined(SUNOS_4)
    void bcopy(void* b1,void* b2,int length);
#   define memmove(a,b,c) bcopy((b),(a),(c))
#endif

#ifdef CRAY
#  include <fortran.h>
#endif

#include "L3Environnement.h"
#include "L3Ftn.h"

/*-
				 #####  ######
				#     # #     #
				      # #     #
				 #####  #     #
				      # #     #
				#     # #     #
				 #####  ######
 */

/* --------------------------------- LINEZ -------------------------- */
/*
 * void linez(int i, float *x, float *y, float *z, int N) cf line mais en 3D
 */
void
#ifdef CRAY_PROTO
LINEZ(im, x1, y1, z1, n)
#else
FUNCDECL(linez)(im, x1, y1, z1, n)
#endif
	int            *im, *n;
	float          *x1, *y1, *z1;
{
	REEL           *rx1, *ry1, *rz1;
	int             i;

	rx1 = (REEL *) malloc((unsigned) (*n * sizeof(REEL)));
	ry1 = (REEL *) malloc((unsigned) (*n * sizeof(REEL)));
	rz1 = (REEL *) malloc((unsigned) (*n * sizeof(REEL)));

	for (i = 0; i < *n; i++) {
		rx1[i] = (REEL) x1[i];
		ry1[i] = (REEL) y1[i];
		rz1[i] = (REEL) z1[i];
	}

	L3LigneBrisee3D(*im, rx1, ry1, rz1, *n);

	free(rx1);
	free(ry1);
	free(rz1);
}

/* --------------------------------- PLOTZ -------------------------- */
/*
 * void linez(int i, float *x, float *y, float *z, int N, int kod) cf plot
 * mais en 3D
 */

void
#ifdef CRAY_PROTO
PLOTZ(im, x1, y1, z1, n, kod)
#else
FUNCDECL(plotz)(im, x1, y1, z1, n, kod)
#endif
	int            *im, *n, *kod;
	float          *x1, *y1, *z1;
{
	REEL           *rx1, *ry1, *rz1;
	int             i;

	rx1 = (REEL *) malloc((unsigned) (*n * sizeof(REEL)));
	ry1 = (REEL *) malloc((unsigned) (*n * sizeof(REEL)));
	rz1 = (REEL *) malloc((unsigned) (*n * sizeof(REEL)));

	for (i = 0; i < *n; i++) {
		rx1[i] = (REEL) x1[i];
		ry1[i] = (REEL) y1[i];
		rz1[i] = (REEL) z1[i];
	}

	L3Marqueur3D(*im, rx1, ry1, rz1, *n, *kod);

	free(rx1);
	free(ry1);
	free(rz1);
}

/* --------------------------------- STRTZ -------------------------- */
/*
 * void strtz(int i, float *x, float *y, float *z) cf strt mais en 3D
 */

void
#ifdef CRAY_PROTO
STRTZ(im, x1, y1, z1)
#else
FUNCDECL(strtz)(im, x1, y1, z1)
#endif
	int            *im;
	float          *x1, *y1, *z1;
{
	REEL            rx1 = (REEL) (*x1);
	REEL            ry1 = (REEL) (*y1);
	REEL            rz1 = (REEL) (*z1);

	L3PointCourant3D(*im, rx1, ry1, rz1);

}

/* --------------------------------- SGMTZ -------------------------- */
/*
 * void sgmtz(int i, float *x1, float *y1, float *z1, float *x2, float *y2,
 * float *z2, int N) cf sgmt mais en 3D
 */
void
#ifdef CRAY_PROTO
SGMTZ(im, x1, y1, z1, x2, y2, z2, n)
#else
FUNCDECL(sgmtz)(im, x1, y1, z1, x2, y2, z2, n)
#endif
	int            *im, *n;
	float          *x1, *y1, *z1;
	float          *x2, *y2, *z2;
{
	REEL           *rx1, *ry1, *rz1, *rx2, *ry2, *rz2;
	int             i;

	rx1 = (REEL *) malloc((unsigned) (*n * sizeof(REEL)));
	ry1 = (REEL *) malloc((unsigned) (*n * sizeof(REEL)));
	rz1 = (REEL *) malloc((unsigned) (*n * sizeof(REEL)));
	rx2 = (REEL *) malloc((unsigned) (*n * sizeof(REEL)));
	ry2 = (REEL *) malloc((unsigned) (*n * sizeof(REEL)));
	rz2 = (REEL *) malloc((unsigned) (*n * sizeof(REEL)));

	for (i = 0; i < *n; i++) {
		rx1[i] = (REEL) x1[i];
		ry1[i] = (REEL) y1[i];
		rz1[i] = (REEL) z1[i];
		rx2[i] = (REEL) x2[i];
		ry2[i] = (REEL) y2[i];
		rz2[i] = (REEL) z2[i];
	}

	L3Segment3D(*im, rx1, ry1, rz1, rx2, ry2, rz2, *n);

	free(rx1);
	free(ry1);
	free(rz1);
	free(rx2);
	free(ry2);
	free(rz2);
}

/* --------------------------------- DSPACZ -------------------------- */
/*
 * void dspacz(int i, float ox, float oy, float oz, float vx, float vy, float
 * vz, float tga) coordonnees du point de vue (oeil) ox oy oz coordonnees du
 * point vise          vx vy vz tangente du 1/2 angle de champ     tga
 */
void
#ifdef CRAY_PROTO
DSPACZ(im, x1, y1, z1, x2, y2, z2, tga)
#else
FUNCDECL(dspacz)(im, x1, y1, z1, x2, y2, z2, tga)
#endif
	int            *im;
	float          *x1, *y1, *z1;
	float          *x2, *y2, *z2;
	float          *tga;
{
	REEL            rx1 = (REEL) (*x1);
	REEL            ry1 = (REEL) (*y1);
	REEL            rz1 = (REEL) (*z1);
	REEL            rx2 = (REEL) (*x2);
	REEL            ry2 = (REEL) (*y2);
	REEL            rz2 = (REEL) (*z2);
	double            dtga = (double) (*tga);

	L3DefSpace3D(*im, rx1, ry1, rz1, rx2, ry2, rz2, dtga);

}

/* --------------------------------- PHOTO -------------------------- */
/*
 * void photo(float x, float y, float z, float u, float v, int k) coordonnees
 * xyz dans l'espace -> coordonnees UV [-1,1]X[-1,1] du point 2D dans
 * l'espace de l'image. CODE RETOUR : K=1 point devant l'oeil k=2 derriere
 */
void
#ifdef CRAY_PROTO
PHOTO(im, x1, y1, z1, u, v, k)
#else
FUNCDECL(photo)(im, x1, y1, z1, u, v, k)
#endif
	int            *im;
	float          *x1, *y1, *z1;
	float          *u, *v;
	int            *k;
{
	REEL            rx1 = (REEL) (*x1);
	REEL            ry1 = (REEL) (*y1);
	REEL            rz1 = (REEL) (*z1);
	REEL            ru, rv;

	L3Projection2D(*im, rx1, ry1, rz1, &ru, &rv, k);
	*u = (float) ru;
	*v = (float) rv;
}

/* --------------------------------- MAPL3 -------------------------- */

void
#ifdef CRAY_PROTO
MAPL3(a, x, y, z, xp, yp, zp, n)
#else
FUNCDECL(mapl3)(a, x, y, z, xp, yp, zp, n)
#endif
	float           a[4][4];
	float          *x, *y, *z, *xp, *yp, *zp;
	int            *n;
{
	/*
	 * Attention ordre des indices inverse entre C et FORTRAN
	 */
	double          xprov, yprov, wprov, zprov;
	int             i;

	for (i = 0; i < *n; i++)
	{
	    wprov = (a[0][3] * x[i] + a[1][3] * y[i] + a[2][3] * z[i] +
			    a[3][3]);
	    xprov = (a[0][0] * x[i] + a[1][0] * y[i] + a[2][0] * z[i] +
			    a[3][0]) / wprov;
	    yprov = (a[0][1] * x[i] + a[1][1] * y[i] + a[2][1] * z[i] +
			    a[3][1]) / wprov;
	    zprov = (a[0][2] * x[i] + a[1][2] * y[i] + a[2][2] * z[i] +
			    a[3][2]) / wprov;
	    xp[i] = (float) xprov;
	    yp[i] = (float) yprov;
	    zp[i] = (float) zprov;
	}

}

/* --------------------------------- UNIT3 -------------------------- */

void
#ifdef CRAY_PROTO
UNIT3(float x[4][4])
#else
FUNCDECL(unit3)(float x[4][4])
#endif
{
	static float u[16] =
	{
	    1., 0., 0., 0.,
	    0., 1., 0., 0.,
	    0., 0., 1., 0.,
	    0., 0., 0., 1.
	};

	memmove((char*) x, (char*) u, 16 * sizeof(float));
}

/* --------------------------------- ROTX3 -------------------------- */

void
#ifdef CRAY_PROTO
ROTX3(float a[4][4],float* al)
#else
FUNCDECL(rotx3)(float a[4][4],float* al)
#endif
{
	/*
	 * Attention ordre des indices inverse entre C et FORTRAN
	 */

#ifdef CRAY_PROTO
	UNIT3(a);
#else
	FUNCDECL(unit3)(a);
#endif
	a[1][1] = (float) cos((double) *al);
	a[2][1] = -(float) sin((double) *al);
	a[1][2] = (float) sin((double) *al);
	a[2][2] = (float) cos((double) *al);
}

/* --------------------------------- ROTY3 -------------------------- */

void
#ifdef CRAY_PROTO
ROTY3(float a[4][4],float* al)
#else
FUNCDECL(roty3)(float a[4][4],float* al)
#endif
{
	/*
	 * Attention ordre des indices inverse entre C et FORTRAN
	 */

#ifdef CRAY_PROTO
	UNIT3(a);
#else
	FUNCDECL(unit3)(a);
#endif
	a[0][0] = (float) cos((double) *al);
	a[2][0] = (float) sin((double) *al);
	a[0][2] = -(float) sin((double) *al);
	a[2][2] = (float) cos((double) *al);
}
/* --------------------------------- ROTZ3 -------------------------- */
void
#ifdef CRAY_PROTO
ROTZ3(a, al)
#else
FUNCDECL(rotz3)(a, al)
#endif
	float           a[4][4], *al;
{
	/*
	 * Attention ordre des indices inverse entre C et FORTRAN
	 */

#ifdef CRAY
	UNIT3(a);
#else
	unit3_(a);
#endif
	a[0][0] = (float) cos((double) *al);
	a[1][0] = -(float) sin((double) *al);
	a[1][0] = (float) sin((double) *al);
	a[1][1] = (float) cos((double) *al);
}
/* --------------------------------- TRAN3 -------------------------- */
void
#ifdef CRAY_PROTO
TRAN3(a, tx, ty, tz)
#else
FUNCDECL(tran3)(a, tx, ty, tz)
#endif
	float           a[4][4], *tx, *ty, *tz;
{
	/*
	 * Attention ordre des indices inverse entre C et FORTRAN
	 */

#ifdef CRAY
	UNIT3(a);
#else
	unit3_(a);
#endif
	a[3][0] = *tx;
	a[3][1] = *ty;
	a[2][2] = *tz;
}
/* --------------------------------- SCAL3 -------------------------- */
void
#ifdef CRAY_PROTO
SCAL3(a, tx, ty, tz)
#else
FUNCDECL(scal3)(a, tx, ty, tz)
#endif
	float           a[4][4], *tx, *ty, *tz;
{
	/*
	 * Attention ordre des indices inverse entre C et FORTRAN
	 */

#ifdef CRAY
	UNIT3(a);
#else
	unit3_(a);
#endif
	a[0][0] = *tx;
	a[1][1] = *ty;
	a[2][2] = *tz;
}

/* --------------------------------- MULT3 -------------------------- */

void
#ifdef CRAY_PROTO
MULT3(float a[4][4],float b[4][4],float c[4][4])
#else
FUNCDECL(mult3)(float a[4][4],float b[4][4],float c[4][4])
#endif
{
	/*
	 * Attention ordre des indices inverse entre C et FORTRAN
	 */

	float           D[4][4];
	int             i, j;

	for (i = 0; i < 4; i++)
	{

	    for (j = 0; j < 4; j++)
	    {
		D[j][i] = (float) (
				(double) b[0][i] + (double) a[j][0] +
		(double) b[1][i] + (double) a[j][1] +
		(double) b[2][i] + (double) a[j][2] +
		(double) b[3][i] + (double) a[j][3]
				);
	    }

	}

	memmove((char*) c, (char*) D, 16 * sizeof(float));
}

/* --------------------------------- TRAN3P -------------------------- */

void
#ifdef CRAY_PROTO
TRAN3P(a, tx, ty, tz)
#else
FUNCDECL(tran3p)(a, tx, ty, tz)
#endif
	float           a[4][4], *tx, *ty, *tz;
{
	/*
	 * Attention ordre des indices inverse entre C et FORTRAN
	 */

	float           m[4][4];


#ifdef CRAY_PROTO
	TRAN3(m, tx, ty, tz);
#else
	FUNCDECL(tran3)(m, tx, ty, tz);
#endif

#ifdef CRAY
	MULT3(a, m, a);
#else
	mult3_(a, m, a);
#endif
}

/* --------------------------------- SCAL3P -------------------------- */

void
#ifdef CRAY_PROTO
SCAL3P(a, tx, ty, tz)
#else
FUNCDECL(scal3p)(a, tx, ty, tz)
#endif
	float           a[4][4], *tx, *ty, *tz;
{
	/*
	 * Attention ordre des indices inverse entre C et FORTRAN
	 */

	float           m[4][4];

#ifdef CRAY
	SCAL3(m, tx, ty, tz);
#else
	scal3_(m, tx, ty, tz);
#endif

#ifdef CRAY
	MULT3(a, m, a);
#else
	mult3_(a, m, a);
#endif
}

/* --------------------------------- ROTX3P -------------------------- */
void
#ifdef CRAY_PROTO
ROTX3P(a, al)
#else
FUNCDECL(rotx3p)(a, al)
#endif
	float           a[4][4], *al;
{
	/*
	 * Attention ordre des indices inverse entre C et FORTRAN
	 */

	float           m[4][4];

#ifdef CRAY_PROTO
	ROTX3(m, al);
	MULT3(a, m, a);
#else
	FUNCDECL(rotx3)(m, al);
	FUNCDECL(mult3)(a, m, a);
#endif

}

/* --------------------------------- ROTY3P -------------------------- */
void
#ifdef CRAY_PROTO
ROTY3P(a, al)
#else
FUNCDECL(roty3p)(a, al)
#endif
	float           a[4][4], *al;
{
	/*
	 * Attention ordre des indices inverse entre C et FORTRAN
	 */

	float           m[4][4];

#ifdef CRAY_PROTO
	ROTY3(m, al);
	MULT3(a, m, a);
#else
	FUNCDECL(roty3)(m, al);
	FUNCDECL(mult3)(a, m, a);
#endif

}

/* --------------------------------- ROTZ3P -------------------------- */
void
#ifdef CRAY_PROTO
ROTZ3P(a, al)
#else
FUNCDECL(rotz3p)(a, al)
#endif
	float           a[4][4], *al;
{
	/*
	 * Attention ordre des indices inverse entre C et FORTRAN
	 */

	float           m[4][4];

#ifdef CRAY_PROTO
	ROTZ3(m, al);
	MULT3(a, m, a);
#else
	FUNCDECL(rotz3)(m, al);
	FUNCDECL(mult3)(a, m, a);
#endif

}
/* Fin de Fichier */
