/*
 * Module interface qui permet d'appeler le C depuis FORTRAN
 */

/* Auteur G. Colin de Verdiere DMA/SIE/IG %H% */

#ifndef lint
    static const char sccs_id[] = "%W% %G% G. Colin de Verdiere";
#endif

#include <stdio.h>
#include <malloc.h>
#include <math.h>
#include <ctype.h>
#include <string.h>

#ifdef CRAY
#include <fortran.h>
#endif

#include "L3Environnement.h"
#include "L3Ftn.h"

/* --------------------------------- RESCUR -------------------------- */
/*
 * void rescur(void) restaure la position courante du curseur texte
 */
void
#ifdef CRAY_PROTO
RESCUR()
#else
FUNCDECL(rescur)()
#endif
{
   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "RESCUR");
      fprintf(FicTrace, "\n");
   }
   L3RestoreCurseur();
}

/* --------------------------------- SAVCUR -------------------------- */
/*
 * void savcur(void) sauvegarde la position courante du curseur texte
 */
void
#ifdef CRAY_PROTO
SAVCUR()
#else
FUNCDECL(savcur)()
#endif
{
   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "SAVCUR");
      fprintf(FicTrace, "\n");
   }
   L3SaveCurseur();
}

/* --------------------------------- GETPT -------------------------- */
/*
 * void getpt(int i, float *x, float *y) coordonnees du point designe dans
 * l'espace utilisateur de la sous image i
 */
void
#ifdef CRAY_PROTO
GETPT(int *im, float *x, float *y)
#else
FUNCDECL(getpt)(int *im, float *x, float *y)
#endif
{
   REEL            rx = (REEL) (*x);
   REEL            ry = (REEL) (*y);
   int             codel3[14];

   SETSOUSIMAGE(*im);

   L3DialogueGETPOS(codel3, GRANDECROIX,
		    SANSELASTIQUE,
		    SANSMARQUE,
		    SANSTRACE);
   L3ConvCoordL3VersUtil(codel3[7], codel3[8], &rx, &ry);

   *x = rx;
   *y = ry;

   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "GETPT");
      if (L3DebugFlag >= 2) {
	 fprintf(FicTrace, " : %d %f %f", *im, *x, *y);
      }
      fprintf(FicTrace, "\n");
   }
}

/* --------------------------------- GETPTN -------------------------- */
/*
 * void getpt(int i, float *x, float *y, int *ch)
 * idem getpt mais en plus retourne le code de la touche frappee
 */
void
#ifdef CRAY_PROTO
GETPTN(im, x, y, ch)
#else
FUNCDECL(getptn)(im, x, y, ch)
#endif
   int            *im, *ch;
   float          *x, *y;
{
   REEL            rx = (REEL) (*x);
   REEL            ry = (REEL) (*y);
   int             codel3[14];

   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "GETPTN");
      if (L3DebugFlag >= 2) {
	 fprintf(FicTrace, " : %d", *im);
      }
      fprintf(FicTrace, "\n");
   }
   SETSOUSIMAGE(*im);

   L3DialogueGETPOS(codel3, GRANDECROIX,
		    SANSELASTIQUE,
		    SANSMARQUE,
		    SANSTRACE);
   L3ConvCoordL3VersUtil(codel3[7], codel3[8], &rx, &ry);

   *ch = codel3[9];
   *x = rx;
   *y = ry;
}

/* --------------------------------- GETPTR -------------------------- */
/*
 * void getptr(int i, float *x, float *y) coordonnees du point designe dans
 * l'espace utilisateur de la sous image i avec le rectangle elastique
 */
void
#ifdef CRAY_PROTO
GETPTR(im, x, y)
#else
FUNCDECL(getptr)(im, x, y)
#endif
   int            *im;
   float          *x, *y;
{
   REEL            rx = (REEL) (*x);
   REEL            ry = (REEL) (*y);
   int             codel3[14];

   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "GETPTR");
      if (L3DebugFlag >= 2) {
	 fprintf(FicTrace, " : %d", *im);
      }
      fprintf(FicTrace, "\n");
   }
   SETSOUSIMAGE(*im);

   L3DialogueGETPOS(codel3, GRANDECROIX,
		    RECTANGLEELASTIQUE,
		    SANSMARQUE,
		    SANSTRACE);
   L3ConvCoordL3VersUtil(codel3[7], codel3[8], &rx, &ry);

   *x = rx;
   *y = ry;
}

/* --------------------------------- GETPTV -------------------------- */
/*
 * void getptr(int i, float *x, float *y) coordonnees du point designe dans
 * l'espace utilisateur de la sous image i avec le vecteur elastique
 */
void
#ifdef CRAY_PROTO
GETPTV(im, x, y)
#else
FUNCDECL(getptv)(im, x, y)
#endif
   int            *im;
   float          *x, *y;
{
   REEL            rx = (REEL) (*x);
   REEL            ry = (REEL) (*y);
   int             codel3[14];

   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "GETPTV");
      if (L3DebugFlag >= 2) {
	 fprintf(FicTrace, " : %d", *im);
      }
      fprintf(FicTrace, "\n");
   }
   SETSOUSIMAGE(*im);

   L3DialogueGETPOS(codel3, GRANDECROIX,
		    VECTEURELASTIQUE,
		    SANSMARQUE,
		    SANSTRACE);
   L3ConvCoordL3VersUtil(codel3[7], codel3[8], &rx, &ry);

   *x = rx;
   *y = ry;
}

/* --------------------------------- GETPTA -------------------------- */
/*
 * void getpta(int i, int j) coordonnees du point designe en coordonnees
 * ecran (0-1023, 0-1023)
 */
void
#ifdef CRAY_PROTO
GETPTA(x, y)
#else
FUNCDECL(getpta)(x, y)
#endif
   int            *x, *y;
{
   int             codel3[14];

   if (L3DebugFlag >= 1) {
      fprintf(FicTrace, "GETPTA");
      fprintf(FicTrace, "\n");
   }
   L3DialogueGETPOS(codel3, GRANDECROIX,
		    SANSELASTIQUE,
		    SANSMARQUE,
		    SANSTRACE);

   *x = codel3[7];
   *y = codel3[8];
}

/* Fin de Fichier */
