/********************************************************/
/* Identification du source */
/* Auteur G. Colin de Verdiere DMA/SIE/IG 6/20/94 */
/********************************************************/

#ifndef lint
    static const char sccs_id[] = "%W% %G% DMA/AIM/LM";
#endif

/********************************************************/
/* Includes */
/********************************************************/

#include <errno.h>
#include <sys/param.h>
#include <netdb.h>

#include "L3NegsDriverP.h"
#include "nx.h"
#include "nxsystem.h"

/********************************************************/
/* Declarations */
/********************************************************/

/*
   Gestionnaires d'ecrans Negs : Il peut etre important de disposer de
   plusieurs ecrans Negs en meme temps ce qui necessite d'avoir un
   etat par connexion
   */

typedef struct
{
	FILE*           fdw /* stream d'ecriture */ ;
	FILE*           fdr /* stream de lecture */ ;
	int             openfait /* indicateur de creation */ ;
	int             nusock /* numero de canal utilise par defaut */ ;
	char            machine[MAXHOSTNAMELEN];
	char            user[L_U8GetUser];
	char            appli[8];
} ECRAN;

#define MAXCONNECT 5
static ECRAN    Ecran[MAXCONNECT];
static ECRAN   *ptr = Ecran /* Pointeur d'Ecran courant */ ;

#define FDW		ptr->fdw
#define FDR		ptr->fdr
#define OPENFAIT	ptr->openfait

static char            destl3[256];

/********************************************************/
/* Declarations fonctions */
/********************************************************/

static int      FaitOpen(void);
static void     EcritL3(int codel3[], FILE *fd);
static void     LitL3(int codel3[], FILE *fd);

/********************************************************/
/* Implementation fonctions */
/********************************************************/

void
L3NegsEcran(int i)
{

	if (i < MAXCONNECT)
	    ptr = &Ecran[i];
	else
	{
	    ptr = Ecran;
	    fprintf(stderr, "Ecran NeGS inexistant %d\n", i);
	}

}

void
L3NegsInitialize()
{
	int i;
	char h_name[MAXHOSTNAMELEN];
	char username[L_U8GetUser];

	U8GetUser(username);

	if(!strlen(username))
	{
	    fprintf(stderr,"erreur : L3NegsInitialize : Initialisation NeGS "
			    "impossible\n");
	    return;
	}

	for (i = 0; i < MAXCONNECT; i++)
	{
	    L3NegsEcran(i);
	    FDW = NULL;
	    FDR = NULL;
	    OPENFAIT = FALSE;
	    ptr->nusock = 2106 + i;
	    strcpy(ptr->appli, "l3");
	    strcpy(ptr->user,username);
	    gethostname(h_name, sizeof(h_name));
	    strcpy(ptr->machine, h_name);
	}

	ptr = Ecran /* Ecran 0 par defaut */ ;
}

void
L3NegsDriver(int codel3[])     
{
   static flag_yesno  cmap = NON ;

   switch (codel3[1]) {
    case CODEL3FLUSH:
      if (!OPENFAIT)
	OPENFAIT = FaitOpen();

      EcritL3(codel3, FDW);
      (void) fflush(FDW);
      break;
    case CODEL3DIALOGUE:
      switch (codel3[2]) {
       case SOUSCODEL3_MGRAPHCLEAR:
	 /* vient de L3SautDePage */
	 if (OPENFAIT) {
	    EcritL3(codel3close, FDW);
	    (void) fflush(FDW);
	    OPENFAIT = FALSE;
	 }
	 break;
       case SOUSCODEL3_GETPOS:
	 if (FDW == NULL) {
	    OPENFAIT = FaitOpen();
	 }
	 (void)fprintf(stdout,"Designez un point dans la fenetre NeGS...\n");
	 (void)fflush(stdout);
	 EcritL3(codel3, FDW);
	 (void) fflush(FDW);
	 if (FDR == NULL)
	   FDR = fdopen(dup(fileno(FDW)), "r");
	 LitL3(codel3, FDR);
	 break;
       default:;
      }
      break;

    case CODEL3OPEN:
      /* l'open est traite plus haut a cause du saut de page */
      if (!OPENFAIT) {
	 OPENFAIT = FaitOpen();
      }
      break;

    case CODEL3CLOSE:
      if (OPENFAIT) {
	 /* fin reelle du graphique : vient de L3Tmgraf */
	 EcritL3(codel3, FDW);
	 (void) fflush(FDW);
	 OPENFAIT = FALSE;
	 cmap = NON ;
      }
      break;
      
      /* branche des options dont il faut garder la memoire */
      
    case CODEL3COLORMAP:
      if (OPENFAIT) {
	 EcritL3(codel3, FDW);
      }
      L3CMap = TRUE;
      cmap = OUI;
      break;
      
    case CODEL3BRILLANCE:
      if (!L3CMap) {
	 /* acces impossible en mode colormap */
	 if (OPENFAIT) {
	    EcritL3(codel3, FDW);
	 }
      }
      break;
      
    case CODEL3CADRE:
    case CODEL3COULEUR: 
    case CODEL3EPAISSEUR: 
      if (OPENFAIT) {
	 EcritL3(codel3, FDW);
      }
      break;
      
      /* branche des autres cas */
    default:
      if (!OPENFAIT) OPENFAIT = FaitOpen();
      EcritL3(codel3, FDW);
   }
}


static void
EcritL3(int codel3[], FILE* fd)
{
	static char* buf = NULL;
    /*
      taille par defaut du buffer compte en item l3
    */
	static int lgbuf = 64;
	size_t index = 0;
	int i;

	if (!buf)
	    buf = (char*) malloc(lgbuf * 2);

	if (codel3[0] > lgbuf) {
	/*-
	  On se realloue un buffer plus grand car celui prevu par defaut
	  est trop petit. But : eviter des malloc trop frequents.
	  Le lgbuf va garder la memoire de la taille effective du tampon
	*/
	    lgbuf = codel3[0];
	    buf = realloc(buf, lgbuf * 2);
	}
   
	for (i = 0; i < codel3[0]; i++) {
	    buf[index++] = (char) ((codel3[i] >> 8) & 0xFF);
	    buf[index++] = (char) (codel3[i] & 0xFF);
	}
   
	if (fwrite(buf, 1, index, fd) != index) {
	    fprintf(stderr, " Pb d'ecriture.\n");
	    exit(1);
	}

}


static void
LitL3(int codel3[], FILE *fd)
{
   int             i, longueur;
   unsigned char   buf2[2];
   int             temp, temp2;

   (void) fflush(fd);
   if (fread(buf2, 1, 2, fd) != 2) {
      (void) fprintf(stderr, " Erreur de lecture \n");
      exit(2);
   }
   temp = buf2[0];
   longueur = (temp << 8) + buf2[1];
   codel3[0] = longueur;
   if (longueur == 0) {
      (void) fprintf(stderr, " Erreur de lecture l=NULL\n");
      exit(3);
   }
   longueur--;
   for (i = 0; i < longueur; i++) {
      (void) fread(buf2, 1, 2, fd);
      temp2 = 0;
      if (buf2[0] > 127)
	temp2 = -1;

      temp2 = (temp2 << 8) | buf2[0];
      temp2 = (temp2 << 8) | buf2[1];
      codel3[i + 1] = temp2;
   }
}

static void
L3CraqueAdresse(chaine)
     char           *chaine;
{
   char            p[256];
   char           *q;

   /* On va exploser la chaine composee de "machine+port+appli+user" en ses
      differents composants. L'ancien format d'adresse reseau
      acceptait un + en tete. Les divers composantes sont stockees
      dans les caracteristiques d'Ecran.
      */

   strcpy(p, chaine);
   q = p;

   if (*q == '+')
     q++ /* dans le cas ou on commence par un + */ ;


   q = strtok(q, "+");
   if (q != NULL) {
      strcpy(ptr->machine, q);
      q = strtok(NULL, "+");
      if (q != NULL) {
	 ptr->nusock = atoi(q);
	 q = strtok(NULL, "+");
	 if (q != NULL) {
	    strcpy(ptr->appli, q);
	    q = strtok(NULL, "+");
	    if (q != NULL) {
	       strcpy(ptr->user, q);
	    }
	 }
      }
   }
}

static void
NouvelleTentative()
{
    /* On essaye de se reconnecter a NeGS en demandant de
       l'aide a l'utilisateur.*/

   char            nom[256];
   int             essais = 10;
   FILE           *fdnew;

      printf("En utilisant : \"%s\"\n\n", destl3);
   while (!feof(stdin)) {
      printf("      La connexion a NeGS a echoue \n\n");
      printf("Vous pouvez choisir\n");
      printf(" Ou [return] pour retenter la connexion\n");
      printf(" Ou donner une nouvelle adresse du type\n");
      printf("               (machine+adresse+l3+user)\n");
      printf(" Ou [exit] ou [^D] pour abandonner \n");

      printf("Votre reponse : ");
      fflush(stdout);
      fgets(nom, 256, stdin);
      if (feof(stdin))
	break;
      nom[strlen(nom) - 1] = '\0' /* suppression du RETURN */ ;
      if (strncmp(nom, "exit", 4) == 0) {
	 fprintf(stderr, " Abandon Sanglier L3QuestionDriver\n");
	 exit(1);
      }
      if ((int) strlen(nom) > 0) {
	 strcpy(destl3, nom);
	 strcat(nom, "DESTL3=");
	 strcat(nom, destl3);
	 putenv(nom);
      }
      printf("Nouvel essai avec %s\n", destl3);
      fdnew = fnxopen(destl3, "w");

      if (fdnew) {
	 FDW = fdnew;
	 L3CraqueAdresse(destl3);
	 return;
      }

      essais--;

      if (essais <= 0) {
	 printf("Sanglier : Trop d'essais -> abandon\n");
	 exit(1);
      }

   }

   fprintf(stderr, " Abandon Sanglier L3QuestionDriver sur EOF\n");
   exit(1);
}

static int
FaitOpen(void)
{
   int             i;

   if (FDW == (FILE *) NULL) {	/* open physique non fait */
      if (ptr == Ecran) {
	 if (getenv("DESTL3") != NULL) {
	    (void) strcpy(destl3, getenv("DESTL3"));
	 } else {
	    (void) fprintf(stderr,
			   "DESTL3 (machine+2106+l3+user+) non initialise\n");
	    exit(1);
	 }
	 L3CraqueAdresse(destl3);
      } else {
	 /* on va genere une connexion sur le port suivant */
	 for (i = 1; i < MAXCONNECT; i++) {
	    if (ptr == &Ecran[i]) {
	       ptr->nusock = Ecran[i - 1].nusock + 1;
	       strcpy(ptr->machine, Ecran[i - 1].machine);
	       strcpy(ptr->appli, Ecran[i - 1].appli);
	       strcpy(ptr->user, Ecran[i - 1].user);
	       break;
	    }
	 }

	 sprintf(destl3, "%s+%d+%s+%s",
		 ptr->machine,
		 ptr->nusock,
		 ptr->appli,
		 ptr->user);
      }

      FDW = fnxopen(destl3, "w");

      if (!FDW) {
	 NouvelleTentative();
      }

   }
   /* envoi des parametres memorise (defaut ou non) */
   EcritL3(codel3open, FDW);
   if (L3CMap)  EcritL3(codel3cmap, FDW);
   EcritL3(codel3cadre, FDW);
   EcritL3(codel3couleur, FDW);
   if (!L3CMap) EcritL3(codel3brillance, FDW);
   EcritL3(codel3epaisseur, FDW);

   return TRUE;
}

/* Fin de fichier */
