/**************/
/* L3Outils.c */
/**************/

#include <stdio.h>
#include <math.h>

#include <stdlib.h>
#include <string.h>
#include <malloc.h>

#include "L3Environnement.h"
#include "L3Outils.h"

#ifndef lint
    static const char sccs_id[] = "%W% %G% G. Colin de Verdiere";
#endif

/*********************************************************************
 * Procedure interne : L3SelectionSousImage                          *
 *-------------------------------------------------------------------*
 * Parametres :                                                      *
 *   sousimage : numero de la sous-image a selectionner entre 0 et   *
 *                 la valeur de NBSOUSIMAGESL3 (= 10) - 1            *
 *                                                                   *
 * Valeur de retour : aucune                                         *
 *                                                                   *
 * Description : selectionne la sous-image courante, donne acces a   *
 *   ses caracteristiques.                                           *
 *                                                                   *
 * Erreurs :                                                         *
 *   numero de sous-image incorrect                                  *
 *********************************************************************/

void
L3SelectionSousImage(int sousimage)
{

	if((sousimage >= 0) && (sousimage < NBSOUSIMAGESL3))
	    P_ParamSousImage = &ParamSousImage[sousimage];
	else
	{
	/*------------------------------------------*/
	/* Erreur : numero de sous-image incorrect. */
	/*------------------------------------------*/
	    fprintf(stderr, "L3SelectionSousImage : numero de "
			    "sous-image incorrect : %d\n", sousimage);
	}

}

/*************************************************************************
 * Procedure interne : L3CalculValClip                                   *
 *-----------------------------------------------------------------------*
 * Parametres : aucun                                                    *
 *                                                                       *
 * Valeur de retour : aucune                                             *
 *                                                                       *
 * Description : met a jour les coordonnees L3 de la sous-image courante *
 *   en vue du decoupage des segments et polygones.                      *
 *                                                                       *
 * Erreurs : aucune                                                      *
 *************************************************************************/

void
L3CalculValClip()
{
   double          lxmin, lxmax, lymin, lymax;
   double          lxa, lya, lxb, lyb;
   double          lxq, lyq, lxp, lyp;
   double          soix = 65535.0, tren = 32768.0;

   lxp = (double) XP;
   lyp = (double) YP;
   lxq = (double) XQ;
   lyq = (double) YQ;
   lxa = (double) XA;
   lya = (double) YA;
   lxb = (double) XB;
   lyb = (double) YB;

   lxmin = (soix * (lxa - lxp) / (lxq - lxp) - tren);
   lymin = (soix * (lya - lyp) / (lyq - lyp) - tren);
   lxmax = (soix * (lxb - lxp) / (lxq - lxp) - tren);
   lymax = (soix * (lyb - lyp) / (lyq - lyp) - tren);

   XMIN = (int) lxmin;
   XMAX = (int) lxmax;
   YMIN = (int) lymin;
   YMAX = (int) lymax;
}

/*************************************************************************
 * Procedure interne : L3CalculConvCoord                                 *
 *-----------------------------------------------------------------------*
 * Parametres : aucun                                                    *
 *                                                                       *
 * Valeur de retour : aucune                                             *
 *                                                                       *
 * Description : met a jour les coefficients permettant le passage en    *
 *   coordonnees L3 dans la sous-image courante.                         *
 *                                                                       *
 * Erreurs : aucune                                                      *
 *************************************************************************/

void
L3CalculConvCoord()
{
   XCOEFFICIENTLIN = 65535.0 * (XB - XA) / (XV - XU) / (XQ - XP);
   XCONSTANTELIN = 65535.0 * ((XA - XP) * XV - (XB - XP) * XU) / (XV - XU) / (XQ - XP) - 32768.0;

   if ((XU > 0.0) && (XV > 0.0)) {
      XCOEFFICIENTLOG = 65535.0 * (XB - XA) / (log10(XV) - log10(XU)) / (XQ - XP);
      XCONSTANTELOG = 65535.0 * ((XA - XP) * log10(XV) - (XB - XP) * log10(XU)) / (log10(XV) - log10(XU)) / (XQ - XP) - 32768.0;
   }
   YCOEFFICIENTLIN = 65535.0 * (YB - YA) / (YV - YU) / (YQ - YP);
   YCONSTANTELIN = 65535.0 * ((YA - YP) * YV - (YB - YP) * YU) / (YV - YU) / (YQ - YP) - 32768.0;

   if ((YU > 0.0) && (YV > 0.0)) {
      YCOEFFICIENTLOG = 65535.0 * (YB - YA) / (log10(YV) - log10(YU)) / (YQ - YP);
      YCONSTANTELOG = 65535.0 * ((YA - YP) * log10(YV) - (YB - YP) * log10(YU)) / (log10(YV) - log10(YU)) / (YQ - YP) - 32768.0;
   }
}


/*************************************************************************************************
 * Procedure interne : L3ConversionCoordonnees                                                   *
 *-----------------------------------------------------------------------------------------------*
 * Parametres :                                                                                  *
 *   x       : abcisse du point de l'espace utilisateur a convertir                              *
 *   y       : ordonnee du point de l'espace utilisateur a convertir                             *
 *   p_xpage : pointeur sur la variable destinee a recueillir l'abcisse L3 du point a convertir  *
 *   p_ypage : pointeur sur la variable destinee a recueillir l'ordonnee L3 du point a convertir *
 *                                                                                               *
 * Valeur de retour : aucune                                                                     *
 *                                                                                               *
 * Description : calcule les coordonnees L3 du point passe en parametre.                         *
 *                                                                                               *
 * Erreurs : aucune                                                                              *
 *************************************************************************************************/


void
L3ConversionCoordonnees(REEL x, REEL y, int *p_xpage, int *p_ypage)
{
   L3ConversionCoordonneesLog(x, y, p_xpage, p_ypage, XCOORDLOG, YCOORDLOG);
}


/*************************************************************************************************
 * Procedure interne : L3ConversionCoordonneesLog                                                *
 *-----------------------------------------------------------------------------------------------*
 * Parametres :                                                                                  *
 *   x       : abcisse du point de l'espace utilisateur a convertir                              *
 *   y       : ordonnee du point de l'espace utilisateur a convertir                             *
 *   p_xpage : pointeur sur la variable destinee a recueillir l'abcisse L3 du point a convertir  *
 *   p_ypage : pointeur sur la variable destinee a recueillir l'ordonnee L3 du point a convertir *
 *   xcoordlog : indicateur de mode logarithmique en X                                           *
 *   ycoordlog : indicateur de mode logarithmique en Y                                           *
 *                                                                                               *
 * Valeur de retour : aucune                                                                     *
 *                                                                                               *
 * Description : calcule les coordonnees L3 du point passe en parametre.                         *
 *                                                                                               *
 * Erreurs :                                                                                     *
 *   abcisse negative en mode logarithmique                                                      *
 *   ordonnee negative en mode logarithmique                                                     *
 *************************************************************************************************/


void
L3ConversionCoordonneesLog(REEL x, REEL y, int *p_xpage, int *p_ypage, int xcoordlog, int ycoordlog)
{
   if (xcoordlog == FAUX) {
      *p_xpage = (int) (XCONSTANTELIN + XCOEFFICIENTLIN * x);
   } else {
      if (x > 0.0) {
	 *p_xpage = (int) (XCONSTANTELOG + XCOEFFICIENTLOG * log10(x));
      } else {
	 /*---------------------------------------------------*/
	 /* Erreur : abscisse negative en mode logarithmique. */
	 /*---------------------------------------------------*/
	 *p_xpage = (int) (XCONSTANTELOG + XCOEFFICIENTLOG * (-37.0));
      }
   }

   if (ycoordlog == FAUX) {
      *p_ypage = (int) (YCONSTANTELIN + YCOEFFICIENTLIN * y);
   } else {
      if (y > 0.0) {
	 *p_ypage = (int) (YCONSTANTELOG + YCOEFFICIENTLOG * log10(y));
      } else {
	 /*---------------------------------------------------*/
	 /* Erreur : ordonnee negative en mode logarithmique. */
	 /*---------------------------------------------------*/
	 *p_ypage = (int) (YCONSTANTELOG + YCOEFFICIENTLOG * (-37.0));
      }
   }
}


/*************************************************************************************************
 * Procedure interne : L3ConvCoordL3VersUtil                                                     *
 *-----------------------------------------------------------------------------------------------*
 * Parametres :                                                                                  *
 *   xl3     : abcisse L3 du point a convertir                                                   *
 *   yl3     : ordonnee L3 du point a convertir                                                  *
 *   p_xutil : pointeur sur la variable destinee a recueillir l'abcisse du point a convertir     *
 *   p_yutil : pointeur sur la variable destinee a recueillir l'ordonnee du point a convertir    *
 *                                                                                               *
 * Valeur de retour : aucune                                                                     *
 *                                                                                               *
 * Description : calcule les coordonnees dans l'espace utilisateur du point L3 passe en          *
 *   parametre.                                                                                  *
 *                                                                                               *
 * Erreurs : aucune                                                                              *
 *************************************************************************************************/


void
L3ConvCoordL3VersUtil(xl3, yl3, p_xutil, p_yutil)
   int             xl3, yl3;
   REEL           *p_xutil, *p_yutil;

{
   L3ConvCoordL3VersUtilLog(xl3, yl3, p_xutil, p_yutil, XCOORDLOG, YCOORDLOG);
}


/*************************************************************************************************
 * Procedure interne : L3ConvCoordL3VersUtilLog                                                  *
 *-----------------------------------------------------------------------------------------------*
 * Parametres :                                                                                  *
 *   xl3     : abcisse L3 du point a convertir                                                   *
 *   yl3     : ordonnee L3 du point a convertir                                                  *
 *   p_xutil : pointeur sur la variable destinee a recueillir l'abcisse du point a convertir     *
 *   p_yutil : pointeur sur la variable destinee a recueillir l'ordonnee du point a convertir    *
 *   xcoordlog : indicateur de mode logarithmique en X                                           *
 *   ycoordlog : indicateur de mode logarithmique en Y                                           *
 *                                                                                               *
 * Valeur de retour : aucune                                                                     *
 *                                                                                               *
 * Description : calcule les coordonnees dans l'espace utilisateur du point L3 passe en          *
 *   parametre.                                                                                  *
 *                                                                                               *
 * Erreurs : aucune                                                                              *
 *************************************************************************************************/

void
L3ConvCoordL3VersUtilLog(xl3, yl3, p_xutil, p_yutil, xcoordlog, ycoordlog)
   int             xl3, yl3;
   REEL           *p_xutil, *p_yutil;
   int             xcoordlog, ycoordlog;

{
   if (xcoordlog == FAUX) {
      *p_xutil = ((REEL) xl3 - XCONSTANTELIN) / XCOEFFICIENTLIN;
   } else {
      *p_xutil = exp10(((REEL) xl3 - XCONSTANTELOG) / XCOEFFICIENTLOG);
   }

   if (ycoordlog == FAUX) {
      *p_yutil = ((REEL) yl3 - YCONSTANTELIN) / YCOEFFICIENTLIN;
   } else {
      *p_yutil = exp10(((REEL) yl3 - YCONSTANTELOG) / YCOEFFICIENTLOG);
   }
}

/*******************************************************************************
 * Fonction interne : L3ClippingSegmentGen                                     *
 *-----------------------------------------------------------------------------*
 * Parametres :                                                                *
 *   XClipMin : abcisse L3 min                                                 *
 *   XClipMax : abcisse L3 max                                                 *
 *   YClipMin : ordonnee L3 min                                                *
 *   YClipMax : ordonnee L3 max                                                *
 *   xpage1 : abcisse L3 initiale du segment a decouper                        *
 *   ypage1 : ordonnee L3 initiale du segment a decouper                       *
 *   xpage2 : abcisse L3 finale du segment a decouper                          *
 *   ypage2 : ordonnee L3 finale du segment a decouper                         *
 *   xypage : tableau a 4 elements destine a recevoir les coordonnees          *
 *           L3 du segment decoupe                                             *
 *                                                                             *
 * Valeur de retour : FAUX si le segment est hors de la fenetre de decoupage   *
 *                    VRAI sinon                                               *
 *                                                                             *
 * Description : calcule les coordonnees L3 de la portion du segment fourni    *
 *                    contenue dans la fenetre de decoupage.                   *
 *                                                                             *
 * Algorithme : decoupage d'une droite par une fenetre rectangulaire           *
 *   (cf ouvrage "Synthese d'image : algorithmes elementaires" de G. HEGRON)   *
 *   par la methode de PAVLIDIS.                                               *
 *                                                                             *
 * Erreurs : aucune                                                            *
 *******************************************************************************/

int
L3ClippingSegmentGen(int XClipMin,int XClipMax,int YClipMin,int YClipMax,
		int xpage1,int ypage1,int xpage2,int ypage2,int xypage[])
{
   register int    tracer;
   int             entierement_dehors;
   int             entierement_dedans;
   int             x, y;
   int             j;
   double          dx, dy, dxy;
   double          qxmin, qymin, qxmax, qymax;
   double          u1, u2, u3, u4;

/*
	si le segment est entierement dedans on le trace tel quel
	sinon
		si il est entierement dehors on ne trace rien
		sinon on regarde en detail....
*/

   tracer = VRAI;

   entierement_dedans = (xpage1 >= XClipMin) && (xpage1 <= XClipMax)
      && (ypage1 >= YClipMin) && (ypage1 <= YClipMax)
      && (xpage2 >= XClipMin) && (xpage2 <= XClipMax)
      && (ypage2 >= YClipMin) && (ypage2 <= YClipMax);
      
   if (!entierement_dedans)
   {
      entierement_dehors = ((xpage1 < XClipMin) && (xpage2 < XClipMin))
	 || ((xpage1 > XClipMax) && (xpage2 > XClipMax))
	 || ((ypage1 < YClipMin) && (ypage2 < YClipMin))
	 || ((ypage1 > YClipMax) && (ypage2 > YClipMax));
      tracer = FAUX;
      
      if (!entierement_dehors)
      {
         /* Il y a un cas a probleme -> on traite en detail */

	 if (ypage1 > ypage2)
         {
	    x = xpage1;
	    xpage1 = xpage2;
	    xpage2 = x;
	    y = ypage1;
	    ypage1 = ypage2;
	    ypage2 = y;
	 }

	 if (ypage1 == ypage2)
         {

	    if (xpage1 > xpage2)
            {
	       x = xpage1;
	       xpage1 = xpage2;
	       xpage2 = x;
	    }

	    if (xpage1 < XClipMin)
	       xpage1 = XClipMin;

	    if (xpage2 > XClipMax)
	       xpage2 = XClipMax;

	    tracer = VRAI;
	 }
         else if (xpage1 == xpage2)
         {

	    if (ypage1 < YClipMin)
	       ypage1 = YClipMin;

	    if (ypage2 > YClipMax)
	       ypage2 = YClipMax;

	    tracer = VRAI;
	 }
         else
         {
	    dy = (double) ypage1 - (double) ypage2;
	    dx = (double) xpage1 - (double) xpage2;
	    dxy = (double) xpage1 * (double) ypage2 -
			(double) ypage1 * (double) xpage2;
	    qxmin = (double) XClipMin *dy;
	    qxmax = (double) XClipMax *dy;
	    qymin = (double) YClipMin *dx;
	    qymax = (double) YClipMax *dx;
	    u1 = qxmin - qymin + dxy;
	    u2 = qxmin - qymax + dxy;
	    u3 = qxmax - qymax + dxy;
	    u4 = qxmax - qymin + dxy;
	    j = 0;

	    if (u1 * u2 <= 0.0)
            {
	       j++;

	       if (xpage1 < XClipMin)
               {
		  ypage1 = (XClipMin * dy + dxy) / dx;
		  xpage1 = XClipMin;
		  dy = (double) ypage1 - (double) ypage2;
		  dx = (double) xpage1 - (double) xpage2;
		  dxy = (double) xpage1 * (double) ypage2 -
			(double) ypage1 * (double) xpage2;
	       }

	       if (xpage2 < XClipMin)
               {
		  ypage2 = (XClipMin * dy + dxy) / dx;
		  xpage2 = XClipMin;
		  dy = (double) ypage1 - (double) ypage2;
		  dx = (double) xpage1 - (double) xpage2;
		  dxy = (double) xpage1 * (double) ypage2 -
			(double) ypage1 * (double) xpage2;
	       }

	    }

	    if (u2 * u3 <= 0.0)
            {
	       j++;

	       if (ypage2 > YClipMax)
               {
		  xpage2 = (YClipMax * dx - dxy) / dy;
		  ypage2 = YClipMax;
		  dy = (double) ypage1 - (double) ypage2;
		  dx = (double) xpage1 - (double) xpage2;
		  dxy = (double) xpage1 * (double) ypage2 -
			(double) ypage1 * (double) xpage2;
	       }

	    }

	    if (u3 * u4 <= 0.0)
            {
	       j++;

	       if (xpage1 > XClipMax)
               {
		  ypage1 = (XClipMax * dy + dxy) / dx;
		  xpage1 = XClipMax;
		  dy = (double) ypage1 - (double) ypage2;
		  dx = (double) xpage1 - (double) xpage2;
		  dxy = (double) xpage1 * (double) ypage2 -
			(double) ypage1 * (double) xpage2;
	       }

	       if (xpage2 > XClipMax)
               {
		  ypage2 = (XClipMax * dy + dxy) / dx;
		  xpage2 = XClipMax;
		  dy = (double) ypage1 - (double) ypage2;
		  dx = (double) xpage1 - (double) xpage2;
		  dxy = (double) xpage1 * (double) ypage2 -
			(double) ypage1 * (double) xpage2;
	       }

	    }

	    if (u4 * u1 <= 0.0)
            {
	       j++;

	       if (ypage1 < YClipMin)
               {
		  xpage1 = (YClipMin * dx - dxy) / dy;
		  ypage1 = YClipMin;
	       }

	    }

	    if (j > 0)
	       tracer = VRAI;

	 }

      }

   }

   if (tracer == VRAI)
   {
      xypage[0] = xpage1;
      xypage[1] = ypage1;
      xypage[2] = xpage2;
      xypage[3] = ypage2;
   }

   return (tracer);
}

/*******************************************************************************
 * Fonction interne : L3ClippingSegment                                        *
 *-----------------------------------------------------------------------------*
 * Parametres :                                                                *
 *   xpage1 : abcisse L3 initiale du segment a decouper                        *
 *   ypage1 : ordonnee L3 initiale du segment a decouper                       *
 *   xpage2 : abcisse L3 finale du segment a decouper                          *
 *   ypage2 : ordonnee L3 finale du segment a decouper                         *
 *   xypage : tableau a 4 elements destine a recevoir les coordonnees          *
 *           L3 du segment decoupe                                             *
 *                                                                             *
 * Valeur de retour : FAUX si le segment est hors de la fenetre de decoupage   *
 *                    VRAI sinon                                               *
 *                                                                             *
 * Description : calcule les coordonnees L3 de la portion du segment fourni    *
 *                    contenue dans la fenetre de decoupage.                   *
 *                                                                             *
 * Algorithme : decoupage d'une droite par une fenetre rectangulaire           *
 *   (cf ouvrage "Synthese d'image : algorithmes elementaires" de G. HEGRON)   *
 *   par la methode de PAVLIDIS.                                               *
 *                                                                             *
 * Erreurs : aucune                                                            *
 *******************************************************************************/

int
L3ClippingSegment(int xpage1,int ypage1,int xpage2,int ypage2,int xypage[])
{
	return L3ClippingSegmentGen(XCLIPMIN,XCLIPMAX,YCLIPMIN,YCLIPMAX,
			xpage1,ypage1,xpage2,ypage2,xypage);
}

/*************************************************************************************************
 * Procedure interne : L3ClippingPolygoneDroite                                                  *
 *-----------------------------------------------------------------------------------------------*
 * Parametres :                                                                                  *
 *   xmin   : abcisse L3 de la droite verticale de decoupage                                     *
 *   signe  : coefficient (+1 ou -1) indiquant si on garde le demi-plan a droite (+1) ou a       *
 *              gauche (-1) de la droite de decoupage                                            *
 *   x      : tableau des abcisses L3 des sommets du polygone a decouper                         *
 *   y      : tableau des ordonnees L3 des sommets du polygone a decouper                        *
 *   xres   : tableau des abcisse L3 des sommets du polygone decoupe                             *
 *   yres   : tableau des ordonnees L3 des sommets du polygone decoupe                           *
 *   p_nsom : pointeur sur la variable contenant :                                               *
 *              au debut : le nombre de sommets du polygone a decouper                           *
 *              a la fin : le nombre de sommets du polygone decoupe                              *
 *                                                                                               *
 * Valeur de retour : aucune                                                                     *
 *                                                                                               *
 * Description : calcule les coordonnees L3 de la portion du polygone fourni contenue dans le    *
 *   demi-plan defini par 'xmin' et 'signe'.                                                     *
 *                                                                                               *
 * Algorithme : decoupage d'un polygone non convexe par une droite (cf ouvrage "Synthese         *
 *   d'image : algorithmes elementaires" de G. HEGRON) par la methode de SUTHERLAND et HODGMAN.  *
 *                                                                                               *
 * Erreurs : aucune                                                                              *
 *************************************************************************************************/

void
L3ClippingPolygoneDroite(int xmin,int signe,int x[],int y[],
		int xres[],int yres[],int* p_nsom)
{
	int             i;
	int             p;
	int             premier;
	double          xs = 0.0, ys = 0.0;
	double          xj = 0.0, yj = 0.0;
	double          xmini;

	xmini = (double) xmin;
	i = 0;
	p = *p_nsom - 1;
	premier = VRAI;

	while (p >= 0)
	{
	    double xp = (double) x[p];
	    double yp = (double) y[p];

	    if (premier == VRAI)
	    {
		premier = FAUX;
		xj = xp;
		yj = yp;
	    }
	    else if ((xs - xmini) * (xp - xmini) < 0.0)
	    {
		xres[i] = xmin;
		yres[i] = (int) (ys + (yp - ys) * (xmini - xs) / (xp - xs));
		i++;
	    }

	    xs = xp;
	    ys = yp;

	    if ((xs - xmini) * signe >= 0.0)
	    {
		xres[i] = (int) xs;
		yres[i] = (int) ys;
		i++;
	    }

	    p--;
	}

	if(*p_nsom >= 1 && (xs - xmini) * (xj - xmini) < 0.0)
	{
	    xres[i] = xmin;
	    yres[i] = (int) (ys + (yj - ys) * (xmini - xs) / (xj - xs));
	    i++;
	}

	*p_nsom = i;
}


/*****************************************************************************************
 * Fonction interne : L3ClippingPoint                                                    *
 *---------------------------------------------------------------------------------------*
 * Parametres :                                                                          *
 *   xl3     : abcisse L3 du point a decouper                                            *
 *   yl3     : ordonnee L3 du point a decouper                                           *
 *                                                                                       *
 * Valeur de retour : FAUX si le point est hors de la fenetre de decoupage               *
 *                    VRAI sinon                                                         *
 *                                                                                       *
 * Description : teste si le point fourni est dans la fenetre de decoupage.              *
 *                                                                                       *
 * Erreurs : aucune                                                                      *
 *****************************************************************************************/


int
L3ClippingPoint(xl3, yl3)
   int             xl3, yl3;

{
   int             tracer;

   if ((xl3 >= XCLIPMIN) && (xl3 <= XCLIPMAX) && (yl3 >= YCLIPMIN) && (yl3 <= YCLIPMAX))
      tracer = VRAI;
   else
      tracer = FAUX;

   return (tracer);
}


/*****************************************************************************************
 * Procedure interne : L3DialogueGETPOS                                                  *
 *---------------------------------------------------------------------------------------*
 * Parametres :                                                                          *
 *   codel3        : pointeur sur le code l3 de demande de dialogue GETPOS               *
 *   typecroix     : type de croix de selection                                          *
 *   typeelastique : type d'elastique entre le point initial et le point selectionne     *
 *   typemarque    : type de marquage du point selectionne                               *
 *   typetrace     : type de trace entre le point initial et le point selectionne        *
 *                                                                                       *
 * Valeur de retour : aucune.                                                            *
 *                                                                                       *
 * Description : fait une demande de dialogue interactif de type GETPOS (sous-code 102). *
 *   Le driver complete les champs 7 a 13 du code l3 fourni.                             *
 *                                                                                       *
 * Erreurs : aucune                                                                      *
 *****************************************************************************************/


void
L3DialogueGETPOS(codel3, typecroix, typeelastique, typemarque, typetrace)
   int             codel3[];
   int             typecroix, typeelastique, typemarque, typetrace;

{
   codel3[0] = 14;
   codel3[1] = CODEL3DIALOGUE;
   codel3[2] = SOUSCODEL3_GETPOS;
   codel3[3] = typecroix;
   codel3[4] = typeelastique;
   codel3[5] = typemarque;
   codel3[6] = typetrace;
   codel3[7] = 0;
   codel3[8] = 0;
   codel3[9] = 0;
   codel3[10] = 0;
   codel3[11] = 0;
   codel3[12] = 0;
   codel3[13] = 0;

   L3AppelDrivers(codel3);
}

/*****************************************************************************************
 * Procedure interne : L3TranscodeChaine                                                 *
 *---------------------------------------------------------------------------------------*
 * Parametres :                                                                          *
 *   source        : pointeur sur la chaine a transcoder
 *   cible         : pointeur sur la chaine transcodee
 *                                                                                       *
 * Valeur de retour : aucune.                                                            *
 *                                                                                       *
 * Description : transcode une chaine avec des caracteres de controle saisis en          *
 *               interactif, en chaine avec codes de controles internes sanglier         *
 *                                                                                       *
 * Erreurs : chaine nulle ou vide
 *****************************************************************************************/
void
L3TranscodeChaine(char* source, char* cible)
{
/*-
	Recode la chaine d'entree au format BBC dans le format SANGLIER
 */
   int             longueur;
   int             index_lecture;
   int             index_recopie;
   char            code1, code2;

   if (source == NULL || cible == NULL)
   {
        /*********************************/
        /* Erreur : chaine nulle         */
        /*********************************/
   	(void)fprintf(stderr,"L3TranscodeChaine : chaine nulle\n");
   	return;
   }

   longueur = strlen(source);

   if(longueur == 0)
   {
	cible[0] = '\0';
	return;
   }

   index_lecture = longueur;
   
   /* on commence par eliminer tous les blancs en fin de chaine */

   code1 = source[longueur - 1];

   while ((code1 == ' ') && (index_lecture > 0))
   {
   	index_lecture--;

   	if(index_lecture > 0) code1 = source[index_lecture - 1];
   }

   source[index_lecture] = '\0';

   longueur = strlen(source);
   index_lecture = 0;
   index_recopie = 0;
   
   /* 
      on ne recherche que les sequences de (deux au plus) caracteres
      de controle; on ne recopie jamais directement le deuxieme
      caractere s'il n'est pas un controle car il peut participer
      a une sequence de controle contigue (sauf dans le cas de 
      recopie forcee par ! isole)
   */

   while (index_lecture < longueur)
   {
   	code1 = source[index_lecture];
   	index_lecture += 1;
   	switch(code1)
   	{
   	 case('%'):
   	  if (index_lecture < longueur)
   	  {
   	   code2 = source[index_lecture];
   	   index_lecture += 1;
   	   switch(code2)
   	   {
   	    case('%'): /* passage en jeu de caracteres graphiques */
   	     cible[index_recopie] = MODGRAPHIQUE;
   	     index_recopie += 1;
   	     break;
   	    case('J'): /* symbole "il existe" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = 'J';
   	     index_recopie += 1;
   	     break;
   	    case('V'): /* symbole "nabla" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = 'V';
   	     index_recopie += 1;
   	     break;
   	    case('j'): /* symbole "infini" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = 'j';
   	     index_recopie += 1;
   	     break;
   	    case('v'): /* symbole "quelque soit" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = 'v';
   	     index_recopie += 1;
   	     break;
   	    default: /* prochain caractere graphique */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     index_lecture -= 1;
   	   }
   	  }
   	  else
   	  {
   	   cible[index_recopie] = code1;
   	   index_recopie += 1;
   	  }
   	  break;
   	 case('&'):
   	  if (index_lecture < longueur)
   	  {
   	   code2 = source[index_lecture];
   	   index_lecture += 1;
   	   switch(code2)
   	   {
   	    case('&'): /* passage en jeu de caracteres non graphiques */
   	     break;
   	    default: /* prochain caractere non graphique */
   	     cible[index_recopie] = MODTEXTLOCAL;
   	     index_recopie += 1;
   	     index_lecture -= 1;
   	   }
   	  }
   	  else
   	  {
   	   cible[index_recopie] = code1;
   	   index_recopie += 1;
   	  }
   	  break;
   	 case('@'): /* recul d'un caractere */
   	  cible[index_recopie] = MODRECUL;
   	  index_recopie += 1;
   	  break;
   	 case('!'):
   	  if (index_lecture < longueur)
   	  {
   	   code2 = source[index_lecture];
   	   index_lecture += 1;
   	   switch(code2)
   	   {
   	    case ('0'): /* passage en exposant de meme taille */
   	     cible[index_recopie] = MODDECALESUP;
   	     index_recopie += 1;
   	     break;
   	    case ('1'): /* passage en indice de meme taille */
   	     cible[index_recopie] = MODDECALEINF;
   	     index_recopie += 1;
   	     break;
   	    case ('2'): /* passage en taille reduite */
   	     cible[index_recopie] = MODREDUIT;
   	     index_recopie += 1;
   	     break;
   	    case ('3'): /* passage en taille agrandie */
   	     cible[index_recopie] = MODAGRANDI;
   	     index_recopie += 1;
   	     break;
   	    case ('r'): /* ouvrir racine */
   	     cible[index_recopie] = MODDEBHAB;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)RACINE;
   	     index_recopie += 1;
   	     break;
   	    case ('q'): /* fermer racine */
   	     cible[index_recopie] = MODFINHAB;
   	     index_recopie += 1;
   	     break;
   	    case ('f'): /* ouvre fraction */
   	     cible[index_recopie] = MODFRACTION;
   	     index_recopie += 1;
   	     break;
   	    case ('k'): /* ferme fraction */
   	     cible[index_recopie] = MODFINFRACTION;
   	     index_recopie += 1;
   	     break;
   	    case ('g'): /* numerateur -> denominateur */
   	     cible[index_recopie] = MODNUMDEN;
   	     index_recopie += 1;
   	     break;
   	    case ('d'): /* symbole "d rond" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\052';
   	     index_recopie += 1;
   	     break;
   	    case ('l'): /* symbole "l rond" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\056';
   	     index_recopie += 1;
   	     break;
   	    case ('*'): /* symbole "proportionnel" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\061';
   	     index_recopie += 1;
   	     break;
   	    case ('='): /* symbole "environ" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\047';
   	     index_recopie += 1;
   	     break;
   	    case ('#'): /* symbole "different" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\073';
   	     index_recopie += 1;
   	     break;
   	    case ('s'): /* symbole "integrale" */
   	    case ('['): /* symbole "integrale" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\062';
   	     index_recopie += 1;
   	     break;
   	    case (']'): /* symbole "racine" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\063';
   	     index_recopie += 1;
   	     break;
   	    case ('('): /* symbole "accolade ouvrante" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\173';
   	     index_recopie += 1;
   	     break;
   	    case (')'): /* symbole "accolade fermante" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\175';
   	     index_recopie += 1;
   	     break;
   	    case ('@'): /* symbole "arrobas" */
   	     cible[index_recopie] = MODTEXTLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\100';
   	     index_recopie += 1;
   	     break;
   	    case ('b'): /* symbole "appartient" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\074';
   	     index_recopie += 1;
   	     break;
   	    case ('a'): /* symbole "angstrom" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\050';
   	     index_recopie += 1;
   	     break;
   	    case ('m'): /* symbole "cha" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\057';
   	     index_recopie += 1;
   	     break;
   	    case ('i'): /* symbole "barre de valeur absolue" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\174';
   	     index_recopie += 1;
   	     break;
   	    case ('h'): /* symbole "plus ou moins" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\054';
   	     index_recopie += 1;
   	     break;
   	    case ('n'): /* symbole "intersection" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\060';
   	     index_recopie += 1;
   	     break;
   	    case ('p'): /* symbole "paragraphe" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\075';
   	     index_recopie += 1;
   	     break;
   	    case ('c'): /* symbole "c cedille" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\051';
   	     index_recopie += 1;
   	     break;
   	    case ('e'): /* symbole "e accent aigu" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\053';
   	     index_recopie += 1;
   	     break;
   	    case ('\\'): /* symbole "accent grave", recul automatique */
   	     cible[index_recopie] = MODRECUL;
   	     index_recopie += 1;
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\044';
   	     index_recopie += 1;
   	     break;
   	    case ('^'): /* symbole "accent circonflexe", recul automatique */
   	     cible[index_recopie] = MODRECUL;
   	     index_recopie += 1;
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\045';
   	     index_recopie += 1;
   	     break;
   	    case ('$'): /* symbole "fleche de vecteur", recul automatique */
   	     cible[index_recopie] = MODRECUL;
   	     index_recopie += 1;
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\046';
   	     index_recopie += 1;
   	     break;
   	    case ('-'): /* symbole "souligne", recul automatique */
   	     cible[index_recopie] = MODRECUL;
   	     index_recopie += 1;
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\041';
   	     index_recopie += 1;
   	     break;
   	    case ('+'): /* symbole "surligne", recul automatique */
   	     cible[index_recopie] = MODRECUL;
   	     index_recopie += 1;
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\040';
   	     index_recopie += 1;
   	     break;
   	    case ('"'): /* symbole "trema", recul automatique */
   	     cible[index_recopie] = MODRECUL;
   	     index_recopie += 1;
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\043';
   	     index_recopie += 1;
   	     break;
   	    case ('I'): /* integrale */
   	     cible[index_recopie] = MODDEBINTEG;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)INTEGRALE;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)SIMPLE;
   	     index_recopie += 1;
   	     break;
   	    case ('S'): /* sommation */
   	     cible[index_recopie] = MODDEBINTEG;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)SOMME;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)SIMPLE;
   	     index_recopie += 1;
   	     break;
   	    case ('P'): /* produit */
   	     cible[index_recopie] = MODDEBINTEG;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)PRODUIT;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)SIMPLE;
   	     index_recopie += 1;
   	     break;
   	    case ('G'): /* separateur de terme integral */
   	     cible[index_recopie] = MODSEPINTEG;
   	     index_recopie += 1;
   	     break;
   	    case ('K'): /* fin de terme integral */
   	     cible[index_recopie] = MODFININTEG;
   	     index_recopie += 1;
   	     break;
   	    case ('V'): /* vecteur */
   	     cible[index_recopie] = MODDEBHAB;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)VECTEUR;
   	     index_recopie += 1;
   	     break;
   	    case ('<'): /* souligne */
   	     cible[index_recopie] = MODDEBHAB;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)SOULIGNE;
   	     index_recopie += 1;
   	     break;
   	    case ('>'): /* surligne */
   	     cible[index_recopie] = MODDEBHAB;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)SURLIGNE;
   	     index_recopie += 1;
   	     break;
   	    case ('A'): /* accolades */
   	     cible[index_recopie] = MODDEBHAB;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)ACCOLADE;
   	     index_recopie += 1;
   	     break;
   	    case ('B'): /* parentheses */
   	     cible[index_recopie] = MODDEBHAB;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)PARENTHESE;
   	     index_recopie += 1;
   	     break;
   	    case ('C'): /* crochets */
   	     cible[index_recopie] = MODDEBHAB;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)CROCHET;
   	     index_recopie += 1;
   	     break;
   	    case ('D'): /* crochets carres */
   	     cible[index_recopie] = MODDEBHAB;
   	     index_recopie += 1;
   	     cible[index_recopie] = (char)CROCHET_CARRE;
   	     index_recopie += 1;
   	     break;
   	    case ('Q'): /* fin d'habillage */
   	     cible[index_recopie] = MODFINHAB;
   	     index_recopie += 1;
   	     break;
   	    default: /* recopie forcee du caractere */
   	     cible[index_recopie] = code2;
   	     index_recopie += 1;
   	     break;
   	   }
   	  }
   	  else
   	  {
   	   cible[index_recopie] = code1;
   	   index_recopie += 1;
   	  }
   	  break;
   	 case('^'):
   	  if (index_lecture < longueur)
   	  {
   	   code2 = source[index_lecture];
   	   index_lecture += 1;
   	   switch(code2)
   	   {
   	    case('^'): /* passage en exposant sureleve */
   	     cible[index_recopie] = MODEXPOSANTSUP;
   	     index_recopie += 1;
   	     break;
   	    default: /* passage en exposant ordinaire */
   	     cible[index_recopie] = MODEXPOSANT;
   	     index_recopie += 1;
   	     index_lecture -= 1;
   	   }
   	  }
   	  else
   	  {
   	   cible[index_recopie] = code1;
   	   index_recopie += 1;
   	  }
   	  break;
   	 case('\\'):
   	  if (index_lecture < longueur)
   	  {
   	   code2 = source[index_lecture];
   	   index_lecture += 1;
   	   switch(code2)
   	   {
   	    case('\\'): /* passage en indice surbaisse */
   	     cible[index_recopie] = MODINDICEINF;
   	     index_recopie += 1;
   	     break;
   	    default: /* passage en indice ordinaire */
   	     cible[index_recopie] = MODINDICE;
   	     index_recopie += 1;
   	     index_lecture -= 1;
   	   }
   	  }
   	  else
   	  {
   	   cible[index_recopie] = code1;
   	   index_recopie += 1;
   	  }
   	  break;
   	 case('/'): /* retour niveau d'indice precedent avec recul */
   	  cible[index_recopie] = MODINDPRECRECUL;
   	  index_recopie += 1;
   	  break;
   	 case('?'): /* retour niveau d'indice precedent sans recul */
   	  cible[index_recopie] = MODINDPREC;
   	  index_recopie += 1;
   	  break;
   	 case('$'): /* retour niveau normal (annule indices/exposants) */
   	  cible[index_recopie] = MODNORMAL;
   	  index_recopie += 1;
   	  break;
   	 case('#'): /* separateur */
   	  break;
   	 case('<'):
   	  if (index_lecture < longueur)
   	  {
   	   code2 = source[index_lecture];
   	   index_lecture += 1;
   	   switch(code2)
   	   {
   	    case('<'): /* symbole "tres inferieur" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\070';
   	     index_recopie += 1;
   	     break;
   	    case('='): /* symbole "inferieur ou egal" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\066';
   	     index_recopie += 1;
   	     break;
   	    case('>'): /* symbole "equivalent" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\071';
   	     index_recopie += 1;
   	     break;
   	    default: /* rien de special */
   	     cible[index_recopie] = code1;
   	     index_recopie += 1;
   	     index_lecture -= 1;
   	     break;
   	   }
   	  }
   	  else
   	  {
   	   cible[index_recopie] = code1;
   	   index_recopie += 1;
   	  }
   	  break;
   	 case('>'):
   	  if (index_lecture < longueur)
   	  {
   	   code2 = source[index_lecture];
   	   index_lecture += 1;
   	   switch(code2)
   	   {
   	    case('<'): /* symbole "identique" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\072';
   	     index_recopie += 1;
   	     break;
   	    case('='): /* symbole "superieur ou egal" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\065';
   	     index_recopie += 1;
   	     break;
   	    case('>'): /* symbole "tres superieur" */
   	     cible[index_recopie] = MODGRAPHLOCAL;
   	     index_recopie += 1;
   	     cible[index_recopie] = '\067';
   	     index_recopie += 1;
   	     break;
   	    default: /* rien de special */
     	     cible[index_recopie] = code1;
   	     index_recopie += 1;
   	     index_lecture -= 1;
   	   }
   	  }
   	  else
   	  {
   	   cible[index_recopie] = code1;
   	   index_recopie += 1;
   	  }
   	  break;
   	 default:
   	  cible[index_recopie] = code1;
   	  index_recopie += 1;
   	  break;
   	}
   }
   cible[index_recopie] = '\0';
}

/*****************************************************
 * Fonction interne : L3BaseMin
 *------------------------------------------------
 * Parametres :
 *   min : valeur a traiter
 *   pas : valeur de traitement
 *
 * Valeur de retour : valeur generee
 *
 * Description : genere le plus grand multiple de pas
 *                  inferieur ou egal a min
 * Erreurs : aucune
 *****************************************************/


double
L3BaseMin(
	  double min,
	  double pas
)
{
   double          basemin;
   double	   rem;
   
/****** ancien code : pbs numeriques (CRAY) 
   basemin = floor(min / pas) * pas;
   if (basemin + pas <= min)
      basemin += pas;
*/

   rem = fmod(min,pas);
   if ( fabs (rem/pas) < 0.001 || fabs((rem-pas)/pas) < 0.001) {
   /* voir man de fmod : peut retourner 0 a +/- pas pres */
   	basemin = min;
   }
   else {
   	if ( rem < 0.0 ) {
  	 	basemin = min-rem-pas;
   	}
   	else {
   		basemin = min-rem;
   	}
   }
   
   return (basemin);
}


/******************************************************
 * Fonction interne : L3BaseMax
 *----------------------------------------------------
 * Parametres :
 *   max : valeur a traiter
 *   pas : valeur de traitement
 *
 * Valeur de retour : valeur generee
 *
 * Description : genere le plus petit multiple de pas superieur
 *                 ou egal a max
 * Erreurs : aucune
 ************************************************/


double
L3BaseMax(
	  double max,
	  double pas
)
{
   double          basemax;
   double	   rem;
   
/********** ancien code : pbs numeriques (CRAY)
   basemax = ceil(max / pas) * pas;
   if (basemax - pas >= max)
      basemax -= pas;
*/

   rem = fmod(max,pas);
   if ( fabs (rem/pas) < 0.001 || fabs((rem-pas)/pas) < 0.001) {
   /* voir man de fmod : peut retourner 0 a +/- pas pres */
   	basemax = max;
   }
   else {
   	if ( rem < 0.0 ) {
   		basemax = max-rem;
   	}
  	else {
   		basemax = max-rem+pas;
   	}
   }
   
   return (basemax);
}


/********************************************************
 * Fonction interne : L3BaseMinLog
 *--------------------------------------------------------
 * Parametres :
 *   max : valeur a traiter
 *   pas : valeur de traitement
 *
 * Valeur de retour : valeur generee
 *
 * Description : genere la plus grande puissance de exp10(pas)
 *                inferieure ou egale a min
 *
 * Erreurs : aucune
 ****************************************************/


double
L3BaseMinLog(
	     double min,
	     double pas
)
{
   double          basemin;

   basemin = exp10(floor(log10(min) / pas) * pas);
   if (basemin * exp10(pas) <= min)
      basemin *= exp10(pas);

   return (basemin);
}

/********************************************
 * Fonction interne : L3ChiffresSignificatifs
 *-------------------------------------------
 * Parametres :
 *   inf : nombre voisin inferieur
 *   val : nombre dont desire evaluer la precision d'affichage
 *   sup : nombre voisin superieur
 *
 * Valeur de retour : nombre de chiffres significatifs
 *   necessaire pour que l'affichage de val soit different
 *   de celui de inf et sup.
 *
 * Description : evalue la precision d'affichage
 *   d'un nombre a partir de deux nombres l'encadrant.
 *
 * Erreurs : aucune
 *********************************************/


int
L3ChiffresSignificatifs(
			double inf,
			double val,
			double sup)
{
   double          delta, rapport;
   int             resultat;

   if (val - inf < sup - val)
      delta = val - inf;
   else
      delta = sup - val;
   if (fabs(delta) > fabs(val))
      delta = val;

   if (delta != 0.0)
      rapport = val / delta;
   else
      rapport = 1.0;

   /* resultat = ((int) floor(log10(fabs(rapport)) + 1.0)); */
   resultat = L3Nint(log10(fabs(rapport)) + 1.0);

   return resultat;
}

/****************************************
 * Fonction interne : L3TailleGraduation
 *--------------------------------------
 * Parametres :
 *   typegraduation : type de la graduation
 *   typeaxe        : type de l'axe
 *
 * Valeur de retour : aucune
 *
 * Description : evalue la taille d'une graduation
 *
 * Erreurs : aucune
 **************************************************/


double
L3TailleGraduation(
		   int typegraduation,
		   int typeaxe
)
{
   double          taille = 1.0;

   switch (typegraduation) {
    case GRAD_EXTERIEURE:
      if (typeaxe == AXE_CREUX)
	 taille = TAILLEGRADEXT;
      else
	 taille = 1.0;
      break;
    case GRAD_PRINCIPALE:
      if (typeaxe == AXE_CREUX)
	 taille = TAILLEGRADPRI;
      else
	 taille = 1.0;
      break;
    case GRAD_INTERMEDIAIRE:
      if (typeaxe != AXE_PLEIN)
	 taille = TAILLEGRADINT;
      else
	 taille = 1.0;
      break;
   }
   return (taille);
}


/***************************************
 * Fonction interne : L3PasDeGraduation
 *--------------------------------------
 * Parametres :
 *   largeur      : espace a graduer
 *   nintervalles : nombre de graduations
 *   p_nf         : pointeur sur la valeur de base generee
 *
 * Valeur de retour : valeur generee
 *
 * Description : genere une valeur "esthetique" du pas de
 *               graduation en accord avec nintervalles
 *
 * Erreurs : aucune
 ********************************************************/


double
L3PasDeGraduation(
		  double largeur,
		  int nintervalles,
		  double *p_nf
)
{
   double          pas, rapport;
   int             fini;
   int             ninter, nintermin = 1, nintermax = 1;
   int             n, bon_n, nmin = 1, nmax = 1;

   n = nintervalles;
   fini = FAUX;
   while (fini == FAUX) {
      pas = L3BeauNombre(largeur / n, VRAI, p_nf);
      
      /*** ancien code : ninter = (int) ceil(largeur / pas)
           meme probleme que dans L3CalculValeurs pour les valeurs
           proches d'un entier par execes -> on les ramene a ce plus
           proche entier pour eviter un saut de +1 non justifie
      ***/
      rapport = largeur/pas;
      if ( fabs(rapport-(double)L3Nint(rapport))< 0.0001*rapport ) {
      	ninter = L3Nint(rapport);
      } else {
        ninter = (int)ceil(rapport);
      }
      
      if (ninter < nintervalles) {
	 n++;
      } else {
	 nmax = n;
	 nintermax = ninter;
	 fini = VRAI;
      }
   }

   n = nintervalles;
   fini = FAUX;
   while (fini == FAUX) {
      pas = L3BeauNombre(largeur / n, VRAI, p_nf);
      
      /*** cf remarque precedente
      ***/
      rapport = largeur/pas;
      if ( fabs(rapport-(double)L3Nint(rapport))< 0.0001*rapport ) {
      	ninter = L3Nint(rapport);
      } else {
        ninter = (int)ceil(rapport);
      }
      
      if (ninter > nintervalles) {
	 if (n >= 2) {
	    n--;
	 } else {
	    nmin = n;
	    nintermin = ninter;
	    fini = VRAI;
	 }
      } else {
	 nmin = n;
	 nintermin = ninter;
	 fini = VRAI;
      }
   }

   if (abs(nintermin - nintervalles) < abs(nintermax - nintervalles)) {
      bon_n = nmin;
   } else {
      bon_n = nmax;
   }

   pas = L3BeauNombre(largeur / bon_n, VRAI, p_nf);
   return (pas);
}

/**********************************
 * Fonction interne : L3BeauNombre
 *---------------------------------
 * Parametres :
 *   x     : valeur initiale
 *   round : indicateur d'arrondi par valeur inferieure
 *   p_nf  : pointeur sur la valeur de base generee
 *
 * Valeur de retour : valeur generee
 *
 * Description : genere une valeur "esthetique" proche
 *                d'une valeur initiale
 * Erreurs : aucune
 ******************************************************/


double
L3BeauNombre(
	     double x,
	     int round,
	     double *p_nf
)
{
   int             exp;
   double          f;

   exp = floor(log10(x));
   f = x / exp10((double) exp);
   if (round == VRAI) {
      if (f < 1.5)
	 *p_nf = 1.0;
      else if (f < 3.0)
	 *p_nf = 2.0;
      else if (f < 7.0)
	 *p_nf = 5.0;
      else
	 *p_nf = 10.0;
   } else {

      /********
	
	      if (f <= 1.0)
		      *p_nf = 1.0;
	      else if (f <= 2.0)
		      *p_nf = 2.0;
	      else if (f <= 5.0)
		      *p_nf = 5.0;
	      else
		      *p_nf = 10.0;
		
      ********/

      if (f <= 1.0)
	 *p_nf = 1.0;
      else if (f > 9.0)
	 *p_nf = 10.0;
      else
	 *p_nf = ceil(f);
   }
   return (*p_nf * exp10((double) exp));
}

void
L3Realloue(
	 int i /* nombre d'element utilise */ ,
	 int *lgtabval /* taille du tableau */ ,
	 GRADUATION ** p_tabvaleur	/* tableau a surveiller */
)
/*-
	Dans le cas de echelles LOG, le nombre de graduation peut
	depasser le nombre prevu. Dans ce cas il faut agrandir le
	tableau reserve pour les graduations.
*/
{
   if (i >= *lgtabval) {
      /* il va y avoir debordement -> on agrandit le tableau */
      *lgtabval += 10;

      if(*lgtabval < i) *lgtabval = i;

      (*p_tabvaleur) = (GRADUATION *) realloc((char *) (*p_tabvaleur), (unsigned) (*lgtabval * sizeof(GRADUATION)));
      if ((*p_tabvaleur) == NULL) {
	 fprintf(stderr, "L3Grilles - Realloue : plus assez de memoire\n");
      }
   }
}

int 
L3Egal(const double x, const double y,  const double ratio)
{
   /* Test de l'egalite reelle en valeur relative a "ratio" pres */
   return (fabs(x - y) < ratio * fabs(y));
}

double          FacteurBorneMin = 1.e-5;

void
L3VerifieBorne(REEL xmin, REEL xmax, REEL *rmin, REEL *rmax)
{
/*
	en mode log l'utilisateur peut fournir des valeurs <= 0
	il faut donc limiter les degats en imposant une valeur
	seuil positve mais faible.
*/
   double          vmin = xmin, vmax = xmax;
   if (L3Egal(vmax, vmin, 1.e-5)) {
      vmax = vmax + 1.01;
      vmin = vmin - 1.01;
   }

   if (vmax <= 0.0) {
      vmax = 1.0;
   }
   if (vmin <= 0.0) {
      vmin = FacteurBorneMin * vmax;
   }
   *rmin = vmin;
   *rmax = vmax;
} /* L3VerifieBorne */

int 
L3Nint(double d)
{
	return ((int) (d >= 0.0 ? d + 0.5: d - 0.5));
}

/* Fin de Fichier */
