/****************/
/* L3Polygone.c */
/****************/

#include <stdio.h>
#include <stdlib.h>

#include "L3Environnement.h"
#include "L3Polygone.h"

#ifndef lint
    static const char sccs_id[] = "%W% %G% DMA/AIM/LM";
#endif

/******************************************
 * Procedure utilisateur : L3Polygone
 *-----------------------------------
 * Parametres
 *   sousimage : numero de la sous-image concernee
 *   x         : tableau des abcisses des sommets
 *   y         : tableau des ordonnees des sommets
 *   nsommets  : nombre de sommets du polygone (non ferme)

 * Valeur de retour : aucune

 * Description : voir document CEL.V DMA/SIE/IG NT 58.9
                 de juillet 1987 "GRAF CIGALE"

 * Erreurs
 *   probleme d'allocation memoire
 *   nombre de sommets incorrect
 ***************************************************/

void
L3Polygone(int sousimage,REEL x[],REEL y[],int nsommets)
{
   static int     *xpage = NULL, *ypage = NULL;
   static int     *codel3 = NULL;
   static int     *xpoly = NULL, *ypoly = NULL;
   static int      lgcodel3 = 18;
   static int      lgtablo = 8;

   register int    polygoneatracer;
   int             nsom = nsommets;
   int		   i;

   if (nsom > 2)
   {

/*--- Creation des tableaux de travail avec une taille par defaut */

      if (codel3 == NULL)
      {
	 codel3 = (int *) malloc((unsigned int) sizeof(int) * lgcodel3);
	 xpage = (int *) malloc((unsigned int) sizeof(int) * lgtablo);
	 ypage = (int *) malloc((unsigned int) sizeof(int) * lgtablo);
	 xpoly = (int *) malloc((unsigned int) sizeof(int) * lgtablo);
	 ypoly = (int *) malloc((unsigned int) sizeof(int) * lgtablo);

         if (codel3 == NULL || xpage == NULL || ypage == NULL ||
		xpoly == NULL || ypoly == NULL)
         {
	    fprintf(stderr, "L3polygone : probleme d'allocation memoire.\n");
	    return;
         }

      }

/*--- Verification que les buffers sont assez grands
      si non agrandissement permanent                 */

      if ((nsom * 4 + 2) > lgcodel3)
      {
	 lgcodel3 = (nsom * 4 + 2);
	 codel3 = (int *) realloc((char *) codel3, lgcodel3 * sizeof(int));

         if (codel3 == NULL)
         {
	    fprintf(stderr, "L3polygone : probleme d'allocation memoire.\n");
	    return;
         }

      }

      if ((nsom * 2) > lgtablo)
      {
	 lgtablo = nsom * 2;
	 xpage = (int *) realloc((char *) xpage, lgtablo * sizeof(int));
	 ypage = (int *) realloc((char *) ypage, lgtablo * sizeof(int));
	 xpoly = (int *) realloc((char *) xpoly, lgtablo * sizeof(int));
	 ypoly = (int *) realloc((char *) ypoly, lgtablo * sizeof(int));

         if (xpage == NULL || ypage == NULL ||
		xpoly == NULL || ypoly == NULL)
         {
	    (void) fprintf(stderr, "L3polygone : probleme d'allocation memoire.\n");
	    return;
         }

      }

      SETSOUSIMAGE(sousimage);
      polygoneatracer = FAUX;

      /*----------------------------*/
      /* Passage en coordonnees L3. */
      /*----------------------------*/

      for (i = 0; i < nsom; i++)
	 L3ConversionCoordonnees(x[i], y[i], xpoly+i, ypoly+i);

/* --- test si le clipping est necessaire */
      {
	 register const int    XClipMin = XCLIPMIN;
	 register const int    YClipMin = YCLIPMIN;
	 register const int    XClipMax = XCLIPMAX;
	 register const int    YClipMax = YCLIPMAX;
	 register const int*   xp = xpoly;
	 register const int*   yp = ypoly;
	 register const int* const xpm = xpoly + nsom;

	 for (; xp < xpm; xp++, yp++)
	 {
	    polygoneatracer = ((*xp >= XClipMin) && (*xp <= XClipMax)) &&
	       ((*yp >= YClipMin) && (*yp <= YClipMax));

	    if (!polygoneatracer) break;

	 }

      }

      if (!polygoneatracer)
      {
	 /*-----------------------------------------------------------------------*
          * Decoupage du polygone par la fenetre rectangulaire de visualisation : *
          * algorithme de SUTHERLAND et HODGMAN (cf ouvrage "Synthese d'image :   *
          * algorithmes elementaires" de G. HEGRON). On decoupe successivement le *
          * polygone par les 4 droites supportant le contour de la fenetre.       *
          * La production de polygones degeneres est possible dans les cas non    *
          * convexes.                                                             *
          *-----------------------------------------------------------------------*/

	 L3ClippingPolygoneDroite(XCLIPMIN, 1, xpoly, ypoly, xpage, ypage, &nsom);

	 if (nsom > 2)
         {
	    L3ClippingPolygoneDroite(YCLIPMIN, 1, ypage, xpage, ypoly, xpoly, &nsom);

	    if (nsom > 2)
            {
	       L3ClippingPolygoneDroite(XCLIPMAX, -1, xpoly, ypoly, xpage, ypage, &nsom);

	       if (nsom > 2)
               {
		  L3ClippingPolygoneDroite(YCLIPMAX, -1, ypage, xpage, ypoly, xpoly, &nsom);

		  if (nsom > 2) polygoneatracer = VRAI;
	       }

	    }

	 }

      }

	 /*------------------------*/
	 /* Generation du code L3. */
	 /*------------------------*/

      if (polygoneatracer == VRAI)
      {
	 register int    i = 0;
	 register int   *xp = xpoly;
	 register int   *yp = ypoly;
	 register int   *cp = codel3;

	 *cp++ = 2 + 2 * nsom;
	 *cp++ = CODEL3POLYGONE;

	 for (; i < nsom; i++)
         {
	    *cp++ = *xp++;
	    *cp++ = *yp++;
	 }

	 L3AppelDrivers(codel3);
      }

   }
   else
   {
      /*---------------------------------------*/
      /* Erreur : nombre de sommets incorrect. */
      /*---------------------------------------*/
      fprintf(stderr,"L3Polygone : nombre de sommets incorrect :%d\n",
		nsommets);
   }

}

/* Fin de Fichier */
