/* identification du source */

static const char* const sccs_id = "%W% %G% Pierre Brochard";

/* Includes */

#include "SpxP.h"
#include "SpxVisuDri.h"
#include "U8const.h"
#include "SpxFtnP.h"
#include "nx.h"
#include "nxsystem.h"

/* Variables d'environnement utilisables */

#define NOMIMSP3	"NOMIMSP3"
#define RUNSP3		"RUNSP3"
#define CASSP3		"CASSP3"
#define RACINESP3	"RACINESP3"
#define NCAS		"NCAS"
#define DESTSP3		"DESTSP3"
#define SERVEURSP3	"SERVEURSP3"
#define PRINTERSP3	"PRINTERSP3"

/*---------------------------------------------------------------------*/

static void     EcritSlotDefinitif();
static void     RemplaceBlanc();

/*---------------------------------------------------------------------*/

static char     ficouvert = FALSE;
static char     dirouvert = FALSE;
static int      numero = 1;
static char     racine[MAX_FILENAME_LENGTH]	/* debut de l'arbre general  */ ;
static char     dircas[MAX_FILENAME_LENGTH]	/* debut de l'arbre du cas   */ ;
static char     runcas[MAX_FILENAME_LENGTH]	/* repertoire courant du cas */ ;
static char     nomfic[MAX_FILENAME_LENGTH]	/* fichier courant  local    */ ;
static char     nomima[MAX_FILENAME_LENGTH]	/* nom de l'image courante   */ ;
static char     spool = FALSE	/* ecriture spooler cray uniquement */ ;

/*---------------------------------------------------------------------*/

void
#ifdef CRAY
SPXOPENFICHIER(_fcd Pathl3 , _fcd Suffixe )
#else
FUNCDECL(spxopenfichier)( char* Pathl3 , char* Suffixe)
#endif
{
#ifdef CRAY
   char * pathl3 = _fcdtocp(Pathl3);
   char * suffixe = _fcdtocp(Suffixe);
#else
   char * pathl3 = Pathl3;
   char * suffixe = Suffixe;
#endif

   SpxOpenFichier(pathl3,suffixe);
}

void
SpxOpenFichier(char* pathl3,char* suffixe)
{
	time_t          temps = time(NULL);
	struct tm      *ladate = localtime(&temps);
	int             i = 0;
	static int      run_ouvert = FALSE;

    /* directory general */
#if defined(SITE_CEA) && defined(CRAY)
	spool = TRUE;
#else
	spool = FALSE;
#endif

	if (getenv(RACINESP3))
	{
	    strcpy(racine, getenv(RACINESP3));
#ifdef CRAY
	    spool = FALSE;
#endif
	}
	else
	{

	    if (!spool)
	    {		/* cas autre que CRAY */
#ifdef SITE_CEA
		char username[L_U8GetUser];

		U8GetUser(username);

		if(!strlen(username))
		{
		    i = -1;
		}
		else
		{
		    i = s8_stappli(username, racine);
		}
#else
		i = -1;
#endif

		if (i < 0)
		{
		    strcpy(racine, getenv("HOME"));
		}

		strcat(racine, "/VISU");
	    }

	}

    /* directory du cas */
	if (getenv(CASSP3) != NULL) {
	    strcpy(dircas, getenv(CASSP3));
	} else {
#if defined(SITE_CEA) && defined(CRAY)
	    if (s8_ginfo(NCAS, dircas) != NULL) {
		strcpy(dircas, "INTER");
	    }
#else
	    if (getenv(NCAS) != NULL) {
		strcpy(dircas, getenv(NCAS));
	    } else {
		strcpy(dircas, "INTER");
	    }
#endif
	}
	if (!run_ouvert) {
	/* un seul directory de run pour ce job */

	/* date et heure du run */
	    sprintf(runcas, "%2d%2d%2d_%2d%2d%2d",
			    ladate->tm_year, ladate->tm_mon + 1, ladate->tm_mday,
			    ladate->tm_hour, ladate->tm_min, ladate->tm_sec);
	    RemplaceBlanc(runcas);

#ifdef CRAY
	    if (getenv("QSUB_REQID") != NULL) {
	    /* serialisation supplementaire pour eviter les conflits */
		char           *reqid = malloc(strlen(getenv("QSUB_REQID")) + 1);
		if (reqid) {
		    char           *numreqid = NULL;

		    strcpy(reqid, getenv("QSUB_REQID"));
		    numreqid = strtok(reqid, ".");
		    if (numreqid) {
			strcat(runcas, "_");
			strcat(runcas, numreqid);
		    }
		    free(reqid);
		}
	    }
#endif

	    if (getenv(RUNSP3) != NULL) {
		strcpy(runcas, getenv(RUNSP3));
	    }
	    run_ouvert = TRUE;
	}
    /* nom de l'image */
	if (getenv(NOMIMSP3) != NULL) {
	    sprintf(nomima, "%s_%3d", getenv(NOMIMSP3), numero);
	} else {
	    sprintf(nomima, "%3d", numero);
	}

	strcat(nomima, suffixe); 
	RemplaceBlanc(nomima);
    /* nom du fichier disque */
	strcpy(nomfic, nomima);

	numero++;

	if (spool) {
   
	/* cas du cray seulement */

	} else {
	/* Creation de la racine si necessaire */
	    strcpy(pathl3, racine);

	    i = mkdir(pathl3, 0700);

	    if (i == -1 && errno != EEXIST)
	    {
		fprintf(stderr, "Creation de %s impossible (racine)\n", pathl3);
		pathl3 = '\0';
		return;
	    }

	/* Creation des repertoires si necessaire */
	    strcat(pathl3, "/");
	    strcat(pathl3, dircas);

	    i = mkdir(pathl3, 0700);

	    if (i == -1 && errno != EEXIST)
	    {
		fprintf(stderr, "Creation de %s impossible (cas)\n", pathl3);
		pathl3 = '\0';
		return;
	    }

	    strcat(pathl3, "/");
	    strcat(pathl3, runcas);

	    i = mkdir(pathl3, 0700);

	    if (i == -1 && errno != EEXIST)
	    {
		fprintf(stderr, "Creation de %s impossible (run)\n", pathl3);
		pathl3 = '\0';
		return;
	    }

	    dirouvert = TRUE;
	}

    /* ouverture du fichier lui-meme */

	if (!spool)
	{
	    strcpy(pathl3, racine);
	    strcat(pathl3, "/");
	    strcat(pathl3, dircas);
	    strcat(pathl3, "/");
	    strcat(pathl3, runcas);
	    strcat(pathl3, "/");
	    strcat(pathl3, nomfic);
	}
	else
	{
	    pid_t mypid = getpid();

	    sprintf(pathl3,"Fichier%d.sp3",(int) mypid);
	}

    /**/

	ficouvert = TRUE;

	return ;
}

/* --------------------------------------------------------- */
/*     ferme fichier                                         */
/* --------------------------------------------------------- */

void
#ifdef CRAY
SPXFERMEFICHIER(char* p )
#else
FUNCDECL(spxfermefichier)(char* p)
#endif
{
   SpxFermeFichier(p);
}

void
SpxFermeFichier(char* p)
{

   if (ficouvert)
   {
      ficouvert = FALSE;
      EcritSlotDefinitif( p );
   }

}

/* Mise a jour du fichier slot definitif */

static void
EcritSlotDefinitif( char* pathl3 )
{
#if defined(SITE_CEA) && defined(CRAY)
   char            spool_path[512];
   FILE           *dst;
   int             i;

   if (spool) {
      /* creation du slot */
      strcpy(spool_path, "");
      i = s8_spool_file("images", pathl3, spool_path);
      if (i != 0) {
	 fprintf(stderr,
		 " Narcisse : PB sur la creation du spool du fichier %s\n",
		 pathl3);
	 return;
      }
      i = s8_spool_slot("images", spool_path, "visu", runcas, nomfic);
      if (i != 0) {
	 fprintf(stderr,
		 " Narcisse : PB sur le slot du fichier%s\n", spool_path);
	 return;
      }
      fprintf(stderr, " Narcisse : fichier %s/%s/%s OK\n", 
              dircas, runcas, nomfic);

      /* Destruction de l'image originale */
      unlink(pathl3);
      return;
   }
#endif
   fprintf(stderr, "Narcisse : %s OK\n", pathl3);
}

static void
RemplaceBlanc(char *c)
{
   char           *p = c;
   while (*p) {
      *p = (*p == ' ') ? '0' : *p;
      p++;
   }
}

#ifdef CRAY
int SYSTEM(_fcd txt){
   int status;
   int l = _fcdlen(txt);
   char *cmd = malloc(l + 1);

   if (!cmd) {
      fprintf(stderr, "SYSTEM : Allocation memoire impossible\n");
      return -1;
   }
   
   strncpy(cmd, _fcdtocp(txt), l);
   cmd[l] = '\0';

   status = system(cmd);
   free(cmd);
   return(status);
}
#endif
/* Fin de fichier */
