/*                           N X - A M                                 */
/*                           ---------                                 */
/*                          C interface                                */

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.  
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/
/* source ident */
/***************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Copyr 1992 MA/AIM";
#endif

/* modifs Pierre Brochard 1996/97/98 */

/***************************************************************************/
/* includes */
/***************************************************************************/

#include "nxP.h"
#include "nxerror.h"

#define NXCONV_H_IMPLEMENT
#define NXP_H_IMPLEMENT
#   include "nx.h"
#undef NXP_H_IMPLEMENT
#undef NXCONV_H_IMPLEMENT

/*******************************************************************/
/*******************************************************************/
/* --------------- Conversion functions     ---------------------- */
/*******************************************************************/
/*******************************************************************/

#if (!defined(CRAY) || defined(_CRAYMPP)) && defined(NX_LITTLEENDIAN)
#   define NXCONV_LITTLE_BIG32(ss) {register nx_unsigned32 stemp = ss;\
	    register nx_unsigned32 sst1 = stemp >> 24 & 0xff;\
	    register nx_unsigned32 sst2 = stemp >> 8  & 0xff00;\
	    register nx_unsigned32 sst3 = stemp << 8  & 0xff0000;\
	    ss = stemp << 24 | sst3 | sst2 | sst1;}
#else
#   define NXCONV_LITTLE_BIG32(ss)
#endif

/*******************************************************************/
/*                     nxconv_real_cr64_to_ie32                    */
/* Conversion of CRAY real to IEEE 32 bits real big endian         */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY CRI where the 24 right bits of the cray mantissa    */
/* are lost...                                                     */
/* les tableaux s et d doivent etre disjoints                      */
/* car vectorise sur CRAY CRI vectoriel                            */
/* sur CRAY non T3D, les tailles de tableaux doivent etre          */
/* multiples de 8.                                                 */
/* nb n'est pas une taille mais un nombre d'elements a convertir   */
/*                                                                 */
/* sur machines little endian, les ie* et cr* seront little endian */
/* il faudra donc inverser avant et apres                          */
/* nxconv_little_big*                                              */
/* nxconv_real_cr64_to_ie32 sur machines little endian <=>         */
/* nxconv_little_big32(buff2,buff2, 2*nb);                         */
/* nxconv_real_cr64_to_ie32(buff2,buff, nb, &cr);version big endian*/
/* nxconv_little_big32(buff,buff, nb);                             */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_cr64_to_ie32(const nx_unsigned32 s[],nx_unsigned32 d[],
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int i = 0;
	register int nb2 = nb*2;

	for(;i<nb2;i+=2)
	{
	    register int j = i/2;
	    register nx_unsigned32 ss = s[i];
	    register nx_unsigned32 b47;

	    NXCONV_LITTLE_BIG32(ss);
	    b47 = ss & 0x8000;

	    if (b47)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_int32 expn = ss >> 16 & 0x7fff;

		expn -= 16258; /* (expn - 16384) + 127 - 1 */

		if (expn < 0)                       /* underflow */
		{
		    d[j] = 0;
		    nxerror1 = 38;
		    rc = -1;
		}
		else if (expn >= 255)                /* overflow  */
		{
		    d[j] = sign | 0x7f7fffff; /* max float ieee */
		    nxerror2 = 39;
		    rc = -1;
		}
		else
		{
		    register nx_unsigned32 mant = ss << 8 & 0x7fff00;
		    register nx_unsigned32 suite = s[i+1];

		    expn = expn << 23 & 0xff800000;
		    NXCONV_LITTLE_BIG32(suite);
		    mant |= suite >> 24 & 0xff;
		    d[j] = sign | expn | mant;
		}

		NXCONV_LITTLE_BIG32(d[j]);
	    }
	    else d[j] = 0;      /* real = 0.00 */

	}

	*cr = rc;
	NXERRNO = nxerror1 + (nxerror2<<10);
}
#else
nxconv_real_cr64_to_ie32(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int i = 0;
	register int nb2 = nb/2;
	register int nb21 = (nb+1)/2;

    /* conversion et compactage */

	for(;i<nb21;i++)
	{
	    register unsigned temp = 0;
	    register unsigned ss = s[2*i];
	    register unsigned b47 = ss & 0x800000000000;

	    if (b47)
	    {
		register int sign = ss & 0x8000000000000000;
		register int expn = ss >> 48 & 0x7fff;
		register int mant;

		expn -= 16258; /* expn - 16384 + 127 - 1 */

		if (expn < 0)                         /* underflow */
		{
		    nxerror1 = 43;
		    rc = -1;
		}
		else if (expn >= 255)                  /* overflow  */
		{
		/* max float ieee */
		    temp = sign | 0x7f7fffff00000000; 
		    nxerror2 = 44;
		    rc = -1;
		}
		else
		{
		    expn <<= 55;
		    mant = ss << 8 & 0x7fffff00000000;
		    temp = sign | expn | mant;
		}

	    }

	    d[i] = temp;
	}

	for(i=0;i<nb2;i++)
	{
	    register unsigned temp = 0;
	    register unsigned ss = s[2*i+1];
	    register unsigned b47 = ss & 0x800000000000;

	    if (b47)
	    {
		register int sign = ss & 0x8000000000000000;
		register int expn = ss >> 48 & 0x7fff;
		register int mant;

		expn -= 16258; /* expn - 16384 + 127 - 1 */

		if (expn < 0)                         /* underflow */
		{
		    nxerror1 = 43;
		    rc = -1;
		}
		else if (expn >= 255)                  /* overflow  */
		{
		/* max float ieee */
		    temp = sign | 0x7f7fffff00000000; 
		    nxerror2 = 44;
		    rc = -1;
		}
		else
		{
		    expn <<= 55;
		    mant = ss << 8 & 0x7fffff00000000;
		    temp = sign | expn | mant;
		}

	    }

	    d[i] |= temp >> 32;
	}

	*cr = rc;
	NXERRNO = nxerror1 + nxerror2<<10;
}
#endif

/*******************************************************************/
/*                    nxconv_real_ie32_to_cr64(S,D,NB)             */
/*                                                                 */
/* Conversion of ieee reals to CRAY reals                          */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian                                              */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie32_to_cr64(const nx_unsigned32 s[],nx_unsigned32 d[],
		int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = 2*i;
	    register nx_unsigned32 ss = s[i];
	/* b0 : 0 IEEE si exposant 0 */
	    register nx_unsigned32 b0;

	    NXCONV_LITTLE_BIG32(ss);
	    b0 = ss & 0x7f800000;

	    if (b0)
	    {
		register nx_int32 sign,expn,mant;

		sign = ss & 0x80000000;
		expn = ss >> 23 & 0xff;
		expn += 16258; /* (expn - 127) + 16384 + 1 */
		expn = expn << 16 & 0xffff0000;
		mant = ((ss & 0x7fffff) | 0x800000) >> 8 & 0xffffff;
		d[j] = sign | expn | mant;
		d[j+1] = ss << 24 & 0xffffff00;
		NXCONV_LITTLE_BIG32(d[j]);
		NXCONV_LITTLE_BIG32(d[j+1]);
	    }
	    else
		d[j] = d[j+1] = 0;

	}

}
#else
nxconv_real_ie32_to_cr64(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = i/2;
	    register int i2 = i%2;
	    register unsigned ss = s[j];
	    register unsigned rs;  
	    register unsigned b0;

	    if(i2)
	    {
		rs = ss & 0xffffffff;
	    }
	    else
	    {
		rs = ss >> 32;
	    }

	    b0 = rs & 0x7f800000;

	    if(b0)
	    {
		register unsigned mant;  
		register unsigned sign = (rs >> 31) << 63;
		register unsigned expn = rs >> 23 & 0xff;

		expn += 16258; /* (expn - 127) + 16384 + 1 */
		expn <<= 48;
		mant = (rs & 0x7fffff | 0x800000) << 24;  /* bits 54 to 32 */
		d[i] = sign | expn | mant;
	    }
	    else d[i] = 0;

	}

}
#endif

/*******************************************************************/
/*                     nxconv_real_cr64_to_ie64                    */
/* Conversion of a CRAY simple precision real to an IEEE           */
/* double precision real                                           */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian                                              */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_cr64_to_ie64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	int nxerror1 = 0,nxerror2 = 0;
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register nx_unsigned32 ss1 = s[i];
	    register nx_unsigned32 b47;

	    NXCONV_LITTLE_BIG32(ss1);
	    b47 = ss1 & 0x8000;

	    if (b47)
	    {
		register nx_unsigned32 sign = ss1 & 0x80000000;
		register nx_int32 expn = ss1 >> 16 & 0x7fff;

		expn -= 15362; /* - 1023 + 16384 + 1 */

		if (expn < 0)                       /* underflow */
		{
		    d[i] = d[i+1] = 0;
		    nxerror1 = 40;
		}
		else if (expn >= 2047)               /* overflow  */
		{
		/* max double ieee */
		    d[i] = sign | 0x7fefffff; 
		    d[i+1] = 0xffffffff;
		    nxerror2 = 41;
		}
		else
		{
		    register nx_unsigned32 ss2 = s[i+1];
		    register nx_unsigned32 mant = ss1 << 5 & 0xfffe0;

		    NXCONV_LITTLE_BIG32(ss2);
		    expn = expn << 20 & 0xfff00000;
		    mant |=  ss2 >> 27 & 0x1f;
		    d[i] = sign | expn | mant;
		    d[i+1] = ss2 << 5 & 0xffffffe0;
		}

		NXCONV_LITTLE_BIG32(d[i]);
		NXCONV_LITTLE_BIG32(d[i+1]);
	    }
	    else d[i] = d[i+1] = 0;      /* real = 0.00 */

	}

	NXERRNO = nxerror1 + (nxerror2<<10);
}
#else
nxconv_real_cr64_to_ie64(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	int nxerror1 = 0,nxerror2 = 0;
        register int i = 0;

	for(;i<nb;i++)
	{
	    register unsigned ss = s[i];
	    register unsigned b47 = ss & 0x800000000000;

	    if (b47)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register int expn = ss >> 48 & 0x7fff;

		expn -= 15362; /* - 1023 + 16384 + 1 */

		if (expn < 0)                         /* underflow */
		{
		    d[i] = 0;
		    nxerror1 = 45;
		}
		else if (expn >= 2047)                 /* overflow  */
		{
		/* max double ieee */
		    d[i] = sign | 0x7fefffffffffffff; 
		    nxerror2 = 46;
		}
		else
		{
		    register unsigned mant = ss << 5 & 0xfffffffffffe0;

		    expn <<= 52;
		    d[i] = sign | expn | mant;
		}

	    }
	    else d[i] = 0;

	}

	NXERRNO = nxerror1 + nxerror2<<10;
}
#endif

/*******************************************************************/
/*                    nxconv_real_ie64_to_cr64(S,D,NB)             */
/*                                                                 */
/* Conversion of ieee double precision reals                       */
/* to CRAY simple precision reals                                  */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian                                              */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie64_to_cr64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register nx_unsigned32 ss = s[i];
	    register nx_unsigned32 b0;

	    NXCONV_LITTLE_BIG32(ss);
	    b0 = ss & 0x7ff00000;

	    if (b0)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_unsigned32 expn = ss >> 20 & 0x7ff;
		register nx_unsigned32 mant =
		    ((ss & 0xfffff) | 0x100000) >> 5 & 0x7ffffff;
		register nx_unsigned32 ss2 = s[i+1];

		expn += 15362; /* (expn - 1023) + 16384 + 1 */
		expn = expn << 16 & 0xffff0000;
		d[i] = sign | expn | mant;
		NXCONV_LITTLE_BIG32(ss2);
		d[i+1] = (ss << 27 & 0xf8000000) + (ss2 >> 5 & 0x7ffffff);

		NXCONV_LITTLE_BIG32(d[i]);
		NXCONV_LITTLE_BIG32(d[i+1]);
	    }
	    else
		d[i] = d[i+1] = 0;

	}

}
#else
nxconv_real_ie64_to_cr64(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register unsigned ss = s[i];
	    register unsigned b0 = ss & 0x7ff0000000000000;

	    if (b0)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register unsigned expn = ss >> 52 & 0x7ff; /* bits 62 to 52 */
		register unsigned mant = ((ss<<12)>>17) | 0x800000000000;  

		expn += 15362; /* - 1023 + 16384 + 1 */
		expn <<= 48;
		d[i] = sign | expn | mant;
	    } 
	    else d[i] = 0;

	}

}
#endif

/*******************************************************************/
/*                     nxconv_real_cr128_to_ie64                   */
/* Conversion of a CRAY double precision real to an IEEE           */
/* double precision real                                           */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian                                              */
/* ou sur CRAY CRI                                                 */
/* The 60 right bits of the cray mantissa are lost...              */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_cr128_to_ie64(const nx_unsigned32 s[],nx_unsigned32 d[],
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register int i2 = 4*j;
	    register nx_unsigned32 ss = s[i2];
	    register nx_unsigned32 b47;

	    NXCONV_LITTLE_BIG32(ss);
	    b47 = ss & 0x8000;

	    if (b47)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_int32 expn = ss >> 16 & 0x7fff;

		expn -= 15362; /* - 1023 + 16384 + 1 */

		if (expn < 0)                       /* underflow */
		{
		    d[i] = d[i+1] = 0;
		    nxerror1 = 40;
		    rc = -1;
		}
		else if (expn >= 2047)               /* overflow  */
		{
		/* max double ieee */
		    d[i] = sign | 0x7fefffff; 
		    d[i+1] = 0xffffffff;
		    nxerror2 = 41;
		    rc = -1;
		}
		else
		{
		    register nx_unsigned32 ss2 = s[i2+1];
		    register nx_unsigned32 ss3 = s[i2+2];
		    register nx_unsigned32 mant,suite1,suite2;

		    NXCONV_LITTLE_BIG32(ss2);
		    NXCONV_LITTLE_BIG32(ss3);
		    expn <<= 20; 
		    expn &= 0xfff00000;
		    mant = ss << 5 & 0xfffe0;
		    suite1 = ss2 >> 27 & 0x1f;
		    mant |= suite1;
		    suite2 = ss2 << 5 & 0xffffffe0;
		    d[i] = sign | expn | mant;
		    d[i+1] = suite2 + (ss3 >> 27 & 0x1f);
		}

		NXCONV_LITTLE_BIG32(d[i]);
		NXCONV_LITTLE_BIG32(d[i+1]);
	    }
	    else d[i] = d[i+1] = 0;      /* real = 0.00 */

	}

	*cr = rc;
	NXERRNO = nxerror1 + (nxerror2<<10);
}
#else
nxconv_real_cr128_to_ie64(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int i2 = 2*i;
	    register unsigned ss = s[i2];
	    register unsigned b47 = ss & 0x800000000000;

	    if (b47)
	    {
		register int sign = ss & 0x8000000000000000;
		register int expn = ss >> 48 & 0x7fff;

		expn -= 15362; /* - 1023 + 16384 + 1 */

		if (expn < 0)                         /* underflow */
		{
		    d[i] = 0;
		    nxerror1 = 45;
		    rc = -1;
		}
		else if (expn >= 2047)                 /* overflow  */
		{
		/* max double ieee */
		    d[i] = sign | 0x7fefffffffffffff; 
		    nxerror2 = 46;
		    rc = -1;
		}
		else
		{
		    register int mant;  

		    expn <<= 52;
		    mant = ((ss<<17)>>12) + (s[i2+1]>>59);
		    d[i] = sign | expn | mant;
		}

	    }
	    else
		d[i] = 0; 
     
	}

	*cr = rc;
	NXERRNO = nxerror1 + nxerror2<<10;
}
#endif

/*******************************************************************/
/*                    nxconv_real_ie64_to_cr128                    */
/*                                                                 */
/* Conversion of IEEE double precision reals                       */
/* to CRAY double precision reals                                  */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie64_to_cr128(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register int i2 = 4*j;
	    register nx_unsigned32 ss = s[i];
	    register nx_unsigned32 b0;

	    NXCONV_LITTLE_BIG32(ss);
	    b0 = ss & 0x7ff00000;

	    if (b0)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_unsigned32 expn = ss >> 20 & 0x7ff;
		register nx_unsigned32 ss2 = s[i+1];
		register nx_unsigned32 mant;  

		expn += 15362; /* - 1023 + 16384 + 1 */
		expn <<= 16;
		expn &= 0xffff0000;
		mant = ((ss & 0x000fffff) | 0x100000) >> 5 & 0x7ffffff;
		d[i2] = sign | expn | mant;
		NXCONV_LITTLE_BIG32(ss2);
		d[i2+1] = (ss<<27 & 0xf8000000) + (ss2 >> 5 & 0x7ffffff);
		d[i2+2] = ss2 << 27 & 0xf8000000;
		d[i2+3] = 0;

		NXCONV_LITTLE_BIG32(d[i2]);
		NXCONV_LITTLE_BIG32(d[i2+1]);
		NXCONV_LITTLE_BIG32(d[i2+2]);
		NXCONV_LITTLE_BIG32(d[i2+3]);
	    }
	    else
		d[i2] = d[i2+1] = d[i2+2] = d[i2+3] = 0;

	}

}
#else
nxconv_real_ie64_to_cr128(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = 2*i;
	    register unsigned ss = s[i];
	    register unsigned b0 = ss & 0x7ff0000000000000;

	    if (b0)
	    {
		register int sign = ss & 0x8000000000000000;
		register int expn =  ss >> 52 & 0x7ff;
		register int mant;  

		expn += 15362; /* - 1023 + 16384 + 1 */
		expn <<= 48;
		mant = ((ss << 12) >> 17) | 0x800000000000;       
		d[j] = sign | expn | mant;
		d[j+1] = ss << 59;
	    } 
	    else d[j] = d[j+1] = 0;

	}

}
#endif

/*******************************************************************/
/*                     nxconv_real_cr128_to_ie32                   */
/* Conversion of a CRAY double precision real to an IEEE           */
/* simple precision real                                           */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian                                              */
/* ou sur CRAY CRI                                                 */
/* The 60 right bits of the cray mantissa are lost...              */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_cr128_to_ie32(const nx_unsigned32 s[],nx_unsigned32 d[],
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 4*j;
	    register nx_unsigned32 ss = s[i];
	    register nx_unsigned32 b47;

	    NXCONV_LITTLE_BIG32(ss);
	    b47 = ss & 0x8000;

	    if(b47)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_int32 expn = ss >> 16 & 0x7fff;

		expn -= 16258; /* - 127 + 16384 + 1 */

		if (expn < 0)                       /* underflow */
		{
		    d[j] = 0;
		    nxerror1 = 40;
		    rc = -1;
		}
		else if (expn >= 255)               /* overflow  */
		{
		/* max ieee 32 bits */
		    d[j] = sign | 0x7f7fffff; 
		    nxerror2 = 41;
		    rc = -1;
		}
		else
		{
		    register nx_unsigned32 ss2 = s[i+1];
		    register nx_unsigned32 mant;

		    expn <<= 23; 
		    expn &= 0xff800000;

		    mant = ss << 8 & 0x7fff00;
		    NXCONV_LITTLE_BIG32(ss2);
		    mant |= ss2 >> 24 & 0xff;
		    d[j] = sign | expn | mant;
		}

	    }
	    else d[j] = 0;      /* real = 0.00 */

	    NXCONV_LITTLE_BIG32(d[j]);
	}

	*cr = rc;
	NXERRNO = nxerror1 + (nxerror2<<10);
}
#else
nxconv_real_cr128_to_ie32(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int i = 0;
	register int nb2 = nb/2;
	register int nb21 = (nb+1)/2;

	for(;i<nb21;i++)
	{
	    register unsigned ss = s[4*i];
	    register unsigned b47 = ss & 0x800000000000;

	    if (b47)
	    {
		register int sign = ss & 0x8000000000000000;
		register int expn =  ss >> 48 & 0x7fff;

		expn -= 16258; /* - 127 + 16384 + 1 */

		if (expn < 0)                         /* underflow */
		{
		    d[i] = 0;
		    nxerror1 = 45;
		    rc = -1;
		}
		else if (expn >= 255)                 /* overflow  */
		{
		/* max ieee 32 bits = Inf -1 */
		    d[i] = sign | 0x7f7fffff00000000; 
		    nxerror2 = 46;
		    rc = -1;
		}
		else
		{
		    register int mant = ss << 8 & 0x7fffff00000000;

		    expn <<= 55;
		    d[i] = sign | expn | mant;
		}

	    }
	    else
		d[i] = 0; 
     
	}

	for(i=0;i<nb2;i++)
	{
	    register unsigned temp = 0;
	    register unsigned ss = s[4*i+2];
	    register unsigned b47 = ss & 0x800000000000;

	    if (b47)
	    {
		register int sign = ss & 0x8000000000000000;
		register int expn =  ss >> 48 & 0x7fff;

		expn -= 16258; /* - 127 + 16384 + 1 */

		if (expn < 0)                         /* underflow */
		{
		    nxerror1 = 45;
		    rc = -1;
		}
		else if (expn >= 255)                 /* overflow  */
		{
		/* max ieee 32 bits = Inf -1 */
		    temp = sign | 0x7f7fffff00000000; 
		    nxerror2 = 46;
		    rc = -1;
		}
		else
		{
		    register int mant = ss << 8 & 0x7fffff00000000;  

		    expn <<= 55;
		    temp = sign | expn | mant;
		}

	    }

	    d[i] |= temp >> 32;
	}

	*cr = rc;
	NXERRNO = nxerror1 + nxerror2<<10;
}
#endif

/*******************************************************************/
/*                    nxconv_real_ie32_to_cr128(S,D,NB)            */
/*                                                                 */
/* Conversion of ieee reals 32 bits to CRAY reals double precision */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian                                              */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie32_to_cr128(const nx_unsigned32 s[],nx_unsigned32 d[],
		int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = 4*i;
	    register nx_unsigned32 ss = s[i];
	    register nx_unsigned32 b0;

	    NXCONV_LITTLE_BIG32(ss);
	    b0 = ss & 0x7f800000;

	    if (b0)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_unsigned32 expn = ss >> 23 & 0xff;
		register nx_unsigned32 mant;

		expn += 16258; /* (expn - 127) + 16384 + 1 */
		expn = expn << 16 & 0xffff0000;
		mant = ((ss & 0x7fffff) | 0x800000) >> 8 & 0xffffff;
		d[j] = sign | expn | mant;
		d[j+1] = ss << 24 & 0xffffff00;
	    }
	    else
		d[j] = d[j+1] = 0;

	    d[j+2] = d[j+3] = 0;
	    NXCONV_LITTLE_BIG32(d[j]);
	    NXCONV_LITTLE_BIG32(d[j+1]);
	}

}
#else
nxconv_real_ie32_to_cr128(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = i/2;
	    register int j2 = i%2;
	    register int i2 = 2*i;
	    register unsigned ss = s[j];
	    register unsigned rs;  
	    register unsigned b0;  

	    if(j2)
	    {
		rs = ss & 0xffffffff;
	    }
	    else
	    {
		rs = ss >> 32;
	    }

	    b0 = rs & 0x7f800000;

	    if(b0)
	    {
		register int sign,expn,mant;  

		sign = (rs >> 31) << 63;                  /* bit No 63 */
		expn = rs >> 23 & 0xff;
		expn += 16258; /* (expn - 127) + 16384 + 1 */
		expn <<= 48;
		mant = (rs & 0x7fffff | 0x800000) << 24;  /* bits 54 to 32 */
		d[i2] = sign | expn | mant;
	    }
	    else d[i2] = 0;

	    d[i2+1] = 0;
	}

}
#endif

/*******************************************************************/
/*                     nxconv_real_cr128_to_cr64                   */
/* Conversion of a CRAY double precision real to a CRAY            */
/* simple precision real                                           */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY CRI                                                 */
/* The 48 right bits of the cray mantissa are lost...              */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_cr128_to_cr64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register int i2 = 4*j;

	    d[i] = s[i2];
	    d[i+1] = s[i2+1];
	}

}
#else
nxconv_real_cr128_to_cr64(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    d[i] = s[2*i];
	}

}
#endif

/*******************************************************************/
/*                     nxconv_real_cr64_to_cr128                   */
/* Conversion of a CRAY simple precision real to a CRAY            */
/* double precision real                                           */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY CRI                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_cr64_to_cr128(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register int i2 = 4*j;

	    d[i2] = s[i];
	    d[i2+1] = s[i+1];
	    d[i2+2] = d[i2+3] = 0;
	}

}
#else
nxconv_real_cr64_to_cr128(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int i2 = 2*i;

	    d[i2] = s[i];
	    d[i2+1] = 0;
	}

}
#endif

/*******************************************************************/
/*                    nxconv_real_ie32_to_ie64(S,D,NB)             */
/*                                                                 */
/* Conversion of IEEE reals to CRAY MPP reals (simple              */
/* precision                                                       */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie32_to_ie64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = 2*i;
	    register nx_unsigned32 ss = s[i];

	    NXCONV_LITTLE_BIG32(ss);

	    if (ss)
	    {
		register nx_int32 sign,expn,mant;
 
		sign = ss & 0x80000000;
		expn = ss >> 23 & 0xff;
		expn += 896 /* (expn - 127) + 1023 */;
		expn <<= 20;
		expn &= 0xfff00000;
		mant = (ss & 0x7fffff) >> 3 & 0x1fffffff;
		d[j] = sign | expn | mant;
		d[j+1] = ss << 29 & 0xfffffff8;
	    }
	    else
		d[j] = d[j+1] = 0;

	    NXCONV_LITTLE_BIG32(d[j]);
	    NXCONV_LITTLE_BIG32(d[j+1]);
	}

}
#else
nxconv_real_ie32_to_ie64(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = i/2;
	    register int i2 = i%2;
	    register unsigned ss = s[j];
	    register int rs;  

	    if(i2)
	    {
		rs = ss & 0xffffffff;
	    }
	    else
	    {
		rs = ss >> 32;
	    }

	    if(rs)
	    {
		register int sign = rs << 32 & 0x8000000000000000;
		register int expn = rs >> 23 & 0xff;
		register int mant;

		expn += 896 /* (expn - 127) + 1023 */;
		expn <<= 52;
		mant = (rs & 0x7fffff) << 29;
		d[i] = sign | expn | mant;
	    }
	    else d[i] = 0;

	}

}
#endif

/*******************************************************************/
/*                     nxconv_real_ie64_to_ie32                    */
/* Conversion of CRAY Mpp real to ieee real, on CRAY Mpp computer  */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian                                              */
/* ou sur CRAY CRI                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie64_to_ie32(const nx_unsigned32 s[], nx_unsigned32 d[],
		int nb, int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int i = 0;
	register int nb2 = nb*2;

	for(;i<nb2;i+=2)
	{
	    register int j = i/2;
	    register nx_unsigned32 ss = s[i];

	    NXCONV_LITTLE_BIG32(ss);

	    if (ss)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_int32 expn = ss >> 20 & 0x7ff;

		expn -= 896 /* (expn - 1023) + 127 */;

		if (expn <= 0)                       /* underflow */
		{
		    d[j] = 0;
		    nxerror1 = 38;
		    rc = -1;
		}
		else if (expn >= 255)                /* overflow  */
		{
		    d[j] = sign | 0x7f7fffff; /* max float ieee */
		    nxerror2 = 39;
		    rc = -1;
		}
		else
		{
		    register nx_unsigned32 ss2 = s[i+1];
		    register nx_unsigned32 mant;

		    NXCONV_LITTLE_BIG32(ss2);
		    expn = expn << 23 & 0xff800000;
		    mant = ss << 3 & 0x7ffff8;
		    mant |= ss2 >> 29 & 0x7;
		    d[j] = sign | expn | mant;
		}

	    }
	    else d[j] = 0;      /* real = 0.00 */

	    NXCONV_LITTLE_BIG32(d[j]);
	}

	*cr = rc;
	NXERRNO = nxerror1 + (nxerror2<<10);
}
#else
nxconv_real_ie64_to_ie32(const unsigned* NXRESTRICT s, unsigned* NXRESTRICT d,
		int nb, int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int i = 0;
	register int nb2 = nb/2;
	register int nb21 = (nb+1)/2;

	for(; i < nb21; i++) 
	{
	    register unsigned temp = 0;
	    register unsigned ss = s[2*i];

	    if (ss) 
	    {
		register int sign = ss & 0x8000000000000000;
		register int expn =  ss >> 52 & 0x7ff;
		register int mant;  

		expn -= 896 /* (expn - 1023) + 127 */;

		if (expn <= 0)  	/* underflow */
		{
		    nxerror1 = 43;
		    rc = -1;
		} 
		else if (expn >= 255)  	/* overflow  */
		{
		/* max float ieee */
		    temp = sign | 0x7f7fffff00000000; /* (right part to zero) */
		    nxerror2 = 44;
		    rc = -1;
		} 
		else 
		{
		    expn <<= 55;
		    mant = ss << 3 & 0x7fffff00000000;
		    temp = sign | expn | mant;
		}

	    }

	    d[i] = temp;
	}

	for(i=0; i < nb2; i++) 
	{
	    register unsigned temp = 0;
	    register unsigned ss = s[2*i+1];

	    if (ss) 
	    {
		register int sign = ss & 0x8000000000000000;
		register int expn =  ss >> 52 & 0x7ff;
		register int mant;  

		expn -= 896 /* (expn - 1023) + 127 */;

		if (expn <= 0)  	/* underflow */
		{
		    nxerror1 = 43;
		    rc = -1;
		} 
		else if (expn >= 255)  	/* overflow  */
		{
		/* max float ieee */
		    temp = sign | (0x7f80000000000000 - 0x100000000)
			& 0xffffffff00000000;	/* (right part to zero) */
		    nxerror2 = 44;
		    rc = -1;
		} 
		else 
		{
		    expn <<= 55;
		    mant = ss << 3 & 0x7fffff00000000;
		    temp = sign | expn | mant;
		}

	    } 

	    d[i] |= temp >> 32;
	}

	*cr = rc;
	NXERRNO = nxerror1 + nxerror2<<10;
}
#endif

/*******************************************************************/
/*                     nxconv_int_cr64_to_ie32                     */
/* Conversion of a CRAY 64 bits integer to an IEEE 32 bits integer */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY CRI                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_int_cr64_to_ie32(const nx_unsigned32 s[],nx_unsigned32 d[],
		int nb,int* cr)
{
	const nx_int32 imax = 0x7fffffff;
	const nx_int32 _imax = (nx_int32) 0x80000000;
	const nx_unsigned32 mask = (nx_unsigned32) ~0;
	int rc = 0,nxerror = 0;
	register unsigned i = 0;
	register unsigned nb2 = nb*2;

	for(;i<nb2;i+=2)
	{
	    register unsigned j = i/2;
	    register nx_unsigned32 ss1 = s[i];
	    register nx_unsigned32 ss2 = s[i+1];
	    register nx_unsigned32 left;

	    NXCONV_LITTLE_BIG32(ss1);
	    NXCONV_LITTLE_BIG32(ss2);
	    left = (ss1 << 1) | (ss2 >> 31 & 0x1);

	    if (ss1 & 0x80000000)
	    {

		if (left ^ mask) /* overflow - */
		{
		    d[j] = _imax;
		    rc = -1;
		    nxerror = 42;
		}
		else /* OK */
		    d[j] = ss2;

	    }
	    else 
	    {

		if(left) /* overflow + */
		{
		    d[j] = imax;
		    rc = -1;
		    nxerror = 42;
		}
		else /* OK */
		    d[j] = ss2;

	    }

	    NXCONV_LITTLE_BIG32(d[j]);
	}

	*cr = rc;
	NXERRNO = nxerror;
}
#else
nxconv_int_cr64_to_ie32(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb,int* cr)
{
	const int ieeemax = 0x7fffffff;
	const int _ieeemax = 0xffffffff80000000;
	int rc = 0,nxerror = 0;
	register int i = 0;
	register int nb2 = nb/2;
	register int nb21 = (nb+1)/2;

    /* cadrage a gauche et compactage */

	for(;i<nb21;i++)
	{
	    register unsigned ss = s[2*i];
	    register int s2 = ss;

	    if(s2 > ieeemax) /* overflow + */
	    {
		ss = ieeemax;
		rc = -1;
		nxerror = 47;
	    }
	    else if(s2 < _ieeemax) /* overflow - */
	    {
		ss = _ieeemax;
		rc = -1;
		nxerror = 47;
	    }

	    d[i] = ss << 32;
	}

	for(i=0;i<nb2;i++)
	{
	    register unsigned ss = s[2*i+1];
	    register int s2 = ss;

	    if (s2 > ieeemax) /* overflow + */
	    {
		ss = ieeemax;
		rc = -1;
		nxerror = 47;
	    }
	    else if (s2 < _ieeemax) /* overflow - */
	    {
		ss = _ieeemax;
		rc = -1;
		nxerror = 47;
	    }


	    d[i] |= ss & 0xffffffff;
	}

	*cr = rc;
	NXERRNO = nxerror;
}
#endif

/*******************************************************************/
/*                    nxconv_int_ie32_to_cr64                      */
/*                                                                 */
/* Conversion of ieee 32 bits integers                             */
/* to CRAY 64 bits integers,                                       */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY CRI                                                 */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_int_ie32_to_cr64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register nx_unsigned32 ss = s[i];
	    register int i2 = 2*i;

	    NXCONV_LITTLE_BIG32(ss);
	    d[i2+1] = ss;

	    if(ss & 0x80000000) d[i2] = 0xffffffff;
	    else d[i2] = 0;

   	    NXCONV_LITTLE_BIG32(d[i2]);
	    NXCONV_LITTLE_BIG32(d[i2+1]);
	}

}
#else
nxconv_int_ie32_to_cr64(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = i/2;
	    register int b32;
	    register unsigned ss = s[j];

	    if (i%2)            
	    {
		b32 = ss & 0x80000000;
		d[i] = ss & 0xffffffff;
	    }
	    else
	    {
		b32 = ss & 0x8000000000000000;
		d[i] = ss >> 32;
	    }

	    if (b32) d[i] |= 0xffffffff00000000;

	}

}
#endif

/****************************************************************/
/*                   NXCONV_ILI(S,D,NB)                         */
/* int <-> long int sur meme machine                            */
/* utile seulement si sizeof(long int) different de sizeof(int) */
/* ce qui est vrai sur machines 64 bits et pas CRAY             */
/* en gros ne pas utiliser cette routine                        */
/****************************************************************/

void
nxconv_ili(int s[],long int d[],int nb)
{
	register int i = 0;

	for(;i<nb;i++) d[i] = s[i];

}

/****************************************************************/
/*                   NXCONV_LII(S,D,NB)                         */
/* int <-> long int sur meme machine                            */
/* utile seulement si sizeof(long int) different de sizeof(int) */
/* ce qui est vrai sur machines 64 bits et pas CRAY             */
/* en gros ne pas utiliser cette routine                        */
/****************************************************************/

void
nxconv_lii(long int s[],int d[],int nb)
{
	register int i = 0;

	for(;i<nb;i++) d[i] = (int) s[i];

}

/*******************************************************/
/*                    NXCONV_FD(S,D,NB)                */
/*                                                     */
/* Conversion NXFLT to NXDBLE sur meme machine         */
/* sur CRAY double precision FORTRAN = long double C   */
/*                                                     */
/*******************************************************/

void
nxconv_fd(const NXFLT* NXRESTRICT s,NXDBLE* NXRESTRICT d,int nb)
{
	register int i = 0;

	for(;i<nb;i++) d[i] = s[i];

}

/*******************************************************/
/*                    NXCONV_FLT_DBLE(S,D,NB)          */
/*                                                     */
/* Conversion NXFLT to NXDBLE sur meme machine         */
/* sur CRAY double precision FORTRAN = long double C   */
/*                                                     */
/*******************************************************/

void
nxconv_flt_dble(const NXFLT* NXRESTRICT s,NXDBLE* NXRESTRICT d,int nb)
{
	register int i = 0;

	for(;i<nb;i++) d[i] = s[i];

}

/*******************************************************/
/*                    NXCONV_DF(S,D,NB)                */
/*                                                     */
/* Conversion NXDBLE to NXFLT sur meme machine         */
/*                                                     */
/*******************************************************/

void
nxconv_df(const NXDBLE* NXRESTRICT s,NXFLT* NXRESTRICT d,int nb)
{
	register int i = 0;

	for(;i<nb;i++) d[i] = (NXFLT) s[i];

}

/*******************************************************/
/*                    NXCONV_DBLE_FLT(S,D,NB)          */
/*                                                     */
/* Conversion NXDBLE to NXFLT sur meme machine         */
/* les double sont bornes pour eviter les erreurs      */
/* sur OSF1 ou Linux par exemple                       */
/*                                                     */
/*******************************************************/

void
nxconv_dble_flt(const NXDBLE* NXRESTRICT s,NXFLT* NXRESTRICT d,int nb)
{
	register int i = 0;
	const NXDBLE Bmax = MAXFLOAT;
	register NXDBLE bmin = -Bmax;
	register NXDBLE bmax = Bmax;

	for(;i<nb;i++)
	{
	    register NXDBLE ss = s[i];

	    d[i] = (NXFLT) MIN(MAX(ss,bmin),bmax);
	}

}

/*******************************************************/
/*                    NXCONV_FD_ARRONDI(S,D,NB)        */
/*                                                     */
/* Conversion NXFLT to NXDBLE avec arrondi             */
/*                                                     */
/*******************************************************/

void 
nxconv_fd_arrondi(const NXFLT* NXRESTRICT s, NXDBLE* NXRESTRICT d, int nb)
{
#if defined(CRAY) && !defined(_CRAYMPP) && !defined(_CRAYIEEE)
	NXFLT* NXRESTRICT dd = (NXFLT*) d;
	int i;

	for(i = 0; i < nb; i++) dd[2 * i] = s[i];

#elif defined(CRAY) && !defined(_CRAYMPP) && defined(_CRAYIEEE)
	nxconv_real_ie64_to_ie128((unsigned*) s,(unsigned*) d,nb);
#elif defined(_CRAYMPP)
	memcpy(d,s,nb*sizeof(double));
#else
	int             n;
	double          s2;
	double          fact2;
	double          dix = 10.0;
	int i;
#define FACT 1.0E+7

	for(i = 0; i < nb; i++)
	{

	    if (s[i] != 0.00)
	    {
		n = (fabs(s[i]) < 1.00) ? 0 : 1;
		fact2 = FACT/(pow(dix,
				(double) ((int) (log10(fabs(s[i])) + n))));
		s2 = (s[i] - (int) s[i]) * fact2;	/* "rounding" */
		s2 = (s2 - (int) s2) / fact2;
		d[i] = s[i] - s2;
	    }
	    else d[i] = 0.00;

	}

#endif
}

/*******************************************************/
/*                    NXCONV_DF_ARRONDI(S,D,NB)        */
/*                                                     */
/* Conversion NXDBLE to NXFLT avec arrondi             */
/*                                                     */
/*******************************************************/

void 
nxconv_df_arrondi(const NXDBLE* NXRESTRICT s,NXFLT* NXRESTRICT d,int nb)
{
#if defined(CRAY) && !defined(_CRAYMPP) && !defined(_CRAYIEEE)
	const NXFLT* NXRESTRICT ss = (const NXFLT*) s;
	int i;

	for(i = 0;i < nb;i++) d[i] = ss[2*i];

#elif defined(CRAY) && !defined(_CRAYMPP) && defined(_CRAYIEEE)
	int cr = 0;

	nxconv_real_ie128_to_ie64((unsigned*) s,(unsigned*) d,nb,&cr);
#elif defined(_CRAYMPP)
	memcpy(d,s,nb*sizeof(double));
#else
	int i;

	for(i = 0; i < nb; i++) d[i] = (NXFLT) s[i];

#endif
}

/**********************************************************************/
/*                   nxconv_little_big                                */
/* Conversion of big endian ieee integer to little endian ieee integer*/
/* sur des mots de 32 ou 64 suivant la machine                        */
/**********************************************************************/

void
nxconv_little_big(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,int nb)
{

	if(sizeof(unsigned) == 8)
	{
#if defined(CRAY) && !defined(_CRAYMPP)
	/* boucles vectorises */
	/* 2 cas possibles seulement : s et d disjoints ou s = d */
	/* sinon ca donne n'importe quoi */

	    if(s == d)
	    {
		register int i = 0;

		for(;i<nb;i++)
		{
		    register unsigned ss = d[i];
		    register unsigned ss1 = ss >> 56 & 0xff;
		    register unsigned ss2 = ss >> 40 & 0xff00;
		    register unsigned ss3 = ss >> 24 & 0xff0000;
		    register unsigned ss4 = ss >> 8  & 0xff000000;
		    register unsigned ss5 = ss << 8  & 0xff00000000;
		    register unsigned ss6 = ss << 24 & 0xff0000000000;
		    register unsigned ss7 = ss << 40 & 0xff000000000000;
		    register unsigned dd = ss << 56 | ss7 | ss6 | ss5 | ss4 |
			ss3 | ss2 | ss1;

		    d[i] = dd;
		}

	    }
	    else
	    {
		register int i = 0;

		for(;i<nb;i++)
		{
		    register unsigned ss = s[i];
		    register unsigned ss1 = ss >> 56 & 0xff;
		    register unsigned ss2 = ss >> 40 & 0xff00;
		    register unsigned ss3 = ss >> 24 & 0xff0000;
		    register unsigned ss4 = ss >> 8  & 0xff000000;
		    register unsigned ss5 = ss << 8  & 0xff00000000;
		    register unsigned ss6 = ss << 24 & 0xff0000000000;
		    register unsigned ss7 = ss << 40 & 0xff000000000000;
		    register unsigned dd = ss << 56 | ss7 | ss6 | ss5 | ss4 |
			ss3 | ss2 | ss1;

		    d[i] = dd;
		}

	    }
#elif defined(_CRAYMPP)
	    register int i = 0;

	    for(;i<nb;i++)
	    {
		register unsigned ss = s[i];
		register unsigned ss1 = ss >> 56 & 0xff;
		register unsigned ss2 = ss >> 40 & 0xff00;
		register unsigned ss3 = ss >> 24 & 0xff0000;
		register unsigned ss4 = ss >> 8  & 0xff000000;
		register unsigned ss5 = ss << 8  & 0xff00000000;
		register unsigned ss6 = ss << 24 & 0xff0000000000;
		register unsigned ss7 = ss << 40 & 0xff000000000000;
		register unsigned dd = ss << 56 | ss7 | ss6 | ss5 | ss4 |
		    ss3 | ss2 | ss1;

		d[i] = dd;
	    }
#endif

	}
	else if(sizeof(unsigned) == 4)
	{
	    register int i = 0;

	    for(;i<nb;i++)
	    {
		register nx_unsigned32 stemp = s[i];
		register nx_unsigned32 ss1 = stemp >> 24 & 0xff;
		register nx_unsigned32 ss2 = stemp >> 8  & 0xff00;
		register nx_unsigned32 ss3 = stemp << 8  & 0xff0000;

		d[i]  = stemp << 24 | ss3 | ss2 | ss1;
	    }

	}

}

/**********************************************************************/
/*                   nxconv_little_big32                              */
/* Conversion of big endian ieee integer to little endian ieee integer*/
/* sur des mots de 32 bits                                            */
/**********************************************************************/

void
nxconv_little_big32(const NXCONV_UNSIGNED* NXRESTRICT s,
		NXCONV_UNSIGNED* NXRESTRICT d,int nb)
{

#if defined(CRAY) && !defined(_CRAYMPP)
    /* boucles vectorises */
    /* 2 cas possibles seulement : s et d disjoints ou s = d */
    /* sinon ca donne n'importe quoi */

	if(s == d)
	{
	    register int i = 0;
	    register int nb2 = nb/2;

	    for(;i<nb2;i++)
	    {
		register unsigned ss = d[i];
		register unsigned ss1 = ss >> 24 & 0xff00000000;
		register unsigned ss2 = ss >> 8  & 0xff0000000000;
		register unsigned ss3 = ss << 8  & 0xff000000000000;
		register unsigned ss4 = ss << 24 & 0xff00000000000000;
		register unsigned ss5 = ss >> 24 & 0xff;
		register unsigned ss6 = ss >> 8  & 0xff00;
		register unsigned ss7 = ss << 8  & 0xff0000;
		register unsigned ss8 = ss << 24 & 0xff000000;
		register unsigned dd = ss4 | ss3 | ss2 | ss1 | ss8 |
		    ss7 | ss6 | ss5;

		d[i] = dd;
	    }

	}
	else
	{
	    register int i = 0;
	    register int nb2 = nb/2;

	    for(;i<nb2;i++)
	    {
		register unsigned ss = s[i];
		register unsigned ss1 = ss >> 24 & 0xff00000000;
		register unsigned ss2 = ss >> 8  & 0xff0000000000;
		register unsigned ss3 = ss << 8  & 0xff000000000000;
		register unsigned ss4 = ss << 24 & 0xff00000000000000;
		register unsigned ss5 = ss >> 24 & 0xff;
		register unsigned ss6 = ss >> 8  & 0xff00;
		register unsigned ss7 = ss << 8  & 0xff0000;
		register unsigned ss8 = ss << 24 & 0xff000000;
		register unsigned dd = ss4 | ss3 | ss2 | ss1 | ss8 |
		    ss7 | ss6 | ss5;

		d[i] = dd;
	    }

	}

	if(nb%2)
	{
	    register int i = nb/2;
	    register unsigned ss = s[i];
	    register unsigned ss1 = ss >> 24 & 0xff00000000;
	    register unsigned ss2 = ss >> 8  & 0xff0000000000;
	    register unsigned ss3 = ss << 8  & 0xff000000000000;
	    register unsigned ss4 = ss << 24 & 0xff00000000000000;
	    register unsigned dd = ss4 | ss3 | ss2 | ss1;

	    d[i] = dd;
	}
#else
	register int i = 0;

	for(;i<nb;i++)
	{
	    register nx_unsigned32 stemp = s[i];
	    register nx_unsigned32 ss1 = stemp >> 24 & 0xff;
	    register nx_unsigned32 ss2 = stemp >> 8  & 0xff00;
	    register nx_unsigned32 ss3 = stemp << 8  & 0xff0000;

	    d[i]  = stemp << 24 | ss3 | ss2 | ss1;
	}
#endif

}

/**********************************************************************/
/*                   nxconv_little_big64                              */
/* Conversion of big endian ieee integer to little endian ieee integer*/
/* sur des mots de 64 bits                                            */
/**********************************************************************/

void
nxconv_little_big64(const NXCONV_UNSIGNED* NXRESTRICT s,
		NXCONV_UNSIGNED* NXRESTRICT d,int nb)
{

#if defined(CRAY) && !defined(_CRAYMPP)
    /* boucles vectorises */
    /* 2 cas possibles seulement : s et d disjoints ou s = d */
    /* sinon ca donne n'importe quoi */

	if(s == d)
	{
	    register int i = 0;

	    for(;i<nb;i++)
	    {
		register unsigned ss = d[i];
		register unsigned ss1 = ss >> 56 & 0xff;
		register unsigned ss2 = ss >> 40 & 0xff00;
		register unsigned ss3 = ss >> 24 & 0xff0000;
		register unsigned ss4 = ss >> 8  & 0xff000000;
		register unsigned ss5 = ss << 8  & 0xff00000000;
		register unsigned ss6 = ss << 24 & 0xff0000000000;
		register unsigned ss7 = ss << 40 & 0xff000000000000;
		register unsigned dd = ss << 56 | ss7 | ss6 | ss5 | ss4 |
		    ss3 | ss2 | ss1;

		d[i] = dd;
	    }

	}
	else
	{
	    register int i = 0;

	    for(;i<nb;i++)
	    {
		register unsigned ss = s[i];
		register unsigned ss1 = ss >> 56 & 0xff;
		register unsigned ss2 = ss >> 40 & 0xff00;
		register unsigned ss3 = ss >> 24 & 0xff0000;
		register unsigned ss4 = ss >> 8  & 0xff000000;
		register unsigned ss5 = ss << 8  & 0xff00000000;
		register unsigned ss6 = ss << 24 & 0xff0000000000;
		register unsigned ss7 = ss << 40 & 0xff000000000000;
		register unsigned dd = ss << 56 | ss7 | ss6 | ss5 | ss4 |
		    ss3 | ss2 | ss1;

		d[i] = dd;
	    }

	}
#else
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j1t = 2*i;
	    register int j2t = 2*i+1;
	    register nx_unsigned32 stemp1 = s[j1t];
	    register nx_unsigned32 stemp2 = s[j2t];
	    register nx_unsigned32 sst1 = stemp1 >> 24 & 0xff;
	    register nx_unsigned32 sst2 = stemp1 >> 8  & 0xff00;
	    register nx_unsigned32 sst3 = stemp1 << 8  & 0xff0000;
	    register nx_unsigned32 sst5 = stemp2 >> 24 & 0xff;
	    register nx_unsigned32 sst6 = stemp2 >> 8  & 0xff00;
	    register nx_unsigned32 sst7 = stemp2 << 8  & 0xff0000;

	    d[j1t] = stemp2 << 24 | sst7 | sst6 | sst5;
	    d[j2t] = stemp1 << 24 | sst3 | sst2 | sst1;
	}
#endif

}

/**********************************************************************/
/*                   nxconv_little_big128                             */
/* Conversion of big endian ieee integer to little endian ieee integer*/
/* sur des mots de 128 bits                                           */
/**********************************************************************/

void
nxconv_little_big128(const NXCONV_UNSIGNED* NXRESTRICT s,
		NXCONV_UNSIGNED* NXRESTRICT d,int nb)
{

#if defined(CRAY) && !defined(_CRAYMPP)
    /* boucles vectorises */
    /* 2 cas possibles seulement : s et d disjoints ou s = d */
    /* sinon ca donne n'importe quoi */

	if(s == d)
	{
	    register int i = 0;

	    for(;i<nb;i++)
	    {
		register int j1t = 2*i;
		register int j2t = 2*i+1;
		register unsigned stemp1 = d[j1t];
		register unsigned stemp2 = d[j2t];
		register unsigned sst1 = stemp1 >> 56 & 0xff;
		register unsigned sst2 = stemp1 >> 40 & 0xff00;
		register unsigned sst3 = stemp1 >> 24 & 0xff0000;
		register unsigned sst4 = stemp1 >> 8  & 0xff000000;
		register unsigned sst5 = stemp1 << 8  & 0xff00000000;
		register unsigned sst6 = stemp1 << 24 & 0xff0000000000;
		register unsigned sst7 = stemp1 << 40 & 0xff000000000000;
		register unsigned sst9 = stemp1 >> 56 & 0xff;
		register unsigned sst10 = stemp2 >> 40 & 0xff00;
		register unsigned sst11 = stemp2 >> 24 & 0xff0000;
		register unsigned sst12 = stemp2 >> 8  & 0xff000000;
		register unsigned sst13 = stemp2 << 8  & 0xff00000000;
		register unsigned sst14 = stemp2 << 24 & 0xff0000000000;
		register unsigned sst15 = stemp2 << 40 & 0xff000000000000;

		d[j1t] = stemp2 << 56 | sst15 | sst14 | sst13 | sst12 |
		    sst11 | sst10 | sst9;
		d[j2t] = stemp1 << 56 | sst7 | sst6 | sst5 | sst4 |
		    sst3 | sst2 | sst1;
	    }

	}
	else
	{
	    register int i = 0;

	    for(;i<nb;i++)
	    {
		register int j1t = 2*i;
		register int j2t = 2*i+1;
		register unsigned stemp1 = s[j1t];
		register unsigned stemp2 = s[j2t];
		register unsigned sst1 = stemp1 >> 56 & 0xff;
		register unsigned sst2 = stemp1 >> 40 & 0xff00;
		register unsigned sst3 = stemp1 >> 24 & 0xff0000;
		register unsigned sst4 = stemp1 >> 8  & 0xff000000;
		register unsigned sst5 = stemp1 << 8  & 0xff00000000;
		register unsigned sst6 = stemp1 << 24 & 0xff0000000000;
		register unsigned sst7 = stemp1 << 40 & 0xff000000000000;
		register unsigned sst9 = stemp1 >> 56 & 0xff;
		register unsigned sst10 = stemp2 >> 40 & 0xff00;
		register unsigned sst11 = stemp2 >> 24 & 0xff0000;
		register unsigned sst12 = stemp2 >> 8  & 0xff000000;
		register unsigned sst13 = stemp2 << 8  & 0xff00000000;
		register unsigned sst14 = stemp2 << 24 & 0xff0000000000;
		register unsigned sst15 = stemp2 << 40 & 0xff000000000000;

		d[j1t] = stemp2 << 56 | sst15 | sst14 | sst13 | sst12 |
		    sst11 | sst10 | sst9;
		d[j2t] = stemp1 << 56 | sst7 | sst6 | sst5 | sst4 |
		    sst3 | sst2 | sst1;
	    }

	}
#else
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j1t = 4*i;
	    register int j2t = 4*i+1;
	    register int j3t = 4*i+1;
	    register int j4t = 4*i+1;
	    register nx_unsigned32 stemp1 = s[j1t];
	    register nx_unsigned32 stemp2 = s[j2t];
	    register nx_unsigned32 stemp3 = s[j3t];
	    register nx_unsigned32 stemp4 = s[j4t];
	    register nx_unsigned32 sst1 = stemp1 >> 24 & 0xff;
	    register nx_unsigned32 sst2 = stemp1 >> 8  & 0xff00;
	    register nx_unsigned32 sst3 = stemp1 << 8  & 0xff0000;
	    register nx_unsigned32 sst5 = stemp2 >> 24 & 0xff;
	    register nx_unsigned32 sst6 = stemp2 >> 8  & 0xff00;
	    register nx_unsigned32 sst7 = stemp2 << 8  & 0xff0000;
	    register nx_unsigned32 sst9 = stemp3 >> 24 & 0xff;
	    register nx_unsigned32 sst10 = stemp3 >> 8  & 0xff00;
	    register nx_unsigned32 sst11 = stemp3 << 8  & 0xff0000;
	    register nx_unsigned32 sst13 = stemp4 >> 24 & 0xff;
	    register nx_unsigned32 sst14 = stemp4 >> 8  & 0xff00;
	    register nx_unsigned32 sst15 = stemp4 << 8  & 0xff0000;

	    d[j1t] = stemp4 << 24 | sst15 | sst14 | sst13;
	    d[j2t] = stemp3 << 24 | sst11 | sst10 | sst9;
	    d[j3t] = stemp2 << 24 | sst7 | sst6 | sst5;
	    d[j4t] = stemp1 << 24 | sst3 | sst2 | sst1;
	}
#endif

}
