/*                           N X - A M                                 */
/*                           ---------                                 */
/*                          C interface                                */

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.  
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/
/* source ident */
/***************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Pierre Brochard";
#endif

/***************************************************************************/
/* includes */
/***************************************************************************/

#include "nxP.h"
#include "nxerror.h"

#define NXCONV_H_IMPLEMENT
#define NXP_H_IMPLEMENT
#   include "nx.h"
#undef NXP_H_IMPLEMENT
#undef NXCONV_H_IMPLEMENT

/*******************************************************************/
/*******************************************************************/
/* --------------- Conversion functions avec big endian ---------- */
/*******************************************************************/
/*******************************************************************/

#if (!defined(CRAY) || defined(_CRAYMPP)) && defined(NX_LITTLEENDIAN)
#   define NXCONV_LITTLE_BIG32(ss) {register nx_unsigned32 stemp = ss;\
	    register nx_unsigned32 sst1 = stemp >> 24 & 0xff;\
	    register nx_unsigned32 sst2 = stemp >> 8  & 0xff00;\
	    register nx_unsigned32 sst3 = stemp << 8  & 0xff0000;\
	    ss = stemp << 24 | sst3 | sst2 | sst1;}
#else
#   define NXCONV_LITTLE_BIG32(ss)
#endif

/*******************************************************************/
/*                     nxconv_real_cr64_to_ie128                   */
/* Conversion of a CRAY 64 bits real to an IEEE 128 bits real      */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_cr64_to_ie128(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	int nxerror = 0;
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register int i2 = 4*j;
	    register nx_unsigned32 ss1 = s[i];
	    register nx_unsigned32 b47;

	    NXCONV_LITTLE_BIG32(ss1);
	    b47 = ss1 & 0x8000;

	    if (b47)
	    {
		register nx_unsigned32 sign = ss1 & 0x80000000;
		register nx_int32 expn = ss1 >> 16 & 0x7fff;

		expn -= 2; /* (expn - 16384) + 16383 - 1 */

		if (expn < 0)                       /* underflow */
		{
		    d[i2] = d[i2+1] = d[i2+2] = 0;
		    nxerror = 40;
		}
		else
		{
		    register nx_unsigned32 ss2 = s[i+1];
		    register nx_unsigned32 mant = ss1 << 1 & 0xfffe;

		    NXCONV_LITTLE_BIG32(ss2);
		    expn = expn << 16 & 0xffff0000;
		    mant |=  ss2 >> 31 & 0x1;
		    d[i2] = sign | expn | mant;
		    d[i2+1] = ss2 << 1 & 0xfffffffe;
		    d[i2+2] = ss2 >> 31 & 0x1;
		    NXCONV_LITTLE_BIG32(d[i2]);
		    NXCONV_LITTLE_BIG32(d[i2+1]);
		    NXCONV_LITTLE_BIG32(d[i2+2]);
		}

	    }
	    else d[i2] = d[i2+1] = d[i2+2] = 0; /* real = 0.00 */

	    d[i2+3] = 0;
	}

	NXERRNO = nxerror;
}
#else
nxconv_real_cr64_to_ie128(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	int nxerror = 0;
        register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = 2*i;
	    register unsigned ss = s[i];
	    register unsigned b47 = ss & 0x800000000000;

	    if (b47)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register int expn = ss >> 48 & 0x7fff;

		expn -= 2; /* (expn - 16384) + 16383 - 1 */

		if (expn < 0)                         /* underflow */
		{
		    d[j] = d[j+1] = 0;
		    nxerror = 45;
		}
		else
		{
		    register unsigned mant = ss << 1 & 0xfffffffffffe;

		    expn <<=  48;
		    d[j] = sign | expn | mant;
		    d[j+1] = ss >> 63 & 0x1;
		}

	    }
	    else d[j] = d[j+1] = 0; 
     
	}

	NXERRNO = nxerror;
}
#endif

/*******************************************************************/
/*                     nxconv_real_ie128_to_cr64                   */
/* Conversion of an IEEE 128 bits real to a CRAY 64 bits real      */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie128_to_cr64(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	int nxerror = 0;
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register int i2 = 4*j;
	    register nx_unsigned32 ss = s[i2];
	    register nx_unsigned32 b0;

	    NXCONV_LITTLE_BIG32(ss);
	    b0 = ss & 0x7fff0000;

	    if (b0)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_unsigned32 expn = ss >> 16 & 0x7fff;

		expn += 2; /* (expn - 16383) + 16384 + 1 */

		if (expn >= 32767)               /* overflow  */
		{
		/* max double CRAY CRI 64 bits */
		    d[i] = sign | 0x7fffffff; 
		    d[i+1] = 0xffffffff;
		    nxerror = 41;
		}
		else
		{
		    register nx_unsigned32 mant =
			((ss & 0xffff) | 0x10000) >> 1 & 0x7fffffff;
		    register nx_unsigned32 ss2 = s[i2+1];

		    NXCONV_LITTLE_BIG32(ss2);
		    expn = expn << 16 & 0xffff0000;
		    d[i] = sign | expn | mant;
		    d[i+1] = (ss << 31 & 0x80000000) + (ss2 >> 1 & 0x7fffffff);
		}

		NXCONV_LITTLE_BIG32(d[i]);
		NXCONV_LITTLE_BIG32(d[i+1]);
	    }
	    else
		d[i] = d[i+1] = 0;

	}

	NXERRNO = nxerror;
}
#else
nxconv_real_ie128_to_cr64(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	int nxerror = 0;
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int i2 = 2*i;
	    register unsigned ss = s[i2];
	    register unsigned b0 = ss & 0x7fff000000000000;

	    if (b0)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register unsigned expn = ss >> 48 & 0x7fff;

		expn += 2; /* (expn - 16383) + 16384 + 1 */

		if (expn >= 32767)               /* overflow  */
		{
		/* max double CRAY CRI 64 bits */
		    d[i] = sign | 0x7fffffffffffffff;
		    nxerror = 41;
		}
		else
		{
		    register const unsigned mant =
			((ss<<16)>>17) | 0x800000000000;  

		    expn <<= 48;
		    d[i] = sign | expn | mant;
		}

	    } 
	    else d[i] = 0;

	}

	NXERRNO = nxerror;
}
#endif

/*******************************************************************/
/*                     nxconv_real_ie64_to_ie128                   */
/* Conversion of an IEEE 64 bits real to an IEEE 128 bits real     */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie64_to_ie128(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register int i2 = 4*j;
	    register nx_unsigned32 ss1 = s[i];

	    NXCONV_LITTLE_BIG32(ss1);

	    if (ss1)
	    {
		register nx_unsigned32 ss2 = s[i+1];
		register nx_unsigned32 sign = ss1 & 0x80000000;
		register nx_unsigned32 expn = ss1 >> 20 & 0x7ffff;
		register nx_unsigned32 mant = ss1 >> 4 & 0xffff;

		NXCONV_LITTLE_BIG32(ss2);
		expn += 15360; /* (expn - 1023) + 16383 */
		expn = expn << 16 & 0xffff0000;
		d[i2] = sign | expn | mant;
		d[i2+1] = ss1 << 28 & 0xf0000000;
		d[i2+1] |= ss2 >> 4 & 0xfffffff;
		d[i2+2] = ss2 << 28 & 0xf0000000;
		NXCONV_LITTLE_BIG32(d[i2]);
		NXCONV_LITTLE_BIG32(d[i2+1]);
		NXCONV_LITTLE_BIG32(d[i2+2]);
	    }
	    else d[i2] = d[i2+1] = d[i2+2] = 0; /* real = 0.00 */

	    d[i2+3] = 0;
	}

}
#else
nxconv_real_ie64_to_ie128(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
        register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = 2*i;
	    register unsigned ss = s[i];

	    if (ss)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register unsigned expn = ss >> 52 & 0x7ffff;
		register unsigned mant = ss >> 4 & 0xffffffffffff;

		expn += 15360; /* (expn - 1023) + 16383 */
		expn <<=  48;
		d[j] = sign | expn | mant;
		d[j+1] = ss << 60 & 0xf000000000000000;
	    }
	    else d[j] = d[j+1] = 0; 
     
	}

}
#endif

/*******************************************************************/
/*                     nxconv_real_ie128_to_ie64                   */
/* Conversion of an IEEE 128 bits real to an IEEE 64 bits real     */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie128_to_ie64(const nx_unsigned32 s[],nx_unsigned32 d[],
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register int i2 = 4*j;
	    register nx_unsigned32 ss = s[i2];

	    NXCONV_LITTLE_BIG32(ss);

	    if (ss)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_int32 expn = ss >> 16 & 0x7fff;

		expn -= 15360 /* (expn - 16383) + 1023 */;

		if (expn <= 0)                       /* underflow */
		{
		    d[i+1] = d[i] = 0;
		    nxerror1 = 38;
		    rc = -1;
		}
		else if (expn >= 2047)                /* overflow  */
		{
		    d[i] = sign | 0x7fefffff; 
		    d[i+1] = 0xffffffff;
		    nxerror2 = 39;
		    rc = -1;
		}
		else
		{
		    register nx_unsigned32 ss2 = s[i2+1];
		    register nx_unsigned32 ss3 = s[i2+2];
		    register nx_unsigned32 mant = ss << 4 & 0xffff0;

		    NXCONV_LITTLE_BIG32(ss2);
		    NXCONV_LITTLE_BIG32(ss3);
		    expn = expn << 20 & 0xfff00000;
		    mant |= ss2 >> 28 & 0xf;
		    d[i] = sign | expn | mant;
		    d[i+1] = ss2 << 4 & 0xfffffff0;
		    d[i+1] |= ss3 >> 28 & 0xf;
		}

	    }
	    else
		d[i] = d[i+1] = 0;

	    NXCONV_LITTLE_BIG32(d[i]);
	    NXCONV_LITTLE_BIG32(d[i+1]);
	}

	*cr = rc;
	NXERRNO = nxerror1 + (nxerror2<<10);
}
#else
nxconv_real_ie128_to_ie64(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int i2 = 2*i;
	    register unsigned ss = s[i2];
	    register unsigned temp = 0;

	    if (ss)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register unsigned expn = ss >> 48 & 0x7fff;

		expn -= 15360 /* (expn - 16383) + 1023 */;

		if (expn <= 0)  	/* underflow */
		{
		    nxerror1 = 43;
		    rc = -1;
		} 
		else if (expn >= 2047)  	/* overflow  */
		{
		/* max float ieee */
		    temp = sign | 0x7fefffffffffffff; /* (right part to zero) */
		    nxerror2 = 44;
		    rc = -1;
		} 
		else 
		{
		    register unsigned ss2 = s[i2+1];
		    register unsigned mant = ss << 4 & 0xffffffffffff0;  

		    expn <<= 52;
		    mant |= ss2 >> 60 & 0xf;
		    temp = sign | expn | mant;
		}

	    } 

	    d[i] = temp;
	}

	*cr = rc;
	NXERRNO = nxerror1 + nxerror2<<10;
}
#endif

/*******************************************************************/
/*                     nxconv_real_ie32_to_ie128                   */
/* Conversion of an IEEE 32 bits real to an IEEE 128 bits real     */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie32_to_ie128(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = 4*i;
	    register nx_unsigned32 ss = s[i];

	    NXCONV_LITTLE_BIG32(ss);

	    if (ss)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_unsigned32 expn = ss >> 23 & 0xff;
		register nx_unsigned32 mant = ss >> 7 & 0xffff;

		expn += 16256; /* (expn - 127) + 16383 */
		expn = expn << 16 & 0xffff0000;
		d[j] = sign | expn | mant;
		d[j+1] = ss << 25 & 0xfe000000;
	    }
	    else
		d[j] = d[j+1] = 0;

	    d[j+2] = d[j+3] = 0;
	    NXCONV_LITTLE_BIG32(d[j]);
	    NXCONV_LITTLE_BIG32(d[j+1]);
	}

}
#else
nxconv_real_ie32_to_ie128(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = i/2;
	    register int j2 = i%2;
	    register int i2 = 2*i;
	    register unsigned ss = s[j];
	    register unsigned rs;  

	    if(j2)
	    {
		rs = ss & 0xffffffff;
	    }
	    else
	    {
		rs = ss >> 32;
	    }

	    if(rs)
	    {
		register unsigned sign = rs & 0x80000000;
		register unsigned expn = rs >> 23 & 0xff; 
		register unsigned mant = (rs & 0x7fffff) << 25;  

		expn += 16256; /* (expn - 127) + 16383 */
		expn <<= 48;
		d[i2] = sign | expn | mant;
	    }
	    else d[i2] = 0;

	    d[i2+1] = 0;
	}

}
#endif

/*******************************************************************/
/*                     nxconv_real_ie128_to_ie32                   */
/* Conversion of an IEEE 128 bits real to an IEEE 32 bits real     */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie128_to_ie32(const nx_unsigned32 s[],nx_unsigned32 d[],
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i2 = 4*j;
	    register nx_unsigned32 ss = s[i2];

	    NXCONV_LITTLE_BIG32(ss);

	    if (ss)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_int32 expn = ss >> 16 & 0x7fff;

		expn -= 16256 /* (expn - 16383) + 127 */;

		if (expn <= 0)                       /* underflow */
		{
		    d[j] = 0;
		    nxerror1 = 38;
		    rc = -1;
		}
		else if (expn >= 255)                /* overflow  */
		{
		    d[j] = sign | 0x7f7fffff; 
		    nxerror2 = 39;
		    rc = -1;
		}
		else
		{
		    register nx_unsigned32 ss2 = s[i2+1];
		    register nx_unsigned32 mant = ss << 7 & 0x7fff80;

		    NXCONV_LITTLE_BIG32(ss2);
		    expn = expn << 23 & 0xff800000;
		    mant |= ss2 >> 25 & 0x7f;
		    d[j] = sign | expn | mant;
		}

		NXCONV_LITTLE_BIG32(d[j]);
	    }
	    else
		d[j] = 0;

	}

	*cr = rc;
	NXERRNO = nxerror1 + (nxerror2<<10);
}
#else
nxconv_real_ie128_to_ie32(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int i = 0;
	register int nb2 = nb/2;
	register int nb21 = (nb+1)/2;

	for(;i<nb21;i++)
	{
	    register unsigned ss = s[4*i];

	    if (ss)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register int expn =  ss >> 48 & 0x7fff;

		expn -= 16256 /* (expn - 16383) + 127 */;

		if (expn <= 0)                         /* underflow */
		{
		    d[i] = 0;
		    nxerror1 = 45;
		    rc = -1;
		}
		else if (expn >= 255)                 /* overflow  */
		{
		/* max ieee 32 bits = Inf -1 */
		    d[i] = sign | 0x7f7fffff00000000; 
		    nxerror2 = 46;
		    rc = -1;
		}
		else
		{
		    register unsigned mant = ss << 7 & 0x7fffff00000000;

		    expn <<= 55;
		    d[i] = sign | expn | mant;
		}

	    }
	    else
		d[i] = 0; 
     
	}

	for(i=0;i<nb2;i++)
	{
	    register unsigned temp = 0;
	    register unsigned ss = s[4*i+2];

	    if (ss)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register int expn =  ss >> 48 & 0x7fff;

		expn -= 16256 /* (expn - 16383) + 127 */;

		if (expn <= 0)                         /* underflow */
		{
		    nxerror1 = 45;
		    rc = -1;
		}
		else if (expn >= 255)                 /* overflow  */
		{
		/* max ieee 32 bits = Inf -1 */
		    temp = sign | 0x7f7fffff00000000; 
		    nxerror2 = 46;
		    rc = -1;
		}
		else
		{
		    register unsigned mant = ss << 7 & 0x7fffff00000000;

		    expn <<= 55;
		    temp = sign | expn | mant;
		}

	    }

	    d[i] |= temp >> 32;
	}

	*cr = rc;
	NXERRNO = nxerror1 + nxerror2<<10;
}
#endif

/*******************************************************************/
/*                     nxconv_real_cr128_to_ie128                  */
/* Conversion of a CRAY 128 bits real to an IEEE 128 bits real     */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_cr128_to_ie128(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	int nxerror = 0;
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 4*j;
	    register nx_unsigned32 ss1 = s[i];
	    register nx_unsigned32 b47;

	    NXCONV_LITTLE_BIG32(ss1);
	    b47 = ss1 & 0x8000;

	    if (b47)
	    {
		register nx_unsigned32 sign = ss1 & 0x80000000;
		register nx_int32 expn = ss1 >> 16 & 0x7fff;

		expn -= 2; /* (expn - 16384) + 16383 - 1 */

		if (expn < 0)                       /* underflow */
		{
		    d[i] = d[i+1] = d[i+2] = d[i+3] = 0;
		    nxerror = 40;
		}
		else
		{
		    register nx_unsigned32 ss2 = s[i+1];
		    register nx_unsigned32 ss3 = s[i+2];
		    register nx_unsigned32 ss4 = s[i+3];
		    register nx_unsigned32 mant = ss1 << 1 & 0xfffe;

		    NXCONV_LITTLE_BIG32(ss2);
		    NXCONV_LITTLE_BIG32(ss3);
		    NXCONV_LITTLE_BIG32(ss4);
		    expn = expn << 16 & 0xffff0000;
		    mant |=  ss2 >> 31 & 0x1;
		    d[i] = sign | expn | mant;
		    d[i+1] = ss2 << 1 & 0xfffffffe;
		    d[i+2] = (ss3 << 17 & 0xfffe0000) | (ss4 >> 15 & 0x7fff);
		    d[i+3] = ss4 << 17 & 0xfffe0000;
		    NXCONV_LITTLE_BIG32(d[i]);
		    NXCONV_LITTLE_BIG32(d[i+1]);
		    NXCONV_LITTLE_BIG32(d[i+2]);
		    NXCONV_LITTLE_BIG32(d[i+3]);
		}

	    }
	    else d[i] = d[i+1] = d[i+2] = d[i+3] = 0; /* real = 0.00 */

	}

	NXERRNO = nxerror;
}
#else
nxconv_real_cr128_to_ie128(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	int nxerror = 0;
        register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = 2*i;
	    register unsigned ss = s[j];
	    register unsigned b47 = ss & 0x800000000000;

	    if (b47)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register int expn = ss >> 48 & 0x7fff;

		expn -= 2; /* (expn - 16384) + 16383 - 1 */

		if (expn < 0)                         /* underflow */
		{
		    d[j] = d[j+1] = 0;
		    nxerror = 45;
		}
		else
		{
		    register unsigned mant = ss << 1 & 0xfffffffffffe;
		    register unsigned ss2 = s[j+1];

		    expn <<=  48;
		    d[j] = sign | expn | mant;
		    d[j+1] = ss2 << 17 & 0xfffffffffffe0000;
		}

	    }
	    else d[j] = d[j+1] = 0; 
     
	}

	NXERRNO = nxerror;
}
#endif

/*******************************************************************/
/*                     nxconv_real_ie128_to_cr128                  */
/* Conversion of an IEEE 128 bits real to a CRAY 128 bits real     */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie128_to_cr128(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	int nxerror = 0;
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 4*j;
	    register nx_unsigned32 ss = s[i];
	    register nx_unsigned32 b0;

	    NXCONV_LITTLE_BIG32(ss);
	    b0 = ss & 0x7fff0000;

	    if (b0)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_int32 expn = ss >> 16 & 0x7fff;

		expn += 2; /* (expn - 16383) + 16384 + 1 */

		if (expn >= 32767)               /* overflow  */
		{
		/* max double CRAY CRI 128 bits */
		    d[i] = sign | 0x7fffffff; 
		    d[i+1] = d[i+3] = 0xffffffff;
		    d[i+2] = 0xffff;
		    nxerror = 41;
		}
		else
		{
		    register nx_unsigned32 mant =
			((ss & 0xffff) | 0x10000) >> 1 & 0x7fffffff;
		    register nx_unsigned32 ss2 = s[i+1];
		    register nx_unsigned32 ss3 = s[i+2];
		    register nx_unsigned32 ss4 = s[i+3];

		    expn = expn << 16 & 0xffff0000;
		    d[i] = sign | expn | mant;
		    NXCONV_LITTLE_BIG32(ss2);
		    NXCONV_LITTLE_BIG32(ss3);
		    NXCONV_LITTLE_BIG32(ss4);
		    d[i+1] = (ss << 31 & 0x80000000) + (ss2 >> 1 & 0x7fffffff);
		    d[i+2] = (ss3 >> 17 & 0x7fff) | (ss4 << 15 & 0xffff8000);
		    d[i+3] = ss4 >> 17 & 0x7fff;
		}

		NXCONV_LITTLE_BIG32(d[i]);
		NXCONV_LITTLE_BIG32(d[i+1]);
		NXCONV_LITTLE_BIG32(d[i+2]);
		NXCONV_LITTLE_BIG32(d[i+3]);
	    }
	    else
		d[i] = d[i+1] = d[i+2] = d[i+3] = 0;

	}

	NXERRNO = nxerror;
}
#else
nxconv_real_ie128_to_cr128(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	int nxerror = 0;
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int i2 = 2*i;
	    register unsigned ss = s[i2];
	    register unsigned b0 = ss & 0x7fff000000000000;

	    if (b0)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register int expn = ss >> 48 & 0x7fff;

		expn += 2; /* (expn - 16383) + 16384 + 1 */

		if (expn >= 32767)               /* overflow  */
		{
		/* max double CRAY CRI 64 bits */
		    d[i2] = sign | 0x7fffffffffffffff;
		    d[i2+1] = 0xffffffffffff;
		    nxerror = 41;
		}
		else
		{
		    register unsigned mant = ((ss<<16)>>17) | 0x800000000000;  
		    register unsigned ss2 = s[i2+1];

		    expn <<= 48;
		    d[i2] = sign | expn | mant;
		    d[i2+1] = ss2 >> 17 & 0x7fffffffffff;
		}

	    } 
	    else d[i2] = d[i2+1] = 0;

	}

	NXERRNO = nxerror;
}
#endif

/*******************************************************************/
/*******************************************************************/
/* --------------- Conversion functions avec little endian-------- */
/*******************************************************************/
/*******************************************************************/

#if defined(NXCONV_LITTLE_BIG32)
#   undef NXCONV_LITTLE_BIG32
#endif

#if !defined(CRAY) || defined(_CRAYMPP)
#   define NXCONV_GEN_LITTLE_BIG32(ss) {register nx_unsigned32 stemp = ss;\
		register nx_unsigned32 stemp1 = stemp >> 24 & 0xff;\
	        register nx_unsigned32 stemp2 = stemp >> 8  & 0xff00;\
	        register nx_unsigned32 stemp3 = stemp << 8  & 0xff0000;\
	        ss = stemp << 24 | stemp3 | stemp2 | stemp1;}
#endif

#if defined(CRAY) && !defined(_CRAYMPP)
#   define NXCONV_BIG_LITTLE_BIGL32(ss) {register unsigned stemp = ss;\
		register unsigned sst1 = stemp >> 24 & 0xff00000000;\
		register unsigned sst2 = stemp >> 8  & 0xff0000000000;\
		register unsigned sst3 = stemp << 8  & 0xff000000000000;\
		register unsigned sst4 = stemp << 24 & 0xff00000000000000;\
		ss = sst4 | sst3 | sst2 | sst1;}
#   define NXCONV_BIG_LITTLE_BIGR32(ss) {register unsigned stemp = ss;\
		register unsigned sst5 = stemp >> 24 & 0xff;\
		register unsigned sst6 = stemp >> 8  & 0xff00;\
		register unsigned sst7 = stemp << 8  & 0xff0000;\
		register unsigned sst8 = stemp << 24 & 0xff000000;\
		ss = sst8 | sst7 | sst6 | sst5;}
#   define NXCONV_BIG_LITTLE_BIG64(ss) {register unsigned stemp = ss;\
	        register unsigned stemp1 = stemp >> 56 & 0xff;\
		register unsigned stemp2 = stemp >> 40 & 0xff00;\
		register unsigned stemp3 = stemp >> 24 & 0xff0000;\
		register unsigned stemp4 = stemp >> 8  & 0xff000000;\
		register unsigned stemp5 = stemp << 8  & 0xff00000000;\
		register unsigned stemp6 = stemp << 24 & 0xff0000000000;\
		register unsigned stemp7 = stemp << 40 & 0xff000000000000;\
		ss = stemp << 56 | stemp7 | stemp6 | stemp5 | stemp4 |\
		stemp3 | stemp2 | stemp1;}
#   define NXCONV_BIG_LITTLE_BIG128(ss,dd) {\
		register unsigned stemp1 = ss;\
		register unsigned stemp2 = dd;\
		register unsigned sst1 = stemp1 >> 56 & 0xff;\
		register unsigned sst2 = stemp1 >> 40 & 0xff00;\
		register unsigned sst3 = stemp1 >> 24 & 0xff0000;\
		register unsigned sst4 = stemp1 >> 8  & 0xff000000;\
		register unsigned sst5 = stemp1 << 8  & 0xff00000000;\
		register unsigned sst6 = stemp1 << 24 & 0xff0000000000;\
		register unsigned sst7 = stemp1 << 40 & 0xff000000000000;\
		register unsigned sst9 = stemp1 >> 56 & 0xff;\
		register unsigned sst10 = stemp2 >> 40 & 0xff00;\
		register unsigned sst11 = stemp2 >> 24 & 0xff0000;\
		register unsigned sst12 = stemp2 >> 8  & 0xff000000;\
		register unsigned sst13 = stemp2 << 8  & 0xff00000000;\
		register unsigned sst14 = stemp2 << 24 & 0xff0000000000;\
		register unsigned sst15 = stemp2 << 40 & 0xff000000000000;\
		ss = stemp2 << 56 | sst15 | sst14 | sst13 | sst12 |\
		    sst11 | sst10 | sst9;\
		dd = stemp1 << 56 | sst7 | sst6 | sst5 | sst4 |\
		    sst3 | sst2 | sst1;}
#elif !defined(NX_LITTLEENDIAN)
#   define NXCONV_LITTLE_BIG32(ss)
#   define NXCONV_BIG_LITTLE_BIG32(ss) NXCONV_GEN_LITTLE_BIG32(ss)
#   define NXCONV_BIG_LITTLE_BIG64(ss,dd) {\
	        register nx_unsigned32 stemp1 = ss;\
		register nx_unsigned32 stemp2 = dd;\
		register nx_unsigned32 sst1 = stemp1 >> 24 & 0xff;\
		register nx_unsigned32 sst2 = stemp1 >> 8  & 0xff00;\
		register nx_unsigned32 sst3 = stemp1 << 8  & 0xff0000;\
		register nx_unsigned32 sst5 = stemp2 >> 24 & 0xff;\
		register nx_unsigned32 sst6 = stemp2 >> 8  & 0xff00;\
		register nx_unsigned32 sst7 = stemp2 << 8  & 0xff0000;\
		ss = stemp2 << 24 | sst7 | sst6 | sst5;\
		dd = stemp1 << 24 | sst3 | sst2 | sst1;}
#   define NXCONV_BIG_LITTLE_BIG128(ss1,ss2,ss3,ss4) {\
	    register nx_unsigned32 stemp1 = ss1;\
	    register nx_unsigned32 stemp2 = ss2;\
	    register nx_unsigned32 stemp3 = ss3;\
	    register nx_unsigned32 stemp4 = ss4;\
	    register nx_unsigned32 sst1 = stemp1 >> 24 & 0xff;\
	    register nx_unsigned32 sst2 = stemp1 >> 8  & 0xff00;\
	    register nx_unsigned32 sst3 = stemp1 << 8  & 0xff0000;\
	    register nx_unsigned32 sst5 = stemp2 >> 24 & 0xff;\
	    register nx_unsigned32 sst6 = stemp2 >> 8  & 0xff00;\
	    register nx_unsigned32 sst7 = stemp2 << 8  & 0xff0000;\
	    register nx_unsigned32 sst9 = stemp3 >> 24 & 0xff;\
	    register nx_unsigned32 sst10 = stemp3 >> 8  & 0xff00;\
	    register nx_unsigned32 sst11 = stemp3 << 8  & 0xff0000;\
	    register nx_unsigned32 sst13 = stemp4 >> 24 & 0xff;\
	    register nx_unsigned32 sst14 = stemp4 >> 8  & 0xff00;\
	    register nx_unsigned32 sst15 = stemp4 << 8  & 0xff0000;\
	    ss1 = stemp4 << 24 | sst15 | sst14 | sst13;\
	    ss2 = stemp3 << 24 | sst11 | sst10 | sst9;\
	    ss3 = stemp2 << 24 | sst7 | sst6 | sst5;\
	    ss4 = stemp1 << 24 | sst3 | sst2 | sst1;}
#else
#   define NXCONV_LITTLE_BIG32(ss) NXCONV_GEN_LITTLE_BIG32(ss)
#   define NXCONV_BIG_LITTLE_BIG32(ss)
#   define NXCONV_BIG_LITTLE_BIG64(ss,dd) {register nx_unsigned32 stemp = ss;\
		ss = dd;dd = stemp;}
#   define NXCONV_BIG_LITTLE_BIG128(ss1,ss2,ss3,ss4) {\
		register nx_unsigned32 stemp = ss1;\
		ss1=ss4;ss4=stemp;stemp=ss2;ss2=ss3;ss3=stemp;}
#endif

/*******************************************************************/
/*                     nxconv_real_li64_to_ie128                   */
/* Conversion of an IEEE 64 bits real little endian to an IEEE     */
/* 128 bits real big endian                                        */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_li64_to_ie128(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register int i2 = 4*j;
	    register nx_unsigned32 ss1 = s[i];
	    register nx_unsigned32 ss2 = s[i+1];

	    NXCONV_BIG_LITTLE_BIG64(ss1,ss2);
	    d[i2+3] = 0;

	    if (ss1)
	    {
		register nx_unsigned32 sign = ss1 & 0x80000000;
		register nx_unsigned32 expn = ss1 >> 20 & 0x7ffff;
		register nx_unsigned32 mant = ss1 >> 4 & 0xffff;

		expn += 15360; /* (expn - 1023) + 16383 */
		expn = expn << 16 & 0xffff0000;
		d[i2] = sign | expn | mant;
		d[i2+1] = ss1 << 28 & 0xf0000000;
		d[i2+1] |= ss2 >> 4 & 0xfffffff;
		d[i2+2] = ss2 << 28 & 0xf0000000;
		NXCONV_LITTLE_BIG32(d[i2]);
		NXCONV_LITTLE_BIG32(d[i2+1]);
		NXCONV_LITTLE_BIG32(d[i2+2]);
	    }
	    else d[i2] = d[i2+1] = d[i2+2] = 0; /* real = 0.00 */

	}

}
#else
nxconv_real_li64_to_ie128(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
        register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = 2*i;
	    register unsigned ss = s[i];

	    NXCONV_BIG_LITTLE_BIG64(ss);

	    if (ss)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register unsigned expn = ss >> 52 & 0x7ffff;
		register unsigned mant = ss >> 4 & 0xffffffffffff;

		expn += 15360; /* (expn - 1023) + 16383 */
		expn <<=  48;
		d[j] = sign | expn | mant;
		d[j+1] = ss << 60 & 0xf000000000000000;
	    }
	    else d[j] = d[j+1] = 0; 
     
	}

}
#endif

/*******************************************************************/
/*                     nxconv_real_ie128_to_li64                   */
/* Conversion of an IEEE 128 bits real big endian to an IEEE       */
/* 64 bits real little endian                                      */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie128_to_li64(const nx_unsigned32 s[],nx_unsigned32 d[],
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register int i2 = 4*j;
	    register nx_unsigned32 ss = s[i2];

	    NXCONV_LITTLE_BIG32(ss);

	    if (ss)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_int32 expn = ss >> 16 & 0x7fff;

		expn -= 15360 /* (expn - 16383) + 1023 */;

		if (expn <= 0)                       /* underflow */
		{
		    d[i+1] = d[i] = 0;
		    nxerror1 = 38;
		    rc = -1;
		}
		else if (expn >= 2047)                /* overflow  */
		{
		    d[i] = sign | 0x7fefffff; 
		    d[i+1] = 0xffffffff;
		    nxerror2 = 39;
		    rc = -1;
		}
		else
		{
		    register nx_unsigned32 ss2 = s[i2+1];
		    register nx_unsigned32 ss3 = s[i2+2];
		    register nx_unsigned32 mant = ss << 4 & 0xffff0;

		    NXCONV_LITTLE_BIG32(ss2);
		    NXCONV_LITTLE_BIG32(ss3);
		    expn = expn << 20 & 0xfff00000;
		    mant |= ss2 >> 28 & 0xf;
		    d[i] = sign | expn | mant;
		    d[i+1] = ss2 << 4 & 0xfffffff0;
		    d[i+1] |= ss3 >> 28 & 0xf;
		}

		NXCONV_BIG_LITTLE_BIG64(d[i],d[i+1]);
	    }
	    else
		d[i] = d[i+1] = 0;

	}

	*cr = rc;
	NXERRNO = nxerror1 + (nxerror2<<10);
}
#else
nxconv_real_ie128_to_li64(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb,int* cr)
{
	nxconv_real_ie128_to_ie64(s,d,nb,cr);
	nxconv_little_big64(d,d,nb);
}
#endif

/*******************************************************************/
/*                     nxconv_real_li32_to_ie128                   */
/* Conversion of an IEEE 32 bits real little endian to an IEEE     */
/* 128 bits real big endian                                        */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_li32_to_ie128(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = 4*i;
	    register nx_unsigned32 ss = s[i];

	    NXCONV_BIG_LITTLE_BIG32(ss);

	    if (ss)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_unsigned32 expn = ss >> 23 & 0xff;
		register nx_unsigned32 mant = ss >> 7 & 0xffff;

		expn += 16256; /* (expn - 127) + 16383 */
		expn = expn << 16 & 0xffff0000;
		d[j] = sign | expn | mant;
		d[j+1] = ss << 25 & 0xfe000000;
		NXCONV_LITTLE_BIG32(d[j]);
		NXCONV_LITTLE_BIG32(d[j+1]);
	    }
	    else
		d[j] = d[j+1] = 0;

	    d[j+2] = d[j+3] = 0;
	}

}
#else
nxconv_real_li32_to_ie128(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = i/2;
	    register int j2 = i%2;
	    register int i2 = 2*i;
	    register unsigned ss = s[j];
	    register unsigned rs;

	    if(j2)
	    {
		NXCONV_BIG_LITTLE_BIGR32(ss);
		rs = ss & 0xffffffff;
	    }
	    else
	    {
		NXCONV_BIG_LITTLE_BIGL32(ss);
		rs = ss >> 32;
	    }

	    if(rs)
	    {
		register const unsigned sign = rs & 0x80000000;
		register unsigned expn = rs >> 23 & 0xff; 
		register const unsigned mant = (rs & 0x7fffff) << 25;  

		expn += 16256; /* (expn - 127) + 16383 */
		expn <<= 48;
		d[i2] = sign | expn | mant;
	    }
	    else d[i2] = 0;

	    d[i2+1] = 0;
	}

}
#endif

/*******************************************************************/
/*                     nxconv_real_ie128_to_li32                   */
/* Conversion of an IEEE 128 bits real big endian to an IEEE       */
/* 32 bits real little endian                                      */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_ie128_to_li32(const nx_unsigned32 s[],nx_unsigned32 d[],
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i2 = 4*j;
	    register nx_unsigned32 ss = s[i2];

	    NXCONV_LITTLE_BIG32(ss);

	    if (ss)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_int32 expn = ss >> 16 & 0x7fff;

		expn -= 16256 /* (expn - 16383) + 127 */;

		if (expn <= 0)                       /* underflow */
		{
		    d[j] = 0;
		    nxerror1 = 38;
		    rc = -1;
		}
		else if (expn >= 255)                /* overflow  */
		{
		    d[j] = sign | 0x7f7fffff; 
		    nxerror2 = 39;
		    rc = -1;
		}
		else
		{
		    register nx_unsigned32 ss2 = s[i2+1];
		    register nx_unsigned32 mant = ss << 7 & 0x7fff80;

		    NXCONV_LITTLE_BIG32(ss2);
		    expn = expn << 23 & 0xff800000;
		    mant |= ss2 >> 25 & 0x7f;
		    d[j] = sign | expn | mant;
		}

		NXCONV_BIG_LITTLE_BIG32(d[j]);
	    }
	    else
		d[j] = 0;

	}

	*cr = rc;
	NXERRNO = nxerror1 + (nxerror2<<10);
}
#else
nxconv_real_ie128_to_li32(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb,int* cr)
{
	nxconv_real_ie128_to_ie32(s,d,nb,cr);
	nxconv_little_big32(d,d,nb);
}
#endif

/*******************************************************************/
/*                     nxconv_real_li128_to_li32                   */
/* Conversion of an IEEE 128 bits real little endian to an IEEE    */
/* 32 bits real little endian                                      */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_li128_to_li32(const nx_unsigned32 s[],nx_unsigned32 d[],
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i2 = 4*j;
	    register nx_unsigned32 ss = s[i2];
	    register nx_unsigned32 ss2 = s[i2+1];
	    register nx_unsigned32 ss3 = s[i2+2];
	    register nx_unsigned32 ss4 = s[i2+3];

	    NXCONV_BIG_LITTLE_BIG128(ss,ss2,ss3,ss4);

	    if (ss)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_int32 expn = ss >> 16 & 0x7fff;

		expn -= 16256 /* (expn - 16383) + 127 */;

		if (expn <= 0)                       /* underflow */
		{
		    d[j] = 0;
		    nxerror1 = 38;
		    rc = -1;
		}
		else if (expn >= 255)                /* overflow  */
		{
		    d[j] = sign | 0x7f7fffff; 
		    nxerror2 = 39;
		    rc = -1;
		}
		else
		{
		    register nx_unsigned32 mant = ss << 7 & 0x7fff80;

		    expn = expn << 23 & 0xff800000;
		    mant |= ss2 >> 25 & 0x7f;
		    d[j] = sign | expn | mant;
		}

		NXCONV_BIG_LITTLE_BIG32(d[j]);
	    }
	    else
		d[j] = 0;

	}

	*cr = rc;
	NXERRNO = nxerror1 + (nxerror2<<10);
}
#else
nxconv_real_li128_to_li32(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int i = 0;
	register int nb2 = nb/2;
	register int nb21 = (nb+1)/2;

	for(;i<nb21;i++)
	{
	    register unsigned ss = s[4*i];
	    register unsigned ss1 = s[4*i+1];

	    NXCONV_BIG_LITTLE_BIG128(ss,ss1);

	    if (ss)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register int expn =  ss >> 48 & 0x7fff;

		expn -= 16256 /* (expn - 16383) + 127 */;

		if (expn <= 0)                         /* underflow */
		{
		    d[i] = 0;
		    nxerror1 = 45;
		    rc = -1;
		}
		else if (expn >= 255)                 /* overflow  */
		{
		/* max ieee 32 bits = Inf -1 */
		    d[i] = sign | 0x7f7fffff00000000; 
		    nxerror2 = 46;
		    rc = -1;
		}
		else
		{
		    register unsigned mant = ss << 7 & 0x7fffff00000000;

		    expn <<= 55;
		    d[i] = sign | expn | mant;
		}

	    }
	    else
		d[i] = 0; 
     
	}

	for(i=0;i<nb2;i++)
	{
	    register unsigned temp = 0;
	    register unsigned ss = s[4*i+2];
	    register unsigned ss1 = s[4*i+3];

	    NXCONV_BIG_LITTLE_BIG128(ss,ss1);

	    if (ss)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register int expn =  ss >> 48 & 0x7fff;

		expn -= 16256 /* (expn - 16383) + 127 */;

		if (expn <= 0)                         /* underflow */
		{
		    nxerror1 = 45;
		    rc = -1;
		}
		else if (expn >= 255)                 /* overflow  */
		{
		/* max ieee 32 bits = Inf -1 */
		    temp = sign | 0x7f7fffff00000000; 
		    nxerror2 = 46;
		    rc = -1;
		}
		else
		{
		    register unsigned mant = ss << 7 & 0x7fffff00000000;

		    expn <<= 55;
		    temp = sign | expn | mant;
		}

	    }

	    d[i] |= temp >> 32;
	}

	nxconv_little_big32(d,d,nb);
	*cr = rc;
	NXERRNO = nxerror1 + nxerror2<<10;
}
#endif

/*******************************************************************/
/*                     nxconv_real_li128_to_li64                   */
/* Conversion of an IEEE 128 bits real big endian to an IEEE       */
/* 64 bits real little endian                                      */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_li128_to_li64(const nx_unsigned32 s[],nx_unsigned32 d[],
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register int i2 = 4*j;
	    register nx_unsigned32 ss = s[i2];
	    register nx_unsigned32 ss2 = s[i2+1];
	    register nx_unsigned32 ss3 = s[i2+2];
	    register nx_unsigned32 ss4 = s[i2+3];

	    NXCONV_BIG_LITTLE_BIG128(ss,ss2,ss3,ss4);

	    if (ss)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_int32 expn = ss >> 16 & 0x7fff;

		expn -= 15360 /* (expn - 16383) + 1023 */;

		if (expn <= 0)                       /* underflow */
		{
		    d[i+1] = d[i] = 0;
		    nxerror1 = 38;
		    rc = -1;
		}
		else if (expn >= 2047)                /* overflow  */
		{
		    d[i] = sign | 0x7fefffff; 
		    d[i+1] = 0xffffffff;
		    nxerror2 = 39;
		    rc = -1;
		}
		else
		{
		    register nx_unsigned32 mant = ss << 4 & 0xffff0;

		    expn = expn << 20 & 0xfff00000;
		    mant |= ss2 >> 28 & 0xf;
		    d[i] = sign | expn | mant;
		    d[i+1] = ss2 << 4 & 0xfffffff0;
		    d[i+1] |= ss3 >> 28 & 0xf;
		}

		NXCONV_BIG_LITTLE_BIG64(d[i],d[i+1]);
	    }
	    else
		d[i] = d[i+1] = 0;

	}

	*cr = rc;
	NXERRNO = nxerror1 + (nxerror2<<10);
}
#else
nxconv_real_li128_to_li64(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb,int* cr)
{
	int rc = 0;
	int nxerror1 = 0,nxerror2 = 0;
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int i2 = 2*i;
	    register unsigned ss = s[i2];
	    register unsigned ss1 = s[i2+1];
	    register unsigned temp = 0;

	    NXCONV_BIG_LITTLE_BIG128(ss,ss1);

	    if (ss)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register unsigned expn = ss >> 48 & 0x7fff;

		expn -= 15360 /* (expn - 16383) + 1023 */;

		if (expn <= 0)  	/* underflow */
		{
		    nxerror1 = 43;
		    rc = -1;
		} 
		else if (expn >= 2047)  	/* overflow  */
		{
		/* max float ieee */
		    temp = sign | 0x7fefffffffffffff; /* (right part to zero) */
		    nxerror2 = 44;
		    rc = -1;
		} 
		else 
		{
		    register unsigned ss2 = s[i2+1];
		    register unsigned mant = ss << 4 & 0xffffffffffff0;  

		    expn <<= 52;
		    mant |= ss2 >> 60 & 0xf;
		    temp = sign | expn | mant;
		}

	    } 

	    d[i] = temp;
	}

	nxconv_little_big64(d,d,nb);
	*cr = rc;
	NXERRNO = nxerror1 + nxerror2<<10;
}
#endif

/*******************************************************************/
/*                     nxconv_real_li32_to_li128                   */
/* Conversion of an IEEE 32 bits real little endian to an IEEE     */
/* 128 bits real big endian                                        */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_li32_to_li128(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = 4*i;
	    register nx_unsigned32 ss = s[i];

	    NXCONV_BIG_LITTLE_BIG32(ss);
	    d[j] = d[j+1] = d[j+2] = d[j+3] = 0;

	    if (ss)
	    {
		register nx_unsigned32 sign = ss & 0x80000000;
		register nx_unsigned32 expn = ss >> 23 & 0xff;
		register nx_unsigned32 mant = ss >> 7 & 0xffff;

		expn += 16256; /* (expn - 127) + 16383 */
		expn = expn << 16 & 0xffff0000;
		d[j] = sign | expn | mant;
		d[j+1] = ss << 25 & 0xfe000000;
		NXCONV_BIG_LITTLE_BIG128(d[j],d[j+1],d[j+2],d[j+3]);
	    }

	}

}
#else
nxconv_real_li32_to_li128(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
	register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = i/2;
	    register int j2 = i%2;
	    register int i2 = 2*i;
	    register unsigned ss = s[j];
	    register unsigned rs;

	    if(j2)
	    {
		NXCONV_BIG_LITTLE_BIGR32(ss);
		rs = ss & 0xffffffff;
	    }
	    else
	    {
		NXCONV_BIG_LITTLE_BIGL32(ss);
		rs = ss >> 32;
	    }

	    if(rs)
	    {
		register const unsigned sign = rs & 0x80000000;
		register unsigned expn = rs >> 23 & 0xff; 
		register const unsigned mant = (rs & 0x7fffff) << 25;  

		expn += 16256; /* (expn - 127) + 16383 */
		expn <<= 48;
		d[i2] = sign | expn | mant;
	    }
	    else d[i2] = 0;

	    d[i2+1] = 0;
	}

	nxconv_little_big128(d,d,nb);
}
#endif

/*******************************************************************/
/*                     nxconv_real_li64_to_li128                   */
/* Conversion of an IEEE 64 bits real little endian to an IEEE     */
/* 128 bits real big endian                                        */
/* sur machines ou existe un type entier non signe 32 bits en C    */
/* de type big endian ou little endian                             */
/* ou sur CRAY                                                     */
/*                                                                 */
/*******************************************************************/

void
#if !defined(CRAY) || defined(_CRAYMPP)
nxconv_real_li64_to_li128(const nx_unsigned32 s[],nx_unsigned32 d[],int nb)
{
	register int j = 0;

	for(;j<nb;j++)
	{
	    register int i = 2*j;
	    register int i2 = 4*j;
	    register nx_unsigned32 ss1 = s[i];
	    register nx_unsigned32 ss2 = s[i+1];

	    NXCONV_BIG_LITTLE_BIG64(ss1,ss2);
	    d[i2+3] = 0;

	    if (ss1)
	    {
		register nx_unsigned32 sign = ss1 & 0x80000000;
		register nx_unsigned32 expn = ss1 >> 20 & 0x7ffff;
		register nx_unsigned32 mant = ss1 >> 4 & 0xffff;

		expn += 15360; /* (expn - 1023) + 16383 */
		expn = expn << 16 & 0xffff0000;
		d[i2] = sign | expn | mant;
		d[i2+1] = ss1 << 28 & 0xf0000000;
		d[i2+1] |= ss2 >> 4 & 0xfffffff;
		d[i2+2] = ss2 << 28 & 0xf0000000;
		NXCONV_BIG_LITTLE_BIG128(d[i2],d[i2+1],d[i2+2],d[i2+3]);
	    }
	    else d[i2] = d[i2+1] = d[i2+2] = 0; /* real = 0.00 */

	}

}
#else
nxconv_real_li64_to_li128(const unsigned* NXRESTRICT s,unsigned* NXRESTRICT d,
		int nb)
{
        register int i = 0;

	for(;i<nb;i++)
	{
	    register int j = 2*i;
	    register unsigned ss = s[i];

	    NXCONV_BIG_LITTLE_BIG64(ss);

	    if (ss)
	    {
		register unsigned sign = ss & 0x8000000000000000;
		register unsigned expn = ss >> 52 & 0x7ffff;
		register unsigned mant = ss >> 4 & 0xffffffffffff;

		expn += 15360; /* (expn - 1023) + 16383 */
		expn <<=  48;
		d[j] = sign | expn | mant;
		d[j+1] = ss << 60 & 0xf000000000000000;
	    }
	    else d[j] = d[j+1] = 0; 
     
	}

	nxconv_little_big128(d,d,nb);
}
#endif
