/*                           N X - A M                                 */
/*                           ---------                                 */
/*                          C interface                                */

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.  
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/
/* source ident */
/***************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Copyr 1992 MA/AIM";
#endif

/* Modifs Pierre Brochard 1997/1998 */

/***************************************************************************/
/* includes */
/***************************************************************************/

#include "nxP.h"
#include "nxerror.h"
#include "nxfonc.h"

#define NXP_H_IMPLEMENT
#   include "nx.h"
#undef NXP_H_IMPLEMENT

#include "nxfrmt.h"

/* sources pour machines ieee 32 bits big endian */

#if defined(IEEE) && !defined(NX_LITTLEENDIAN)
#include "nxconvP.h"

/* ============================================================ */
/*  protos pour fonctions statiques                             */
/* ============================================================ */

/* sur machines IEEE 32 bits :
   int est ie32
   type est ie32 ou ie64
   float est ie32 = NXFLT
   double est ie64 = NXDBLE
   long double est ie128
   */

/* ieee - vms */

static void nx_int_ie32_from_vms32(FILE* fd,int nb,char* buff,int* rc);
static void nx_int_nblk_ie32_from_vms32(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);
static void nx_type_from_vms32_vms64(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc);
static void nx_type_nblk_from_vms32_vms64(FILE* fd,char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);
static void nx_float_ie32_from_vms32(FILE* fd,int nb,char* buff,int* rc);
static void nx_float_nblk_ie32_from_vms32(FILE* fd,char* buff,int nb,int* nblus,
		int* rc);

static void nx_int_ie32_to_vms32(FILE* fd,int nb,const char* buff,int* rc);
static void nx_int_nblk_ie32_to_vms32(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);
static void nx_type_to_vms32_vms64(FILE* fd,int nb,const char* buff,
		const char* icod,const char* ocod,int* rc);
static void nx_type_nblk_to_vms32_vms64(FILE* fd,const char* buff,int nb,
		const char* icod,const char* ocod,int* nblus,int* rc);
static void nx_float_ie32_to_vms32(FILE* fd,int nb,const char* buff,int* rc);
static void nx_float_nblk_ie32_to_vms32(FILE* fd,const char* buff,int nb,
		int* nbecrits,int* rc);

/* ------------------------------------------------------------- */
/*  init_access for big endian ieee 32 bits machines */
/*  on lit au format mpp (ieee 64 bits), cray 64 bits ou */
/*  little endian ieee 32 bits et on converti au format ieee 32 bits */
/* ------------------------------------------------------------- */

const ACCESS*
nx_init_access_ieee_vms32(void)
{

	static  ACCESS my_access =
	{ 
	    NULL,
	    nx_int_ie32_from_vms32,
	    _nx_rfc_not_implemented,
	    _nx_rfc_not_implemented,
	    nx_float_ie32_from_vms32,
	    _nx_rfc_not_implemented,
	    nx_type_from_vms32_vms64,
	    nx_int_nblk_ie32_from_vms32,
	    _nx_nblk_rfc_not_implemented,
	    _nx_nblk_rfc_not_implemented,
	    nx_float_nblk_ie32_from_vms32,
	    _nx_nblk_rfc_not_implemented,
	    nx_type_nblk_from_vms32_vms64,
	    NULL,
	    nx_int_ie32_to_vms32,
	    _nx_wfc_not_implemented,
	    _nx_wfc_not_implemented,
	    nx_float_ie32_to_vms32,
	    _nx_wfc_not_implemented,
	    nx_type_to_vms32_vms64,
	    nx_int_nblk_ie32_to_vms32,
	    _nx_nblk_wfc_not_implemented,
	    _nx_nblk_wfc_not_implemented,
	    nx_float_nblk_ie32_to_vms32,
	    _nx_nblk_wfc_not_implemented,
	    nx_type_nblk_to_vms32_vms64
	};

	return &my_access;
}

/*               Specific access functions                    */
/*               -------------------------                    */

/* ----------------------------------------------------------------------- */
/* 		            IEEE - VMS                                     */
/* ----------------------------------------------------------------------- */

/* ----------------------------------------------------------------------- */
/* 		====>      NX_INT_IE32_FROM_VMS32   <====                  */
/* ----------------------------------------------------------------------- */

static void
nx_int_ie32_from_vms32(FILE* fd,int nb,char* buff,int* rc)
{
	NX_READ(fd,buff,NX_SIZE_32,nb,"nx_int_ie32_from_vms32",rc);
	nxconv_little_big32(buff,buff,nb);
}

/* ----------------------------------------------------------------------- */
/* 		====>      NX_INT_NBLK_IE32_FROM_VMS32    <====            */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_ie32_from_vms32(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	NX_READ_NBLK(fd,buff,NX_SIZE_32,nb,nblus,
			"nx_int_nblk_ie32_from_vms32",rc);
	nxconv_little_big32(buff,buff,*nblus);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_IE32_FROM_VMS32    <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_float_ie32_from_vms32(FILE* fd,int nb,char* buff,int* rc)
{
	const size_t u = nb*NX_SIZE_FLT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_32,
			nxconv_real_vms32_to_ie32(buff2,buff,nbt,&cr);
			if(cr) fnxperror("nx_float_ie32_from_vms32"),
			"nx_float_ie32_from_vms32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_IE32_FROM_VMS32    <====            */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_ie32_from_vms32(FILE* fd,char* buff,int nb,int* nblus,int* rc)
{
	const size_t u = nb*NX_SIZE_FLT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab;
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_READBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_32,
			nxconv_real_vms32_to_ie32(buff2,buff,n,&cr);
			if(cr) fnxperror("nx_float_nblk_ie32_from_vms32"),
			nblus,"nx_float_nblk_ie32_from_vms32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_FROM_VMS32_VMS64    <====                 */
/* ----------------------------------------------------------------------- */

static void
nx_type_from_vms32_vms64(FILE* fd,int nb,char* buff,const char* icod,
		const char* ocod,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float vms - float ieee */
    /* conversion float vms - double ieee */
    /* conversion double vms - float ieee */
    /* conversion double vms - double ieee */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_from_vms32_vms64"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_32,nxconv_real_vms32_to_ie32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_READBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_32,nxconv_real_vms32_to_ie64(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_vms64_to_ie32(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_vms64_to_ie64(
							buff2,buff,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_FROM_VMS32_VMS64    <====            */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_from_vms32_vms64(FILE* fd,char* buff,int nb,const char* icod,
		const char* ocod,int* nblus,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float vms - float ieee */
    /* conversion float vms - double ieee */
    /* conversion double vms - float ieee */
    /* conversion double vms - double ieee */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_from_vms32_vms64"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_32,nxconv_real_vms32_to_ie32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_READBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_32,nxconv_real_vms32_to_ie64(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_vms64_to_ie32(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_READBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_vms64_to_ie64(
							buff2,buff,n,&cr);
					if(cr) fnxperror(NXERRMES),
					nblus,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_IE32_TO_VMS32                              */
/* ----------------------------------------------------------------------- */

static void
nx_int_ie32_to_vms32(FILE* fd,int nb,const char* buff,int* rc)
{
	const size_t u = nb*NX_SIZE_INT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_32,
			nxconv_little_big32(buff,buff2,nbt),
			"nx_int_ie32_to_vms32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_INT_NBLK_IE32_TO_VMS32 <====                   */
/* ----------------------------------------------------------------------- */

static void
nx_int_nblk_ie32_to_vms32(FILE* fd,const char* buff,int nb,int* nbecrits,int* rc)
{
	const size_t u = nb*NX_SIZE_INT;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_INT,NX_BUF_SIZE_32,
			nxconv_little_big32(buff,buff2,nbt),
			nbecrits,"nx_int_nblk_ie32_to_vms32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_IE32_TO_VMS32                            */
/* ----------------------------------------------------------------------- */

static void
nx_float_ie32_to_vms32(FILE* fd,int nb,const char* buff,int* rc)
{
	const size_t u = nb*NX_SIZE_FLT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_32,
			nxconv_real_ie32_to_vms32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_float_ie32_to_vms32"),
			"nx_float_ie32_to_vms32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_FLOAT_NBLK_IE32_TO_VMS32   <====               */
/* ----------------------------------------------------------------------- */

static void
nx_float_nblk_ie32_to_vms32(FILE* fd,const char* buff,int nb,int* nbecrits,
		int* rc)
{
	const size_t u = nb*NX_SIZE_FLT;
	int cr = 0;
	unsigned tab[HEAP_ALLOC_RED];
	char* buff2 = (char*) tab; /* incremente comme un char* = important */
	char* buff3 = NULL;

	NX_ALLOCBUF(buff2,buff3,u);
	NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,NX_BUF_SIZE_32,
			nxconv_real_ie32_to_vms32(buff,buff2,nbt,&cr);
			if(cr) fnxperror("nx_float_nblk_ie32_to_vms32"),
			nbecrits,"nx_float_nblk_ie32_to_vms32",rc);
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_TO_VMS32_VMS64   <====                    */
/* ----------------------------------------------------------------------- */

static void
nx_type_to_vms32_vms64(FILE* fd,int nb,const char* buff,const char* icod,
		const char* ocod,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float vms */
    /* conversion double ieee - float vms */
    /* conversion float ieee - double vms */
    /* conversion double ieee - double vms */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_to_vms32_vms64"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_32,nxconv_real_ie32_to_vms32(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_WRITEBUF(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_32,nxconv_real_ie64_to_vms32(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_ie32_to_vms64(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_DBLE,
					NX_BUF_SIZE_64,nxconv_real_ie64_to_vms64(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					NXERRMES,rc));
	NX_FREEBUFU(buff3);
}

/* ----------------------------------------------------------------------- */
/* 		====>    NX_TYPE_NBLK_TO_VMS32_VMS64   <====               */
/* ----------------------------------------------------------------------- */

static void
nx_type_nblk_to_vms32_vms64(FILE* fd,const char* buff,int nb,const char* icod,
		const char* ocod,int* nbecrits,int* rc)
{
	unsigned tab[HEAP_ALLOC_RED];
	int cr = 0;
	char* buff2 = (char*) tab;
	char* buff3 = NULL;
	nxconv_type nct = NXCONV_ERROR;

    /* dans l'ordre : */
    /* conversion float ieee - float vms */
    /* conversion double ieee - float vms */
    /* conversion float ieee - double vms */
    /* conversion double ieee - double vms */

#ifdef NXERRMES
#   undef NXERRMES
#endif
#define NXERRMES "nx_type_nblk_to_vms32_vms64"

	NXCONV_TYPE(icod,ocod,nct);
	NXCONV_CONVERT(nct,
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_32,nxconv_real_ie32_to_vms32(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,nb*NX_SIZE_FLT);
		        NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_32,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_32,
					nxconv_real_ie64_to_vms32(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,NX_SIZE_FLT,
					NX_BUF_SIZE_64,nxconv_real_ie32_to_vms64(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					nbecrits,NXERRMES,rc),
			NX_ALLOCBUF(buff2,buff3,2*nb*NX_SIZE_FLT);
			NX_WRITEBUF_NBLK(fd,buff2,NX_SIZE_64,nb,buff,
					NX_SIZE_DBLE,NX_BUF_SIZE_64,
					nxconv_real_ie64_to_vms64(
							buff,buff2,nbt,&cr);
					if(cr) fnxperror(NXERRMES),
					nbecrits,NXERRMES,rc));
	NX_FREEBUFU(buff3);
}
#endif /* IEEE et !BIGENDIAN */
