/*                           N X - A M                                 */
/*                           ---------                                 */
/*                          C interface                                */

/*
*************************************************************************
* 			 Limeil National Center				*
*  Division for Supercomputing, Networking and Workstation Applications	*
*************************************************************************
*  December 23, 1992
*  (C) Copyright 1992 The Board of Trustees of MA/AIM of Limeil.
*
*  Permission is granted to any individual or institution to use, copy,
*  modify, or redistribute this software and its documentation provided
*  this notice and the copyright notices are retained.  This software
*  may not be distributed for profit, either in original form or in
*  derivative works.  MA/AIM of Limeil makes no representations
*  about the suitability of this software for any purpose.  
*  MA/AIM OF LIMEIL GIVES NO WARRANTY,
*  EITHER EXPRESS OR IMPLIED, FOR THE PROGRAM AND/OR DOCUMENTATION PROVIDED,
*  INCLUDING, WITHOUT LIMITATION, WARRANTY OF MERCHANTABILITY AND WARRANTY
*  OF FITNESS FOR A PARTICULAR PURPOSE.
*/

/***************************************************************************/
/* source ident */
/***************************************************************************/

#ifndef lint
    static const char sccsid[] = "%W% %G% Copyr 1992 MA/AIM";
#endif

/***************************************************************************/
/* includes */
/***************************************************************************/

#include "nxP.h"
#include "nxerror.h"
#include "nxfonc.h"
#include "nxfrmt.h"

/* ============================================================ */
/*  protos pour fonctions statiques                             */
/* ============================================================ */

static void r_char_xdr(FILE* fd,char* buff,int nb,int* nlu,int* rc);
static void r_int_xdr(FILE* fd,int nb,char* buff,int* rc);
static void r_float_xdr(FILE* fd,int nb,char* buff,int* rc);
static void r_int_nblk_xdr(FILE* fd,char* buff,int nb,int* nlu,int* rc);
static void r_float_nblk_xdr(FILE* fd,char* buff,int nb,int* nlu,int* rc);

static void w_char_xdr(FILE* fd,const char* buff,int nb,int* nbecrit,int* rc);
static void w_int_xdr(FILE* fd,int nb,const char* buff,int* rc);
static void w_float_xdr(FILE* fd,int nb,const char* buff,int* rc);
static void w_int_nblk_xdr(FILE* fd,const char* buff,int nb,int* nec,int* rc);
static void w_float_nblk_xdr(FILE* fd,const char* buff,int nb,int* nec,int* rc);

/* ------------------------------------------------------------ */
/* XDR_NXTEXT   ---> a revoir */
/* ------------------------------------------------------------ */

bool_t    /* True is success, False is failure */
xdr_nxtexte(XDR* xdrs,struct NxTexte* p) 
{

	if(xdr_int(xdrs,&p->lg) &&
			xdr_bytes(xdrs, &p->data, (u_int *)&p->lg, MAX_L3))
	{
	    return TRUE;
	}

	return FALSE;
}

/* ------------------------------------------------------------ */
/*  init_access for xdr fonctions */
/* ------------------------------------------------------------ */

const ACCESS*
nx_init_access_xdr(void)
{

	static const ACCESS access_xdr =
	{
	    r_char_xdr,
	    r_int_xdr,
	    NULL,
	    NULL,
	    r_float_xdr,
	    NULL,
	    NULL,
	    r_int_nblk_xdr,
	    NULL,
	    NULL,
	    r_float_nblk_xdr,
	    NULL,
	    NULL,
	    w_char_xdr,
	    w_int_xdr,
	    NULL,
	    NULL,
	    w_float_xdr,
	    NULL,
	    NULL,
	    w_int_nblk_xdr,
	    NULL,
	    NULL,
	    w_float_nblk_xdr,
	    NULL,
	    NULL
	};

	return &access_xdr;
}

/*               Specific access functions                    */
/*               -------------------------                    */

/* ----------------------------------------------------------------------- */
/* 		====>    r_char_xdr    <====                               */
/* ----------------------------------------------------------------------- */

static void
r_char_xdr(FILE* fd,char* buff,int nb,int* nlu,int* rc)
{
	XDR xdrs;
	u_int size = sizeof(char);
	u_int nbu = nb;
	char** ap;
	char* bp = buff;

	ap = &bp;

	xdrstdio_create(&xdrs,fd,XDR_DECODE);

	if (!xdr_array(&xdrs,ap,&nbu,nb,size,(xdrproc_t) xdr_string))
	{
	    NXERRNO = 18;
	    *rc = -1;
	}
	else
	{
	    *nlu = nbu;
	    *rc = 0;
	}

}

/* ----------------------------------------------------------------------- */
/* 		===>    R_INT_XDR    <====	                           */
/* ----------------------------------------------------------------------- */

static void
r_int_xdr(FILE* fd,int nb,char* buff,int* rc)
{
	XDR xdrs;
	u_int size = sizeof(int);
	u_int nbu = nb;
	char** ap;
	char* bp = buff;

	ap = &bp;

	xdrstdio_create(&xdrs,fd,XDR_DECODE);

	if (!xdr_array(&xdrs, ap, &nbu, nb, size,(xdrproc_t) xdr_int))
	{
	    NXERRNO = 19;
	    *rc = -1;
	}

}

static void
r_int_nblk_xdr(FILE* fd,char* buff,int nb,int* nlu,int* rc)
{
	XDR xdrs;
	u_int size = sizeof(int);
	u_int nbu = nb;
	char** ap;
	char* bp = buff;

	ap = &bp;

	xdrstdio_create(&xdrs,fd,XDR_DECODE);

	if (!xdr_array(&xdrs,ap,&nbu,nb,size,(xdrproc_t) xdr_int))
	{
	    NXERRNO = 19;
	    *rc = -1;
	}
	else
	{
	    *nlu = nbu;
	    *rc = 0;
	}

}

/* ----------------------------------------------------------------------- */
/* 		====>    R_FLOAT_XDR    <====                              */
/* ----------------------------------------------------------------------- */

static void
r_float_xdr(FILE* fd,int nb,char* buff,int* rc)
{
	XDR xdrs;
	u_int size = sizeof(float);
	u_int nbu = nb;
	char** ap;
	char* bp = buff;

	ap = &bp;

	xdrstdio_create(&xdrs,fd,XDR_DECODE);

	if (!xdr_array(&xdrs, ap, &nbu, nb, size,(xdrproc_t) xdr_float))
	{
	    NXERRNO = 20;
	    *rc = -1;
	}

}

static void
r_float_nblk_xdr(FILE* fd,char* buff,int nb,int* nlu,int* rc)
{
	XDR xdrs;
	u_int size = sizeof(float);
	u_int nbu = nb;
	char** ap;
	char* bp = buff;

	ap = &bp;

	xdrstdio_create(&xdrs,fd,XDR_DECODE);

	if (!xdr_array(&xdrs, ap, &nbu, nb, size,(xdrproc_t) xdr_float))
	{
	    NXERRNO = 20;
	    *rc = -1;
	}
	else
	{
	    *nlu = nbu;
	    *rc = 0;
	}

}

/* ----------------------------------------------------------------------- */
/* 		====>    W_CHAR_XDR    <====                               */
/* ----------------------------------------------------------------------- */

static void
w_char_xdr(FILE* fd,const char* buff,int nb,int* nbecrit,int* rc)
{
	XDR xdrs;
	u_int size = sizeof(char);
	u_int nbu = nb;
	char** ap;
	const char* bp = buff;

	ap = (char**) &bp;

	xdrstdio_create(&xdrs, fd, XDR_ENCODE);
	if (!xdr_array(&xdrs, ap, &nbu, nb, size,(xdrproc_t) xdr_string))
	{
	    NXERRNO = 21;
	    *rc = -1;
	}
	else
	{
	    *nbecrit = nbu;
	    *rc = 0;
	}
}

/* ----------------------------------------------------------------------- */
/* 		====>    W_INT_XDR    <====                                */
/* ----------------------------------------------------------------------- */

static void
w_int_xdr(FILE* fd,int nb,const char* buff,int* rc)
{
	XDR xdrs;
	u_int size = sizeof(int);
	u_int nbu = nb;
	char **ap;
	const char* bp = buff;

	ap = (char**) &bp ;

	xdrstdio_create(&xdrs, fd, XDR_ENCODE);

	if (!xdr_array(&xdrs,ap, &nbu, nb, size,(xdrproc_t) xdr_int))
	{
	    NXERRNO = 22;
	    *rc = -1;
	}

}

static void
w_int_nblk_xdr(FILE* fd,const char* buff,int nb,int* nec,int* rc)
{
	XDR xdrs;
	u_int size = sizeof(int);
	u_int nbu = nb;
	char **ap;
	const char* bp= buff ;

	ap = (char**) &bp ;

	xdrstdio_create(&xdrs, fd, XDR_ENCODE);

	if (!xdr_array(&xdrs,ap, &nbu, nb, size,(xdrproc_t) xdr_int))
	{
	    NXERRNO = 22;
	    *rc = -1;
	}
	else
	{
	    *nec = nbu;
	    *rc = 0;
	}

}

/* ----------------------------------------------------------------------- */
/* 		       ====>    W_FLOAT_XDR    <====                       */
/* ----------------------------------------------------------------------- */

static void
w_float_xdr(FILE* fd,int nb,const char* buff,int* rc)
{
	XDR xdrs;
	u_int size = sizeof(float);
	u_int nbu = nb;
	char **ap;
	const char* bp = buff;

	ap = (char**) &bp;

	xdrstdio_create(&xdrs,fd,XDR_ENCODE);

	if (!xdr_array(&xdrs, ap, &nbu, nb, size,(xdrproc_t) xdr_float))
	{
	    NXERRNO = 23;
	    *rc = -1;
	}

}

static void
w_float_nblk_xdr(FILE* fd,const char* buff,int nb,int* nec,int* rc)
{
	XDR xdrs;
	u_int size = sizeof(float);
	u_int nbu = nb;
	char **ap;
	const char* bp = buff;

	ap = (char**) &bp;

	xdrstdio_create(&xdrs,fd,XDR_ENCODE);

	if (!xdr_array(&xdrs, ap, &nbu, nb, size,(xdrproc_t) xdr_float))
	{
	    NXERRNO = 23;
	    *rc = -1;
	}
	else
	{
	    *nec = nbu;
	    *rc = 0;
	}

}
