#include "narcisse.h"
#include <stdlib.h>
#include <stdio.h>

/* origine : Univ. Evry P. Nedellec pour License de Physique */

typedef struct position
{
	double x,y;
} position;

static short giDum = -1;

double rand0(short* idum);
void moveSquare(double pas,double* x,double*y);
void moveTriangle(double pas,double* x,double*y);

int
main(void)
{
#define NBRPAS 400
#define NBRBEES 200
	unsigned long seed;
	double pas = 10.0;
	double xmin,ymin,xmax,ymax;
	position* pos;
	int nbrPas = NBRPAS,nbrBees = NBRBEES,n = 0,nb;

	SPXFILE* spx_file = NULL;
	SpxArg spx_arg[100];

	rand0(&giDum);

	if((pos = (position*) calloc((long) nbrBees,sizeof(position))) == NULL)
	{
	    fprintf(stderr,"Pb alloc\n");
	    return 1;
	}

	ymax = xmax = 0.2*pas*nbrPas;
	ymin = xmin = -xmax;

	SpxInitArg(spx_arg,100);

	SpxSetArgInt(spx_arg+n,"output",0);n++;
	SpxSetArgInt(spx_arg+n,"calcul_socket",-1);n++;
	SpxSetArgInt(spx_arg+n,"calcul_ihm",0);n++;

	SpxSetArgReel(spx_arg+n,"axe_x_min",xmin);n++;
	SpxSetArgReel(spx_arg+n,"axe_x_max",xmax);n++;
	SpxSetArgReel(spx_arg+n,"axe_y_min",ymin);n++;
	SpxSetArgReel(spx_arg+n,"axe_y_max",ymax);n++;

	SpxSetArgReel(spx_arg+n,"boite_cur_x_min",0.);n++;
	SpxSetArgReel(spx_arg+n,"boite_cur_x_max",1.);n++;
	SpxSetArgReel(spx_arg+n,"boite_cur_y_min",0.);n++;
	SpxSetArgReel(spx_arg+n,"boite_cur_y_max",1.);n++;

	spx_file = SpxOpen(" ");

	SpxSetValues(spx_file,spx_arg,n);
	SpxPurgeArg(spx_arg,n);
	SpxBase2DDeleteAll(spx_file);

    /*for(n=0;n<nbrPas;n++)
	{

	    for(nb=0;nb<nbrBees;nb++)
	    {
		moveTriangle(pas,&pos[nb].x,&pos[nb].y);
		SpxBase2DSegment(pos[nb].x,pos[nb].y,pos[nb].x,pos[nb].y,
				1,spx_file);
	    }

	}*/

	{
	    float x[NBRBEES];
	    float y[NBRBEES];
	    double xcol1 = 53./(nbrPas-1.);
	    double xcol2 = 10.;

	    for(n=0;n<nbrPas;n++)
	    {
		int col = n*xcol1+xcol2;

		for(nb=0;nb<nbrBees;nb++)
		{
		    moveTriangle(pas,&pos[nb].x,&pos[nb].y);
		    x[nb] = pos[nb].x;
		    y[nb] = pos[nb].y;
		}

	    /*SpxBase2DTabSegment(x,y,x,y,nbrBees,1,spx_file);*/
		SpxBase2DTabSegment(x,y,x,y,nbrBees,col,spx_file);
	    }

	}

	SpxVaSetValues(spx_file,"calcul_ihm","1",NULL);
	SpxClose(spx_file);

	free((char*) pos);

	return 0;
}

/*
  rand0 renvoit un nombre aleatoire dans [0.1[
  */

double
rand0(short* idum)
{
	static double gY,gV[98];
	short j;
	double dum,maxRan = RAND_MAX+1.0;

	if(*idum<0)
	{
	    srand(*idum>=0 ? *idum : -*idum);
	    *idum = 1;

	    for(j=0;j<97;j++) dum = rand();

	    for(j=0;j<97;j++) gV[j] = rand();

	    gY = rand();
	}

	j = 97.0*gY/maxRan;
	gY = gV[j];
	gV[j] = rand();

	return gY/maxRan;
}

#define NORD 0
#define EST 1
#define SUD 2
#define OUEST 3

void
moveSquare(double pas,double* x,double* y)
{
	short ran = 4.*rand0(&giDum);

	switch(ran)
	{
	  case NORD:
	    *y -= pas;
	    break;
	  case EST:
	    *x += pas;
	    break;
	  case SUD:
	    *y += pas;
	    break;
	  case OUEST:
	    *x -= pas;
	    break;
	}

}

#undef NORD
#undef EST
#undef SUD
#undef OUEST

#define NORDEST 0
#define EST 1
#define SUDEST 2
#define SUDOUEST 3
#define OUEST 4
#define NORDOUEST 5

void
moveTriangle(double pas,double* x,double* y)
{
	short ran = 6.*rand0(&giDum);

	switch(ran)
	{
	  case NORDEST:
	    *x += 0.5*pas;
	    *y -= 0.866025404*pas;
	    break;
	  case EST:
	    *x += pas;
	    break;
	  case SUDEST:
	    *x += 0.5*pas;
	    *y += 0.866025404*pas;
	    break;
	  case SUDOUEST:
	    *x -= 0.5*pas;
	    *y += 0.866025404*pas;
	    break;
	  case OUEST:
	    *x -= pas;
	    break;
	  case NORDOUEST:
	    *x -= 0.5*pas;
	    *y -= 0.866025404*pas;
	    break;
	}

}

#undef NORDEST
#undef EST
#undef SUDEST
#undef SUDOUEST
#undef OUEST
#undef NORDOUEST
