#ifndef X11
#   define X11
#endif

#include <stdio.h>
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include "X11/cursorfont.h"
#include <Xm/Label.h>
#include "SpxCallback.h"

static void
MyEnterCB(Widget w,XtPointer clientdata,XtPointer calldata)
{
	printf("MyEnterCB (Widget : %x, Client data : %s, Call data %x)\n",
			w,clientdata,calldata);
}

static void
MyLeaveCB(Widget w,XtPointer clientdata,XtPointer calldata)
{
	printf("MyLeaveCB (Widget : %x, Client data : %s, Call data %x)\n",
			w,clientdata,calldata);
}

static void
MyBpressCB(Widget w,XtPointer clientdata,XtPointer calldata)
{
	SpxButtonCallbackStruct* const sbcs =
	    (SpxButtonCallbackStruct* const) calldata;

	printf("MyBpressCB (Widget : %x, Client data : %s, Call data %x)\n",
			w,clientdata,calldata);
	printf("x = %d, y = %d, state %d, button %d\n",sbcs->x,sbcs->y,
			sbcs->state,sbcs->button);

	if(sbcs->x < 30 && sbcs->y < 30)
	{
	    SPXFILE* spxfile = SpcOpen(" ");

	    SpxRemoveCallback(w,SpxNbpressAffCallback,
			    (XtCallbackProc) MyBpressCB,
			    (XtPointer) clientdata,spxfile);
	    SpxClose(spxfile);
	}

}

static void
MyBpressbaseCB(Widget w,XtPointer clientdata,XtPointer calldata)
{
	SpxButtonBaseCallbackStruct* const sbcs =
	    (SpxButtonBaseCallbackStruct* const) calldata;

	printf("MyBpressbaseCB (Widget : %x, Client data : %s, Call data %x)\n",
			w,clientdata,calldata);
	printf("x = %f, y = %f, state %d, button %d\n",sbcs->x,sbcs->y,
			sbcs->state,sbcs->button);
}

static void
MyBreleaseCB(Widget w,XtPointer clientdata,XtPointer calldata)
{
	SpxButtonCallbackStruct* const sbcs =
	    (SpxButtonCallbackStruct* const) calldata;

	printf("MyBreleaseCB (Widget : %x, Client data : %s, Call data %x)\n",
			w,clientdata,calldata);
	printf("x = %d, y = %d, state %d, button %d\n",sbcs->x,sbcs->y,
			sbcs->state,sbcs->button);

	if(sbcs->x < 30 && sbcs->y < 30)
	{
	    SPXFILE* spxfile = SpcOpen(" ");

	    SpxSortie(spxfile);
	}
	else
	{
	    SpxRemovePosAfficheur(1);
	}

}

static void
MyBreleasebaseCB(Widget w,XtPointer clientdata,XtPointer calldata)
{
	SpxButtonBaseCallbackStruct* const sbcs =
	    (SpxButtonBaseCallbackStruct* const) calldata;

	printf("MyBreleasebaseCB (Widget : %x, Client data : %s, "
			"Call data %x)\n",w,clientdata,calldata);
	printf("x = %f, y = %f, state %d, button %d\n",sbcs->x,sbcs->y,
			sbcs->state,sbcs->button);
}

static void
MyMnotifyCB(Widget w,XtPointer clientdata,XtPointer calldata)
{
	SpxMotionCallbackStruct* const sbcs =
	    (SpxMotionCallbackStruct* const) calldata;

	printf("MyMnotifyCB (Widget : %x, Client data : %s, Call data %x)\n",
			w,clientdata,calldata);
	printf("x = %d, y = %d, state %d\n",sbcs->x,sbcs->y,sbcs->state);

	if(sbcs->x > 256 && sbcs->y > 256)
	{
	    SPXFILE* spxfile = SpcOpen(" ");

	    SpxUnmapAfficheur(spxfile);
	    SpxClose(spxfile);
	}

}

static void
MyMnotifybaseCB(Widget w,XtPointer clientdata,XtPointer calldata)
{
	SpxMotionBaseCallbackStruct* const sbcs =
	    (SpxMotionBaseCallbackStruct* const) calldata;

	printf("MyMnotifybaseCB (Widget : %x, Client data : %s, Call data %x)\n",
			w,clientdata,calldata);
	printf("x = %f, y = %f, state %d\n",sbcs->x,sbcs->y,sbcs->state);
}

int
main(int argc,char* argv[])
{
	XtAppContext appc;
	Widget toplevel;
	SPXFILE* spxfile = NULL;
	unsigned cur = 0;

	toplevel = XtVaAppInitialize(&appc,
			"app",
			NULL, 0,
			&argc, argv,
			NULL, NULL);

	XtVaCreateManagedWidget( "Hello you",
                        xmLabelWidgetClass, toplevel,NULL);

	spxfile = SpcOpen(" ");

	SpxAddCallback(toplevel,SpxNenterAffCallback,
			(XtCallbackProc) MyEnterCB,
			(XtPointer) "essai1",spxfile);

	SpxAddCallback(toplevel,SpxNleaveAffCallback,
			(XtCallbackProc) MyLeaveCB,
			(XtPointer) "essai2",spxfile);

	SpxAddCallback(toplevel,SpxNbpressAffCallback,
			(XtCallbackProc) MyBpressCB,
			(XtPointer) "essai3",spxfile);
	SpxAddCallback(toplevel,SpxNbpressbaseAffCallback,
			(XtCallbackProc) MyBpressbaseCB,
			(XtPointer) "essai4",spxfile);

	SpxAddCallback(toplevel,SpxNbreleaseAffCallback,
			(XtCallbackProc) MyBreleaseCB,
			(XtPointer) "essai5",spxfile);
	SpxAddCallback(toplevel,SpxNbreleasebaseAffCallback,
			(XtCallbackProc) MyBreleasebaseCB,
			(XtPointer) "essai6",spxfile);

	SpxAddCallback(toplevel,SpxNmnotifyAffCallback,
			(XtCallbackProc) MyMnotifyCB,
			(XtPointer) "essai7",spxfile);
	SpxAddCallback(toplevel,SpxNmnotifybaseAffCallback,
			(XtCallbackProc) MyMnotifybaseCB,
			(XtPointer) "essai8",spxfile);

	cur = SpxSetCurseur(XC_arrow,spxfile);
	SpxAddPosAfficheur(toplevel,100,200,spxfile);
	SpxMapAfficheur(spxfile);
	SpxAddMapAfficheur(toplevel,spxfile);

	SpxClose(spxfile);

	XtRealizeWidget(toplevel);
	XtVaSetValues(toplevel,XmNwidth,800,XmNheight,800,NULL);
	XtAppMainLoop(appc);

	return 0;
}
