#ifndef SPXINCLUDE_H
#define SPXINCLUDE_H

#ifdef __cplusplus
extern "C" {
#endif

/* types REEL_S et REEL identiques a NXFLT et NXDBLE de nx
   REEL8 est 64 bits s'il existe sinon la taille la plus proche
   REEL4 est 32 bits s'il existe sinon la taille la plus proche
   taille la plus proche est pris au sens min(abs(taille-taille_existante))
   si 2 solutions existent alors on prend la plus grande taille */

#if defined(CRAY) && !defined(_CRAYMPP)
    typedef float REEL_S; /* 64 bits */
    typedef long double REEL; /* 128 bits */
    typedef float REEL4; /* 64 bits */
    typedef double REEL8; /* 64 bits */
#elif defined(_CRAYMPP)
    typedef double REEL_S; /* 64 bits */
    typedef long double REEL; /* 64 bits */
    typedef float REEL4; /* 32 bits */
    typedef double REEL8; /* 64 bits */
#else
    typedef float REEL_S; /* 32 bits */
    typedef double REEL; /* 64 bits */
    typedef float REEL4; /* 32 bits */
    typedef double REEL8; /* 64 bits */
#endif

    typedef struct _SPXFILE SPXFILE;

    SPXFILE* SpxOpen(char* nom_fichier);
    SPXFILE* Sp3Open(char* nom_fichier);
    SPXFILE* SpcOpen(char* nom_fichier);
    SPXFILE* SpetoileOpen(char* nom_fichier);
    SPXFILE* SpcolOpen(char* nom_fichier);
    SPXFILE* SpuOpen(char* nom_fichier);

    void SpxClose(SPXFILE* spx_file);
    void SpxSync(SPXFILE* spx_file);

    int SpxQuery(const char* nom_fichier);
    int SpxWhich(const char* nom_fichier);

    int SpxLance(const char* nomexe,const char* nom_fichier,
		    const char* machine,const char* port_name,char port[]);
    SPXFILE* SpcLOpen(const char* nomexe,const char* nom_fichier,
		    const char* machine,const char* port_name);

    void SpxMonoCourbe(REEL_S x[],REEL_S y[],int nx,SPXFILE* spx_file);
    void SpxDMonoCourbe(REEL x[],REEL y[],int nx,SPXFILE* spx_file);
    void Spx4MonoCourbe(REEL4 x[],REEL4 y[],int nx,SPXFILE* spx_file);
    void Spx8MonoCourbe(REEL8 x[],REEL8 y[],int nx,SPXFILE* spx_file);

    void SpxMulCourbe(REEL_S x[],REEL_S y[],int nx[],int ny,SPXFILE* spx_file);
    void SpxMulYMonoX(REEL_S x[],REEL_S y[],int nx,int ny,SPXFILE* spx_file);
    void SpxMulTYMonoX(REEL_S x[],REEL_S y[],int nx,int ny,SPXFILE* spx_file);
    void SpxMulXMonoY(REEL_S x[],REEL_S y[],int nx,int ny,SPXFILE* spx_file);

    void SpxDMulCourbe(REEL x[],REEL y[],int nx[],int ny,SPXFILE* spx_file);
    void SpxDMulYMonoX(REEL x[],REEL y[],int nx,int ny,SPXFILE* spx_file);
    void SpxDMulTYMonoX(REEL x[],REEL y[],int nx,int ny,SPXFILE* spx_file);
    void SpxDMulXMonoY(REEL x[],REEL y[],int nx,int ny,SPXFILE* spx_file);

    void Spx4MulCourbe(REEL4 x[],REEL4 y[],int nx[],int ny,SPXFILE* spx_file);
    void Spx4MulYMonoX(REEL4 x[],REEL4 y[],int nx,int ny,SPXFILE* spx_file);
    void Spx4MulTYMonoX(REEL4 x[],REEL4 y[],int nx,int ny,SPXFILE* spx_file);
    void Spx4MulXMonoY(REEL4 x[],REEL4 y[],int nx,int ny,SPXFILE* spx_file);

    void Spx8MulCourbe(REEL8 x[],REEL8 y[],int nx[],int ny,SPXFILE* spx_file);
    void Spx8MulYMonoX(REEL8 x[],REEL8 y[],int nx,int ny,SPXFILE* spx_file);
    void Spx8MulTYMonoX(REEL8 x[],REEL8 y[],int nx,int ny,SPXFILE* spx_file);
    void Spx8MulXMonoY(REEL8 x[],REEL8 y[],int nx,int ny,SPXFILE* spx_file);

    void SpxNonStruc4d(int npoints,const REEL_S x[],const REEL_S y[],
		    const REEL_S z[],const REEL_S c[],int nmailles,int dimmailles,
		    const int mailles[],SPXFILE* spx_file);
    void SpxMailleNonStruc4d(int npoints,const REEL_S x[],const REEL_S y[],
		    const REEL_S z[],const REEL_S c[],int nmailles,int dimmailles,
		    const int mailles[],SPXFILE* spx_file);
    void SpxBrasNonStruc4d(int npoints,const REEL_S x[],const REEL_S y[],
		    const REEL_S z[],const REEL_S c[],int nmailles,int dimmailles,
		    const int mailles[],int nbras,const int bras[],
		    SPXFILE* spx_file);

    void SpxDNonStruc4d(int npoints,const REEL x[],const REEL y[],const REEL z[],
		    const REEL c[],int nmailles,int dimmailles,
		    const int mailles[],SPXFILE* spx_file);
    void SpxDMailleNonStruc4d(int npoints,const REEL x[],const REEL y[],
		    const REEL z[],const REEL c[],int nmailles,int dimmailles,
		    const int mailles[],SPXFILE* spx_file);
    void SpxDBrasNonStruc4d(int npoints,const REEL x[],const REEL y[],
		    const REEL z[],const REEL c[],int nmailles,int dimmailles,
		    const int mailles[],int nbras,const int bras[],
		    SPXFILE* spx_file);

    void Spx4NonStruc4d(int npoints,const REEL4 x[],const REEL4 y[],
		    const REEL4 z[],const REEL4 c[],int nmailles,int dimmailles,
		    const int mailles[],SPXFILE* spx_file);
    void Spx4MailleNonStruc4d(int npoints,const REEL4 x[],const REEL4 y[],
		    const REEL4 z[],const REEL4 c[],int nmailles,int dimmailles,
		    const int mailles[],SPXFILE* spx_file);
    void Spx4BrasNonStruc4d(int npoints,const REEL4 x[],const REEL4 y[],
		    const REEL4 z[],const REEL4 c[],int nmailles,int dimmailles,
		    const int mailles[],int nbras,const int bras[],
		    SPXFILE* spx_file);

    void Spx8NonStruc4d(int npoints,const REEL8 x[],const REEL8 y[],
		    const REEL8 z[],const REEL8 c[],int nmailles,int dimmailles,
		    const int mailles[],SPXFILE* spx_file);
    void Spx8MailleNonStruc4d(int npoints,const REEL8 x[],const REEL8 y[],
		    const REEL8 z[],const REEL8 c[],int nmailles,int dimmailles,
		    const int mailles[],SPXFILE* spx_file);
    void Spx8BrasNonStruc4d(int npoints,const REEL8 x[],const REEL8 y[],
		    const REEL8 z[],const REEL8 c[],int nmailles,int dimmailles,
		    const int mailles[],int nbras,const int bras[],
		    SPXFILE* spx_file);

/* cas particulier : points 4d petit cercle */

    void SpxPointsEpais4d(int npoints,const REEL_S x[],const REEL_S y[],
		    const REEL_S z[],const REEL_S c[],SPXFILE* spx_file);

/* cas particulier : points 4d pixel */

    void SpxPointsPixel4d(int npoints,const REEL_S x[],const REEL_S y[],
		    const REEL_S z[],const REEL_S c[],SPXFILE* spx_file);

/************************************************************************/
/* maillage 3d structure parallelepipedique rectangle x(i), y(j), z(k), */
/*  c(i,j,k)                                                            */
/************************************************************************/

    void SpxParRect4d(int nx,int ny,int nz,REEL_S x[],REEL_S y[],REEL_S z[],
		    REEL_S c[],int dimc1,int dimc2,SPXFILE* spx_file);
    void SpxDParRect4d(int nx,int ny,int nz,REEL x[],REEL y[],REEL z[],
		    REEL c[],int dimc1,int dimc2,SPXFILE* spx_file);
    void Spx4ParRect4d(int nx,int ny,int nz,REEL4 x[],REEL4 y[],REEL4 z[],
		    REEL4 c[],int dimc1,int dimc2,SPXFILE* spx_file);
    void Spx8ParRect4d(int nx,int ny,int nz,REEL8 x[],REEL8 y[],REEL8 z[],
		    REEL8 c[],int dimc1,int dimc2,SPXFILE* spx_file);

    void SpxPrintSurf(int nx,int ny,REEL_S z[],int dimz,SPXFILE* spx_file);
    void SpxDPrintSurf(int nx,int ny,REEL z[],int dimz,SPXFILE* spx_file);
    void Spx4PrintSurf(int nx,int ny,REEL4 z[],int dimz,SPXFILE* spx_file);
    void Spx8PrintSurf(int nx,int ny,REEL8 z[],int dimz,SPXFILE* spx_file);

    void SpxRect3d(int nx,int ny,REEL_S x[],REEL_S y[],REEL_S z[],int dimz,
		    SPXFILE* spx_file);
    void SpxMailleRect3d(int nx,int ny,REEL_S x[],REEL_S y[],REEL_S c[],int dimc,
		    SPXFILE* spx_file);

    void SpxDRect3d(int nx,int ny,REEL x[],REEL y[],REEL z[],int dimz,
		    SPXFILE* spx_file);
    void SpxDMailleRect3d(int nx,int ny,REEL x[],REEL y[],REEL c[],int dimc,
		    SPXFILE* spx_file);

    void Spx4Rect3d(int nx,int ny,REEL4 x[],REEL4 y[],REEL4 z[],int dimz,
		    SPXFILE* spx_file);
    void Spx4MailleRect3d(int nx,int ny,REEL4 x[],REEL4 y[],REEL4 c[],int dimc,
		    SPXFILE* spx_file);

    void Spx8Rect3d(int nx,int ny,REEL8 x[],REEL8 y[],REEL8 z[],int dimz,
		    SPXFILE* spx_file);
    void Spx8MailleRect3d(int nx,int ny,REEL8 x[],REEL8 y[],REEL8 c[],int dimc,
		    SPXFILE* spx_file);

    void SpxRect4d(int nx,int ny,REEL_S x[],REEL_S y[],REEL_S z[],int dimz,
		    REEL_S c[],int dimc,SPXFILE* spx_file);
    void SpxMailleRect4d(int nx,int ny,REEL_S x[],REEL_S y[],REEL_S z[],int dimz,
		    REEL_S c[],int dimc,SPXFILE* spx_file);

    void SpxDRect4d(int nx,int ny,REEL x[],REEL y[],REEL z[],int dimz,
		    REEL c[],int dimc,SPXFILE* spx_file);
    void SpxDMailleRect4d(int nx,int ny,REEL x[],REEL y[],REEL z[],int dimz,
		    REEL c[],int dimc,SPXFILE* spx_file);

    void Spx4Rect4d(int nx,int ny,REEL4 x[],REEL4 y[],REEL4 z[],int dimz,
		    REEL4 c[],int dimc,SPXFILE* spx_file);
    void Spx4MailleRect4d(int nx,int ny,REEL4 x[],REEL4 y[],REEL4 z[],int dimz,
		    REEL4 c[],int dimc,SPXFILE* spx_file);

    void Spx8Rect4d(int nx,int ny,REEL8 x[],REEL8 y[],REEL8 z[],int dimz,
		    REEL8 c[],int dimc,SPXFILE* spx_file);
    void Spx8MailleRect4d(int nx,int ny,REEL8 x[],REEL8 y[],REEL8 z[],int dimz,
		    REEL8 c[],int dimc,SPXFILE* spx_file);

    typedef struct
    {
	    unsigned type;
	    char* name;
    } SpxArgTemplate;

    typedef struct
    {
	    unsigned type;
	    char* name;
	    int value;
    } SpxArgInt;

    typedef struct
    {
	    unsigned type;
	    char* name;
	    REEL_S value;
    } SpxArgReel;

    typedef struct
    {
	    unsigned type;
	    char* name;
	    char* value;
    } SpxArgChar;

    typedef struct
    {
	    unsigned type;
	    char* name;
	    unsigned taille;
	    int* value;
    } SpxArgTabInt;

    typedef struct
    {
	    unsigned type;
	    char* name;
	    unsigned taille;
	    REEL_S* value;
    } SpxArgTabReel;

    typedef struct
    {
	    unsigned type;
	    char* name;
	    unsigned taille;
	    char** value;
    } SpxArgTabChar;

    typedef union
    {
	    unsigned type;
	    SpxArgTemplate gen;
	    SpxArgInt entier;
	    SpxArgReel reel;
	    SpxArgChar caractere;
	    SpxArgTabInt tab_entier;
	    SpxArgTabReel tab_reel;
	    SpxArgTabChar tab_caractere;
    } SpxArg;

/****************************************************************/
/* Routines utilisant un tableau spx_arg cree par l'utilisateur */
/****************************************************************/

    void SpxInitArg(SpxArg spx_arg[],unsigned taille);
    void SpxPurgeArg(SpxArg spx_arg[],unsigned taille);

    void SpxSetArgInt(SpxArg* spx_arg,char* name,int value);
    void SpxSetArgReel(SpxArg* spx_arg,char* name,REEL_S value);
    void SpxSetArgChar(SpxArg* spx_arg,char* name,char* value);

/* si indice vaut -1 le tableau est remis a zero */
/* mettre tous les indices du meme tableau dans le meme spx_arg !! */

    void SpxSetArgTabInt(SpxArg* spx_arg,char* name,int value,int indice);
    void SpxSetArgTabReel(SpxArg* spx_arg,char* name,REEL_S value,int indice);
    void SpxSetArgTabChar(SpxArg* spx_arg,char* name,char* value,int indice);

/****************************************************************/
/* Routines utilisant un tableau spxarg cache a l'utilisateur   */
/****************************************************************/

    void SpxVPurgeArg(SPXFILE* spx_file);

    void SpxVSetArgInt(SPXFILE* spx_file,char* name,int value);
    void SpxVSetArgReel(SPXFILE* spx_file,char* name,REEL_S value);
    void SpxVSetArgChar(SPXFILE* spx_file,char* name,char* value);

/* si indice vaut -1 le tableau est remis a zero */
/* mettre tous les indices du meme tableau dans le meme spx_arg !! */

    void SpxVSetArgTabInt(SPXFILE* spx_file,char* name,int value,int indice);
    void SpxVSetArgTabReel(SPXFILE* spx_file,char* name,REEL_S value,int indice);
    void SpxVSetArgTabChar(SPXFILE* spx_file,char* name,char* value,int indice);

/****************************************************************/
/* Routines utiles                                              */
/****************************************************************/

    char* SpxStrCopy(char* string,int len);

    void SpxSetColor(const char* type,const char* tabcol,SPXFILE* spx_file);

    void SpxSetValues(SPXFILE* spx_file,SpxArg* spx_arg,unsigned taille);
    void SpxVSetValues(SPXFILE* spx_file);
    void SpxVaSetValues(SPXFILE* spx_file,...);

    void SpxTetra3d(int nx,int ny,REEL_S x[],int dimx,REEL_S y[],int dimy,
		    REEL_S z[],int dimz,SPXFILE* spx_file);
    void SpxMailleTetra3d(int nx,int ny,REEL_S x[],int dimx,REEL_S y[],int dimy,
		    REEL_S c[],int dimc,SPXFILE* spx_file);

    void SpxDTetra3d(int nx,int ny,REEL x[],int dimx,REEL y[],int dimy,
		    REEL z[],int dimz,SPXFILE* spx_file);
    void SpxDMailleTetra3d(int nx,int ny,REEL x[],int dimx,REEL y[],int dimy,
		    REEL c[],int dimc,SPXFILE* spx_file);

    void Spx4Tetra3d(int nx,int ny,REEL4 x[],int dimx,REEL4 y[],int dimy,
		    REEL4 z[],int dimz,SPXFILE* spx_file);
    void Spx4MailleTetra3d(int nx,int ny,REEL4 x[],int dimx,REEL4 y[],int dimy,
		    REEL4 c[],int dimc,SPXFILE* spx_file);

    void Spx8Tetra3d(int nx,int ny,REEL8 x[],int dimx,REEL8 y[],int dimy,
		    REEL8 z[],int dimz,SPXFILE* spx_file);
    void Spx8MailleTetra3d(int nx,int ny,REEL8 x[],int dimx,REEL8 y[],int dimy,
		    REEL8 c[],int dimc,SPXFILE* spx_file);

    void SpxTetra4d(int nx,int ny,REEL_S x[],int dimx,REEL_S y[],int dimy,
		    REEL_S z[],int dimz,REEL_S c[],int dimc,SPXFILE* spx_file);
    void SpxMailleTetra4d(int nx,int ny,REEL_S x[],int dimx,REEL_S y[],int dimy,
		    REEL_S z[],int dimz,REEL_S c[],int dimc,SPXFILE* spx_file);

    void SpxDTetra4d(int nx,int ny,REEL x[],int dimx,REEL y[],int dimy,
		    REEL z[],int dimz,REEL c[],int dimc,SPXFILE* spx_file);
    void SpxDMailleTetra4d(int nx,int ny,REEL x[],int dimx,REEL y[],int dimy,
		    REEL z[],int dimz,REEL c[],int dimc,SPXFILE* spx_file);

    void Spx4Tetra4d(int nx,int ny,REEL4 x[],int dimx,REEL4 y[],int dimy,
		    REEL4 z[],int dimz,REEL4 c[],int dimc,SPXFILE* spx_file);
    void Spx4MailleTetra4d(int nx,int ny,REEL4 x[],int dimx,REEL4 y[],int dimy,
		    REEL4 z[],int dimz,REEL4 c[],int dimc,SPXFILE* spx_file);

    void Spx8Tetra4d(int nx,int ny,REEL8 x[],int dimx,REEL8 y[],int dimy,
		    REEL8 z[],int dimz,REEL8 c[],int dimc,SPXFILE* spx_file);
    void Spx8MailleTetra4d(int nx,int ny,REEL8 x[],int dimx,REEL8 y[],int dimy,
		    REEL8 z[],int dimz,REEL8 c[],int dimc,SPXFILE* spx_file);

#ifdef __cplusplus
}
#endif

#endif
