#ifndef U8SIGNAL_H
#define U8SIGNAL_H

/* on utilise les signaux BSD4.3 (cray = bsdsignal) */
/* sur solaris 2.x on prend les signaux SVR4 */
/* @(#)U8signal.h	1.2 10/12/10 Pierre Brochard */

#ifdef __cplusplus
extern "C" {
#endif

#if defined(IRIX) && defined(__cplusplus)
#   include <osfcn.h>
#   include <libc.h>
#endif

#if defined(IRIX_4)
#   define _BSD_SIGNALS
#endif

#if defined(HP_UX)
#   if !defined(_INCLUDE_POSIX_SOURCE)
#       define _INCLUDE_POSIX_SOURCE
#   endif

#   if !defined(_INCLUDE_XOPEN_SOURCE)
#       define _INCLUDE_XOPEN_SOURCE
#   endif

#   if !defined(_INCLUDE_HPUX_SOURCE)
#       define _INCLUDE_HPUX_SOURCE
#   endif

#   if !defined(_SVID2)
#       define _SVID2
#   endif
#endif

#if defined(U8_SIGNAL_BSD)
#   undef U8_SIGNAL_BSD
#endif

#ifdef U8_EXTENSIONS
#   undef U8_EXTENSIONS
#endif

#if defined(SUNOS_4) || defined(IRIX_4) || defined(LINUX_1) ||\
	(defined(EPIX) && defined(_SYSTYPE_BSD43))
#  define U8_SIGNAL_BSD
#  define U8_SIGNAL_HOLD if(poulpe_flag_sig_usr2) \
	sigblock((int) sigmask(SIGUSR2))
#  define U8_SIGNAL_HOLD_2 _mask = sigblock((int) sigmask(SIGUSR2))
#  define U8_SIGNAL_HOLD_INIT _mask = sigblock((int) sigmask(SIGUSR2))
#  define U8_SIGNAL_RELSE if(poulpe_flag_sig_usr2) sigsetmask(_mask)
#  define U8_SIGNAL_RELSE_2 sigsetmask(_mask)
#else
#  if !defined(__EXTENSIONS__) && defined(SUNOS)
#    define U8_EXTENSIONS
#    define __EXTENSIONS__
#  endif
#  define U8_SIGNAL_HOLD if(poulpe_flag_sig_usr2) sighold(SIGUSR2)
#  define U8_SIGNAL_HOLD_2 sighold(SIGUSR2)
#  define U8_SIGNAL_HOLD_INIT sighold(SIGUSR2)
#  define U8_SIGNAL_RELSE if(poulpe_flag_sig_usr2) sigrelse(SIGUSR2)
#  define U8_SIGNAL_RELSE_2 sigrelse(SIGUSR2)
#endif

#define IHM_SIGNAL_BSD

#if !defined(U8_SIGNAL_BSD)
#   undef IHM_SIGNAL_BSD
#endif

#if !defined(LINUX_1)
#   include <signal.h>
#else
#   include <bsd/signal.h>
#endif

#ifdef U8_EXTENSIONS
#   undef __EXTENSIONS__
#endif

#if defined(_IBMR2) || defined(HP_UX) || defined(CRAY) || defined(__GNUC__) ||\
	defined(OSF1)
    typedef void (*SIG_PF)(int);
#endif

#if defined(U8_SIGNAL_BSD)
#   define sigset(a,b) signal((a),(b))
#   undef U8_SIGNAL_BSD
#endif

/* Macros pour tenir compte de l'interruption possible de certaines
   fonctions. */

#define U8READ(fd,buf,nb,ret)						\
    {									\
	    size_t n2 = (nb);(ret) = (nb);				\
	    while(n2)							\
	    {								\
		char* const xtbuf = ((char*) (buf)) + (nb) - n2;	\
		const ssize_t n = read((fd),xtbuf,n2);			\
		if(n > 0)						\
		    n2 -= n;						\
		else if(!n)						\
		{(ret) -= n2;break;}					\
		else if(n < 0 && errno != EINTR)			\
		{(ret) = -1;break;}					\
	    }								\
    }

#define U8WRITE(fd,buf,nb,ret)						\
    {									\
	    size_t n2 = (nb);(ret) = (nb);				\
	    while(n2)							\
	    {								\
		const char* const xtbuf = ((const char*) (buf)) + (nb) - n2; \
		const ssize_t n = write((fd),xtbuf,n2);			\
		if(n > 0)						\
		    n2 -= n;						\
		else if(!n)						\
		{(ret) -= n2;break;}					\
		else if(n < 0 && errno != EINTR)			\
		{(ret) = -1;break;}					\
	    }								\
    }

#define U8CLOSE(fd,ret)					\
    while(((ret) = close((fd))) < 0 && errno == EINTR)

#define U8FFLUSH(fd,ret)				\
    while(((ret) = fflush((fd))) < 0 && errno == EINTR)

#define U8SFFLUSH(fd)				\
    while(fflush((fd)) < 0 && errno == EINTR)

#define U8FCLOSE(fd,ret)					\
    {								\
	    const int myfdi = fileno((fd));				\
	    if(((ret) = fclose((fd))) == -1 && errno == EINTR)		\
		while(((ret) = close(myfdi)) == -1 && errno == EINTR);	\
    }

#define U8SFCLOSE(fd)					\
    {								\
	    const int myfdi = fileno((fd));				\
	    if(fclose((fd)) == -1 && errno == EINTR)		\
		while(close(myfdi) == -1 && errno == EINTR);	\
    }

#define U8CLOSEDIR(fd,ret)					\
    while(((ret) = closedir((fd))) < 0 && errno == EINTR)

#define U8OPEN(name,flags,mode,fd)					\
    while(((fd) = open((name),(flags),(mode))) < 0 && errno == EINTR)

#define U8FOPEN(name,mode,fd)						\
    while(((fd) = fopen((name),(mode))) == NULL && errno == EINTR)

#define U8FOPEN64(name,mode,fd)						\
    while(((fd) = fopen64((name),(mode))) == NULL && errno == EINTR)

#define U8FREOPEN(name,mode,fd,ret)					\
    {									\
	    const int myfdi = fileno((fd));				\
	    if(((ret) = freopen((name),(mode),(fd))) == NULL && errno == EINTR) \
	    {								\
		while(close(myfdi) == -1 && errno == EINTR);		\
		while(((ret) = fopen((name),(mode))) == NULL && errno == EINTR); \
	    }								\
    }

#define U8FREAD(buf,nb,fd,ret)					\
    {								\
	    size_t n2 = (nb);					\
	    while(n2)						\
	    {							\
		char* const xtbuf = ((char*) (buf)) + (nb) - n2;	\
		const size_t nn = fread(xtbuf,1,n2,(fd));		\
		n2 -= nn;					\
		if(!n2 || !ferror((fd)) || errno != EINTR)	\
		    break;					\
		else						\
		    clearerr((fd));				\
	    }							\
	    (ret) = (nb) - n2;					\
    }

#define U8FWRITE(buf,nb,fd,ret)					\
    {								\
	    size_t n2 = (nb);					\
	    while(n2)						\
	    {							\
		const char* const xtbuf = ((const char*) (buf)) + (nb) - n2; \
		const size_t nn = fwrite(xtbuf,1,n2,(fd));	\
		n2 -= nn;					\
		if(!n2 || !ferror((fd)) || errno != EINTR)	\
		    break;					\
		else						\
		    clearerr((fd));				\
	    }							\
	    (ret) = (nb) - n2;					\
    }

#define U8GETC(fd,ret)				\
    while(((ret) = getc((fd))) == EOF && ferror((fd)) && errno == EINTR) \
	clearerr((fd));

#define U8GETCU(fd,ret)				\
    while(((ret) = getc_unlocked((fd))) == EOF && ferror((fd)) &&	\
		    errno == EINTR)					\
	clearerr((fd));

#define U8PUTC(carac,fd,ret)						\
    while(((ret) = putc((carac),(fd))) == EOF && ferror((fd)) &&	\
		    errno == EINTR)					\
	clearerr((fd));

#ifdef __cplusplus
}
#else /* !__cplusplus */

#ifdef SUNOS_5
    extern  void(*sigset())();
#endif

#endif

#endif /* U8SIGNAL_H */
