#ifndef P4TABGEN_H
#define P4TABGEN_H

// @(#)P4tabgen.h	1.15 10/12/10 Pierre Brochard CEA/DAM-Ile de France

// *********************************************************
//    declaration de la classe generique TabGEN
// *********************************************************

#include "U8machcomp.h"
#include "P4constant.h"
#include "S6compat.h"
#include "dinosaure/S6unsigned.h"

enum TypeAlloc {P4MALLOC=0,P4UNKNOWN=1};

#if !defined(NAR_SUN_ISO) && !defined(NAR_GNU_ISO)
    inline void* operator new(size_t,void* buf) {return buf;}
// Sinon :
// Compilateur ISO C++ ou pre ISO
// Sur Solaris 7 SC5.0 l'operateur new ci-dessus est defini
// et identique = bon pour ce qu'on veut faire.
#endif

#if defined(P4VERSION_AVEC_TEMPLATE) /* version avec template */
#   define INCR22
#   define P4TABGEN_TEMPLATE template <class GEN>
#   define P4TABGEN_GEN <GEN>
#else
#   define P4TABGEN_TEMPLATE
#   define P4TABGEN_GEN
#endif // fin ! version avec template

#endif // P4TABGEN_H

#ifndef P4TABGEN_H2

#if defined(P4VERSION_AVEC_TEMPLATE) /* version avec template */
#   define P4TABGEN_H2
#endif // fin version avec template

P4TABGEN_TEMPLATE class TabGEN
{
    public:

        TabGEN() : _tab(NULL),_ctab(NULL),_taille(0),_type_alloc(P4MALLOC) {};
	TabGEN(GEN* const tab,unsigned tai) : _tab(tab),_ctab(tab),_taille(tai),
	    _type_alloc(P4UNKNOWN) {};
	TabGEN(const GEN* const tab,unsigned tai) : _tab(NULL),_ctab(tab),
		_taille(tai),_type_alloc(P4UNKNOWN) {};
        TabGEN(GEN* const tab,unsigned tai,TypeAlloc type_alloc);
        TabGEN(const GEN* const tab,unsigned tai,TypeAlloc type_alloc);
	TabGEN(S6Unsigned);
        TabGEN(const TabGEN P4TABGEN_GEN& t);
        ~TabGEN();

    // operateurs renvoyant un const GEN&

        const  GEN& operator()(unsigned i) const {return _ctab[i];};
        const  GEN& operator()(int i) const {return _ctab[i];};

    // operateurs de reinitialisation

        void operator()() {_type_alloc=P4MALLOC;_taille=0;_ctab=_tab=NULL;};
        void operator()(const TabGEN P4TABGEN_GEN& t);

    // Methodes

        GEN    Lit(unsigned) const;
        TabGEN Lit(unsigned,unsigned) const;

        void   Lier(GEN*,unsigned);
        void   Lier(const GEN*,unsigned);
        void   Lier(const TabGEN P4TABGEN_GEN&);

        void   Ajoute(const GEN& ob);
        void   AjouteClean(const GEN& ob); // plus sur que ajoute mais plus cher
        void   Ajoute(const TabGEN P4TABGEN_GEN&,unsigned);
        void   Ajoute(const TabGEN P4TABGEN_GEN&);
        void   Ajoute(const GEN* const,unsigned);
        void   Ajoute(const GEN* RESTRICT const,unsigned nx,unsigned ny,
			int dimx);
        void   AjouteLog(const TabGEN P4TABGEN_GEN& t,const P4Boolean log);
        void   Agrand(const GEN,unsigned);
        void   ModVal(const GEN& ob,unsigned i) const
		{
			if(_tab && i<_taille) _tab[i] = ob;
		}
        void   ModValSimple(const GEN& ob,unsigned i) const {_tab[i] = ob;};
#ifndef INCR22
	void   Increment(unsigned i) {++_tab[i];};
#endif
        void   ModTab(const GEN* const,unsigned);
        void   ModTab(const TabGEN P4TABGEN_GEN&,unsigned);
        void   ModTab(const TabGEN P4TABGEN_GEN&);
        void   ModValn(const GEN,unsigned);
        void   ModValn(unsigned);

        void   Enleve();
        void   Enleve(unsigned);
        void   Enleve(unsigned,unsigned);

        void   Permute(unsigned,unsigned) const;
        void   Permute(unsigned,unsigned,unsigned,unsigned) const;

        P4Boolean Contient(const GEN& ob) const;

    // operateurs renvoyant un GEN&

	GEN&   operator[] (unsigned indi) const {return _tab[indi];};
	GEN&   operator[] (int indi) const {return _tab[indi];};
	GEN&   operator* () const {return *_tab;};

    // operateurs d'affectation et de comparaison

        void   operator = (const TabGEN P4TABGEN_GEN& t);
        int    operator < (const TabGEN P4TABGEN_GEN& t) const;
        int    operator == (const TabGEN P4TABGEN_GEN& t) const;

    // operateurs renvoyant _tab ou _ctab

	operator GEN*() const {return _tab;};
	operator void*() const {return _tab;};

        operator const GEN*() const {return _ctab;};
        operator const void*() const {return _ctab;};

    // operateurs renvoyant la taille

        operator unsigned() const {return _taille;};
        operator int() const {return int(_taille);};

    // operateur d'impression

	ostream& operator << (ostream& ost) const;

    protected:

        GEN* RESTRICT _tab;
        const GEN* RESTRICT _ctab;
        unsigned  _taille;
        TypeAlloc _type_alloc;
};

P4TABGEN_TEMPLATE ostream&
operator << (ostream& ost,TabGEN P4TABGEN_GEN t)
{
	t << ost;
	return ost;
}

P4TABGEN_TEMPLATE class TabGENbuf : public TabGEN P4TABGEN_GEN
{
    public:

        TabGENbuf() : TabGEN P4TABGEN_GEN(),_taille_allouee(0) {};
        TabGENbuf(GEN* const tab,unsigned taille) :
			TabGEN P4TABGEN_GEN(tab,taille),
			_taille_allouee(taille) {};
        TabGENbuf(const GEN* const tab,unsigned taille) :
			TabGEN P4TABGEN_GEN(tab,taille),
			_taille_allouee(taille) {};
        TabGENbuf(S6Unsigned);
        TabGENbuf(const TabGEN P4TABGEN_GEN& t);
        ~TabGENbuf();

    // operateurs renvoyant un const GEN&

        const GEN& operator()(unsigned i) const {return TabGEN P4TABGEN_GEN::_ctab[i];};
        const GEN& operator()(int i) const {return TabGEN P4TABGEN_GEN::_ctab[i];};

    // operateurs de reinitialisation

        void operator()() {_taille_allouee = 0;
	  TabGEN P4TABGEN_GEN::_type_alloc=P4MALLOC;
	  TabGEN P4TABGEN_GEN::_taille=0;
	  TabGEN P4TABGEN_GEN::_ctab=TabGEN P4TABGEN_GEN::_tab=NULL;};

    // Methodes

        unsigned TailleAllouee() const {return _taille_allouee;};

        void   Lier(GEN*,unsigned);
        void   Lier(const GEN*,unsigned);
        void   Lier(const TabGEN P4TABGEN_GEN&);

        void   Ajoute(const GEN& ob);
        void   Ajoute(const GEN& ob,unsigned buf_taille);
        void   AjouteClean(const GEN& ob);  // plus sur que ajoute mais plus cher
        void   Ajoute(const TabGEN P4TABGEN_GEN&,unsigned);
        void   Ajoute(const TabGEN P4TABGEN_GEN&);
        void   Ajoute(const GEN* const,unsigned taille);
        void   Ajoute(const GEN* const,unsigned taille,unsigned buf_taille);
        void   Ajoute(const GEN* RESTRICT const,unsigned nx,unsigned ny,
			int dimx);
        void   Agrand(const GEN,unsigned);
        void   ModTab(const GEN* const,unsigned);
        void   ModTab(const TabGEN P4TABGEN_GEN&,unsigned);
        void   ModTab(const TabGEN P4TABGEN_GEN&);
        void   ModValn(const GEN,unsigned);
        void   ModValn(unsigned);

        void   Enleve();
        void   Vide();
        void   Enleve(unsigned);
        void   Enleve(unsigned,unsigned);

        P4Boolean Compare(const TabGENbuf P4TABGEN_GEN& t) const;

    // operateurs d'affectation et de comparaison

        void   operator = (const TabGEN P4TABGEN_GEN& t);
        void   operator = (const TabGENbuf P4TABGEN_GEN& t)
		{*this = (const TabGEN P4TABGEN_GEN&) t;}

    // operateur d'impression

	ostream& operator << (ostream& ost) const;

    private:

        unsigned _taille_allouee;
};

P4TABGEN_TEMPLATE ostream&
operator << (ostream& ost,TabGENbuf P4TABGEN_GEN t)
{
	t << ost;
	return ost;
}

#   define P4TABGEN_MKDEP
#   include "dinosaure/P4tabgen.c.h"
#   include "dinosaure/P4tabgensup.c.h"
#   undef P4TABGEN_MKDEP
#endif // P4TABGEN_H2
