#ifndef NXXDRSTREAM_H
#define NXXDRSTREAM_H

// @(#)nxxdrstream.h	1.3 03/17/21 P. Brochard & M. Chineaud

#include "nxstream.h"

#if !defined(SUNOS_4) && !defined(HP_UX) && !defined(LINUX)
#   include <sys/select.h>
#endif

#include "nxxdr.h"

// NX_RPC_V1 est defini si on n'utilise pas les mecanismes supplementaires
// fournis avec RPC V2 pour explorer le flot XDR comme la
// fonction xdr_control.
// On note que cette fonction utilisee pour regarder ce qui reste
// dans les buffers d'XDR provoque un flush bloquant qui est genant.
// On defini donc systematiquement NX_RPC_V1 meme si les RPC sont de type V2
// comme sur Solaris de SUN.

#define NX_RPC_V1

#if defined(IRIX)
    typedef void* type_handle;
    typedef void* type_constbuf;
    typedef void* type_buf;
    typedef u_int type_int;
#elif defined(_IBMR2) || defined(CRAY) || defined(__DECCXX)
    typedef int type_handle;
    typedef const void* type_constbuf;
    typedef void* type_buf;
    typedef size_t type_int;
#elif defined(OSF1)
    typedef int type_handle;
    typedef void* type_constbuf;
    typedef void* type_buf;
    typedef size_t type_int;
#elif defined(SUNOS)
    typedef void* type_handle;
    typedef char* type_constbuf;
    typedef char* type_buf;
    typedef int type_int;
#elif defined(TIRPC)
    typedef void* type_handle;
    typedef void* type_constbuf;
    typedef void* type_buf;
    typedef int type_int;
#elif defined(__GNUC__)
    typedef char* type_handle;
    typedef char* type_constbuf;
    typedef char* type_buf;
    typedef int type_int;
#else
    typedef void* type_handle;
    typedef char* type_constbuf;
    typedef char* type_buf;
    typedef int type_int;
#endif

// class nxXdrstream encapsulant XDR*

class nxXdrstream : public nxstream
{
    public:

	nxXdrstream(FILE* f);
	nxXdrstream(const char* name,const char* mode);
	nxXdrstream(nxstream* nx_serveur);
	nxXdrstream(nxstream& nx_serveur);
	nxXdrstream(nxstream& nx_serveur,char* handler_name);
	~nxXdrstream();

	XDR* xdrread() const;
	XDR* xdrwrite() const;

	int DataInBuffer();

    private:

	void init(FILE* f);

	static int read(type_handle iohandle,type_buf buf,type_int nbytes);
	static int write(type_handle iohandle,type_buf buf,type_int nbytes);

	XDR* _xr; // flot XDR en lecture
	XDR* _xw; // flot XDR en ecriture
};

// fonction membres inline

inline XDR*
nxXdrstream::xdrread() const
{
	return _xr;
}

inline XDR*
nxXdrstream::xdrwrite() const
{
	return _xw;
}

#endif /* fin NXXDRSTREAM_H */
