#ifndef REPERTOIRE_H
#define REPERTOIRE_H

//
// Gestion du repertoire
// @(#)Repertoire.h	1.7 10/12/10 G. Colin de Verdiere
// modifie Pierre Brochard
//

class ScrollList : public ListeSimple
{
    protected:

	int nbentries; /* nombre de boutons crees */
	char** filename; /* tableau des noms des boutons */

    public:

	ScrollList(const Widget& parent);
	~ScrollList();

	virtual void ChangeListe(const char* const dir,
			const char* const filtre = NULL);
	void ChangeListeGen(const char* const dir,const int type,
			const char* const filtre = NULL);
};

class Directory;

class ScrollFileList : public ScrollList
{
    private: 

	Directory* _pere;
	Label*     _fichier;

    public:

	ScrollFileList(const Widget& parent, const char* dir, 
			Label* const fichier,Directory* const pere);
	~ScrollFileList();

	void ChangeListe(const char* const dir,const char* const filtre = NULL);
	void SelectCB(XtPointer call_data);
	void SelectDoubleCB(XtPointer call_data);
	void SelectExtendedCB(XtPointer call_data);
	char* GetNomFichier() {return _fichier->GetLabel();};
};


class ScrollDirList : public ScrollList
{
    private: 

	ScrollFileList* _files; /* gestionnaire des fichiers simples */
	Saisie* _chemin;
	Saisie* _filtre;
	Label*  _fichier;

    public:

	ScrollDirList(const Widget& parent, const char* const dir, 
			Label* const fichier, Saisie* const chemin, 
			ScrollFileList* const files,
			Saisie* const filtre);
	~ScrollDirList();

	void ChangeListe(const char* const dir,const char* const filtre = NULL);
	void SelectCB(XtPointer call_data);
	char* GetNomRepertoire() {return _chemin->GetLabel();};
};

class CancelButton : public Bouton
{
    private:

        PopUp* _pere;

    public:

	CancelButton(const Widget& parent,const char* label,PopUp* const pere);

	void SelectCB(XtPointer call_data);
};

struct Communiq
{
	Communiq() {};
	virtual ~Communiq() {};

	virtual void FichierChoisi(char**,int,int = 1) {} ;
	virtual void CheminChoisi(const char* const,int = 1) {};
	virtual const char* DirectoryInitial() {return ".";};
};

class OKButton : public Bouton
{
    private:

	Directory* _pere;
	char* _fichierChoisi;
	char* _cheminChoisi;
	Communiq* _ptr; // pour retour a l'appli

    public:

	OKButton(const Widget& parent,const char* label,Directory* const pere,
			Communiq* const ptr);
	~OKButton();

	void SelectCB(XtPointer call_data);

	char* FichierChoisi() { return _fichierChoisi; };
	char* CheminChoisi() { return _cheminChoisi; };
};

class SaisieDir : public Saisie
{
    private:

	Directory* _pere;

    public:

	SaisieDir(const Widget& parent, const char* label,
			Directory* const pere);
	~SaisieDir() {};

	void SelectCB(XtPointer call_data);
};

class SaisieFiltre : public Saisie
{
    private:

	Directory* _pere;

    public:

	SaisieFiltre(const Widget& parent, const char* label,
			Directory* const pere);
	~SaisieFiltre() {};

	void SelectCB(XtPointer call_data);
};

class Help;

class AideButton : public Bouton
{
    private:

	Help* _help; 

    public:
 
	AideButton(const Widget& parent, const char *label, 
			const char* classe) :
			Bouton(parent,label,classe),_help(NULL) {};
	~AideButton() {};
      
	void SelectCB(XtPointer call_data);
};

class Directory : public PullDownEntry
{
    private:

	int		_done;

	PopUp*		_pere;
	Conteneur*	_LeTout;

	ConteneurJoli*  _LesBtnsJ;
	Conteneur*	_LesBtns;
	ConteneurJoli*  _LesLabsJ;
	Conteneur*	_LesLabs;
	ConteneurJoli*  _LesListesJ;
	Conteneur*	_LesListes;
	ConteneurJoli*  _LesLabs2J;
	Conteneur*	_LesLabs2;

	ScrollDirList*	_direcs;
	ScrollFileList*	_fichiers;

	CancelButton*	_cancel;
	OKButton*	_ok;
	AideButton*	_help;

	SaisieDir*	_chemin;
	SaisieFiltre*	_filtre;
	Label*		_chemin_label;
	Label*		_filtre_label;

	Label*		_fichier;
	Label*		_fichier_label;

	Communiq*	_ptr;

	char**       	_labels;
	unsigned      	_labels_taille;

    public:

	Directory(const Widget& root, const Widget& parent, const char *label,
			Communiq* const ptr = NULL,
			const char* const* labels = NULL,
			unsigned labels_taille = 0);
	~Directory();

	ScrollDirList* GetDir() { return _direcs; } ;
	SaisieDir* GetChem() { return _chemin; };
	SaisieFiltre* GetFiltre() { return _filtre; };

	void SelectCB(XtPointer call_data);
	void Cache() { _pere->Cache(); };

	char* GetNomFichier();
	char* GetNomRepertoire();

	void OKAction();
	void OKAction(char** selected,int nbselected);
};

#endif
