#define KEEPOPEN

#include "defs.h"
#include <sys/stat.h>
#include <pwd.h>
#include <ar.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>

#ifndef CRAY
#   define AR_HDR_SIZE sizeof(struct ar_hdr)
#endif

#ifdef AIX
#   define MAKE_ARFMAG AIAFMAG
#   define MAKE_FMAG AIAFMAG
#   define MAKE_ARMAG AIAMAG
#   define MAKE_SARMAG SAIAMAG
#else
#   define MAKE_ARFMAG ARFMAG
#   define MAKE_FMAG ar_port.ar_fmag
#   define MAKE_ARMAG ARMAG
#   define MAKE_SARMAG SARMAG
#endif

#define equaln          !strncmp

char    archmem[1024];    /* archive file member name to search for */
char    archname[1024];   /* archive file to be opened */

long    first_ar_mem;   /* where first archive member header is at */
long    sym_begin;      /* where the symbol lookup starts */
long    num_symbols;    /* the number of symbols available */
long    sym_size;       /* length of symbol directory file */

struct ar_hdr ar_port;

TIMETYPE        afilescan();
TIMETYPE        entryscan();

#ifdef  KEEPOPEN
#  define MAXARDIR        512
#  define ARBLKSZ         4096
   FILE            *arfd = NULL;

   struct ardir {
        char    *name;
        long    date;
   } ardir[MAXARDIR];

   int ardiri = 0;
   long arnext = 0;
   char *arsavstr();
#else
   FILE            *arfd = NULL;
#endif

int amatch();
int umatch();
int openarch();
void clarch();
void arfre();

/* routines de misc.c */

void fatal1();
void fatal();

TIMETYPE
exists(pname)
NAMEBLOCK pname;
{
	register CHARSTAR s;
	struct stat buf;
	TIMETYPE lookarch();
	CHARSTAR filename;

	filename = pname->namep;

	if(any(filename, LPAREN)) return(lookarch(filename));

	if(stat(filename,&buf) < 0)
	{
	    s = findfl(filename);
	    if(s != (CHARSTAR )-1)
	    {
		pname->alias = copys(s);
		if(stat(pname->alias, &buf) == 0) return(buf.st_mtime);
	    }
	    return(0);
	}
	else
	    return(buf.st_mtime);
}


TIMETYPE
prestime()
{
	TIMETYPE t;
	time(&t);
	return(t);
}

DEPBLOCK
srchdir(pat, mkchain, nextdbl)
register CHARSTAR pat;          /* pattern to be matched in directory */
int mkchain;                    /* nonzero if results to be remembered */
DEPBLOCK nextdbl;               /* final value for chain */
{
	DIR * dirf;
	CHARSTAR dirname, dirpref, endir, filepat, p;
	char temp[100];
	char fullname[100];
	CHARSTAR p1;
	NAMEBLOCK q;
	DEPBLOCK thisdbl;
	OPENDIR od;
	int dirofl = 0;
	static int opendirs = 0;
	PATTERN patp;

/*struct direct entry[32];        ** NFS changes **/
	struct dirent *entry;

	thisdbl = 0;

	if(mkchain == NO)
	    for(patp=firstpat ; patp!=0 ; patp = patp->nextpattern)
		if(equal(pat,patp->patval)) return(0);

	patp = ALLOC(pattern);
	patp->nextpattern = firstpat;
	firstpat = patp;
	patp->patval = copys(pat);

	endir = 0;

	for(p=pat; *p!=CNULL; ++p)
	    if(*p==SLASH) endir = p;

	if(endir==0)
	{
	    dirname = ".";
	    dirpref = "";
	    filepat = pat;
	}
	else
	{
	    *endir = CNULL;
	    dirpref = concat(pat, "/", temp);
	    filepat = endir+1;
	    dirname = temp;
	}

	dirf = NULL;

	for(od=firstod ; od!=0; od = od->nextopendir)
	    if(equal(dirname, od->dirn))
	    {
		dirf = od->dirfc;
		rewinddir(dirf);   /* NFS changes */
/*      fseek(dirf,0L,0);  * start over at the beginning  */
		break;
	    }

	if(dirf == NULL)
	{
	    dirf = opendir(dirname);   /* NFS changes */
/*      dirf = fopen(dirname, "r");  */
	    if(++opendirs < MAXODIR)
	    {
		od = ALLOC(opendir);
		od->nextopendir = firstod;
		firstod = od;
		od->dirfc = dirf;
		od->dirn = copys(dirname);
	    }
	    else
		dirofl = 1;
	}

	if(dirf == NULL)
	{
	    fprintf(stderr, "Directory %s: ", dirname);
	    fatal("Cannot open");
	}
	else    while ((entry = readdir(dirf)))
	{
	    p1 = entry->d_name;
	    if( amatch(entry->d_name,filepat) )
	    {
		concat(dirpref,entry->d_name,fullname);
		if( (q=srchname(fullname)) == 0)
		    q = makename(copys(fullname));
		if(mkchain)
		{
		    thisdbl = ALLOC(depblock);
		    thisdbl->nextdep = nextdbl;
		    thisdbl->depname = q;
		    nextdbl = thisdbl;
		}
	    }
	}

	if(endir != 0) *endir = SLASH;

	if(dirofl) closedir(dirf);  /* NFS changes */
/*             fclose(dirf);                 */

	return(thisdbl);
}

/* stolen from glob through find */

int
amatch(s, p)
CHARSTAR s, p;
{
	register int cc, scc, k;
	int c, lc;

	scc = *s;
	lc = 077777;
	switch (c = *p)
	{

	  case LSQUAR:
	    k = 0;
	    while ((cc = *++p))
	    {
		switch (cc)
		{

		  case RSQUAR:
		    if (k)
			return(amatch(++s, ++p));
		    else
			return(0);

		  case MINUS:
		    k |= (lc <= scc) & (scc <= (cc=p[1]));
		}
		if(scc==(lc=cc)) k++;
	    }
	    return(0);

	  case QUESTN:
    caseq:
	    if(scc) return(amatch(++s, ++p));
	    return(0);

	  case STAR:
	    return(umatch(s, ++p));

	  case 0:
	    return(!scc);
	}
	if(c==scc) goto caseq;
	return(0);
}

int
umatch(s, p)
register CHARSTAR s, p;
{
	if(*p==0) return(1);

	while(*s)
	    if(amatch(s++,p)) return(1);

	return(0);
}

/* 
 *  look inside archives for notations a(b) and a((b))
 *      a(b)    is file member   b   in archive a
 *      a((b))  is entry point   b  in object archive a
 */


TIMETYPE
lookarch(filename)
register CHARSTAR filename;
{
	register int i;
	CHARSTAR p, q;
	char s[15];
	int nc, objarch;
	TIMETYPE la();

	for(p = filename; *p!= LPAREN ; ++p);
	i = p - filename;
	strncpy(archname, filename, i);
	archname[i] = CNULL;
	if(archname[0] == CNULL)
	    fatal1("Null archive name `%s'", filename);
	p++;
	if(*p == LPAREN)
	{
	    objarch = YES;
	    ++p;
	    if((q = strchr(p, RPAREN)) == NULL)
		q = p + strlen(p);
	    strncpy(s,p,q-p);
	    s[q-p] = CNULL;
	}
	else
	{
	    objarch = NO;
	    if((q = strchr(p, RPAREN)) == NULL) q = p + strlen(p);
	    i = q - p;
	    strncpy(archmem, p, i);
	    archmem[i] = CNULL;
	    nc = 14;
	    if(archmem[0] == CNULL)
		fatal1("Null archive member name `%s'", filename);
	    if((q = strrchr(archmem, SLASH)))
		++q;
	    else
		q = archmem;
	    strncpy(s, q, nc);
	    s[nc] = CNULL;
	}
	return(la(s, objarch));
}

TIMETYPE
la(am,flag)
register char *am;
register int flag;
{
	TIMETYPE date = 0L;

	if(openarch(archname) == -1) return(0L);

	if(flag)
	    date = entryscan(am);   /* fatals if cannot find entry */
	else
	    date = afilescan(am);
#ifndef KEEPOPEN
	clarch();
#endif

	return(date);
}

TIMETYPE
afilescan(name)             /* return date for named archive member file */
char *name;
{

	if(fseek(arfd,first_ar_mem,0) != 0) goto seek_error;

	for(;;)
	{
	    int len = strlen(name);
	    long ptr;
	    int test_name;

	    memset((char*) &ar_port,0,AR_HDR_SIZE);

	    if(fread((char*) &ar_port, AR_HDR_SIZE , 1, arfd) != 1 || 
			    !equaln(MAKE_FMAG,MAKE_ARFMAG,sizeof(MAKE_FMAG)))
	    {

		if(feof(arfd)) return(0L);

		break;
	    }

#ifdef AIX
	    {
		CHARSTAR ar_name_temp;
		int size = 0;
		int size_pair;

		if(ar_port.ar_date[0] != '\0' &&
				sscanf(ar_port.ar_namlen,"%d",&size) != 1)
		{
		/* la table des symbloles globaux ne semble pas faire la taille
		   correcte si ar d ... !! */
		    return(0L);
		/*fatal1("Bad size_name field for %.4s in archive %s",
				    ar_port.ar_namlen,archname);*/
		}

		size_pair = size + (size & 01);

		if(size != len)
		{

		    if(fseek(arfd,size_pair,1) != 0) goto seek_error;

		    test_name = 0;
		}
		else
		{

		    if(!(ar_name_temp = malloc(size_pair+2)))
			fatal("Cannot alloc mem");

		    ar_name_temp[0] = ar_port._ar_name.ar_name[0];
		    ar_name_temp[1] = ar_port._ar_name.ar_name[1];

		    if(fread(ar_name_temp+2,size_pair,1,arfd) != 1)
		    {

			if(!equaln(ar_name_temp+size_pair,MAKE_ARFMAG,
					sizeof(ar_port._ar_name.ar_fmag)))
			{
			    fatal1("Bad name field for %s in archive %s",
					    name, archname);
			}

		    }

		    ar_name_temp[size] = '\0';
		    test_name = equaln(ar_name_temp,name,len);
		    free(ar_name_temp);
		}

	    }
#else
	    test_name = equaln(ar_port.ar_name,name,len);
#endif

	    if(test_name)
	    {
		long date;

		if(sscanf(ar_port.ar_date, "%ld", &date) != 1)
		{
		    fatal1("Bad date field for %.14s in %s", name, archname);
		}

		return (date);
	    }

	    if (sscanf(ar_port.ar_size, "%ld", &ptr) != 1)
	    {
		fatal1("Bad size field for %.10s in archive %s", name, archname);
	    }

	    ptr += (ptr & 01);

	    if (fseek(arfd, ptr, 1) != 0) goto seek_error;

	}
    /*
     * Only here if fread() [or equaln()] failed and not at EOF
     */
	fatal1("Read error on archive %s", archname);
    /*NOTREACHED*/
seek_error:;
	fatal1("Seek error on archive file %s", archname);

	return 0;
}

TIMETYPE
entryscan(name)         /* return date of member containing global var named */
char *name; {
#if 0
/* LeTex
   #ifndef CRAY
   */
	extern char *malloc();
	int strtablen;
	register char *offs;    /* offsets table */
	register char *syms;    /* string table */
	register char *strend;  /* end of string table */
	char *strbeg;

    /*
     * Format of the symbol directory for this format is
     * as follows:   [sputl()d number_of_symbols]
     *               [sputl()d first_symbol_offset]
     *                       ...
     *               [sputl()d number_of_symbols'_offset]
     *               [null_terminated_string_table_of_symbols]
     */
	if ((offs = malloc(num_symbols * sizeof(long))) == NULL)
	{
	    fatal1("Cannot alloc offsets table for archive %s", archname);
	}
	if (fread((char *)offs, sizeof(long), num_symbols, arfd) != num_symbols)
	    goto read_error;
	strtablen = sym_size - ((num_symbols + 1) * sizeof(long));
	if ((syms = malloc(strtablen)) == NULL)
	{
	    fatal1("Cannot alloc string table for archive %s", archname);
	}
	if (fread((char *)syms, sizeof(char), strtablen, arfd) != strtablen)
	    goto read_error;
	strend = &syms[strtablen];
	strbeg = syms;
	while (syms < strend)
	{
	    if (equal(syms, name))
	    {
		long ptr, date;

		ptr = sgetl(offs);
		if(fseek(arfd, ptr, 0) != 0) goto seek_error;
		if(fread((char *)&ar_port, sizeof(ar_port), 1, arfd) != 1 || 
				!equaln(ar_port.ar_fmag,ARFMAG,
						sizeof(ar_port.ar_fmag)) )
		{
		    goto read_error;
		}
		if (sscanf(ar_port.ar_date, "%ld", &date) != 1)
		{
		    fatal1("Bad date for %.14s, archive %s", ar_port.ar_name,
				    archname);
		}
		free(strbeg);
		return (date);
	    }
	    syms += strlen(syms) + 1;
	    offs += sizeof(long);

	    free(strbeg);
	}
	fatal1("Cannot find symbol %s in archive %s", name, archname);

seek_error:;
	fatal1("Seek error on archive file %s", archname);

read_error:;
	fatal1("Read error on archive %s", archname);
#endif /* CRAY */

	return 0;
}

int
openarch(f)
register CHARSTAR f;
{
	char mag_port[MAKE_SARMAG];   /* port (6.0) archive format */

#ifdef  KEEPOPEN
	static char name[1024];
	static int oldndo;

	if (arfd && !strcmp(f, name) && oldndo == ndocoms)
	    return(0);

	clarch();
	oldndo = ndocoms;
	strcpy(name, f);
#endif

	arfd = fopen(f, "r");

	if(arfd == NULL) return(-1);

	if(fseek(arfd, 0L, 0) != 0 ||
			fread((char*) mag_port,MAKE_SARMAG, 1, arfd) != 1)
	{
	    return (-1);
	}

	if(equaln(mag_port,MAKE_ARMAG, MAKE_SARMAG))
	{
	/*
	 * Must read in first member header to find out
	 * if there is a symbol directory
	 */

#ifdef AIX
	    if(fseek(arfd,sizeof(struct fl_hdr),0) != 0)
	    {
		return (-1);
	    }
#endif
	    first_ar_mem = ftell(arfd);

	    if(fread((char *)&ar_port,sizeof(ar_port), 1, arfd) != 1 ||
			    !equaln(MAKE_ARFMAG,MAKE_FMAG,sizeof(MAKE_FMAG)))
	    {
		first_ar_mem = 0;
		return (-1);
	    }

	    sym_size = num_symbols = sym_begin = 0L;
	}
	else
	    fatal1("%s is not an archive", f);

	return(0);
}

void
clarch()
{
#ifdef  KEEPOPEN
	ardiri = 0;
	arnext = 0;
	arfre();
#endif
	if(arfd != NULL) fclose(arfd);

#ifdef  KEEPOPEN
	arfd = NULL;
#endif
}

/*
 *      Used when unlinking files. If file cannot be stat'ed or it is
 *      a directory, then do not remove it.
 */

int
isdir(p)
char *p;
{
	struct stat statbuf;

	if(stat(p, &statbuf) == -1) return(1);     /* no stat, no remove */

	if((statbuf.st_mode&S_IFMT) == S_IFDIR) return(1);

	return(0);
}

#ifdef  KEEPOPEN
struct arblock {
        int left;
        struct arblock *next;
        char buf[ARBLKSZ];
} arblockroot;

struct arblock *arblock = &arblockroot;

char *
arsavstr(s, n)
char *s;
int n;
{
#define MALLOC(type) (type *)malloc( sizeof(type) )

	if (arblock->left <= n) {
	    if (arblock->next == NULL) {
		if ( (arblock->next = MALLOC(struct arblock)) == 0 )
		    fatal1("Out of memory for archive directory\n");
		arblock->next->next = 0;
	    }
	    arblock = arblock->next;
	    arblock->left = ARBLKSZ;
	}
	strncpy(&arblock->buf[ARBLKSZ-arblock->left], s, n);
	++n;
	arblock->buf[ARBLKSZ-arblock->left+n] = 0;
	arblock->left -= n;
	return(&arblock->buf[ARBLKSZ-arblock->left-n]);
}

void
arfre()
{
	arblock = &arblockroot;
	arblock->left = ARBLKSZ;
}
#endif
