#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <malloc.h>
#include <stdlib.h>
#include <errno.h>

#define MAXLIG 4096  /* longueur de la ligne stockant les dependences */
#define MAXREP 1024  /* longueur max des nom de fichiers */
#define MAXLGSUFF 10   /* longueur max du suffixe apres -o */
#define MAXPATHINC 30   /* nombre maxi de chemin avec -I    */
#define	DIESE_INCLUDE "#include"
#define	DBLE_COTE '"'

typedef enum
{
	FAUX,
	VRAI 
} Tbooleen ;

Tbooleen		mk_inc = FAUX;
Tbooleen		mk_lib = FAUX;
Tbooleen		mk_file= FAUX;
Tbooleen		mk_suff= FAUX;
Tbooleen		mk_sccs= FAUX;
Tbooleen		mk_cmd = FAUX;

int			mk_nom_en_sccs = 0;
int			mk_i_inc = 0;
char*		mk_inc_path[MAXPATHINC];
char*		mk_lib_path = NULL;
char		mk_file_out[MAXREP];
char		mk_suff_name[MAXLGSUFF];
char*		mk_err_file = NULL;
int mk_nb_include = 0; /* Nbre d'includes memorise pour un fichier c ou f */
char** mk_memo_include = NULL; /* tableau de memorisation des nom d'includes
				  d'un fichier */

static int findch(int is_include,char* file,char* ligne,int cr);

void
usage()
{
	fprintf(stderr,"usage: mkdep [-Iincpath] [-Llibpath] [-f mkfile] "
		"[-sccs] [-cmd] [-oobjsuf] fichier_sources...\n");
	exit(0);
}

/*  =================================================================  */
/*  Fonction de recherche du nom complet de l'include                  */
/*  Il s'agit de mettre le bon repertoire devant le nom si necessaire  */
/*  =================================================================  */

int
QuelInclude(char* fullName,char* include)
{
	Tbooleen trouve = FAUX;
	int i_inc = 0;
	struct stat buffStat;
	int cr = 1; /* pour forcer l'ecriture de la ligne */

	if ( mk_inc == VRAI )
	{

	    for ( i_inc=0 ; trouve == FAUX && i_inc < mk_i_inc ; i_inc++ )
	    {
		strcpy(fullName,mk_inc_path[i_inc]);
		strcat(fullName,"/");
		strcat(fullName,include);

		if ( stat(fullName,&buffStat) == 0 )
		{
		    trouve = VRAI;
		}
		
	    } 

	    if ( trouve == FAUX )
	    {
		printf("mkdep : impossible de trouver l'include %s\n",include);
		cr = 0 ; /* pour annuler l'ecriture de la ligne de dep. */
			 /* et continuer sans erreur*/
	    }

	}
	else
	{
	    strcpy(fullName,include);
	}

	return cr;
}

/*  =================================================================  */
/*  Fonction de recherche et de memorisation du nom d'un include       */
/*  pour savoir si il a deja ete utilise dans la ligne de dependence   */
/*  =================================================================  */

Tbooleen
NouvelInclude(char* include)
{
	int i_include = 0;
	Tbooleen trouve = FAUX;
	Tbooleen nouveau = VRAI;

/*  Boucle de recherche du nom de l'include  */
/*  ---------------------------------------  */
	while ( trouve == FAUX && i_include < mk_nb_include )
	{

	    if ( strcmp(include,mk_memo_include[i_include]) == 0 )
	    {
		trouve = VRAI;
	    }
	    else
	    {
		i_include++;
	    }

	}

	if ( trouve == FAUX )
	{

	    /*  Si non trouve alors memorisation  */
	    /*  --------------------------------  */
	    mk_nb_include++;

	    if ( mk_nb_include == 1 )
	    {
		mk_memo_include = (char**) malloc(mk_nb_include*sizeof(char*));
	    }
	    else
	    {
		mk_memo_include = (char**) realloc((char**)mk_memo_include,
						   mk_nb_include*sizeof(char*));
	    }

	    if ( mk_memo_include == NULL )
	    {
		printf("mkdep:memoire insuffisante...\n");
		exit(1);
	    }
	    else
	    {
		mk_memo_include[mk_nb_include-1] = (char*)
		          malloc((strlen(include)+1)*sizeof(char));
		strcpy(mk_memo_include[mk_nb_include-1],include);
	    }

	}

	nouveau = trouve == FAUX ? VRAI : FAUX;
	return nouveau;
}

/* ================================================= */
/* Fonction de consruction de la ligne de dependance */
/* ================================================= */

int
construit_ligne(char* file,char* ligne)
{
	char objet[MAXREP];
	int flag = 0;
	int i = 0;

	strcpy(objet,file);
	objet[strcspn(objet,".")] = '\0';

/*  Preparation du debut de la ligne de dependance  */
/*  ----------------------------------------------  */

	if ( mk_lib == VRAI )
	{
	    strcpy(ligne,mk_lib_path);
	    strcat(ligne,"(");
	    strcat(ligne,objet);
	    strcat(ligne,mk_suff_name);
	    strcat(ligne,") : ");

	    if(mk_sccs)
	    {
		strcat(ligne,"s.");
	    }

	    strcat(ligne,file);
	}
	else
	{

	    if ( mk_nom_en_sccs )
	    {
		strcpy(ligne,"s.");
	    }
	    else
	    {
		strcpy(ligne,"");
	    }

	    strcat(ligne,objet);
	    strcat(ligne,mk_suff_name);
	    strcat(ligne," : ");
	    strcat(ligne,file);
	}

/*  Re-initialisation du tableau de memoristion des includes  */
/*  --------------------------------------------------------  */

	if ( mk_nb_include > 0 )
	{

	    for ( i = 0 ; i < mk_nb_include ; i++ )
	    {
		free(mk_memo_include[i]);
	    }

	    free(mk_memo_include);
	    mk_nb_include = 0;
	}

	/*  	Recherche des dependances  */
	/*  	-------------------------  */

	flag = findch(0,file,ligne,0);

	if ( flag == -1 )
	{
	    fprintf(stderr,"mkdep: Impossible d'ouvrir le fichier %s\n",
		    mk_err_file);
	    exit(-1);
	}

	return flag;
}

/*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
/*  FONCTION FOURNISSANT TOUS LES NOMS DES FICHIERS SOLLICITES PAR */
/* LE FICHIER fich DANS L'INSTRUCTION INCLUDE.                     */
/*_________________________________________________________________*/

static int
findch(int is_include,char* file,char* ligne,int cr)
{

    /* [I] is_include=1 : le fichier "fich" est un include*/
    /* [I] fichier a traiter */
    /* [O] stockage des dependances */ 

	FILE* fsource = NULL;
	char my_buff[MAXLIG];
	int lg_diese_include = strlen(DIESE_INCLUDE);
	char include_name[MAXREP];
	char include_fullname[MAXREP];

	my_buff[0] = '\0';

/*  ouverture du fichier a explorer  */
/*  -------------------------------  */

	if ((fsource = fopen(file,"r"))==NULL)
	{
	    cr = -1;
	    mk_err_file = file;

	    return cr;
	}

/*  Boucle de lecture et recherche de #   include  */
/*  ---------------------------------------------  */

	while( fgets(my_buff,MAXLIG,fsource) != NULL && cr != -1 )
	{

	    if(my_buff && lg_diese_include > 0 && my_buff[0] == DIESE_INCLUDE[0])
	    {
		char* pt_buff = my_buff + 1;

		while ( *pt_buff == ' ' ) pt_buff++;

		if(strncmp(pt_buff,DIESE_INCLUDE+1,lg_diese_include-1) != 0 )
		  continue;

		pt_buff += lg_diese_include - 1;

		while ( *pt_buff == ' ' ) pt_buff++;

		if ( *pt_buff == DBLE_COTE )
		{
		    pt_buff++;
		    strcpy(include_name,pt_buff);
		    pt_buff = strchr(include_name,DBLE_COTE);

		    if ( pt_buff != NULL )
		    {
			*pt_buff = '\0';

			/* include trouve ; quel est son nom complet  */
			/* -----------------------------------------  */

			cr = QuelInclude(include_fullname,include_name);

			/* cr peut valoir 2 valeur :  */
			/* 0: include non trouve -> non ecriture ds ecrit_ligne*/
			/* 1: include trouve -> forcage ecriture ds ecrit_ligne*/

			/*   est il deja utilise  */
			/*   -------------------  */

			if ( cr == 1 )
			{

			    if ( NouvelInclude(include_fullname) == VRAI )
			    {

				/* recherche d'include dans les includes   */
				/* -------------------------------------  */
				cr = findch(1,include_fullname,ligne,cr);
			    }

			}

		    }

		}

	    }

	}

	fclose(fsource);

	return cr;
}

static void
ecrit_ligne(FILE* ft,char* identif,char* ligne,int flag,char* action)
{

	if( flag == 0 )
	{
	    fprintf(ft,"%s\n",identif);
	    fprintf(ft,"%s\n\t@$(%s)\n\n",ligne,action);
	}
	else if ( flag == 1 )
	{
	    int i = 0;

	    fprintf(ft,"%s\n",identif);
	    fprintf(ft,"%s\\\n",ligne);

	    for (; i < mk_nb_include-1 ; i++ )
	    {
		fprintf(ft,"  %s\\\n",mk_memo_include[i]);
	    }

	    fprintf(ft,"  %s\n",mk_memo_include[mk_nb_include-1]);
	    fprintf(ft,"\t@$(%s)\n\n",action);

	    if ( mk_cmd )
	    {

		if ( mk_sccs )
		{
		    fprintf(ft,"\t$(MKDEP_CMD)\n");
		}
		else
		{
		    fprintf(ft,"\t$(MKDEP_CMD_NOSCCS)\n");
		}

	    }

	}

}

/***********************************************************/
/*   PROGRAMME DE RECHERCHE DES DEPENDANCES DE FICHIERS    */
/*        ENTRE EUX EN VUE DE L'UTILISATION DE MAKE.       */
/***********************************************************/

int
main(int argc,char* argv[]) 
{
	int i_arg = 0;
	Tbooleen trouve = FAUX;
	Tbooleen fin = FAUX;
	FILE* fd = NULL;
	FILE* ft = NULL;
	char ligne[MAXLIG];
	char ligneCmd[MAXLIG];
	char ligne_dep[MAXLIG];
	char file[MAXREP];
	char identif[MAXREP];
	int flag = 0;
	int posit = 0;
	char* tmpdir = NULL;
	char ftemp[MAXREP];
	char* paction = NULL;
	char action[MAXLIG];

	strcpy(mk_suff_name,".c");

	if((tmpdir = getenv("TMPDIR")) && strlen(tmpdir) != 0)
	{
	    sprintf(ftemp,"%s/mkdep.tmp.%d",tmpdir,getpid());
	    mkdir(tmpdir,0700);
	    errno = 0;
	}
	else
	{
	    sprintf(ftemp,"/tmp/mkdep.tmp.%d",getpid());
	}

	if((paction = getenv("MKDEP_ACTION")) && strlen(paction) != 0)
	{
	    sprintf(action,"%s",paction);
	    mkdir(tmpdir,0700);
	    errno = 0;
	}
	else
	{
	    sprintf(action,"MKDEP_ACTION");
	}

	if (argc==1) usage();

	for ( i_arg = 1 ; trouve == FAUX && i_arg < argc ; i_arg++ )
	{

	    if ( argv[i_arg][0] == '-' )
	    {

		if ( !strncmp(argv[i_arg],"-I",2))
		{
		    mk_inc_path[mk_i_inc] = &argv[i_arg][2];
		    if ( mk_i_inc++ >= MAXPATHINC ) usage();
		    mk_inc = VRAI;
		}
		else if ( !strncmp(argv[i_arg],"-L",2))
		{
		    mk_lib_path = &argv[i_arg][2];
		    mk_lib = VRAI;
		    strcpy(mk_suff_name,".o");
		}
		else if ( !strcmp(argv[i_arg],"-f"))
		{
		    if (i_arg++ >= argc ) usage();
		    strcpy(mk_file_out,argv[i_arg]);
		    mk_file	= VRAI;
		}
		else if ( !strcmp(argv[i_arg],"-sccs"))
		{
		    mk_sccs = VRAI;
		}
		else if ( !strcmp(argv[i_arg],"-cmd"))
		{
		    mk_cmd  = VRAI;
		}
		else if ( !strncmp(argv[i_arg],"-o",2))
		{
		    strcpy(mk_suff_name,&argv[i_arg][2]);
		    mk_suff = VRAI;
		}

	    }
	    else
	    {
		trouve = VRAI;
		i_arg--;
	    }

	}

	if ( trouve == FAUX ) usage();
	
	mk_nom_en_sccs = (!strncmp(argv[i_arg],"s.",2));
	
	/* Recherche du nom du fichier de dependances */
	/*  ----------------------------------------  */

	if ( mk_file == FAUX )
	{  
	    strcpy(mk_file_out,".depend");
	    strcat(mk_file_out,strrchr(argv[i_arg],'.')+1);
	}

	/*  Ouverture des fichiers de travail  */
	/*  ---------------------------------  */
	/* fd : fichier de dependances a mettre a jour */

	fd = fopen(mk_file_out,"r"); 

	if ( fd == NULL )
	{
	    fd=fopen(mk_file_out,"w");
	}

	ft = fopen(ftemp,"w"); 	   /* ft : fichier temporaire */

	if ( fd == NULL || ft == NULL )
	{
	    fprintf(stderr,"mkdep: Impossible d'ouvrir %s ou %s\n",
		    mk_file_out,ftemp);
	}

	ligne[0] = '\0';

	if ( fgets(ligne,MAXLIG,fd) == NULL ) fin = VRAI;

	for ( ;i_arg < argc ; i_arg++) /* Boucle sur les fichiers a traiter */
	{ 
	    /*  	Construction de la ligne de dependance  */
	    /*  	--------------------------------------  */

	    if ( mk_nom_en_sccs )
	    {
		strcpy(file,&argv[i_arg][2]);
		mk_sccs = VRAI;
	    }
	    else
	    {
		strcpy(file,argv[i_arg]);
	    }

	    flag = construit_ligne(file,ligne_dep);

	    strcpy(identif,"# ");
	    strcat(identif,file);

	    trouve = FAUX;

	    while ( trouve == FAUX && fin == FAUX )
	    {
		posit = strncmp(ligne,identif,strlen(identif)) ;

		if ( posit < 0 )
		{

		    do
		    {
			fputs(ligne,ft);
			ligne[0] = '\0';

			if ( fgets(ligne,MAXLIG,fd) == NULL ) fin = VRAI;

		    }
		    while ( fin == FAUX && ligne[0] != '#' );

		}
		else if ( posit == 0 )
		{
		    trouve = VRAI;

		    do
		    {
			ligne[0] = '\0';

			if ( fgets(ligne,MAXLIG,fd) == NULL )
			  fin = VRAI;
		    }
		    while ( fin == FAUX && ligne[0] != '#' );

		    ecrit_ligne(ft,identif,ligne_dep,flag,action);
		}
		else if ( posit > 0 )
		{
		    trouve = VRAI;
		    ecrit_ligne(ft,identif,ligne_dep,flag,action);
		}

	    }

	    if ( fin == VRAI && trouve == FAUX )
	    {
		ecrit_ligne(ft,identif,ligne_dep,flag,action);
	    }

	} /* next fichier */

	if ( fin == FAUX )
	{

	    do
	    { 
		fputs(ligne,ft);
		ligne[0] = '\0';
	    }
	    while ( fgets(ligne,MAXLIG,fd) != NULL ) ;

	}
	
	fclose(ft);
	fclose(fd);

	strcpy(ligneCmd,"mv -f ");
	strcat(ligneCmd,ftemp);
	strcat(ligneCmd," ");
	strcat(ligneCmd,mk_file_out);
	flag = system(ligneCmd);

	if(flag == -1)
	{
	    fprintf(stderr,"mkdep: Impossible deplacer fichier %s vers %s\n",
		ftemp,mk_file_out);
	}

	return 0;
}
